/*
 *      callback.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "callback.h"
#include "dxutils.h"
#include "utilsmm.h"
#include "preferences.h"
#include "ui/log.h"

dxcCallbacks::dxcCallbacks(const std::string& path)
{
    m_purple = new PURPLE::Purple(this, "dxim", VERSION, "http://dxim.dxsolutions.org", path);
}

dxcCallbacks::~dxcCallbacks()
{}

PURPLE::Purple *dxcCallbacks::getPurplemm()
{
    return m_purple;
}

/** Allow your application to be notified about errors on library layer */
void dxcCallbacks::showError(std::string msg)
{
    signalError(msg);
}

/** Allow your application to be notified about libpurple debug messages */
void dxcCallbacks::showDebug(std::string debug)
{
    if(LOG) LOG->Debug("[PURPLE] %s", debug.c_str());
}

/** New account added */
void dxcCallbacks::accountAdded(PURPLE::Account account)
{
    signalAccountAdded(account);
}

/** Account removed */
void dxcCallbacks::accountRemoved(std::string account, std::string protocolID)
{
    signalAccountRemoved(account, protocolID);
}

/** Account connecting */
void dxcCallbacks::accountConnecting(PURPLE::Account account, std::string msg)
{
    signalAccountConnecting(account, msg);
}

/** Acount connected*/
void dxcCallbacks::accountConnected(PURPLE::Account account)
{
    signalAccountConnected(account);
}

/** Acount disconnected*/
void dxcCallbacks::accountDisconnected(PURPLE::Account account, std::string reason)
{
    signalAccountDisconnected(account, reason);
}

/** Account status changed*/
void dxcCallbacks::accountStatusChanged(PURPLE::Account account)
{
    signalAccountStatusChanged(account);
}

/** Account signed on, so show buddy list */
void dxcCallbacks::accountSignedOn(PURPLE::Account account)
{
    signalAccountSignedOn(account);
}

/** New IM created */
void dxcCallbacks::newIM(PURPLE::Conversation *conv)
{
    signalNewIM(conv);
}

/** New Chat created */
void dxcCallbacks::newChat(PURPLE::Conversation *conv)
{
    signalNewChat(conv);
}

/** IM closed */
void dxcCallbacks::closedIM(PURPLE::Conversation *conv)
{
    signalCloseConvTab(conv);
}

/** Chat closed */
void dxcCallbacks::closedChat(PURPLE::Conversation *conv)
{
    signalCloseConvTab(conv);
}

/** New my message
 * @param conv          The conversation.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param action        Is message in 3rd person (/me)?
 */
void dxcCallbacks::newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID)
{
    signalNewMyMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), msg, action, messageID);
}

void dxcCallbacks::messageRecived(PURPLE::Conversation *conv, std::string messageID)
{
    if(DXPREFERENCES.m_receiptsMark && !messageID.empty()) signalMessageRecived(conv, messageID);
}

/** New IM message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxcCallbacks::newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    std::string timeStr = delayed?purple_utf8_strftime("[%x %H:%M:%S] ", localtime(&time)):purple_utf8_strftime("[%H:%M:%S] ", localtime(&time));
    signalNewIMMessage(conv, timeStr, sender, msg, highlight, action);
}

/** New Chat message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxcCallbacks::newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    std::string timeStr = delayed?purple_utf8_strftime("[%x %H:%M:%S] ", localtime(&time)):purple_utf8_strftime("[%H:%M:%S] ", localtime(&time));
    signalNewChatMessage(conv, timeStr, sender, msg, delayed, highlight, action);
}

/** New system mesage for conversation
 *
 * @param conv  The conversation
 * @param msg  The message text
 * @param time  The time the message was sent
 */
void dxcCallbacks::newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    signalNewConvSystemMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), msg);
}

/** New error mesage for conversation
 *
 * @param conv      conversation
 * @param msg       message text
 * @param time      time the message was sent
 */
void dxcCallbacks::newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    signalNewConvErrorMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), msg);
}

/** A new list was created */
void dxcCallbacks::roomlistCreated(PURPLE::RoomList *rlist)
{
    signalRoomlistCreated(rlist);
}

/** Sets the columns of RoomList */
void dxcCallbacks::roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns)
{
    signalRoomlistColumns(rlist, columns);
}

/** Add a room to the list */
void dxcCallbacks::roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields)
{
    signalRoomlistAddRoom(rlist, fields);
}

/** Listing RoomList finished */
void dxcCallbacks::roomlistFinished(PURPLE::RoomList *rlist)
{
    signalRoomlistFinished(rlist);
}

/** New InputRequest */
void dxcCallbacks::newInputRequest(PURPLE::InputRequest *request)
{
    signalInputRequest(request);
}

/** New ActionRequest */
void dxcCallbacks::newActionRequest(PURPLE::ActionRequest *request)
{
    signalActionRequest(request);
}

/** New ChoiceRequest */
void dxcCallbacks::newChoiceRequest(PURPLE::ChoiceRequest *request)
{
    signalChoiceRequest(request);
}

/** New FieldsRequest */
void dxcCallbacks::newFieldsRequest(PURPLE::FieldsRequest *request)
{
    signalFieldsRequest(request);
}

/** New FileRequest */
void dxcCallbacks::newFileRequest(PURPLE::FileRequest *request)
{
    signalFileRequest(request);
}

/** ChatBuddy joined chat */
void dxcCallbacks::chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    signalChatBuddyJoined(conv, buddy.getName());
}

/** ChatBuddy updated */
void dxcCallbacks::chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy)
{
    signalChatBuddyUpdated(conv, buddy);
}

/** ChatBuddy left chat */
void dxcCallbacks::chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
{
    signalChatBuddyLeft(conv, buddy);
}

/** ChatBuddy renamed
* @param who       buddy which was renamed
* @param nbuddy    new buddy established by rename
*/
void dxcCallbacks::chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy)
{
    if(nbuddy.isValid())
        signalChatBuddyRenamed(conv, who, nbuddy.getName());
}

/** Chat topic changed
* @param who  topic changer
* @param topic  new topic
*/
void dxcCallbacks::chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic)
{
    signalChatTopic(conv, who, topic);
}

/** Buddy typing on IM
* @param typing =true typing, =false stoped typing
*/
void dxcCallbacks::buddyTyping(PURPLE::Conversation *conv, bool typing)
{
    signalBuddyTyping(conv, typing);
}

/** Buddy signed on */
void dxcCallbacks::buddySignedOn(PURPLE::Buddy buddy)
{
    signalBuddySignedOn(buddy);
}

/** Buddy signed off */
void dxcCallbacks::buddySignedOff(PURPLE::Buddy buddy)
{
    signalBuddySignedOff(buddy);
}

/** Buddy status changed */
void dxcCallbacks::buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive /*oldStatus*/, PurpleStatusPrimitive newStatus)
{
    signalBuddyStatusChanged(buddy, newStatus);
}

/** Buddy icon changed */
void dxcCallbacks::buddyIconChanged(PURPLE::Buddy buddy)
{
    signalBuddyIconChanged(buddy);
}

/** Buddy added */
void dxcCallbacks::buddyAdded(PURPLE::Buddy buddy)
{
    signalBuddyAdded(buddy);
}

/** Buddy removed */
void dxcCallbacks::buddyRemoved(PURPLE::Account account, std::string buddy)
{
    signalBuddyRemoved(account, buddy);
}

/** Chat added */
void dxcCallbacks::chatAdded(PURPLE::Chat chat)
{
    signalChatAdded(chat);
}

/** Chat removed */
void dxcCallbacks::chatRemoved(PURPLE::Account account, std::string chat)
{
    signalChatRemoved(account, chat);
}

/** Notification message to the user
* @param type  notification type
* @param title  title of the message
* @param primary  main point of the message
* @param secondary  secondary information
*/
void dxcCallbacks::notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary)
{
    signalNotifyMessage(type, title, dxutils::sformat("%s<br><br>%s", primary.c_str(), secondary.c_str()));
}

/** Email notification to the user
* @param subject  subject of the email
* @param from  from address
* @param to  estination address
* @param url  URL where the message can be read
*/
void dxcCallbacks::notifyEmail(std::string subject, std::string from, std::string to, std::string url)
{
    signalNotifyEmail(dxutils::sformat(_("From: %s\nTo: %s\nSubject: %s"), from.c_str(), to.c_str(), subject.c_str()), url);
}

/** Notification with formatted text
* @param title  title of the message
* @param primary  main point of the message
* @param secondary  secondary information
* @param text  formatted text
*/
void dxcCallbacks::notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated)
{
    if(secondary.empty())
        signalNotifyMessage(PURPLE_NOTIFY_MSG_INFO, title, dxutils::sformat("%s<br><br>%s", primary.c_str(), formated.c_str()));
    else
        signalNotifyMessage(PURPLE_NOTIFY_MSG_INFO, title, dxutils::sformat("%s<br><br>%s<br><br>%s", primary.c_str(), secondary.c_str(), formated.c_str()));

}

/** Notification information about buddy
* @param account   requested account
* @param buddy  Buddy which about
* @param info  vector with UserInfo values
*/
void dxcCallbacks::notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info)
{
    std::string text, value;
    std::vector<PURPLE::Notify::UserInfo>::iterator it;
    for(it=info.begin(); it!=info.end(); it++)
    {
        value = (*it).getValue();
        if(value.find("<img id='")!=std::string::npos)
        {
            value = PURPLE::imagePathByID(dxutils::stringSection(value, '\'', 1, false));
        }
        if(value.empty()) text.append(dxutils::sformat("<b>%s</b>:<br>", (*it).getLabel().c_str()));
        else text.append(dxutils::sformat("<b>%s</b>: %s<br>", (*it).getLabel().c_str(), PURPLE::markupHtml(value).c_str()));
    }
    signalNotifyUserinfo(account, buddy, text);
}

/** Opens a URI or somehow presents it to the user
* @param uri    The URI to display or go to.
*/
void dxcCallbacks::notifyURI(std::string uri)
{
    signalNotifyUri(uri);
}

/** New FileTransfer */
void dxcCallbacks::newFileTransfer(PURPLE::FileTransfer *ft)
{
    signalNewFileTransfer(ft);
}

/** FileTransfer finished */
void dxcCallbacks::fileTransferFinished(PURPLE::FileTransfer *ft)
{
    signalFileTransferFinished(ft);
}

/** FileTransfer started */
void dxcCallbacks::fileTransferStarted(PURPLE::FileTransfer *ft)
{
    signalFileTransferStarted(ft);
}

/** FileTransfer progress */
void dxcCallbacks::fileTransferProgress(PURPLE::FileTransfer *ft, double /*percent*/)
{
    signalFileTransferProgress(ft);
}

/** FileTransfer canceled */
void dxcCallbacks::fileTransferCanceled(PURPLE::FileTransfer *ft)
{
    signalFileTransferCanceled(ft);
}

