/*     dxim.cpp
*
*      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 3 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
*      MA 02110-1301, USA.
*/

#include <cppconsui/ColorScheme.h>
#include <cppconsui/KeyConfig.h>
#include <errno.h>
#include <glib/gprintf.h>
#include <getopt.h>
#include <typeinfo>
#include "gettext.h"
#include "dxim.h"
#include "dxutils.h"
#include "ui/footer.h"
#include "ui/log.h"
#include "ui/roster.h"
#include "ui/tabbook.h"
#include "ui/mainmenu.h"
#include "ui/statusmenu.h"
#include "ui/newimmenu.h"
#include "ui/joinchatmenu.h"
#include "ui/inputrequestdialog.h"
#include "ui/actionrequestdialog.h"
#include "ui/choicerequestdialog.h"
#include "ui/fieldsrequestdialog.h"
#include "ui/filerequestdialog.h"
#include "ui/filetransferdialog.h"
#include "ui/chattabitem.h"
#include "ui/imtabitem.h"
#include "ui/msgdialog.h"
#include "preferences.h"

const char *dxim::m_named_colors[] = {
  "default", /* -1 */
  "black",   /*  0 */
  "red",     /*  1 */
  "green",   /*  2 */
  "yellow",  /*  3 */
  "blue",    /*  4 */
  "magenta", /*  5 */
  "cyan",    /*  6 */
  "white"    /*  7 */
};

dxim::dxim()
: m_convs_expanded(false), m_ftdialog(NULL)
{
    m_mngr = CppConsUI::CoreManager::Instance();
    m_resize_conn = m_mngr->signal_resize.connect(sigc::mem_fun(this, &dxim::OnScreenResized));
    m_top_window_change_conn = m_mngr->signal_top_window_change.connect(sigc::mem_fun(this, &dxim::OnTopWindowChanged));
    DeclareBindables();
}

dxim* dxim::Instance()
{
    static dxim instance;
    return &instance;
}

PURPLE::Purple* dxim::GetPurplemm()
{
    return m_cb->getPurplemm();
}

bool dxim::ProcessInput(const TermKeyKey& key)
{
    GetPurplemm()->idleTouch();
    return InputProcessor::ProcessInput(key);
}

int dxim::Run(const std::string& path, bool offline)
{
    m_cb = new dxcCallbacks(path);
    m_cb->signalError.connect(sigc::mem_fun(this, &dxim::OnPurpleError));
    m_cb->signalNotifyMessage.connect(sigc::mem_fun(this, &dxim::OnNotifyMessage));
    m_cb->signalNotifyEmail.connect(sigc::mem_fun(this, &dxim::OnNotifyEmail));
    m_cb->signalNotifyUserinfo.connect(sigc::mem_fun(this, &dxim::OnUserInfo));
    m_cb->signalNotifyUri.connect(sigc::mem_fun(this, &dxim::OnNotifyUri));
    m_cb->signalInputRequest.connect(sigc::mem_fun(this, &dxim::OnNewInputRequest));
    m_cb->signalActionRequest.connect(sigc::mem_fun(this, &dxim::OnNewActionRequest));
    m_cb->signalChoiceRequest.connect(sigc::mem_fun(this, &dxim::OnNewChoiceRequest));
    m_cb->signalFieldsRequest.connect(sigc::mem_fun(this, &dxim::OnNewFieldsRequest));
    m_cb->signalFileRequest.connect(sigc::mem_fun(this, &dxim::OnNewFileRequest));
    m_cb->signalNewFileTransfer.connect(sigc::mem_fun(this, &dxim::OnNewFileTransfer));
    LoadColorSchemeConfig();
    LoadKeyConfig();
    Log::Init();
    Footer::Init();
    Roster::Init();
    TabBook::Init();
    GetPurplemm()->initPurple(offline);
    GetPurplemm()->loadSavedPlugins("/dxim/plugins/loaded");
    DXPREFERENCES.load();
    ROSTER->FillRoster();
    m_mngr->SetTopInputProcessor(*this);
    m_mngr->EnableResizing();
    m_mngr->StartMainLoop();
    m_resize_conn.disconnect();
    m_top_window_change_conn.disconnect();
    Roster::Finalize();
    TabBook::Finalize();
    Footer::Finalize();
    Log::Finalize();
    return 0;
}

void dxim::Quit()
{
    DXPREFERENCES.save();
    m_mngr->QuitMainLoop();
}

CppConsUI::Rect dxim::GetScreenArea(ScreenArea area)
{
    return m_areas[area];
}

CppConsUI::Rect dxim::GetScreenAreaCentered(ScreenArea area)
{
    CppConsUI::Rect s = m_areas[WHOLE_AREA];
    CppConsUI::Rect r = m_areas[area];
    int x = (s.width - r.width) / 2;
    int y = (s.height - r.height) / 2;
    return CppConsUI::Rect(x, y, r.width, r.height);
}

bool dxim::LoadColorSchemeConfig()
{
    xmlnode *root = purple_util_read_xml_from_file("colorschemes.xml", "color schemes");
    if(!root)
    {
        // read error, first time run?
        LoadDefaultColorSchemeConfig();
        if(SaveColorSchemeConfig())
            return true;
        return false;
    }
    COLORSCHEME->Clear();
    bool res = false;
    for(xmlnode *scheme = xmlnode_get_child(root, "scheme"); scheme; scheme = xmlnode_get_next_twin(scheme))
    {
        const char *name = xmlnode_get_attrib(scheme, "name");
        if(!name)
        {
            LOG->Error(_("Missing 'name' attribute in the scheme definition."));
            goto out;
        }
        for(xmlnode *color = xmlnode_get_child(scheme, "color"); color; color = xmlnode_get_next_twin(color))
        {
            const char *widget = xmlnode_get_attrib(color, "widget");
            if(!widget)
            {
                LOG->Error(_("Missing 'widget' attribute in the color definition."));
                goto out;
            }
            const char *property = xmlnode_get_attrib(color, "property");
            if(!property)
            {
                LOG->Error(_("Missing 'property' attribute in the color definition."));
                goto out;
            }
            const char *fgs = xmlnode_get_attrib(color, "foreground");
            const char *bgs = xmlnode_get_attrib(color, "background");
            const char *attrs = xmlnode_get_attrib(color, "attributes");
            int fg = CppConsUI::Curses::Color::DEFAULT;
            int bg = CppConsUI::Curses::Color::DEFAULT;
            int attr = 0;
            if(fgs && !StringToColor(fgs, &fg))
            {
                LOG->Error(_("Unrecognized color '%s'."), fgs);
                goto out;
            }
            if(bgs && !StringToColor(bgs, &bg))
            {
                LOG->Error(_("Unrecognized color '%s'."), bgs);
                goto out;
            }
            if(attrs && !StringToColorAttributes(attrs, &attr))
            {
                LOG->Error(_("Unrecognized attributes '%s'."), attrs);
                goto out;
            }
            COLORSCHEME->SetColorPair(name, widget, property, fg, bg, attr);
        }
    }
    res = true;
out:
    if(!res)
    {
        LOG->Error(_("Error parsing 'colorschemes.xml', " "loading default color scheme."));
        LoadDefaultColorSchemeConfig();
    }
    xmlnode_free(root);
    return res;
}

bool dxim::LoadKeyConfig()
{
    xmlnode *root = purple_util_read_xml_from_file("binds.xml", _("key bindings"));
    if(!root)
    {
        // read error, first time run?
        LoadDefaultKeyConfig();
        if(SaveKeyConfig())
            return true;
        return false;
    }
    KEYCONFIG->Clear();
    bool res = false;
    for(xmlnode *bind = xmlnode_get_child(root, "bind"); bind; bind = xmlnode_get_next_twin(bind))
    {
        const char *context = xmlnode_get_attrib(bind, "context");
        if(!context)
        {
            LOG->Error(_("Missing 'context' attribute in the bind definition."));
            goto out;
        }
        const char *action = xmlnode_get_attrib(bind, "action");
        if(!action)
        {
            LOG->Error(_("Missing 'action' attribute in the bind definition."));
            goto out;
        }
        const char *key = xmlnode_get_attrib(bind, "key");
        if(!key)
        {
            LOG->Error(_("Missing 'key' attribute in the bind definition."));
            goto out;
        }
        if(!KEYCONFIG->BindKey(context, action, key))
        {
            LOG->Error(_("Unrecognized key '%s'."), key);
            goto out;
        }
    }
    res = true;
out:
    if(!res)
    {
        LOG->Error(_("Error parsing 'binds.xml', loading default keys."));
        LoadDefaultKeyConfig();
    }
    xmlnode_free(root);
    return res;
}

void dxim::LoadDefaultColorSchemeConfig()
{
    COLORSCHEME->Clear();
    // default colors init
    COLORSCHEME->SetColorPair("footer", "label", "text", CppConsUI::Curses::Color::BLACK, CppConsUI::Curses::Color::WHITE);
    COLORSCHEME->SetColorPair("footer", "container", "background", CppConsUI::Curses::Color::BLACK, CppConsUI::Curses::Color::WHITE);

    COLORSCHEME->SetColorPair("log", "panel", "line", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);
    COLORSCHEME->SetColorPair("log", "textview", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("log", "textview", "color1", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT); //debug
    COLORSCHEME->SetColorPair("log", "textview", "color2", CppConsUI::Curses::Color::YELLOW, CppConsUI::Curses::Color::DEFAULT); //warning
    COLORSCHEME->SetColorPair("log", "textview", "color3", CppConsUI::Curses::Color::RED, CppConsUI::Curses::Color::DEFAULT); //error

    COLORSCHEME->SetColorPair("tabitem", "textview", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("tabitem", "textview", "color1", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT); //mymsg
    COLORSCHEME->SetColorPair("tabitem", "textview", "color2", CppConsUI::Curses::Color::YELLOW, CppConsUI::Curses::Color::DEFAULT); //action
    COLORSCHEME->SetColorPair("tabitem", "textview", "color3", CppConsUI::Curses::Color::RED, CppConsUI::Curses::Color::DEFAULT); //error
    COLORSCHEME->SetColorPair("tabitem", "textview", "color4", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT); //join,delayed
    COLORSCHEME->SetColorPair("tabitem", "textview", "color5", CppConsUI::Curses::Color::MAGENTA, CppConsUI::Curses::Color::DEFAULT); //highlight
    COLORSCHEME->SetColorPair("tabitem", "panel", "line", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);
    COLORSCHEME->SetColorPair("tabitem", "horizontalline", "line", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);
    COLORSCHEME->SetColorPair("tabitem", "textedit", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("tabitem", "label", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("tabbook", "label", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("tabbook-active", "label", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);

    COLORSCHEME->SetColorPair("tabbook-unread", "label", "text", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("tabbook-highlight", "label", "text", CppConsUI::Curses::Color::RED, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("roster", "treeview", "line", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("roster", "panel", "line", CppConsUI::Curses::Color::BLUE, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);
    COLORSCHEME->SetColorPair("roster", "button", "normal", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosteraccount_online", "button", "normal", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosteraccount_online", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosteraccount_away", "button", "normal", CppConsUI::Curses::Color::MAGENTA, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosteraccount_away", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosteraccount_dnd", "button", "normal", CppConsUI::Curses::Color::YELLOW, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosteraccount_dnd", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosteraccount_offline", "button", "normal", CppConsUI::Curses::Color::RED, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosteraccount_offline", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosterbuddy_online", "button", "normal", CppConsUI::Curses::Color::GREEN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosterbuddy_online", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosterbuddy_away", "button", "normal", CppConsUI::Curses::Color::MAGENTA, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosterbuddy_away", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosterbuddy_dnd", "button", "normal", CppConsUI::Curses::Color::YELLOW, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosterbuddy_dnd", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rosterbuddy_offline", "button", "normal", CppConsUI::Curses::Color::RED, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("rosterbuddy_offline", "button", "status", CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("rostergroup", "button", "normal", CppConsUI::Curses::Color::MAGENTA, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("generalmenu", "panel", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generalmenu", "panel", "title", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generalmenu", "horizontalline", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generalmenu", "button", "normal", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("generaldialog", "panel", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "panel", "title", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "horizontalline", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "verticalline", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "button", "normal", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "treeview", "line", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "checkbox", "normal", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "textview", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);
    COLORSCHEME->SetColorPair("generaldialog", "label", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("dialoggroup", "button", "normal", CppConsUI::Curses::Color::MAGENTA, CppConsUI::Curses::Color::DEFAULT);

    COLORSCHEME->SetColorPair("fieldgroup", "label", "text", CppConsUI::Curses::Color::CYAN, CppConsUI::Curses::Color::DEFAULT, CppConsUI::Curses::Attr::BOLD);
}

bool dxim::SaveColorSchemeConfig()
{
    xmlnode *root = xmlnode_new("colorscheme");
    xmlnode_set_attrib(root, "version", "1.0");
    for(CppConsUI::ColorScheme::Schemes::const_iterator si = COLORSCHEME->GetSchemes().begin(); si != COLORSCHEME->GetSchemes().end(); si++)
    {
        xmlnode *scheme = xmlnode_new("scheme");
        xmlnode_set_attrib(scheme, "name", si->first.c_str());
        xmlnode_insert_child(root, scheme);
        for(CppConsUI::ColorScheme::Widgets::const_iterator wi = si->second.begin(); wi != si->second.end(); wi++)
        {
            for (CppConsUI::ColorScheme::Properties::const_iterator pi = wi->second.begin(); pi != wi->second.end(); pi++)
            {
                xmlnode *color = xmlnode_new("color");
                xmlnode_set_attrib(color, "widget", wi->first.c_str());
                xmlnode_set_attrib(color, "property", pi->first.c_str());
                xmlnode_insert_child(scheme, color);
                char *str;
                if(pi->second.foreground != CppConsUI::Curses::Color::DEFAULT)
                {
                    str = ColorToString(pi->second.foreground);
                    xmlnode_set_attrib(color, "foreground", str);
                    g_free(str);
                }
                if(pi->second.background != CppConsUI::Curses::Color::DEFAULT)
                {
                    str = ColorToString(pi->second.background);
                    xmlnode_set_attrib(color, "background", str);
                    g_free(str);
                }
                if((str = ColorAttributesToString(pi->second.attrs)))
                {
                    xmlnode_set_attrib(color, "attributes", str);
                    g_free(str);
                }
            }
        }
    }
    char *data = xmlnode_to_formatted_str(root, NULL);
    bool res = true;
    if(!purple_util_write_data_to_file("colorschemes.xml", data, -1))
    {
        LOG->Error(_("Error saving 'colorschemes.xml'."));
        res = false;
    }
    g_free(data);
    xmlnode_free(root);
    return res;
}

char *dxim::ColorToString(int color)
{
    if(color >= -1 && color < static_cast<int>(G_N_ELEMENTS(m_named_colors) - 1))
        return g_strdup(m_named_colors[color + 1]);
    return g_strdup_printf("%d", color);
}

bool dxim::StringToColor(const char *str, int *color)
{
    g_assert(str);
    g_assert(color);
    *color = 0;
    if(g_ascii_isdigit(str[0]) || str[0] == '-')
    {
        // numeric colors
        long int i = strtol(str, NULL, 10);
        if(errno == ERANGE || i > INT_MAX || i < -1)
            return false;
        *color = i;
        return true;
    }
    // symbolic colors
    for(int i = -1; i < static_cast<int>(G_N_ELEMENTS(m_named_colors) - 1); i++)
        if(!strcmp(str, m_named_colors[i + 1]))
        {
            *color = i;
            return true;
        }
    return false;
}

char *dxim::ColorAttributesToString(int attrs)
{
#define APPEND(str) do { \
        if (s.size()) \
            s.append("|"); \
        s.append(str); \
    } while (0)
    std::string s;
    if(attrs == CppConsUI::Curses::Attr::NORMAL)
        return NULL;
    if(attrs & CppConsUI::Curses::Attr::STANDOUT)
        APPEND("standout");
    if(attrs & CppConsUI::Curses::Attr::REVERSE)
        APPEND("reverse");
    if(attrs & CppConsUI::Curses::Attr::BLINK)
        APPEND("blink");
    if(attrs & CppConsUI::Curses::Attr::DIM)
        APPEND("dim");
    if(attrs & CppConsUI::Curses::Attr::BOLD)
        APPEND("bold");
    return g_strdup(s.c_str());
#undef APPEND
}

bool dxim::StringToColorAttributes(const char *str, int *attrs)
{
    g_assert(str);
    g_assert(attrs);
    gchar **tokens = g_strsplit(str, "|", 0);
    *attrs = 0;
    bool valid = true;
    for(size_t i = 0; tokens[i]; i++)
    {
        if(!strcmp("normal", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::NORMAL;
            continue;
        }
        if(!strcmp("standout", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::STANDOUT;
            continue;
        }
        if(!strcmp("reverse", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::REVERSE;
            continue;
        }
        if(!strcmp("blink", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::BLINK;
            continue;
        }
        if(!strcmp("dim", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::DIM;
            continue;
        }
        if(!strcmp("bold", tokens[i]))
        {
            *attrs |= CppConsUI::Curses::Attr::BOLD;
            continue;
        }
        // unrecognized attribute
        valid = false;
        break;
    }
    g_strfreev(tokens);
    return valid;
}

void dxim::LoadDefaultKeyConfig()
{
    KEYCONFIG->Clear();
    KEYCONFIG->LoadDefaultKeyConfig();
    KEYCONFIG->BindKey("dxim", "quit", "Ctrl-q");
    KEYCONFIG->BindKey("dxim", "roster-active", "F2");
    KEYCONFIG->BindKey("dxim", "tabitem-active", "F3");
    KEYCONFIG->BindKey("dxim", "statusmenu", "F4");
    KEYCONFIG->BindKey("dxim", "mainmenu", "F5");
    KEYCONFIG->BindKey("dxim", "mainmenu", "Ctrl-g");
    KEYCONFIG->BindKey("dxim", "newimmenu", "F8");
    KEYCONFIG->BindKey("dxim", "joinchatmenu", "F9");
    KEYCONFIG->BindKey("dxim", "roster-toggle-offline", "F6");
    KEYCONFIG->BindKey("dxim", "tabbook-expand", "F7");
    KEYCONFIG->BindKey("dxim", "tabbook-closecurrent", "Ctrl-w");
    KEYCONFIG->BindKey("dxim", "tabitem-prev", "Ctrl-p");
    KEYCONFIG->BindKey("dxim", "tabitem-next", "Ctrl-n");
    KEYCONFIG->BindKey("dxim", "tabitem-number1", "Alt-1");
    KEYCONFIG->BindKey("dxim", "tabitem-number2", "Alt-2");
    KEYCONFIG->BindKey("dxim", "tabitem-number3", "Alt-3");
    KEYCONFIG->BindKey("dxim", "tabitem-number4", "Alt-4");
    KEYCONFIG->BindKey("dxim", "tabitem-number5", "Alt-5");
    KEYCONFIG->BindKey("dxim", "tabitem-number6", "Alt-6");
    KEYCONFIG->BindKey("dxim", "tabitem-number7", "Alt-7");
    KEYCONFIG->BindKey("dxim", "tabitem-number8", "Alt-8");
    KEYCONFIG->BindKey("dxim", "tabitem-number9", "Alt-9");
    KEYCONFIG->BindKey("tabitem", "send", "Ctrl-x");
    KEYCONFIG->BindKey("tabitem", "clear", "Ctrl-u");
    KEYCONFIG->BindKey("roster", "contextmenu", "Ctrl-d");
    //Blackened request for less, most and Vim bindings :)
    KEYCONFIG->BindKey("container", "focus-begin", "H");
    KEYCONFIG->BindKey("container", "focus-down", "j");
    KEYCONFIG->BindKey("container", "focus-end", "L");
    KEYCONFIG->BindKey("container", "focus-left", "h");
    KEYCONFIG->BindKey("container", "focus-page-down", "J");
    KEYCONFIG->BindKey("container", "focus-page-up", "K");
    KEYCONFIG->BindKey("container", "focus-right", "l");
    KEYCONFIG->BindKey("container", "focus-up", "k");
    KEYCONFIG->BindKey("textentry", "cursor-left", "Ctrl-b");
    KEYCONFIG->BindKey("textentry", "cursor-right", "Ctrl-f");
}

bool dxim::SaveKeyConfig()
{
    xmlnode *root = xmlnode_new("keyconfig");
    xmlnode_set_attrib(root, "version", "1.0");
    const CppConsUI::KeyConfig::KeyBinds *binds = KEYCONFIG->GetKeyBinds();
    for(CppConsUI::KeyConfig::KeyBinds::const_iterator bi = binds->begin();bi != binds->end(); bi++)
    {
        /* Invert the map because the output should be sorted by context+action,
         * not by context+key. */
        typedef std::multimap<std::string, TermKeyKey> InvertedMap;
        InvertedMap inverted;
        for(CppConsUI::KeyConfig::KeyBindContext::const_iterator ci = bi->second.begin(); ci != bi->second.end(); ci++)
            inverted.insert(std::make_pair(ci->second, ci->first));
        for(InvertedMap::iterator ci = inverted.begin(); ci != inverted.end(); ci++)
        {
            xmlnode *bind = xmlnode_new("bind");
            xmlnode_set_attrib(bind, "context", bi->first.c_str());
            xmlnode_set_attrib(bind, "action", ci->first.c_str());
            char *key;
            if((key = KEYCONFIG->TermKeyToString(ci->second)))
            {
                xmlnode_set_attrib(bind, "key", key);
                g_free(key);
            }
            xmlnode_insert_child(root, bind);
        }
    }
    char *data = xmlnode_to_formatted_str(root, NULL);
    bool res = true;
    if(!purple_util_write_data_to_file("binds.xml", data, -1))
    {
        LOG->Error(_("Error saving 'binds.xml'."));
        res = false;
    }
    g_free(data);
    xmlnode_free(root);
    return res;
}

void dxim::DeclareBindables()
{
    DeclareBindable("dxim", "quit", sigc::mem_fun(this, &dxim::Quit), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "roster-active", sigc::mem_fun(this, &dxim::ActionRosterActive), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "tabitem-active", sigc::mem_fun(this, &dxim::ActionTabitemActive), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "statusmenu", sigc::mem_fun(this, &dxim::ActionStatusMenu), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "mainmenu", sigc::mem_fun(this, &dxim::ActionMainMenu), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "newimmenu", sigc::mem_fun(this, &dxim::ActionNewImMenu), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "joinchatmenu", sigc::mem_fun(this, &dxim::ActionJoinChatMenu), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "roster-toggle-offline", sigc::mem_fun(this, &dxim::ActionRosterToggleOffline), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "tabbook-expand", sigc::mem_fun(this, &dxim::ActionTabbookExpand), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "tabbook-closecurrent", sigc::mem_fun(this, &dxim::ActionCloseCurrentTabitem), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "tabitem-prev", sigc::mem_fun(this, &dxim::ActionTabitemPrev), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("dxim", "tabitem-next", sigc::mem_fun(this, &dxim::ActionTabitemNext), InputProcessor::BINDABLE_OVERRIDE);
    char action[] = "tabitem-numberXX";
    for (int i = 1; i <= 10; i++)
    {
        g_sprintf(action + sizeof(action) - 3, "%d", i);
        DeclareBindable("dxim", action, sigc::bind(sigc::mem_fun(this, &dxim::ActionFocusTabitem), i), InputProcessor::BINDABLE_OVERRIDE);
    }
}

void dxim::Resize()
{
    m_mngr->OnScreenResized();
}

void dxim::OnScreenResized()
{
    CppConsUI::Rect size;
    int screen_width = CppConsUI::Curses::getmaxx();
    int screen_height = CppConsUI::Curses::getmaxy();
    int roster_width;
    int log_height;
    if(m_convs_expanded)
    {
        roster_width = 0;
        log_height = 0;
    }
    else
    {
        roster_width = DXPREFERENCES.m_rosterWidth;
        log_height = DXPREFERENCES.m_logHeight;
    }
    int footer_height = 1;
    size.y = 0;
    size.width = screen_width / 100.0 * roster_width;
    size.height = screen_height - footer_height;
    size.x = screen_width - size.width;
    m_areas[ROSTER_AREA] = size;
    size.x = 0;
    size.width = screen_width - m_areas[ROSTER_AREA].width;
    size.height = screen_height / 100.0 * log_height;
    size.y = screen_height - size.height - footer_height;
    m_areas[LOG_AREA] = size;
    size.x = 0;
    size.y = 0;
    size.width = screen_width - m_areas[ROSTER_AREA].width;
    size.height = screen_height - m_areas[LOG_AREA].height - footer_height;
    if(m_convs_expanded)
    {
        size.x -= 2;
        size.width += 4;
    }
    m_areas[CHAT_AREA] = size;
    size.x = 0;
    size.y = screen_height - 1;
    size.width = screen_width;
    size.height = footer_height;
    m_areas[FOOTER_AREA] = size;
    size.x = 0;
    size.y = 0;
    size.width = screen_width;
    size.height = screen_height;
    m_areas[WHOLE_AREA] = size;
}

void dxim::OnTopWindowChanged()
{
    if(!m_convs_expanded)
        return;
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && typeid(IMTabItem) != typeid(*top) && typeid(ChatTabItem) != typeid(*top)
            && typeid(TabItem) != typeid(*top))
    {
        m_convs_expanded = false;
        TABBOOK->SetExpandedTabBook(m_convs_expanded);
        m_mngr->OnScreenResized();
    }
}

void dxim::ActionRosterActive()
{
    ROSTER->Show();
}

void dxim::ActionTabitemActive()
{
    TABBOOK->FocusActiveTabitem();
}

void dxim::ActionStatusMenu()
{
    /* Don't allow to open the menu if there is any 'top' window. */
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && top->GetType() == CppConsUI::FreeWindow::TYPE_TOP)
    {
        return;
    }
    StatusMenu *menu = new StatusMenu();
    menu->Show();
}

void dxim::ActionMainMenu()
{
    /* Don't allow to open the menu if there is any 'top' window. */
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && top->GetType() == CppConsUI::FreeWindow::TYPE_TOP)
    {
        return;
    }
    MainMenu *menu = new MainMenu();
    menu->Show();
}

void dxim::ActionNewImMenu()
{
    if(!GetPurplemm()->isSomeAccountConnected())
    {
        LOG->Warning("[%s] %s", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."));
        return;
    }
    /* Don't allow to open the menu if there is any 'top' window. */
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && top->GetType() == CppConsUI::FreeWindow::TYPE_TOP)
    {
        return;
    }
    NewImMenu *menu = new NewImMenu();
    menu->Show();
}

void dxim::ActionJoinChatMenu()
{
    if(!GetPurplemm()->isSomeAccountConnected())
    {
        LOG->Warning("[%s] %s", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."));
        return;
    }
    /* Don't allow to open the menu if there is any 'top' window. */
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && top->GetType() == CppConsUI::FreeWindow::TYPE_TOP)
    {
        return;
    }
    JoinChatMenu *menu = new JoinChatMenu();
    menu->Show();
}

void dxim::ActionRosterToggleOffline()
{
    DXPREFERENCES.m_hideOffline = !DXPREFERENCES.m_hideOffline;
    ROSTER->ToggleHideOffline();
}

void dxim::ActionTabbookExpand()
{
    CppConsUI::FreeWindow *top = m_mngr->GetTopWindow();
    if(top && top->GetType() == CppConsUI::FreeWindow::TYPE_TOP)
        return;
    if(!m_convs_expanded)
    {
        TABBOOK->FocusActiveTabitem();
        top = m_mngr->GetTopWindow();
        if(!top || (typeid(IMTabItem) != typeid(*top)
                && typeid(ChatTabItem) != typeid(*top)
                && typeid(TabItem) != typeid(*top)))
            return;
    }
    m_convs_expanded = !m_convs_expanded;
    TABBOOK->SetExpandedTabBook(m_convs_expanded);
    m_mngr->OnScreenResized();
}

void dxim::ActionTabitemPrev()
{
    TABBOOK->FocusPrevTabitem();
}

void dxim::ActionTabitemNext()
{
    TABBOOK->FocusNextTabItem();
}

void dxim::ActionFocusTabitem(int i)
{
    TABBOOK->FocusTabitem(i);
}

void dxim::ActionCloseCurrentTabitem()
{
    TABBOOK->CloseCurrentTabitem();
}

void dxim::OnUserInfo(PURPLE::Account account, std::string buddy, std::string text)
{
    std::string id = dxutils::sformat("%s-%s-%s", account.getUsername().c_str(), account.getProtocol().getName().c_str(), buddy.c_str());
    int idn= TABBOOK->FindTabItem(id);
    if(idn == -1)
    {
        TABBOOK->CreateNewItem(id);
    }
    TABBOOK->AppendText(id, text, true);
}

void dxim::OnPurpleError(std::string error)
{
    MsgDialog *dialog = new MsgDialog("", _("Error"));
    dialog->WriteText(error);
    dialog->Show();
}

void dxim::OnNotifyMessage(PurpleNotifyMsgType /*type*/, std::string title, std::string text)
{
    MsgDialog *dialog = new MsgDialog("", title);
    dialog->WriteText(text);
    dialog->Show();
}

void dxim::OnNotifyEmail(std::string text, std::string url)
{
    MsgDialog *dialog = new MsgDialog("", _("New email"));
    dialog->WriteText(text);
    dialog->WriteText(" ");
    dialog->WriteText(dxutils::sformat("URL: %s", url.c_str()));
    dialog->Show();
}

void dxim::OnNotifyUri(std::string uri)
{
    MsgDialog *dialog = new MsgDialog("", _("Link"));
    dialog->WriteText(uri);
    dialog->Show();
}

void dxim::OnNewInputRequest(PURPLE::InputRequest* request)
{
    InputRequestDialog *dialog = new InputRequestDialog(request);
    dialog->Show();
}

void dxim::OnNewActionRequest(PURPLE::ActionRequest* request)
{
    ActionRequestDialog *dialog = new ActionRequestDialog(request);
    dialog->Show();
}

void dxim::OnNewChoiceRequest(PURPLE::ChoiceRequest* request)
{
    ChoiceRequestDialog *dialog = new ChoiceRequestDialog(request);
    dialog->Show();
}

void dxim::OnNewFieldsRequest(PURPLE::FieldsRequest* request)
{
    FieldsRequestDialog *dialog = new FieldsRequestDialog(request);
    dialog->Show();
}

void dxim::OnNewFileRequest(PURPLE::FileRequest* request)
{
    FileRequestDialog *dialog = new FileRequestDialog(request);
    dialog->Show();
}

void dxim::OnNewFileTransfer(PURPLE::FileTransfer* ft)
{
    if(!m_ftdialog)
    {
        m_ftdialog = new FileTransferDialog();
    }
    m_ftdialog->Show();
    m_ftdialog->NewFileTransfer(ft);
}

static void print_usage(FILE *out)
{
    fprintf(out,
            _("\
\nUsage: dxim [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [PATH]          Use PATH for configuration.\n\
        -o, --offline      Start with all accounts set offline.\n\
\n"));
}

static void print_version(FILE *out)
{
  fprintf(out, "dxim-curses %s\n", VERSION);
}

int main(int argc, char **argv)
{
    g_set_prgname(PACKAGE);
    setlocale(LC_ALL, "");
#if ENABLE_NLS
    bindtextdomain("dxim", LOCALEDIR);
    bind_textdomain_codeset("dxim", "UTF-8");
    bind_textdomain_codeset("pidgin", "UTF-8");
    textdomain("dxim");
#endif
    signal(SIGPIPE, SIG_IGN);
    bool offline = false;
    std::string configPath = g_build_filename(g_get_user_config_dir(), "dxim", NULL);
    int opt;
    struct option long_options[] = {
        {"help",      no_argument,       NULL, 'h'},
        {"version",   no_argument,       NULL, 'v'},
        {"configdir", required_argument, NULL, 'l'},
        {"offline",   no_argument,       NULL, 'o'},
        {NULL,        0,                 NULL,  0 }
    };
    while ((opt = getopt_long(argc, argv, "ahvl:o", long_options, NULL)) != -1)
    {
        switch (opt) {
          case 'h':
            print_usage(stdout);
            return 0;
          case 'v':
            print_version(stdout);
            return 0;
          case 'l':
            configPath = optarg;
            break;
          case 'o':
            offline = true;
            break;
          default:
            print_usage(stderr);
            return 1;
        }
    }
    if(optind < argc)
    {
        fprintf(stderr, _("%s: unexpected argument after options\n"), argv[0]);
        print_usage(stderr);
        return 1;
    }
    dxim *im = dxim::Instance();
    return im->Run(configPath, offline);
}

