/*
 *      dxutils.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <ctime>
#include <fstream>
#include "dxutils.h"
#include "gettext.h"

dxutils::dxutils()
{
    m_haveNotify = false;
#ifndef _WIN32
    std::vector<std::string> paths = stringSplit(getenv("PATH"), G_SEARCHPATH_SEPARATOR);
    FOREACH(std::vector<std::string>, paths, it)
    {
        if(g_file_test(g_build_filename((*it).c_str(), "notify-send", NULL), G_FILE_TEST_EXISTS)
            && !g_file_test(g_build_filename((*it).c_str(), "notify-send", NULL), G_FILE_TEST_IS_DIR))
        {
            m_haveNotify = true;
            break;
        }
    }
#endif // _WIN32
}

dxutils::~dxutils()
{}

dxutils &dxutils::instance()
{
    static dxutils iutils;
    return iutils;
}

std::string dxutils::getPresenceString(PurpleStatusPrimitive presence)
{
    switch(presence) {
    case PURPLE_STATUS_UNSET:
    case PURPLE_STATUS_OFFLINE:
    {
        return "[-] ";
    }
    case PURPLE_STATUS_UNAVAILABLE:
    {
        return "[d] ";
    }
    case PURPLE_STATUS_INVISIBLE:
    {
        return "[i] ";
    }
    case PURPLE_STATUS_AWAY:
    {
        return "[a] ";
    }
    case PURPLE_STATUS_EXTENDED_AWAY:
    {
        return "[x] ";
    }
    case PURPLE_STATUS_AVAILABLE:
    case PURPLE_STATUS_MOBILE:
    case PURPLE_STATUS_TUNE:
    default:
    {
        return "[o] ";
    }
    }
}

std::string dxutils::getPresenceScheme(std::string base, PurpleStatusPrimitive presence)
{
    switch(presence) {
    case PURPLE_STATUS_UNSET:
    case PURPLE_STATUS_OFFLINE:
    {
        return base+"_offline";
    }
    case PURPLE_STATUS_UNAVAILABLE:
    {
        return base+"_dnd";
    }
    case PURPLE_STATUS_INVISIBLE:
    {
        return base+"_offline";
    }
    case PURPLE_STATUS_AWAY:
    {
        return base+"_away";
    }
    case PURPLE_STATUS_EXTENDED_AWAY:
    {
        return base+"_away";
    }
    case PURPLE_STATUS_AVAILABLE:
    case PURPLE_STATUS_MOBILE:
    case PURPLE_STATUS_TUNE:
    default:
    {
        return base+"_online";
    }
    }
}

//ChatRole to std::string
std::string dxutils::getChatRoleString(PurpleConvChatBuddyFlags role)
{
    if(role & PURPLE_CBFLAGS_FOUNDER)
        return "*";
    else if(role & PURPLE_CBFLAGS_OP)
        return "@";
    else if(role & PURPLE_CBFLAGS_HALFOP)
        return "%";
    else if(role & PURPLE_CBFLAGS_VOICE)
        return "+";
    else
        return " ";
}

guint dxutils::hash(const std::string& str)
{
    int len = str.length();
    guint h = 0;
    for(int i=0; i<len; i++)
        h = ((h << 5) + h) ^ str[i];
    return h;
}

time_t dxutils::now()
{
    return ::time(NULL);
}

std::string dxutils::time(const char *format, time_t time)
{
    if(!time) time = now();
    char buffer[512];
    int len=strftime(buffer, sizeof(buffer), format, localtime(&time));
    return std::string(buffer,len);
}

std::vector<std::string> dxutils::stringSplit(const std::string &text, char splitter)
{
    std::vector<std::string> v;
    std::string::size_type lastPos = text.find_first_not_of(splitter, 0);
    std::string::size_type pos = text.find_first_of(splitter, lastPos);
    while(std::string::npos != pos || std::string::npos != lastPos)
    {
        v.push_back(text.substr(lastPos, pos - lastPos));
        //needed for empty section
        if(text.find(splitter, pos+1) == pos+1)
        {
            lastPos = text.find(splitter, pos);
            pos = text.find(splitter, pos+1);
        }
        else
        {
            lastPos = text.find_first_not_of(splitter, pos);
            pos = text.find_first_of(splitter, lastPos);
        }
    }
    return v;
}

std::string dxutils::stringSection(const std::string& text, char splitter, int section, bool toEnd)
{
    if(text.empty()) return "";
    int i=0;
    std::string::size_type lastPos = text.find_first_not_of(splitter, 0);
    std::string::size_type pos = text.find_first_of(splitter, lastPos);
    while(std::string::npos != pos || std::string::npos != lastPos)
    {
        if(i==section)
        {
            if(toEnd) return text.substr(lastPos);
            else return text.substr(lastPos, pos - lastPos);
        }
        //needed for empty section
        if(text.find(splitter, pos+1) == pos+1)
        {
            lastPos = text.find(splitter, pos);
            pos = text.find(splitter, pos+1);
        }
        else
        {
            lastPos = text.find_first_not_of(splitter, pos);
            pos = text.find_first_of(splitter, lastPos);
        }
        i++;
    }
    return "";
}

bool dxutils::stringContains(std::string text, std::string sub, bool ignoreCase)
{
    if(ignoreCase)
    {
        text = g_utf8_strdown(text.c_str(), -1);
        sub = g_utf8_strdown(sub.c_str(), -1);
        return text.find(sub)!=std::string::npos;
    }
    else
    {
        return text.find(sub)!=std::string::npos;
    }
}

std::string dxutils::normalizeLabelString(std::string label, char normalize)
{
    std::string text = "";
    FOREACH(std::string, label, it)
    {
        if((*it)==normalize) continue;
        text+=(*it);
    }
    return text;
}

//Return file size in human readable form
std::string dxutils::getFileSize(size_t size)
{
    double fsize = 0.0;
    if(size > 1000000000)
    {
        fsize = size/1073741824.0;
        return sformat("%.2f %s", fsize, _("GB"));
    }
    if(size > 100000)
    {
        fsize = size/1048576.0;
        return sformat("%.2f %s", fsize, _("MB"));
    }
    if(size > 1000)
    {
        fsize = size/1024.0;
        return sformat("%.2f %s", fsize, _("KB"));
    }
    return sformat("%llu %s", size, _("bytes"));
}

//Return remaining time of download/send
std::string dxutils::getRemaining(size_t size, size_t speed)
{
    if(!speed) return "?";
    size_t remainingTime = size/speed;
    if(remainingTime <= 0 || remainingTime > 54000) return "?";
    else
    {
        unsigned int secs = (unsigned int)remainingTime;
        unsigned int hours = remainingTime/3600;
        secs -= hours*3600;
        unsigned int mins = secs/60;
        secs -= mins*60;
        return sformat("%d:%02d:%02d", hours, mins, secs);
    }
}

//typing to human readable string
std::string dxutils::typingToString(std::string nick, bool typing)
{
    if(typing)
        return sformat(_("%s is composing."), nick.c_str());
    else
        return sformat(_("%s stoped compose."), nick.c_str());
}

std::string dxutils::sformat(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    gchar* s = g_strdup_vprintf(fmt, args);
    va_end(args);
    std::string retStr = s;
    g_free(s);
    return retStr;
}

void dxutils::showNotify(std::string text)
{
    if(m_haveNotify)
    {
        gchar* exec = g_strdup_printf("notify-send -i %s/icons/dxim.png dxim '%s'", DXIM_DATADIR, text.c_str());
        system(exec);
        g_free(exec);
    }
}

bool dxutils::haveNotify()
{
    return m_haveNotify;
}

//is file with name on path right dxim log file?
bool dxutils::isRightLogFile(const std::string &path, const std::string &name)
{
    if(g_regex_match_simple("^\\d\\d\\d\\d-\\d\\d-\\d\\d+$", name.c_str(), G_REGEX_CASELESS, G_REGEX_MATCH_ANCHORED))
    {
        std::ifstream file(path.c_str());
        std::string buf;
        if(!file.is_open())
            return false;
        getline(file, buf);
        file.close();
        buf = buf.substr(0, 22);
        if(g_regex_match_simple("^\\[(.*)(\\d\\d:\\d\\d:\\d\\d\\])", buf.c_str(), G_REGEX_CASELESS, G_REGEX_MATCH_ANCHORED))
            return true;
        else
            return false;
    }
    return false;
}


