/*
 *      accountdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "accountdialog.h"
#include "log.h"
#include <errno.h>
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include "cppconsui/MessageDialog.h"
#include <utilsmm.h>

AccountDialog::AccountDialog(PURPLE::Account account)
: SplitDialog(0, 0, 80, 24, _("Modify account"))
{
    this->m_editing = true;
    this->m_protocol = account.getProtocol();
    this->m_account = account;
    this->m_options = account.getOptions();
    CreateWidgets();
}

AccountDialog::AccountDialog(PURPLE::Protocol protocol)
: SplitDialog(0, 0, 80, 24, _("Create new account"))
{
    this->m_editing = false;
    this->m_protocol = protocol;
    this->m_account = PURPLE::Account();
    this->m_options = protocol.getOptions();
    CreateWidgets();
}

void AccountDialog::CreateWidgets()
{
    SetColorScheme("generaldialog");
    CppConsUI::TreeView *tree = new CppConsUI::TreeView(AUTOSIZE, AUTOSIZE);
    SetContainer(*tree);
    CppConsUI::TreeView::NodeReference parent;
    CppConsUI::TreeView::ToggleCollapseButton *b = new CppConsUI::TreeView::ToggleCollapseButton(_("Account settings"));
    b->SetColorScheme("dialoggroup");
    parent = tree->AppendNode(tree->GetRootNode(), *b);
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Protocol"), m_protocol.getName().c_str())));
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    std::string username;
    if(m_editing) username = m_account.getUsername();
    if(splits.size())
    {
        std::string::size_type pos = username.find(splits[0].getSeparator());
        if(m_editing && m_account.isEnabled())
        {
            StringOption *so = new StringOption(_("Username"), username.substr(0, pos).c_str(), false);
            tree->AppendNode(parent, *so);
            m_usernameFields.push_back(so);
        }
        else
        {
            StringOption *so = new StringOption(_("Username"), m_editing?username.substr(0, pos).c_str():"");
            tree->AppendNode(parent, *so);
            m_usernameFields.push_back(so);
        }
        username = username.substr(pos+1);
        for(size_t i=0; i<splits.size(); i++)
        {
            pos = username.find(splits[i+1].getSeparator());
            if(m_editing && m_account.isEnabled())
            {
                StringOption *so = new StringOption(splits[i].getText().c_str(), username.substr(0, pos).c_str(), false);
                tree->AppendNode(parent, *so);
                m_usernameFields.push_back(so);
            }
            else
            {
                StringOption *so = new StringOption(splits[i].getText().c_str(), m_editing?username.substr(0, pos).c_str():"");
                tree->AppendNode(parent, *so);
                m_usernameFields.push_back(so);
            }
            username = username.substr(pos+1);
        }
    }
    else
    {
        if(m_editing && m_account.isEnabled())
        {
            StringOption *so = new StringOption(_("Username"), username.c_str(), false);
            tree->AppendNode(parent, *so);
            m_usernameFields.push_back(so);
        }
        else
        {
            StringOption *so = new StringOption(_("Username"), m_editing?username.c_str():"");
            tree->AppendNode(parent, *so);
            m_usernameFields.push_back(so);
        }
    }
    m_password = new StringOption(_("Password"), m_editing?m_account.getPassword().c_str():"");
    m_password->SetMasked(true);
    tree->AppendNode(parent, *m_password);
    m_rememberPassword = new BooleanOption(_("Remember password"), (!m_editing || m_account.getRememberPassword())?true:false);
    tree->AppendNode(parent, *m_rememberPassword);
    m_alias = new StringOption(_("Alias"), m_editing?m_account.getAlias().c_str():"");
    tree->AppendNode(parent, *m_alias);
    if(m_protocol.hasCheckMail())
    {
        m_checkEmail = new BooleanOption(_("Check new mails"), (m_editing && m_account.getCheckMail())?true:false);
        tree->AppendNode(parent, *m_checkEmail);
    }
    if(!m_editing)
    {
        m_registerAccount = new BooleanOption(_("Register account"), false);
        tree->AppendNode(parent, *m_registerAccount);
    }
    if(m_editing)
    {
        m_enabled = new BooleanOption(_("Enabled"), m_account.isEnabled());
        tree->AppendNode(parent, *m_enabled);
        CppConsUI::Button *btnDelete = new CppConsUI::Button(_("Delete this account"));
        btnDelete->signal_activate.connect(sigc::mem_fun(this, &AccountDialog::OnDelete));
        tree->AppendNode(parent, *btnDelete);
    }
    b = new CppConsUI::TreeView::ToggleCollapseButton(_("Protocol options"));
    b->SetColorScheme("dialoggroup");
    parent = tree->AppendNode(tree->GetRootNode(), *b);
    for(size_t i=0; i<m_options.size(); i++)
    {
        PURPLE::Protocol::Option option = m_options[i];
        std::map<std::string, std::string> values = option.getStringList();
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                StringOption *edit = new StringOption(option.getText().c_str(), option.getStringValue().c_str());
                tree->AppendNode(parent, *edit);
                m_optionsFields.push_back(edit);
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                ChoiceOption *combo = new ChoiceOption(option.getText().c_str());
                std::map<std::string, std::string>::iterator value = values.begin();
                for(; value!=values.end(); ++value)
                {
                    combo->AddOption((*value).first.c_str(), (*value).second.c_str(), option.getStringValue().c_str());
                }
                tree->AppendNode(parent, *combo);
                m_optionsFields.push_back(combo);
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                IntegerOption *edit = new IntegerOption(option.getText().c_str(), option.getIntValue());
                tree->AppendNode(parent, *edit);
                m_optionsFields.push_back(edit);
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                BooleanOption *check = new BooleanOption(option.getText().c_str(), option.getBoolValue());
                tree->AppendNode(parent, *check);
                m_optionsFields.push_back(check);
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                m_optionsFields.push_back(NULL);
                break;
        }
    }
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &AccountDialog::OnOk));
}

void AccountDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

AccountDialog::BooleanOption::BooleanOption(const char *text, bool value)
: CheckBox(text), m_value(value)
{
    g_assert(text);
    SetState(m_value);
    signal_toggle.connect(sigc::mem_fun(this, &BooleanOption::OnToggle));
}

AccountDialog::BooleanOption::~BooleanOption()
{}

void AccountDialog::BooleanOption::OnToggle(CheckBox& /*activator*/, bool new_state)
{
    m_value = new_state;
}

AccountDialog::StringOption::StringOption(const char *text, const char *value, bool editable)
: Button(FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    m_value = value;
    SetValue(value);
    if(editable) signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

AccountDialog::StringOption::~StringOption()
{}

void AccountDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void AccountDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      m_value = PURPLE::p2s(activator.GetText());
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}

AccountDialog::IntegerOption::IntegerOption(const char *text, int value)
: Button(FLAG_VALUE, text), m_value(value)
{
    g_assert(text);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &IntegerOption::OnActivate));
}

AccountDialog::IntegerOption::~IntegerOption()
{}

void AccountDialog::IntegerOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->SetFlags(CppConsUI::TextEntry::FLAG_NUMERIC);
    dialog->signal_response.connect(sigc::mem_fun(this, &IntegerOption::ResponseHandler));
    dialog->Show();
}

void AccountDialog::IntegerOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    const char *text;
    long int i;
    switch (response) {
        case CppConsUI::AbstractDialog::RESPONSE_OK:
            text = activator.GetText();
            errno = 0;
            i = strtol(text, NULL, 10);
            if (errno == ERANGE || i > INT_MAX || i < INT_MIN)
                LOG->Warning(_("Value is out of range."));
            m_value = CLAMP(i, INT_MIN, INT_MAX);
            SetValue(m_value);
            break;
        default:
        break;
    }
}

AccountDialog::ChoiceOption::ChoiceOption(const char *text)
: ComboBox(text)
{
    g_assert(text);
    signal_selection_changed.connect(sigc::mem_fun(this,  &ChoiceOption::OnSelectionChanged));
}

AccountDialog::ChoiceOption::~ChoiceOption()
{
    for(ComboBoxEntries::iterator i = options.begin(); i != options.end(); i++)
        g_free(reinterpret_cast<char*>(i->data));
}

void AccountDialog::ChoiceOption::AddOption(const char *title, const char *value, const char *defaultValue)
{
    g_assert(title);
    g_assert(value);
    g_assert(defaultValue);
    m_value = PURPLE::p2s(defaultValue);
    int item = AddOptionPtr(title, g_strdup(value));
    if(!g_ascii_strcasecmp(defaultValue, value))
        SetSelected(item);
}

void AccountDialog::ChoiceOption::OnSelectionChanged(ComboBox& /*activator*/, int /*new_entry*/, const char * /*title*/, intptr_t data)
{
    m_value = PURPLE::p2s(reinterpret_cast<const char*>(data));
}

std::string AccountDialog::JoinUsername()
{
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    std::string username = m_usernameFields[0]->m_value;
    if(splits.size())
    {
        for(size_t i=0; i<splits.size(); i++)
        {
            username += splits[i].getSeparator();
            username += m_usernameFields[i+1]->m_value;
        }
    }
    return username;
}

void AccountDialog::FillProtocolOptions()
{
    PURPLE::Protocol::Options::iterator it;
    guint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Protocol::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                option.setValue(static_cast<StringOption*>(m_optionsFields[i])->m_value);
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                option.setValue(static_cast<ChoiceOption*>(m_optionsFields[i])->m_value);
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                option.setValue(PURPLE::t2s<int>(static_cast<IntegerOption*>(m_optionsFields[i])->m_value));
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                option.setValue(static_cast<BooleanOption*>(m_optionsFields[i])->m_value ? "true" : "false");
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

void AccountDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    FillProtocolOptions();
    if(m_editing)
    {
        m_account.setOptions(m_options);
        m_account.setPassword(m_password->m_value, m_rememberPassword->m_value);
        m_account.setCheckMail(m_checkEmail->m_value);
        m_account.setAlias(m_alias->m_value);
        if(m_account.isEnabled() && !m_enabled->m_value)
            m_account.disconnect();
        if(!m_account.isEnabled() && m_enabled->m_value)
            m_account.connect();
    }
    else
    {
        PURPLE::Account acc = DXIM->GetPurplemm()->addAccount(m_protocol, JoinUsername(), m_password->m_value, m_rememberPassword->m_value, m_options, m_registerAccount->m_value);
        acc.setCheckMail(m_checkEmail->m_value);
        acc.setAlias(m_alias->m_value);
    }
    Close();
}

void AccountDialog::OnDelete(CppConsUI::Button& /*activator*/)
{
    CppConsUI::MessageDialog *dialog = new CppConsUI::MessageDialog(_("Remove account"), dxutils::sformat(_("Do you want really remove %s?"), m_account.getDisplayName().c_str()).c_str());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &AccountDialog::OnDeleteResponseHandler));
    dialog->Show();
}

void AccountDialog::OnDeleteResponseHandler(CppConsUI::MessageDialog& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        DXIM->GetPurplemm()->deleteAccount(m_account);
        Close();
    }
}

