/*
 *      accountsdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <cppconsui/ComboBox.h>
#include "accountsdialog.h"
#include "accountdialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"

class ProtocolDialog
: public CppConsUI::AbstractDialog
{
public:
    ProtocolDialog() : AbstractDialog(5, 5, 50, 5, _("Select protocol"))
    {
        AddButton(OK_BUTTON_TEXT, RESPONSE_OK);
        SetColorScheme("generaldialog");
        m_box = new CppConsUI::ComboBox(AUTOSIZE, 1, _("Protocol"));
        std::vector<PURPLE::Protocol> v = DXIM->GetPurplemm()->getProtocolsList();
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            m_box->AddOption(it->getName().c_str());
        }
        layout->InsertWidget(0, *m_box);
    }
    virtual ~ProtocolDialog() {}

    sigc::signal<void, ProtocolDialog&, ResponseType> signal_response;

    const char* GetProtocolName() const { return m_box->GetSelectedTitle(); }
protected:
    // AbstractDialog
    void EmitResponse(ResponseType response)
    {
        signal_response(*this, response);
    }
private:
    ProtocolDialog(const ProtocolDialog&);
    ProtocolDialog& operator=(const ProtocolDialog&);

    CppConsUI::ComboBox *m_box;
};

AccountsDialog::AccountsDialog()
: SplitDialog(0, 0, 80, 24, _("Accounts list"))
{
    SetColorScheme("generaldialog");
    m_box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*m_box);
    std::vector<PURPLE::Account> v = DXIM->GetPurplemm()->getAccountsList();
    FOREACH(std::vector<PURPLE::Account>, v, it)
    {
        AddAccount(*it);
    }
    buttons->AppendItem(_("Add"), sigc::mem_fun(this, &AccountsDialog::OnAdd));
    buttons->AppendSeparator();
    buttons->AppendItem(_("Close"), sigc::hide(sigc::mem_fun(this, &CppConsUI::SplitDialog::Close)));
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalAccountAdded.connect(sigc::mem_fun(this, &AccountsDialog::OnAccountAdded));
    cb->signalAccountRemoved.connect(sigc::mem_fun(this, &AccountsDialog::OnAccountDeleted));
}

void AccountsDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void AccountsDialog::OnAccountAdded(PURPLE::Account account)
{
    AddAccount(account);
}

void AccountsDialog::OnAccountDeleted(std::string account, std::string protocolID)
{
    std::vector<AccountItem*>::iterator it = m_items.begin();
    for(; it!=m_items.end(); it++)
    {
        if(account == (*it)->GetAccount().getUsername() &&
            protocolID == (*it)->GetAccount().getProtocol().getPurpleID())
        {
            (*it)->SetVisibility(false);
        }
    }
}

void AccountsDialog::AddAccount(PURPLE::Account account)
{
    AccountItem *item = new AccountItem(account, this);
    m_box->AppendWidget(*item);
    m_items.push_back(item);
}

void AccountsDialog::OnAdd(CppConsUI::Button& /*activator*/)
{
    ProtocolDialog *dialog = new ProtocolDialog();
    dialog->signal_response.connect(sigc::mem_fun(this, &AccountsDialog::OnProtocol));
    dialog->Show();
}

void AccountsDialog::OnProtocol(ProtocolDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        std::vector<PURPLE::Protocol> v = DXIM->GetPurplemm()->getProtocolsList();
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            if(!it->getName().compare(activator.GetProtocolName()))
            {
                AccountDialog *dialog = new AccountDialog(*it);
                dialog->Show();
            }
        }
    }
}

AccountsDialog::AccountItem::AccountItem(PURPLE::Account account, SplitDialog *parent)
: Button(AUTOSIZE, 1, dxutils::sformat("%s [%s]", account.getUsername().c_str(), account.getProtocol().getName().c_str()).c_str()),
  m_account(account), m_parent(parent)
{
    if(account.isEnabled())
        SetColorScheme("rosteraccount_online");
    else
        SetColorScheme("rosteraccount_offline");
    signal_activate.connect(sigc::mem_fun(this, &AccountItem::OnActivate));
}

AccountsDialog::AccountItem::~AccountItem()
{}

void AccountsDialog::AccountItem::OnActivate(CppConsUI::Button& /*activator*/)
{
    AccountDialog *dialog = new AccountDialog(m_account);
    dialog->Show();
    if(m_account.isEnabled())
        SetColorScheme("rosteraccount_online");
    else
        SetColorScheme("rosteraccount_offline");
}

PURPLE::Account AccountsDialog::AccountItem::GetAccount()
{
    return m_account;
}


