/*
 *      chatjoindialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "chatjoindialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include <errno.h>
#include "log.h"
#include <utilsmm.h>

ChatJoinDialog::ChatJoinDialog(PURPLE::Account account)
: SplitDialog(0, 0, 80, 24, _("Connect chat"))
{
    m_account = account;
    SetColorScheme("generaldialog");
    CreateWidgets();
    buttons->AppendItem(_("Join"), sigc::mem_fun(this, &ChatJoinDialog::OnJoin));
}

void ChatJoinDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void ChatJoinDialog::CreateWidgets()
{
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Account"),
                                                 dxutils::sformat("%s [%s]", m_account.getUsername().c_str(), m_account.getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    std::vector<PURPLE::Account::ChatEntry> fields = m_account.getChatParameters();
    IntegerOption *intopt;
    StringOption *stropt;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        if(entry.getIsInt())
        {
            intopt = new IntegerOption(dxutils::normalizeLabelString(entry.getLabel().substr(0, entry.getLabel().size()-1), '_').c_str(),
                                entry.getValue().c_str(), entry.getMin(), entry.getMax());
            box->AppendWidget(*intopt);
            m_fields.push_back(intopt);
        }
        else
        {
            stropt = new StringOption(dxutils::normalizeLabelString(entry.getLabel().substr(0, entry.getLabel().size()-1), '_').c_str(),
                                entry.getValue().c_str());
            stropt->SetMasked(entry.getSecret());
            box->AppendWidget(*stropt);
            m_fields.push_back(stropt);
        }
    }
}

std::vector<PURPLE::Account::ChatEntry> ChatJoinDialog::FillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_account.getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    guint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++, i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        entry.setValue(PURPLE::p2s(m_fields[i]->GetValue()));
    }
    return fields;
}

void ChatJoinDialog::OnJoin(CppConsUI::Button& /*activator*/)
{
    m_account.joinChat(FillChatEntryFields());
    Close();
}

ChatJoinDialog::StringOption::StringOption(const char *text, const char *value)
: Button(AUTOSIZE, 1, FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

ChatJoinDialog::StringOption::~StringOption()
{}

void ChatJoinDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void ChatJoinDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}

ChatJoinDialog::IntegerOption::IntegerOption(const char *text, const char *value, int min, int max)
: Button(AUTOSIZE, 1, FLAG_VALUE, text), m_min(min), m_max(max)
{
    g_assert(text);
    g_assert(value);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &IntegerOption::OnActivate));
}

ChatJoinDialog::IntegerOption::~IntegerOption()
{}

void ChatJoinDialog::IntegerOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->SetFlags(CppConsUI::TextEntry::FLAG_NUMERIC);
    dialog->signal_response.connect(sigc::mem_fun(this, &IntegerOption::ResponseHandler));
    dialog->Show();
}

void ChatJoinDialog::IntegerOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    const char *text;
    long int i;
    int val;
    switch (response) {
        case CppConsUI::AbstractDialog::RESPONSE_OK:
            text = activator.GetText();
            errno = 0;
            i = strtol(text, NULL, 10);
            if (errno == ERANGE || i > INT_MAX || i < INT_MIN)
                LOG->Warning(_("Value is out of range."));
            val = CLAMP(i, m_min, m_max);
            SetValue(val);
            break;
        default:
        break;
    }
}

