/*
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <cppconsui/KeyConfig.h>
#include <cppconsui/HorizontalLine.h>
#include "chattabitem.h"
#include "roster.h"
#include "footer.h"
#include <sys/stat.h>
#include "gettext.h"
#include "dxutils.h"
#include "preferences.h"
#include <utilsmm.h>
#include "dxim.h"

bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

bool compareChatBuddies(std::string i, std::string j)
{
    return g_ascii_strcasecmp(i.c_str(), j.c_str())<0;
}

void sortChatBuddies(std::vector<std::string>& v)
{
    std::sort(v.begin(), v.end(), compareChatBuddies);
}

ChatTabItem::ChatTabItem(PURPLE::Conversation *conv)
: TabItem(conv)
{
    SetColorScheme("tabitem");
    m_topic = new CppConsUI::Label(width - 2, 1);
    m_topicline = new CppConsUI::HorizontalLine(width);
    m_input = new CppConsUI::TextEdit(width - 2, height);
    m_line = new ChatTabItemLine(dxutils::sformat("%s (%s)", m_conv->getName().c_str(), m_conv->getAccount().getUsername().c_str()).c_str());
    AddWidget(*m_topic, 1, 0);
    AddWidget(*m_topicline, 1, 0);
    AddWidget(*m_input, 1, 1);
    AddWidget(*m_line, 0, height);
    m_input->GrabFocus();
    m_commands.push_back("clear");
    m_commands.push_back("help");
    m_commands.push_back("say");
    m_commands.push_back("ignored");
    m_commands.push_back("im");
    std::vector<std::string> cmds = m_conv->getCommandsList();
    FOREACH(std::vector<std::string>, cmds, it)
        m_commands.push_back(*it);
    if(!vectorContains(m_commands, "me"))
        m_commands.push_back("me");
    if(!vectorContains(m_commands, "ignore"))
        m_commands.push_back("ignore");
    if(!vectorContains(m_commands, "unignore"))
        m_commands.push_back("unignore");
    std::sort(m_commands.begin(), m_commands.end());
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalChatBuddyJoined.connect(sigc::mem_fun(this, &ChatTabItem::OnChatBuddyJoined));
    cb->signalChatBuddyLeft.connect(sigc::mem_fun(this, &ChatTabItem::OnChatBuddyLeft));
    cb->signalChatBuddyRenamed.connect(sigc::mem_fun(this, &ChatTabItem::OnChatBuddyRenamed));
    cb->signalChatBuddyUpdated.connect(sigc::mem_fun(this, &ChatTabItem::OnChatBuddyUpdated));
    cb->signalChatTopic.connect(sigc::mem_fun(this, &ChatTabItem::OnChatTopic));
    LOG->Debug("ChatTabItem::ChatTabItem(): this=%p, title=%s", static_cast<void*>(this), m_conv->getTitle().c_str());
}

ChatTabItem::~ChatTabItem()
{
    LOG->Debug("ChatTabItem::~ChatTabItem(): this=%p", static_cast<void*>(this));
}

bool ChatTabItem::ProcessInput(const TermKeyKey& key)
{
    if(m_view->ProcessInput(key))
        return true;
    if(!g_ascii_strcasecmp(KEYCONFIG->TermKeyToString(key), KEYCONFIG->GetKeyBind("window", "close-window")))
        return true;
    if(key.code.sym == TERMKEY_SYM_TAB)
        return ProcessCompletion();
    return Window::ProcessInput(key);
}

void ChatTabItem::MoveResize(int newx, int newy, int neww, int newh)
{
    Window::MoveResize(newx, newy, neww, newh);
    int view_height = height - 5;
    if(view_height < 1)
        view_height = 1;
    int input_height = height - view_height - 2;
    if(input_height < 2)
        input_height = 2;
    m_topic->MoveResize(1, 0, width -2, 1);
    m_topicline->MoveResize(0, 1, width, 1);
    m_view->MoveResize(1, 2, width - 2, view_height);
    m_input->MoveResize(1, view_height + 3, width - 2, input_height);
    m_line->MoveResize(0, view_height + 2, width, 1);
}

ChatTabItem::ChatTabItemLine::ChatTabItemLine(const char *text)
: AbstractLine(AUTOSIZE, 1)
{
    g_assert(text);
    m_text = g_strdup(text);
    m_text_width = CppConsUI::Curses::onscreen_width(m_text);
}

ChatTabItem::ChatTabItemLine::~ChatTabItemLine()
{
    g_free(m_text);
}

void ChatTabItem::ChatTabItemLine::Draw()
{
    ProceedUpdateArea();
    int realw;
    if(!area || (realw = area->getmaxx()) == 0 || area->getmaxy() != 1)
        return;
    int l;
    if(m_text_width + 5 >= static_cast<unsigned>(realw))
        l = 0;
    else
        l = realw - m_text_width - 5;
    // use HorizontalLine colors
    int attrs = GetColorPair("horizontalline", "line");
    area->attron(attrs);
    int i;
    for (i = 0; i < l; i++)
        area->mvaddlinechar(i, 0, CppConsUI::Curses::LINE_HLINE);
    i += area->mvaddstring(i, 0, m_text);
    for (; i < realw; i++)
        area->mvaddlinechar(i, 0, CppConsUI::Curses::LINE_HLINE);
    area->attroff(attrs);
}

void ChatTabItem::OnChatBuddies(PURPLE::Conversation *conv, std::vector<std::string> buddies)
{
    if(m_conv == conv)
    {
        for(size_t i=0; i<buddies.size(); i++)
            m_buddies.push_back(buddies[i]);
        sortChatBuddies(m_buddies);
    }
}

void ChatTabItem::OnChatBuddyJoined(PURPLE::Conversation *conv, std::string buddy)
{
    if(m_conv == conv)
    {
        m_buddies.push_back(buddy);
        sortChatBuddies(m_buddies);
    }
}

void ChatTabItem::OnChatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
{
    if(m_conv == conv)
    {
        for(std::vector<std::string>::iterator it = m_buddies.begin(); it != m_buddies.end(); it++)
        {
            if((*it) == buddy)
            {
                m_buddies.erase(it);
                break;
            }
        }
    }
}

void ChatTabItem::OnChatBuddyRenamed(PURPLE::Conversation *conv, std::string buddy, std::string nbuddy)
{
    if(m_conv == conv)
    {
        for(std::vector<std::string>::iterator it = m_buddies.begin(); it != m_buddies.end(); it++)
        {
            if((*it) == buddy)
            {
                *it = nbuddy;
                break;
            }
        }
        sortChatBuddies(m_buddies);
    }
}

void ChatTabItem::OnChatBuddyUpdated(PURPLE::Conversation */*conv*/, std::string /*buddy*/)
{}

void ChatTabItem::OnChatTopic(PURPLE::Conversation *conv, std::string /*who*/, std::string topic)
{
    if(m_conv == conv)
    {
        m_topic->SetText(topic.c_str());
    }
}

bool ChatTabItem::ProcessCompletion()
{
    std::string commandtext = m_input->GetText();
    if(!g_ascii_strcasecmp(commandtext.substr(0,6).c_str(), "/help ")
           && commandtext.length()>6)
    {
        CompleteHelpCommand();
    }
    else if(commandtext[0] == '/')
    {
        std::string::size_type pos = commandtext.find_first_of(' ');
        std::string command = commandtext.substr(1, pos-1);
        if(pos == std::string::npos
            || !commandtext.substr(pos+1).length())
        {
            for(size_t i=0; i<m_commands.size(); i++)
            {
                if(g_ascii_strcasecmp(command.c_str(), m_commands[i].c_str()) == 0)
                {
                    if((i+1) < m_commands.size())
                    {
                        m_input->SetText(dxutils::sformat("/%s ", m_commands[++i].c_str()).c_str());
                    }
                    else
                    {
                        m_input->SetText(dxutils::sformat("/%s ", m_commands[0].c_str()).c_str());
                    }
                    break;
                }
                else if(g_ascii_strcasecmp(m_commands[i].substr(0, command.length()).c_str(), command.c_str()) == 0)
                {
                    m_input->SetText(dxutils::sformat("/%s ", m_commands[i].c_str()).c_str());
                    break;
                }
            }
        }
    }
    else if(commandtext[0] != '/'
            && (commandtext.find(' ')==std::string::npos || commandtext.substr(commandtext.find(' ')+1).empty()))
    {
        std::string command = commandtext.substr(0, commandtext.find_first_of(':'));
        for(size_t i=0; i<m_buddies.size(); i++)
        {
            if(g_ascii_strcasecmp(command.c_str(), m_buddies[i].c_str()) == 0)
            {
                if((i+1) < m_buddies.size())
                {
                    m_input->SetText(dxutils::sformat("%s: ", m_buddies[++i].c_str()).c_str());
                }
                else
                {
                    m_input->SetText(dxutils::sformat("%s: ", m_buddies[0].c_str()).c_str());
                }
                break;
            }
            else if(g_ascii_strcasecmp(m_buddies[i].substr(0, command.length()).c_str(), command.c_str()) == 0)
            {
                m_input->SetText(dxutils::sformat("%s: ", m_buddies[i].c_str()).c_str());
                break;
            }
        }
    }
    else if(commandtext.find(' ') != std::string::npos)
    {
        m_input->SetText(GetCompletion(commandtext, m_input->GetCursorPos()).c_str());
    }
    return true;
}

PURPLE::ChatBuddy ChatTabItem::GetChatBuddy(std::string buddy)
{
    return m_conv->findChatBuddy(buddy);
}

std::string ChatTabItem::GetCompletion(std::string line, size_t cursorPos)
{
    if(line[cursorPos] == ' ') cursorPos = cursorPos-1;
    int pos = line.substr(0, cursorPos).find_last_of(' ')+1;
    int n = line.find(' ', cursorPos)!=std::string::npos ? line.find(' ', cursorPos)-pos : line.length()-pos;
    std::string toCompletion = line.substr(pos, n);
    std::string nick;
    size_t j = 0;
    for(std::vector<std::string>::iterator it = m_buddies.begin(); it != m_buddies.end(); it++, j++)
    {
        nick = (*it);
        if(g_ascii_strcasecmp(toCompletion.c_str(), nick.c_str()) == 0)
        {
            if((j+1) < m_buddies.size())
            {
                it++;
            }
            else
            {
                it = m_buddies.begin();
            }
            nick = (*it);
            cursorPos = pos+nick.length();
            return line.replace(pos, n, nick);
        }
        else if(g_ascii_strcasecmp(toCompletion.c_str(), nick.substr(0, toCompletion.length()).c_str()) == 0)
        {
            nick = (*it);
            cursorPos = pos+nick.length();
            return line.replace(pos, n, nick);
        }

    }
    return line;
}

