/*
 *      fieldsrequestdialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "gettext.h"
#include "dxim.h"
#include "fieldsrequestdialog.h"
#include "log.h"
#include "dxutils.h"
#include <errno.h>
#include <utilsmm.h>

static bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

FieldsRequestDialog::FieldsRequestDialog(PURPLE::FieldsRequest *request)
: SplitDialog(0, 0, 80, 24, request->getTitle().empty()?_("New request"):request->getTitle().c_str()), m_request(request)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getPrimary().c_str());
    box->AppendWidget(*b);
    if(!m_request->getSecondary().empty())
    {
        b = new CppConsUI::Button(AUTOSIZE, 1, m_request->getSecondary().c_str());
        box->AppendWidget(*b);
    }
    CreateFields(box);
    buttons->AppendItem(dxutils::normalizeLabelString(request->getOkText(), '_').c_str(), sigc::bind(sigc::mem_fun(this, &FieldsRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_OK));
    buttons->AppendSeparator();
    buttons->AppendItem(dxutils::normalizeLabelString(request->getCancelText(), '_').c_str(), sigc::bind(sigc::mem_fun(this, &FieldsRequestDialog::OnResponse), CppConsUI::AbstractDialog::RESPONSE_CANCEL));
    LOG->Debug("FieldsRequestDialog::FieldsRequestDialog(): this=%p", static_cast<void*>(this));
}

void FieldsRequestDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void FieldsRequestDialog::OnResponse(CppConsUI::Button& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        FillFields();
        m_request->acceptRequest();
    }
    else
        m_request->dennyRequest();
    Close();
}

void FieldsRequestDialog::CreateFields(CppConsUI::ListBox *box)
{
    std::vector<PURPLE::FieldsRequest::Field> fields = m_request->getFields();
    std::vector<PURPLE::FieldsRequest::Field>::iterator it = fields.begin();
    std::string group = "";
    for(; it!=fields.end(); it++)
    {
        PURPLE::FieldsRequest::Field field = *it;
        if(group != field.getGroup())
        {
            group = field.getGroup();
            if(!group.empty())
            {
                CppConsUI::Label *l = new CppConsUI::Label(AUTOSIZE, 1, group.c_str());
                l->SetColorScheme("fieldgroup");
                box->AppendWidget(*l);
            }
        }
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            StringField *edit = new StringField(field.getLabel().c_str(), field.getStringValue().c_str(), field.getEditable());
            if(field.getMasked()) edit->SetMasked(true);
            box->AppendWidget(*edit);
            m_fields.push_back(edit);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            IntegerField *edit = new IntegerField(field.getLabel().c_str(), field.getIntValue());
            box->AppendWidget(*edit);
            m_fields.push_back(edit);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            BooleanField *check = new BooleanField(field.getLabel().c_str(), field.getBoolValue());
            box->AppendWidget(*check);
            m_fields.push_back(check);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            ChoiceField *combo = new ChoiceField(field.getLabel().c_str());
            std::vector<std::string> s = field.getLabels();
            std::vector<std::string>::iterator it = s.begin();
            for(; it!=s.end(); it++)
            {
                combo->AddOption((*it).c_str());
            }
            box->AppendWidget(*combo);
            m_fields.push_back(combo);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            if(field.getMultiple())
            {
                ListField *list = new ListField(field.getLabel().c_str());
                std::vector<std::string> s = field.getLabels();
                std::vector<std::string> sel = field.getSelected();
                FOREACH(std::vector<std::string>, s, sit)
                {
                    ListField::ListFieldItem *item = new ListField::ListFieldItem((*sit).c_str(), vectorContains(sel, *sit)?true:false);
                    list->AppendItem(item);
                }
                list->SetHeight(s.size());
                box->AppendWidget(*list);
                m_fields.push_back(list);
            }
            else
            {
                SingleField *combo = new SingleField(field.getLabel().c_str());
                std::vector<std::string> s = field.getLabels();
                std::vector<std::string> sel = field.getSelected();
                FOREACH(std::vector<std::string>, s, sit)
                {
                    combo->AddField((*sit).c_str(), sel[0].c_str());
                }
                box->AppendWidget(*combo);
                m_fields.push_back(combo);
            }
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FIMAGE)
        {
            StringField *img = new StringField(_("Image"), field.getStringValue().c_str(), false);
            box->AppendWidget(*img);
        }
    }
}

void FieldsRequestDialog::FillFields()
{
    PURPLE::FieldsRequest::Fields fields = m_request->getFields();
    PURPLE::FieldsRequest::Fields::iterator it;
    guint i = 0;
    for(it=fields.begin(); it!=fields.end(); it++)
    {
        PURPLE::FieldsRequest::Field& field = *it;
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            field.setValue(static_cast<StringField*>(m_fields[i])->m_value);
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            field.setValue(static_cast<IntegerField*>(m_fields[i])->m_value);
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            field.setValue(static_cast<BooleanField*>(m_fields[i])->m_value);
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            field.setValue(static_cast<ChoiceField*>(m_fields[i])->m_value);
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            std::vector<std::string> v;
            if(field.getMultiple())
            {
                ListField *list = static_cast<ListField*>(m_fields[i]);
                FOREACH(std::vector<ListField::ListFieldItem*>, list->m_fields, it)
                {
                    if((*it)->m_value) v.push_back(PURPLE::p2s((*it)->GetText()));
                }
            }
            else
            {
                v.push_back(static_cast<SingleField*>(m_fields[i])->m_value);
            }
            field.setSelected(v);
            i++;
        }
        else
            continue;
    }
    m_request->setFields(fields);
}

FieldsRequestDialog::BooleanField::BooleanField(const char *text, bool value)
: CheckBox(text), m_value(value)
{
    g_assert(text);
    SetState(m_value);
    signal_toggle.connect(sigc::mem_fun(this, &BooleanField::OnToggle));
}

FieldsRequestDialog::BooleanField::~BooleanField()
{}

void FieldsRequestDialog::BooleanField::OnToggle(CheckBox& /*activator*/, bool new_state)
{
    m_value = new_state;
}

FieldsRequestDialog::StringField::StringField(const char *text, const char *value, bool editable)
: Button(AUTOSIZE, 1, FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    m_value = value;
    SetValue(value);
    if(editable) signal_activate.connect(sigc::mem_fun(this, &StringField::OnActivate));
}

FieldsRequestDialog::StringField::~StringField()
{}

void FieldsRequestDialog::StringField::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringField::ResponseHandler));
    dialog->Show();
}

void FieldsRequestDialog::StringField::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      m_value = PURPLE::p2s(activator.GetText());
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}

FieldsRequestDialog::IntegerField::IntegerField(const char *text, int value)
: Button(AUTOSIZE, 1, FLAG_VALUE, text), m_value(value)
{
    g_assert(text);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &IntegerField::OnActivate));
}

FieldsRequestDialog::IntegerField::~IntegerField()
{}

void FieldsRequestDialog::IntegerField::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->SetFlags(CppConsUI::TextEntry::FLAG_NUMERIC);
    dialog->signal_response.connect(sigc::mem_fun(this, &IntegerField::ResponseHandler));
    dialog->Show();
}

void FieldsRequestDialog::IntegerField::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    const char *text;
    long int i;
    switch (response) {
        case CppConsUI::AbstractDialog::RESPONSE_OK:
            text = activator.GetText();
            errno = 0;
            i = strtol(text, NULL, 10);
            if (errno == ERANGE || i > INT_MAX || i < INT_MIN)
                LOG->Warning(_("Value is out of range."));
            m_value = CLAMP(i, INT_MIN, INT_MAX);
            SetValue(m_value);
            break;
        default:
        break;
    }
}

FieldsRequestDialog::ChoiceField::ChoiceField(const char *text)
: ComboBox(AUTOSIZE, 1, text)
{
    g_assert(text);
    m_value = 0;
    signal_selection_changed.connect(sigc::mem_fun(this,  &ChoiceField::OnSelectionChanged));
}

FieldsRequestDialog::ChoiceField::~ChoiceField()
{
    for(ComboBoxEntries::iterator i = options.begin(); i != options.end(); i++)
        g_free(reinterpret_cast<char*>(i->data));
}

void FieldsRequestDialog::ChoiceField::OnSelectionChanged(ComboBox& /*activator*/, int new_entry, const char * /*title*/, intptr_t /*data*/)
{
    m_value = new_entry;
}

FieldsRequestDialog::ListField::ListField(const char *text)
: HorizontalListBox(AUTOSIZE, 1)
{
    AppendWidget(*(new CppConsUI::Label(text)));
    m_list = new CppConsUI::ListBox(AUTOSIZE, 1);
    AppendWidget(*m_list);
}

void FieldsRequestDialog::ListField::AppendItem(ListFieldItem* item)
{
    m_list->AppendWidget(*item);
    m_fields.push_back(item);
}

FieldsRequestDialog::ListField::ListFieldItem::ListFieldItem(const char *text, bool selected)
{
    SetText(text);
    SetState(selected);
    m_value = selected;
    signal_toggle.connect(sigc::mem_fun(this, &ListFieldItem::OnToggle));
}

void FieldsRequestDialog::ListField::ListFieldItem::OnToggle(CheckBox& /*activator*/, bool new_state)
{
    m_value = new_state;
}

FieldsRequestDialog::SingleField::SingleField(const char *text)
: ComboBox(AUTOSIZE, 1, text)
{
    g_assert(text);
    signal_selection_changed.connect(sigc::mem_fun(this,  &SingleField::OnSelectionChanged));
}

FieldsRequestDialog::SingleField::~SingleField()
{
}

void FieldsRequestDialog::SingleField::AddField(const char *title, const char *selected)
{
    g_assert(title);
    g_assert(selected);
    m_value = PURPLE::p2s(selected);
    int item = AddOption(title);
    if(!g_ascii_strcasecmp(selected, title))
        SetSelected(item);
}

void FieldsRequestDialog::SingleField::OnSelectionChanged(ComboBox& /*activator*/, int /*new_entry*/, const char * title, intptr_t /*data*/)
{
    m_value = PURPLE::p2s(title);
}
