/*
 *      optiondialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "filetransferdialog.h"
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include "preferences.h"
#include <utilsmm.h>

static std::string statusForFT(PURPLE::FileTransfer *ft)
{
    if(!ft->isStarted()) return _("Waiting");
    if(ft->isCanceled()) return _("Canceled");
    if(ft->isFinished()) return _("Finished");
    if(ft->isReceiving()) return dxutils::sformat(_("Downloading %s of %s"), dxutils::getFileSize(ft->getBytesSent()).c_str(), dxutils::getFileSize(ft->getFileSize()).c_str());
    else return dxutils::sformat(_("Uploading %s of %s"), dxutils::getFileSize(ft->getBytesSent()).c_str(), dxutils::getFileSize(ft->getFileSize()).c_str());
}

FileTransferDialog::FileTransferDialog()
: SplitDialog(0, 0, 80, 24, _("File transfers"))
{
    SetColorScheme("generaldialog");
    m_tree = new CppConsUI::TreeView(AUTOSIZE, AUTOSIZE);
    SetContainer(*m_tree);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &FileTransferDialog::OnOk));
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalFileTransferStarted.connect(sigc::mem_fun(this, &FileTransferDialog::OnFileTransferStarted));
}

void FileTransferDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void FileTransferDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    Hide();
}

void FileTransferDialog::NewFileTransfer(PURPLE::FileTransfer* ft)
{
    if(ft->isSending())
    {
        FileTransferItem *item = new FileTransferItem(ft);
        CppConsUI::TreeView::NodeReference ref = m_tree->AppendNode(m_tree->GetRootNode(), *item);
        item->SetRefNode(ref, m_tree);
    }
}

void FileTransferDialog::OnFileTransferStarted(PURPLE::FileTransfer* ft)
{
    if(ft->isReceiving())
    {
        FileTransferItem *item = new FileTransferItem(ft);
        CppConsUI::TreeView::NodeReference ref = m_tree->AppendNode(m_tree->GetRootNode(), *item);
        item->SetRefNode(ref, m_tree);
    }
}

FileTransferDialog::FileTransferItem::FileTransferItem(PURPLE::FileTransfer* ft)
: ToggleCollapseButton(ft->getFileName().c_str()), m_ft(ft)
{
}

void FileTransferDialog::FileTransferItem::SetRefNode(CppConsUI::TreeView::NodeReference n, CppConsUI::TreeView *tree)
{
    m_ref = n;
    m_account = new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Account"), dxutils::sformat("%s [%s]", m_ft->getAccount().getUsername().c_str(), m_ft->getAccount().getProtocol().getName().c_str()).c_str());
    tree->AppendNode(m_ref, *m_account);
    m_direction = new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, m_ft->isReceiving()?_("From"):_("To"), m_ft->getRemoteUser().c_str());
    tree->AppendNode(m_ref, *m_direction);
    m_path = new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Path"), m_ft->getLocalFileName().c_str());
    tree->AppendNode(m_ref, *m_path);
    m_progress = new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Status"), statusForFT(m_ft).c_str());
    tree->AppendNode(m_ref, *m_progress);
    m_remaining = new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Remaining time"), "--:--");
    tree->AppendNode(m_ref, *m_remaining);
    m_cancel = new CppConsUI::CheckBox(_("Cancel"), false);
    m_cancel->signal_toggle.connect(sigc::mem_fun(this, &FileTransferItem::OnCancelToggle));
    tree->AppendNode(m_ref, *m_cancel);
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalFileTransferStarted.connect(sigc::mem_fun(this, &FileTransferItem::OnFileTransferStarted));
    cb->signalFileTransferProgress.connect(sigc::mem_fun(this, &FileTransferItem::OnFileTransferProgress));
    cb->signalFileTransferFinished.connect(sigc::mem_fun(this, &FileTransferItem::OnFileTransferFinished));
    cb->signalFileTransferCanceled.connect(sigc::mem_fun(this, &FileTransferItem::OnFileTransferCanceled));
}

void FileTransferDialog::FileTransferItem::OnCancelToggle(CppConsUI::CheckBox& /*activator*/, bool new_state)
{
    if(new_state) m_ft->cancelFT();
}

void FileTransferDialog::FileTransferItem::OnFileTransferStarted(PURPLE::FileTransfer* ft)
{
    if(m_ft == ft && ft->isSending()) OnFileTransferProgress(ft);
}

void FileTransferDialog::FileTransferItem::OnFileTransferProgress(PURPLE::FileTransfer* ft)
{
    if(m_ft == ft)
    {
        m_progress->SetValue(statusForFT(m_ft).c_str());
        m_remaining->SetValue(dxutils::getRemaining(m_ft->getFileSize()-m_ft->getBytesSent(), m_ft->getSpeed()).c_str());
    }
}

void FileTransferDialog::FileTransferItem::OnFileTransferFinished(PURPLE::FileTransfer* ft)
{
    if(m_ft == ft)
    {
        m_progress->SetValue(statusForFT(m_ft).c_str());
        m_remaining->SetValue("--:--");
        m_cancel->SetVisibility(false);
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(dxutils::sformat(_("Transfer of %s finished"), ft->getFileName().c_str()));
    }
}

void FileTransferDialog::FileTransferItem::OnFileTransferCanceled(PURPLE::FileTransfer* ft)
{
    if(m_ft == ft)
    {
        m_progress->SetValue(statusForFT(m_ft).c_str());
        m_remaining->SetValue("--:--");
        m_cancel->SetVisibility(false);
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(dxutils::sformat(_("Transfer of %s canceled"), ft->getFileName().c_str()));
    }
}

