/*
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <cppconsui/KeyConfig.h>
#include "imtabitem.h"
#include "roster.h"
#include "footer.h"
#include <sys/stat.h>
#include "gettext.h"
#include "dxutils.h"
#include "preferences.h"
#include <utilsmm.h>
#include <fstream>

static bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

IMTabItem::IMTabItem(PURPLE::Conversation *conv)
: TabItem(conv)
{
    SetColorScheme("tabitem");
    m_input = new CppConsUI::TextEdit(width - 2, height);
    m_input->signal_text_change.connect(sigc::mem_fun(this, &IMTabItem::OnInputTextChange));
    PURPLE::Buddy buddy(m_conv->getAccount().findBuddy(m_conv->getName()));
    m_line = new IMTabItemLine(dxutils::sformat("%s%s (%s)", dxutils::getPresenceString(buddy.isValid()?buddy.getStatusPrimitive():PURPLE_STATUS_OFFLINE).c_str(), m_conv->getName().c_str(),
                                 m_conv->getAccount().getUsername().c_str()).c_str());
    AddWidget(*m_input, 1, 1);
    AddWidget(*m_line, 0, height);
    m_input->GrabFocus();
    m_commands.push_back("clear");
    m_commands.push_back("help");
    m_commands.push_back("say");
    std::vector<std::string> cmds = m_conv->getCommandsList();
    FOREACH(std::vector<std::string>, cmds, it)
        m_commands.push_back(*it);
    if(!vectorContains(m_commands, "me"))
        m_commands.push_back("me");
    std::sort(m_commands.begin(), m_commands.end());
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalBuddySignedOn.connect(sigc::mem_fun(this, &IMTabItem::OnBuddySignedOn));
    cb->signalBuddySignedOff.connect(sigc::mem_fun(this, &IMTabItem::OnBuddySignedOff));
    cb->signalBuddyStatusChanged.connect(sigc::mem_fun(this, &IMTabItem::OnBuddyStatusChanged));

    COREMANAGER->TimeoutConnect(sigc::mem_fun(this, &IMTabItem::OnTimeout), 2000);
    
    LOG->Debug("IMTabItem::IMTabItem(): this=%p, title=%s", static_cast<void*>(this), m_conv->getTitle().c_str());
}

IMTabItem::~IMTabItem()
{
    LOG->Debug("IMTabItem::~IMTabItem(): this=%p", static_cast<void*>(this));
}

bool IMTabItem::ProcessInput(const TermKeyKey& key)
{
    if(m_view->ProcessInput(key))
        return true;
    if(!g_ascii_strcasecmp(KEYCONFIG->TermKeyToString(key), KEYCONFIG->GetKeyBind("window", "close-window")))
        return true;
    if(key.code.sym == TERMKEY_SYM_TAB)
        return ProcessCompletion();
    return Window::ProcessInput(key);
}

void IMTabItem::OnInputTextChange(CppConsUI::TextEdit& /*activator*/)
{
    m_lastKeyPress = dxutils::now();
}

void IMTabItem::MoveResize(int newx, int newy, int neww, int newh)
{
    Window::MoveResize(newx, newy, neww, newh);
    int view_height = height - 3;
    if(view_height < 1)
        view_height = 1;
    int input_height = height - view_height - 2;
    if(input_height < 2)
        input_height = 2;
    m_view->MoveResize(1, 0, width - 2, view_height);
    m_input->MoveResize(1, view_height + 1, width - 2, input_height);
    m_line->MoveResize(0, view_height, width, 1);
}

bool IMTabItem::OnTimeout()
{
    if(!m_lastKeyPress) return true;
    if(dxutils::now()-m_lastKeyPress>5)
    {
        if(m_imState==STARTED)
        {
            m_imState = PAUSED;
            m_conv->sendTyping(false);
        }
        return true;
    }
    else
    {
        if(m_imState==ACTIVE || m_imState==PAUSED)
        {
            m_imState = STARTED;
            m_conv->sendTyping(true);
        }
        return true;
    }
    return true;
}

IMTabItem::IMTabItemLine::IMTabItemLine(const char *text)
: AbstractLine(AUTOSIZE, 1)
{
    g_assert(text);
    m_text = g_strdup(text);
    m_text_width = CppConsUI::Curses::onscreen_width(m_text);
}

IMTabItem::IMTabItemLine::~IMTabItemLine()
{
    g_free(m_text);
}

void IMTabItem::IMTabItemLine::SetText(const char* text)
{
    g_assert(text);
    m_text = g_strdup(text);
    m_text_width = CppConsUI::Curses::onscreen_width(m_text);
}

void IMTabItem::IMTabItemLine::Draw()
{
    ProceedUpdateArea();
    int realw;
    if(!area || (realw = area->getmaxx()) == 0 || area->getmaxy() != 1)
        return;
    int l;
    if(m_text_width + 5 >= static_cast<unsigned>(realw))
        l = 0;
    else
        l = realw - m_text_width - 5;
    // use HorizontalLine colors
    int attrs = GetColorPair("horizontalline", "line");
    area->attron(attrs);
    int i;
    for (i = 0; i < l; i++)
        area->mvaddlinechar(i, 0, CppConsUI::Curses::LINE_HLINE);
    i += area->mvaddstring(i, 0, m_text);
    for (; i < realw; i++)
        area->mvaddlinechar(i, 0, CppConsUI::Curses::LINE_HLINE);
    area->attroff(attrs);
}

bool IMTabItem::ProcessCompletion()
{
    std::string commandtext = m_input->GetText();
    if(!g_ascii_strcasecmp(commandtext.substr(0,6).c_str(), "/help ")
           && commandtext.length()>6)
    {
        CompleteHelpCommand();
    }
    else if(commandtext[0] == '/')
    {
        std::string::size_type pos = commandtext.find_first_of(' ');
        std::string command = commandtext.substr(1, pos-1);
        if(pos == std::string::npos
            || !commandtext.substr(pos+1).length())
        {
            for(size_t i=0; i<m_commands.size(); i++)
            {
                if(g_ascii_strcasecmp(command.c_str(), m_commands[i].c_str()) == 0)
                {
                    if((i+1) < m_commands.size())
                    {
                        m_input->SetText(dxutils::sformat("/%s ", m_commands[++i].c_str()).c_str());
                    }
                    else
                    {
                        m_input->SetText(dxutils::sformat("/%s ", m_commands[0].c_str()).c_str());
                    }
                    break;
                }
                else if(g_ascii_strcasecmp(m_commands[i].substr(0, command.length()).c_str(), command.c_str()) == 0)
                {
                    m_input->SetText(dxutils::sformat("/%s ", m_commands[i].c_str()).c_str());
                    break;
                }
            }
        }
    }
    return true;
}

void IMTabItem::OnBuddySignedOn(PURPLE::Buddy buddy)
{
    if(m_conv->getName() == buddy.getUsername())
    {
        m_line->SetText(dxutils::sformat("%s%s (%s)", dxutils::getPresenceString(buddy.isValid()?buddy.getStatusPrimitive():PURPLE_STATUS_OFFLINE).c_str(), m_conv->getName().c_str(),
                                 m_conv->getAccount().getUsername().c_str()).c_str());
    }
}

void IMTabItem::OnBuddySignedOff(PURPLE::Buddy buddy)
{
    if(m_conv->getName() == buddy.getUsername())
    {
        m_line->SetText(dxutils::sformat("%s%s (%s)", dxutils::getPresenceString(buddy.isValid()?buddy.getStatusPrimitive():PURPLE_STATUS_OFFLINE).c_str(), m_conv->getName().c_str(),
                                 m_conv->getAccount().getUsername().c_str()).c_str());
    }
}

void IMTabItem::OnBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive /*status*/)
{
    if(m_conv->getName() == buddy.getUsername())
    {
        m_line->SetText(dxutils::sformat("%s%s (%s)", dxutils::getPresenceString(buddy.isValid()?buddy.getStatusPrimitive():PURPLE_STATUS_OFFLINE).c_str(), m_conv->getName().c_str(),
                                 m_conv->getAccount().getUsername().c_str()).c_str());
    }
}

void IMTabItem::LoadCurrentLog()
{
    std::string folder = m_conv->getName();
    folder = folder.substr(0, folder.find_last_of('/'));
    std::string username = m_conv->getAccount().getUsername();
    username = username.substr(0, username.find_last_of('/'));
    std::string name = purple_utf8_strftime("%Y-%m-%d", NULL);
    std::string path = g_build_filename(DXPREFERENCES.m_logPath.c_str(), username.c_str(), folder.c_str(), name.c_str(), NULL);
    if(!dxutils::isRightLogFile(path, name))
    {
        return;
    }
    std::ifstream file(path.c_str());
    if(!file.is_open())
    {
        return;
    }
    std::string line;
    while(getline(file, line))
    {
        std::string::size_type pos = line.find(']')+2;
        if(pos<2) pos = 0;
        if(line.length()>pos+1 && line[pos]=='\014') //log style mark
        {
            std::string style = dxutils::stringSection(line, '\014', 1);
            if(dxutils::stringSection(line, '\014', 2, true)[0]=='<') //has nick
            {
                int color = 0;
                std::string nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
                std::string msg = dxutils::stringSection(line, '>',1, true);
                if(style == "mymsg")
                {
                    color = 1;
                }
                else if(style == "highlight")
                {
                    color = 5;
                }
                m_view->Append(PURPLE::stripHtml(dxutils::stringSection(line, '\014', 0)+nick+":"+msg).c_str(), color);
            }
            else
            {
                int color = 0;
                if(style == "action")
                {
                    color = 2;
                    
                }
                else if(style == "error")
                {
                    color = 3;
                }
                else if(style == "user")
                {
                    color = 4;
                }
                m_view->Append(PURPLE::stripHtml(dxutils::stringSection(line, '\014', 0)+dxutils::stringSection(line, '\014', 2, true)).c_str(), color);
            }
        }
        else
        {
            if(line.length()>pos+1 && line[pos]=='<') //line with nick
            {
                std::string nick = dxutils::stringSection(dxutils::stringSection(line, '<', 1), '>', 0);
                std::string msg = dxutils::stringSection(line, '>',1, true);
                m_view->Append(PURPLE::stripHtml(dxutils::stringSection(line, '<', 0)+nick+":"+msg).c_str());
                
            }
            else
            {
                m_view->Append(PURPLE::stripHtml(line).c_str());
            }
        }
    }
    file.close();
}

