/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "log.h"
#include <cppconsui/HorizontalListBox.h>
#include <cppconsui/Spacer.h>
#include "gettext.h"

Log *Log::m_instance = NULL;

Log *Log::Instance()
{
    return m_instance;
}

void Log::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::LOG_AREA));
}

#define WRITE_METHOD(name, level)                       \
void Log::name(const char *fmt, ...)                    \
{                                                       \
  va_list args;                                         \
  char *text;                                           \
  va_start(args, fmt);                                  \
  text = g_strdup_vprintf(fmt, args);                   \
  va_end(args);                                         \
  switch(level) {                                       \
  case LEVEL_NONE:                                      \
  case LEVEL_MESSAGE: Write(text); break;               \
  case LEVEL_DEBUG: Write(text, 1); break;              \
  case LEVEL_WARNING: Write(text, 2); break;            \
  case LEVEL_ERROR: Write(text, 3); break;              \
  default: Write(text);                                 \
  }                                                     \
  g_free(text);                                         \
}

WRITE_METHOD(Error, LEVEL_ERROR)
WRITE_METHOD(Warning, LEVEL_WARNING)
WRITE_METHOD(Message, LEVEL_MESSAGE)
#ifdef DEBUG
WRITE_METHOD(Debug, LEVEL_DEBUG)
#else
void Log::Debug(const char* fmt, ...)
{}
#endif

#undef WRITE_METHOD

Log::Log()
: Window(0, 0, 80, 24, NULL, TYPE_NON_FOCUSABLE)
{
    SetColorScheme("log");
    CppConsUI::HorizontalListBox *lbox = new CppConsUI::HorizontalListBox(AUTOSIZE, AUTOSIZE);
    AddWidget(*lbox, 0, 0);
    lbox->AppendWidget(*(new CppConsUI::Spacer(1, AUTOSIZE)));
    m_textview = new CppConsUI::TextView(AUTOSIZE, AUTOSIZE, true);
    lbox->AppendWidget(*m_textview);
    lbox->AppendWidget(*(new CppConsUI::Spacer(1, AUTOSIZE)));
    
    #define REGISTER_G_LOG_HANDLER(name, handler) g_log_set_handler((name), (GLogLevelFlags)G_LOG_LEVEL_MASK, (handler), this)
    // register the glib log handlers
    m_default_handler = REGISTER_G_LOG_HANDLER(NULL, default_log_handler);
    m_glib_handler = REGISTER_G_LOG_HANDLER("GLib", glib_log_handler);
    m_gmodule_handler = REGISTER_G_LOG_HANDLER("GModule", glib_log_handler);
    m_glib_gobject_handler = REGISTER_G_LOG_HANDLER("GLib-GObject", glib_log_handler);
    m_gthread_handler = REGISTER_G_LOG_HANDLER("GThread", glib_log_handler);
    m_cppconsui_handler = REGISTER_G_LOG_HANDLER("cppconsui", cppconsui_log_handler);
}

Log::~Log()
{
    g_log_remove_handler(NULL, m_default_handler);
    g_log_remove_handler("GLib", m_glib_handler);
    g_log_remove_handler("GModule", m_gmodule_handler);
    g_log_remove_handler("GLib-GObject", m_glib_gobject_handler);
    g_log_remove_handler("GThread", m_gthread_handler);
    g_log_remove_handler("cppconsui", m_cppconsui_handler);
}

void Log::Init()
{
    g_assert(!m_instance);
    m_instance = new Log;
    m_instance->Show();
}

void Log::Finalize()
{
    g_assert(m_instance);
    delete m_instance;
    m_instance = NULL;
}

void Log::ShortenWindowText()
{
    size_t lines_num = m_textview->GetLinesNumber();
    if(lines_num > 200) {
        // remove 40 extra lines
        m_textview->Erase(0, lines_num - 200 + 40);
    }
}

void Log::Write(const char *text, int color)
{
    m_textview->Append(text, color);
    ShortenWindowText();
}

void Log::logHandler(const char *domain, const char *msg)
{
    if(!msg)
        return;
    Debug("%s: %s", domain ? domain : "g_log", msg);
}

