/*
 *      optiondialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "optiondialog.h"
#include "roster.h"
#include "log.h"
#include <cppconsui/TreeView.h>
#include <errno.h>
#include "gettext.h"
#include "preferences.h"
#include "dxutils.h"
#include "dxim.h"
#include "tabbook.h"
#include <utilsmm.h>

OptionDialog::OptionDialog()
: SplitDialog(0, 0, 80, 24, _("Preferences"))
{
    SetColorScheme("generaldialog");
    CppConsUI::TreeView *tree = new CppConsUI::TreeView(AUTOSIZE, AUTOSIZE);
    SetContainer(*tree);
    //General settings
    CppConsUI::TreeView::NodeReference parent;
    CppConsUI::TreeView::ToggleCollapseButton *b = new CppConsUI::TreeView::ToggleCollapseButton(_("General"));
    b->SetColorScheme("dialoggroup");
    parent = tree->AppendNode(tree->GetRootNode(), *b);
    m_useAway = new BooleanOption(_("Use away"), DXPREFERENCES.m_useAway);
    tree->AppendNode(parent, *m_useAway);
    m_awayTimeout = new IntegerOption(_("Time for status \"Away\" in minutes"), DXPREFERENCES.m_awayTimeout);
    tree->AppendNode(parent, *m_awayTimeout);
    m_logging = new BooleanOption(_("Logging chats"), DXPREFERENCES.m_logging);
    tree->AppendNode(parent, *m_logging);
    m_logPath = new StringOption(_("Log path"), DXPREFERENCES.m_logPath.c_str());
    tree->AppendNode(parent, *m_logPath);
    m_logLoadCurrent = new BooleanOption(_("Load current log for opened IM"), DXPREFERENCES.m_logging?DXPREFERENCES.m_logLoadCurrent:false);
    tree->AppendNode(parent, *m_logLoadCurrent);
    m_receiptsMark = new BooleanOption(_("Use delivery receipts mark on XMPP protocol"), DXPREFERENCES.m_receiptsMark);
    tree->AppendNode(parent, *m_receiptsMark);
    //Notification settings
    if(dxutils::instance().haveNotify())
    {
        b = new CppConsUI::TreeView::ToggleCollapseButton(_("Notification"));
        b->SetColorScheme("dialoggroup");
        parent = tree->AppendNode(tree->GetRootNode(), *b);
        m_notify = new BooleanOption(_("Use notify popup"), DXPREFERENCES.m_notify);
        tree->AppendNode(parent, *m_notify);
        m_notifyAccountConnect = new BooleanOption(_("Account connected"), DXPREFERENCES.m_notifyAccountConnect);
        tree->AppendNode(parent, *m_notifyAccountConnect);
        m_notifyAccountDisconnect = new BooleanOption(_("Account disconnected"), DXPREFERENCES.m_notifyAccountDisconnect);
        tree->AppendNode(parent, *m_notifyAccountDisconnect);
        m_notifyBuddyConnect = new BooleanOption(_("Friend connected"), DXPREFERENCES.m_notifyBuddyConnect);
        tree->AppendNode(parent, *m_notifyBuddyConnect);
        m_notifyBuddyDisconnect = new BooleanOption(_("Friend disconnected"), DXPREFERENCES.m_notifyBuddyDisconnect);
        tree->AppendNode(parent, *m_notifyBuddyDisconnect);
        m_notifyMessage = new BooleanOption(_("Highlighted or im message"), DXPREFERENCES.m_notifyMessage);
        tree->AppendNode(parent, *m_notifyMessage);
        m_notifyFiletransfer = new BooleanOption(_("File transfer finished or canceled"), DXPREFERENCES.m_notifyFiletransfer);
        tree->AppendNode(parent, *m_notifyFiletransfer);
    }
    //Other settings value
    b = new CppConsUI::TreeView::ToggleCollapseButton(_("Other"));
    b->SetColorScheme("dialoggroup");
    parent = tree->AppendNode(tree->GetRootNode(), *b);
    m_hideOffline = new BooleanOption(_("Hide offline friends"), DXPREFERENCES.m_hideOffline);
    tree->AppendNode(parent, *m_hideOffline);
    m_showStatusmsg = new BooleanOption(_("Show friends status message"), DXPREFERENCES.m_showStatusmsg);
    tree->AppendNode(parent, *m_showStatusmsg);
    m_showRosterTree = new BooleanOption(_("Show accounts and groups in friends list"), DXPREFERENCES.m_showRosterTree);
    tree->AppendNode(parent, *m_showRosterTree);
    m_rosterWidth = new IntegerOption(_("Roster width"), DXPREFERENCES.m_rosterWidth);
    tree->AppendNode(parent, *m_rosterWidth);
    m_logHeight = new IntegerOption(_("Log height"), DXPREFERENCES.m_logHeight);
    tree->AppendNode(parent, *m_logHeight);
    buttons->AppendItem(_("Done"), sigc::mem_fun(this, &OptionDialog::OnDone));
}

void OptionDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

OptionDialog::BooleanOption::BooleanOption(const char *text, bool value)
: CheckBox(text), m_value(value)
{
    g_assert(text);
    SetState(m_value);
    signal_toggle.connect(sigc::mem_fun(this, &BooleanOption::OnToggle));
}

OptionDialog::BooleanOption::~BooleanOption()
{}

void OptionDialog::BooleanOption::OnToggle(CheckBox& /*activator*/, bool new_state)
{
    m_value = new_state;
}

OptionDialog::StringOption::StringOption(const char *text, const char *value)
: Button(FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    m_value = value;
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

OptionDialog::StringOption::~StringOption()
{}

void OptionDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void OptionDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      m_value = PURPLE::p2s(activator.GetText());
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}

OptionDialog::IntegerOption::IntegerOption(const char *text, int value)
: Button(FLAG_VALUE, text), m_value(value)
{
    g_assert(text);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &IntegerOption::OnActivate));
}

OptionDialog::IntegerOption::~IntegerOption()
{}

void OptionDialog::IntegerOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->SetFlags(CppConsUI::TextEntry::FLAG_NUMERIC);
    dialog->signal_response.connect(sigc::mem_fun(this, &IntegerOption::ResponseHandler));
    dialog->Show();
}

void OptionDialog::IntegerOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    const char *text;
    long int i;
    switch (response) {
        case CppConsUI::AbstractDialog::RESPONSE_OK:
            text = activator.GetText();
            errno = 0;
            i = strtol(text, NULL, 10);
            if (errno == ERANGE || i > INT_MAX || i < INT_MIN)
                LOG->Warning(_("Value is out of range."));
            m_value = CLAMP(i, INT_MIN, INT_MAX);
            SetValue(m_value);
            break;
        default:
        break;
    }
}

void OptionDialog::OnDone(CppConsUI::Button& /*activator*/)
{
    //General settings
    DXPREFERENCES.m_useAway = m_useAway->m_value;
    DXPREFERENCES.m_awayTimeout = m_awayTimeout->m_value;
    DXPREFERENCES.m_logging = m_logging->m_value;
    DXPREFERENCES.m_logPath = m_logPath->m_value;
    DXPREFERENCES.m_logLoadCurrent = m_logLoadCurrent->m_value;
    //Notification settings
    if(dxutils::instance().haveNotify())
    {
        DXPREFERENCES.m_notify = m_notify->m_value;
        DXPREFERENCES.m_notifyAccountConnect = m_notifyAccountConnect->m_value;
        DXPREFERENCES.m_notifyAccountDisconnect = m_notifyAccountDisconnect->m_value;
        DXPREFERENCES.m_notifyBuddyConnect = m_notifyBuddyConnect->m_value;
        DXPREFERENCES.m_notifyBuddyDisconnect = m_notifyBuddyDisconnect->m_value;
        DXPREFERENCES.m_notifyMessage = m_notifyMessage->m_value;
        DXPREFERENCES.m_notifyFiletransfer = m_notifyFiletransfer->m_value;
    }
    //Other settings value
    DXPREFERENCES.m_hideOffline = m_hideOffline->m_value;
    DXPREFERENCES.m_showStatusmsg = m_showStatusmsg->m_value;
    DXPREFERENCES.m_showRosterTree = m_showRosterTree->m_value;
    DXPREFERENCES.m_rosterWidth = m_rosterWidth->m_value;
    DXPREFERENCES.m_logHeight = m_logHeight->m_value;
    DXIM->Resize();
    DXIM->GetPurplemm()->setIdle(m_useAway->m_value, m_awayTimeout->m_value);
    //works for hideOffline and showStatusmsg
    ROSTER->ToggleHideOffline();
    ROSTER->ToggleShowTree();
    TABBOOK->StopLogging();
    DXPREFERENCES.save();
    Close();
}

