/*
 *      plugindialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "plugindialog.h"
#include "log.h"
#include <errno.h>
#include "gettext.h"
#include "dxutils.h"
#include "dxim.h"
#include <utilsmm.h>

PluginDialog::PluginDialog(PURPLE::Plugin plugin)
: SplitDialog(0, 0, 80, 24, _("Modify plugin"))
{
    this->m_plugin = plugin;
    this->m_options = plugin.getOptions();
    CreateWidgets();
}

void PluginDialog::CreateWidgets()
{
    SetColorScheme("generaldialog");
    CppConsUI::TreeView *tree = new CppConsUI::TreeView(AUTOSIZE, AUTOSIZE);
    SetContainer(*tree);
    CppConsUI::TreeView::NodeReference parent;
    CppConsUI::TreeView::ToggleCollapseButton *b = new CppConsUI::TreeView::ToggleCollapseButton(_("Plugin"));
    b->SetColorScheme("dialoggroup");
    parent = tree->AppendNode(tree->GetRootNode(), *b);
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Name"), m_plugin.getName().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Summary"), m_plugin.getSummary().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Version"), m_plugin.getVersion().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Description"), m_plugin.getDescription().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Author"), m_plugin.getAuthor().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Path"), m_plugin.getPath().c_str())));
    tree->AppendNode(parent, *(new CppConsUI::Button(CppConsUI::Button::FLAG_VALUE, _("Homepage"), m_plugin.getHomepage().c_str())));
    if(m_plugin.isLoadable())
    {
        m_enabled = new BooleanOption(_("Enabled"), m_plugin.isLoaded());
        tree->AppendNode(parent, *m_enabled);
    }
    if(m_plugin.isLoadable() && m_plugin.hasOptions())
    {
        b = new CppConsUI::TreeView::ToggleCollapseButton(_("Options"));
        b->SetColorScheme("dialoggroup");
        parent = tree->AppendNode(tree->GetRootNode(), *b);
        for(size_t i=0; i<m_options.size(); i++)
        {
            PURPLE::Plugin::Option option = m_options[i];
            std::vector<std::string> svalues = option.getStringList();
            std::map<std::string, int> ivalues = option.getIntList();
            switch(option.getType())
            {
                case PURPLE::Protocol::Option::STR:
                {
                    StringOption *edit = new StringOption(dxutils::normalizeLabelString(option.getText(), '_').c_str(), option.getStringValue().c_str());
                    tree->AppendNode(parent, *edit);
                    m_optionsFields.push_back(edit);
                    break;
                }
                case PURPLE::Protocol::Option::STRLIST:
                {
                    ChoiceOption *combo = new ChoiceOption(dxutils::normalizeLabelString(option.getText(), '_').c_str());
                    std::vector<std::string>::iterator value = svalues.begin();
                    for(; value!=svalues.end(); ++value)
                    {
                        combo->AddOption((*value).c_str(), (*value).c_str(), option.getStringValue().c_str());
                    }
                    tree->AppendNode(parent, *combo);
                    m_optionsFields.push_back(combo);
                    break;
                }
                case PURPLE::Protocol::Option::INT:
                {
                    IntegerOption *edit = new IntegerOption(dxutils::normalizeLabelString(option.getText(), '_').c_str(), option.getIntValue(), option.getIntMin(), option.getIntMax());
                    tree->AppendNode(parent, *edit);
                    m_optionsFields.push_back(edit);
                    break;
                }
                case PURPLE::Plugin::Option::INTLIST:
                {
                    ChoiceOption *combo = new ChoiceOption(dxutils::normalizeLabelString(option.getText(), '_').c_str());
                    std::map<std::string, int>::iterator value = ivalues.begin();
                    for(; value!=ivalues.end(); ++value)
                    {
                        combo->AddOption((*value).first.c_str(), dxutils::sformat("%d", (*value).second).c_str(), option.getStringValue().c_str());
                    }
                    tree->AppendNode(parent, *combo);
                    m_optionsFields.push_back(combo);
                    break;
                }
                case PURPLE::Protocol::Option::BOOL:
                {
                    BooleanOption *check = new BooleanOption(dxutils::normalizeLabelString(option.getText(), '_').c_str(), option.getBoolValue());
                    tree->AppendNode(parent, *check);
                    m_optionsFields.push_back(check);
                    break;
                }
                case PURPLE::Protocol::Option::NONE:
                    m_optionsFields.push_back(NULL);
                    break;
            }
        }
    }
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &PluginDialog::OnOk));
}

void PluginDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

PluginDialog::BooleanOption::BooleanOption(const char *text, bool value)
: CheckBox(text), m_value(value)
{
    g_assert(text);
    SetState(m_value);
    signal_toggle.connect(sigc::mem_fun(this, &BooleanOption::OnToggle));
}

PluginDialog::BooleanOption::~BooleanOption()
{}

void PluginDialog::BooleanOption::OnToggle(CheckBox& /*activator*/, bool new_state)
{
    m_value = new_state;
}

PluginDialog::StringOption::StringOption(const char *text, const char *value, bool editable)
: Button(FLAG_VALUE, text)
{
    g_assert(text);
    g_assert(value);
    m_value = value;
    SetValue(value);
    if(editable) signal_activate.connect(sigc::mem_fun(this, &StringOption::OnActivate));
}

PluginDialog::StringOption::~StringOption()
{}

void PluginDialog::StringOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &StringOption::ResponseHandler));
    dialog->Show();
}

void PluginDialog::StringOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
  switch (response) {
    case CppConsUI::AbstractDialog::RESPONSE_OK:
      m_value = PURPLE::p2s(activator.GetText());
      SetValue(activator.GetText());
      break;
    default:
      break;
  }
}

PluginDialog::IntegerOption::IntegerOption(const char *text, int value, int min, int max)
: Button(FLAG_VALUE, text), m_value(value), m_min(min), m_max(max)
{
    g_assert(text);
    SetValue(value);
    signal_activate.connect(sigc::mem_fun(this, &IntegerOption::OnActivate));
}

PluginDialog::IntegerOption::~IntegerOption()
{}

void PluginDialog::IntegerOption::OnActivate(Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(GetText(), GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->SetFlags(CppConsUI::TextEntry::FLAG_NUMERIC);
    dialog->signal_response.connect(sigc::mem_fun(this, &IntegerOption::ResponseHandler));
    dialog->Show();
}

void PluginDialog::IntegerOption::ResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    const char *text;
    long int i;
    switch (response) {
        case CppConsUI::AbstractDialog::RESPONSE_OK:
            text = activator.GetText();
            errno = 0;
            i = strtol(text, NULL, 10);
            if (errno == ERANGE || i > INT_MAX || i < INT_MIN)
                LOG->Warning(_("Value is out of range."));
            m_value = CLAMP(i, INT_MIN, INT_MAX);
            m_value = CLAMP(i, m_min, m_max);
            SetValue(m_value);
            break;
        default:
        break;
    }
}

PluginDialog::ChoiceOption::ChoiceOption(const char *text)
: ComboBox(text)
{
    g_assert(text);
    signal_selection_changed.connect(sigc::mem_fun(this,  &ChoiceOption::OnSelectionChanged));
}

PluginDialog::ChoiceOption::~ChoiceOption()
{
    for(ComboBoxEntries::iterator i = options.begin(); i != options.end(); i++)
        g_free(reinterpret_cast<char*>(i->data));
}

void PluginDialog::ChoiceOption::AddOption(const char *title, const char *value, const char *defaultValue)
{
    g_assert(title);
    g_assert(value);
    g_assert(defaultValue);
    m_value = PURPLE::p2s(defaultValue);
    int item = AddOptionPtr(title, g_strdup(value));
    if(!g_ascii_strcasecmp(defaultValue, value))
        SetSelected(item);
}

void PluginDialog::ChoiceOption::OnSelectionChanged(ComboBox& /*activator*/, int /*new_entry*/, const char * /*title*/, intptr_t data)
{
    m_value = PURPLE::p2s(reinterpret_cast<const char*>(data));
}

void PluginDialog::SaveOptions()
{
    PURPLE::Plugin::Options::iterator it;
    guint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Plugin::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                option.setValue(static_cast<StringOption*>(m_optionsFields[i])->m_value);
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                option.setValue(static_cast<ChoiceOption*>(m_optionsFields[i])->m_value);
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                option.setValue(PURPLE::t2s<int>(static_cast<IntegerOption*>(m_optionsFields[i])->m_value));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                option.setValue(static_cast<ChoiceOption*>(m_optionsFields[i])->m_value);
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                option.setValue(static_cast<BooleanOption*>(m_optionsFields[i])->m_value ? "true" : "false");
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

void PluginDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    if(m_plugin.isLoadable())
    {
        if(m_enabled->m_value)
            m_plugin.load();
        else
            m_plugin.unload();
    }
    SaveOptions();
    Close();
}

