/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "roster.h"
#include "footer.h"
#include "log.h"
#include <cppconsui/ListBox.h>
#include <cppconsui/HorizontalListBox.h>
#include <cppconsui/Spacer.h>
#include <cppconsui/KeyConfig.h>
#include "gettext.h"
#include "rosteritem.h"
#include "preferences.h"

Roster *Roster::m_instance = NULL;

Roster *Roster::Instance()
{
  return m_instance;
}

bool Roster::RestoreFocus()
{
    FOOTER->SetText(_("%s act tabitem, %s main menu, %s context menu, %s offline friends"), "dxim|tabitem-active", "dxim|mainmenu", "roster|contextmenu", "dxim|roster-toggle-offline");
    return CppConsUI::Window::RestoreFocus();
}

void Roster::UngrabFocus()
{
    FOOTER->SetText(NULL);
    CppConsUI::Window::UngrabFocus();
}

void Roster::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::ROSTER_AREA));
}

Roster::Roster()
: Window(0, 0, 80, 24)
{
    SetColorScheme("roster");
    CppConsUI::ListBox *lbox = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    AddWidget(*lbox, 0, 0);
    CppConsUI::HorizontalListBox *hbox = new CppConsUI::HorizontalListBox(AUTOSIZE, AUTOSIZE);
    lbox->AppendWidget(*hbox);
    hbox->AppendWidget(*(new CppConsUI::Spacer(1, AUTOSIZE)));
    m_treeview = new CppConsUI::TreeView(AUTOSIZE, AUTOSIZE);
    hbox->AppendWidget(*m_treeview);
    hbox->AppendWidget(*(new CppConsUI::Spacer(1, AUTOSIZE)));
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalAccountAdded.connect(sigc::mem_fun(this, &Roster::OnAccountAdded));
    cb->signalAccountRemoved.connect(sigc::mem_fun(this, &Roster::OnAccountRemoved));
    cb->signalAccountSignedOn.connect(sigc::mem_fun(this, &Roster::OnAccountSignedOn));
    cb->signalBuddyAdded.connect(sigc::mem_fun(this, &Roster::OnBuddyAdded));
    cb->signalBuddyRemoved.connect(sigc::mem_fun(this, &Roster::OnBuddyRemoved));
    cb->signalChatAdded.connect(sigc::mem_fun(this, &Roster::OnChatAdded));
    cb->signalChatRemoved.connect(sigc::mem_fun(this, &Roster::OnChatRemoved));
}

Roster::~Roster()
{}

void Roster::Init()
{
    g_assert(!m_instance);
    m_instance = new Roster;
    m_instance->Show();
}

void Roster::Finalize()
{
    g_assert(m_instance);
    delete m_instance;
    m_instance = NULL;
}

bool Roster::ProcessInput(const TermKeyKey& key)
{
    if(!g_ascii_strcasecmp(KEYCONFIG->TermKeyToString(key), KEYCONFIG->GetKeyBind("window", "close-window")))
        return true;
    return Window::ProcessInput(key);
}

void Roster::FillRoster()
{
    std::vector<PURPLE::Account> accounts = DXIM->GetPurplemm()->getAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        LOG->Debug("Account: %s",  accounts[i].getUsername().c_str());
        RosterAccountItem *aitem = NULL;
        if(DXPREFERENCES.m_showRosterTree)
        {
            aitem = AddAccount(accounts[i]);
            m_items.push_back(aitem);
        }
        std::vector<std::string> g = accounts[i].getGroups();
        std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
        std::vector<PURPLE::Chat> ch = accounts[i].getChats();
        for(guint j=0; j<g.size(); j++)
        {
            LOG->Debug("\tGroup: %s",  g[j].c_str());
            RosterGroupItem *gitem = NULL;
            if(DXPREFERENCES.m_showRosterTree)
            {
                gitem = AddGroup(aitem, g[j]);
                m_items.push_back(gitem);
            }
            for(guint k=0; k<b.size(); k++)
            {
                if(g[j] == b[k].getGroupName())
                {
                    LOG->Debug("\t\tBuddy: %s",  b[k].getUsername().c_str());
                    RosterBuddyItem *bitem = AddBuddy(gitem, b[k]);
                    m_items.push_back(bitem);
                }
            }
            for(guint m=0; m<ch.size(); m++)
            {
                if(g[j] == ch[m].getGroupName())
                {
                    LOG->Debug("\t\tChat: %s",  ch[m].getAlias().c_str());
                    RosterChatItem *chitem = AddChat(gitem, ch[m]);
                    m_items.push_back(chitem);
                }
            }
            if(DXPREFERENCES.m_showRosterTree) gitem->Update();
        }
        if(DXPREFERENCES.m_showRosterTree) aitem->Update();
    }
    Sort();
}

RosterAccountItem *Roster::AddAccount(PURPLE::Account account)
{
    RosterAccountItem *item = new RosterAccountItem(account, m_treeview);
    CppConsUI::TreeView::NodeReference ref = m_treeview->AppendNode(m_treeview->GetRootNode(), *item);
    item->Update();
    item->SetRefNode(ref);
    return item;
}

RosterGroupItem *Roster::AddGroup(RosterAccountItem *account, std::string name)
{
    RosterGroupItem *item = new RosterGroupItem(account->GetAccount(), m_treeview, name);
    CppConsUI::TreeView::NodeReference ref = m_treeview->AppendNode(account->GetRefNode(), *item);
    item->SetRefNode(ref);
    return item;
}

RosterBuddyItem *Roster::AddBuddy(RosterGroupItem* group, PURPLE::Buddy buddy)
{
    RosterBuddyItem *item = new RosterBuddyItem(buddy, m_treeview);
    CppConsUI::TreeView::NodeReference ref = m_treeview->AppendNode(group? group->GetRefNode() : m_treeview->GetRootNode(), *item);
    item->Update();
    item->SetRefNode(ref);
    return item;
}

RosterChatItem *Roster::AddChat(RosterGroupItem* group, PURPLE::Chat chat)
{
    RosterChatItem *item = new RosterChatItem(chat, m_treeview);
    CppConsUI::TreeView::NodeReference ref = m_treeview->AppendNode(group? group->GetRefNode() : m_treeview->GetRootNode(), *item);
    item->Update();
    item->SetRefNode(ref);
    return item;
}

void Roster::OnAccountAdded(PURPLE::Account account)
{
    LOG->Debug("Account: %s",  account.getUsername().c_str());
    RosterAccountItem *aitem = NULL;
    if(DXPREFERENCES.m_showRosterTree)
    {
        aitem = AddAccount(account);
        m_items.push_back(aitem);
    }
    std::vector<std::string> g = account.getGroups();
    std::vector<PURPLE::Buddy> b = account.getBuddies();
    std::vector<PURPLE::Chat> ch = account.getChats();
    for(guint j=0; j<g.size(); j++)
    {
        LOG->Debug("\tGroup: %s",  g[j].c_str());
        RosterGroupItem *gitem = NULL;
        if(DXPREFERENCES.m_showRosterTree)
        {
            AddGroup(aitem, g[j]);
            m_items.push_back(gitem);
        }
        for(guint k=0; k<b.size(); k++)
        {
            if(g[j] == b[k].getGroupName())
            {
                LOG->Debug("\t\tBuddy: %s",  b[k].getUsername().c_str());
                RosterBuddyItem *bitem = AddBuddy(gitem, b[k]);
                m_items.push_back(bitem);
            }
        }
        for(guint m=0; m<ch.size(); m++)
        {
            if(g[j] == ch[m].getGroupName())
            {
                LOG->Debug("\t\tChat: %s",  ch[m].getAlias().c_str());
                RosterChatItem *chitem = AddChat(gitem, ch[m]);
                m_items.push_back(chitem);
            }
        }
        if(DXPREFERENCES.m_showRosterTree)gitem->Update();
    }
    Sort();
}

void Roster::OnAccountRemoved(std::string account, std::string protocolID)
{
    for(std::vector<RosterItem*>::iterator it=m_items.begin(); it!=m_items.end(); ++it)
    {
        if((*it)->GetAccount().getUsername() == account && (*it)->GetType() == RosterItem::Account
            && (*it)->GetAccount().getProtocol().getPurpleID() == protocolID)
        {
            m_treeview->DeleteNodeChildren((*it)->GetRefNode(), false);
            m_treeview->DeleteNode((*it)->GetRefNode(), false);
            m_items.erase(it);
            break;
        }
    }
}

void Roster::OnAccountSignedOn(PURPLE::Account account)
{
    RosterAccountItem *aitem = FindAccount(account.getUsername(), account.getProtocol().getPurpleID());
    if(!aitem) return;
    std::vector<std::string> g = account.getGroups();
    std::vector<PURPLE::Buddy> b = account.getBuddies();
    std::vector<PURPLE::Chat> ch = account.getChats();
    for(guint j=0; j<g.size(); j++)
    {
        RosterGroupItem *gitem = FindGroup(account, g[j]);
        if(!gitem)
        {
            gitem = AddGroup(aitem, g[j]);
            m_items.push_back(gitem);
        }
        for(guint k=0; k<b.size(); k++)
        {
            if(g[j] == b[k].getGroupName())
            {
                RosterBuddyItem *bitem = FindBuddy(account, b[k]);
                if(!bitem)
                {
                    bitem = AddBuddy(gitem, b[k]);
                    m_items.push_back(bitem);
                }
                bitem->Update();
            }
        }
        for(guint m=0; m<ch.size(); m++)
        {
            if(g[j] == ch[m].getGroupName())
            {
                RosterChatItem *chitem = FindChat(account, ch[m]);
                if(!chitem)
                {
                    RosterChatItem *chitem = AddChat(gitem, ch[m]);
                    m_items.push_back(chitem);
                }
            }
        }
        gitem->Update();
    }
    Sort();
}

RosterAccountItem *Roster::FindAccount(std::string account, std::string protocolID)
{
    RosterAccountItem *item = NULL;
    for(guint i=0; i<m_items.size(); i++)
    {
        if(m_items[i]->GetAccount().getUsername() == account && m_items[i]->GetType() == RosterItem::Account
            && m_items[i]->GetAccount().getProtocol().getPurpleID() == protocolID)
        {
            item = static_cast<RosterAccountItem*>(m_items[i]);
            break;
        }
    }
    return item;
}

RosterGroupItem *Roster::FindGroup(PURPLE::Account account, std::string group)
{
    RosterGroupItem *item = NULL;
    for(guint i=0; i<m_items.size(); i++)
    {
        if(m_items[i]->GetAccount() == account && m_items[i]->GetType() == RosterItem::Group
            && static_cast<RosterGroupItem*>(m_items[i])->GetName() == group)
        {
            item = static_cast<RosterGroupItem*>(m_items[i]);
            break;
        }
    }
    return item;
}

RosterBuddyItem *Roster::FindBuddy(PURPLE::Account account, PURPLE::Buddy buddy)
{
    RosterBuddyItem *item = NULL;
    for(guint i=0; i<m_items.size(); i++)
    {
        if(m_items[i]->GetAccount() == account && m_items[i]->GetType() == RosterItem::Buddy
            && static_cast<RosterBuddyItem*>(m_items[i])->GetBuddy() == buddy)
        {
            item = static_cast<RosterBuddyItem*>(m_items[i]);
            break;
        }
    }
    return item;
}

RosterChatItem *Roster::FindChat(PURPLE::Account account, PURPLE::Chat chat)
{
    RosterChatItem *item = NULL;
    for(guint i=0; i<m_items.size(); i++)
    {
        if(m_items[i]->GetAccount() == account && m_items[i]->GetType() == RosterItem::Chat
            && static_cast<RosterChatItem*>(m_items[i])->GetChat() == chat)
        {
            item = static_cast<RosterChatItem*>(m_items[i]);
            break;
        }
    }
    return item;
}

void Roster::OnBuddyAdded(PURPLE::Buddy buddy)
{
    RosterGroupItem *gitem = FindGroup(buddy.getAccount(), buddy.getGroupName());
    if(!gitem && DXPREFERENCES.m_showRosterTree)
    {
        gitem = AddGroup(FindAccount(buddy.getAccount().getUsername(), buddy.getAccount().getProtocol().getPurpleID()), buddy.getGroupName());
        m_items.push_back(gitem);
    }
    RosterBuddyItem *bitem = AddBuddy(gitem, buddy);
    m_items.push_back(bitem);
    if(gitem) gitem->Update();
    Sort();
}

void Roster::OnBuddyRemoved(PURPLE::Account account, std::string buddy)
{
    RosterGroupItem *item = NULL;
    for(std::vector<RosterItem*>::iterator it=m_items.begin(); it!=m_items.end(); ++it)
    {
        if((*it)->GetAccount() == account && (*it)->GetType() == RosterItem::Buddy
            && static_cast<RosterBuddyItem*>(*it)->GetBuddy().getUsername() == buddy)
        {
            item = FindGroup((*it)->GetAccount(), static_cast<RosterBuddyItem*>(*it)->GetBuddy().getGroupName());
            m_treeview->DeleteNode((*it)->GetRefNode(), false);
            m_items.erase(it);
            break;
        }
    }
    if(item)
        item->Update();
}

void Roster::OnChatAdded(PURPLE::Chat chat)
{
    RosterGroupItem *gitem = FindGroup(chat.getAccount(), chat.getGroupName());
    if(!gitem && DXPREFERENCES.m_showRosterTree)
    {
        gitem = AddGroup(FindAccount(chat.getAccount().getUsername(), chat.getAccount().getProtocol().getPurpleID()), chat.getGroupName());
        m_items.push_back(gitem);
    }
    RosterChatItem *chitem = AddChat(gitem, chat);
    m_items.push_back(chitem);
    if(gitem) gitem->Update();
    Sort();
}

void Roster::OnChatRemoved(PURPLE::Account account, std::string chat)
{
    RosterGroupItem *item = NULL;
    for(std::vector<RosterItem*>::iterator it=m_items.begin(); it!=m_items.end(); ++it)
    {
        if((*it)->GetAccount() == account && (*it)->GetType() == RosterItem::Chat
            && static_cast<RosterChatItem*>(*it)->GetChat().getAlias() == chat)
        {
            item = FindGroup((*it)->GetAccount(), static_cast<RosterChatItem*>(*it)->GetChat().getGroupName());
            m_treeview->DeleteNode((*it)->GetRefNode(), false);
            m_items.erase(it);
            break;
        }
    }
    if(item)
        item->Update();
}

void Roster::Sort()
{
    for(guint k=0; k<m_items.size(); k++)
    {
        CppConsUI::TreeView::NodeReference parent_ref;
        if(m_items[k]->GetType() == RosterItem::Account)
            parent_ref = m_treeview->GetRootNode();
        else if(m_items[k]->GetType() == RosterItem::Group)
            parent_ref = FindAccount(m_items[k]->GetAccount().getUsername(), m_items[k]->GetAccount().getProtocol().getPurpleID())->GetRefNode();
        else if(m_items[k]->GetType() == RosterItem::Buddy)
        {
            if(DXPREFERENCES.m_showRosterTree) parent_ref = FindGroup(m_items[k]->GetAccount(), static_cast<RosterBuddyItem*>(m_items[k])->GetBuddy().getGroupName())->GetRefNode();
            else parent_ref = m_treeview->GetRootNode();
        }
        else //Chat
        {
            if(DXPREFERENCES.m_showRosterTree) parent_ref = FindGroup(m_items[k]->GetAccount(), static_cast<RosterChatItem*>(m_items[k])->GetChat().getGroupName())->GetRefNode();
            else parent_ref = m_treeview->GetRootNode();
        }
        CppConsUI::TreeView::SiblingIterator i = --parent_ref.end();
        while(true) {
            // sref is a node that we want to sort in
            CppConsUI::TreeView::SiblingIterator sref = i;
            // calculate a stop condition
            bool stop_flag;
            if(i != parent_ref.begin())
            {
                stop_flag = false;
                i--;
            }
            else
                stop_flag = true;
            RosterItem *swidget = dynamic_cast<RosterItem*>(sref->GetWidget());
            g_assert(swidget);
            CppConsUI::TreeView::SiblingIterator j = sref;
            j++;
            while(j != parent_ref.end()) {
                RosterItem *n = dynamic_cast<RosterItem*>(j->GetWidget());
                g_assert(n);
                if(g_utf8_collate(swidget->GetName().c_str(), n->GetName().c_str()) <= 0)
                {
                    m_treeview->MoveNodeBefore(sref, j);
                    break;
                }
                j++;
            }
            // the node is last in the list
            if(j == parent_ref.end())
                m_treeview->MoveNodeAfter(sref, --j);
            if (stop_flag)
            break;
        }
    }
}

void Roster::ToggleHideOffline()
{
    for(guint i=0; i<m_items.size(); i++)
    {
        m_items[i]->Update();
    }
}

void Roster::ToggleShowTree()
{
    m_treeview->Clear();
    for(std::vector<RosterItem*>::iterator it=m_items.begin(); it!=m_items.end();)
    {
        it = m_items.erase(it);
    }
    FillRoster();
}

