/*
 * rosteritem.cpp
 *
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "rosteritem.h"
#include "dxutils.h"
#include "callback/callback.h"
#include "dxim.h"
#include "log.h"
#include "gettext.h"
#include "preferences.h"
#include "cppconsui/InputDialog.h"
#include "addbuddydialog.h"
#include "chatdialog.h"
#include "chatjoindialog.h"
#include "statusdialog.h"
#include "accountdialog.h"
#include "utilsmm.h"

RosterItem::RosterItem(PURPLE::Account account, CppConsUI::TreeView* tree, RosterItemType type)
: m_status(NULL), m_account(account), m_tree(tree), m_type(type)
{
    DeclareBindables();
}

RosterItem::~RosterItem()
{
    if(m_status)
        g_free(m_status);
}

void RosterItem::DeclareBindables()
{
    DeclareBindable("roster", "contextmenu", sigc::mem_fun(this, &RosterItem::ActionOpenContextMenu), InputProcessor::BINDABLE_NORMAL);
}

void RosterItem::ActionOpenContextMenu()
{
    OpenContextMenu();
}

void RosterItem::Draw()
{
    ProceedUpdateArea();
    if(!area || !text)
        return;
    int attrs;
    if(has_focus)
        attrs = GetColorPair("button", "focus") | CppConsUI::Curses::Attr::REVERSE;
    else
        attrs = GetColorPair("button", "normal");
    area->attron(attrs);
    int realw = area->getmaxx();
    int realh = area->getmaxy();
    // print text
    area->fill(attrs, 0, 0, text_width, realh);
    int y = 0;
    const char *start, *end;
    start = end = text;
    while(*end)
    {
        if(*end == '\n')
        {
            if(y >= realh)
                break;
            area->mvaddstring(0, y, realw, start, end);
            y++;
            start = end + 1;
        }
        end++;
    }
    if(y < realh)
        area->mvaddstring(0, y, realw, start, end);
    y++;
    area->attroff(attrs);
    //print status
    if(m_status)
    {
        int x = CppConsUI::Curses::onscreen_width("[o]");
        if(has_focus)
            attrs = GetColorPair("button", "focus") | CppConsUI::Curses::Attr::REVERSE;
        else
            attrs = GetColorPair("button", "status");
        area->attron(attrs);
        area->fill(attrs, x, y, text_width, realh-y);
        start = end = m_status;
        while(*end)
        {
            if(*end == '\n')
            {
                if(y >= realh)
                    break;
                area->mvaddstring(x, y, realw-x, start, end);
                y++;
                start = end + 1;
            }
            end++;
        }
        if(y < realh)
            area->mvaddstring(x, y, realw-x, start, end);
        area->attroff(attrs);
    }
}

void RosterItem::SetText(const char* new_text)
{
    if(text)
        g_free(text);
    text = g_strdup(new_text);
    UpdateWidthHeight();
}

void RosterItem::SetStatus(std::string new_status)
{
    if(m_status)
        g_free(m_status);
    if(new_status.empty()) m_status = NULL;
    else m_status = g_strdup(new_status.c_str());
    UpdateWidthHeight();
}

void RosterItem::UpdateWidthHeight()
{
    // update text_width, text_height and wish height
    text_width = 0;
    text_height = 1;
    if(text)
    {
        const char *start, *end;
        start = end = text;
        int w;
        while(*end)
        {
            if(*end == '\n')
            {
                w = CppConsUI::Curses::onscreen_width(start, end);
                if(w > text_width)
                    text_width = w;
                text_height++;
                start = end + 1;
            }
        end++;
        }
        w = CppConsUI::Curses::onscreen_width(start, end);
        if(w > text_width)
            text_width = w;
    }
    if(m_status)
    {
        text_height++;
        const char *start, *end;
        start = end = m_status;
        int w;
        while(*end)
        {
            if(*end == '\n')
            {
                w = CppConsUI::Curses::onscreen_width("[o]");
                w += CppConsUI::Curses::onscreen_width(start, end);
                if(w > text_width)
                    text_width = w;
                text_height++;
                start = end + 1;
            }
        end++;
        }
        w = CppConsUI::Curses::onscreen_width("[o]");
        w += CppConsUI::Curses::onscreen_width(start, end);
        if(w > text_width)
            text_width = w;
    }
    SetWishHeight(text_height);
    Redraw();
}

RosterAccountItem::RosterAccountItem(PURPLE::Account account, CppConsUI::TreeView *tree)
: RosterItem(account, tree, RosterItem::Account)
{
    signal_activate.connect(sigc::mem_fun(this, &RosterAccountItem::OnActivate));
    SetColorScheme("rosteraccount");
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalAccountConnecting.connect(sigc::mem_fun(this, &RosterAccountItem::OnAccountConnecting));
    cb->signalAccountConnected.connect(sigc::mem_fun(this, &RosterAccountItem::OnAccountConnected));
    cb->signalAccountDisconnected.connect(sigc::mem_fun(this, &RosterAccountItem::OnAccountDisconnected));
    cb->signalAccountStatusChanged.connect(sigc::mem_fun(this, &RosterAccountItem::OnAccountStatusChanged));
}

void RosterAccountItem::OnActivate(CppConsUI::Button& /*activator*/)
{
    m_tree->ToggleCollapsed(m_ref);
}

void RosterAccountItem::Update()
{
    std::string text = dxutils::getPresenceString(m_account.getStatus());
    text += m_account.getUsername();
    text += " [";
    text += m_account.getProtocol().getName();
    text += "]";
    SetText(text.c_str());
    SetStatus(m_account.getStatusMessage());
    UpdateColorScheme();
}

void RosterAccountItem::UpdateColorScheme()
{
    SetColorScheme(dxutils::getPresenceScheme("rosteraccount", m_account.getStatus()).c_str());
}

void RosterAccountItem::OpenContextMenu()
{
    m_popup = new CppConsUI::MenuWindow(*this, AUTOSIZE, AUTOSIZE);
    m_popup->SetColorScheme("generalmenu");
    if(m_account.isEnabled())
    {
        m_popup->AppendItem(_("Status"), sigc::mem_fun(this, &RosterAccountItem::OnStatus));
        m_popup->AppendSeparator();
        m_popup->AppendItem(_("Reconnect"), sigc::mem_fun(this, &RosterAccountItem::OnReconnect));
        m_popup->AppendSeparator();
        m_popup->AppendItem(_("Disable"), sigc::mem_fun(this, &RosterAccountItem::OnEnable));
    }
    else
    {
        m_popup->AppendItem(_("Enable"), sigc::mem_fun(this, &RosterAccountItem::OnEnable));
    }
    m_popup->AppendSeparator();
    m_popup->AppendItem(_("Account settings"), sigc::mem_fun(this, &RosterAccountItem::OnConfig));
    m_popup->AppendItem(_("Alias"), sigc::mem_fun(this, &RosterAccountItem::OnAlias));
    if(m_account.isConnected())
    {
        m_popup->AppendSeparator();
        m_popup->AppendItem(_("New IM"), sigc::mem_fun(this, &RosterAccountItem::OnIM));
        if(m_account.supportsChats())
            m_popup->AppendItem(_("Join chat"), sigc::mem_fun(this, &RosterAccountItem::OnJoinChat));
        m_popup->AppendSeparator();
        m_popup->AppendItem(_("Add buddy"), sigc::mem_fun(this, &RosterAccountItem::OnAddBuddy));
        if(m_account.supportsChats())
            m_popup->AppendItem(_("Add chat"), sigc::mem_fun(this, &RosterAccountItem::OnAddChat));
        m_popup->AppendItem(_("Add group"), sigc::mem_fun(this, &RosterAccountItem::OnAddGroup));
        std::vector<std::string> v = m_account.getCommandsList();
        if(v.size())
            m_popup->AppendSeparator();
        FOREACH(std::vector<std::string>, v, it)
        {
            if((*it).compare("----"))
            {
                m_popup->AppendItem((*it).c_str(), sigc::bind(sigc::mem_fun(this, &RosterAccountItem::OnAccountCb), (*it)));
            }
            else
            {
                m_popup->AppendSeparator();
            }
        }
    }
    m_popup->Show();
}

void RosterAccountItem::OnStatus(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    StatusDialog *dialog = new StatusDialog(m_account);
    dialog->Show();
}

void RosterAccountItem::OnReconnect(CppConsUI::Button& /*activator*/)
{
    if(m_account.isEnabled() && !m_account.isConnecting())
    {
        if(m_account.isConnected())
        {
            m_account.disconnect();
        }
        m_account.connect();
    }
    m_popup->Close();
}

void RosterAccountItem::OnEnable(CppConsUI::Button& /*activator*/)
{
    if(m_account.isEnabled())
        m_account.disconnect();
    else
        m_account.connect();
    m_popup->Close();
}

void RosterAccountItem::OnAlias(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Set alias"), m_account.getAlias().c_str());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &RosterAccountItem::OnAliasResponseHandler));
    dialog->Show();
}

void RosterAccountItem::OnAliasResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_account.setAlias(PURPLE::p2s(activator.GetText()));
    }
    m_popup->Close();
}

void RosterAccountItem::OnAddBuddy(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    AddBuddyDialog *dialog = new AddBuddyDialog(m_account);
    dialog->Show();
}

void RosterAccountItem::OnAddChat(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    ChatDialog *dialog = new ChatDialog(m_account);
    dialog->Show();
}

void RosterAccountItem::OnAddGroup(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(_("Add group"), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &RosterAccountItem::OnAddGroupResponseHandler));
    dialog->Show();
}

void RosterAccountItem::OnAddGroupResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_account.addGroup(PURPLE::p2s(activator.GetText()));
    }
    m_popup->Close();
}

void RosterAccountItem::OnAccountCb(CppConsUI::Button& /*activator*/, std::string cmd)
{
    m_account.callCommand(cmd);
    m_popup->Close();
}

void RosterAccountItem::OnIM(CppConsUI::Button& /*activator*/)
{
    PopupDialog *dialog = new PopupDialog(m_account);
    dialog->Show();
    m_popup->Close();
}

void RosterAccountItem::OnJoinChat(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    ChatJoinDialog *dialog = new ChatJoinDialog(m_account);
    dialog->Show();
}

void RosterAccountItem::OnConfig(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    AccountDialog *dialog = new AccountDialog(m_account);
    dialog->Show();
}

void RosterAccountItem::OnAccountConnecting(PURPLE::Account account, std::string msg)
{
    if(m_account == account)
        LOG->Message("%s [%s]: %s", account.getUsername().c_str(), account.getProtocol().getName().c_str(), msg.c_str());
}

void RosterAccountItem::OnAccountConnected(PURPLE::Account account)
{
    if(m_account == account)
    {
        //hack for disable idle after connected
        account.setStatus(PURPLE_STATUS_AVAILABLE);
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountConnect) dxutils::instance().showNotify(dxutils::sformat(_("Connected %s-%s"), account.getProtocol().getName().c_str(), account.getUsername().c_str()));
        LOG->Message(_("%s [%s]: Connected"), account.getUsername().c_str(), account.getProtocol().getName().c_str());
        Update();
    }
}

void RosterAccountItem::OnAccountDisconnected(PURPLE::Account account, std::string reason)
{
    if(m_account == account)
    {
        if(reason.empty())
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountDisconnect) dxutils::instance().showNotify(dxutils::sformat(_("Disconnected %s-%s"), account.getProtocol().getName().c_str(), account.getUsername().c_str()));
            LOG->Message(_("%s [%s]: Disconnected"), account.getUsername().c_str(), account.getProtocol().getName().c_str());
        }
        else
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountDisconnect) dxutils::instance().showNotify(dxutils::sformat(_("Disconnected %s-%s\n%s"), account.getProtocol().getName().c_str(), account.getUsername().c_str(), reason.c_str()));
            LOG->Message(_("%s [%s]: Disconnected (%s)"), account.getUsername().c_str(), account.getProtocol().getName().c_str(), reason.c_str());
        }
        Update();
    }
}

void RosterAccountItem::OnAccountStatusChanged(PURPLE::Account account)
{
    if(m_account == account)
    {
        Update();
    }
}

RosterGroupItem::RosterGroupItem(PURPLE::Account account, CppConsUI::TreeView *tree, std::string name)
: RosterItem(account, tree, RosterItem::Group), m_name(name)
{
    signal_activate.connect(sigc::mem_fun(this, &RosterGroupItem::OnActivate));
    SetColorScheme("rostergroup");
    SetText(name.c_str());
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalAccountConnected.connect(sigc::mem_fun(this, &RosterGroupItem::OnAccountConnected));
    cb->signalAccountDisconnected.connect(sigc::mem_fun(this, &RosterGroupItem::OnAccountDisconnected));
    cb->signalBuddySignedOn.connect(sigc::mem_fun(this, &RosterGroupItem::OnBuddySignedOn));
    cb->signalBuddySignedOff.connect(sigc::mem_fun(this, &RosterGroupItem::OnBuddySignedOff));
}

void RosterGroupItem::OpenContextMenu()
{}

void RosterGroupItem::OnAccountConnected(PURPLE::Account account)
{
    if(m_account == account)
    {
        Update();
    }
}

void RosterGroupItem::OnAccountDisconnected(PURPLE::Account account, std::string /*reason*/)
{
    if(m_account == account)
    {
        Update();
    }
}

void RosterGroupItem::OnBuddySignedOn(PURPLE::Buddy buddy)
{
    if(m_account == buddy.getAccount() && buddy.getGroupName() == m_name)
    {
        Update();
    }
}

void RosterGroupItem::OnBuddySignedOff(PURPLE::Buddy buddy)
{
    if(m_account == buddy.getAccount() && buddy.getGroupName() == m_name)
    {
        Update();
    }
}

void RosterGroupItem::OnActivate(CppConsUI::Button& /*activator*/)
{
    m_tree->ToggleCollapsed(m_ref);
}

void RosterGroupItem::Update()
{
    if(!m_account.isConnected())
        SetVisibility(false);
    else
    {
        if(DXPREFERENCES.m_hideOffline)
        {
            LOG->Debug("%s is online: %d", m_name.c_str(), m_account.isGroupOnline(m_name));
            if(m_account.isGroupOnline(m_name))
                SetVisibility(true);
            else
                SetVisibility(false);
        }
        else
        {
            SetVisibility(true);
        }
    }
}

RosterBuddyItem::RosterBuddyItem(PURPLE::Buddy buddy, CppConsUI::TreeView *tree)
: RosterItem(buddy.getAccount(), tree, RosterItem::Buddy), m_buddy(buddy), m_lastChange(dxutils::now())
{
    signal_activate.connect(sigc::mem_fun(this, &RosterBuddyItem::OnActivate));
    SetColorScheme("rosterbuddy");
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalAccountConnected.connect(sigc::mem_fun(this, &RosterBuddyItem::OnAccountConnected));
    cb->signalBuddySignedOn.connect(sigc::mem_fun(this, &RosterBuddyItem::OnBuddySignedOn));
    cb->signalBuddySignedOff.connect(sigc::mem_fun(this, &RosterBuddyItem::OnBuddySignedOff));
    cb->signalBuddyStatusChanged.connect(sigc::mem_fun(this, &RosterBuddyItem::OnBuddyStatusChanged));
}

void RosterBuddyItem::OnActivate(Button&)
{
    DXIM->GetPurplemm()->createConversation(m_buddy.getAccount(), m_buddy);
}

void RosterBuddyItem::Update()
{
    std::string text = dxutils::getPresenceString(m_buddy.getStatusPrimitive());
    text += m_buddy.getAlias();
    SetText(text.c_str());
    if(DXPREFERENCES.m_showStatusmsg) SetStatus(m_buddy.getStatusMsg());
    else SetStatus("");
    UpdateColorScheme();
    if(DXPREFERENCES.m_hideOffline)
    {
        if(m_buddy.isOnline())
            SetVisibility(true);
        else
            SetVisibility(false);
    }
    else
    {
        if(m_account.isConnected()) SetVisibility(true);
        else SetVisibility(false);
    }
    m_lastChange = dxutils::now();
}

void RosterBuddyItem::UpdateColorScheme()
{
    SetColorScheme(dxutils::getPresenceScheme("rosterbuddy", m_buddy.getStatusPrimitive()).c_str());
}

void RosterBuddyItem::OpenContextMenu()
{
    m_popup = new CppConsUI::MenuWindow(*this, AUTOSIZE, AUTOSIZE);
    m_popup->SetColorScheme("generalmenu");
    if(m_buddy.isOnline())
    {
        m_popup->AppendItem(_("IM"), sigc::mem_fun(this, &RosterBuddyItem::OnIM));
    }
    m_popup->AppendItem(_("Info"), sigc::mem_fun(this, &RosterBuddyItem::OnInfo));
    if(m_buddy.isOnline() && m_buddy.getAccount().supportsSendFile())
    {
        m_popup->AppendItem(_("Send file"), sigc::mem_fun(this, &RosterBuddyItem::OnSendFile));
    }
    m_popup->AppendSeparator();
    m_popup->AppendItem(_("Rename"), sigc::mem_fun(this, &RosterBuddyItem::OnRename));
    if(m_account.getAllGroups().size() > 1 || (m_account.getAllGroups().size() == 1 && m_buddy.getGroupName() != _("Friends")))
        m_popup->AppendItem(_("Move"), sigc::mem_fun(this, &RosterBuddyItem::OnMove));
    m_popup->AppendItem(_("Remove"), sigc::mem_fun(this, &RosterBuddyItem::OnRemove));
    std::vector<PURPLE::Buddy::BuddyMenuItem> v = m_buddy.getMenuItems();
    if(v.size()) m_popup->AppendSeparator();
    FOREACH(std::vector<PURPLE::Buddy::BuddyMenuItem>, v, it)
    {
        m_popup->AppendItem((*it).getLabel().c_str(), sigc::bind(sigc::mem_fun(this, &RosterBuddyItem::OnBuddyMenuItem), *it));
    }
    m_popup->Show();
}

void RosterBuddyItem::OnIM(CppConsUI::Button& /*activator*/)
{
    DXIM->GetPurplemm()->createConversation(m_buddy.getAccount(), m_buddy);
    m_popup->Close();
}

void RosterBuddyItem::OnInfo(CppConsUI::Button& /*activator*/)
{
    m_buddy.retrieveInfo();
    LOG->Message(_("Retrieving info about %s"), m_buddy.getAlias().c_str());
    m_popup->Close();
}

void RosterBuddyItem::OnSendFile(CppConsUI::Button& /*activator*/)
{
    m_buddy.sendFile();
    m_popup->Close();
}

void RosterBuddyItem::OnRename(CppConsUI::Button& /*activator*/)
{
    AliasDialog *dialog = new AliasDialog(m_buddy, this);
    dialog->Show();
    m_popup->Close();
}

void RosterBuddyItem::OnMove(CppConsUI::Button& /*activator*/)
{
    MoveDialog *dialog = new MoveDialog(m_buddy);
    dialog->Show();
    m_popup->Close();
}

void RosterBuddyItem::OnRemove(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    CppConsUI::MessageDialog *dialog = new CppConsUI::MessageDialog(_("Question"), dxutils::sformat(_("Do you want really remove %s?"), m_buddy.getUsername().c_str()).c_str());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &RosterBuddyItem::OnRemoveResponseHandler));
    dialog->Show();
}

void RosterBuddyItem::OnRemoveResponseHandler(CppConsUI::MessageDialog& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_account.removeBuddy(m_buddy);
    }
}

void RosterBuddyItem::OnBuddyMenuItem(CppConsUI::Button& /*activator*/, PURPLE::Buddy::BuddyMenuItem item)
{
    m_popup->Close();
    item.runCallback();
}

void RosterBuddyItem::OnAccountConnected(PURPLE::Account account)
{
    if(m_buddy.getAccount() == account)
    {
        m_lastChange = dxutils::now();
        Update();
    }
}

void RosterBuddyItem::OnBuddySignedOn(PURPLE::Buddy buddy)
{
    if(m_buddy == buddy)
    {
        if(dxutils::now()-m_lastChange>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyConnect) dxutils::instance().showNotify(dxutils::sformat(_("%s\n(%s)\nis online"), buddy.getAlias().c_str(), buddy.getUsername().c_str()));
            LOG->Message(_("%s (%s) is online"), m_buddy.getAlias().c_str(), m_buddy.getUsername().c_str());
        }
        Update();
    }
}

void RosterBuddyItem::OnBuddySignedOff(PURPLE::Buddy buddy)
{
    if(m_buddy == buddy)
    {
        if(dxutils::now()-m_lastChange>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyDisconnect) dxutils::instance().showNotify(dxutils::sformat(_("%s\n(%s)\nis offline"), buddy.getAlias().c_str(), buddy.getUsername().c_str()));
            LOG->Message(_("%s (%s) is offline"), m_buddy.getAlias().c_str(), m_buddy.getUsername().c_str());
        }
        Update();
    }
}

void RosterBuddyItem::OnBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive /*status*/)
{
    if(m_buddy == buddy)
    {
        Update();
    }
}

RosterBuddyItem::AliasDialog::AliasDialog(PURPLE::Buddy buddy, RosterBuddyItem *parent)
: SplitDialog(0, 0, 80, 24, _("Rename friend")), m_parent(parent), m_buddy(buddy)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("ID"),
                                                 dxutils::sformat("%s [%s]", m_buddy.getUsername().c_str(), m_buddy.getAccount().getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_alias = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("Alias"), m_buddy.getAlias().c_str());
    m_alias->signal_activate.connect(sigc::mem_fun(this, &AliasDialog::OnAlias));
    box->AppendWidget(*m_alias);
    m_serverAlias = new CppConsUI::CheckBox(AUTOSIZE, 1, _("Set on server too"),
                                            m_buddy.getAccount().hasServerAliases() ? true : false);
    if(m_buddy.getAccount().hasServerAliases()) box->AppendWidget(*m_serverAlias);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &AliasDialog::OnOk));
}

void RosterBuddyItem::AliasDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void RosterBuddyItem::AliasDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    std::string alias = PURPLE::p2s(m_alias->GetValue());
    if(!alias.empty() && m_buddy.getAlias().compare(alias))
    {
        m_buddy.setAlias(alias, m_serverAlias->GetState());
        m_parent->Update();
    }
    Close();
}

void RosterBuddyItem::AliasDialog::OnAlias(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(m_alias->GetText(), m_alias->GetValue());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &AliasDialog::OnAliasResponseHandler));
    dialog->Show();
}

void RosterBuddyItem::AliasDialog::OnAliasResponseHandler(CppConsUI::InputDialog& activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_alias->SetValue(activator.GetText());
    }
}

RosterBuddyItem::MoveDialog::MoveDialog(PURPLE::Buddy buddy)
: SplitDialog(0, 0, 80, 24, _("Move friend")), m_buddy(buddy)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("ID"),
                                                 dxutils::sformat("%s [%s]", m_buddy.getUsername().c_str(), m_buddy.getAccount().getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_groups = new CppConsUI::ComboBox(AUTOSIZE, 1, _("Group"));
    std::vector<std::string> v = m_buddy.getAccount().getAllGroups();
    bool friends = false;
    FOREACH(std::vector<std::string>, v, it)
    {
        if(*it != m_buddy.getGroupName()) m_groups->AddOption((*it).c_str());
        if(*it == _("Friends")) friends = true;
    }
    if(m_buddy.getGroupName() != _("Friends") && !friends) m_groups->AddOption(_("Friends"));
    box->AppendWidget(*m_groups);
    buttons->AppendItem(OK_BUTTON_TEXT, sigc::mem_fun(this, &MoveDialog::OnOk));
}

void RosterBuddyItem::MoveDialog::OnScreenResized()
{
    MoveResizeRect(DXIM->GetScreenArea(dxim::CHAT_AREA));
}

void RosterBuddyItem::MoveDialog::OnOk(CppConsUI::Button& /*activator*/)
{
    std::string oldgroup = m_buddy.getGroupName();
    std::string newgroup = m_groups->GetSelectedTitle();
    if(oldgroup != newgroup) m_buddy.getAccount().moveBuddy(m_buddy, newgroup);
    Close();
}

RosterChatItem::RosterChatItem(PURPLE::Chat chat, CppConsUI::TreeView *tree)
: RosterItem(chat.getAccount(), tree, RosterItem::Chat), m_chat(chat)
{
    signal_activate.connect(sigc::mem_fun(this, &RosterChatItem::OnActivate));
    SetText(chat.getAlias().c_str());
}

void RosterChatItem::OnActivate(Button&)
{
    m_chat.join();
}

void RosterChatItem::OpenContextMenu()
{
    m_popup = new CppConsUI::MenuWindow(*this, AUTOSIZE, AUTOSIZE);
    m_popup->SetColorScheme("generalmenu");
    m_popup->AppendItem(_("Join"), sigc::mem_fun(this, &RosterChatItem::OnJoin));
    m_popup->AppendItem(_("Modify"), sigc::mem_fun(this, &RosterChatItem::OnModify));
    m_popup->AppendItem(_("Remove"), sigc::mem_fun(this, &RosterChatItem::OnRemove));
    m_popup->Show();
}

void RosterChatItem::OnJoin(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    m_chat.join();
}

void RosterChatItem::OnModify(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    ChatDialog *dialog = new ChatDialog(m_chat);
    dialog->Show();
}

void RosterChatItem::OnRemove(CppConsUI::Button& /*activator*/)
{
    m_popup->Close();
    CppConsUI::MessageDialog *dialog = new CppConsUI::MessageDialog(_("Question"), dxutils::sformat(_("Do you want really remove %s?"), m_chat.getAlias().c_str()).c_str());
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &RosterChatItem::OnRemoveResponseHandler));
    dialog->Show();
}

void RosterChatItem::OnRemoveResponseHandler(CppConsUI::MessageDialog& /*activator*/, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_account.removeChat(m_chat);
    }
}

RosterAccountItem::PopupDialog::PopupDialog(PURPLE::Account account)
: SplitDialog(5, 5, 50, 6, _("New IM")), m_account(account)
{
    SetColorScheme("generaldialog");
    CppConsUI::ListBox *box = new CppConsUI::ListBox(AUTOSIZE, AUTOSIZE);
    SetContainer(*box);
    CppConsUI::Button *b = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("ID"),
                                                 dxutils::sformat("%s [%s]", account.getUsername().c_str(), account.getProtocol().getName().c_str()).c_str());
    box->AppendWidget(*b);
    m_with = new CppConsUI::Button(AUTOSIZE, 1, CppConsUI::Button::FLAG_VALUE, _("With"));
    m_with->signal_activate.connect(sigc::mem_fun(this, &PopupDialog::OnWith));
    box->AppendWidget(*m_with);
    buttons->AppendItem(_("IM"), sigc::mem_fun(this, &PopupDialog::OnIM));
}

void RosterAccountItem::PopupDialog::OnWith(CppConsUI::Button& /*activator*/)
{
    CppConsUI::InputDialog *dialog = new CppConsUI::InputDialog(m_with->GetText(), "");
    dialog->SetColorScheme("generaldialog");
    dialog->signal_response.connect(sigc::mem_fun(this, &PopupDialog::OnWithResponseHandler));
    dialog->Show();
}

void RosterAccountItem::PopupDialog::OnWithResponseHandler(CppConsUI::InputDialog &activator, CppConsUI::AbstractDialog::ResponseType response)
{
    if(response == CppConsUI::AbstractDialog::RESPONSE_OK)
    {
        m_with->SetValue(activator.GetText());
    }
}

void RosterAccountItem::PopupDialog::OnIM(CppConsUI::Button& /*activator*/)
{
    std::string with = PURPLE::p2s(m_with->GetValue());
    if(with.empty())
    {
        Close();
        return;
    }
    DXIM->GetPurplemm()->createConversation(m_account, with, false);
    Close();
}
