/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <cppconsui/KeyConfig.h>
#include "tabbook.h"
#include "gettext.h"
#include "imtabitem.h"
#include "chattabitem.h"
#include "dxutils.h"
#include "preferences.h"

TabBook *TabBook::m_instance = NULL;

TabBook *TabBook::Instance()
{
    return m_instance;
}

void TabBook::OnScreenResized()
{
    CppConsUI::Rect r = DXIM->GetScreenArea(dxim::CHAT_AREA);
    r.y = r.GetBottom();
    r.height = 1;
    MoveResizeRect(r);
}

void TabBook::FocusActiveTabitem()
{
    ActivateTabitem(m_active);
}

void TabBook::FocusTabitem(int i)
{
    g_assert(i >= 1);
    if(m_items.empty())
        return;
    // the external tab #1 is the internal conversation #0
    i -= 1;
    int s = static_cast<int>(m_items.size());
    if(i < s)
        ActivateTabitem(i);
    else
    {
        // if there is less than i active tab then active the last one
        ActivateTabitem(s - 1);
    }
}

void TabBook::FocusPrevTabitem()
{
    int i = PrevActiveTabItem(m_active);
    if(i != -1)
        ActivateTabitem(i);
}

void TabBook::FocusNextTabItem()
{
    int i = NextActiveTabItem(m_active);
    if(i != -1)
        ActivateTabitem(i);
}

void TabBook::SetExpandedTabBook(bool expanded)
{
    if(expanded)
    {
        // make small room on the left and right
        m_left_spacer->SetWidth(1);
        m_right_spacer->SetWidth(1);
    }
    else
    {
        m_left_spacer->SetWidth(0);
        m_right_spacer->SetWidth(0);
    }
    ActivateTabitem(m_active);
}

TabBook::TabBook()
: FreeWindow(0, 0, 80, 1, TYPE_NON_FOCUSABLE)
, m_active(-1)
{
    SetColorScheme("tabbook");
    m_outer_list = new CppConsUI::HorizontalListBox(AUTOSIZE, 1);
    AddWidget(*m_outer_list, 0, 0);
    m_left_spacer = new CppConsUI::Spacer(0, 1);
    m_right_spacer = new CppConsUI::Spacer(0, 1);
    m_item_list = new CppConsUI::HorizontalListBox(AUTOSIZE, 1);
    m_outer_list->AppendWidget(*m_left_spacer);
    m_outer_list->AppendWidget(*m_item_list);
    m_outer_list->AppendWidget(*m_right_spacer);
    dxcCallbacks *cb = DXIM->GetCallback();
    cb->signalNewIM.connect(sigc::mem_fun(this, &TabBook::OnNewTab));
    cb->signalNewChat.connect(sigc::mem_fun(this, &TabBook::OnNewTab));
    cb->signalNewMyMessage.connect(sigc::mem_fun(this, &TabBook::OnNewMyMsg));
    cb->signalMessageRecived.connect(sigc::mem_fun(this, &TabBook::OnMessageRecived));
    cb->signalNewIMMessage.connect(sigc::mem_fun(this, &TabBook::OnNewIMMsg));
    cb->signalNewChatMessage.connect(sigc::mem_fun(this, &TabBook::OnNewChatMsg));
    cb->signalNewConvSystemMessage.connect(sigc::mem_fun(this, &TabBook::OnNewConvSystemMsg));
    cb->signalNewConvErrorMessage.connect(sigc::mem_fun(this, &TabBook::OnNewConvErrorMsg));
    cb->signalBuddyTyping.connect(sigc::mem_fun(this, &TabBook::OnBuddyTyping));
    cb->signalCloseConvTab.connect(sigc::mem_fun(this, &TabBook::OnCloseConvTab));
    cb->signalAccountConnected.connect(sigc::mem_fun(this, &TabBook::OnAccountConnected));
    cb->signalAccountDisconnected.connect(sigc::mem_fun(this, &TabBook::OnAccountDisconnected));
}

TabBook::~TabBook()
{
}
void TabBook::Init()
{
    g_assert(!m_instance);
    m_instance = new TabBook;
    m_instance->Show();
}

void TabBook::Finalize()
{
    g_assert(m_instance);
    delete m_instance;
    m_instance = NULL;
}

int TabBook::FindConversation(PURPLE::Conversation *conv)
{
    for(int i = 0; i < static_cast<int>(m_items.size()); i++)
        if(*m_items[i].conv == *conv)
            return i;
    return -1;
}

int TabBook::FindTabItem(const std::string& id)
{
    for(int i = 0; i < static_cast<int>(m_items.size()); i++)
        if(m_items[i].item->GetID() == id)
            return i;
    return -1;
}

int TabBook::PrevActiveTabItem(int current)
{
    g_assert(current < static_cast<int>(m_items.size()));
    if(m_items.empty())
    {
        // there is no tabitem
        return -1;
    }
    if(current == 0)
    {
        // return the last tabitem
        return m_items.size() - 1;
    }
    return current - 1;
}

int TabBook::NextActiveTabItem(int current)
{
    g_assert(current < static_cast<int>(m_items.size()));
    if(m_items.empty())
    {
        // there is no tabitem
        return -1;
    }
    if(current == static_cast<int>(m_items.size() - 1))
    {
        // return the first tabitem
        return 0;
    }
    return current + 1;
}

void TabBook::ActivateTabitem(int i)
{
    g_assert(i >= -1);
    g_assert(i < static_cast<int>(m_items.size()));
    if(m_active == i)
    {
        if(m_active != -1)
            m_items[m_active].item->Show();
        return;
    }
    if(i != -1)
    {
        // show a new active tabitem
        m_items[i].label->SetVisibility(true);
        m_items[i].label->SetColorScheme("tabbook-active");
        m_items[i].item->Show();
    }
    // hide old active tabitem if there is any
    if(m_active != -1)
    {
        m_items[m_active].label->SetColorScheme(NULL);
        m_items[m_active].item->Hide();
    }
    m_active = i;
}

void TabBook::CloseCurrentTabitem()
{
    if(m_active != -1)
    {
        int id = m_active;
        if(m_items.size() == 1)
        {
            // the last tabitem is closed
            m_active = -1;
        }
        else
        {
            if(m_active == static_cast<int>(m_items.size() - 1))
                FocusPrevTabitem();
            else
                FocusNextTabItem();
        }
        if(m_items[id].item->GetPurpleConversation()->isValid())
            m_items[id].item->GetPurpleConversation()->leave();
        delete m_items[id].item;
        m_item_list->RemoveWidget(*m_items[id].label);
        m_items.erase(m_items.begin() + id);
        if(m_active > id)
        {
            // fix up the number of the active tabitem
            m_active--;
        }
        UpdateLabels();
    }
}

void TabBook::UpdateLabel(int i)
{
    g_assert(i >= 0);
    g_assert(i < static_cast<int>(m_items.size()));
    if(m_items[i].conv->isValid())
    {
        m_items[i].label->SetText(dxutils::sformat(" %d|%s%c", i + 1, m_items[i].conv->getTitle().c_str(), m_items[i].typing_status).c_str());
    }
    else
    {
        m_items[i].label->SetText(dxutils::stringSection(m_items[i].item->GetID(), '-', 2).c_str());
    }
}

void TabBook::UpdateLabels()
{
    /* Note: This can be a little slow if there are too many opened
    * tabitems. */
    for(guint i = 0; i < m_items.size(); i++)
        UpdateLabel(i);
}

void TabBook::OnNewTab(PURPLE::Conversation *conv)
{
    int id = FindConversation(conv);
    if(id != -1) // IMTabItem created by roster or NewImMenu
    {
        ActivateTabitem(id);
        return;
    }
    if(conv->getType() == PURPLE_CONV_TYPE_IM)
    {
        IMTabItem *item = new IMTabItem(conv);
        ItemChild c;
        c.conv = conv;
        c.item = item;
        c.label = new CppConsUI::Label(AUTOSIZE, 1);
        c.typing_status = ' ';
        m_item_list->AppendWidget(*c.label);
        m_items.push_back(c);
        if(DXPREFERENCES.m_logging && DXPREFERENCES.m_logLoadCurrent) item->LoadCurrentLog();
    }
    if(conv->getType() == PURPLE_CONV_TYPE_CHAT)
    {
        ChatTabItem *item = new ChatTabItem(conv);
        ItemChild c;
        c.conv = conv;
        c.item = item;
        c.label = new CppConsUI::Label(AUTOSIZE, 1);
        c.typing_status = ' ';
        m_item_list->AppendWidget(*c.label);
        m_items.push_back(c);
    }
    UpdateLabels();
    // show the first tabitem if there isn't any already
    if(m_active == -1)
        ActivateTabitem(m_items.size() - 1);
}

void TabBook::OnNewMyMsg(PURPLE::Conversation *conv, std::string time, std::string msg, bool action, std::string messageID)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->AppendMyMsg(time, msg, action, messageID);
    }
}

void TabBook::OnMessageRecived(PURPLE::Conversation *conv, std::string messageID)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->MessageRecived(messageID);
    }
}

void TabBook::OnNewIMMsg(PURPLE::Conversation *conv, std::string time, std::string sender, std::string msg, bool highlight, bool action)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->AppendMsg(time, sender, msg, action, highlight, false);
        if(id != m_active)
            m_items[id].label->SetColorScheme(highlight ? "tabbook-highlight" : "tabbook-unread");
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(dxutils::sformat(_("New message on IM with %s"), conv->getTitle().c_str()));
    }
}

void TabBook::OnNewChatMsg(PURPLE::Conversation *conv, std::string time, std::string sender, std::string msg, bool delayed, bool highlight, bool action)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->AppendMsg(time, sender, msg, action, highlight, delayed);
        if(id != m_active)
            m_items[id].label->SetColorScheme(highlight ? "tabbook-highlight" : "tabbook-unread");
        if(highlight && DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(dxutils::sformat(_("New highlighted message on %s"), conv->getTitle().c_str()));
    }
}

void TabBook::OnNewConvSystemMsg(PURPLE::Conversation *conv, std::string time, std::string msg)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->AppendSystemMsg(time, msg);
    }
}

void TabBook::OnNewConvErrorMsg(PURPLE::Conversation *conv, std::string time, std::string msg)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].item->AppendErrorMsg(time, msg);
    }
}

void TabBook::OnBuddyTyping(PURPLE::Conversation *conv, bool typing)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        m_items[id].typing_status = typing ? '*' : ' ';
        UpdateLabel(id);
    }
}

void TabBook::OnCloseConvTab(PURPLE::Conversation *conv)
{
    int id = FindConversation(conv);
    if(id != -1)
    {
        if(id == m_active)
        {
            if(m_items.size() == 1)
            {
                // the last tabitem is closed
                m_active = -1;
            }
            else
            {
                if(m_active == static_cast<int>(m_items.size() - 1))
                    FocusPrevTabitem();
                else
                    FocusNextTabItem();
            }
        }
        delete m_items[id].item;
        m_item_list->RemoveWidget(*m_items[id].label);
        m_items.erase(m_items.begin() + id);
        if(m_active > id)
        {
            // fix up the number of the active tabitem
            m_active--;
        }
        UpdateLabels();
    }
}

void TabBook::OnAccountDisconnected(PURPLE::Account account, std::string)
{
    for(int i = 0; i < static_cast<int>(m_items.size()); i++)
    {
        if(m_items[i].conv->getAccount() == account)
        {
            m_items[i].item->AppendSystemMsg(dxutils::time("[%H:%M:%S] "), dxutils::sformat(_("%s [%s]: Disconnected"), account.getUsername().c_str(), account.getProtocol().getName().c_str(), NULL));
        }
    }
}

void TabBook::OnAccountConnected(PURPLE::Account account)
{
    for(int i = 0; i < static_cast<int>(m_items.size()); i++)
    {
        if(m_items[i].conv->getAccount() == account)
        {
            m_items[i].item->AppendSystemMsg(dxutils::time("[%H:%M:%S] "), dxutils::sformat(_("%s [%s]: Connected"), account.getUsername().c_str(), account.getProtocol().getName().c_str(), NULL));
        }
    }
}

void TabBook::StopLogging()
{
    for(guint i = 0; i < m_items.size(); i++)
        if(m_items[i].item->GetID().empty()) m_items[i].item->StopLogging();
}

void TabBook::ClearCurrentTabItem()
{
    if(m_active != -1 && (guint)m_active < m_items.size() && m_items[m_active].item->GetID().empty())
        m_items[m_active].item->ClearText();
}

void TabBook::ClearAllTabItems()
{
    for(guint i = 0; i < m_items.size(); i++)
    {
        if(m_items[i].item->GetID().empty()) m_items[i].item->ClearText();
        if(i == (guint)m_active) m_items[i].label->SetColorScheme("tabbook-active");
        else m_items[i].label->SetColorScheme("tabbook");
    }
}

TabItem *TabBook::CreateNewItem(const std::string& id)
{
    TabItem *item = new TabItem(id);
    ItemChild c;
    c.conv = new PURPLE::Conversation();
    c.item = item;
    c.label = new CppConsUI::Label(AUTOSIZE, 1);
    c.typing_status = ' ';
    m_item_list->AppendWidget(*c.label);
    m_items.push_back(c);
    UpdateLabels();
    // show the first tabitem if there isn't any already
    if(m_active == -1)
        ActivateTabitem(m_items.size() - 1);
    return item;
}

void TabBook::AppendText(const std::string& id, const std::string& text, bool clear)
{
    int idn = FindTabItem(id);
    if(idn != -1)
    {
        if(clear) m_items[idn].item->ClearText();
        m_items[idn].item->AppendText(text);
        if(idn != m_active)
            m_items[idn].label->SetColorScheme("tabbook-unread");
    }
}

