/*
 * Copyright (C) 2007 by Mark Pustjens <pustjens@dds.nl>
 * Copyright (C) 2010-2012 by CenterIM developers
 * Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <cppconsui/KeyConfig.h>
#include "tabitem.h"
#include "roster.h"
#include "footer.h"
#include <sys/stat.h>
#include "gettext.h"
#include "dxutils.h"
#include "preferences.h"
#include <utilsmm.h>

TabItem::TabItem(PURPLE::Conversation *conv)
: Window(0, 0, 80, 24), m_conv(conv), m_id("")
{
    SetColorScheme("tabitem");
    m_view = new CppConsUI::TextView(width - 2, height, true, true);
    AddWidget(*m_view, 1, 0);
    DeclareBindables();
    m_lastKeyPress = 0;
    m_imState = ACTIVE;
    LOG->Debug("TabItem::TabItem(): this=%p, title=%s", static_cast<void*>(this), m_conv->getTitle().c_str());
}

TabItem::TabItem(const std::string &id)
: Window(0, 0, 80, 24), m_conv(new PURPLE::Conversation()), m_id(id)
{
    SetColorScheme("tabitem");
    m_view = new CppConsUI::TextView(width - 2, height, true, true);
    AddWidget(*m_view, 1, 0);
    DeclareBindables();
    m_lastKeyPress = 0;
    m_imState = ACTIVE;
    LOG->Debug("TabItem::TabItem(): this=%p, title=%s", static_cast<void*>(this), m_id.c_str());
}

TabItem::~TabItem()
{
    LOG->Debug("TabItem::~TabItem(): this=%p", static_cast<void*>(this));
}

bool TabItem::ProcessInput(const TermKeyKey& key)
{
    if(m_view->ProcessInput(key))
        return true;
    if(!g_ascii_strcasecmp(KEYCONFIG->TermKeyToString(key), KEYCONFIG->GetKeyBind("window", "close-window")))
        return true;
    return Window::ProcessInput(key);
}

bool TabItem::RestoreFocus()
{
    if(FOOTER) FOOTER->SetText(_("%s buddy list, %s main menu, "
                    "%s/%s next/prev tabitem, %s close tabitem, %s send, %s expand, %s offline friends"),
                    "dxim|roster-active", "dxim|mainmenu",
                    "dxim|tabitem-next", "dxim|tabitem-prev",
                    "dxim|tabbook-closecurrent",
                    "tabitem|send", "dxim|tabbook-expand", "dxim|roster-toggle-offline");
    return Window::RestoreFocus();
}

void TabItem::UngrabFocus()
{
    FOOTER->SetText(NULL);
    Window::UngrabFocus();
}

void TabItem::MoveResize(int newx, int newy, int neww, int newh)
{
    Window::MoveResize(newx, newy, neww, newh);
    int view_height = height - 1;
    if(view_height < 1)
        view_height = 1;
    m_view->MoveResize(1, 0, width - 2, view_height);
}

void TabItem::Show()
{
    /* Update the scrollbar setting. It is delayed until the conversation window
    * is actually displayed, so screen lines recalculations in TextView (caused
    * by changing the scrollbar setting) aren't triggered if it isn't really
    * necessary. */
    m_view->SetScrollBar(!DXIM->GetExpandedConversations());
    Window::Show();
}

void TabItem::Close()
{
    signal_close(*this);
    if(m_conv->isValid()) m_conv->leave();
}

void TabItem::OnScreenResized()
{
    CppConsUI::Rect r = DXIM->GetScreenArea(dxim::CHAT_AREA);
    // make room for conversations list
    r.height--;
    MoveResizeRect(r);
}

void TabItem::StartLogging()
{
    if(!DXPREFERENCES.m_logging) return;
    if(m_logger.isReady()) return;
    std::string folder = m_conv->getName();
    folder = folder.substr(0, folder.find_last_of('/'));
    std::string username = m_conv->getAccount().getUsername();
    username = username.substr(0, username.find_last_of('/'));
    m_logger.setPath(g_build_filename(DXPREFERENCES.m_logPath.c_str(), username.c_str(), folder.c_str(), NULL));
}

void TabItem::AppendMyMsg(std::string time, std::string msg, bool action, std::string messageID)
{
    StartLogging();
    if(action)
    {
        m_view->Append(PURPLE::stripHtml(dxutils::sformat("%s%s %s", time.c_str(), m_conv->getMyNick().c_str(), msg.c_str())).c_str(), 2, messageID.c_str());
        m_logger.logStyledLine(time, "action", m_conv->getMyNick()+" "+msg);
    }
    else
    {
        m_view->Append(PURPLE::stripHtml(dxutils::sformat("%s%s: %s", time.c_str(), m_conv->getMyNick().c_str(), msg.c_str())).c_str(), 1, messageID.c_str());
        m_logger.logMyMsgLine(time, m_conv->getMyNick(), msg);
    }
}

void TabItem::MessageRecived(std::string messageID)
{
    m_view->MessageRecived(messageID.c_str());
}

void TabItem::AppendMsg(std::string time, std::string sender, std::string msg, bool action, bool highlight, bool delayed)
{
    int color = 0;
    if(action)
        color = 2;
    if(highlight)
        color = 5;
    if(delayed)
        color = 4;
    m_view->Append(PURPLE::stripHtml(dxutils::sformat("%s%s%s%s", time.c_str(), sender.c_str(), action || highlight ? " " : ": ", msg.c_str())).c_str(), color);
    if(!delayed)
    {
        StartLogging();
        if(action)
            m_logger.logStyledLine(time, "action", sender+" "+msg);
        else if(highlight)
            m_logger.logStyledLine(time, "highlight", sender+": "+msg);
        else
            m_logger.logNickLine(time, sender, msg);
    }
}

void TabItem::AppendSystemMsg(std::string time, std::string msg, bool log)
{
    m_view->Append(PURPLE::stripHtml(dxutils::sformat("%s%s", time.c_str(), msg.c_str())).c_str(), 4);
    if(log)
    {
        StartLogging();
        m_logger.logStyledLine(time, "user", msg);
    }
}

void TabItem::AppendErrorMsg(std::string time, std::string msg)
{
    m_view->Append(PURPLE::stripHtml(dxutils::sformat("%s%s", time.c_str(), msg.c_str())).c_str(), 3);
    StartLogging();
    m_logger.logStyledLine(time, "error", msg);
}

void TabItem::AppendText(const std::string& text)
{
    m_view->Append(PURPLE::stripHtml(text).c_str());
}

void TabItem::ActionSend()
{
    if(!m_conv->getAccount().isConnected())
        return;
    std::string commandtext = PURPLE::p2s(m_input->GetText());
    m_input->Clear();
    if(commandtext.empty())
        return;
    if(commandtext[0] == '/')
    {
        std::string::size_type pos = commandtext.find_first_of(' ');
        std::string command = commandtext.substr(1, pos-1);
        std::string arg = "";
        if(pos != std::string::npos) arg = commandtext.substr(pos+1);
        if(g_ascii_strcasecmp(command.c_str(), "help")==0)
        {
            if(arg.empty())
            {
                std::string commands = "";
                for(size_t i=0; i<m_commands.size(); i++)
                {
                    if(i+1!=m_commands.size())
                        commands.append(m_commands[i]+", ");
                    else
                        commands.append(m_commands[i]+".");
                }
                m_view->Append(dxutils::sformat(_("Available commands: %s"), commands.c_str()).c_str(), 1);
            }
            else if(g_ascii_strcasecmp(arg.c_str(), "help")==0 || g_ascii_strcasecmp(arg.c_str(), "me")==0
                    || g_ascii_strcasecmp(arg.c_str(), "say")==0
                    || (m_conv->getType() == PURPLE_CONV_TYPE_CHAT && g_ascii_strcasecmp(arg.c_str(), "ignored")==0)
                    || (m_conv->getType() == PURPLE_CONV_TYPE_CHAT && g_ascii_strcasecmp(arg.c_str(), "ignore")==0)
                    || (m_conv->getType() == PURPLE_CONV_TYPE_CHAT && g_ascii_strcasecmp(arg.c_str(), "unignore")==0)
                    || (m_conv->getType() == PURPLE_CONV_TYPE_CHAT && g_ascii_strcasecmp(arg.c_str(), "im")==0)
                    || g_ascii_strcasecmp(arg.c_str(), "clear")==0)
            {
                ShowInternalHelp(arg);
            }
            else
            {
                std::string response = PURPLE::stripHtml(m_conv->getCommandHelp(arg));
                if(response.empty())
                {
                    m_view->Append(dxutils::sformat(_("Command '%s' doesn't exist or hasn't help"), arg.c_str()).c_str(), 3);
                }
                else
                {
                    m_view->Append(dxutils::sformat(_("Help for: %s"), response.c_str()).c_str(), 1);
                }
            }
        }
        else if(g_ascii_strcasecmp(command.c_str(), "me")==0)
        {
            if(!arg.empty()) m_conv->sendActionMessage(EscapeText(arg));
        }
        else if(g_ascii_strcasecmp(command.c_str(), "say")==0)
        {
            if(!arg.empty()) m_conv->sendMessage(EscapeText(arg));
        }
        else if(m_conv->getType() == PURPLE_CONV_TYPE_CHAT && g_ascii_strcasecmp(command.c_str(), "ignored")==0)
        {
            std::string ignored = "";
            std::vector<std::string> v = m_conv->getIgnored();
            for(size_t i=0; i<v.size(); i++)
            {
                if(i+1!=v.size())
                    ignored.append(v[i]+", ");
                else
                    ignored.append(v[i]+".");
            }
            if(v.size())
            {
                m_view->Append(dxutils::sformat(_("Ignored buddies: %s"), ignored.c_str()).c_str(), 1);
            }
            else
            {
                m_view->Append(_("Nobody ignored"), 1);
            }
        }
        else if(g_ascii_strcasecmp(command.c_str(), "ignore")==0)
        {
            if(!arg.empty()) m_conv->ignore(arg);
        }
        else if(g_ascii_strcasecmp(command.c_str(), "unignore")==0)
        {
            if(!arg.empty()) m_conv->unignore(arg);
        }
        else if(g_ascii_strcasecmp(command.c_str(), "im")==0)
        {
            if(!arg.empty()) m_conv->createIM(GetChatBuddy(arg));
        }
        else if(g_ascii_strcasecmp(command.c_str(), "clear")==0)
        {
            ClearText();
        }
        else
        {
            int ret = m_conv->sendCommand(commandtext.substr(1));
            ProcessCommandReturn(ret, command);
        }
    }
    else
    {
        m_conv->sendMessage(EscapeText(commandtext));
    }
    m_lastKeyPress = 0;
    m_imState = ACTIVE;
}

void TabItem::ActionClear()
{
    m_input->Clear();
}

void TabItem::DeclareBindables()
{
    DeclareBindable("tabitem", "send", sigc::mem_fun(this, &TabItem::ActionSend), InputProcessor::BINDABLE_OVERRIDE);
    DeclareBindable("tabitem", "clear", sigc::mem_fun(this, &TabItem::ActionClear), InputProcessor::BINDABLE_OVERRIDE);
}

std::string TabItem::EscapeText(std::string text)
{
    size_t i = 0;
    size_t length = text.length();
    while(i<length)
    {
        if(text[i]=='<')
        {
            text.replace(i, 1, "&lt;");
            length += 3;
        }
        else if(text[i]=='>')
        {
            text.replace(i, 1, "&gt;");
            length += 3;
        }
        i++;
    }
    return text;
}

void TabItem::ShowInternalHelp(const std::string &command)
{
    if(g_ascii_strcasecmp(command.c_str(), "help")==0)
    {
        m_view->Append(_("Help for: help [command]: lists available commands or show help for command"), 1);
    }
    else if(g_ascii_strcasecmp(command.c_str(), "me")==0)
    {
        m_view->Append(_("Help for: me <text>: sends text in 3rd person"), 1);
    }
    else if(g_ascii_strcasecmp(command.c_str(), "say")==0)
    {
        m_view->Append(_("Help for: say <text>: sends text"), 1);
    }
    else if(g_ascii_strcasecmp(command.c_str(), "ignored")==0)
    {
        m_view->Append(_("Help for: ignored: show list ignored buddies"), 1);
    }
    else if(g_ascii_strcasecmp(command.c_str(), "clear")==0)
    {
        m_view->Append(_("Help for: clear: clear text window"), 1);
    }
}

//process return m_conv->sendCommand
void TabItem::ProcessCommandReturn(int ret, const std::string &command)
{
    std::string text = "";
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            return;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            text = dxutils::sformat(_("Unknown command '%s'."), command.c_str());
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            text = dxutils::sformat(_("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%s'."), command.c_str());
            break;
        case PURPLE_CMD_STATUS_FAILED:
            text = dxutils::sformat(_("Your command '%s' failed for an unknown reason."), command.c_str());
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conv->getType() == PURPLE_CONV_TYPE_IM)
                text = dxutils::sformat(_("That command '%s' only works in chats, not IMs."), command.c_str());
            else
                text = dxutils::sformat(_("That command '%s' only works in IMs, not chats."), command.c_str());
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            text = dxutils::sformat(_("That command '%s' doesn't work on this protocol."), command.c_str());
            break;
    }
    m_view->Append(text.c_str(), 3);
}

void TabItem::CompleteHelpCommand()
{
    std::string commandtext = m_input->GetText();
    std::string::size_type pos = commandtext.find_first_of(' ');
    std::string arg = commandtext.substr(pos+1);
    for(size_t i=0; i<m_commands.size(); i++)
    {
        if(g_ascii_strcasecmp(arg.c_str(), m_commands[i].c_str()) == 0)
        {
            std::string text = "";
            if((i+1) < m_commands.size()) text = dxutils::sformat("/help %s", m_commands[++i].c_str());
            else text = dxutils::sformat("/help %s", m_commands[0].c_str());
            m_input->SetText(text.c_str());
            break;
        }
        else if(g_ascii_strcasecmp(arg.c_str(), m_commands[i].substr(0, arg.length()).c_str()) == 0)
        {
            m_input->SetText(dxutils::sformat("/help %s", m_commands[i].c_str()).c_str());
            break;
        }
    }
}

void TabItem::StopLogging()
{
    if(!DXPREFERENCES.m_logging) m_logger.stopLogging();
}

void TabItem::ClearText()
{
    m_view->Clear();
}

