/*
 *      FXTrayIcon.cpp
 *
 *      Copyright (c) 2008, Hendrik Rittich
 *      Copyright (C) 2006-2011 by Sander Jansen
 *      Copyright 2012 David Vachulka <david@konstrukce-cad.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <xincs.h>
#include "FXTrayIcon.h"
#include "FXTrayApp.h"

namespace FX {

#ifdef WIN32
FXDEFMAP(FXTrayIcon) FXTrayIconMap[] = {
    FXMAPFUNC(SEL_TIMEOUT, FXTrayIcon::ID_POPTIMEOUT, FXTrayIcon::onTimeout),
    FXMAPFUNC(SEL_MOTION, 0, FXTrayIcon::onEvent)
};
#else
enum
{
    SYSTEM_TRAY_REQUEST_DOCK = 0,
    SYSTEM_TRAY_BEGIN_MESSAGE = 1,
    SYSTEM_TRAY_CANCEL_MESSAGE = 2,
};

enum
{
    SYSTEM_TRAY_HORIZONTAL = 0,
    SYSTEM_TRAY_VERTICAL = 1,
    SYSTEM_TRAY_UNKNOWN = 2
};


FXDEFMAP(FXTrayIcon) FXTrayIconMap[] = {
    FXMAPFUNC(SEL_PAINT, 0, FXTrayIcon::onPaint),
    FXMAPFUNC(SEL_CONFIGURE, 0, FXTrayIcon::onConfigure),
    FXMAPFUNC(SEL_LEFTBUTTONPRESS, 0, FXTrayIcon::onLeftBtnPress),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, 0, FXTrayIcon::onRightBtnRelease),
    FXMAPFUNC(SEL_QUERY_TIP, 0, FXTrayIcon::onQueryTip),
    FXMAPFUNC(SEL_EMBED_NOTIFY, 0, FXTrayIcon::onEmbedded),
    FXMAPFUNC(SEL_TIMEOUT, FXTrayIcon::ID_POPTIMEOUT, FXTrayIcon::onTimeout)
};
#endif

FXIMPLEMENT(FXTrayIcon, FXTopWindow, FXTrayIconMap, ARRAYNUMBER(FXTrayIconMap))

#ifdef WIN32
DWORD FXTrayIcon::sTrayIconCount = 1;

FXTrayIcon::FXTrayIcon(FXApp* app, const FXString& text, FXIcon* icon,
        FXPopup* popup, FXObject* target, FXSelector sel,
        FXuint opts) :
    FXTopWindow(app, text, 0, 0, 0, 0,0,0,0, 0,0,0,0, 0,0),
    mIcon(icon),
    m_popup(popup),
    m_opts(opts),
    mTooltip(text)
{
    mWIcon = 0;
    mTrayID = sTrayIconCount;
    sTrayIconCount++;

    setTarget(target);
    setSelector(sel);
}

FXTrayIcon::~FXTrayIcon() {
    if (mWIcon)
    {
        // Icon entfernen
        NOTIFYICONDATA tray_data;
        ZeroMemory(&tray_data, sizeof(tray_data));
        tray_data.cbSize = sizeof(tray_data);
        tray_data.hWnd = static_cast<HWND>(xid);
        tray_data.uID = mTrayID;
        Shell_NotifyIcon(NIM_DELETE, &tray_data);

        // Icon loeschen
        DestroyIcon(mWIcon);
    }
}

void FXTrayIcon::create() {
    FXTopWindow::create();

    if (mWIcon)
        return;

    mIcon->create();

    // Windows Icon erzeugen
    mWIcon = createMswIcon(mIcon);

    mapToManager();
}

void FXTrayIcon::mapToManager()
{
    NOTIFYICONDATA tray_data;
    setupNotifyData(&tray_data);

    Shell_NotifyIcon(NIM_ADD, &tray_data);
}

void FXTrayIcon::setupNotifyData(NOTIFYICONDATA* data)
{
    ZeroMemory(data, sizeof(*data));
    data->cbSize = sizeof(*data);
    data->hWnd = static_cast<HWND>(xid);
    data->uID = mTrayID;
    data->hIcon = mWIcon;
    data->uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
    data->uCallbackMessage = WM_MOUSEMOVE; // Change ID

    // Tooltip
#ifdef UNICODE
    utf2ncs(data->szTip, mTooltip.text(), 64);
#else
    //SetWindowTextA((HWND)xid,title.text());
#error USE DEFINE UNICODE
#endif
}

HICON FXTrayIcon::createMswIcon(FXIcon* icon)
{
    // Windows Icon erzeugen
    ICONINFO iconinfo;
    ZeroMemory(&iconinfo, sizeof(iconinfo));
    iconinfo.fIcon=TRUE;
    iconinfo.hbmMask=(HBITMAP)icon->shape;
    iconinfo.hbmColor=(HBITMAP)icon->xid;
    return CreateIconIndirect(&iconinfo);
}

long FXTrayIcon::onEvent(FXObject* obj, FXSelector, void* ptr)
{
    if (ptr == 0)
        return 0;

    FXEvent* event = static_cast<FXEvent*>(ptr);
    int lParam = event->root_x;

    if ((lParam == WM_LBUTTONUP && (m_opts & TRAY_MENU_ON_LEFT)) ||
        (lParam == WM_RBUTTONUP && (m_opts & TRAY_MENU_ON_RIGHT)))
    {
        POINT p;
        GetCursorPos(&p);
        m_popup->popup(0, p.x, p.y);
        getApp()->addTimeout(this, ID_POPTIMEOUT, 10000);

        return 1;
    }
    if (target &&
        (lParam == WM_LBUTTONUP && (m_opts & TRAY_CMD_ON_LEFT)) ||
        (lParam == WM_RBUTTONUP && (m_opts & TRAY_CMD_ON_RIGHT)))
    {
        return target->tryHandle(obj, FXSEL(SEL_COMMAND, message), ptr);
    }
    return 0;
}

void FXTrayIcon::setIcon(FXIcon* icon)
{
    mIcon = icon;

    if (!mWIcon)
        return; // not created

    // Icon loeschen
    DestroyIcon(mWIcon);

    mWIcon = createMswIcon(icon);

    NOTIFYICONDATA tray_data;
    setupNotifyData(&tray_data);
    Shell_NotifyIcon(NIM_MODIFY, &tray_data);
}

void FXTrayIcon::setText(const FXString& text)
{
    mTooltip = text;

    if (!mWIcon)
        return; // not created

    NOTIFYICONDATA tray_data;
    setupNotifyData(&tray_data);
    Shell_NotifyIcon(NIM_MODIFY, &tray_data);
}
#else
FXTrayIcon::FXTrayIcon()
{
    m_icon = NULL;
    flags |= FLAG_ENABLED;
}

FXTrayIcon::FXTrayIcon(FXApp *app, const FXString& tip, FXIcon* icon,
            FXPopup* popup, FXObject* target, FXSelector sel,
            FXuint opts)
: FXTopWindow(app, "test", NULL, NULL, DECOR_NONE, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0), m_xtraywindow(0),
  m_xtrayopcode(0), m_socket(0), m_icon(icon), m_opaque(false), m_tip(tip), m_popup(popup), m_opts(opts), m_traycolor(getBackColor())
{
    flags |= FLAG_ENABLED;
    setTarget(target);
    setSelector(sel);
    static_cast<FXTrayApp*> (app)->registerIcon(this);
}

FXTrayIcon::~FXTrayIcon()
{
    static_cast<FXTrayApp*> (getApp())->unregisterIcon();
}

bool FXTrayIcon::doesOverrideRedirect() const
{
    return true;
}

FXbool FXTrayIcon::findSystemTray()
{
    FXString systemtray = FXStringFormat("_NET_SYSTEM_TRAY_S%d", DefaultScreen((Display*) getApp()->getDisplay()));
    Atom xtrayselection = XInternAtom((Display*) getApp()->getDisplay(), systemtray.text(), 0);
    if (xtrayselection != None)
    {
        m_xtraywindow = (FXID) XGetSelectionOwner((Display*) getApp()->getDisplay(), xtrayselection);
    }
    return (m_xtraywindow != 0);
}

void FXTrayIcon::requestDock()
{
    if (xid && m_xtraywindow)
    {
        XEvent ev;
        memset(&ev, 0, sizeof (ev));
        ev.xclient.type = ClientMessage;
        ev.xclient.window = m_xtraywindow;
        ev.xclient.message_type = m_xtrayopcode;
        ev.xclient.format = 32;
        ev.xclient.data.l[0] = CurrentTime;
        ev.xclient.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
        ev.xclient.data.l[2] = xid;
        XSendEvent((Display*) getApp()->getDisplay(), m_xtraywindow, False, NoEventMask, &ev);
        XSync((Display*) getApp()->getDisplay(), 0);
    }
}

FXuint FXTrayIcon::getTrayOrientation()
{
    if (m_xtrayorientation || m_xtrayxfceorientation)
    {
        FXuint orientation;
        Atom returntype;
        int returnformat;
        unsigned long nitems;
        unsigned long nbytes;
        long * bytes = NULL;

        if (m_xtrayorientation && (XGetWindowProperty((Display*) getApp()->getDisplay(), (Atom) m_xtraywindow, (Atom) m_xtrayorientation, 0, 2, False, XA_CARDINAL, &returntype, &returnformat, &nitems, &nbytes, (unsigned char**) (void*) &bytes) == Success) && returntype == XA_CARDINAL)
        {
            orientation = *(long*) bytes;
            XFree(bytes);
            return orientation;
        }

        if (bytes != NULL)
        {
            XFree(bytes);
            bytes = NULL;
        }

        if (m_xtrayxfceorientation && (XGetWindowProperty((Display*) getApp()->getDisplay(), (Atom) m_xtraywindow, (Atom) m_xtrayxfceorientation, 0, 2, False, XA_CARDINAL, &returntype, &returnformat, &nitems, &nbytes, (unsigned char**) (void*) &bytes) == Success) && returntype == XA_CARDINAL)
        {
            orientation = *(long*) bytes;
            XFree(bytes);
            return orientation;
        }

        if (bytes != NULL)
        {
            XFree(bytes);
            bytes = NULL;
        }
    }
    return SYSTEM_TRAY_UNKNOWN;
}

FXuint FXTrayIcon::getTrayVisual()
{
    if (m_xtrayvisual)
    {
        FXuint visualid;
        Atom returntype;
        int returnformat;
        unsigned long nitems;
        unsigned long nbytes;
        long * bytes = NULL;

        if (m_xtrayvisual && (XGetWindowProperty((Display*) getApp()->getDisplay(), m_xtraywindow, m_xtrayvisual, 0, 2, False, XA_VISUALID, &returntype, &returnformat, &nitems, &nbytes, (unsigned char**) (void*) &bytes) == Success) && returntype == XA_VISUALID)
        {
            visualid = *(long*) bytes;
            XFree(bytes);
            return visualid;
        }

        if (bytes != NULL)
        {
            XFree(bytes);
            bytes = NULL;
        }
    }
    return 0;
}

void FXTrayIcon::create()
{
    m_xtrayopcode = (FXID) XInternAtom((Display*) getApp()->getDisplay(), "_NET_SYSTEM_TRAY_OPCODE", 0);
    m_xtrayorientation = (FXID) XInternAtom((Display*) getApp()->getDisplay(), "_NET_SYSTEM_TRAY_ORIENTATION", 0);
    m_xtrayxfceorientation = (FXID) XInternAtom((Display*) getApp()->getDisplay(), "_NET_XFCE_TRAY_MANAGER_ORIENTATION", 0);
    m_xtrayvisual = (FXID) XInternAtom((Display*) getApp()->getDisplay(), "_NET_SYSTEM_TRAY_VISUAL", 0);

    FXTopWindow::create();
    if (xid)
    {
        Atom xembedinfo = XInternAtom((Display*) getApp()->getDisplay(), "_XEMBED_INFO", 0);
        if (xembedinfo != None)
        {
            unsigned long info[2] = {0, (1 << 0)};
            XChangeProperty((Display*) getApp()->getDisplay(), xid, xembedinfo, xembedinfo, 32, PropModeReplace, (unsigned char*) info, 2);
        }

        /// Set the size hints...
        XSizeHints size;
        size.flags = PMinSize | PMaxSize | PBaseSize | PAspect;
        size.x = 0;
        size.y = 0;
        size.width = 0;
        size.height = 0;
        size.width_inc = 0;
        size.height_inc = 0;
        size.min_aspect.x = 1;
        size.min_aspect.y = 1;
        size.max_aspect.x = 1;
        size.max_aspect.y = 1;
        size.win_gravity = 0;
        size.win_gravity = 0;

        size.min_width = 8;
        size.min_height = 8;
        size.max_width = 64;
        size.max_height = 64;
        size.base_width = 32;
        size.base_height = 32;
        XSetWMNormalHints((Display*) getApp()->getDisplay(), xid, &size);
    }
}

FXbool FXTrayIcon::dock()
{
    if (findSystemTray())
    {
        FXuint id = getTrayVisual();
        if (id)
        {
            if (id != XVisualIDFromVisual((Visual*) getVisual()->getVisual()))
                m_opaque = true;
            else
                m_opaque = false;
        }
        if (!m_opaque)
        {
            /// Don't draw the background
            XSetWindowAttributes sattr;
            sattr.background_pixmap = ParentRelative;
            XChangeWindowAttributes((Display*) getApp()->getDisplay(), xid, CWBackPixmap, &sattr);
        }
        requestDock();
        return true;
    }
    return false;
}

void FXTrayIcon::setFocus()
{
    FXShell::setFocus();
    if (xid && m_socket)
    {
        XEvent ev;
        memset(&ev, 0, sizeof (ev));
        ev.xclient.type = ClientMessage;
        ev.xclient.window = m_socket;
        ev.xclient.message_type = ((FXTrayApp*) getApp())->mXembed;
        ev.xclient.format = 32;
        ev.xclient.data.l[0] = CurrentTime;
        ev.xclient.data.l[1] = XEMBED_REQUEST_FOCUS;
        XSendEvent((Display*) getApp()->getDisplay(), m_socket, False, NoEventMask, &ev);
    }
}

long FXTrayIcon::onConfigure(FXObject*, FXSelector, void*ptr)
{
    FXEvent * event = (FXEvent*) ptr;
    FXuint orientation = getTrayOrientation();
    FXint size = 0;
    switch (orientation)
    {
    case SYSTEM_TRAY_HORIZONTAL: size = event->rect.h;
        break;
    case SYSTEM_TRAY_VERTICAL: size = event->rect.w;
        break;
    default: size = FXMAX(event->rect.h, event->rect.w);
        break;
    };
    resize(size, size);
    XSizeHints hint;
    hint.flags = PMinSize | PMaxSize | PBaseSize;
    hint.min_width = hint.max_width = hint.base_width = size;
    hint.min_height = hint.max_height = hint.base_height = size;
    XSetWMNormalHints((Display*) getApp()->getDisplay(), xid, &hint);
    return 1;
}

long FXTrayIcon::onRightBtnRelease(FXObject *obj, FXSelector, void *ptr)
{
    FXEvent *event = (FXEvent*) ptr;
    if (event->moved) return 0;
    if (m_opts & TRAY_MENU_ON_RIGHT)
    {
        FXEvent* event = static_cast<FXEvent*> (ptr);
        popup(event->click_x, event->click_y);
        return 1;
    }
    if (m_opts & TRAY_CMD_ON_RIGHT && target)
    {
        return target->tryHandle(obj, FXSEL(SEL_COMMAND, message), ptr);
    }
    return 0;
}

long FXTrayIcon::onLeftBtnPress(FXObject *obj, FXSelector, void *ptr)
{
    if (m_opts & TRAY_MENU_ON_LEFT)
    {
        FXEvent* event = static_cast<FXEvent*> (ptr);
        popup(event->click_x, event->click_y);
        return 1;
    }
    if (m_opts & TRAY_CMD_ON_LEFT && target)
    {
        return target->tryHandle(obj, FXSEL(SEL_COMMAND, message), ptr);
    }
    return 0;
}

long FXTrayIcon::onQueryTip(FXObject*sender, FXSelector sel, void* ptr)
{
    if (FXTopWindow::onQueryTip(sender, sel, ptr)) return 1;
    if ((flags & FLAG_TIP) && !m_tip.empty() && !grabbed())
    {
        sender->handle(this, FXSEL(SEL_COMMAND, ID_SETSTRINGVALUE), (void*) &m_tip);
        return 1;
    }
    return 0;
}

long FXTrayIcon::onPaint(FXObject*, FXSelector, void*)
{
    if (m_icon)
    {
        FXDCWindow dc(this);
        FXint dx = 0;
        FXint w = getWidth();
        FXint iw = m_icon->getWidth();
        if(w>iw) dx = (w-iw)/2;
        FXint dy = 0;
        FXint h = getHeight();
        FXint ih = m_icon->getHeight();
        if(h>ih) dy = (h-ih)/2;
        if (m_opaque)
        {
            dc.setForeground(m_traycolor);
            dc.fillRectangle(0, 0, w, h);
            dc.drawIcon(m_icon, dx, dy);
        }
        else
        {
            dc.drawIcon(m_icon, 0, 0);
            //move(dx,dy);
        }
    }
    return 1;
}

long FXTrayIcon::onEmbedded(FXObject*, FXSelector, void*ptr)
{
    flags |= FLAG_SHOWN;
    m_socket = (FXID) (FXival) ptr;
    return 1;
}

void FXTrayIcon::popup(FXint x, FXint y)
{
    if(!m_popup) return;
    translateCoordinatesTo(x, y, getRoot(), x, y);

    if (y > getRoot()->getHeight() / 2)
    {
        y -= m_popup->getDefaultHeight();
    }

    m_popup->popup(0, x, y);
    getApp()->addTimeout(this, ID_POPTIMEOUT, 10000);

}

void FXTrayIcon::setIcon(FXIcon* ic)
{
    if(m_icon != ic && ic)
    {
        m_icon = ic;
        if(!m_opaque) setShape(m_icon);
        update();
    }
}

FXIcon* FXTrayIcon::getIcon() const
{
    return m_icon;
}

void FXTrayIcon::setTrayColor(FXColor color)
{
    m_traycolor = color;
    if(m_opaque) update();
}
#endif

long FXTrayIcon::onTimeout(FXObject*, FXSelector, void*)
{
    if(!m_popup->shown()) return 1;
    FXint x,y, px, py;
    FXuint button;
    getCursorPosition(x,y,button);
    translateCoordinatesTo(px,py,getParent(),x,y);
    m_popup->contains(px,py) ? getApp()->addTimeout(this, ID_POPTIMEOUT, 10000) : m_popup->popdown();
    return 1;
}
}
