/*
 *      callback.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "defs.h"
#include "dxutils.h"
#include "callback.h"
#include "config.h"
#include "i18n.h"
#include "utilsmm.h"
#include "preferences.h"

static GMainLoop *gMainLoop = NULL;

gpointer purpleMainEventLoop(gpointer data)
{
    if(gMainLoop)
    {
        dxutils::debugLine("[CALLBACK] gMainLoop already created");
    }
    gMainLoop = g_main_loop_new(NULL, FALSE);
    dxutils::debugLine("[CALLBACK] Starting gMainLoop");
    g_main_loop_run(gMainLoop);
    g_main_loop_unref(gMainLoop);
    dxutils::debugLine("[CALLBACK] gMainLoop stopped");
    gMainLoop = NULL;
    g_thread_exit(NULL);
    return NULL;
}

dxfCallbacks::dxfCallbacks(FXApp *app, FXObject *target, bool offline) :
    m_feedback(app), m_target(target)
{
    m_purple = new PURPLE::Purple(this, "dxim", VERSION, "http://dxim.dxsolutions.org", dxutils::toStdString(dxutils::instance().getIniPath()));
    m_purple->setIdle(DXPREFERENCES.m_useAway, DXPREFERENCES.m_awayTimeout);
    m_purple->initPurple(offline);
    m_purple->loadSavedPlugins("/dxim/plugins/loaded");
#if GLIB_VERSION_MIN_REQUIRED > GLIB_VERSION_2_30
    g_thread_new("dxim-fox", purpleMainEventLoop, NULL);
#else
    if(!g_thread_supported())
    {
        g_thread_init(NULL);
    }
    g_thread_create(purpleMainEventLoop, NULL, FALSE, NULL);
#endif
}

dxfCallbacks::~dxfCallbacks()
{
    if(gMainLoop)
    {
        dxutils::debugLine("[CALLBACK] Stopping gMainLoop");
        g_main_loop_quit(gMainLoop);
    }
    else
    {
        dxutils::debugLine("[CALLBACK] No gMainLoop created");
    }
}

void dxfCallbacks::emitEvent(FXObject *tgt, FXuint type, FXuint id, const void* data, FXint size)
{
    m_feedback.message(tgt, FXSEL(type, id), data, size);
}

PURPLE::Purple *dxfCallbacks::getPurplemm()
{
    return m_purple;
}

/** Allow your application to be notified about errors on library layer */
void dxfCallbacks::showError(std::string msg)
{
    MsgInfo *event = new MsgInfo;
    event->type = MSGERROR;
    event->title = _("Error");
    event->text = dxutils::fromStdString(msg);
    event->okText = _("&OK");
    event->time = FXSystem::now();
    emitEvent(m_target, DX_NEWNOTIFYMSG, Purple_EVENT, &event, sizeof(MsgInfo*));
}

/** Libpurple inited */
void dxfCallbacks::buddylistLoaded()
{
    emitEvent(m_target, DX_BUDDYLOADED, Purple_EVENT, NULL, 0);
}

/** Allow your application to be notified about libpurple debug messages */
void dxfCallbacks::showDebug(std::string debug)
{
    dxutils::debugLine(dxutils::fromStdString(debug));
}

/** New account added */
void dxfCallbacks::accountAdded(PURPLE::Account account)
{
    emitEvent(m_target, DX_NEWACCOUNT, Purple_EVENT, &account, sizeof(PURPLE::Account));
}

/** Account removed */
void dxfCallbacks::accountRemoved(std::string account, std::string protocolID)
{
    AccountRemoved *event = new AccountRemoved;
    event->account = dxutils::fromStdString(account);
    event->protocolID = dxutils::fromStdString(protocolID);
    emitEvent(m_target, DX_ACCOUNTREMOVED, Purple_EVENT, &event, sizeof(AccountRemoved*));
}

/** Account connecting */
void dxfCallbacks::accountConnecting(PURPLE::Account account, std::string msg)
{
    AccountEvent *event = new AccountEvent;
    event->account = account;
    event->msg = dxutils::fromStdString(msg);
    emitEvent(m_target, DX_CONNECTING, Purple_EVENT, &event, sizeof(AccountEvent*));
}

/** Acount connected*/
void dxfCallbacks::accountConnected(PURPLE::Account account)
{
    emitEvent(m_target, DX_CONNECTED, Purple_EVENT, &account, sizeof(PURPLE::Account));
}

/** Acount disconnected*/
void dxfCallbacks::accountDisconnected(PURPLE::Account account, std::string reason)
{
    AccountEvent *event = new AccountEvent;
    event->account = account;
    event->msg = dxutils::fromStdString(reason);
    emitEvent(m_target, DX_DISCONNECTED, Purple_EVENT, &event, sizeof(AccountEvent*));
}

/** Account status changed*/
void dxfCallbacks::accountStatusChanged(PURPLE::Account account)
{
    emitEvent(m_target, DX_ACCOUNTPRESENCE, Purple_EVENT, &account, sizeof(PURPLE::Account));
}

/** Account signed on, so show buddy list */
void dxfCallbacks::accountSignedOn(PURPLE::Account account)
{
    emitEvent(m_target, DX_SIGNEDON, Purple_EVENT, &account, sizeof(PURPLE::Account));
}

/** Buddy signed on */
void dxfCallbacks::buddySignedOn(PURPLE::Buddy buddy)
{
    emitEvent(m_target, DX_BUDDYSIGNEDON, Purple_EVENT, &buddy, sizeof(PURPLE::Buddy));
}

/** Buddy signed off */
void dxfCallbacks::buddySignedOff(PURPLE::Buddy buddy)
{
    emitEvent(m_target, DX_BUDDYSIGNEDOFF, Purple_EVENT, &buddy, sizeof(PURPLE::Buddy));
}

/** Buddy status changed */
void dxfCallbacks::buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus)
{
    BuddyStatus *status = new BuddyStatus;
    status->buddy = buddy;
    status->oldStatus = oldStatus;
    status->newStatus = newStatus;
    emitEvent(m_target, DX_BUDDYPRESENCE, Purple_EVENT, &status, sizeof(BuddyStatus*));
}

/** Buddy icon changed */
void dxfCallbacks::buddyIconChanged(PURPLE::Buddy buddy)
{
    emitEvent(m_target, DX_BUDDYICON, Purple_EVENT, &buddy, sizeof(PURPLE::Buddy));
}

/** Buddy added */
void dxfCallbacks::buddyAdded(PURPLE::Buddy buddy)
{
    emitEvent(m_target, DX_NEWBUDDY, Purple_EVENT, &buddy, sizeof(PURPLE::Buddy));
}

/** Buddy removed */
void dxfCallbacks::buddyRemoved(PURPLE::Account account, std::string buddy)
{
    NodeRemoved *event = new NodeRemoved;
    event->account = account;
    event->who = dxutils::fromStdString(buddy);
    emitEvent(m_target, DX_BUDDYREMOVED, Purple_EVENT, &event, sizeof(NodeRemoved*));
}

/** Chat added */
void dxfCallbacks::chatAdded(PURPLE::Chat chat)
{
    emitEvent(m_target, DX_NEWCHAT, Purple_EVENT, &chat, sizeof(PURPLE::Chat));
}

/** Chat removed */
void dxfCallbacks::chatRemoved(PURPLE::Account account, std::string chat)
{
    NodeRemoved *event = new NodeRemoved;
    event->account = account;
    event->who = dxutils::fromStdString(chat);
    emitEvent(m_target, DX_CHATREMOVED, Purple_EVENT, &event, sizeof(NodeRemoved*));
}

/** New IM created */
void dxfCallbacks::newIM(PURPLE::Conversation *conv)
{
    emitEvent(m_target, DX_NEWTAB, Purple_EVENT, &conv, sizeof(PURPLE::Conversation*));
}

/** New Chat created */
void dxfCallbacks::newChat(PURPLE::Conversation *conv)
{
    emitEvent(m_target, DX_NEWTAB, Purple_EVENT, &conv, sizeof(PURPLE::Conversation*));
}

/** IM closed */
void dxfCallbacks::closedIM(PURPLE::Conversation *conv)
{
    emitEvent(m_target, DX_CLOSETAB, Purple_EVENT, &conv, sizeof(PURPLE::Conversation*));
}

/** Chat closed */
void dxfCallbacks::closedChat(PURPLE::Conversation *conv)
{
    emitEvent(m_target, DX_CLOSETAB, Purple_EVENT, &conv, sizeof(PURPLE::Conversation*));
}

/** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     */
void dxfCallbacks::newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID)
{
    NewMsg *event = new NewMsg;
    event->conv = conv;
    event->msg = dxutils::fromStdString(msg);
    event->time = time;
    event->action = action;
    event->messageID = dxutils::fromStdString(messageID);
    emitEvent(m_target, DX_NEWMYMSG, Purple_EVENT, &event, sizeof(NewMsg*));
}

void dxfCallbacks::messageRecived(PURPLE::Conversation *conv, std::string messageID)
{
    if(DXPREFERENCES.m_receiptsMark && !messageID.empty())
    {
        NewMsg *event = new NewMsg;
        event->conv = conv;
        event->messageID = dxutils::fromStdString(messageID);
        emitEvent(m_target, DX_MESSAGERECIVED, Purple_EVENT, &event, sizeof(NewMsg*));
    }
}

/** New IM message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxfCallbacks::newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    NewMsg *event = new NewMsg;
    event->conv = conv;
    event->sender = dxutils::fromStdString(sender);
    event->msg = dxutils::fromStdString(msg);
    event->time = time;
    event->history = delayed;
    event->highlight = highlight;
    event->action = action;
    event->messageID = "";
    emitEvent(m_target, DX_NEWMSG, Purple_EVENT, &event, sizeof(NewMsg*));
}

/** New Chat message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxfCallbacks::newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    NewMsg *event = new NewMsg;
    event->conv = conv;
    event->sender = dxutils::fromStdString(sender);
    event->msg = dxutils::fromStdString(msg);
    event->time = time;
    event->history = delayed;
    event->highlight = highlight;
    event->action = action;
    event->messageID = "";
    emitEvent(m_target, DX_NEWMSG, Purple_EVENT, &event, sizeof(NewMsg*));
}

/** New system mesage for conversation
    *
    * @param conv  The conversation
    * @param msg  The message text
    * @param time  The time the message was sent
    */
void dxfCallbacks::newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    NewSystemMsg *event = new NewSystemMsg;
    event->conv = conv;
    event->msg = dxutils::fromStdString(msg);
    event->time = time;
    emitEvent(m_target, DX_NEWSYSTEMMSG, Purple_EVENT, &event, sizeof(NewSystemMsg*));
}

/** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       error text
     * @param time      time the error was sent
     */
void dxfCallbacks::newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    NewSystemMsg *event = new NewSystemMsg;
    event->conv = conv;
    event->msg = dxutils::fromStdString(msg);
    event->time = time;
    emitEvent(m_target, DX_NEWERRORMSG, Purple_EVENT, &event, sizeof(NewSystemMsg*));
}

/** ChatBuddy joined chat */
void dxfCallbacks::chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    ChatBuddyEvent *event = new ChatBuddyEvent;
    event->conv = conv;
    event->buddy = buddy;
    event->action = CBJOINED;
    emitEvent(m_target, DX_CHATBUDDY, Purple_EVENT, &event, sizeof(ChatBuddyEvent*));
}

/** ChatBuddy updated */
void dxfCallbacks::chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy)
{
    ChatBuddyEvent *event = new ChatBuddyEvent;
    event->conv = conv;
    event->who = dxutils::fromStdString(buddy);
    event->action = CBUPDATED;
    emitEvent(m_target, DX_CHATBUDDY, Purple_EVENT, &event, sizeof(ChatBuddyEvent*));
}

/** ChatBuddy left chat */
void dxfCallbacks::chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
{
    ChatBuddyEvent *event = new ChatBuddyEvent;
    event->conv = conv;
    event->who = dxutils::fromStdString(buddy);
    event->action = CBLEFT;
    emitEvent(m_target, DX_CHATBUDDY, Purple_EVENT, &event, sizeof(ChatBuddyEvent*));
}

/** ChatBuddy renamed
    * @param obuddy  old buddy which was renamed
    * @param nbuddy  new buddy established by rename
    */
void dxfCallbacks::chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy)
{
    ChatBuddyEvent *event = new ChatBuddyEvent;
    event->conv = conv;
    event->buddy = nbuddy;
    event->who = dxutils::fromStdString(who);
    event->action = CBRENAMED;
    emitEvent(m_target, DX_CHATBUDDY, Purple_EVENT, &event, sizeof(ChatBuddyEvent*));
}

/** Chat topic changed
    * @param who  topic changer
    * @param topic  new topic
    */
void dxfCallbacks::chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic)
{
    ChatTopic *event = new ChatTopic;
    event->conv = conv;
    event->who = dxutils::fromStdString(who);
    event->topic = dxutils::fromStdString(topic);
    emitEvent(m_target, DX_CHATTOPIC, Purple_EVENT, &event, sizeof(ChatTopic*));
}

/** Buddy typing on IM
    * @param typing =true typing, =false stoped typing
    */
void dxfCallbacks::buddyTyping(PURPLE::Conversation *conv, bool typing)
{
    BuddyTyping *event = new BuddyTyping;
    event->conv = conv;
    event->typing = typing;
    emitEvent(m_target, DX_BUDDYTYPING, Purple_EVENT, &event, sizeof(BuddyTyping*));
}

/** New FileTransfer */
void dxfCallbacks::newFileTransfer(PURPLE::FileTransfer *ft)
{
    emitEvent(m_target, DX_NEWFT, Purple_EVENT, &ft, sizeof(PURPLE::FileTransfer*));
}

/** FileTransfer finished */
void dxfCallbacks::fileTransferFinished(PURPLE::FileTransfer *ft)
{
    emitEvent(m_target, DX_FINISHEDFT, Purple_EVENT, &ft, sizeof(PURPLE::FileTransfer*));
}

/** FileTransfer started */
void dxfCallbacks::fileTransferStarted(PURPLE::FileTransfer *ft)
{
    emitEvent(m_target, DX_STARTEDFT, Purple_EVENT, &ft, sizeof(PURPLE::FileTransfer*));
}

/** FileTransfer progress */
void dxfCallbacks::fileTransferProgress(PURPLE::FileTransfer *ft, double /*percent*/)
{
    emitEvent(m_target, DX_PROGRESSFT, Purple_EVENT, &ft, sizeof(PURPLE::FileTransfer*));
}

/** FileTransfer canceled */
void dxfCallbacks::fileTransferCanceled(PURPLE::FileTransfer *ft)
{
    emitEvent(m_target, DX_CANCELEDFT, Purple_EVENT, &ft, sizeof(PURPLE::FileTransfer*));
}

/** A new list was created */
void dxfCallbacks::roomlistCreated(PURPLE::RoomList *rlist)
{
    emitEvent(m_target, DX_NEWROOMLIST, Purple_EVENT, &rlist, sizeof(PURPLE::RoomList*));
}

/** Sets the columns of RoomList */
void dxfCallbacks::roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns)
{
    RoomlistColumns *event = new RoomlistColumns;
    event->list = rlist;
    event->columns = columns;
    emitEvent(m_target, DX_ROOMLISTCOLUMNS, Purple_EVENT, &event, sizeof(RoomlistColumns*));
}

/** Add a room to the list */
void dxfCallbacks::roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields)
{
    Room *event = new Room;
    event->list = rlist;
    event->fields = fields;
    emitEvent(m_target, DX_ROOMLISTROOM, Purple_EVENT, &event, sizeof(Room*));
}

/** Listing RoomList finished */
void dxfCallbacks::roomlistFinished(PURPLE::RoomList *rlist)
{
    emitEvent(m_target, DX_ROOMLISTFINISHED, Purple_EVENT, &rlist, sizeof(PURPLE::RoomList*));
}

/** Notification message to the user
 * @param type  notification type
 * @param title  title of the message
 * @param primary  main point of the message
 * @param secondary  secondary information
*/
void dxfCallbacks::notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary)
{
    MsgInfo *event = new MsgInfo;
    switch(type){
        case PURPLE_NOTIFY_MSG_ERROR:
            event->type = MSGERROR;
            break;
        case PURPLE_NOTIFY_MSG_INFO:
            event->type = MSGINFORMATION;
            break;
        case PURPLE_NOTIFY_MSG_WARNING:
            event->type = MSGWARNING;
            break;
    };
    event->title = dxutils::fromStdString(title);
    event->text.append(dxutils::fromStdString(primary));
    event->text.append("<br>");
    event->text.append(dxutils::fromStdString(secondary));
    event->okText = _("&OK");
    event->time = FXSystem::now();
    emitEvent(m_target, DX_NEWNOTIFYMSG, Purple_EVENT, &event, sizeof(MsgInfo*));
}

/** Email notification to the user
 * @param subject  subject of the email
 * @param from  from address
 * @param to  estination address
 * @param url  URL where the message can be read
*/
void dxfCallbacks::notifyEmail(std::string subject, std::string from, std::string to, std::string url)
{
    MsgInfo *event = new MsgInfo;
    if(url.empty())
    {
        event->type = MSGINFORMATION;
        event->okText = _("&OK");
    }
    else
    {
        event->type = MSGNEWMAIL;
        event->url = dxutils::fromStdString(url);
        event->okText = _("&Show in browser");
        event->cancelText = _("&Close");
    }
    event->title = _("New email");
    event->text.append(FXStringFormat(_("From: %s"), from.c_str()));
    event->text.append("<br>");
    event->text.append(FXStringFormat(_("To: %s"), to.c_str()));
    event->text.append("<br>");
    event->text.append(FXStringFormat(_("Subject: %s"), subject.c_str()));
    event->time = FXSystem::now();
    emitEvent(m_target, DX_NEWNOTIFYMSG, Purple_EVENT, &event, sizeof(MsgInfo*));
}

/** Notification with formatted text
 * @param title  title of the message
 * @param primary  main point of the message
 * @param secondary  secondary information
 * @param text  formatted text
*/
void dxfCallbacks::notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated)
{
    MsgInfo *event = new MsgInfo;
    event->type = MSGINFORMATION;
    event->title = dxutils::fromStdString(title);
    event->text.append(dxutils::fromStdString(primary));
    event->text.append("<br>");
    event->text.append(dxutils::fromStdString(secondary));
    event->text.append("<br>");
    event->text.append(dxutils::fromStdString(formated));
    event->okText = _("&OK");
    event->time = FXSystem::now();
    emitEvent(m_target, DX_NEWNOTIFYMSG, Purple_EVENT, &event, sizeof(MsgInfo*));
}

/** Notification information about buddy
     * @param account   requested account
     * @param buddy     buddy which about
     * @param info      vector with UserInfo values
    */
void dxfCallbacks::notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info)
{
    UserInfo *event = new UserInfo;
    event->account = account;
    event->buddy = dxutils::fromStdString(buddy);
    std::vector<PURPLE::Notify::UserInfo>::iterator it = info.begin();
    for(; it!=info.end(); ++it)
    {
        FXString value = dxutils::fromStdString((*it).getValue());
        if(!compare(value.left(9), "<img id='"))
        {
            std::string label = (*it).getLabel();
            int id = FXIntVal(value.after('\'').before('\''));
            FXString path = dxutils::fromStdString(PURPLE::imagePathByID(id));
            if(path.empty())
            {
                value = _("Image couldn't be loaded");
            }
            else
            {
                value = FXStringFormat("\023#%d:%s", id, path.text());
            }
            info.erase(it);
            info.push_back(PURPLE::Notify::UserInfo(label, dxutils::toStdString(value)));
        }
    }
    event->info = info;
    emitEvent(m_target, DX_USERINFO, Purple_EVENT, &event, sizeof(UserInfo*));
}

/** Opens a URI or somehow presents it to the user
 * @param uri    The URI to display or go to.
*/
void dxfCallbacks::notifyURI(std::string uri)
{
    if(!uri.empty())
    {
        MsgInfo *event = new MsgInfo;
        event->type = MSGNEWMAIL;
        event->okText = _("&Show in browser");
        event->cancelText = _("&Close");
        event->title = _("Link");
        event->text = dxutils::fromStdString(uri);
        event->time = FXSystem::now();
        emitEvent(m_target, DX_NEWNOTIFYMSG, Purple_EVENT, &event, sizeof(MsgInfo*));
    }
}

/** Play sound for given event */
void dxfCallbacks::playSound(PurpleSoundEventID /*event*/)
{}

/** New InputRequest */
void dxfCallbacks::newInputRequest(PURPLE::InputRequest *request)
{
    emitEvent(m_target, DX_INPUTREQUEST, Purple_EVENT, &request, sizeof(PURPLE::InputRequest*));
}

/** New ChoiceRequest */
void dxfCallbacks::newChoiceRequest(PURPLE::ChoiceRequest *request)
{
    emitEvent(m_target, DX_CHOICEREQUEST, Purple_EVENT, &request, sizeof(PURPLE::ChoiceRequest*));
}

/** New ActionRequest */
void dxfCallbacks::newActionRequest(PURPLE::ActionRequest *request)
{
    emitEvent(m_target, DX_ACTIONREQUEST, Purple_EVENT, &request, sizeof(PURPLE::ActionRequest*));
}

/** New FileRequest */
void dxfCallbacks::newFileRequest(PURPLE::FileRequest *request)
{
    emitEvent(m_target, DX_FILEREQUEST, Purple_EVENT, &request, sizeof(PURPLE::FileRequest*));
}

/** New FieldsRequest */
void dxfCallbacks::newFieldsRequest(PURPLE::FieldsRequest *request)
{
    emitEvent(m_target, DX_FIELDSREQUEST, Purple_EVENT, &request, sizeof(PURPLE::FieldsRequest*));
}

/** Close given request (ex. dialog) */
void dxfCallbacks::closeRequest(PURPLE::BasicRequest */*request*/)
{}

