/*
 *      accounteditor.cpp
 *
 *      Copyright 2010 David Schwarz <david.schwarz@live.com>
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "accounteditor.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "dxutils.h"
#include "preferences.h"

#define MAX_ICON_SIZE 128

//{ FOX method mapping

FXDEFMAP(AccountEditor) AccountEditorMap[] =
{
    FXMAPFUNC(FX::SEL_COMMAND,          AccountEditor_SAVE,         AccountEditor::onSave),
    FXMAPFUNC(FX::SEL_COMMAND,          AccountEditor_CANCEL,       AccountEditor::onCancel),
    FXMAPFUNC(FX::SEL_CLOSE,            0,                          AccountEditor::onCancel),
    FXMAPFUNC(FX::SEL_KEYPRESS,         0,                          AccountEditor::onKeyPress),
    FXMAPFUNC(FX::SEL_MOUSEWHEEL,       AccountEditor_TABS,         AccountEditor::onMouseWheel),
    FXMAPFUNC(FX::SEL_COMMAND,          AccountEditor_ICON,         AccountEditor::onBuddyIcon),
    FXMAPFUNC(FX::SEL_COMMAND,          AccountEditor_REMOVE,       AccountEditor::onRemoveBuddyIcon)
};

FXIMPLEMENT(AccountEditor, FXDialogBox, AccountEditorMap, ARRAYNUMBER(AccountEditorMap))

//}

//{ Constructor & Destructor

/* Constructor - Create new account */
AccountEditor::AccountEditor(FXWindow *owner, PURPLE::Protocol protocol, bool gtalk)
    : FXDialogBox(owner, _("Create new account"), DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
{
    this->m_editing = false;
    this->m_protocol = protocol;
    this->m_account = PURPLE::Account();
    this->m_options = protocol.getOptions();
    this->m_gtalk = gtalk;
    this->initializeEditor();
}
/* Constructor - Modify existing account */
AccountEditor::AccountEditor(FXWindow *owner, PURPLE::Account account)
: FXDialogBox(owner, _("Modify account"), DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
{
    this->m_editing = true;
    this->m_protocol = account.getProtocol();
    this->m_account = account;
    this->m_options = account.getOptions();
    this->m_gtalk = false;
    this->initializeEditor();
}

/* Destructor */
AccountEditor::~AccountEditor()
{
}

//}

//{ Methods

/* Initializies editor */
void AccountEditor::initializeEditor()
{
    FXHorizontalFrame *buttonFrame = new FXHorizontalFrame(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
    new dxEXButton(buttonFrame, _("&Cancel"), NULL, this, AccountEditor_CANCEL, FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);
    new dxEXButton(buttonFrame, _("&OK"), NULL, this, AccountEditor_SAVE, BUTTON_INITIAL|BUTTON_DEFAULT|FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT, 0, 0, 0, 0, 10, 10, 2, 2);

    FXHorizontalFrame *contents = new FXHorizontalFrame(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabbook = new dxTabBook(contents, this, AccountEditor_TABS, PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabbook->setTabStyle(TABBOOK_LEFTTABS);
    createAccountTab();
    createProtocolTab();
}

/* Create tab for account settings */
void AccountEditor::createAccountTab()
{
    new dxEXTabItem(m_tabbook, _("&Account settings"), NULL, TAB_LEFT);
    FXVerticalFrame *accountpane = new FXVerticalFrame(m_tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXMatrix *matrix = new FXMatrix(accountpane, 2, LAYOUT_FILL_X|MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    new FXLabel(matrix, _("Protocol"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    if(m_gtalk) new FXLabel(matrix, "Google Talk", dxutils::getProtocolIcon("gtalk"), ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    else new FXLabel(matrix, dxutils::fromStdString(m_protocol.getName()), dxutils::getProtocolIcon(dxutils::fromStdString(m_protocol.getPurpleID())), ICON_BEFORE_TEXT|JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    FXTextField *field = NULL;
    FXString username = "";
    if(m_editing) username = dxutils::fromStdString(m_account.getUsername());
    if(splits.size())
    {
        new FXLabel(matrix, _("User name"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        field = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        if(m_editing)
        {
            field->setText(username.before(splits[0].getSeparator()));
            username = username.after(splits[0].getSeparator());
            if(m_account.isEnabled()) field->disable();
        }
        m_usernameFields.append(field);
        for(FXulong i=0; i<splits.size(); i++)
        {
            new FXLabel(matrix, dxutils::fromStdString(splits[i].getText()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
            field = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
            if(m_editing)
            {
                if(i+1 == splits.size())
                {
                    field->setText(username);
                }
                else
                {
                    field->setText(username.before(splits[i+1].getSeparator()));
                    username = username.after(splits[i+1].getSeparator());
                }
                if(m_account.isEnabled()) field->disable();
            }
            m_usernameFields.append(field);
        }
        if(!m_editing && m_gtalk)
        {
            m_usernameFields[1]->setText("gmail.com");
        }
    }
    else
    {
        new FXLabel(matrix, _("User name"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
        field = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
        if(m_editing)
        {
            field->setText(dxutils::fromStdString(m_account.getUsername()));
            if(m_account.isEnabled()) field->disable();
        }
        m_usernameFields.append(field);
    }
    new FXLabel(matrix, _("Password"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_password = new FXTextField(matrix, 30, NULL, 0, TEXTFIELD_PASSWD|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    if(m_editing) m_password->setText(dxutils::fromStdString(m_account.getPassword()));
    new FXLabel(matrix, "", NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_rememberPassword = new FXCheckButton(matrix, _("Remember password"));
    if(!m_editing || m_account.getRememberPassword()) m_rememberPassword->setCheck(true);
    new FXLabel(matrix, _("Alias"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_alias = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
    if(m_editing) m_alias->setText(dxutils::fromStdString(m_account.getAlias()));
    m_checkEmail = new FXCheckButton(accountpane, _("Check new mails"));
    if(m_editing) m_checkEmail->setCheck(m_account.getCheckMail());
    if(!m_protocol.hasCheckMail()) m_checkEmail->hide();
    m_buddyIconPath = "";
    FXVerticalFrame *buddyFrame = new FXVerticalFrame(accountpane, LAYOUT_FILL_Y);
    new FXLabel(buddyFrame, _("Icon"));
    FXHorizontalFrame *buttonFrame = new FXHorizontalFrame(buddyFrame);
    m_buddyIcon = new dxEXButton(buttonFrame, "", NULL, this, AccountEditor_ICON);
    m_removeBuddyIcon = new dxEXButton(buttonFrame, _("Remove"), NULL, this, AccountEditor_REMOVE);
    if(m_protocol.hasBuddyIcon())
    {
        if(!m_editing || m_account.getBuddyIcon().empty())
        {
            m_buddyIcon->setText(_("Icon isn't set"));
            m_removeBuddyIcon->hide();
        }
        else
        {
            m_buddyIconPath = dxutils::fromStdString(m_account.getBuddyIcon());
            m_buddyIcon->setIcon(makeScaledIcon(getApp(), m_buddyIconPath, MAX_ICON_SIZE, DXPREFERENCES.m_appTheme.back));
        }
    }
    else
    {
        buddyFrame->hide();
    }
    m_registerAccount = new FXCheckButton(accountpane, _("Register account"));
    m_registerAccount->setCheck(false);
    if(m_editing) m_registerAccount->hide();
}

/* Create tab for protocol options */
void AccountEditor::createProtocolTab()
{
    new dxEXTabItem(m_tabbook, _("&Protocol options"), NULL, TAB_LEFT);
    FXVerticalFrame *protocolpane = new FXVerticalFrame(m_tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXMatrix *matrix = new FXMatrix(protocolpane, 2, MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    FXTextField *text;
    FXListBox *combo;
    FXCheckButton *check;
    FOREACH(PURPLE::Protocol::Options, m_options, it)
    {
        PURPLE::Protocol::Option& option = *it;
        std::map<std::string, std::string> values = option.getStringList();
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                new FXLabel(matrix, dxutils::fromStdString(option.getText()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                text = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                new FXLabel(matrix, dxutils::fromStdString(option.getText()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                combo = new FXListBox(matrix, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                std::map<std::string, std::string>::iterator value = values.begin();
                FXint i=0;
                for(; value!=values.end(); ++value, i++)
                {
                    combo->appendItem(dxutils::fromStdString((*value).first));
                    if(!option.getStringValue().compare((*value).second))
                        combo->setCurrentItem(i);
                }
                m_optionsFields.append(combo);
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                new FXLabel(matrix, dxutils::fromStdString(option.getText()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                text = new FXTextField(matrix, 30, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
                text->setTextStyle(TEXTFIELD_INTEGER);
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                new FXLabel(matrix, dxutils::fromStdString(option.getText()), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
                check = new FXCheckButton(matrix, "", NULL, 0);
                check->setCheck(option.getBoolValue());
                m_optionsFields.append(check);
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                m_optionsFields.append(NULL);
                break;
        }
    }
}

std::string AccountEditor::getStrlistValue(std::map<std::string, std::string> map, std::string key)
{
    std::string result;
    std::map<std::string, std::string>::iterator it = map.find(key);
    if(it!=map.end())
        result = (*it).second;
    return result;
}

void AccountEditor::fillProtocolOptions()
{
    PURPLE::Protocol::Options::iterator it;
    FXuint i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Protocol::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                option.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_optionsFields[i])->getText()));
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                FXListBox *combo = static_cast<FXListBox*>(m_optionsFields[i]);
                option.setValue(getStrlistValue(option.getStringList(), dxutils::toStdString(combo->getItemText(combo->getCurrentItem()))));
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                option.setValue(dxutils::toStdString(static_cast<FXTextField*>(m_optionsFields[i])->getText()));
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                option.setValue(static_cast<FXCheckButton*>(m_optionsFields[i])->getCheck() ? "true" : "false");
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

std::string AccountEditor::joinUsername()
{
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    FXString username = m_usernameFields[0]->getText();
    if(splits.size())
    {
        for(FXulong i=0; i<splits.size(); i++)
        {
            username.append(splits[i].getSeparator());
            username.append(m_usernameFields[i+1]->getText());
        }
    }
    return dxutils::toStdString(username);
}

/* Handles cancelation of modifying or creating account */
long AccountEditor::onCancel(FXObject *, FXSelector, void *)
{
    // Nothing to do..
    getApp()->stopModal(this, FALSE);
    this->hide();
    return 1;
}

/* Handles modifying & creating account */
long AccountEditor::onSave(FXObject *, FXSelector, void *)
{
    getApp()->stopModal(this, TRUE);
    this->hide();
    return 1;
}

/* Handles users key pressing */
long AccountEditor::onKeyPress(FXObject *sender, FXSelector selector, void *ptr)
{
    if(FXTopWindow::onKeyPress(sender, selector, ptr))
    {
        return 1;
    }

    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        handle(this, FXSEL(SEL_COMMAND, AccountEditor_CANCEL), NULL);
        return 1;
    }

    return 0;
}

/* Handles mouse wheel for tabbook */
long AccountEditor::onMouseWheel(FXObject*, FXSelector, void *ptr)
{
    FXEvent *event = (FXEvent*)ptr;
    FXint index = m_tabbook->getCurrent();
    if(event->code > 0) //positive movement
    {
        if(m_tabbook->numChildren())
        {
            if((index-1) >= 0) m_tabbook->setCurrent(index-1, TRUE);
            else m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
        }
    }
    else
    {
        if(m_tabbook->numChildren())
        {
            if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
            else m_tabbook->setCurrent(0, TRUE);
        }
    }
    return 1;
}

/* Handles click on m_buddyIcon */
long AccountEditor::onBuddyIcon(FXObject*, FXSelector, void*)
{
    dxEXFileDialog dialog(this, _("Select buddy icon"));
    dialog.setFilename(m_buddyIconPath);
    dialog.setPattern("*.png,*.jpg");
    if(dialog.execute())
    {
        bool needResize = m_buddyIconPath.empty()?true:false;
        FXint h = m_buddyIcon->getHeight();
        m_buddyIconPath = dialog.getFilename();
        FXIcon *icon = makeScaledIcon(getApp(), m_buddyIconPath, MAX_ICON_SIZE, DXPREFERENCES.m_appTheme.back);
        m_buddyIcon->setIcon(icon);
        m_buddyIcon->setText("");
        m_removeBuddyIcon->show();
        if(needResize && icon) resize(getWidth(), getHeight()+icon->getHeight()-h);
    }
    return 1;
}

/* Handles click on m_removeBuddyIcon */
long AccountEditor::onRemoveBuddyIcon(FXObject*, FXSelector, void*)
{
    m_buddyIconPath = "";
    m_buddyIcon->setIcon(NULL);
    m_buddyIcon->setText(_("Icon isn't set"));
    m_removeBuddyIcon->hide();
    return 1;
}

/* Fills new Account */
void AccountEditor::fillAccount(NewAccount *account)
{
    account->protocol = m_protocol;
    account->username = joinUsername();
    account->password = dxutils::toStdString(m_password->getText());
    account->iconPath = dxutils::toStdString(m_buddyIconPath);
    account->registerAccount = m_registerAccount->getCheck();
    account->checkEmail = m_checkEmail->getCheck();
    account->rememberPassword = m_rememberPassword->getCheck();
    account->alias = dxutils::toStdString(m_alias->getText());
    fillProtocolOptions();
    account->options = m_options;
}


//}
