/*
 *      configdialog.cpp
 *
 *      Copyright 2010 David Vachulka <arch_dvx@users.sourceforge.net>
 *      Copyright (C) 2004-2005 Sander Jansen (some code)
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "configdialog.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "dxutils.h"
#include "tabitems/dxtabbook.h"
#include "keybindingsdialog.h"
#include "preferences.h"
#include "dxim.h"

#define HEADER "# dxim smiley settings\n# path is relative to file\n# Example: /home/xxx/smile.png=:)\n"

static FXIcon *createIconFromName(FXApp *app, const FXString& path)
{
    FXIconSource iconsource(app);
    FXIcon *icon = NULL;
    if(!FXStat::exists(path))
        return NULL;
    icon = iconsource.loadScaledIconFile(path,16);
    if(icon)
    {
        icon->blend(app->getBaseColor());
        icon->create();
        return icon;
    }
    return NULL;
}

static void updateLabelIcon(FXApp* app, FXLabel* label, const FXString& path)
{
    FXIconSource iconsource(app);
    FXIcon *icon = NULL;
    if(label->getIcon())
    {
        icon = label->getIcon();
        label->setIcon(NULL);
        delete icon;
        icon = NULL;
    }
    icon = iconsource.loadScaledIconFile(path,16);
    if(icon)
    {
        icon->blend(app->getBaseColor());
        icon->create();
        label->setIcon(icon);
    }
}

FXDEFMAP(SmileyDialog) SmileyDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, SmileyDialog_ACCEPT, SmileyDialog::onAccept),
    FXMAPFUNC(SEL_COMMAND, SmileyDialog_CANCEL, SmileyDialog::onCancel),
    FXMAPFUNC(SEL_COMMAND, SmileyDialog_PATH, SmileyDialog::onPath)
};

FXIMPLEMENT(SmileyDialog, FXDialogBox, SmileyDialogMap, ARRAYNUMBER(SmileyDialogMap))

SmileyDialog::SmileyDialog(FXWindow* owner, FXString title, FXString smiley, FXString path)
        : FXDialogBox(owner, title, DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    FXHorizontalFrame *closeframe = new FXHorizontalFrame(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
    new dxEXButton(closeframe, _("&Cancel"), NULL, this, SmileyDialog_CANCEL, LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK, 0,0,0,0, 10,10,2,2);
    dxEXButton *ok = new dxEXButton(closeframe, _("&OK"), NULL, this, SmileyDialog_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK, 0,0,0,0, 10,10,2,2);
    ok->addHotKey(KEY_Return);

    FXMatrix *matrix = new FXMatrix(this,2,MATRIX_BY_COLUMNS|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXLabel(matrix, _("Smiley:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_smileyText = new FXTextField(matrix, 25, NULL, 0, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    if(!smiley.empty()) m_smileyText->setText(smiley);
    new FXLabel(matrix, _("Path:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    FXHorizontalFrame *pathframe = new FXHorizontalFrame(matrix, LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,0,0, 0,0,0,0);
    m_pathText = new FXTextField(pathframe, 25, NULL, 0, TEXTFIELD_READONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X);
    m_pathText->setBackColor(getApp()->getBaseColor());
    if(!path.empty()) m_pathText->setText(path);
    m_pathButton = new dxEXButton(pathframe, "...", NULL, this, SmileyDialog_PATH, FRAME_RAISED|FRAME_THICK);
    new FXLabel(matrix, _("Preview:"), NULL, JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
    m_previewLabel = new FXLabel(matrix, "", path.empty() ? NULL : createIconFromName(getApp(), path), JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
}

SmileyDialog::~SmileyDialog()
{}

long SmileyDialog::onAccept(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,TRUE);
    hide();
    return 1;
}

long SmileyDialog::onCancel(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

long SmileyDialog::onPath(FXObject*, FXSelector, void*)
{
    dxEXFileDialog dialog(this, _("Select file"));
    if(!m_pathText->getText().empty()) dialog.setFilename(m_pathText->getText());
    else dialog.setFilename((FXString)DXIM_DATADIR+PATHSEPSTRING+"icons"+PATHSEPSTRING+"smileys"+PATHSEPSTRING);
    if(dialog.execute())
    {
        m_pathText->setText(dialog.getFilename());
        updateLabelIcon(getApp(), m_previewLabel, dialog.getFilename());
    }
    return 1;
}

FXbool SmileyDialog::iconExist()
{
    if(m_previewLabel->getIcon()) return TRUE;
    return FALSE;
}

FXDEFMAP(ConfigDialog) ConfigDialogMap[] = {
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_ACCEPT, ConfigDialog::onAccept),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_CANCEL, ConfigDialog::onCancel),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_STATUSICONS, ConfigDialog::onStatusIconsChanged),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_ADDSTATUSICONS, ConfigDialog::onAddStatusIcons),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_DELETESTATUSICONS, ConfigDialog::onDeleteStatusIcons),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_ROLEICONS, ConfigDialog::onRoleIconsChanged),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_ADDROLEICONS, ConfigDialog::onAddRoleIcons),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_DELETEROLEICONS, ConfigDialog::onDeleteRoleIcons),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_TRAY, ConfigDialog::onTray),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_LOG, ConfigDialog::onLogChanged),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_LOGPATH, ConfigDialog::onPathSelect),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_NICK, ConfigDialog::onNickCharChanged),
    FXMAPFUNC(SEL_KEYPRESS, 0, ConfigDialog::onKeyPress),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_IMCOLORS, ConfigDialog::onColor),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_IMCOLORS, ConfigDialog::onColor),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_COLORS, ConfigDialog::onThemeColorChanged),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_COLORS, ConfigDialog::onThemeColorChanged),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_TABCOLORS, ConfigDialog::onTabColorChanged),
    FXMAPFUNC(SEL_CHANGED, ConfigDialog_TABCOLORS, ConfigDialog::onTabColorChanged),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_THEME, ConfigDialog::onTheme),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_FONT, ConfigDialog::onFont),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_IMFONT, ConfigDialog::onImFont),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_TABPOS, ConfigDialog::onTabPosition),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_NOTIFYPOS, ConfigDialog::onNotifyPosition),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_SOUNDS, ConfigDialog::onSounds),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_SOUNDCONNECT, ConfigDialog::onSoundConnect),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_SOUNDDISCONNECT, ConfigDialog::onSoundDisconnect),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_SOUNDMESSAGE, ConfigDialog::onSoundMessage),
    FXMAPFUNCS(SEL_COMMAND, ConfigDialog_PLAYCONNECT, ConfigDialog_PLAYMESSAGE, ConfigDialog::onPlay),
    FXMAPFUNCS(SEL_COMMAND, ConfigDialog_SELECTCONNECT, ConfigDialog_SELECTMESSAGE, ConfigDialog::onSelectPath),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_NOTIFY, ConfigDialog::onNotify),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_SHOWNOTIFY, ConfigDialog::onShowNotify),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_USESMILEYS, ConfigDialog::onUseSmileys),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_ADDSMILEY, ConfigDialog::onAddSmiley),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_MODIFYSMILEY, ConfigDialog::onModifySmiley),
    FXMAPFUNC(SEL_DOUBLECLICKED, ConfigDialog_SMILEY, ConfigDialog::onModifySmiley),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_DELETESMILEY, ConfigDialog::onDeleteSmiley),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_IMPORTSMILEY, ConfigDialog::onImportSmiley),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_EXPORTSMILEY, ConfigDialog::onExportSmiley),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_USEAWAY, ConfigDialog::onUseAway),
    FXMAPFUNC(SEL_MOUSEWHEEL, ConfigDialog_TABS, ConfigDialog::onMouseWheel),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_MODIFYBINDINGS, ConfigDialog::onModifyBindings),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_REFRESHBINDINGS, ConfigDialog::onRefreshBindings),
    FXMAPFUNC(SEL_COMMAND, ConfigDialog_RAISEWINDOW, ConfigDialog::onRaiseWindow),
};

FXIMPLEMENT(ConfigDialog, FXDialogBox, ConfigDialogMap, ARRAYNUMBER(ConfigDialogMap))

ConfigDialog::ConfigDialog(FXMainWindow *owner)
    : FXDialogBox(owner, _("Preferences"), DECOR_RESIZE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
    m_preferences = DXPREFERENCES;
    //create m_imFont
    if(!m_preferences.m_imFont.empty())
    {
        m_imFont = new FXFont(getApp(), m_preferences.m_imFont);
        m_imFont->create();
    }
    else
    {
        getApp()->getNormalFont()->create();
        FXFontDesc fontdescription;
        getApp()->getNormalFont()->getFontDesc(fontdescription);
        m_imFont = new FXFont(getApp(),fontdescription);
        m_imFont->create();
    }
    //set theme
    m_themeCurrent = m_preferences.m_appTheme;

    m_showImportwarning = TRUE;

    //General tab
    m_targetOneInstance.connect(m_preferences.m_oneInstance);
    m_targetAwayTimeout.connect(m_preferences.m_awayTimeout);
    m_targetAwayTimeout.setTarget(this);
    m_targetAwayTimeout.setSelector(ConfigDialog_AWAY);
    m_targetUseAway.connect(m_preferences.m_useAway);
    m_targetUseAway.setTarget(this);
    m_targetUseAway.setSelector(ConfigDialog_USEAWAY);
    m_targetTray.connect(m_preferences.m_useTray);
    m_targetTray.setTarget(this);
    m_targetTray.setSelector(ConfigDialog_TRAY);
    m_targetCloseToTray.connect(m_preferences.m_closeToTray);
    m_targetMinToTray.connect(m_preferences.m_minToTray);
    m_targetLogging.connect(m_preferences.m_logging);
    m_targetLogging.setTarget(this);
    m_targetLogging.setSelector(ConfigDialog_LOG);
    m_targetLogPath.connect(m_preferences.m_logPath);
    m_targetLogLoadCurrent.connect(m_preferences.m_logLoadCurrent);
    m_targetShowRosterTree.connect(m_preferences.m_showRosterTree);
    m_targetReceiptsMark.connect(m_preferences.m_receiptsMark);

    //Look tab
    m_targetThemeBack.connect(m_themeCurrent.back);
    m_targetThemeBack.setTarget(this);
    m_targetThemeBack.setSelector(ConfigDialog_COLORS);
    m_targetThemeBase.connect(m_themeCurrent.base);
    m_targetThemeBase.setTarget(this);
    m_targetThemeBase.setSelector(ConfigDialog_COLORS);
    m_targetThemeBorder.connect(m_themeCurrent.border);
    m_targetThemeBorder.setTarget(this);
    m_targetThemeBorder.setSelector(ConfigDialog_COLORS);
    m_targetThemeFore.connect(m_themeCurrent.fore);
    m_targetThemeFore.setTarget(this);
    m_targetThemeFore.setSelector(ConfigDialog_COLORS);
    m_targetThemeMenuback.connect(m_themeCurrent.menuback);
    m_targetThemeMenuback.setTarget(this);
    m_targetThemeMenuback.setSelector(ConfigDialog_COLORS);
    m_targetThemeMenufore.connect(m_themeCurrent.menufore);
    m_targetThemeMenufore.setTarget(this);
    m_targetThemeMenufore.setSelector(ConfigDialog_COLORS);
    m_targetThemeSelback.connect(m_themeCurrent.selback);
    m_targetThemeSelback.setTarget(this);
    m_targetThemeSelback.setSelector(ConfigDialog_COLORS);
    m_targetThemeSelfore.connect(m_themeCurrent.selfore);
    m_targetThemeSelfore.setTarget(this);
    m_targetThemeSelfore.setSelector(ConfigDialog_COLORS);
    m_targetThemeTipback.connect(m_themeCurrent.tipback);
    m_targetThemeTipback.setTarget(this);
    m_targetThemeTipback.setSelector(ConfigDialog_COLORS);
    m_targetThemeTipfore.connect(m_themeCurrent.tipfore);
    m_targetThemeTipfore.setTarget(this);
    m_targetThemeTipfore.setSelector(ConfigDialog_COLORS);
    m_targetThemeNotifyback.connect(m_themeCurrent.notifyback);
    m_targetThemeNotifyback.setTarget(this);
    m_targetThemeNotifyback.setSelector(ConfigDialog_COLORS);
    m_targetThemeNotifyfore.connect(m_themeCurrent.notifyfore);
    m_targetThemeNotifyfore.setTarget(this);
    m_targetThemeNotifyfore.setSelector(ConfigDialog_COLORS);
    m_targetTrayColor.connect(m_preferences.m_trayColor);
    m_targetUnreadColor.connect(m_preferences.m_unreadColor);
    m_targetUnreadColor.setTarget(this);
    m_targetUnreadColor.setSelector(ConfigDialog_TABCOLORS);
    m_targetHighlightColor.connect(m_preferences.m_highlightColor);
    m_targetHighlightColor.setTarget(this);
    m_targetHighlightColor.setSelector(ConfigDialog_TABCOLORS);

    //Imtext tab
    m_targetIMColorText.connect(m_preferences.m_colors.text);
    m_targetIMColorText.setTarget(this);
    m_targetIMColorText.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorBack.connect(m_preferences.m_colors.back);
    m_targetIMColorBack.setTarget(this);
    m_targetIMColorBack.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorUser.connect(m_preferences.m_colors.user);
    m_targetIMColorUser.setTarget(this);
    m_targetIMColorUser.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorAction.connect(m_preferences.m_colors.action);
    m_targetIMColorAction.setTarget(this);
    m_targetIMColorAction.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorError.connect(m_preferences.m_colors.error);
    m_targetIMColorError.setTarget(this);
    m_targetIMColorError.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorHilight.connect(m_preferences.m_colors.hilight);
    m_targetIMColorHilight.setTarget(this);
    m_targetIMColorHilight.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorLink.connect(m_preferences.m_colors.link);
    m_targetIMColorLink.setTarget(this);
    m_targetIMColorLink.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorMymsg.connect(m_preferences.m_colors.mymsg);
    m_targetIMColorMymsg.setTarget(this);
    m_targetIMColorMymsg.setSelector(ConfigDialog_IMCOLORS);
    m_targetIMColorHistory.connect(m_preferences.m_colors.history);
    m_targetIMColorHistory.setTarget(this);
    m_targetIMColorHistory.setSelector(ConfigDialog_IMCOLORS);
    m_targetSameCmd.connect(m_preferences.m_sameCmd);
    m_targetSameList.connect(m_preferences.m_sameList);
    m_targetColoredNick.connect(m_preferences.m_coloredNick);

    //Notification tab
    m_targetSound.connect(m_preferences.m_sounds);
    m_targetSound.setTarget(this);
    m_targetSound.setSelector(ConfigDialog_SOUNDS);
    m_targetSoundConnect.connect(m_preferences.m_soundConnect);
    m_targetSoundConnect.setTarget(this);
    m_targetSoundConnect.setSelector(ConfigDialog_SOUNDCONNECT);
    m_targetSoundDisconnect.connect(m_preferences.m_soundDisconnect);
    m_targetSoundDisconnect.setTarget(this);
    m_targetSoundDisconnect.setSelector(ConfigDialog_SOUNDDISCONNECT);
    m_targetSoundMessage.connect(m_preferences.m_soundMessage);
    m_targetSoundMessage.setTarget(this);
    m_targetSoundMessage.setSelector(ConfigDialog_SOUNDMESSAGE);
    m_targetPathConnect.connect(m_preferences.m_pathConnect);
    m_targetPathDisconnect.connect(m_preferences.m_pathDisconnect);
    m_targetPathMessage.connect(m_preferences.m_pathMessage);
    m_targetNotify.connect(m_preferences.m_notify);
    m_targetNotify.setTarget(this);
    m_targetNotify.setSelector(ConfigDialog_NOTIFY);
    m_targetNotifyAccountConnect.connect(m_preferences.m_notifyAccountConnect);
    m_targetNotifyAccountDisconnect.connect(m_preferences.m_notifyAccountDisconnect);
    m_targetNotifyBuddyConnect.connect(m_preferences.m_notifyBuddyConnect);
    m_targetNotifyBuddyDisconnect.connect(m_preferences.m_notifyBuddyDisconnect);
    m_targetNotifyMessage.connect(m_preferences.m_notifyMessage);
    m_targetNotifyFiletransfer.connect(m_preferences.m_notifyFiletransfer);
    m_targetRaiseWindow.connect(m_preferences.m_raiseWindow);
    m_targetRaiseWindow.setTarget(this);
    m_targetRaiseWindow.setSelector(ConfigDialog_RAISEWINDOW);
    m_targetNotifyMessageTray.connect(m_preferences.m_notifyMessageTray);

    //Smileys tab
    m_targetUseSmileys.connect(m_preferences.m_useSmileys);
    m_targetUseSmileys.setTarget(this);
    m_targetUseSmileys.setSelector(ConfigDialog_USESMILEYS);

    getApp()->getNormalFont()->create();
    FXFontDesc fontdescription;
    getApp()->getNormalFont()->getFontDesc(fontdescription);
    m_font = new FXFont(getApp(),fontdescription);
    m_font->create();

    FXHorizontalFrame *closeframe = new FXHorizontalFrame(this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
    dxEXButton *ok = new dxEXButton(closeframe, _("&Save&&Close"), NULL, this, ConfigDialog_ACCEPT, BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK, 0,0,0,0, 10,10,2,2);
    ok->addHotKey(KEY_Return);
    new dxEXButton(closeframe, _("&Cancel"), NULL, this, ConfigDialog_CANCEL, LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK, 0,0,0,0, 10,10,2,2);

    FXHorizontalFrame *contents = new FXHorizontalFrame(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabbook = new dxTabBook(contents, this, ConfigDialog_TABS, PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabbook->setTabStyle(TABBOOK_LEFTTABS);

    createGeneralTab(m_tabbook);
    createIconsTab(m_tabbook);
    createLookTab(m_tabbook);
    createImtextTab(m_tabbook);
    createNotificationTab(m_tabbook);
    createSmileysTab(m_tabbook);
    createKeybindingsTab(m_tabbook);

    updateColors();
    updateFont();
    updateImFont();
}


ConfigDialog::~ConfigDialog()
{
    delete m_font;
    delete m_imFont;
}

void ConfigDialog::createImtextTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("&IM Text"), NULL, TAB_LEFT);
    FXVerticalFrame *colorpane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *hframe = new FXHorizontalFrame(colorpane, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXVerticalFrame *cframe = new FXVerticalFrame(hframe, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXMatrix *colormatrix = new FXMatrix(cframe, 2, MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorBack, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Text backround color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorText, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Text color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorUser, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("User events text color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorAction, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Message in 3rd person text color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorError, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Error text color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorHilight, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Highlight message text color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorLink, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("Link color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorMymsg, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("My message color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(colormatrix, FXRGB(0,0,255), &m_targetIMColorHistory, FXDataTarget::ID_VALUE, COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW, 0,0,40,24);
    new FXLabel(colormatrix, _("History line color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXLabel(colormatrix, _("Font"));
    m_imfontButton = new dxEXButton(colormatrix, " ", NULL, this, ConfigDialog_IMFONT, LAYOUT_CENTER_Y|FRAME_RAISED|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_X, 0,0,0,0, 10,10,2,2);
    new FXCheckButton(cframe, _("Use same font for commandline"), &m_targetSameCmd, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    new FXCheckButton(cframe, _("Use same font for user list"), &m_targetSameList, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXHorizontalFrame *nickpane = new FXHorizontalFrame(cframe, LAYOUT_FILL_X);
    new FXLabel(nickpane, _("Nick completion char"), NULL, LAYOUT_LEFT);
    m_nickCharField = new FXTextField(nickpane, 1, this, ConfigDialog_NICK, TEXTFIELD_LIMITED|FRAME_THICK|FRAME_SUNKEN);
    m_nickCharField->setText(m_preferences.m_nickCompletionChar);
    new FXCheckButton(cframe, _("Use colored nick"), &m_targetColoredNick, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXVerticalFrame *tframe = new FXVerticalFrame(hframe, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_text = new FXText(tframe, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY);
    m_text->setScrollStyle(HSCROLLING_OFF);
    for(int i=0; i<7; i++)
    {
        m_textStyle[i].normalForeColor = m_preferences.m_colors.text;
        m_textStyle[i].normalBackColor = m_preferences.m_colors.back;
        m_textStyle[i].selectForeColor = getApp()->getSelforeColor();
        m_textStyle[i].selectBackColor = getApp()->getSelbackColor();
        m_textStyle[i].hiliteForeColor = getApp()->getHiliteColor();
        m_textStyle[i].hiliteBackColor = FXRGB(255, 128, 128); // from FXText.cpp
        m_textStyle[i].activeBackColor = m_preferences.m_colors.back;
        m_textStyle[i].style = 0;
    }
    //user commands
    m_textStyle[0].normalForeColor = m_preferences.m_colors.user;
    //Actions
    m_textStyle[1].normalForeColor = m_preferences.m_colors.action;
    //Errors
    m_textStyle[2].normalForeColor = m_preferences.m_colors.error;
    //Highlight
    m_textStyle[3].normalForeColor = m_preferences.m_colors.hilight;
    //link
    m_textStyle[4].normalForeColor = m_preferences.m_colors.link;
    m_textStyle[4].style = FXText::STYLE_UNDERLINE;
    //Mymsg
    m_textStyle[5].normalForeColor = m_preferences.m_colors.mymsg;
    //history msg
    m_textStyle[6].normalForeColor = m_preferences.m_colors.history;

    m_text->setStyled(TRUE);
    m_text->setHiliteStyles(m_textStyle);
    m_text->setTextColor(m_preferences.m_colors.text);
    m_text->setBackColor(m_preferences.m_colors.back);
    m_text->appendText("[1.1.2000 00:00:00] ");
    m_text->appendStyledText(FXString("dvx: msg from history\n"), 7);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("dvx has joined to dxim\n"), 1);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("dvx: Welcome in dxim\n"), 6);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("bm: dvx, dxim is nice\n"), 4);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("dvx is online\n"), 2);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("No error\n"), 3);
    m_text->appendText("[00:00:00] ");
    m_text->appendStyledText(FXString("dvx: "), 6);
    m_text->appendStyledText(FXString("http://dxsolutions.org"), 5);
    m_text->appendText(" \n");
}

void ConfigDialog::createGeneralTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("&General"), NULL, TAB_LEFT);
    FXVerticalFrame *otherpane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXCheckButton(otherpane, _("Run one instance"), &m_targetOneInstance, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXHorizontalFrame *awpane = new FXHorizontalFrame(otherpane, LAYOUT_FILL_X);
    new FXCheckButton(awpane, _("Time for status \"Away\" in minutes"), &m_targetUseAway, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL);
    m_awaySpinner = new FXSpinner(awpane, 4, &m_targetAwayTimeout, FXDataTarget::ID_VALUE, SPIN_CYCLIC|FRAME_GROOVE);
    m_awaySpinner->setRange(1,500);
#ifdef HAVE_TRAY
    new FXCheckButton(otherpane, _("Use trayicon"), &m_targetTray, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    m_closeToTrayButton = new FXCheckButton(otherpane, _("Close button hides application"), &m_targetCloseToTray, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    m_minToTrayButton = new FXCheckButton(otherpane, _("Start application minimized to tray"), &m_targetMinToTray, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_useTray)
    {
        m_closeToTrayButton->disable();
        m_minToTrayButton->disable();
    }
#endif
    new FXCheckButton(otherpane, _("Logging chats"), &m_targetLogging, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXHorizontalFrame *logpane = new FXHorizontalFrame(otherpane, LAYOUT_FILL_X);
    new FXLabel(logpane, _("Log path"), NULL, LAYOUT_LEFT);
    (new FXTextField(logpane, 25, &m_targetLogPath, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X))->disable();
    if(!FXStat::exists(m_preferences.m_logPath)) m_preferences.m_logPath = FXSystem::getHomeDirectory();
    m_selectPath = new dxEXButton(logpane, "...", NULL, this, ConfigDialog_LOGPATH, FRAME_RAISED|FRAME_THICK);
    if(m_preferences.m_logging) m_selectPath->enable();
    else m_selectPath->disable();
    m_logLoadCurrent = new FXCheckButton(otherpane, _("Load current log for opened IM"), &m_targetLogLoadCurrent, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(m_preferences.m_logging) m_logLoadCurrent->enable();
    else
    {
        m_preferences.m_logLoadCurrent = false;
        m_logLoadCurrent->disable();
    }
    new FXCheckButton(otherpane, _("Show accounts and groups in friends list"), &m_targetShowRosterTree, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    new FXCheckButton(otherpane, _("Use delivery receipts mark on XMPP protocol"), &m_targetReceiptsMark, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
}

void ConfigDialog::createIconsTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("I&cons"), NULL, TAB_LEFT);
    FXVerticalFrame *iconspane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXGroupBox *statusgroup = new FXGroupBox(iconspane, _("Status icons themes"), FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_SIDE_TOP);
    FXVerticalFrame *statusbuttons = new FXVerticalFrame(statusgroup, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    m_addStatusTheme = new dxEXButton(statusbuttons, _("Add"), NULL, this, ConfigDialog_ADDSTATUSICONS, FRAME_RAISED|FRAME_THICK);
    m_deleteStatusTheme = new dxEXButton(statusbuttons, _("Delete"), NULL, this, ConfigDialog_DELETESTATUSICONS, FRAME_RAISED|FRAME_THICK);
    FXVerticalFrame *statuslist = new FXVerticalFrame(statusgroup, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK);
    m_statusIcons = new FXList(statuslist, this, ConfigDialog_STATUSICONS, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    fillStatusIcons();
    for(FXint i=0; i<m_statusIcons->getNumItems(); i++)
    {
        if(m_statusIcons->getItemText(i) == m_preferences.m_themeStatusPath)
        {
            m_statusIcons->setCurrentItem(i);
            break;
        }
    }
    m_statusIcons->getNumItems()>1 ? m_deleteStatusTheme->enable() : m_deleteStatusTheme->disable();
    m_iconsStatusBar = new FXToolBar(statuslist, LAYOUT_FILL_X);
    m_iconAway = new dxEXButton(m_iconsStatusBar, _("\tAway"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "away.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconIdle = new dxEXButton(m_iconsStatusBar, _("\tIdle"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "idle.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconDnd = new dxEXButton(m_iconsStatusBar, _("\tBusy"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "dnd.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconInvisible = new dxEXButton(m_iconsStatusBar, _("\tInvisible"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "invisible.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconOffline = new dxEXButton(m_iconsStatusBar, _("\tOffline"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "offline.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconOnline = new dxEXButton(m_iconsStatusBar, _("\tOnline"), makeIcon(getApp(), m_preferences.m_themeStatusPath, "online.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    FXGroupBox *rolegroup = new FXGroupBox(iconspane, _("Role icons themes"), FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_SIDE_TOP);
    FXVerticalFrame *rolebuttons = new FXVerticalFrame(rolegroup, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    m_addRoleTheme = new dxEXButton(rolebuttons, _("Add"), NULL, this, ConfigDialog_ADDROLEICONS, FRAME_RAISED|FRAME_THICK);
    m_deleteRoleTheme = new dxEXButton(rolebuttons, _("Delete"), NULL, this, ConfigDialog_DELETEROLEICONS, FRAME_RAISED|FRAME_THICK);
    FXVerticalFrame *rolelist = new FXVerticalFrame(rolegroup, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK);
    m_roleIcons = new FXList(rolelist, this, ConfigDialog_ROLEICONS, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    fillRoleIcons();
    for(FXint i=0; i<m_roleIcons->getNumItems(); i++)
    {
        if(m_roleIcons->getItemText(i) == m_preferences.m_themeRolePath)
        {
            m_roleIcons->setCurrentItem(i);
            break;
        }
    }
    m_roleIcons->getNumItems()>1 ? m_deleteRoleTheme->enable() : m_deleteRoleTheme->disable();
    m_iconsRoleBar = new FXToolBar(rolelist, LAYOUT_FILL_X);
    m_iconFounder = new dxEXButton(m_iconsRoleBar, _("\tFounder"), makeIcon(getApp(), m_preferences.m_themeRolePath, "founder.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconHalfop = new dxEXButton(m_iconsRoleBar, _("\tHalfop"), makeIcon(getApp(), m_preferences.m_themeRolePath, "halfop.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconNormal = new dxEXButton(m_iconsRoleBar, _("\tNormal"), makeIcon(getApp(), m_preferences.m_themeRolePath, "normal.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconOp = new dxEXButton(m_iconsRoleBar, _("\tOp"), makeIcon(getApp(), m_preferences.m_themeRolePath, "op.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
    m_iconVoice = new dxEXButton(m_iconsRoleBar, _("\tVoice"), makeIcon(getApp(), m_preferences.m_themeRolePath, "voice.png", TRUE), NULL, 0, BUTTON_TOOLBAR);
}

void ConfigDialog::createKeybindingsTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("Key&bindings"), NULL, TAB_LEFT);
    FXVerticalFrame *kbpane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXPacker *kbpack = new FXPacker(kbpane, LAYOUT_FILL_X);
    new dxEXButton(kbpack, _("Modify key bindings..."), ICO_KEYBINDINGS, this, ConfigDialog_MODIFYBINDINGS, FRAME_RAISED|FRAME_THICK|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
    new dxEXButton(kbpack, _("Restore default key bindings..."), ICO_REFRESH, this, ConfigDialog_REFRESHBINDINGS, FRAME_RAISED|FRAME_THICK|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
}

void ConfigDialog::createSmileysTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("S&mileys"), NULL, TAB_LEFT);
    FXVerticalFrame *smileypane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXCheckButton(smileypane, _("Use smileys"), &m_targetUseSmileys, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXGroupBox *smileysgroup = new FXGroupBox(smileypane, _("Smileys"), FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXVerticalFrame *smileysbuttons = new FXVerticalFrame(smileysgroup, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH);
    m_addSmiley = new dxEXButton(smileysbuttons, _("Add"), NULL, this, ConfigDialog_ADDSMILEY, FRAME_RAISED|FRAME_THICK);
    m_modifySmiley = new dxEXButton(smileysbuttons, _("Modify"), NULL, this, ConfigDialog_MODIFYSMILEY, FRAME_RAISED|FRAME_THICK);
    m_deleteSmiley = new dxEXButton(smileysbuttons, _("Delete"), NULL, this, ConfigDialog_DELETESMILEY, FRAME_RAISED|FRAME_THICK);
    m_importSmiley = new dxEXButton(smileysbuttons, _("Import"), NULL, this, ConfigDialog_IMPORTSMILEY, FRAME_RAISED|FRAME_THICK);
    m_exportSmiley = new dxEXButton(smileysbuttons, _("Export"), NULL, this, ConfigDialog_EXPORTSMILEY, FRAME_RAISED|FRAME_THICK);
    if(m_preferences.m_useSmileys)
    {
        m_addSmiley->enable();
        m_importSmiley->enable();
        if((FXint)m_preferences.m_smileysMap.size())
        {
            m_modifySmiley->enable();
            m_deleteSmiley->enable();
            m_exportSmiley->enable();
        }
        else
        {
            m_modifySmiley->disable();
            m_deleteSmiley->disable();
            m_exportSmiley->disable();
        }
    }
    else
    {
        m_addSmiley->disable();
        m_modifySmiley->disable();
        m_deleteSmiley->disable();
        m_importSmiley->disable();
        m_exportSmiley->disable();
    }
    FXVerticalFrame *smileyspane = new FXVerticalFrame(smileysgroup, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK);
    FXHorizontalFrame *smileysframe = new FXHorizontalFrame(smileyspane, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_smileys = new FXList(smileysframe, this, ConfigDialog_SMILEY, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    fillSmileys();
}

void ConfigDialog::createNotificationTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("N&otification"), NULL, TAB_LEFT);
    FXVerticalFrame *soundpane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXCheckButton(soundpane, _("Use sounds"), &m_targetSound, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    FXGroupBox *eventsgroup = new FXGroupBox(soundpane, _("Events"), FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_TOP);
    FXVerticalFrame *eventsframe = new FXVerticalFrame(eventsgroup, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_checkConnect = new FXCheckButton(eventsframe, _("Friend connected"), &m_targetSoundConnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_sounds)
        m_checkConnect->disable();
    FXHorizontalFrame *connectframe = new FXHorizontalFrame(eventsframe, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT);
    (new FXTextField(connectframe, 30, &m_targetPathConnect, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X))->disable();
    m_selectConnect = new dxEXButton(connectframe, "...", NULL, this, ConfigDialog_SELECTCONNECT, FRAME_RAISED|FRAME_THICK);
    m_playConnect = new dxEXButton(connectframe, _("\tPlay"), ICO_PLAY, this, ConfigDialog_PLAYCONNECT, FRAME_RAISED|FRAME_THICK);
    if(!m_preferences.m_soundConnect || !m_preferences.m_sounds)
    {
        m_selectConnect->disable();
        m_playConnect->disable();
    }
    if(!FXStat::exists(m_preferences.m_pathConnect))
        m_playConnect->disable();
    m_checkDisconnect = new FXCheckButton(eventsframe, _("Friend disconnected"), &m_targetSoundDisconnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_sounds)
        m_checkDisconnect->disable();
    FXHorizontalFrame *disconnectframe = new FXHorizontalFrame(eventsframe, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT);
    (new FXTextField(disconnectframe, 30, &m_targetPathDisconnect, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X))->disable();
    m_selectDisconnect = new dxEXButton(disconnectframe, "...", NULL, this, ConfigDialog_SELECTDISCONNECT, FRAME_RAISED|FRAME_THICK);
    m_playDisconnect = new dxEXButton(disconnectframe, _("\tPlay"), ICO_PLAY, this, ConfigDialog_PLAYDISCONNECT, FRAME_RAISED|FRAME_THICK);
    if(!m_preferences.m_soundDisconnect || !m_preferences.m_sounds)
    {
        m_selectDisconnect->disable();
        m_playDisconnect->disable();
    }
    if(!FXStat::exists(m_preferences.m_pathDisconnect))
        m_playDisconnect->disable();
    m_checkMessage = new FXCheckButton(eventsframe, _("Highlighted or im message"), &m_targetSoundMessage, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_sounds)
        m_checkMessage->disable();
    FXHorizontalFrame *messageframe = new FXHorizontalFrame(eventsframe, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT);
    (new FXTextField(messageframe, 30, &m_targetPathMessage, FXDataTarget::ID_VALUE, TEXTFIELD_READONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X))->disable();
    m_selectMessage = new dxEXButton(messageframe, "...", NULL, this, ConfigDialog_SELECTMESSAGE, FRAME_RAISED|FRAME_THICK);
    m_playMessage = new dxEXButton(messageframe, _("\tPlay"), ICO_PLAY, this, ConfigDialog_PLAYMESSAGE, FRAME_RAISED|FRAME_THICK);
    if(!m_preferences.m_soundMessage || !m_preferences.m_sounds)
    {
        m_selectMessage->disable();
        m_playMessage->disable();
    }
    if(!FXStat::exists(m_preferences.m_pathMessage))
        m_playMessage->disable();
    FXHorizontalFrame *notifyframe = new FXHorizontalFrame(soundpane, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT);
    new FXCheckButton(notifyframe, _("Use notify popup"), &m_targetNotify, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    new dxEXButton(notifyframe, _("Show"), NULL, this, ConfigDialog_SHOWNOTIFY, FRAME_RAISED|FRAME_THICK|LAYOUT_SIDE_LEFT);
    if(!dxutils::instance().haveNotify())
    {
        FXHorizontalFrame *notifyposframe = new FXHorizontalFrame(soundpane, LAYOUT_FILL_X|PACK_UNIFORM_HEIGHT);
        new FXLabel(notifyposframe, _("Notify position"), NULL, LAYOUT_LEFT);
        m_listNotifyPos = new FXListBox(notifyposframe, this, ConfigDialog_NOTIFYPOS);
        m_listNotifyPos->appendItem(_("Left Top"));
        m_listNotifyPos->appendItem(_("Right Top"));
        m_listNotifyPos->appendItem(_("Left Bottom"));
        m_listNotifyPos->appendItem(_("Right Bottom"));
        m_listNotifyPos->setNumVisible(4);
        if(m_preferences.m_notifyPosition>=0 && m_preferences.m_notifyPosition<4) m_listNotifyPos->setCurrentItem(m_preferences.m_notifyPosition);
    }
    FXGroupBox *neventsgroup = new FXGroupBox(soundpane, _("Events"), FRAME_GROOVE|LAYOUT_FILL_X);
    FXVerticalFrame *neventsframe = new FXVerticalFrame(neventsgroup, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_checkNotifyAccountConnect = new FXCheckButton(neventsframe, _("Account connected"), &m_targetNotifyAccountConnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyAccountConnect->disable();
    m_checkNotifyAccountDisconnect = new FXCheckButton(neventsframe, _("Account disconnected"), &m_targetNotifyAccountDisconnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyAccountDisconnect->disable();
    m_checkNotifyBuddyConnect = new FXCheckButton(neventsframe, _("Friend connected"), &m_targetNotifyBuddyConnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyBuddyConnect->disable();
    m_checkNotifyBuddyDisconnect = new FXCheckButton(neventsframe, _("Friend disconnected"), &m_targetNotifyBuddyDisconnect, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyBuddyDisconnect->disable();
    m_checkNotifyMessage = new FXCheckButton(neventsframe, _("Highlighted or im message"), &m_targetNotifyMessage, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyMessage->disable();
    m_checkNotifyFiletransfer = new FXCheckButton(neventsframe, _("File transfer finished or canceled"), &m_targetNotifyFiletransfer, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    if(!m_preferences.m_notify)
        m_checkNotifyFiletransfer->disable();
#ifdef HAVE_TRAY
    new FXCheckButton(soundpane, _("Show application window on highlighted or im message"), &m_targetRaiseWindow, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
    m_checkNotifyMessageTray = new FXCheckButton(soundpane, _("Change tray icon for all messages if application in tray"), &m_targetNotifyMessageTray, FXDataTarget::ID_VALUE, CHECKBUTTON_NORMAL|LAYOUT_FILL_X|LAYOUT_SIDE_LEFT|JUSTIFY_LEFT);
#endif
}

void ConfigDialog::createLookTab(dxTabBook* tabbook)
{
    new dxEXTabItem(tabbook, _("&Look"), NULL, TAB_LEFT);
    FXVerticalFrame *lookpane = new FXVerticalFrame(tabbook, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXHorizontalFrame *hframe1 = new FXHorizontalFrame(lookpane, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    FXVerticalFrame *vframe1 = new FXVerticalFrame(hframe1, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_vframe2 = new FXVerticalFrame(hframe1, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_RAISED);
    new FXLabel(vframe1, _("Theme:"), NULL, LAYOUT_CENTER_Y);
    m_themes = new FXListBox(vframe1,this,ConfigDialog_THEME,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK);
    m_themes->setNumVisible(9);
    fillThemes();
    new FXSeparator(vframe1, SEPARATOR_GROOVE|LAYOUT_FILL_X);
    FXMatrix *themeMatrix = new FXMatrix(vframe1, 2, LAYOUT_FILL_Y|MATRIX_BY_COLUMNS, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING, 1,1);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeBase, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Base Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeBorder, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Border Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeFore,FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Text Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeBack, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Background Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeSelfore, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Selected Text Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeSelback, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Selected Background Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeMenufore, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Selected Menu Text Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeMenuback, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Selected Menu Background Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeTipfore, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Tip Text Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeTipback, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Tip Background Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    if(!dxutils::instance().haveNotify())
    {
        new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeNotifyfore, FXDataTarget::ID_VALUE);
        new FXLabel(themeMatrix, _("Notify Text Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
        new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetThemeNotifyback, FXDataTarget::ID_VALUE);
        new FXLabel(themeMatrix, _("Notify Background Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    }
#ifdef HAVE_TRAY
#ifndef _WIN32
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetTrayColor, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Tray Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
#endif
#endif
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetUnreadColor, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Unreaded Tab Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    new FXColorWell(themeMatrix, FXRGB(0,0,255), &m_targetHighlightColor, FXDataTarget::ID_VALUE);
    new FXLabel(themeMatrix, _("Highlighted Tab Color"), NULL, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
    FXHorizontalFrame *tabpane = new FXHorizontalFrame(vframe1, LAYOUT_FILL_X);
    new FXLabel(tabpane, _("Tab position"), NULL, LAYOUT_LEFT);
    m_listTabs = new FXListBox(tabpane, this, ConfigDialog_TABPOS);
    m_listTabs->appendItem(_("Bottom"));
    m_listTabs->appendItem(_("Left"));
    m_listTabs->appendItem(_("Top"));
    m_listTabs->appendItem(_("Right"));
    m_listTabs->setNumVisible(4);
    if(m_preferences.m_tabPosition>=0 && m_preferences.m_tabPosition<4) m_listTabs->setCurrentItem(m_preferences.m_tabPosition);
    m_label = new FXLabel(m_vframe2, "Label");
    m_textFrame1 = new FXHorizontalFrame(m_vframe2, LAYOUT_FILL_X);
    m_textTest = new FXTextField(m_textFrame1, 30, NULL, 0, LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN);
    m_textTest->setText(_("Select this text, to see the selected colors"));
    m_textFrame2 = new FXHorizontalFrame(m_vframe2, LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN, 0,0,0,0,2,2,2,2,0,0);
    m_labelSelected = new FXLabel(m_textFrame2, _("Selected Text (with focus)"), NULL, LAYOUT_FILL_X, 0,0,0,0,1,1,1,1);
    m_textFrame3 = new FXHorizontalFrame(m_vframe2, LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN, 0,0,0,0,2,2,2,2,0,0);
    m_labelNocurrent = new FXLabel(m_textFrame3, _("Selected Text (no focus)"), NULL, LAYOUT_FILL_X, 0,0,0,0,1,1,1,1);
    m_sep1 = new FXSeparator(m_vframe2, LAYOUT_FILL_X|SEPARATOR_LINE);
    m_labelTip = new FXLabel(m_vframe2, _("Tooltip example"), NULL, FRAME_LINE|LAYOUT_CENTER_X);
    m_menuGroup = new FXGroupBox(m_vframe2, _("Menu example"), FRAME_GROOVE|LAYOUT_FILL_Y|LAYOUT_FILL_X);
    m_menuFrame = new FXVerticalFrame(m_menuGroup, FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_X|LAYOUT_CENTER_Y, 0,0,0,0,0,0,0,0,0,0);
    m_menuLabels[0]=new FXLabel(m_menuFrame, _("&Server list"), NULL, LABEL_NORMAL, 0,0,0,0,16,4);
    m_menuLabels[1]=new FXLabel(m_menuFrame, _("Selected Menu Entry"), NULL, LABEL_NORMAL, 0,0,0,0,16,4);
    m_sep2 = new FXSeparator(m_menuFrame, LAYOUT_FILL_X|SEPARATOR_LINE);
    m_menuLabels[2]=new FXLabel(m_menuFrame, _("&Quit"), NULL, LABEL_NORMAL, 0,0,0,0,16,4);
    if(!dxutils::instance().haveNotify())
        m_labelNotify = new FXLabel(m_vframe2, _("Notify example"), ICO_BIGICON, FRAME_LINE|LAYOUT_CENTER_X|ICON_BEFORE_TEXT);
    m_tabFrame = new FXHorizontalFrame(m_vframe2, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabs = new dxTabBook(m_tabFrame, NULL, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tab = new dxEXTabItem(m_tabs, "linux", ICO_CHAT);
    m_tabframe1 = new FXVerticalFrame(m_tabs, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabLabel1 = new FXLabel(m_tabframe1, _("Standard tab"));
    m_unread = new dxEXTabItem(m_tabs, "bsd", ICO_CHAT);
    m_tabframe2 = new FXVerticalFrame(m_tabs, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabLabel2 = new FXLabel(m_tabframe2, _("Unreaded tab"));
    m_highlight = new dxEXTabItem(m_tabs, "hurd", ICO_CHATNEWMSG);
    m_tabframe3 = new FXVerticalFrame(m_tabs, FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    m_tabLabel3 = new FXLabel(m_tabframe3, _("Highlighted tab"));
    updateTabPosition();
    FXHorizontalFrame *fontframe = new FXHorizontalFrame(lookpane, LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING,DEFAULT_SPACING);
    new FXLabel(fontframe, _("Font"));
    m_fontButton = new dxEXButton(fontframe, " ", NULL, this, ConfigDialog_FONT, LAYOUT_CENTER_Y|FRAME_RAISED|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_X);
}

long ConfigDialog::onColor(FXObject*, FXSelector, void*)
{
    m_text->setTextColor(m_preferences.m_colors.text);
    m_text->setBackColor(m_preferences.m_colors.back);
    for(int i=0; i<7; i++)
    {
        m_textStyle[i].normalBackColor = m_preferences.m_colors.back;
    }
    m_textStyle[0].normalForeColor = m_preferences.m_colors.user;
    m_textStyle[1].normalForeColor = m_preferences.m_colors.action;
    m_textStyle[2].normalForeColor = m_preferences.m_colors.error;
    m_textStyle[3].normalForeColor = m_preferences.m_colors.hilight;
    m_textStyle[4].normalForeColor = m_preferences.m_colors.link;
    m_textStyle[5].normalForeColor = m_preferences.m_colors.mymsg;
    m_textStyle[6].normalForeColor = m_preferences.m_colors.history;
    m_text->update();
    return 1;
}

long ConfigDialog::onTabPosition(FX::FXObject *, FX::FXSelector, void *ptr)
{
    m_preferences.m_tabPosition = (FXint)(FXival)ptr;
    updateTabPosition();
    return 1;
}

void ConfigDialog::updateTabPosition()
{
    switch(m_preferences.m_tabPosition) {
        case 0: //bottom
            {
                m_tabs->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabs->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabs->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabs->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabs->setPackingHints(packing);
            }break;
        case 1: //left
            {
                m_tabs->setTabStyle(TABBOOK_LEFTTABS);
                for(FXint i = 0; i<m_tabs->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabs->childAtIndex(i))->setTabOrientation(TAB_LEFT);
                }
                FXuint packing = m_tabs->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabs->setPackingHints(packing);
            }break;
        case 2: //top
            {
                m_tabs->setTabStyle(TABBOOK_TOPTABS);
                for(FXint i = 0; i<m_tabs->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabs->childAtIndex(i))->setTabOrientation(TAB_TOP);
                }
                FXuint packing = m_tabs->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabs->setPackingHints(packing);
            }break;
        case 3: //right
            {
                m_tabs->setTabStyle(TABBOOK_RIGHTTABS);
                for(FXint i = 0; i<m_tabs->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabs->childAtIndex(i))->setTabOrientation(TAB_RIGHT);
                }
                FXuint packing = m_tabs->getPackingHints();
                packing |= PACK_UNIFORM_WIDTH;
                m_tabs->setPackingHints(packing);
            }break;
        default:
            {
                m_tabs->setTabStyle(TABBOOK_BOTTOMTABS);
                for(FXint i = 0; i<m_tabs->numChildren(); i+=2)
                {
                    static_cast<FXTabItem*>(m_tabs->childAtIndex(i))->setTabOrientation(TAB_BOTTOM);
                }
                FXuint packing = m_tabs->getPackingHints();
                packing &= ~PACK_UNIFORM_WIDTH;
                m_tabs->setPackingHints(packing);
            }
    }
}

long ConfigDialog::onNotifyPosition(FX::FXObject *, FX::FXSelector, void *ptr)
{
    m_preferences.m_notifyPosition = (FXint)(FXival)ptr;
    return 1;
}

long ConfigDialog::onAccept(FXObject*, FXSelector, void*)
{
    getApp()->stopModal(this,TRUE);
    m_preferences.m_appTheme = m_themeCurrent;
    m_preferences.m_font = m_font->getFont();
    m_preferences.m_imFont = m_imFont->getFont();
    DXPREFERENCES = m_preferences;
    DXPREFERENCES.save();
    hide();
    return 1;
}

long ConfigDialog::onCancel(FXObject*,FXSelector,void*)
{
    getApp()->stopModal(this,FALSE);
    hide();
    return 1;
}

long ConfigDialog::onKeyPress(FXObject *sender,FXSelector sel,void *ptr)
{
    if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
    if(((FXEvent*)ptr)->code == KEY_Escape)
    {
        getApp()->stopModal(this,FALSE);
        hide();
        return 1;
    }
    return 0;
}

long ConfigDialog::onImportSmiley(FXObject*, FXSelector, void*)
{
    if(m_showImportwarning && m_preferences.m_smileysMap.size())
        if(dxEXMessageBox::warning(this, MBOX_OK_CANCEL, _("Warning"), _("File import overwrites current settings"))==4) {m_showImportwarning=FALSE; return 1;}
    dxEXFileDialog dialog(this, _("Select file"));
    if(m_showImportwarning) dialog.setFilename((FXString)DXIM_DATADIR+PATHSEPSTRING+"icons"+PATHSEPSTRING+"smileys"+PATHSEPSTRING+"dxim.scfg");
    dialog.setPattern("*.scfg");
    if(dialog.execute())
    {
        m_smileys->clearItems();
        m_preferences.m_smileysMap.clear();
        std::ifstream file(dialog.getFilename().text());
        if(file.is_open())
        {
            std::string buf;
            while(std::getline(file, buf))
            {
                if(buf[0] != '#')
                {
                    FXString smiley, path;
                    size_t pos = buf.find_first_of('=');
                    if(pos != std::string::npos)
                    {
                        path = FXPath::absolute(FXPath::directory(dialog.getFilename()), dxutils::fromStdString(buf.substr(0, pos)));
                        smiley = dxutils::fromStdString(buf.substr(pos+1));
                        if(!path.empty() && !smiley.empty() && FXStat::exists(path))
                            m_preferences.m_smileysMap.insert(dxStringPair(smiley,path));
                    }
                }
            }
            file.close();
        }
        fillSmileys();
        if(m_preferences.m_smileysMap.size())
        {
            m_modifySmiley->enable();
            m_deleteSmiley->enable();
            m_exportSmiley->enable();
        }
    }
    m_showImportwarning=FALSE;
    return 1;
}

long ConfigDialog::onExportSmiley(FXObject*, FXSelector, void*)
{
    dxEXFileDialog dialog(this, _("Save smiley settings as"));
    dialog.setPattern("*.scfg");
    if(dialog.execute())
    {
        FXString filename = dialog.getFilename();
        if(FXPath::extension(filename)!="scfg")
        {
            filename = FXPath::stripExtension(filename);
            filename += ".scfg";
        }
        std::ofstream file(filename.text(), std::ios::out|std::ios::app);
        if(file.is_open())
        {
            file << HEADER;
            dxStringIt it = m_preferences.m_smileysMap.begin();
            for(; it!=m_preferences.m_smileysMap.end(); it++)
            {
                file << FXPath::relative(FXPath::directory(dialog.getFilename()),(*it).second).text()
                    << "=" << (*it).first.text() << std::endl;
            }
        }
    }
    return 1;
}

//useaway changed
long ConfigDialog::onUseAway(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_useAway)
    {
        m_awaySpinner->enable();
    }
    else
    {
        m_awaySpinner->disable();
    }
    return 1;
}

long ConfigDialog::onMouseWheel(FXObject *, FXSelector, void *ptr)
{
    FXEvent *event = (FXEvent*)ptr;
    FXint index = m_tabbook->getCurrent();
    if(event->code > 0) //positive movement
    {
        if(m_tabbook->numChildren())
        {
            if((index-1) >= 0) m_tabbook->setCurrent(index-1, TRUE);
            else m_tabbook->setCurrent(m_tabbook->numChildren()/2-1, TRUE);
        }
    }
    else
    {
        if(m_tabbook->numChildren())
        {
            if((index+1)*2 < m_tabbook->numChildren()) m_tabbook->setCurrent(index+1, m_tabbook->numChildren() > index*2 ? TRUE : FALSE);
            else m_tabbook->setCurrent(0, TRUE);
        }
    }
    return 1;
}

long ConfigDialog::onModifyBindings(FXObject*, FXSelector, void*)
{
    KeybindingsDialog dialog(this);
    dialog.execute();
    return 1;
}

long ConfigDialog::onRefreshBindings(FXObject*, FXSelector, void*)
{
    dxEXMessageBox box(this, _("Restore default key bindings"), _("Do you really want to restore the default key bindings?\nAll your customizations will be lost!"), ICO_KEYBINDINGS, MBOX_OK_CANCEL|DECOR_TITLE|DECOR_BORDER);
    if(box.execute() != MBOX_CLICKED_OK)
        return 0;
    dxim::instance()->setTempKeybindings(DXPREFERENCES.getDefaultKeybindings());
    if(dxEXMessageBox::question(this, MBOX_YES_NO, _("Restart"), _("Key bindings will be changed after restart.\nRestart dxim now?"))==1)
    {
        dxim::instance()->handle(this, FXSEL(SEL_COMMAND, dxim_RESTART), NULL);
    }
    return 1;
}

long ConfigDialog::onAddStatusIcons(FXObject*, FXSelector, void*)
{
    dxEXDirDialog dirdialog(this, _("Select theme directory"));
    if(dirdialog.execute(PLACEMENT_CURSOR))
    {
        if(!FXPath::search(dirdialog.getDirectory(), "online.png").empty() && !themeStatusExist(dirdialog.getDirectory())) m_statusIcons->appendItem(dirdialog.getDirectory());
    }
    m_statusIcons->getNumItems()>1 ? m_deleteStatusTheme->enable() : m_deleteStatusTheme->disable();
    updateStatusIcons();
    return 1;
}

long ConfigDialog::onDeleteStatusIcons(FXObject*, FXSelector, void*)
{
    FXint i = m_statusIcons->getCurrentItem();
    m_statusIcons->removeItem(i);
    i = m_statusIcons->getCurrentItem();
    m_preferences.m_themeStatusPath = m_statusIcons->getItemText(i);
    m_iconAway->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "away.png", TRUE));
    m_iconIdle->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "idle.png", TRUE));
    m_iconDnd->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "dnd.png", TRUE));
    m_iconInvisible->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "invisible.png", TRUE));
    m_iconOffline->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "offline.png", TRUE));
    m_iconOnline->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "online.png", TRUE));
    m_statusIcons->getNumItems()>1 ? m_deleteStatusTheme->enable() : m_deleteStatusTheme->disable();
    updateStatusIcons();
    return 1;
}

long ConfigDialog::onStatusIconsChanged(FXObject*, FXSelector, void *ptr)
{
    FXint i = (FXint)(FXival)ptr;
    m_preferences.m_themeStatusPath = m_statusIcons->getItemText(i);
    m_iconAway->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "away.png", TRUE));
    m_iconIdle->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "idle.png", TRUE));
    m_iconDnd->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "dnd.png", TRUE));
    m_iconInvisible->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "invisible.png", TRUE));
    m_iconOffline->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "offline.png", TRUE));
    m_iconOnline->setIcon(makeIcon(getApp(), m_preferences.m_themeStatusPath, "online.png", TRUE));
    return 1;
}

long ConfigDialog::onAddRoleIcons(FXObject*, FXSelector, void*)
{
    dxEXDirDialog dirdialog(this, _("Select theme directory"));
    if(dirdialog.execute(PLACEMENT_CURSOR))
    {
        if(!FXPath::search(dirdialog.getDirectory(), "normal.png").empty() && !themeRoleExist(dirdialog.getDirectory())) m_roleIcons->appendItem(dirdialog.getDirectory());
    }
    m_roleIcons->getNumItems()>1 ? m_deleteRoleTheme->enable() : m_deleteRoleTheme->disable();
    updateRoleIcons();
    return 1;
}

long ConfigDialog::onDeleteRoleIcons(FXObject*, FXSelector, void*)
{
    FXint i = m_roleIcons->getCurrentItem();
    m_roleIcons->removeItem(i);
    i = m_roleIcons->getCurrentItem();
    m_preferences.m_themeRolePath = m_roleIcons->getItemText(i);
    m_iconFounder->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "founder.png", TRUE));
    m_iconHalfop->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "halfop.png", TRUE));
    m_iconNormal->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "normal.png", TRUE));
    m_iconOp->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "op.png", TRUE));
    m_iconVoice->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "voice.png", TRUE));
    m_roleIcons->getNumItems()>1 ? m_deleteRoleTheme->enable() : m_deleteRoleTheme->disable();
    updateRoleIcons();
    return 1;
}

long ConfigDialog::onRoleIconsChanged(FXObject*, FXSelector, void *ptr)
{
    FXint i = (FXint)(FXival)ptr;
    m_preferences.m_themeRolePath = m_roleIcons->getItemText(i);
    m_iconFounder->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "founder.png", TRUE));
    m_iconHalfop->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "halfop.png", TRUE));
    m_iconNormal->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "normal.png", TRUE));
    m_iconOp->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "op.png", TRUE));
    m_iconVoice->setIcon(makeIcon(getApp(), m_preferences.m_themeRolePath, "voice.png", TRUE));
    return 1;
}

long ConfigDialog::onTheme(FXObject*, FXSelector, void *ptr)
{
    FXint no = (FXint)(FXival)ptr;
    ColorTheme *themeSelected = reinterpret_cast<ColorTheme*>(m_themes->getItemData(no));
    if(themeSelected)
    {
        m_themeCurrent.back = themeSelected->back;
        m_themeCurrent.base = themeSelected->base;
        m_themeCurrent.border = themeSelected->border;
        m_themeCurrent.fore = themeSelected->fore;
        m_themeCurrent.menuback = themeSelected->menuback;
        m_themeCurrent.menufore = themeSelected->menufore;
        m_themeCurrent.selback = themeSelected->selback;
        m_themeCurrent.selfore = themeSelected->selfore;
        m_themeCurrent.tipback = themeSelected->tipback;
        m_themeCurrent.tipfore = themeSelected->tipfore;
        m_themeCurrent.notifyback = themeSelected->notifyback;
        m_themeCurrent.notifyfore = themeSelected->notifyfore;
        updateColors();
    }
    return 1;
}

long ConfigDialog::onFont(FXObject*, FXSelector, void*)
{
    dxEXFontDialog dialog(this, _("Select font"));
    FXFontDesc fontdescription;
    m_font->getFontDesc(fontdescription);
    strncpy(fontdescription.face,m_font->getActualName().text(),sizeof(fontdescription.face));
    dialog.setFontSelection(fontdescription);
    if(dialog.execute(PLACEMENT_SCREEN))
    {
        FXFont *oldfont = m_font;
        dialog.getFontSelection(fontdescription);
        m_font = new FXFont(getApp(),fontdescription);
        m_font->create();
        delete oldfont;
        updateFont();
    }
    return 1;
}

long ConfigDialog::onImFont(FXObject*, FXSelector, void*)
{
    dxEXFontDialog dialog(this, _("Select font"));
    FXFontDesc fontdescription;
    m_imFont->getFontDesc(fontdescription);
    strncpy(fontdescription.face,m_imFont->getActualName().text(),sizeof(fontdescription.face));
    dialog.setFontSelection(fontdescription);
    if(dialog.execute(PLACEMENT_SCREEN))
    {
        FXFont *oldfont = m_imFont;
        dialog.getFontSelection(fontdescription);
        m_imFont = new FXFont(getApp(),fontdescription);
        m_imFont->create();
        delete oldfont;
        updateImFont();
    }
    return 1;
}

long ConfigDialog::onThemeColorChanged(FXObject*, FXSelector, void*)
{
    m_themes->setCurrentItem(m_themes->getNumItems()-1);
    updateColors();
    return 1;
}

long ConfigDialog::onTabColorChanged(FXObject*obj, FXSelector, void*)
{
    if(m_themeCurrent.fore == m_preferences.m_unreadColor)
    {
        if(m_themeCurrent.fore != FXRGB(0,0,255)) m_preferences.m_unreadColor = FXRGB(0,0,255);
        else m_preferences.m_unreadColor = FXRGBA(FXREDVAL(m_themeCurrent.fore),FXGREENVAL(m_themeCurrent.fore),FXBLUEVAL(m_themeCurrent.fore)+1,FXALPHAVAL(m_themeCurrent.fore));
    }
    if(m_themeCurrent.fore == m_preferences.m_highlightColor)
    {
        if(m_themeCurrent.fore != FXRGB(255,0,0)) m_preferences.m_highlightColor = FXRGB(255,0,0);
        else m_preferences.m_highlightColor = FXRGBA(FXREDVAL(m_themeCurrent.fore)+1,FXGREENVAL(m_themeCurrent.fore),FXBLUEVAL(m_themeCurrent.fore),FXALPHAVAL(m_themeCurrent.fore));
    }
    m_unread->setTextColor(m_preferences.m_unreadColor);
    m_highlight->setTextColor(m_preferences.m_highlightColor);
    return 1;
}

void ConfigDialog::updateColors()
{
    m_themeCurrent.shadow = makeShadowColor(m_themeCurrent.base);
    m_themeCurrent.hilite = makeHiliteColor(m_themeCurrent.base);

    m_vframe2->setBorderColor(m_themeCurrent.border);
    m_vframe2->setBaseColor(m_themeCurrent.base);
    m_vframe2->setBackColor(m_themeCurrent.base);
    m_vframe2->setShadowColor(m_themeCurrent.shadow);
    m_vframe2->setHiliteColor(m_themeCurrent.hilite);

    m_label->setBorderColor(m_themeCurrent.border);
    m_label->setBaseColor(m_themeCurrent.base);
    m_label->setBackColor(m_themeCurrent.base);
    m_label->setTextColor(m_themeCurrent.fore);
    m_label->setShadowColor(m_themeCurrent.shadow);
    m_label->setHiliteColor(m_themeCurrent.hilite);

    m_textFrame1->setBorderColor(m_themeCurrent.border);
    m_textFrame1->setBaseColor(m_themeCurrent.base);
    m_textFrame1->setBackColor(m_themeCurrent.base);
    m_textFrame1->setShadowColor(m_themeCurrent.shadow);
    m_textFrame1->setHiliteColor(m_themeCurrent.hilite);
    m_textTest->setBorderColor(m_themeCurrent.border);
    m_textTest->setBackColor(m_themeCurrent.back);
    m_textTest->setBaseColor(m_themeCurrent.base);
    m_textTest->setTextColor(m_themeCurrent.fore);
    m_textTest->setSelTextColor(m_themeCurrent.selfore);
    m_textTest->setSelBackColor(m_themeCurrent.selback);
    m_textTest->setCursorColor(m_themeCurrent.fore);
    m_textTest->setShadowColor(m_themeCurrent.shadow);
    m_textTest->setHiliteColor(m_themeCurrent.hilite);

    m_textFrame2->setBorderColor(m_themeCurrent.border);
    m_textFrame2->setBaseColor(m_themeCurrent.base);
    m_textFrame2->setBackColor(m_themeCurrent.back);
    m_textFrame2->setShadowColor(m_themeCurrent.shadow);
    m_textFrame2->setHiliteColor(m_themeCurrent.hilite);
    m_labelSelected->setBorderColor(m_themeCurrent.border);
    m_labelSelected->setBaseColor(m_themeCurrent.base);
    m_labelSelected->setBackColor(m_themeCurrent.selback);
    m_labelSelected->setTextColor(m_themeCurrent.selfore);
    m_labelSelected->setShadowColor(m_themeCurrent.shadow);
    m_labelSelected->setHiliteColor(m_themeCurrent.hilite);

    m_textFrame3->setBorderColor(m_themeCurrent.border);
    m_textFrame3->setBaseColor(m_themeCurrent.base);
    m_textFrame3->setBackColor(m_themeCurrent.back);
    m_textFrame3->setShadowColor(m_themeCurrent.shadow);
    m_textFrame3->setHiliteColor(m_themeCurrent.hilite);
    m_labelNocurrent->setBorderColor(m_themeCurrent.border);
    m_labelNocurrent->setBaseColor(m_themeCurrent.base);
    m_labelNocurrent->setBackColor(m_themeCurrent.base);
    m_labelNocurrent->setTextColor(m_themeCurrent.fore);
    m_labelNocurrent->setShadowColor(m_themeCurrent.shadow);
    m_labelNocurrent->setHiliteColor(m_themeCurrent.hilite);

    m_sep1->setBorderColor(m_themeCurrent.border);
    m_sep1->setBaseColor(m_themeCurrent.base);
    m_sep1->setBackColor(m_themeCurrent.base);
    m_sep1->setShadowColor(m_themeCurrent.shadow);
    m_sep1->setHiliteColor(m_themeCurrent.hilite);

    m_labelTip->setBorderColor(m_themeCurrent.tipfore);
    m_labelTip->setBaseColor(m_themeCurrent.tipback);
    m_labelTip->setBackColor(m_themeCurrent.tipback);
    m_labelTip->setTextColor(m_themeCurrent.tipfore);
    m_labelTip->setShadowColor(m_themeCurrent.shadow);
    m_labelTip->setHiliteColor(m_themeCurrent.hilite);

    if(!dxutils::instance().haveNotify())
    {
        m_labelNotify->setBorderColor(m_themeCurrent.notifyfore);
        m_labelNotify->setBaseColor(m_themeCurrent.notifyback);
        m_labelNotify->setBackColor(m_themeCurrent.notifyback);
        m_labelNotify->setTextColor(m_themeCurrent.notifyfore);
        m_labelNotify->setShadowColor(m_themeCurrent.shadow);
        m_labelNotify->setHiliteColor(m_themeCurrent.hilite);
    }

    m_menuGroup->setBorderColor(m_themeCurrent.border);
    m_menuGroup->setBaseColor(m_themeCurrent.base);
    m_menuGroup->setBackColor(m_themeCurrent.base);
    m_menuGroup->setShadowColor(m_themeCurrent.shadow);
    m_menuGroup->setHiliteColor(m_themeCurrent.hilite);
    m_menuGroup->setTextColor(m_themeCurrent.fore);
    m_menuFrame->setBorderColor(m_themeCurrent.border);
    m_menuFrame->setBaseColor(m_themeCurrent.base);
    m_menuFrame->setBackColor(m_themeCurrent.base);
    m_menuFrame->setShadowColor(m_themeCurrent.shadow);
    m_menuFrame->setHiliteColor(m_themeCurrent.hilite);
    m_sep2->setBorderColor(m_themeCurrent.border);
    m_sep2->setBaseColor(m_themeCurrent.base);
    m_sep2->setBackColor(m_themeCurrent.base);
    m_sep2->setShadowColor(m_themeCurrent.shadow);
    m_sep2->setHiliteColor(m_themeCurrent.hilite);
    for(int i=0; i<3; i++)
    {
        m_menuLabels[i]->setBorderColor(m_themeCurrent.border);
        m_menuLabels[i]->setBaseColor(m_themeCurrent.base);
        m_menuLabels[i]->setBackColor(m_themeCurrent.base);
        m_menuLabels[i]->setTextColor(m_themeCurrent.fore);
        m_menuLabels[i]->setShadowColor(m_themeCurrent.shadow);
        m_menuLabels[i]->setHiliteColor(m_themeCurrent.hilite);
    }
    m_menuLabels[1]->setBorderColor(m_themeCurrent.border);
    m_menuLabels[1]->setBaseColor(m_themeCurrent.menuback);
    m_menuLabels[1]->setBackColor(m_themeCurrent.menuback);
    m_menuLabels[1]->setTextColor(m_themeCurrent.menufore);
    m_menuLabels[1]->setShadowColor(m_themeCurrent.shadow);
    m_menuLabels[1]->setHiliteColor(m_themeCurrent.hilite);

    m_tabFrame->setBorderColor(m_themeCurrent.border);
    m_tabFrame->setBaseColor(m_themeCurrent.base);
    m_tabFrame->setBackColor(m_themeCurrent.base);
    m_tabFrame->setShadowColor(m_themeCurrent.shadow);
    m_tabFrame->setHiliteColor(m_themeCurrent.hilite);
    m_tabs->setBorderColor(m_themeCurrent.border);
    m_tabs->setBaseColor(m_themeCurrent.base);
    m_tabs->setBackColor(m_themeCurrent.base);
    m_tabs->setShadowColor(m_themeCurrent.shadow);
    m_tabs->setHiliteColor(m_themeCurrent.hilite);
    m_tabframe1->setBorderColor(m_themeCurrent.border);
    m_tabframe1->setBaseColor(m_themeCurrent.base);
    m_tabframe1->setBackColor(m_themeCurrent.base);
    m_tabframe1->setShadowColor(m_themeCurrent.shadow);
    m_tabframe1->setHiliteColor(m_themeCurrent.hilite);
    m_tabframe2->setBorderColor(m_themeCurrent.border);
    m_tabframe2->setBaseColor(m_themeCurrent.base);
    m_tabframe2->setBackColor(m_themeCurrent.base);
    m_tabframe2->setShadowColor(m_themeCurrent.shadow);
    m_tabframe2->setHiliteColor(m_themeCurrent.hilite);
    m_tabframe3->setBorderColor(m_themeCurrent.border);
    m_tabframe3->setBaseColor(m_themeCurrent.base);
    m_tabframe3->setBackColor(m_themeCurrent.base);
    m_tabframe3->setShadowColor(m_themeCurrent.shadow);
    m_tabframe3->setHiliteColor(m_themeCurrent.hilite);
    m_tab->setBorderColor(m_themeCurrent.border);
    m_tab->setBaseColor(m_themeCurrent.base);
    m_tab->setBackColor(m_themeCurrent.base);
    m_tab->setTextColor(m_themeCurrent.fore);
    m_tab->setShadowColor(m_themeCurrent.shadow);
    m_tab->setHiliteColor(m_themeCurrent.hilite);
    m_unread->setBorderColor(m_themeCurrent.border);
    m_unread->setBaseColor(m_themeCurrent.base);
    m_unread->setBackColor(m_themeCurrent.base);
    m_unread->setTextColor(m_preferences.m_unreadColor);
    m_unread->setShadowColor(m_themeCurrent.shadow);
    m_unread->setHiliteColor(m_themeCurrent.hilite);
    m_highlight->setBorderColor(m_themeCurrent.border);
    m_highlight->setBaseColor(m_themeCurrent.base);
    m_highlight->setBackColor(m_themeCurrent.base);
    m_highlight->setTextColor(m_preferences.m_highlightColor);
    m_highlight->setShadowColor(m_themeCurrent.shadow);
    m_highlight->setHiliteColor(m_themeCurrent.hilite);
    m_tabLabel1->setBorderColor(m_themeCurrent.border);
    m_tabLabel1->setBaseColor(m_themeCurrent.base);
    m_tabLabel1->setBackColor(m_themeCurrent.base);
    m_tabLabel1->setTextColor(m_themeCurrent.fore);
    m_tabLabel1->setShadowColor(m_themeCurrent.shadow);
    m_tabLabel1->setHiliteColor(m_themeCurrent.hilite);
    m_tabLabel2->setBorderColor(m_themeCurrent.border);
    m_tabLabel2->setBaseColor(m_themeCurrent.base);
    m_tabLabel2->setBackColor(m_themeCurrent.base);
    m_tabLabel2->setTextColor(m_themeCurrent.fore);
    m_tabLabel2->setShadowColor(m_themeCurrent.shadow);
    m_tabLabel2->setHiliteColor(m_themeCurrent.hilite);
    m_tabLabel3->setBorderColor(m_themeCurrent.border);
    m_tabLabel3->setBaseColor(m_themeCurrent.base);
    m_tabLabel3->setBackColor(m_themeCurrent.base);
    m_tabLabel3->setTextColor(m_themeCurrent.fore);
    m_tabLabel3->setShadowColor(m_themeCurrent.shadow);
    m_tabLabel3->setHiliteColor(m_themeCurrent.hilite);
}

static FXString weightToString(FXuint weight){
    switch(weight)
    {
        case FXFont::Thin      : return "thin"; break;
        case FXFont::ExtraLight: return "extralight"; break;
        case FXFont::Light     : return "light"; break;
        case FXFont::Normal    : return "normal"; break;
        case FXFont::Medium    : return "medium"; break;
        case FXFont::DemiBold  : return "demibold"; break;
        case FXFont::Bold      : return "bold"; break;
        case FXFont::ExtraBold : return "extrabold"; break;
        case FXFont::Black     : return "heavy"; break;
        default: return ""; break;
    }
    return "";
}

static FXString slantToString(FXuint slant)
{
    switch(slant)
    {
        case FXFont::Straight       : return "regular"; break;
        case FXFont::Italic         : return "italic"; break;
        case FXFont::Oblique        : return "oblique"; break;
        case FXFont::ReverseItalic  : return "reverse italic"; break;
        case FXFont::ReverseOblique : return "reverse oblique"; break;
        default : return ""; break;
    }
    return "";
}

void ConfigDialog::updateFont()
{
    FXString fontname = m_font->getActualName() +", " + FXStringVal(m_font->getSize()/10);
    if(m_font->getWeight()!=0 && m_font->getWeight()!=FXFont::Normal)
    {
        fontname += ", " + weightToString(m_font->getWeight());
    }
    if (m_font->getSlant()!=0 && m_font->getSlant()!=FXFont::Straight)
    {
        fontname += ", " + slantToString(m_font->getSlant());
    }
    m_fontButton->setText(fontname);

    m_labelSelected->setFont(m_font);
    m_labelNocurrent->setFont(m_font);
    m_labelTip->setFont(m_font);
    m_label->setFont(m_font);
    m_textTest->setFont(m_font);
    m_menuGroup->setFont(m_font);
    m_menuLabels[0]->setFont(m_font);
    m_menuLabels[1]->setFont(m_font);
    m_menuLabels[2]->setFont(m_font);
}

void ConfigDialog::updateImFont()
{
    FXString fontname = m_imFont->getActualName() +", " + FXStringVal(m_imFont->getSize()/10);
    if(m_imFont->getWeight()!=0 && m_imFont->getWeight()!=FXFont::Normal)
    {
        fontname += ", " + weightToString(m_imFont->getWeight());
    }
    if (m_imFont->getSlant()!=0 && m_imFont->getSlant()!=FXFont::Straight)
    {
        fontname += ", " + slantToString(m_imFont->getSlant());
    }
    m_imfontButton->setText(fontname);

    m_text->setFont(m_imFont);
}

long ConfigDialog::onLogChanged(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_logging)
    {
        m_selectPath->enable();
        m_logLoadCurrent->enable();
    }
    else
    {
        m_selectPath->disable();
        m_preferences.m_logLoadCurrent = false;
        m_logLoadCurrent->disable();
    }
    return 1;
}

long ConfigDialog::onNickCharChanged(FXObject*, FXSelector, void*)
{
    m_preferences.m_nickCompletionChar = m_nickCharField->getText().left(1);
    m_nickCharField->setText(m_preferences.m_nickCompletionChar);
    return 1;
}

long ConfigDialog::onTray(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_useTray)
    {
        m_closeToTrayButton->enable();
        m_minToTrayButton->enable();
#ifdef HAVE_TRAY
        m_checkNotifyMessageTray->enable();
#endif
    }
    else
    {
        m_preferences.m_closeToTray = FALSE;
        m_closeToTrayButton->disable();
        m_preferences.m_minToTray = FALSE;
        m_minToTrayButton->disable();
#ifdef HAVE_TRAY
        m_preferences.m_notifyMessageTray = FALSE;
        m_checkNotifyMessageTray->disable();
#endif
    }
    return 1;
}

long ConfigDialog::onPathSelect(FXObject*, FXSelector, void*)
{
    dxEXDirDialog dirdialog(this,_("Select log directory"));
    dirdialog.setDirectory(m_preferences.m_logPath);
    if(dirdialog.execute(PLACEMENT_CURSOR))
    {
        m_preferences.m_logPath = dirdialog.getDirectory();
    }
    return 1;
}

long ConfigDialog::onSounds(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_sounds)
    {
        m_checkConnect->enable();
        m_selectConnect->enable();
        if(FXStat::exists(m_preferences.m_pathConnect)) m_playConnect->enable();
        m_checkDisconnect->enable();
        m_selectDisconnect->enable();
        if(FXStat::exists(m_preferences.m_pathDisconnect)) m_playDisconnect->enable();
        m_checkMessage->enable();
        m_selectMessage->enable();
        if(FXStat::exists(m_preferences.m_pathMessage)) m_playMessage->enable();
    }
    else
    {
        m_checkConnect->disable();
        m_selectConnect->disable();
        m_playConnect->disable();
        m_checkDisconnect->disable();
        m_selectDisconnect->disable();
        m_playDisconnect->disable();
        m_checkMessage->disable();
        m_selectMessage->disable();
        m_playMessage->disable();
    }
    return 1;
}

long ConfigDialog::onSoundConnect(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_soundConnect)
    {
        m_selectConnect->enable();
        if(FXStat::exists(m_preferences.m_pathConnect)) m_playConnect->enable();
    }
    else
    {
        m_selectConnect->disable();
        m_playConnect->enable();
    }
    return 1;
}

long ConfigDialog::onSoundDisconnect(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_soundDisconnect)
    {
        m_selectDisconnect->enable();
        if(FXStat::exists(m_preferences.m_pathDisconnect)) m_playDisconnect->enable();
    }
    else
    {
        m_selectDisconnect->disable();
        m_playDisconnect->enable();
    }
    return 1;
}

long ConfigDialog::onSoundMessage(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_soundMessage)
    {
        m_selectMessage->enable();
        if(FXStat::exists(m_preferences.m_pathMessage)) m_playMessage->enable();
    }
    else
    {
        m_selectMessage->disable();
        m_playMessage->enable();
    }
    return 1;
}

long ConfigDialog::onPlay(FXObject*, FXSelector sel, void*)
{
    switch(FXSELID(sel)) {
        case ConfigDialog_PLAYCONNECT:
            dxutils::instance().playFile(m_preferences.m_pathConnect);
            return 1;
        case ConfigDialog_PLAYDISCONNECT:
            dxutils::instance().playFile(m_preferences.m_pathDisconnect);
            return 1;
        case ConfigDialog_PLAYMESSAGE:
            dxutils::instance().playFile(m_preferences.m_pathMessage);
            return 1;
    }
    return 1;
}

long ConfigDialog::onSelectPath(FXObject*, FXSelector sel, void*)
{
    dxEXFileDialog file(this, _("Select file"));
    file.setPatternList(_("Sound file (*.wav)"));
    switch(FXSELID(sel)) {
        case ConfigDialog_SELECTCONNECT:
            if(file.execute(PLACEMENT_CURSOR))
            {
                m_preferences.m_pathConnect = file.getFilename();
                m_playConnect->enable();
            }
            return 1;
        case ConfigDialog_SELECTDISCONNECT:
            if(file.execute(PLACEMENT_CURSOR))
            {
                m_preferences.m_pathDisconnect = file.getFilename();
                m_playDisconnect->enable();
            }
            return 1;
        case ConfigDialog_SELECTMESSAGE:
            if(file.execute(PLACEMENT_CURSOR))
            {
                m_preferences.m_pathMessage = file.getFilename();
                m_playMessage->enable();
            }
            return 1;
    }
    return 1;
}

long ConfigDialog::onNotify(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_notify)
    {
        m_checkNotifyAccountConnect->enable();
        m_checkNotifyAccountDisconnect->enable();
        m_checkNotifyBuddyConnect->enable();
        m_checkNotifyBuddyDisconnect->enable();
        m_checkNotifyMessage->enable();
        m_checkNotifyFiletransfer->enable();
    }
    else
    {
        m_checkNotifyAccountConnect->disable();
        m_checkNotifyAccountDisconnect->disable();
        m_checkNotifyBuddyConnect->disable();
        m_checkNotifyBuddyDisconnect->disable();
        m_checkNotifyMessage->disable();
        m_checkNotifyFiletransfer->disable();
    }
    return 1;
}

long ConfigDialog::onShowNotify(FXObject*, FXSelector, void*)
{
    dxim::instance()->showNotify(FXStringFormat(_("%s is available"), "dvx"), m_preferences.m_notifyPosition);
    return 1;
}

long ConfigDialog::onUseSmileys(FXObject*, FXSelector, void*)
{
    if(m_preferences.m_useSmileys)
    {
        m_addSmiley->enable();
        m_importSmiley->enable();
        if((FXint)m_preferences.m_smileysMap.size())
        {
            m_modifySmiley->enable();
            m_deleteSmiley->enable();
            m_exportSmiley->enable();
        }
        else
        {
            m_modifySmiley->disable();
            m_deleteSmiley->disable();
            m_exportSmiley->disable();
        }
    }
    else
    {
        m_addSmiley->disable();
        m_modifySmiley->disable();
        m_deleteSmiley->disable();
        m_importSmiley->disable();
        m_exportSmiley->disable();
    }
    return 1;
}

long ConfigDialog::onAddSmiley(FXObject*, FXSelector, void*)
{
    if((FXint)m_preferences.m_smileysMap.size()>=256)
    {
        dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Maximum number of 256 smileys is reached"));
        return 1;
    }
    SmileyDialog dialog(this, _("Add smiley"), "", "");
    if(dialog.execute())
    {
        if(!dialog.iconExist())
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Icon file '%s' doesn't exist or isn't image file"), dialog.getPath().text());
            return 1;
        }
        if(dialog.getSmiley().empty())
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Smiley text is empty"));
            return 1;
        }
        if(smileyExist(dialog.getSmiley()))
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Smiley '%s' already exist"), dialog.getSmiley().text());
            return 1;
        }
        m_preferences.m_smileysMap.insert(dxStringPair(dialog.getSmiley(), dialog.getPath()));
        m_smileys->appendItem(new FXListItem(dialog.getSmiley(), dialog.getIcon()));
        m_modifySmiley->enable();
        m_deleteSmiley->enable();
        m_exportSmiley->enable();
    }
    return 1;
}

long ConfigDialog::onModifySmiley(FXObject*, FXSelector, void*)
{
    FXint index = m_smileys->getCurrentItem();
    FXString oldkey = m_smileys->getItemText(index);
    SmileyDialog dialog(this, _("Modify smiley"), oldkey, m_preferences.m_smileysMap[oldkey]);
    if(dialog.execute())
    {
        if(!dialog.iconExist())
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Icon file '%s' doesn't exist or isn't image file"), dialog.getPath().text());
            return 1;
        }
        if(dialog.getSmiley().empty())
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Smiley text is empty"));
            return 1;
        }
        if(smileyExist(dialog.getSmiley()) && dialog.getSmiley()!=oldkey)
        {
            dxEXMessageBox::information(this, MBOX_OK, _("Information"), _("Smiley '%s' already exist"), dialog.getSmiley().text());
            return 1;
        }
        m_preferences.m_smileysMap.erase(oldkey);
        m_smileys->removeItem(index, TRUE);
        m_preferences.m_smileysMap.insert(dxStringPair(dialog.getSmiley(), dialog.getPath()));
        m_smileys->insertItem(index, new FXListItem(dialog.getSmiley(), dialog.getIcon()), TRUE);
        m_smileys->setCurrentItem(index, TRUE);
    }
    return 1;
}

long ConfigDialog::onDeleteSmiley(FXObject*, FXSelector, void*)
{
    m_preferences.m_smileysMap.erase(m_smileys->getItemText(m_smileys->getCurrentItem()));
    m_smileys->removeItem(m_smileys->getCurrentItem(), TRUE);
    if(!(FXint)m_preferences.m_smileysMap.size())
    {
        m_modifySmiley->disable();
        m_deleteSmiley->disable();
        m_exportSmiley->disable();
    }
    return 1;
}

void ConfigDialog::fillStatusIcons()
{
    if(!m_preferences.m_themesStatusList.empty())
    {
        for(FXint i=0; i<m_preferences.m_themesStatusList.contains(';'); i++)
        {
            m_statusIcons->appendItem(m_preferences.m_themesStatusList.before(';', i+1).rafter(';'));
        }
    }
}

void ConfigDialog::fillRoleIcons()
{
    if(!m_preferences.m_themesRoleList.empty())
    {
        for(FXint i=0; i<m_preferences.m_themesRoleList.contains(';'); i++)
        {
            m_roleIcons->appendItem(m_preferences.m_themesRoleList.before(';', i+1).rafter(';'));
        }
    }
}

void ConfigDialog::fillThemes()
{
    FXint i, scheme=-1;
    for(i=0; i<numThemes; i++)
    {
        if((m_themeCurrent.back == ColorThemes[i].back) &&
                (m_themeCurrent.base == ColorThemes[i].base) &&
                (m_themeCurrent.border == ColorThemes[i].border) &&
                (m_themeCurrent.fore == ColorThemes[i].fore) &&
                (m_themeCurrent.menuback == ColorThemes[i].menuback) &&
                (m_themeCurrent.menufore == ColorThemes[i].menufore) &&
                (m_themeCurrent.selback == ColorThemes[i].selback) &&
                (m_themeCurrent.selfore == ColorThemes[i].selfore) &&
                (m_themeCurrent.tipback == ColorThemes[i].tipback) &&
                (m_themeCurrent.tipfore == ColorThemes[i].tipfore) &&
                (m_themeCurrent.notifyback == ColorThemes[i].notifyback) &&
                (m_themeCurrent.notifyfore == ColorThemes[i].notifyfore))
        {
            scheme = i;
        }
        m_themes->appendItem(ColorThemes[i].name,NULL,(void*)&ColorThemes[i]);
    }
    if(scheme == -1)
        scheme = numThemes;
    m_themes->appendItem(_("User Defined"), NULL, &m_themeCurrent);
    m_themes->setCurrentItem(scheme);
}

void ConfigDialog::fillSmileys()
{
    if((FXint)m_preferences.m_smileysMap.size())
    {
        dxStringIt it;
        for(it=m_preferences.m_smileysMap.begin(); it!=m_preferences.m_smileysMap.end(); it++)
        {
            m_smileys->appendItem(new FXListItem((*it).first, createIconFromName(getApp(), (*it).second)));
        }
    }
}

void ConfigDialog::updateStatusIcons()
{
    m_preferences.m_themesStatusList.clear();
    for(FXint i=0; i<m_statusIcons->getNumItems(); i++)
    {
        m_preferences.m_themesStatusList.append(m_statusIcons->getItemText(i)+';');
    }
}

void ConfigDialog::updateRoleIcons()
{
    m_preferences.m_themesRoleList.clear();
    for(FXint i=0; i<m_statusIcons->getNumItems(); i++)
    {
        m_preferences.m_themesRoleList.append(m_roleIcons->getItemText(i)+';');
    }
}

FXbool ConfigDialog::themeStatusExist(const FXString &ckdTheme)
{
    for(FXint i=0; i<m_statusIcons->getNumItems(); i++)
    {
        if(m_statusIcons->getItemText(i) == ckdTheme)
        {
            return TRUE;
        }
    }
    return FALSE;
}

FXbool ConfigDialog::themeRoleExist(const FXString &ckdTheme)
{
    for(FXint i=0; i<m_roleIcons->getNumItems(); i++)
    {
        if(m_roleIcons->getItemText(i) == ckdTheme)
        {
            return TRUE;
        }
    }
    return FALSE;
}

FXbool ConfigDialog::smileyExist(const FXString& ckdSmiley)
{
    dxStringIt it;
    for(it=m_preferences.m_smileysMap.begin(); it!=m_preferences.m_smileysMap.end(); it++)
    {
        if(!compare((*it).first, ckdSmiley)) return TRUE;
    }
    return FALSE;
}

long ConfigDialog::onRaiseWindow(FXObject*,FXSelector,void*)
{
    if(m_preferences.m_raiseWindow)
    {
        m_checkNotifyMessageTray->disable();
        m_preferences.m_notifyMessageTray = false;
    }
    else
    {
        m_checkNotifyMessageTray->enable();
    }
    return 1;
}

