/*
 *      logviewer.h
 *
 *      Copyright 2009 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LOGVIEWER_H
#define LOGVIEWER_H

#include "defs.h"
#include "fxext.h"
#ifndef DXTEXT_H
#include "dxtext.h"
#endif
#include "purplemm.h"

class LogViewer;

class LogItem : public FXTreeItem
{
    FXDECLARE(LogItem)
    friend class LogViewer;
protected:
    FXTime date; // Time of item
private:
    LogItem(const LogItem&);
    LogItem & operator=(const LogItem&);
protected:

    LogItem() : date(0){}
public:
    enum
    {
        ROOT = 512, /// Root Item
        ACCOUNT = 1024, /// Account item
        CONVERSATION = 2048, /// IM/MUC item
        YEAR = 4096, /// Year item
        MONTH = 8192, /// Month item
        LOGFILE = 16384 /// LogFile item
    };
public:
    /// Constructor
    LogItem(const FXString& text, FXIcon* oi = NULL, FXIcon* ci = NULL, void* ptr = NULL) : FXTreeItem(text, oi, ci, ptr), date(0)
    {
    }
    /// Return TRUE if this is a LogFile item
    FXbool isLogFile() const
    {
        return (state & LOGFILE) != 0;
    }
    /// Return TRUE if this is a Month item
    FXbool isMonth() const
    {
        return (state & MONTH) != 0;
    }
    /// Return TRUE if this is Year item
    FXbool isYear() const
    {
        return (state & YEAR) != 0;
    }
    /// Return TRUE if this is IM/MUC item
    FXbool isConversation() const
    {
        return (state & CONVERSATION) != 0;
    }
    /// Return TRUE if this is Account item
    FXbool isAccount() const
    {
        return (state & ACCOUNT) != 0;
    }
    /// Return TRUE if this is Root item
    FXbool isRoot() const
    {
        return (state & ROOT) != 0;
    }
    /// Return the date for this item
    FXTime getDate() const
    {
        return date;
    }
};

class LogViewer: public FXTopWindow
{
    FXDECLARE(LogViewer)
public:
    LogViewer(FXApp *app, const FXString &lpath, FXFont *fnt);
    LogViewer(FXApp *app, FXFont *fnt, PURPLE::Account acount);
    LogViewer(FXApp *app, FXFont *fnt, PURPLE::Buddy buddy);
    LogViewer(FXApp *app, FXFont *fnt, PURPLE::Chat chat);
    virtual ~LogViewer();

    virtual void show();
    virtual void show(FXuint placement);
    void preferencesUpdated();

    long onClose(FXObject*,FXSelector,void*);
    long onKeyPress(FXObject*,FXSelector,void*);
    long onTree(FXObject*,FXSelector,void*);
    long onRightTree(FXObject*,FXSelector,void*);
    long onSearch(FXObject*,FXSelector,void*);
    long onSearchNext(FXObject*,FXSelector,void*);
    long onReset(FXObject*,FXSelector,void*);
    long onCmdSearchOptions(FXObject*,FXSelector,void*);
    long onPack(FXObject*,FXSelector,void*);
    long onUnpack(FXObject*,FXSelector,void*);
    long onDelete(FXObject*,FXSelector,void*);
    long onTextLink(FXObject*,FXSelector,void*);
    long onTextRightMouse(FXObject*,FXSelector,void*);
    long onTextMiddleMouse(FXObject*,FXSelector,void*);
    long onOpenLink(FXObject*,FXSelector,void*);
    long onCopyLink(FXObject*,FXSelector,void*);
    long onClipboardRequest(FXObject*,FXSelector,void*);
    void setFont(FXFont*);
    void setLogPath(const FXString &path);

private:
    LogViewer() {}
    LogViewer(const LogViewer&);

    void initialize(FXFont *fnt);
    FXbool loadFile(const FXString& file);
    void loadTree();
    FXString getItemPathname(const LogItem* item);
    void listChildItems(LogItem *par);
    void enableAllItems();
    void scan();
    FXint getNickStyle(const FXString &nick);
    void clearLog();
    void appendLogLine(const FXString &line);
    void appendLinkText(const FXString &txt, FXint stylenum);
    FXint hiliteStyleExist(FXColor foreColor, FXColor backColor, FXuint style);
    void createHiliteStyle(FXColor foreColor, FXColor backColor, FXuint style);
    LogItem *getConversationItem(LogItem *item);
    LogItem *getAccountItem(LogItem *item);
    void searchInLogFile();
    FXint searchInConversation(LogItem *item, FXbool showMessage=TRUE);
    FXint searchInAccount(LogItem *item, FXbool showMessage=TRUE);
    void searchInAll();
    void listConversationChildItems(LogItem *conversation);
    void listAccountChildItems(LogItem *account);
    void listRootChildItems(LogItem *root);
    LogItem* getItem(LogItem *parent, FXString text);
    dxStringArray getLogFiles(LogItem *parent);
    FXint getLogFilesCount(LogItem *parent);
    FXString getLogFileName(const LogItem* item);
    FXint stringToStyle(FXString style);
    void sortTreeHistory(LogItem *item);
    void expandTreeHistory(LogItem *item, FXbool notify=FALSE);
    LogItem* appendItemToTreeHistory(LogItem* father, LogItem* item, FXbool notify=FALSE);

    FXVerticalFrame *m_listframe, *m_textframe, *m_treeframe;
    FXHorizontalFrame *m_content, *m_buttonframe, *m_searchframe;
    FXSplitter *m_splitter;
    FXTreeList *m_treeHistory;
    FXTextField *m_searchfield;
    dxEXButton *m_buttonClose, *m_buttonSearch, *m_buttonReset;
    FXGroupBox *m_group;
    FXRadioButton *m_buttonAll, *m_buttonAccount, *m_buttonConversation, *m_buttonFile;
    FXCheckButton *m_buttonIcase;
    dxText *m_text;
    LogItem *m_itemOnRight;
    FXString m_logPath, m_searchstring;
    FXDataTarget m_targetAll, m_targetAccount, m_targetConversation, m_targetFile, m_targetIcase;
    FXbool m_all, m_account, m_conversation, m_file, m_icase, m_treeLoaded;
    dxHiliteArray m_textStyleList;
    FXString m_link;
    /*
     * 0 full logs
     * 1 account logs
     * 2 buddy or chat logs
    */
    FXuint m_logsFor;
    FXString m_accountName;
    FXString m_conversationName;
};

#endif  /* LOGVIEWER_H */

