/*
 *      roster.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "roster.h"
#include "config.h"
#include "i18n.h"
#include "fxext.h"
#include "dxutils.h"
#include "preferences.h"
#include "dxim.h"
#include "icons.h"

#define ICON_SPACING        4   // Spacing between parent and child in x direction
#define SIDE_SPACING        4   // Spacing between side and item
#define MAX_AVATAR_SIZE 32

//Map
FXDEFMAP(dxRosterList) dxRosterListMap [] = {
    FXMAPFUNC(SEL_QUERY_TIP, 0, dxRosterList::onQueryTip)
};

FXIMPLEMENT(dxRosterList, FXTreeList, dxRosterListMap, ARRAYNUMBER(dxRosterListMap))

dxRosterList::dxRosterList(FXComposite* p, FXObject* tgt, FXSelector sel, FXuint opts, FXint x, FXint y, FXint w, FXint h)
: FXTreeList(p, tgt, sel, opts, x, y, w, h)
{
}

//Show tooltip
long dxRosterList::onQueryTip(FXObject* sender,FXSelector sel,void* ptr)
{
    if(FXWindow::onQueryTip(sender,sel,ptr))
        return 1;
    if((flags&FLAG_TIP) && !(options&TREELIST_AUTOSELECT)) // No tip when autoselect!
    {
        FXint x,y;
        FXuint buttons;
        getCursorPosition(x,y,buttons);
        dxRosterTreeItem *item=(dxRosterTreeItem*)getItemAt(x,y);
        if(item)
        {
            FXString tip = "";
            if(item->isChat())
            {
                tip = FXStringFormat("%s\n", item->getChat().getAlias().c_str());
                std::vector<PURPLE::Account::ChatEntry> fields = item->getChat().getChatParameters();
                FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
                {
                    PURPLE::Account::ChatEntry entry = *it;
                    if(entry.getSecret())
                    {
                        if(entry.getValue().empty()) tip.append(FXStringFormat("%s\n", dxutils::normalizeLabelString(entry.getLabel(), '_').text()));
                        else tip.append(FXStringFormat("%s *****\n", dxutils::normalizeLabelString(entry.getLabel(), '_').text()));
                    }
                    else
                        tip.append(FXStringFormat("%s %s\n", dxutils::normalizeLabelString(entry.getLabel(), '_').text(), entry.getValue().c_str()));
                }
                tip.append(FXStringFormat(_("Autojoin: %s"), item->getChat().getAutojoin() ? _("Yes") : _("No")));
            }
            else if(item->isFriend())
            {
                tip = FXStringFormat("%s\n", item->getBuddy().getServerAlias().c_str());
                tip.append(FXStringFormat(_("ID: %s\n"), item->getBuddy().getUsername().c_str()));
                tip.append(FXStringFormat(_("Status: %s"), item->getBuddy().getStatusName().c_str()));
                if(!item->getBuddy().getStatusMsg().empty())
                {
                    tip.append('\n');
                    tip.append(FXStringFormat(_("Message: %s"), item->getBuddy().getStatusMsg().c_str()));
                }
            }
            else if(item->isAccount())
            {
                tip = FXStringFormat("%s\n", item->getAccount().getUsername().c_str());
                if(!item->getAccount().getAlias().empty()) tip.append(FXStringFormat(_("Alias: %s\n"), item->getAccount().getAlias().c_str()));
                tip.append(FXStringFormat(_("Protocol: %s\n"), item->getAccount().getProtocol().getName().c_str()));
                tip.append(FXStringFormat(_("Status: %s"), item->getAccount().getStatusName().c_str()));
                if(item->getAccount().isConnected() && !item->getAccount().getStatusMessage().empty())
                {
                    tip.append('\n');
                    tip.append(FXStringFormat(_("Message: %s"), item->getAccount().getStatusMessage().c_str()));
                }
            }
            sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&tip);
            return 1;
        }
    }
    return 0;
}

//set item second icon
void dxRosterList::setItemSecondIcon(dxRosterTreeItem* item, FXIcon* icon, FXbool owned)
{
    if(item==NULL){ fxerror("%s::setItemSecondIcon: NULL argument.\n",getClassName()); }
    if(item->getSecondIcon()!=icon) recalc();
    item->setSecondIcon(icon,owned);
}

//get item second icon
FXIcon* dxRosterList::getItemSecondIcon(const dxRosterTreeItem* item) const
{
    if(item==NULL){ fxerror("%s::getItemSecondIcon: NULL argument.\n",getClassName()); }
    return item->getSecondIcon();
}

FXIMPLEMENT(dxRosterTreeItem, FXTreeItem, NULL, 0)

// Character width
FXint dxRosterTreeItem::charWidth(FXFont *font, FXwchar ch, FXint col) const
{
    if (ch < ' ')
    {
        if (ch != '\t')
        {
            return font->getCharWidth('#');
        }
        return font->getCharWidth(' ')*(8 - col % 8);
    }
    return font->getCharWidth(ch);
}

// Draw item
void dxRosterTreeItem::draw(const FXTreeList* list,FXDC& dc,FXint xx,FXint yy,FXint ww,FXint hh) const
{
    if((state&HIDDEN)!=0)
    {
        return;
    }
    FXbool showStatus = isFriend() ? DXPREFERENCES.m_showStatusmsg : TRUE;
    FXIcon *icon=(state&OPENED)?openIcon:closedIcon;
    FXIcon *icon2=m_secondIcon;
    FXFont *font=list->getFont();
    FXint th=0,tw=0,sh=0,ih=0,iw=0,ih2=0,ix2=0;
    FXint lw=list->getWidth();
    if(list->verticalScrollBar()->shown()) lw=lw-list->verticalScrollBar()->getWidth();
    xx+=SIDE_SPACING/2;
    if(getNumChildren()>getNumHiddenChildren())
    {
        if(isExpanded())
        {
            iw=ICO_ROSTEROPENED->getWidth();
            ih=ICO_ROSTEROPENED->getHeight();
            dc.drawIcon(ICO_ROSTEROPENED,xx,yy+(hh-ih)/2);
        }
        else
        {
            iw=ICO_ROSTERCLOSED->getWidth();
            ih=ICO_ROSTERCLOSED->getHeight();
            dc.drawIcon(ICO_ROSTERCLOSED,xx,yy+(hh-ih)/2);
        }
    }
    if(!isFriend() && !isChat()) xx+=ICON_SPACING+FXMAX(ICO_ROSTEROPENED->getWidth(),ICO_ROSTERCLOSED->getWidth());
    if(icon)
    {
        iw=icon->getWidth();
        ih=icon->getHeight();
        dc.drawIcon(icon,xx,yy+(hh-ih)/2);
        xx+=ICON_SPACING+iw;
    }
    if(!label.empty())
    {
        tw=4+font->getTextWidth(label.text(),label.length());
        th=4+font->getFontHeight();
        if(showStatus && !getStatusMsg().empty())
        {
            sh=4+font->getFontHeight();
        }
        yy+=(hh-th-sh)/2;
        if(isSelected())
        {
            dc.setForeground(list->getSelBackColor());
            dc.fillRectangle(xx,yy,tw,th);
        }
        if(hasFocus())
        {
            dc.drawFocusRectangle(xx+1,yy+1,tw-2,th-2);
        }
        if(!isEnabled())
            dc.setForeground(makeShadowColor(list->getBackColor()));
        else if(isSelected())
            dc.setForeground(list->getSelTextColor());
        else
        {
            if(isOnline()) dc.setForeground(list->getTextColor());
            else dc.setForeground(dxutils::makeOfflineColor(list->getTextColor()));
        }
        dc.drawText(xx+2,yy+font->getFontAscent()+2,label);
        if(showStatus)
        {
            dc.setForeground(dxutils::makeOfflineColor(list->getTextColor()));
            dc.drawText(xx+2,yy+font->getFontAscent()+2+sh,getWrappedStatusMsg(font, icon2?lw-icon2->getWidth()-xx-2-SIDE_SPACING:lw-xx-2-SIDE_SPACING));
        }
    }
    if(icon2)
    {
        ih2=icon2->getHeight();
        ix2=FXMAX(xx+2+tw+ICON_SPACING,lw-icon2->getWidth()-ICON_SPACING);
        dc.drawIcon(icon2,ix2,yy-(hh-th-sh)/2+(hh-ih2)/2);
    }
}

//Get item width
FXint dxRosterTreeItem::getWidth(const FXTreeList* list) const
{
    if((state&HIDDEN)!=0)
        return 0;
    FXint w=0,oiw=0,ciw=0;
    if(openIcon) oiw=openIcon->getWidth();
    if(closedIcon) ciw=closedIcon->getWidth();
    w=FXMAX(oiw,ciw);
    if(!label.empty())
    {
        if(w) w+=ICON_SPACING;
        w+=4+list->getFont()->getTextWidth(label.text(),label.length());
    }
    if(m_secondIcon) w+=m_secondIcon->getWidth()+ICON_SPACING;
    if(!isFriend() && !isChat()) w+=FXMAX(ICO_ROSTEROPENED->getWidth(),ICO_ROSTERCLOSED->getWidth())+ICON_SPACING;
    return SIDE_SPACING+w;
}

//Get item height
FXint dxRosterTreeItem::getHeight(const FXTreeList* list) const
{
    if((state&HIDDEN)!=0)
        return 0;
    FXbool showStatus = isFriend() ? DXPREFERENCES.m_showStatusmsg : TRUE;
    FXint th=0,oih=0,cih=0,sih=0,gih=0;
    if(openIcon) oih=openIcon->getHeight()+4;
    if(closedIcon) cih=closedIcon->getHeight()+4;
    if(m_secondIcon) sih=m_secondIcon->getHeight()+4;
    if(!isFriend() && !isChat() && getNumChildren()>getNumHiddenChildren())
    {
        if(isExpanded()) gih=ICO_ROSTEROPENED->getHeight()+4;
        else gih=ICO_ROSTERCLOSED->getHeight()+4;
    }
    if(!label.empty())
    {
        th=4+list->getFont()->getFontHeight();
        if(showStatus && !getStatusMsg().empty())
            th+=th;
    }
    return FXMAX(gih,FXMAX4(th,oih,cih,sih));
}

//Get number of hidden child items
FXint dxRosterTreeItem::getNumHiddenChildren() const
{
    dxRosterTreeItem *item=(dxRosterTreeItem*)first;
    FXint n=0;
    while(item){if(item->isHidden())n++;item=(dxRosterTreeItem*)item->next;}
    return n;
}

FXbool dxRosterTreeItem::isOnline() const
{
    if(isGroup() || isChat()) return true;
    else if(isAccount()) return m_account.isConnected();
    else return m_buddy.isOnline();
}

//Get rostertitem protocol
FXString dxRosterTreeItem::getProtocol()
{
    return dxutils::fromStdString(m_account.getProtocol().getPurpleID());
}

//Get status msg for Buddy or Account
FXString dxRosterTreeItem::getStatusMsg() const
{
    if(isFriend() && m_buddy.isValid() && !m_buddy.getStatusMsg().empty())
        return dxutils::fromStdString(m_buddy.getStatusMsg());
    if(isAccount() && m_account.isValid() && m_account.isConnected() && !m_account.getStatusMessage().empty())
        return dxutils::fromStdString(m_account.getStatusMessage());
    return "";
}

//Get status msg for Buddy or Account - wrapped text
FXString dxRosterTreeItem::getWrappedStatusMsg(FXFont *font, FXint w) const
{
    if((isFriend() && m_buddy.isValid() && !m_buddy.getStatusMsg().empty())
       || (isAccount() && m_account.isValid() && m_account.isConnected() && !m_account.getStatusMessage().empty()))
    {
        FXint tw=0;
        FXint sp, sc, cw;
        FXint pw = font->getTextWidth("...");
        FXString msg = dxutils::fromStdString(isAccount()?m_account.getStatusMessage():m_buddy.getStatusMsg());
        FXString result = "";
        for (sp = sc = 0; sp < msg.length(); sp = msg.inc(sp), sc++)
        {
            cw = charWidth(font, msg.wc(sp), sc);
            if(tw+pw+cw > w)
            {
                result = result+"...";
                break;
            }
            tw += cw;
            result.append(&msg[sp], msg.inc(sp)-sp);
        }
        return result;
    }
    return "";
}

//set last time of status change
void dxRosterTreeItem::setStatusChange()
{
    m_lastStatusChange = FXSystem::now();
}

//set PURPLE::Chat
void dxRosterTreeItem::setChat(PURPLE::Chat chat)
{
    m_chat = chat;
}

//change ssl or avatar icon
void dxRosterTreeItem::setSecondIcon(FXIcon* icon, FXbool owned)
{
    if(m_secondIcon && (state&SECONDICONOWNED))
    {
        if(m_secondIcon!=icon) delete m_secondIcon;
        state&=~SECONDICONOWNED;
    }
    m_secondIcon=icon;
    if(m_secondIcon && owned)
    {
        state|=SECONDICONOWNED;
    }
}

//set path for avatar
void dxRosterTreeItem::setAvatarPath(FXString avatarPath)
{
    if(m_avatarPath!=avatarPath && FXStat::exists(avatarPath))
    {
        m_avatarPath = avatarPath;
        updateShowAvatars();
    }
}

void dxRosterTreeItem::setID(const FXString& ID)
{
    if(m_ID!=ID)
    {
        m_ID = ID;
    }
}

void dxRosterTreeItem::updateShowAvatars()
{
    if(isFriend())
    {
        if(DXPREFERENCES.m_showAvatars)
        {
            FXIcon *icon = makeScaledIcon(dxim::instance()->getApp(), m_avatarPath, MAX_AVATAR_SIZE, DXPREFERENCES.m_colors.back);
            if(icon) setSecondIcon(icon, TRUE);
            else setSecondIcon(NULL);
        }
        else
            setSecondIcon(NULL);
    }
}
