/*
 *      dxtabitem.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "dxtabitem.h"
#include "config.h"
#include "icons.h"
#include "i18n.h"
#include "dxutils.h"
#include "preferences.h"
#include "dxim.h"

static FXbool isBadchar(FXchar c)
{
    switch(c) {
        case ' ':
        case ',':
        case '\0':
        case '\02':
        case '\03':
        case '\017':
        case '\021':
        case '\026':
        case '\035':
        case '\037':
        case '\n':
        case '\r':
        case '<':
        case '>':
        case '"':
        case '\'':
            return TRUE;
        default:
            return FALSE;
    }
}

FXDEFMAP(dxTabItem) dxTabItemMap[] = {
    FXMAPFUNC(SEL_TEXTLINK,             dxTabItem_TEXT,     dxTabItem::onTextLink),
    FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,   dxTabItem_TEXT,     dxTabItem::onTextRightMouse),
    FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,  dxTabItem_TEXT,     dxTabItem::onTextMiddleMouse),
    FXMAPFUNC(SEL_COMMAND,              dxPipe_PIPE,        dxTabItem::onPipe),
    FXMAPFUNC(SEL_TIMEOUT,              dxTabItem_PTIME,    dxTabItem::onPipeTimeout)
};

FXIMPLEMENT(dxTabItem, dxEXTabItem, dxTabItemMap, ARRAYNUMBER(dxTabItemMap))

dxTabItem::dxTabItem(dxTabBook *tab, const FXString &tabtext, PURPLE::Conversation *conv, FXIcon *icon, FXuint opts, bool enableClear)
    : dxEXTabItem(tab, tabtext, icon, opts), m_conv(conv), m_pipe(NULL), m_sendPipe(FALSE), m_enableClear(enableClear)
{
    this->setIconPosition(ICON_BEFORE_TEXT);

    m_text = new dxText(tab, this, dxTabItem_TEXT, FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY|TEXT_WORDWRAP|TEXT_SHOWACTIVE|TEXT_AUTOSCROLL);

    dxHiliteStyle style = {getApp()->getForeColor(),getApp()->getBackColor(),getApp()->getSelforeColor(),getApp()->getSelbackColor(),0,FALSE};
    for(int i=0; i<16; i++)
    {
        m_textStyleList.append(style);
    }
    //link style
    m_textStyleList[0].normalForeColor = DXPREFERENCES.m_colors.link;
    m_textStyleList[0].link = TRUE;
    //highlight
    m_textStyleList[1].normalForeColor = DXPREFERENCES.m_colors.hilight;
    //action
    m_textStyleList[2].normalForeColor = DXPREFERENCES.m_colors.action;
    //error
    m_textStyleList[3].normalForeColor = DXPREFERENCES.m_colors.error;
    //user message
    m_textStyleList[4].normalForeColor = DXPREFERENCES.m_colors.user;
    //bold text
    m_textStyleList[5].style = FXText::STYLE_BOLD;
    //mymsg style
    m_textStyleList[6].normalForeColor = DXPREFERENCES.m_colors.mymsg;
    //history msg style
    m_textStyleList[7].normalForeColor = DXPREFERENCES.m_colors.history;
    //next styles for colored nicks
    m_textStyleList[8].normalForeColor = FXRGB(196, 160, 0);
    m_textStyleList[9].normalForeColor = FXRGB(206, 92, 0);
    m_textStyleList[10].normalForeColor = FXRGB(143, 89, 2);
    m_textStyleList[11].normalForeColor = FXRGB(78, 154, 6);
    m_textStyleList[12].normalForeColor = FXRGB(32, 74, 135);
    m_textStyleList[13].normalForeColor = FXRGB(117, 80, 123);
    m_textStyleList[14].normalForeColor = FXRGB(164, 0, 0);
    m_textStyleList[15].normalForeColor = FXRGB(85, 87, 83);

    m_text->setHiliteStyles(m_textStyleList.data());
}

dxTabItem::~dxTabItem()
{
}

long dxTabItem::onTextLink(FXObject *, FXSelector, void *data)
{
    dxutils::launchLink(static_cast<FXchar*>(data));
    return 1;
}

long dxTabItem::onTextRightMouse(FXObject *, FXSelector, void *ptr)
{
    FXEvent* event=(FXEvent*)ptr;
    if(event->moved) return 1;
    FXMenuPane popupmenu(this);
    FXMenuCommand *clear = NULL;
    if(m_enableClear)
    {
        clear = new FXMenuCommand(&popupmenu, _("Clear window"), ICO_CLEAR, m_text, dxText::ID_CLEAR);
        new FXMenuSeparator(&popupmenu);
    }
    FXMenuCommand *selectall = new FXMenuCommand(&popupmenu, _("Select All"), ICO_SELECTALL, m_text, dxText::ID_SELECT_ALL);
    FXMenuCommand *copy = new FXMenuCommand(&popupmenu, _("Copy"), ICO_COPY, m_text, dxText::ID_COPY_SEL);
    new FXMenuSeparator(&popupmenu);
    FXMenuCommand *find = new FXMenuCommand(&popupmenu, _("Find"), ICO_FIND, m_text, dxText::ID_SEARCH);
    if(!m_text->hasSomeText())
    {
        if(m_enableClear) clear->disable();
        selectall->disable();
        copy->disable();
        find->disable();
    }
    if(!m_text->hasSelectedText())
    {
        copy->disable();
    }
    popupmenu.create();
    popupmenu.popup(NULL,event->root_x,event->root_y);
    getApp()->runModalWhileShown(&popupmenu);
    return 1;
}

long dxTabItem::onTextMiddleMouse(FXObject*, FXSelector, void *ptr)
{
    FXEvent* event=(FXEvent*)ptr;
    if(event->moved) return 1;
#ifndef _WIN32 //middleclick copy
    m_text->copySelectedText();
#endif
    return 1;
}

long dxTabItem::onPipe(FXObject*, FXSelector, void *ptr)
{
    FXString text = *(FXString*)ptr;
    if(m_sendPipe)
    {
        if(!getApp()->hasTimeout(this, dxTabItem_PTIME)) getApp()->addTimeout(this, dxTabItem_PTIME);
        m_pipeStrings.append(text);
    }
    else appendText(text, FALSE, FALSE);
    return 1;
}

long dxTabItem::onPipeTimeout(FXObject*, FXSelector, void*)
{
    if(m_pipeStrings.no() > 3)
    {
        if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
        if(m_conv->isValid())
        {
            m_conv->sendMessage(m_pipeStrings[0].text());
        }
        m_pipeStrings.erase(0);
        getApp()->addTimeout(this, dxTabItem_PTIME, 3000);
    }
    else
    {
        while(m_pipeStrings.no())
        {
            if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
            if(m_conv->isValid())
            {
                m_conv->sendMessage(m_pipeStrings[0].text());
            }
            m_pipeStrings.erase(0);
        }
    }
    return 1;
}

//usefull for set tab current
void dxTabItem::makeLastRowVisible()
{
    m_text->makeLastRowVisible(TRUE);
}

void dxTabItem::clearChat()
{
    m_textStyleList.no(16);
    m_text->setHiliteStyles(m_textStyleList.data());
    m_text->clearText();
}

void dxTabItem::removeSmileys()
{
    m_text->removeSmileys();
}

FXString dxTabItem::getAccountID()
{
    return dxutils::fromStdString(m_conv->getAccount().getUsername());
}

FXString dxTabItem::getAccountProtocol()
{
    return dxutils::fromStdString(m_conv->getAccount().getProtocol().getPurpleID());
}

//append history msg
void dxTabItem::appendHistoryMsg(const FXString& nick, const FXString& msg, const FXString& time)
{
    m_text->appendText(time);
    if(!nick.empty()) m_text->appendStyledText(nick+": ", DXPREFERENCES.m_coloredNick?getNickColor(nick):6);
    appendLinkText(dxutils::parseHtml(msg), 8);
}

//append text, check for link
void dxTabItem::appendLinkText(const FXString &txt, FXint stylenum, FXTime time)
{
    if(time) m_text->appendText(FXSystem::time("[%H:%M:%S] ", time));
    FXint i = 0;
    FXint linkLength = 0;
    FXbool bold = FALSE;
    FXbool under = FALSE;
    FXint lastStyle = stylenum;
    FXColor foreColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalForeColor : DXPREFERENCES.m_colors.text;
    FXColor backColor = stylenum && stylenum<=m_textStyleList.no() ? m_textStyleList[stylenum-1].normalBackColor : DXPREFERENCES.m_colors.back;
    FXString normalText = "";
    FXint length = txt.length();
    while(i<length)
    {
        if(txt[i]=='h' && !comparecase(txt.mid(i,7),"http://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>7 ? 1 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='h' && !comparecase(txt.mid(i,8),"https://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>8 ? 1 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='f' && !comparecase(txt.mid(i,6),"ftp://"))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>6 ? 1 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else if(txt[i]=='w' && !comparecase(txt.mid(i,4),"www."))
        {
            if(!normalText.empty())
            {
                m_text->appendStyledText(normalText, lastStyle);
                normalText.clear();
            }
            for(FXint j=i; j<length; j++)
            {
                if(isBadchar(txt[j]))
                {
                    break;
                }
                linkLength++;
            }
            m_text->appendStyledText(txt.mid(i, linkLength), linkLength>4 ? 1 : stylenum);
            i+=linkLength-1;
            linkLength=0;
        }
        else
        {
            if(txt[i] == '\002') //bold
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = !bold;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\026') //reverse
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXColor tempColor = foreColor;
                foreColor = backColor;
                backColor = tempColor;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\037') //underline
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                under = !under;
                FXuint style = 0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                }
            }
            else if(txt[i] == '\021') //fixed
            {
                dxutils::debugLine("Poslan fixed styl");
            }
            else if(txt[i] == '\035') //italic
            {
                dxutils::debugLine("Poslan italic styl");
            }
            else if(txt[i] == '\003') //color foreground
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style=0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXint colorLength = 0;
                foreColor = DXPREFERENCES.m_colors.text;
                backColor = DXPREFERENCES.m_colors.back;
                if(FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+1,6)))
                {
                    foreColor = FXRGB(FXIntVal(txt.mid(i+1,2),16),FXIntVal(txt.mid(i+3,2),16),FXIntVal(txt.mid(i+5,2),16));
                    colorLength +=6;
                }
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                };
                i +=colorLength;
            }
            else if(txt[i] == '\025') //color backround
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                FXuint style=0;
                if(bold && under) style = FXText::STYLE_BOLD|FXText::STYLE_UNDERLINE;
                else if(bold && !under) style = FXText::STYLE_BOLD;
                else if(!bold && under) style = FXText::STYLE_UNDERLINE;
                FXint colorLength = 0;
                backColor = DXPREFERENCES.m_colors.back;
                if(FXRex("^\\h\\h\\h\\h\\h\\h+$").match(txt.mid(i+1,6)))
                {
                    backColor = FXRGB(FXIntVal(txt.mid(i+1,2),16),FXIntVal(txt.mid(i+3,2),16),FXIntVal(txt.mid(i+5,2),16));
                    colorLength +=6;
                }
                lastStyle = hiliteStyleExist(foreColor, backColor, style);
                if(lastStyle == -1)
                {
                    //dxText has available max. 255 styles
                    if(m_textStyleList.no()<256)
                    {
                        createHiliteStyle(foreColor, backColor, style);
                        lastStyle = m_textStyleList.no();
                    }
                    else lastStyle = 0;
                };
                i +=colorLength;
            }
            else if(txt[i] == '\017') //reset
            {
                if(!normalText.empty())
                {
                    m_text->appendStyledText(normalText, lastStyle);
                    normalText.clear();
                }
                bold = FALSE;
                under = FALSE;
                foreColor = DXPREFERENCES.m_colors.text;
                backColor = DXPREFERENCES.m_colors.back;
                lastStyle = stylenum;
            }
            else
            {
                normalText.append(txt[i]);
            }
        }
        i++;
    }
    if(!normalText.empty())
    {
        m_text->appendStyledText(normalText, lastStyle);
    }
    m_text->appendText("\n");
}

FXint dxTabItem::hiliteStyleExist(FXColor foreColor, FXColor backColor, FXuint style)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        if(m_textStyleList[i].normalForeColor == foreColor
                && m_textStyleList[i].normalBackColor == backColor
                && m_textStyleList[i].style == style)
            return i+1;
    }
    return -1;
}

void dxTabItem::createHiliteStyle(FXColor foreColor, FXColor backColor, FXuint style)
{
    dxHiliteStyle nstyle = {foreColor,backColor,getApp()->getSelforeColor(),getApp()->getSelbackColor(),style,FALSE};
    m_textStyleList.append(nstyle);
    m_text->setHiliteStyles(m_textStyleList.data());
}

//preferences updated
void dxTabItem::preferencesUpdated()
{
    setTextForeColor(DXPREFERENCES.m_colors.text);
    setTextBackColor(DXPREFERENCES.m_colors.back);
    setUserColor(DXPREFERENCES.m_colors.user);
    setActionsColor(DXPREFERENCES.m_colors.action);
    setErrorColor(DXPREFERENCES.m_colors.error);
    setHilightColor(DXPREFERENCES.m_colors.hilight);
    setLinkColor(DXPREFERENCES.m_colors.link);
    setMymsgColor(DXPREFERENCES.m_colors.mymsg);
    stopLogging();
}

//set text forecolor
void dxTabItem::setTextForeColor(FXColor clr)
{
    m_textStyleList[5].normalForeColor = clr;
    m_text->setTextColor(clr);
}

//set text backcolor
void dxTabItem::setTextBackColor(FXColor clr)
{
    for(FXint i=0; i<m_textStyleList.no(); i++)
    {
        m_textStyleList[i].normalBackColor = clr;
    }
    m_text->setBackColor(clr);
}

//set color for user message
void dxTabItem::setUserColor(FXColor clr)
{
    m_textStyleList[4].normalForeColor = clr;
}

//set color for action message
void dxTabItem::setActionsColor(FXColor clr)
{
    m_textStyleList[2].normalForeColor = clr;
}

//set color for error message
void dxTabItem::setErrorColor(FXColor clr)
{
    m_textStyleList[3].normalForeColor = clr;
}

//set color for highlighted message
void dxTabItem::setHilightColor(FXColor clr)
{
    m_textStyleList[1].normalForeColor = clr;
}

//set color for link text
void dxTabItem::setLinkColor(FXColor clr)
{
    m_textStyleList[0].normalForeColor = clr;
}

//set color for my msg
void dxTabItem::setMymsgColor(FXColor clr)
{
    m_textStyleList[6].normalForeColor = clr;
}

//update font
void dxTabItem::updateImFont()
{
    m_text->recalc();
    m_text->setFont(dxim::instance()->getImFont());
}

//set smaileys
void dxTabItem::setSmileys(FXbool smiley, dxSmileyArray smileys)
{
    m_text->setSmileys(smiley, smileys);
}

//get nick color for colorednick==TRUE
FXint dxTabItem::getNickColor(const FXString &nick)
{
    //9 is first colored nick style
    return 9+nick.hash()%8;
}

std::string dxTabItem::styleToString(FXint style)
{
    switch(style) {
        case 2: return "highlight";
        case 3: return "action";
        case 4: return "error";
        case 5: return "user";
        default: return "";
    }
}

