/*
 *      callback.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "config.h"
#include "callback.h"
#include "dxutils.h"
#include "i18n.h"
#include "preferences.h"
#include <utilsmm.h>

dxgCallbacks::dxgCallbacks()
{
    m_purple = new PURPLE::Purple(this, "dxim", VERSION, "http://dxim.dxsolutions.org", DXPREFERENCES.getIniPath());
}

dxgCallbacks::~dxgCallbacks()
{}

PURPLE::Purple *dxgCallbacks::getPurplemm()
{
    return m_purple;
}

/** Allow your application to be notified about errors on library layer */
void dxgCallbacks::showError(std::string msg)
{
    signalError(msg);
}

/** Allow your application to be notified about libpurple debug messages */
void dxgCallbacks::showDebug(std::string debug)
{
    dxutils::debugLine(debug);
}

/** Libpurple inited */
void dxgCallbacks::buddylistLoaded()
{
    signalBuddylistLoaded();
}

/** New account added */
void dxgCallbacks::accountAdded(PURPLE::Account account)
{
    signalAccountAdded(account);
}

/** Account removed */
void dxgCallbacks::accountRemoved(std::string account, std::string protocolID)
{
    signalAccountRemoved(account, protocolID);
}

/** Account connecting */
void dxgCallbacks::accountConnecting(PURPLE::Account account, std::string msg)
{
    signalAccountConnecting(account, msg);
}

/** Acount connected*/
void dxgCallbacks::accountConnected(PURPLE::Account account)
{
    signalAccountConnected(account);
}

/** Acount disconnected*/
void dxgCallbacks::accountDisconnected(PURPLE::Account account, std::string reason)
{
    signalAccountDisconnected(account, reason);
}

/** Account status changed*/
void dxgCallbacks::accountStatusChanged(PURPLE::Account account)
{
    signalAccountStatusChanged(account);
}

/** Account signed on, so show buddy list */
void dxgCallbacks::accountSignedOn(PURPLE::Account account)
{
    signalAccountSignedOn(account);
}

/** New IM created */
void dxgCallbacks::newIM(PURPLE::Conversation *conv)
{
    signalNewIM(conv);
}

/** New Chat created */
void dxgCallbacks::newChat(PURPLE::Conversation *conv)
{
    signalNewChat(conv);
}

/** IM closed */
void dxgCallbacks::closedIM(PURPLE::Conversation *conv)
{
    signalCloseConvTab(conv);
}

/** Chat closed */
void dxgCallbacks::closedChat(PURPLE::Conversation *conv)
{
    signalCloseConvTab(conv);
}

/** New my message
 * @param conv          The conversation.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param action        Is message in 3rd person (/me)?
 */
void dxgCallbacks::newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID)
{
    signalNewMyMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), PURPLE::markupHtml(msg), action, messageID);
}

void dxgCallbacks::messageRecived(PURPLE::Conversation *conv, std::string messageID)
{
    if(DXPREFERENCES.m_receiptsMark && !messageID.empty()) signalMessageRecived(conv, messageID);
}

/** New IM message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxgCallbacks::newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    Glib::ustring timeStr = delayed?purple_utf8_strftime("[%x %H:%M:%S] ", localtime(&time)):purple_utf8_strftime("[%H:%M:%S] ", localtime(&time));
    signalNewIMMessage(conv, timeStr, sender, PURPLE::markupHtml(msg), highlight, action);
}

/** New Chat message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxgCallbacks::newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
    Glib::ustring timeStr = delayed?purple_utf8_strftime("[%x %H:%M:%S] ", localtime(&time)):purple_utf8_strftime("[%H:%M:%S] ", localtime(&time));
    signalNewChatMessage(conv, timeStr, sender, PURPLE::markupHtml(msg), delayed, highlight, action);
}

/** New system mesage for conversation
 *
 * @param conv  The conversation
 * @param msg  The message text
 * @param time  The time the message was sent
 */
void dxgCallbacks::newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    signalNewConvSystemMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), msg);
}

/** New error mesage for conversation
 *
 * @param conv      conversation
 * @param msg       message text
 * @param time      time the message was sent
 */
void dxgCallbacks::newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
    signalNewConvErrorMessage(conv, purple_utf8_strftime("[%H:%M:%S] ", localtime(&time)), msg);
}

/** A new list was created */
void dxgCallbacks::roomlistCreated(PURPLE::RoomList *rlist)
{
    signalRoomlistCreated(rlist);
}

/** Sets the columns of RoomList */
void dxgCallbacks::roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns)
{
    signalRoomlistColumns(rlist, columns);
}

/** Add a room to the list */
void dxgCallbacks::roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields)
{
    signalRoomlistAddRoom(rlist, fields);
}

/** Listing RoomList finished */
void dxgCallbacks::roomlistFinished(PURPLE::RoomList *rlist)
{
    signalRoomlistFinished(rlist);
}

/** New InputRequest */
void dxgCallbacks::newInputRequest(PURPLE::InputRequest *request)
{
    signalInputRequest(request);
}

/** New ActionRequest */
void dxgCallbacks::newActionRequest(PURPLE::ActionRequest *request)
{
    signalActionRequest(request);
}

/** New ChoiceRequest */
void dxgCallbacks::newChoiceRequest(PURPLE::ChoiceRequest *request)
{
    signalChoiceRequest(request);
}

/** New FieldsRequest */
void dxgCallbacks::newFieldsRequest(PURPLE::FieldsRequest *request)
{
    signalFieldsRequest(request);
}

/** New FileRequest */
void dxgCallbacks::newFileRequest(PURPLE::FileRequest *request)
{
    signalFileRequest(request);
}

/** ChatBuddy joined chat */
void dxgCallbacks::chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    signalChatBuddyJoined(conv, buddy);
}

/** ChatBuddy updated */
void dxgCallbacks::chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy)
{
    signalChatBuddyUpdated(conv, buddy);
}

/** ChatBuddy left chat */
void dxgCallbacks::chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
{
    signalChatBuddyLeft(conv, buddy);
}

/** ChatBuddy renamed
* @param who       buddy which was renamed
* @param nbuddy    new buddy established by rename
*/
void dxgCallbacks::chatBuddyRenamed(PURPLE::Conversation *conv, std::string who, PURPLE::ChatBuddy nbuddy)
{
    signalChatBuddyRenamed(conv, who, nbuddy);
}

/** Chat topic changed
* @param who  topic changer
* @param topic  new topic
*/
void dxgCallbacks::chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic)
{
    signalChatTopic(conv, who, topic);
}

/** Buddy typing on IM
* @param typing =true typing, =false stoped typing
*/
void dxgCallbacks::buddyTyping(PURPLE::Conversation *conv, bool typing)
{
    signalBuddyTyping(conv, typing);
}

/** Buddy signed on */
void dxgCallbacks::buddySignedOn(PURPLE::Buddy buddy)
{
    signalBuddySignedOn(buddy);
}

/** Buddy signed off */
void dxgCallbacks::buddySignedOff(PURPLE::Buddy buddy)
{
    signalBuddySignedOff(buddy);
}

/** Buddy status changed */
void dxgCallbacks::buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive /*oldStatus*/, PurpleStatusPrimitive newStatus)
{
    signalBuddyStatusChanged(buddy, newStatus);
}

/** Buddy icon changed */
void dxgCallbacks::buddyIconChanged(PURPLE::Buddy buddy)
{
    signalBuddyIconChanged(buddy);
}

/** Buddy added */
void dxgCallbacks::buddyAdded(PURPLE::Buddy buddy)
{
    signalBuddyAdded(buddy);
}

/** Buddy removed */
void dxgCallbacks::buddyRemoved(PURPLE::Account account, std::string buddy)
{
    signalBuddyRemoved(account, buddy);
}

/** Chat added */
void dxgCallbacks::chatAdded(PURPLE::Chat chat)
{
    signalChatAdded(chat);
}

/** Chat removed */
void dxgCallbacks::chatRemoved(PURPLE::Account account, std::string chat)
{
    signalChatRemoved(account, chat);
}

/** Notification message to the user
* @param type  notification type
* @param title  title of the message
* @param primary  main point of the message
* @param secondary  secondary information
*/
void dxgCallbacks::notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary)
{
    signalNotifyMessage(type, title, Glib::ustring::compose("%1<br><br>%2", primary, secondary));
}

/** Email notification to the user
* @param subject  subject of the email
* @param from  from address
* @param to  estination address
* @param url  URL where the message can be read
*/
void dxgCallbacks::notifyEmail(std::string subject, std::string from, std::string to, std::string url)
{
    signalNotifyEmail(Glib::ustring::compose(_("From: %1\nTo: %2\nSubject: %3"), from, to, subject), url);
}

/** Notification with formatted text
* @param title  title of the message
* @param primary  main point of the message
* @param secondary  secondary information
* @param text  formatted text
*/
void dxgCallbacks::notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated)
{
    if(secondary.empty())
        signalNotifyMessage(PURPLE_NOTIFY_MSG_INFO, title, Glib::ustring::compose("%1<br><br>%2", primary, formated));
    else
        signalNotifyMessage(PURPLE_NOTIFY_MSG_INFO, title, Glib::ustring::compose("%1<br><br>%2<br><br>%3", primary, secondary, formated));

}

/** Notification information about buddy
* @param account   requested account
* @param buddy     buddy which about
* @param info      vector with UserInfo values
*/
void dxgCallbacks::notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info)
{
    Glib::ustring text, value;
    std::vector<PURPLE::Notify::UserInfo>::iterator it;
    for(it=info.begin(); it!=info.end(); it++)
    {
        value = (*it).getValue();
        if(value.find("<img id='")!=Glib::ustring::npos)
        {
            value = Glib::ustring::compose("<img src='%1'>", PURPLE::imagePathByID(dxutils::stringSection(value, '\'', 1, false)));
        }
        if(value.empty()) text.append(Glib::ustring::compose("<b>%1</b>:<br>", (*it).getLabel()));
        else text.append(Glib::ustring::compose("<b>%1</b>: %2<br>", (*it).getLabel(), PURPLE::markupHtml(value)));
    }
    signalNotifyUserinfo(account, buddy, text);
}

/** Opens a URI or somehow presents it to the user
* @param uri    The URI to display or go to.
*/
void dxgCallbacks::notifyURI(std::string uri)
{
    signalNotifyUri(uri);
}

/** New FileTransfer */
void dxgCallbacks::newFileTransfer(PURPLE::FileTransfer *ft)
{
    signalNewFileTransfer(ft);
}

/** FileTransfer finished */
void dxgCallbacks::fileTransferFinished(PURPLE::FileTransfer *ft)
{
    signalFileTransferFinished(ft);
}

/** FileTransfer started */
void dxgCallbacks::fileTransferStarted(PURPLE::FileTransfer *ft)
{
    signalFileTransferStarted(ft);
}

/** FileTransfer progress */
void dxgCallbacks::fileTransferProgress(PURPLE::FileTransfer *ft, double /*percent*/)
{
    signalFileTransferProgress(ft);
}

/** FileTransfer canceled */
void dxgCallbacks::fileTransferCanceled(PURPLE::FileTransfer *ft)
{
    signalFileTransferCanceled(ft);
}

