/*
 *      accountdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "accountdialog.h"
#include "accounteditor.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

class ProtocolListDialog : public Gtk::Dialog {
public:
    ProtocolListDialog(Gtk::Window& parent, PURPLE::Purple* purple) : Gtk::Dialog(_("Select protocol"), parent), m_purple(purple)
    {
        m_protocolModel = Gtk::ListStore::create(m_protocolColumns);
        m_protocols.set_model(m_protocolModel);
        std::vector<PURPLE::Protocol> v = m_purple->getProtocolsList();
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            Gtk::TreeModel::Row row = *(m_protocolModel->append());
            row[m_protocolColumns.m_icon] = dxutils::getProtocolIcon(it->getPurpleID());
            row[m_protocolColumns.m_name] = it->getName();
        }
        m_protocols.pack_start(m_protocolColumns.m_icon);
        m_protocols.pack_start(m_protocolColumns.m_name);
        m_protocols.set_active(0);
        Gtk::HBox *frame = Gtk::manage(new Gtk::HBox());
        frame->add(*Gtk::manage(new Gtk::Label(_("Protocol:"))));
        frame->add(m_protocols);
        get_vbox()->pack_start(*frame, Gtk::PACK_SHRINK);
        add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
        add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
        show_all();
    }
    virtual ~ProtocolListDialog() {}

    Glib::ustring getProtocolName()
    {
        Gtk::TreeModel::iterator iter = m_protocols.get_active();
        if(iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row)
            {
                return row[m_protocolColumns.m_name];
            }
        }
        return "";
    }
private:
    PURPLE::Purple *m_purple;
    struct ProtocolModelColumns : public Gtk::TreeModel::ColumnRecord
    {
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > m_icon;
        Gtk::TreeModelColumn<Glib::ustring> m_name;

        ProtocolModelColumns() { add(m_icon); add(m_name); }
    };
    const ProtocolModelColumns m_protocolColumns;
    Glib::RefPtr<Gtk::ListStore> m_protocolModel;
    Gtk::ComboBox m_protocols;
};

AccountDialog::AccountDialog(Gtk::Window& parent, PURPLE::Purple* purple)
: Gtk::Dialog(_("Accounts list"), parent), m_purple(purple)
{
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_accountModel = Gtk::ListStore::create(m_accountColumns);
    m_accounts.set_model(m_accountModel);
    Gtk::TreeView::Column *uColumn = Gtk::manage(new Gtk::TreeView::Column(_("User name")));
    uColumn->pack_start(m_accountColumns.m_userIcon, false);
    uColumn->pack_start(m_accountColumns.m_userName);
    m_accounts.append_column(*uColumn);
    Gtk::TreeView::Column *pColumn = Gtk::manage(new Gtk::TreeView::Column(_("Protocol")));
    pColumn->pack_start(m_accountColumns.m_protocolIcon, false);
    pColumn->pack_start(m_accountColumns.m_protocolName);
    m_accounts.append_column(*pColumn);
    Gtk::CellRendererToggle *check = Gtk::manage(new Gtk::CellRendererToggle());
    check->signal_toggled().connect(sigc::mem_fun(this, &AccountDialog::onEnabledChanged));
    m_accounts.append_column(_("Enabled"), *check);
    Gtk::TreeViewColumn *eColumn = m_accounts.get_column(2);
    if(eColumn)
    {
        eColumn->add_attribute(check->property_active(), m_accountColumns.m_enabled);
    }
    m_accountSelectionModel = m_accounts.get_selection();
    m_accountSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    if(accounts.size())
    {
        for(guint i=0; i<accounts.size(); i++)
        {
            createAccount(accounts[i]);
        }
    }
    m_accounts.set_size_request(-1, 155);
    swin->add(m_accounts);
    get_vbox()->pack_start(*swin, Gtk::PACK_EXPAND_WIDGET);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    m_add = Gtk::manage(new Gtk::Button(_("_Add"), true));
    m_add->signal_clicked().connect(sigc::mem_fun(this, &AccountDialog::addAccount));
    m_modify = Gtk::manage(new Gtk::Button(_("_Modify"), true));
    m_modify->signal_clicked().connect(sigc::mem_fun(this, &AccountDialog::modifyAccount));
    m_delete = Gtk::manage(new Gtk::Button(_("_Delete"), true));
    m_delete->signal_clicked().connect(sigc::mem_fun(this, &AccountDialog::deleteAccount));
    Gtk::Button *closebutton = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebutton->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*m_add, Gtk::PACK_SHRINK);
    btnbox->pack_end(*m_modify, Gtk::PACK_SHRINK);
    btnbox->pack_end(*m_delete, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebutton, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    updateButtons();
    show_all();
}

AccountDialog::~AccountDialog()
{
}

void AccountDialog::updateButtons()
{
    if(m_purple->getAccountsList().size())
    {
        m_modify->set_sensitive(true);
        m_delete->set_sensitive(true);
    }
    else
    {
        m_modify->set_sensitive(false);
        m_delete->set_sensitive(false);
    }
}

void AccountDialog::onEnabledChanged(const Glib::ustring &path_string)
{
    Gtk::TreeModel::iterator it = m_accountModel->get_iter(path_string);
    Gtk::TreeModel::Row row = *it;
    PURPLE::Account account = row[m_accountColumns.m_account];
    if(row[m_accountColumns.m_enabled] == true)
    {
        row[m_accountColumns.m_enabled] = false;
        account.disconnect();
    }
    else
    {
        row[m_accountColumns.m_enabled] = true;
        account.connect();
    }
}

void AccountDialog::addAccount()
{
    ProtocolListDialog dialog(*this, m_purple);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        std::vector<PURPLE::Protocol> v = m_purple->getProtocolsList();
        FOREACH(std::vector<PURPLE::Protocol>, v, it)
        {
            if(!it->getName().compare(dialog.getProtocolName()))
            {
                AccountEditor editor(*this, *it);
                int result = editor.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    NewAccount *account = new NewAccount;
                    editor.fillAccount(account);
                    PURPLE::Account acc = m_purple->addAccount(account->protocol, account->username, account->password, account->rememberPassword, account->options, account->registerAccount);
                    acc.setCheckMail(account->checkEmail);
                    acc.setAlias(account->alias);
                    if(!account->iconPath.empty()) acc.setBuddyIcon(account->iconPath);
                    createAccount(acc);
                }
                updateButtons();
                return;
            }
        }
    }
}

void AccountDialog::modifyAccount()
{
    Gtk::TreeModel::iterator it = m_accountSelectionModel->get_selected();
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        PURPLE::Account acc = row[m_accountColumns.m_account];
        AccountEditor editor(*this, acc);
        int result = editor.run();
        if(result == Gtk::RESPONSE_OK)
        {
            NewAccount *account = new NewAccount;
            editor.fillAccount(account);
            acc.setOptions(account->options);
            acc.setPassword(account->password, account->rememberPassword);
            acc.setCheckMail(account->checkEmail);
            acc.setAlias(account->alias);
            account->iconPath.empty()?acc.removeBuddyIcon():acc.setBuddyIcon(account->iconPath);
            row[m_accountColumns.m_userIcon] = dxutils::createIcon(account->iconPath, 32);
        }
    }
}

void AccountDialog::deleteAccount()
{
    Gtk::TreeModel::iterator it = m_accountSelectionModel->get_selected();
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        Gtk::MessageDialog dialog(Glib::ustring::compose(_("Do you want really remove %1?"), row[m_accountColumns.m_userName]), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
        int result = dialog.run();
        if(result == Gtk::RESPONSE_YES)
        {
            m_purple->deleteAccount(row[m_accountColumns.m_account]);
            m_accountModel->erase(it);
        }
        updateButtons();
    }
}

void AccountDialog::createAccount(PURPLE::Account account)
{
    if(account.isValid())
    {
        Gtk::TreeModel::Row row = *(m_accountModel->append());
        row[m_accountColumns.m_userIcon] = dxutils::createIcon(account.getBuddyIcon(), 32);
        row[m_accountColumns.m_userName] = account.getUsername();
        row[m_accountColumns.m_protocolIcon] = dxutils::getProtocolIcon(account.getProtocol().getPurpleID());
        row[m_accountColumns.m_protocolName] = account.getProtocol().getName();
        row[m_accountColumns.m_enabled] = account.isEnabled();
        row[m_accountColumns.m_account] = account;
        m_accountSelectionModel->select(row);
    }
}


