/*
 *      accounteditor.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "accounteditor.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "widgets/imagelabel.h"

#define MAX_ICON_SIZE 128

/* Constructor - Modify existing account */
AccountEditor::AccountEditor(Gtk::Window& parent, PURPLE::Account account)
    : Gtk::Dialog(_("Modify account"), parent), m_registerAccount(_("Register account")), m_checkEmail(_("Check new mails")), m_rememberPassword(_("Remember password"))
{
    this->m_editing = true;
    this->m_protocol = account.getProtocol();
    this->m_account = account;
    this->m_options = account.getOptions();
    this->m_gtalk = false;
    this->m_iconPath = account.getBuddyIcon();
    this->initializeEditor();
}

/* Constructor - Modify existing account */
AccountEditor::AccountEditor(PURPLE::Account account)
    : Gtk::Dialog(_("Modify account")), m_registerAccount(_("Register account")), m_checkEmail(_("Check new mails")), m_rememberPassword(_("Remember password"))
{
    this->m_editing = true;
    this->m_protocol = account.getProtocol();
    this->m_account = account;
    this->m_options = account.getOptions();
    this->m_gtalk = false;
    this->m_iconPath = account.getBuddyIcon();
    this->initializeEditor();
}

/* Constructor - Create new account */
AccountEditor::AccountEditor(Gtk::Window& parent, PURPLE::Protocol protocol, bool gtalk)
    : Gtk::Dialog(_("Create new account"), parent), m_registerAccount(_("Register account")), m_checkEmail(_("Check new mails")), m_rememberPassword(_("Remember password"))
{
    this->m_editing = false;
    this->m_protocol = protocol;
    this->m_account = PURPLE::Account();
    this->m_options = protocol.getOptions();
    this->m_gtalk = gtalk;
    this->m_iconPath = "";
    this->initializeEditor();
}

/* Destructor */
AccountEditor::~AccountEditor()
{
}

void AccountEditor::initializeEditor()
{
    get_vbox()->pack_start(m_tabbook);
    m_tabbook.set_scrollable(true);
    createAccountTab();
    createProtocolTab();
    add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    show_all();
    if(!m_protocol.hasCheckMail()) m_checkEmail.hide();
    if(m_editing) m_registerAccount.hide();
    if(!m_protocol.hasBuddyIcon()) m_iconBox.hide();
    if(!m_editing || m_account.getBuddyIcon().empty()) m_iconRemove.hide();
}

void AccountEditor::createAccountTab()
{
    Gtk::Table *tab = Gtk::manage(new Gtk::Table());
    tab->attach(*Gtk::manage(new Gtk::Label(_("Protocol"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    if(m_gtalk)
    {
        ImageLabel *label = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon("gtalk"), "Google Talk"));
        tab->attach(*label, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    }
    else
    {
        ImageLabel *label = Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(m_protocol.getPurpleID()), m_protocol.getName()));
        tab->attach(*label, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    }
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    Glib::ustring username = "";
    int row = 1;
    if(m_editing) username = m_account.getUsername();
    if(splits.size())
    {
        tab->attach(*Gtk::manage(new Gtk::Label(_("User name"), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
        tab->attach(*edit, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        m_usernameFields.push_back(edit);
        if(m_editing)
        {
            Glib::ustring::size_type pos = username.find(splits[0].getSeparator());
            edit->set_text(username.substr(0, pos));
            username = username.substr(pos+1);
            if(m_account.isEnabled()) edit->set_sensitive(false);
        }
        row++;
        for(size_t i=0; i<splits.size(); i++, row++)
        {
            tab->attach(*Gtk::manage(new Gtk::Label(splits[i].getText(), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
            Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
            tab->attach(*edit, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
            m_usernameFields.push_back(edit);
            if(m_editing)
            {
                if(i+1 == splits.size())
                {
                    edit->set_text(username);
                }
                else
                {
                    Glib::ustring::size_type pos = username.find(splits[i+1].getSeparator());
                    edit->set_text(username.substr(0, pos));
                    username = username.substr(pos+1);
                }
                if(m_account.isEnabled()) edit->set_sensitive(false);
            }
        }
        if(!m_editing && m_gtalk)
        {
            m_usernameFields[1]->set_text("gmail.com");
        }
    }
    else
    {
        tab->attach(*Gtk::manage(new Gtk::Label(_("User name"), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
        tab->attach(*edit, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        m_usernameFields.push_back(edit);
        if(m_editing)
        {
            edit->set_text(username);
            if(m_account.isEnabled()) edit->set_sensitive(false);
        }
        row++;
    }
    tab->attach(*Gtk::manage(new Gtk::Label(_("Password"), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    tab->attach(m_password, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_password.set_visibility(false);
    m_password.set_invisible_char('*');
    if(m_editing) m_password.set_text(m_account.getPassword());
    row++;
    tab->attach(m_rememberPassword, 0, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 0);
    if(!m_editing || m_account.getRememberPassword()) m_rememberPassword.set_active();
    row++;
    tab->attach(*Gtk::manage(new Gtk::Label(_("Alias"), Gtk::ALIGN_START)), 0, 1, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    tab->attach(m_alias, 1, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    if(m_editing) m_alias.set_text(m_account.getAlias());
    row++;
    tab->attach(m_checkEmail, 0, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 0);
    if(m_editing && m_account.getCheckMail()) m_checkEmail.set_active();
    row++;
    tab->attach(m_iconBox, 0, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 0);
    row++;
    m_iconBox.add(*Gtk::manage(new Gtk::Label(_("Icon"), Gtk::ALIGN_START)));
    Gtk::HBox *btnBox = Gtk::manage(new Gtk::HBox());
    m_iconImage.setIcon(dxutils::createIcon(m_editing?m_account.getBuddyIcon():"", MAX_ICON_SIZE));
    if(!m_editing || m_account.getBuddyIcon().empty()) m_iconImage.setText(_("Icon isn't set"));
    m_iconBtn.add(m_iconImage);
    m_iconBtn.signal_clicked().connect(sigc::mem_fun(this, &AccountEditor::onSetIcon));
    btnBox->pack_start(m_iconBtn, Gtk::PACK_SHRINK, 3);
    Gtk::VBox *removeBox = Gtk::manage(new Gtk::VBox());
    m_iconRemove.set_label(_("Remove"));
    m_iconRemove.signal_clicked().connect(sigc::mem_fun(this, &AccountEditor::onRemoveIcon));
    removeBox->pack_start(m_iconRemove, Gtk::PACK_SHRINK, 3);
    btnBox->pack_start(*removeBox, Gtk::PACK_SHRINK, 3);
    m_iconBox.add(*btnBox);
    tab->attach(m_registerAccount, 0, 2, row, row+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 0);
    show_all();
    m_tabbook.append_page(*tab, _("Account settings"));
}

void AccountEditor::createProtocolTab()
{
    Gtk::Table *tab = Gtk::manage(new Gtk::Table());
    for(size_t i=0; i<m_options.size(); i++)
    {
        PURPLE::Protocol::Option option = m_options[i];
        std::map<std::string, std::string> values = option.getStringList();
        tab->attach(*Gtk::manage(new Gtk::Label(option.getText(), Gtk::ALIGN_START)), 0, 1, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
                edit->set_text(option.getStringValue());
                tab->attach(*edit, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(edit);
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                struct Columns : public Gtk::TreeModel::ColumnRecord
                {
                    Gtk::TreeModelColumn<Glib::ustring> m_value;

                    Columns() { add(m_value); }
                };
                const Columns columns;
                Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
                Gtk::ComboBox *combo = Gtk::manage(new Gtk::ComboBox());
                combo->set_model(model);
                combo->pack_start(columns.m_value);
                std::map<std::string, std::string>::iterator value = values.begin();
                int j=0;
                for(; value!=values.end(); ++value, j++)
                {
                    Gtk::TreeModel::Row row = *(model->append());
                    row[columns.m_value] = (*value).first;
                    if(!option.getStringValue().compare((*value).second))
                        combo->set_active(j);
                }
                tab->attach(*combo, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(combo);
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
                edit->set_text(option.getStringValue());
                tab->attach(*edit, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(edit);
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                Gtk::CheckButton *check = Gtk::manage(new Gtk::CheckButton());
                check->set_active(option.getBoolValue());
                tab->attach(*check, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(check);
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                m_optionsFields.push_back(NULL);
                break;
        }
    }
    show_all();
    m_tabbook.append_page(*tab, _("Protocol options"));
}

/* Fills new Account */
void AccountEditor::fillAccount(NewAccount *account)
{
    account->protocol = m_protocol;
    account->username = joinUsername();
    account->password = m_password.get_text();
    account->registerAccount = m_registerAccount.get_active();
    account->checkEmail = m_checkEmail.get_active();
    account->iconPath = m_iconPath;
    account->rememberPassword = m_rememberPassword.get_active();
    account->alias = m_alias.get_text();
    fillProtocolOptions();
    account->options = m_options;
}

std::string AccountEditor::getStrlistValue(std::map<std::string, std::string> map, std::string key)
{
    std::string result;
    std::map<std::string, std::string>::iterator it = map.find(key);
    if(it!=map.end())
        result = (*it).second;
    return result;
}

std::string AccountEditor::joinUsername()
{
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    std::string username = m_usernameFields[0]->get_text();
    if(splits.size())
    {
        for(size_t i=0; i<splits.size(); i++)
        {
            username += splits[i].getSeparator();
            username += m_usernameFields[i+1]->get_text();
        }
    }
    return username;
}

void AccountEditor::fillProtocolOptions()
{
    PURPLE::Protocol::Options::iterator it;
    guint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Protocol::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                option.setValue(static_cast<Gtk::Entry*>(m_optionsFields[i])->get_text());
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                Gtk::ComboBox *combo = static_cast<Gtk::ComboBox*>(m_optionsFields[i]);
                Gtk::TreeModel::iterator iter = combo->get_active();
                if(iter)
                {
                    Gtk::TreeModel::Row row = *iter;
                    if(row)
                    {
                        Glib::ustring value;
                        row.get_value(0, value);
                        option.setValue(getStrlistValue(option.getStringList(), value));
                    }
                    else
                        option.setValue(getStrlistValue(option.getStringList(), ""));
                }
                else
                    option.setValue(getStrlistValue(option.getStringList(), ""));
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                option.setValue(static_cast<Gtk::Entry*>(m_optionsFields[i])->get_text());
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                option.setValue(static_cast<Gtk::CheckButton*>(m_optionsFields[i])->get_active() ? "true" : "false");
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

void AccountEditor::onSetIcon()
{
#if GTKMM_MAJOR_VERSION == 2
    Gtk::FileFilter imageFilter;
 	imageFilter.set_name(_("Images"));
 	imageFilter.add_pixbuf_formats();
#else
    Glib::RefPtr<Gtk::FileFilter> imageFilter = Gtk::FileFilter::create();
    imageFilter->set_name(_("Images"));
    imageFilter->add_pixbuf_formats();
#endif    
    Gtk::FileChooserDialog dialog(_("Select buddy icon"), Gtk::FILE_CHOOSER_ACTION_OPEN);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
    dialog.set_transient_for(*this);
    dialog.add_filter(imageFilter);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        m_iconImage.setIcon(dxutils::createIcon(dialog.get_filename(), MAX_ICON_SIZE));
        m_iconImage.setText("");
        m_iconPath = dialog.get_filename();
        m_iconRemove.show();
    }
}

void AccountEditor::onRemoveIcon()
{
    m_iconImage.setIcon(Glib::RefPtr<Gdk::Pixbuf>());
    m_iconImage.setText(_("Icon isn't set"));
    m_iconPath = "";
    m_iconRemove.hide();
}


