/*
 *      configdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <string>
#include <fstream>
#include "configdialog.h"
#include "preferences.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "icons.h"

#define HEADER "# dxim smiley settings\n# path is relative to file\n# Example: /home/xxx/smile.png=:)\n"

SmileyDialog::SmileyDialog(const Glib::ustring &title, const Glib::ustring &smiley, const Glib::ustring &path, Gtk::Window& parent)
: Gtk::Dialog(title, parent)
{
    Gtk::Table *table = Gtk::manage(new Gtk::Table());

    table->attach(*Gtk::manage(new Gtk::Label(_("Smiley:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_smiley = new Gtk::Entry();
    m_smiley->set_text(smiley);
    table->attach(*m_smiley, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 3, 3);

    table->attach(*Gtk::manage(new Gtk::Label(_("Path:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
#if GTKMM_MAJOR_VERSION == 2
    Gtk::FileFilter imageFilter;
    imageFilter.add_pixbuf_formats();
#else
    Glib::RefPtr<Gtk::FileFilter> imageFilter = Gtk::FileFilter::create();
    imageFilter->add_pixbuf_formats();
#endif
    m_path = new Gtk::FileChooserButton(_("Select file"));
    m_path->set_filename(path);
    m_path->set_tooltip_text(path);
    m_path->signal_file_set().connect(sigc::mem_fun(this, &SmileyDialog::onPath));
    m_path->set_filter(imageFilter);
    table->attach(*m_path, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);

    table->attach(*Gtk::manage(new Gtk::Label(_("Preview:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_icon = new Gtk::Image();
    if(!path.empty())
        m_icon->set(path);
    table->attach(*m_icon, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);

    get_vbox()->pack_start(*table, Gtk::PACK_SHRINK);

    add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    set_default_response(Gtk::RESPONSE_OK);
    show_all();
}

void SmileyDialog::onPath()
{
    m_path->set_tooltip_text(m_path->get_filename());
    m_icon->set(m_path->get_filename());
}

ConfigDialog::ConfigDialog(Gtk::Window& parent)
: Gtk::Dialog(_("Preferences"), parent)
{
    m_showImportwarning = true;
    get_vbox()->pack_start(m_tabbook);
    m_tabbook.set_scrollable(true);
    createGeneralTab();
    createIconsTab();
    createLookTab();
    createImtextTab();
    createNotificationTab();
    createSmileysTab();
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    Gtk::Button *okbutton = Gtk::manage(new Gtk::Button(Gtk::Stock::OK));
    okbutton->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onOK));
    Gtk::Button *closebutton = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebutton->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*okbutton, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebutton, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    show_all();
}

ConfigDialog::~ConfigDialog()
{
}

void ConfigDialog::onOK()
{
    //GeneralTab
    DXPREFERENCES.m_oneInstance = m_oneInstance->get_active();
    DXPREFERENCES.m_useAway = m_useAway->get_active();
    DXPREFERENCES.m_awayTimeout = m_awayTimeout->get_value_as_int();
    DXPREFERENCES.m_minToTray = m_minToTray->get_active();
    DXPREFERENCES.m_logging = m_logging->get_active();
    DXPREFERENCES.m_logPath = m_logPath->get_filename();
    DXPREFERENCES.m_logLoadCurrent = m_logLoadCurrent->get_active();
    DXPREFERENCES.m_showRosterTree = m_showRosterTree->get_active();
    DXPREFERENCES.m_receiptsMark = m_receiptsMark->get_active();
    //IconsTab
    DXPREFERENCES.m_themeStatusPath = (*(m_themesStatusList->get_selection()->get_selected())).get_value(m_pathColumns.m_path);
    Glib::ustring statusList;
    Gtk::TreeModel::Children children = m_themesStatusModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        statusList.append((*iter).get_value(m_pathColumns.m_path)+";");
    DXPREFERENCES.m_themesStatusList = statusList;
    DXPREFERENCES.m_themeRolePath = (*(m_themesRoleList->get_selection()->get_selected())).get_value(m_pathColumns.m_path);
    Glib::ustring roleList;
    children = m_themesRoleModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        roleList.append((*iter).get_value(m_pathColumns.m_path)+";");
    DXPREFERENCES.m_themesRoleList = roleList;
    //LookTab
#if GTKMM_MAJOR_VERSION == 2
    DXPREFERENCES.m_unreadtabColor = dxColor(m_unreadtabColor->get_color());
    DXPREFERENCES.m_hltabColor = dxColor(m_hltabColor->get_color());
#else
    DXPREFERENCES.m_unreadtabColor = dxColor(m_unreadtabColor->get_rgba());
    DXPREFERENCES.m_hltabColor = dxColor(m_hltabColor->get_rgba());
#endif
    Gtk::TreeModel::iterator iter = m_tabPosition->get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
             DXPREFERENCES.m_tabPosition = row[m_tabColumns.m_position];
        }
    }
    //ImTextTab
#if GTKMM_MAJOR_VERSION == 2
    DXPREFERENCES.m_textColor = dxColor(m_textColor->get_color());
    DXPREFERENCES.m_backroundColor = dxColor(m_backroundColor->get_color());
    DXPREFERENCES.m_userColor = dxColor(m_userColor->get_color());
    DXPREFERENCES.m_actionColor = dxColor(m_actionColor->get_color());
    DXPREFERENCES.m_errorColor = dxColor(m_errorColor->get_color());
    DXPREFERENCES.m_hilightColor = dxColor(m_hilightColor->get_color());
    DXPREFERENCES.m_linkColor = dxColor(m_linkColor->get_color());
    DXPREFERENCES.m_mymsgColor = dxColor(m_mymsgColor->get_color());
    DXPREFERENCES.m_logColor = dxColor(m_logColor->get_color());
#else
    DXPREFERENCES.m_textColor = dxColor(m_textColor->get_rgba());
    DXPREFERENCES.m_backroundColor = dxColor(m_backroundColor->get_rgba());
    DXPREFERENCES.m_userColor = dxColor(m_userColor->get_rgba());
    DXPREFERENCES.m_actionColor = dxColor(m_actionColor->get_rgba());
    DXPREFERENCES.m_errorColor = dxColor(m_errorColor->get_rgba());
    DXPREFERENCES.m_hilightColor = dxColor(m_hilightColor->get_rgba());
    DXPREFERENCES.m_linkColor = dxColor(m_linkColor->get_rgba());
    DXPREFERENCES.m_mymsgColor = dxColor(m_mymsgColor->get_rgba());
    DXPREFERENCES.m_logColor = dxColor(m_logColor->get_rgba());
#endif
    DXPREFERENCES.m_imFont = m_imFont->get_font_name();
    DXPREFERENCES.m_sameCmd = m_sameCmd->get_active();
    DXPREFERENCES.m_sameList = m_sameList->get_active();
    DXPREFERENCES.m_nickCompletionChar = m_nickCompletionChar->get_text().substr(0,1);
    DXPREFERENCES.m_coloredNick = m_coloredNick->get_active();
    //NotificationTab
    DXPREFERENCES.m_sounds = m_sounds->get_active();
    DXPREFERENCES.m_soundConnect = m_soundConnect->get_active();
    DXPREFERENCES.m_pathConnect = m_pathConnect->get_filename();
    DXPREFERENCES.m_soundDisconnect = m_soundDisconnect->get_active();
    DXPREFERENCES.m_pathDisconnect = m_pathDisconnect->get_filename();
    DXPREFERENCES.m_soundMessage = m_soundMessage->get_active();
    DXPREFERENCES.m_pathMessage = m_pathMessage->get_filename();
    DXPREFERENCES.m_notify = m_notify->get_active();
    if(!dxutils::instance().haveNotify())
    {
        iter = m_notifyPosition->get_active();
        if(iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row)
            {
                 DXPREFERENCES.m_notifyPosition = row[m_notifyColumns.m_position];
            }
        }
    }
    DXPREFERENCES.m_notifyAccountConnect = m_notifyAccountConnect->get_active();
    DXPREFERENCES.m_notifyAccountDisconnect = m_notifyAccountDisconnect->get_active();
    DXPREFERENCES.m_notifyBuddyConnect = m_notifyBuddyConnect->get_active();
    DXPREFERENCES.m_notifyBuddyDisconnect = m_notifyBuddyDisconnect->get_active();
    DXPREFERENCES.m_notifyMessage = m_notifyMessage->get_active();
    DXPREFERENCES.m_notifyFiletransfer = m_notifyFiletransfer->get_active();
    DXPREFERENCES.m_raiseWindow = m_raiseWindow->get_active();
    DXPREFERENCES.m_notifyMessageTray = m_notifyMessageTray->get_active();
    //SmileysTab
    DXPREFERENCES.m_useSmileys = m_useSmileys->get_active();
    DXPREFERENCES.m_smileysMap.clear();
    children = m_smileysModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        Gtk::TreeModel::Row row = *iter;
        Glib::ustring key = row[m_smileysColumns.m_smiley];
        Glib::ustring value = row[m_smileysColumns.m_path];
        DXPREFERENCES.m_smileysMap.insert(dxStringPair(key, value));
    }
    DXPREFERENCES.save();
    response(Gtk::RESPONSE_OK);
    hide();
}

void ConfigDialog::createGeneralTab()
{
    Gtk::VBox *tab = Gtk::manage(new Gtk::VBox());

    m_oneInstance = new Gtk::CheckButton(_("Run one instance"));
    m_oneInstance->set_active(DXPREFERENCES.m_oneInstance);
    tab->pack_start(*m_oneInstance, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *abox = Gtk::manage(new Gtk::HBox());
    m_useAway = new Gtk::CheckButton(_("Time for status \"Away\" in minutes"));
    m_useAway->set_active(DXPREFERENCES.m_useAway);
    m_useAway->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onUseAway));
    abox->pack_start(*m_useAway, Gtk::PACK_SHRINK, 3);
    m_awayTimeout = new Gtk::SpinButton();
    m_awayTimeout->set_range(1, 500);
    m_awayTimeout->set_increments(1, 5);
    m_awayTimeout->set_value(DXPREFERENCES.m_awayTimeout);
    abox->pack_start(*m_awayTimeout, Gtk::PACK_SHRINK, 3);
    tab->pack_start(*abox, Gtk::PACK_SHRINK);

    m_minToTray = new Gtk::CheckButton(_("Start application minimized to tray"));
    m_minToTray->set_active(DXPREFERENCES.m_minToTray);
    tab->pack_start(*m_minToTray, Gtk::PACK_SHRINK, 3);

    m_logging = new Gtk::CheckButton(_("Logging chats"));
    m_logging->set_active(DXPREFERENCES.m_logging);
    m_logging->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onLogging));
    tab->pack_start(*m_logging, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *logbox = Gtk::manage(new Gtk::HBox());
    logbox->pack_start(*(new Gtk::Label(_("Log path"))), Gtk::PACK_SHRINK, 3);
    m_logPath = new Gtk::FileChooserButton(Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
    if(Glib::file_test(DXPREFERENCES.m_logPath, Glib::FILE_TEST_IS_DIR))
    {
        m_logPath->set_filename(DXPREFERENCES.m_logPath);
        m_logPath->set_tooltip_text(DXPREFERENCES.m_logPath);
    }
    else
    {
        m_logPath->set_filename(Glib::get_home_dir());
        m_logPath->set_tooltip_text(Glib::get_home_dir());
    }
    m_logPath->set_title(_("Select log directory"));
    m_logPath->signal_file_set().connect(sigc::mem_fun(this, &ConfigDialog::onSetLogPath));
    logbox->pack_start(*m_logPath, Gtk::PACK_SHRINK, 3);
    tab->pack_start(*logbox, Gtk::PACK_SHRINK);
    
    m_logLoadCurrent = new Gtk::CheckButton(_("Load current log for opened IM"));
    m_logLoadCurrent->set_active(DXPREFERENCES.m_logging?DXPREFERENCES.m_logLoadCurrent:false);
    tab->pack_start(*m_logLoadCurrent, Gtk::PACK_SHRINK, 3);

    m_showRosterTree = new Gtk::CheckButton(_("Show accounts and groups in friends list"));
    m_showRosterTree->set_active(DXPREFERENCES.m_showRosterTree);
    tab->pack_start(*m_showRosterTree, Gtk::PACK_SHRINK, 3);

    m_receiptsMark = new Gtk::CheckButton(_("Use delivery receipts mark on XMPP protocol"));
    m_receiptsMark->set_active(DXPREFERENCES.m_receiptsMark);
    tab->pack_start(*m_receiptsMark, Gtk::PACK_SHRINK, 3);

    show_all();
    m_tabbook.append_page(*tab, _("General"));
}

void ConfigDialog::onUseAway()
{
    if(m_useAway->get_active())
    {
        m_awayTimeout->set_sensitive(true);
    }
    else
    {
        m_awayTimeout->set_sensitive(false);
    }
}

void ConfigDialog::onLogging()
{
    if(m_logging->get_active())
    {
        m_logPath->set_sensitive(true);
        m_logLoadCurrent->set_sensitive(true);
    }
    else
    {
        m_logPath->set_sensitive(false);
        m_logLoadCurrent->set_sensitive(false);
        m_logLoadCurrent->set_active(false);
    }
}

void ConfigDialog::onSetLogPath()
{
    m_logPath->set_tooltip_text(m_logPath->get_filename());
}

void ConfigDialog::createIconsTab()
{
    Gtk::VBox *tab = Gtk::manage(new Gtk::VBox());

    Gtk::Frame *statusFrame = Gtk::manage(new Gtk::Frame(_("Status icons themes")));
    Gtk::HBox *statusBox = Gtk::manage(new Gtk::HBox());
    statusFrame->add(*statusBox);
    Gtk::VBox *statusListBox = Gtk::manage(new Gtk::VBox());
    statusBox->pack_start(*statusListBox, Gtk::PACK_EXPAND_WIDGET);
    m_themesStatusModel = Gtk::ListStore::create(m_pathColumns);
    m_themesStatusList = new Gtk::TreeView();
    m_themesStatusList->set_model(m_themesStatusModel);
    m_themesStatusList->append_column("Path", m_pathColumns.m_path);
    m_themesStatusList->set_headers_visible(false);
    std::vector<Glib::ustring> statusList = dxutils::stringSplit(DXPREFERENCES.m_themesStatusList, ';');
    FOREACH(std::vector<Glib::ustring>, statusList, it)
    {
        Gtk::TreeModel::Row row = *(m_themesStatusModel->append());
        row[m_pathColumns.m_path] = *(it);
        if(!DXPREFERENCES.m_themeStatusPath.compare(*(it)))
            m_themesStatusList->get_selection()->select(row);
    }
    Glib::RefPtr<Gtk::TreeSelection> themesStatusSelectionModel = m_themesStatusList->get_selection();
    themesStatusSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
    themesStatusSelectionModel->signal_changed().connect(sigc::mem_fun(this, &ConfigDialog::onStatusThemeChanged));
    Gtk::ScrolledWindow *winStatus = Gtk::manage(new Gtk::ScrolledWindow());
    winStatus->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    winStatus->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    winStatus->add(*m_themesStatusList);
    statusListBox->pack_start(*winStatus, Gtk::PACK_EXPAND_WIDGET, 3);

    Gtk::HBox *statusIconsBox = Gtk::manage(new Gtk::HBox());
    m_iconAway = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"away.png");
    m_iconAway->set_tooltip_text(_("Away"));
    statusIconsBox->pack_start(*m_iconAway, Gtk::PACK_SHRINK, 3);
    m_iconIdle = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"idle.png");
    m_iconIdle->set_tooltip_text(_("Idle"));
    statusIconsBox->pack_start(*m_iconIdle, Gtk::PACK_SHRINK, 3);
    m_iconDnd = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"dnd.png");
    m_iconDnd->set_tooltip_text(_("Busy"));
    statusIconsBox->pack_start(*m_iconDnd, Gtk::PACK_SHRINK, 3);
    m_iconInvisible = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"invisible.png");
    m_iconInvisible->set_tooltip_text(_("Invisible"));
    statusIconsBox->pack_start(*m_iconInvisible, Gtk::PACK_SHRINK, 3);
    m_iconOffline = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"offline.png");
    m_iconOffline->set_tooltip_text(_("Offline"));
    statusIconsBox->pack_start(*m_iconOffline, Gtk::PACK_SHRINK, 3);
    m_iconOnline = new Gtk::Image(DXPREFERENCES.m_themeStatusPath+G_DIR_SEPARATOR_S+"online.png");
    m_iconOnline->set_tooltip_text(_("Online"));
    statusIconsBox->pack_start(*m_iconOnline, Gtk::PACK_SHRINK, 3);
    statusListBox->pack_start(*statusIconsBox, Gtk::PACK_SHRINK);

    Gtk::VBox *statusBtnBox = Gtk::manage(new Gtk::VBox());
    statusBox->pack_start(*statusBtnBox, Gtk::PACK_SHRINK);
    m_themeStatusAdd = new Gtk::Button(_("Add"));
    m_themeStatusAdd->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onAddStatusTheme));
    statusBtnBox->pack_start(*m_themeStatusAdd, Gtk::PACK_SHRINK, 3);
    m_themeStatusDelete = new Gtk::Button(_("Delete"));
    m_themeStatusDelete->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onDeleteStatusTheme));
    if(statusList.size()<2) m_themeStatusDelete->set_sensitive(false);
    statusBtnBox->pack_start(*m_themeStatusDelete, Gtk::PACK_SHRINK, 3);
    tab->pack_start(*statusFrame, Gtk::PACK_EXPAND_WIDGET);

    Gtk::Frame *roleFrame = Gtk::manage(new Gtk::Frame(_("Role icons themes")));
    Gtk::HBox *roleBox = Gtk::manage(new Gtk::HBox());
    roleFrame->add(*roleBox);
    Gtk::VBox *roleListBox = Gtk::manage(new Gtk::VBox());
    roleBox->pack_start(*roleListBox, Gtk::PACK_EXPAND_WIDGET);
    m_themesRoleModel = Gtk::ListStore::create(m_pathColumns);
    m_themesRoleList = new Gtk::TreeView();
    m_themesRoleList->set_model(m_themesRoleModel);
    m_themesRoleList->append_column("Path", m_pathColumns.m_path);
    m_themesRoleList->set_headers_visible(false);
    std::vector<Glib::ustring> roleList = dxutils::stringSplit(DXPREFERENCES.m_themesRoleList, ';');
    FOREACH(std::vector<Glib::ustring>, roleList, it)
    {
        Gtk::TreeModel::Row row = *(m_themesRoleModel->append());
        row[m_pathColumns.m_path] = *(it);
        if(!DXPREFERENCES.m_themeRolePath.compare(*(it)))
            m_themesRoleList->get_selection()->select(row);
    }
    Glib::RefPtr<Gtk::TreeSelection> themesRoleSelectionModel = m_themesRoleList->get_selection();
    themesRoleSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
    themesRoleSelectionModel->signal_changed().connect(sigc::mem_fun(this, &ConfigDialog::onRoleThemeChanged));
    Gtk::ScrolledWindow *winRole = Gtk::manage(new Gtk::ScrolledWindow());
    winRole->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    winRole->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    winRole->add(*m_themesRoleList);
    roleListBox->pack_start(*winRole, Gtk::PACK_EXPAND_WIDGET, 3);

    Gtk::HBox *roleIconsBox = Gtk::manage(new Gtk::HBox());
    m_iconFounder = new Gtk::Image(DXPREFERENCES.m_themeRolePath+G_DIR_SEPARATOR_S+"founder.png");
    m_iconFounder->set_tooltip_text(_("Founder"));
    roleIconsBox->pack_start(*m_iconFounder, Gtk::PACK_SHRINK, 3);
    m_iconHalfop = new Gtk::Image(DXPREFERENCES.m_themeRolePath+G_DIR_SEPARATOR_S+"halfop.png");
    m_iconHalfop->set_tooltip_text(_("Halfop"));
    roleIconsBox->pack_start(*m_iconHalfop, Gtk::PACK_SHRINK, 3);
    m_iconNormal = new Gtk::Image(DXPREFERENCES.m_themeRolePath+G_DIR_SEPARATOR_S+"normal.png");
    m_iconNormal->set_tooltip_text(_("Normal"));
    roleIconsBox->pack_start(*m_iconNormal, Gtk::PACK_SHRINK, 3);
    m_iconOp = new Gtk::Image(DXPREFERENCES.m_themeRolePath+G_DIR_SEPARATOR_S+"op.png");
    m_iconOp->set_tooltip_text(_("Op"));
    roleIconsBox->pack_start(*m_iconOp, Gtk::PACK_SHRINK, 3);
    m_iconVoice = new Gtk::Image(DXPREFERENCES.m_themeRolePath+G_DIR_SEPARATOR_S+"voice.png");
    m_iconVoice->set_tooltip_text(_("Voice"));
    roleIconsBox->pack_start(*m_iconVoice, Gtk::PACK_SHRINK, 3);
    roleListBox->pack_start(*roleIconsBox, Gtk::PACK_SHRINK);

    Gtk::VBox *roleBtnBox = Gtk::manage(new Gtk::VBox());
    roleBox->pack_start(*roleBtnBox, Gtk::PACK_SHRINK);
    m_themeRoleAdd = new Gtk::Button(_("Add"));
    m_themeRoleAdd->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onAddRoleTheme));
    roleBtnBox->pack_start(*m_themeRoleAdd, Gtk::PACK_SHRINK, 3);
    m_themeRoleDelete = new Gtk::Button(_("Delete"));
    m_themeRoleDelete->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onDeleteRoleTheme));
    if(roleList.size()<2) m_themeRoleDelete->set_sensitive(false);
    roleBtnBox->pack_start(*m_themeRoleDelete, Gtk::PACK_SHRINK, 3);
    tab->pack_start(*roleFrame, Gtk::PACK_EXPAND_WIDGET);

    show_all();
    m_tabbook.append_page(*tab, _("Icons"));
}

void ConfigDialog::onAddStatusTheme()
{
    Gtk::FileChooserDialog dialog(_("Select theme directory"), Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
    dialog.set_transient_for(*this);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK
        && !dialog.get_filename().empty()
        && Glib::file_test(dialog.get_filename()+G_DIR_SEPARATOR_S+"online.png", Glib::FILE_TEST_EXISTS)
        && !Glib::file_test(dialog.get_filename()+G_DIR_SEPARATOR_S+"online.png", Glib::FILE_TEST_IS_DIR))
    {
        Gtk::TreeModel::Row row = *(m_themesStatusModel->append());
        row[m_pathColumns.m_path] = dialog.get_filename();
        if(!m_themeStatusDelete->get_sensitive())
            m_themeStatusDelete->set_sensitive(true);
    }
}

void ConfigDialog::onDeleteStatusTheme()
{
    Gtk::TreeModel::iterator it = m_themesStatusList->get_selection()->get_selected();
    if(it)
    {
        m_themesStatusModel->erase(it);
    }
    m_themesStatusList->get_selection()->select(m_themesStatusModel->children().begin());
    if(m_themesStatusModel->children().size()<2) m_themeStatusDelete->set_sensitive(false);
}

void ConfigDialog::onAddRoleTheme()
{
    Gtk::FileChooserDialog dialog(_("Select theme directory"), Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
    dialog.set_transient_for(*this);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK
        && !dialog.get_filename().empty()
        && Glib::file_test(dialog.get_filename()+G_DIR_SEPARATOR_S+"normal.png", Glib::FILE_TEST_EXISTS)
        && !Glib::file_test(dialog.get_filename()+G_DIR_SEPARATOR_S+"normal.png", Glib::FILE_TEST_IS_DIR))
    {
        Gtk::TreeModel::Row row = *(m_themesRoleModel->append());
        row[m_pathColumns.m_path] = dialog.get_filename();
        if(!m_themeRoleDelete->get_sensitive())
            m_themeRoleDelete->set_sensitive(true);
    }
}

void ConfigDialog::onDeleteRoleTheme()
{
    Gtk::TreeModel::iterator it = m_themesRoleList->get_selection()->get_selected();
    if(it)
    {
        m_themesRoleModel->erase(it);
    }
    m_themesRoleList->get_selection()->select(m_themesRoleModel->children().begin());
    if(m_themesRoleModel->children().size()<2) m_themeRoleDelete->set_sensitive(false);
}

void ConfigDialog::onStatusThemeChanged()
{
    Gtk::TreeModel::iterator it = m_themesStatusList->get_selection()->get_selected();
    if(it)
    {
        Glib::ustring path = (*it).get_value(m_pathColumns.m_path);
        m_iconAway->set(path+G_DIR_SEPARATOR_S+"away.png");
        m_iconIdle->set(path+G_DIR_SEPARATOR_S+"idle.png");
        m_iconDnd->set(path+G_DIR_SEPARATOR_S+"dnd.png");
        m_iconInvisible->set(path+G_DIR_SEPARATOR_S+"invisible.png");
        m_iconOffline->set(path+G_DIR_SEPARATOR_S+"offline.png");
        m_iconOnline->set(path+G_DIR_SEPARATOR_S+"online.png");
    }
}

void ConfigDialog::onRoleThemeChanged()
{
    Gtk::TreeModel::iterator it = m_themesRoleList->get_selection()->get_selected();
    if(it)
    {
        Glib::ustring path = (*it).get_value(m_pathColumns.m_path);
        m_iconFounder->set(path+G_DIR_SEPARATOR_S+"founder.png");
        m_iconHalfop->set(path+G_DIR_SEPARATOR_S+"halfop.png");
        m_iconNormal->set(path+G_DIR_SEPARATOR_S+"normal.png");
        m_iconOp->set(path+G_DIR_SEPARATOR_S+"op.png");
        m_iconVoice->set(path+G_DIR_SEPARATOR_S+"voice.png");
    }
}

void ConfigDialog::createLookTab()
{
    Gtk::HBox *tab = Gtk::manage(new Gtk::HBox());

    Gtk::VBox *setBox = Gtk::manage(new Gtk::VBox());
    Gtk::HBox *unreadBox = Gtk::manage(new Gtk::HBox());
    m_unreadtabColor = new Gtk::ColorButton(DXPREFERENCES.m_unreadtabColor.get_color());
    m_unreadtabColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::onTabColor));
    unreadBox->pack_start(*m_unreadtabColor, Gtk::PACK_SHRINK, 3);
    unreadBox->pack_start(*Gtk::manage(new Gtk::Label(_("Unreaded Tab Color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*unreadBox, Gtk::PACK_SHRINK, 3);
    Gtk::HBox *hlightBox = Gtk::manage(new Gtk::HBox());
    m_hltabColor = new Gtk::ColorButton(DXPREFERENCES.m_hltabColor.get_color());
    m_hltabColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::onTabColor));
    hlightBox->pack_start(*m_hltabColor, Gtk::PACK_SHRINK, 3);
    hlightBox->pack_start(*Gtk::manage(new Gtk::Label(_("Highlighted Tab Color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*hlightBox, Gtk::PACK_SHRINK, 3);
    Gtk::HBox *tabBox = Gtk::manage(new Gtk::HBox());
    m_tabModel = Gtk::ListStore::create(m_tabColumns);
    m_tabPosition = new Gtk::ComboBox;
    m_tabPosition->set_model(m_tabModel);
    m_tabPosition->pack_start(m_tabColumns.m_name);
    m_tabPosition->signal_changed().connect(sigc::mem_fun(this, &ConfigDialog::updateTabPosition));
    Gtk::TreeModel::Row row = *(m_tabModel->append());
    row[m_tabColumns.m_name] = _("Bottom");
    row[m_tabColumns.m_position] = 0;
    row = *(m_tabModel->append());
    row[m_tabColumns.m_name] = _("Left");
    row[m_tabColumns.m_position] = 1;
    row = *(m_tabModel->append());
    row[m_tabColumns.m_name] = _("Top");
    row[m_tabColumns.m_position] = 2;
    row = *(m_tabModel->append());
    row[m_tabColumns.m_name] = _("Right");
    row[m_tabColumns.m_position] = 3;
    tabBox->pack_start(*Gtk::manage(new Gtk::Label(_("Tab position"))), Gtk::PACK_SHRINK, 3);
    tabBox->pack_start(*m_tabPosition, Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*tabBox, Gtk::PACK_SHRINK);
    tab->pack_start(*setBox, Gtk::PACK_SHRINK);

    m_lookTabs = new Gtk::Notebook();
    m_lookTabs->append_page(*Gtk::manage(new Gtk::Label(_("Standard tab"))), *Gtk::manage(new ImageLabel(ICO_CHAT->get_pixbuf(), "linux")));
    m_unreadtabLabel = new ImageLabel(ICO_CHAT->get_pixbuf(), "bsd");
    m_unreadtabLabel->setColor(DXPREFERENCES.m_unreadtabColor);
    m_lookTabs->append_page(*Gtk::manage(new Gtk::Label(_("Unreaded tab"))), *m_unreadtabLabel);
    m_hltabLabel = new ImageLabel(ICO_CHATNEWMSG->get_pixbuf(), "hurd");
    m_hltabLabel->setColor(DXPREFERENCES.m_hltabColor);
    m_lookTabs->append_page(*Gtk::manage(new Gtk::Label(_("Highlighted tab"))), *m_hltabLabel);
    if(DXPREFERENCES.m_tabPosition>=0 && DXPREFERENCES.m_tabPosition<4)
        m_tabPosition->set_active(DXPREFERENCES.m_tabPosition);
    else
        m_tabPosition->set_active(0);
    tab->pack_start(*m_lookTabs, Gtk::PACK_EXPAND_WIDGET);

    show_all();
    m_tabbook.append_page(*tab, _("Look"));
}

void ConfigDialog::updateTabPosition()
{
    Gtk::TreeModel::iterator iter = m_tabPosition->get_active();
    if(iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row)
        {
            m_lookTabs->set_tab_pos(dxutils::getTabPosition(row[m_tabColumns.m_position]));
        }
    }
}

void ConfigDialog::onTabColor()
{
#if GTKMM_MAJOR_VERSION == 2
    m_unreadtabLabel->setColor(dxColor(m_unreadtabColor->get_color()));
    m_hltabLabel->setColor(dxColor(m_hltabColor->get_color()));
#else
    m_unreadtabLabel->setColor(dxColor(m_unreadtabColor->get_rgba()));
    m_hltabLabel->setColor(dxColor(m_hltabColor->get_rgba()));
#endif
}

void ConfigDialog::createImtextTab()
{
    Gtk::HBox *tab = Gtk::manage(new Gtk::HBox());

    Gtk::VBox *setBox = Gtk::manage(new Gtk::VBox());
    Gtk::HBox *textBox = Gtk::manage(new Gtk::HBox());
    m_textColor = new Gtk::ColorButton(DXPREFERENCES.m_textColor.get_color());
    m_textColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    textBox->pack_start(*m_textColor, Gtk::PACK_SHRINK, 3);
    textBox->pack_start(*Gtk::manage(new Gtk::Label(_("Text color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*textBox, Gtk::PACK_SHRINK);
    Gtk::HBox *backroundBox = Gtk::manage(new Gtk::HBox());
    m_backroundColor = new Gtk::ColorButton(DXPREFERENCES.m_backroundColor.get_color());
    m_backroundColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    backroundBox->pack_start(*m_backroundColor, Gtk::PACK_SHRINK, 3);
    backroundBox->pack_start(*Gtk::manage(new Gtk::Label(_("Text backround color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*backroundBox, Gtk::PACK_SHRINK);
    Gtk::HBox *userBox = Gtk::manage(new Gtk::HBox());
    m_userColor = new Gtk::ColorButton(DXPREFERENCES.m_userColor.get_color());
    m_userColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    userBox->pack_start(*m_userColor, Gtk::PACK_SHRINK, 3);
    userBox->pack_start(*Gtk::manage(new Gtk::Label(_("User events text color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*userBox, Gtk::PACK_SHRINK);
    Gtk::HBox *actionBox = Gtk::manage(new Gtk::HBox());
    m_actionColor = new Gtk::ColorButton(DXPREFERENCES.m_actionColor.get_color());
    m_actionColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    actionBox->pack_start(*m_actionColor, Gtk::PACK_SHRINK, 3);
    actionBox->pack_start(*Gtk::manage(new Gtk::Label(_("Message in 3rd person text color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*actionBox, Gtk::PACK_SHRINK);
    Gtk::HBox *errorBox = Gtk::manage(new Gtk::HBox());
    m_errorColor = new Gtk::ColorButton(DXPREFERENCES.m_errorColor.get_color());
    m_errorColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    errorBox->pack_start(*m_errorColor, Gtk::PACK_SHRINK, 3);
    errorBox->pack_start(*Gtk::manage(new Gtk::Label(_("Error text color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*errorBox, Gtk::PACK_SHRINK);
    Gtk::HBox *hlBox = Gtk::manage(new Gtk::HBox());
    m_hilightColor = new Gtk::ColorButton(DXPREFERENCES.m_hilightColor.get_color());
    m_hilightColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    hlBox->pack_start(*m_hilightColor, Gtk::PACK_SHRINK, 3);
    hlBox->pack_start(*Gtk::manage(new Gtk::Label(_("Highlight message text color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*hlBox, Gtk::PACK_SHRINK);
    Gtk::HBox *linkBox = Gtk::manage(new Gtk::HBox());
    m_linkColor = new Gtk::ColorButton(DXPREFERENCES.m_linkColor.get_color());
    m_linkColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    linkBox->pack_start(*m_linkColor, Gtk::PACK_SHRINK, 3);
    linkBox->pack_start(*Gtk::manage(new Gtk::Label(_("Link color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*linkBox, Gtk::PACK_SHRINK);
    Gtk::HBox *myBox = Gtk::manage(new Gtk::HBox());
    m_mymsgColor = new Gtk::ColorButton(DXPREFERENCES.m_mymsgColor.get_color());
    m_mymsgColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    myBox->pack_start(*m_mymsgColor, Gtk::PACK_SHRINK, 3);
    myBox->pack_start(*Gtk::manage(new Gtk::Label(_("My message color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*myBox, Gtk::PACK_SHRINK);
    Gtk::HBox *logBox = Gtk::manage(new Gtk::HBox());
    m_logColor = new Gtk::ColorButton(DXPREFERENCES.m_logColor.get_color());
    m_logColor->signal_color_set().connect(sigc::mem_fun(this, &ConfigDialog::updateIMText));
    logBox->pack_start(*m_logColor, Gtk::PACK_SHRINK, 3);
    logBox->pack_start(*Gtk::manage(new Gtk::Label(_("History line color"))), Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*logBox, Gtk::PACK_SHRINK);

    m_imFont = new Gtk::FontButton(DXPREFERENCES.m_imFont);
    m_imFont->signal_font_set().connect(sigc::mem_fun(this, &ConfigDialog::onImFont));
    setBox->pack_start(*m_imFont, Gtk::PACK_SHRINK, 3);
    m_sameCmd = new Gtk::CheckButton(_("Use same font for commandline"));
    m_sameCmd->set_active(DXPREFERENCES.m_sameCmd);
    setBox->pack_start(*m_sameCmd, Gtk::PACK_SHRINK, 3);
    m_sameList = new Gtk::CheckButton(_("Use same font for user list"));
    m_sameList->set_active(DXPREFERENCES.m_sameList);
    setBox->pack_start(*m_sameList, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *nickBox = Gtk::manage(new Gtk::HBox());
    nickBox->pack_start(*Gtk::manage(new Gtk::Label(_("Nick completion char"))), Gtk::PACK_SHRINK, 3);
    m_nickCompletionChar = new Gtk::Entry();
    m_nickCompletionChar->set_text(DXPREFERENCES.m_nickCompletionChar.substr(0,1));
    nickBox->pack_start(*m_nickCompletionChar, Gtk::PACK_SHRINK, 3);
    setBox->pack_start(*nickBox, Gtk::PACK_SHRINK);
    m_coloredNick = new Gtk::CheckButton(_("Use colored nick"));
    m_coloredNick->set_active(DXPREFERENCES.m_coloredNick);
    setBox->pack_start(*m_coloredNick, Gtk::PACK_SHRINK, 3);

    tab->pack_start(*setBox, Gtk::PACK_SHRINK);

    m_imText = new Gtk::TextView;
    m_imText->set_editable(false);
    m_imText->set_can_focus(false);
    m_imText->set_wrap_mode(Gtk::WRAP_NONE);
#if GTKMM_MAJOR_VERSION == 2
    m_imText->modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_imText->modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_imText->modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
#else
    m_imText->override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_imText->override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_imText->override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
#endif
    createText();
    tab->pack_start(*m_imText, Gtk::PACK_EXPAND_WIDGET, 3);

    show_all();
    m_tabbook.append_page(*tab, _("IM Text"));
}

void ConfigDialog::createText()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_imText->get_buffer();
    Glib::RefPtr<Gtk::TextTag> tag;
    tag = buffer->create_tag("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = buffer->create_tag("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = buffer->create_tag("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = buffer->create_tag("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = buffer->create_tag("logs");
    tag->property_foreground().set_value(DXPREFERENCES.m_logColor.to_string());
    tag = buffer->create_tag("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
    tag = buffer->create_tag("link");
    tag->property_foreground().set_value(DXPREFERENCES.m_linkColor.to_string());
    tag->property_underline().set_value(Pango::UNDERLINE_SINGLE);
    buffer->insert(buffer->end(), "[1.1.2000 00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "dvx: msg from history", "logs");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "dvx has joined to dxim", "user");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "dvx: Welcome in dxim", "mymsg");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "bm: dvx, dxim is nice", "highlight");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "dvx is online", "action");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "No error", "error");
    buffer->insert(buffer->end(), "\n");
    buffer->insert(buffer->end(), "[00:00:00] ");
    buffer->insert_with_tag(buffer->end(), "dvx: ", "mymsg");
    buffer->insert_with_tag(buffer->end(), "http://dxsolutions.org", "link");
    buffer->insert(buffer->end(), "\n");
}

void ConfigDialog::updateIMText()
{
#if GTKMM_MAJOR_VERSION == 2
    m_imText->modify_base(Gtk::STATE_NORMAL, m_backroundColor->get_color());
 	m_imText->modify_text(Gtk::STATE_NORMAL, m_textColor->get_color());
#else
    m_imText->override_background_color(m_backroundColor->get_rgba(), Gtk::STATE_FLAG_NORMAL);
    m_imText->override_color(m_textColor->get_rgba(), Gtk::STATE_FLAG_NORMAL);
#endif
    Glib::RefPtr<Gtk::TextTag> tag = m_imText->get_buffer()->get_tag_table()->lookup("user");
    tag->property_foreground().set_value(m_userColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("action");
    tag->property_foreground().set_value(m_actionColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("error");
    tag->property_foreground().set_value(m_errorColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("highlight");
    tag->property_foreground().set_value(m_hilightColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("link");
    tag->property_foreground().set_value(m_linkColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("mymsg");
    tag->property_foreground().set_value(m_mymsgColor->get_color().to_string());
    tag = m_imText->get_buffer()->get_tag_table()->lookup("logs");
    tag->property_foreground().set_value(m_logColor->get_color().to_string());
}

void ConfigDialog::onImFont()
{
#if GTKMM_MAJOR_VERSION == 2
    m_imText->modify_font(Pango::FontDescription(m_imFont->get_font_name()));
#else
    m_imText->override_font(Pango::FontDescription(m_imFont->get_font_name()));
#endif
}

void ConfigDialog::createNotificationTab()
{
    Gtk::VBox *tab = Gtk::manage(new Gtk::VBox());

    m_sounds = new Gtk::CheckButton(_("Use sounds"));
    m_sounds->set_active(DXPREFERENCES.m_sounds);
    m_sounds->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onSounds));
    tab->pack_start(*m_sounds, Gtk::PACK_SHRINK, 3);

    Gtk::Frame *soundsFrame = Gtk::manage(new Gtk::Frame(_("Events")));
#if GTKMM_MAJOR_VERSION == 2
    Gtk::FileFilter wavFilter;
    wavFilter.add_pattern("*.wav");
    wavFilter.add_pattern("*.WAV");
    wavFilter.set_name(_("Sound file (*.wav)"));
#else
    Glib::RefPtr<Gtk::FileFilter> wavFilter = Gtk::FileFilter::create();
    wavFilter->add_pattern("*.wav");
    wavFilter->add_pattern("*.WAV");
    wavFilter->set_name(_("Sound file (*.wav)"));
#endif
    Gtk::VBox *soundsBox = Gtk::manage(new Gtk::VBox());

    Gtk::HBox *connectBox = Gtk::manage(new Gtk::HBox());
    m_soundConnect = new Gtk::CheckButton(_("Friend connected"));
    m_soundConnect->set_active(DXPREFERENCES.m_soundConnect);
    m_soundConnect->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onSounds));
    connectBox->pack_start(*m_soundConnect, Gtk::PACK_SHRINK, 3);
    m_pathConnect = new Gtk::FileChooserButton(_("Select file"));
    m_pathConnect->set_filename(DXPREFERENCES.m_pathConnect);
    m_pathConnect->set_tooltip_text(DXPREFERENCES.m_pathConnect);
    m_pathConnect->set_filter(wavFilter);
    m_pathConnect->signal_file_set().connect(sigc::mem_fun(this, &ConfigDialog::onSoundsPath));
    connectBox->pack_start(*m_pathConnect, Gtk::PACK_EXPAND_WIDGET, 3);
    m_playConnect = new Gtk::Button();
    m_playConnect->set_image(*Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "play.png")));
    m_playConnect->signal_clicked().connect(sigc::bind<int>(sigc::mem_fun(this, &ConfigDialog::onPlay), 1));
    connectBox->pack_start(*m_playConnect, Gtk::PACK_SHRINK, 3);
    soundsBox->pack_start(*connectBox, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *disconnectBox = Gtk::manage(new Gtk::HBox());
    m_soundDisconnect = new Gtk::CheckButton(_("Friend disconnected"));
    m_soundDisconnect->set_active(DXPREFERENCES.m_soundDisconnect);
    m_soundDisconnect->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onSounds));
    disconnectBox->pack_start(*m_soundDisconnect, Gtk::PACK_SHRINK, 3);
    m_pathDisconnect = new Gtk::FileChooserButton(_("Select file"));
    m_pathDisconnect->set_filename(DXPREFERENCES.m_pathDisconnect);
    m_pathDisconnect->set_tooltip_text(DXPREFERENCES.m_pathDisconnect);
    m_pathDisconnect->set_filter(wavFilter);
    m_pathDisconnect->signal_file_set().connect(sigc::mem_fun(this, &ConfigDialog::onSoundsPath));
    disconnectBox->pack_start(*m_pathDisconnect, Gtk::PACK_EXPAND_WIDGET, 3);
    m_playDisconnect = new Gtk::Button();
    m_playDisconnect->set_image(*Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "play.png")));
    m_playDisconnect->signal_clicked().connect(sigc::bind<int>(sigc::mem_fun(this, &ConfigDialog::onPlay), 2));
    disconnectBox->pack_start(*m_playDisconnect, Gtk::PACK_SHRINK, 3);
    soundsBox->pack_start(*disconnectBox, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *messageBox = Gtk::manage(new Gtk::HBox());
    m_soundMessage = new Gtk::CheckButton(_("Highlighted or im message"));
    m_soundMessage->set_active(DXPREFERENCES.m_soundMessage);
    m_soundMessage->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onSounds));
    messageBox->pack_start(*m_soundMessage, Gtk::PACK_SHRINK, 3);
    m_pathMessage = new Gtk::FileChooserButton(_("Select file"));
    m_pathMessage->set_filename(DXPREFERENCES.m_pathMessage);
    m_pathMessage->set_tooltip_text(DXPREFERENCES.m_pathMessage);
    m_pathMessage->set_filter(wavFilter);
    m_pathMessage->signal_file_set().connect(sigc::mem_fun(this, &ConfigDialog::onSoundsPath));
    messageBox->pack_start(*m_pathMessage, Gtk::PACK_EXPAND_WIDGET, 3);
    m_playMessage = new Gtk::Button();
    m_playMessage->set_image(*Gtk::manage(new Gtk::Image(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "play.png")));
    m_playMessage->signal_clicked().connect(sigc::bind<int>(sigc::mem_fun(this, &ConfigDialog::onPlay), 3));
    messageBox->pack_start(*m_playMessage, Gtk::PACK_SHRINK, 3);
    soundsBox->pack_start(*messageBox, Gtk::PACK_SHRINK, 3);

    soundsFrame->add(*soundsBox);
    tab->pack_start(*soundsFrame, Gtk::PACK_SHRINK);
    onSounds();

    Gtk::HBox *showBox = Gtk::manage(new Gtk::HBox());
    m_notify = new Gtk::CheckButton(_("Use notify popup"));
    m_notify->set_active(DXPREFERENCES.m_notify);
    m_notify->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onNotify));
    showBox->pack_start(*m_notify, Gtk::PACK_SHRINK);
    m_showNotify = new Gtk::Button(_("Show"));
    m_showNotify->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onShowNotify));
    showBox->pack_start(*m_showNotify, Gtk::PACK_SHRINK);
    tab->pack_start(*showBox, Gtk::PACK_SHRINK, 3);

    if(!dxutils::instance().haveNotify())
    {
        Gtk::HBox *posBox = Gtk::manage(new Gtk::HBox());
        posBox->pack_start(*Gtk::manage(new Gtk::Label(_("Notify position"))), Gtk::PACK_SHRINK);
        m_notifyModel = Gtk::ListStore::create(m_notifyColumns);
        m_notifyPosition = new Gtk::ComboBox;
        m_notifyPosition->set_model(m_notifyModel);
        m_notifyPosition->pack_start(m_notifyColumns.m_name);
        Gtk::TreeModel::Row row = *(m_notifyModel->append());
        row[m_notifyColumns.m_name] = _("Left Top");
        row[m_notifyColumns.m_position] = 0;
        row = *(m_notifyModel->append());
        row[m_notifyColumns.m_name] = _("Right Top");
        row[m_notifyColumns.m_position] = 1;
        row = *(m_notifyModel->append());
        row[m_notifyColumns.m_name] = _("Left Bottom");
        row[m_notifyColumns.m_position] = 2;
        row = *(m_notifyModel->append());
        row[m_notifyColumns.m_name] = _("Right Bottom");
        row[m_notifyColumns.m_position] = 3;
        if(DXPREFERENCES.m_notifyPosition>=0 && DXPREFERENCES.m_notifyPosition<4)
            m_notifyPosition->set_active(DXPREFERENCES.m_notifyPosition);
        else
            m_notifyPosition->set_active(0);
        posBox->pack_start(*m_notifyPosition, Gtk::PACK_SHRINK);
        tab->pack_start(*posBox, Gtk::PACK_SHRINK, 3);
    }

    Gtk::Frame *notifiesFrame = Gtk::manage(new Gtk::Frame(_("Events")));
    Gtk::VBox *notifiesBox = Gtk::manage(new Gtk::VBox());
    m_notifyAccountConnect = new Gtk::CheckButton(_("Account connected"));
    m_notifyAccountConnect->set_active(DXPREFERENCES.m_notifyAccountConnect);
    notifiesBox->pack_start(*m_notifyAccountConnect, Gtk::PACK_SHRINK, 3);
    m_notifyAccountDisconnect = new Gtk::CheckButton(_("Account disconnected"));
    m_notifyAccountDisconnect->set_active(DXPREFERENCES.m_notifyAccountDisconnect);
    notifiesBox->pack_start(*m_notifyAccountDisconnect, Gtk::PACK_SHRINK, 3);
    m_notifyBuddyConnect = new Gtk::CheckButton(_("Friend connected"));
    m_notifyBuddyConnect->set_active(DXPREFERENCES.m_notifyBuddyConnect);
    notifiesBox->pack_start(*m_notifyBuddyConnect, Gtk::PACK_SHRINK, 3);
    m_notifyBuddyDisconnect = new Gtk::CheckButton(_("Friend disconnected"));
    m_notifyBuddyDisconnect->set_active(DXPREFERENCES.m_notifyBuddyDisconnect);
    notifiesBox->pack_start(*m_notifyBuddyDisconnect, Gtk::PACK_SHRINK, 3);
    m_notifyMessage = new Gtk::CheckButton(_("Highlighted or im message"));
    m_notifyMessage->set_active(DXPREFERENCES.m_notifyMessage);
    notifiesBox->pack_start(*m_notifyMessage, Gtk::PACK_SHRINK, 3);
    m_notifyFiletransfer = new Gtk::CheckButton(_("File transfer finished or canceled"));
    m_notifyFiletransfer->set_active(DXPREFERENCES.m_notifyFiletransfer);
    notifiesBox->pack_start(*m_notifyFiletransfer, Gtk::PACK_SHRINK, 3);
    notifiesFrame->add(*notifiesBox);
    tab->pack_start(*notifiesFrame, Gtk::PACK_SHRINK);

    onNotify();

    m_raiseWindow = new Gtk::CheckButton(_("Show application window on highlighted or im message"));
    m_raiseWindow->set_active(DXPREFERENCES.m_raiseWindow);
    m_raiseWindow->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onRaiseWindow));
    tab->pack_start(*m_raiseWindow, Gtk::PACK_SHRINK, 3);

    m_notifyMessageTray = new Gtk::CheckButton(_("Change tray icon for all messages if application in tray"));
    m_notifyMessageTray->set_active(DXPREFERENCES.m_notifyMessageTray);
    if(DXPREFERENCES.m_raiseWindow) m_notifyMessageTray->set_sensitive(false);
    tab->pack_start(*m_notifyMessageTray, Gtk::PACK_SHRINK, 3);

    show_all();
    m_tabbook.append_page(*tab, _("Notification"));
}

void ConfigDialog::onSounds()
{
    if(m_sounds->get_active())
    {
        m_soundConnect->set_sensitive(true);
        m_soundDisconnect->set_sensitive(true);
        m_soundMessage->set_sensitive(true);
    }
    else
    {
        m_soundConnect->set_sensitive(false);
        m_soundDisconnect->set_sensitive(false);
        m_soundMessage->set_sensitive(false);
    }

    if(m_soundConnect->get_active() && m_sounds->get_active())
    {
        m_pathConnect->set_sensitive(true);
        m_playConnect->set_sensitive(true);
    }
    else
    {
        m_pathConnect->set_sensitive(false);
        m_playConnect->set_sensitive(false);
    }

    if(m_soundDisconnect->get_active() && m_sounds->get_active())
    {
        m_pathDisconnect->set_sensitive(true);
        m_playDisconnect->set_sensitive(true);
    }
    else
    {
        m_pathDisconnect->set_sensitive(false);
        m_playDisconnect->set_sensitive(false);
    }

    if(m_soundMessage->get_active() && m_sounds->get_active())
    {
        m_pathMessage->set_sensitive(true);
        m_playMessage->set_sensitive(true);
    }
    else
    {
        m_pathMessage->set_sensitive(false);
        m_playMessage->set_sensitive(false);
    }
}

void ConfigDialog::onPlay(int sound)
{
    if(sound==1)
        dxutils::playFile(m_pathConnect->get_filename());
    else if(sound==2)
        dxutils::playFile(m_pathDisconnect->get_filename());
    else
        dxutils::playFile(m_pathMessage->get_filename());
}

void ConfigDialog::onSoundsPath()
{
    m_pathConnect->set_tooltip_text(m_pathConnect->get_filename());
    m_pathDisconnect->set_tooltip_text(m_pathDisconnect->get_filename());
    m_pathMessage->set_tooltip_text(m_pathMessage->get_filename());
}

void ConfigDialog::onNotify()
{
    if(m_notify->get_active())
    {
        if(!dxutils::instance().haveNotify())
            m_notifyPosition->set_sensitive(true);
        m_notifyAccountConnect->set_sensitive(true);
        m_notifyAccountDisconnect->set_sensitive(true);
        m_notifyBuddyConnect->set_sensitive(true);
        m_notifyBuddyDisconnect->set_sensitive(true);
        m_notifyMessage->set_sensitive(true);
        m_notifyFiletransfer->set_sensitive(true);
    }
    else
    {
        if(!dxutils::instance().haveNotify())
            m_notifyPosition->set_sensitive(false);
        m_notifyAccountConnect->set_sensitive(false);
        m_notifyAccountDisconnect->set_sensitive(false);
        m_notifyBuddyConnect->set_sensitive(false);
        m_notifyBuddyDisconnect->set_sensitive(false);
        m_notifyMessage->set_sensitive(false);
        m_notifyFiletransfer->set_sensitive(false);
    }
}

void ConfigDialog::onShowNotify()
{
    int position = -1;
    if(!dxutils::instance().haveNotify())
    {
        Gtk::TreeModel::iterator iter = m_notifyPosition->get_active();
        if(iter)
        {
            Gtk::TreeModel::Row row = *iter;
            if(row)
            {
                 position = row[m_notifyColumns.m_position];
            }
        }
    }
    dxutils::instance().showNotify(Glib::ustring::compose(_("%1 is available"), "dvx"), position);
}

void ConfigDialog::createSmileysTab()
{
    Gtk::VBox *tab = Gtk::manage(new Gtk::VBox());

    m_useSmileys = new Gtk::CheckButton(_("Use smileys"));
    m_useSmileys->set_active(DXPREFERENCES.m_useSmileys);
    m_useSmileys->signal_toggled().connect(sigc::mem_fun(this, &ConfigDialog::onSmileys));
    tab->pack_start(*m_useSmileys, Gtk::PACK_SHRINK, 3);

    Gtk::HBox *sbox = Gtk::manage(new Gtk::HBox());

    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_smileysModel = Gtk::ListStore::create(m_smileysColumns);
    dxStringMap smileys = DXPREFERENCES.m_smileysMap;
    dxStringIt it = smileys.begin();
    for(; it!=smileys.end(); ++it)
    {
        Gtk::TreeModel::Row row = *(m_smileysModel->append());
        row[m_smileysColumns.m_icon] = dxutils::createIcon((*it).second);
        row[m_smileysColumns.m_smiley] = (*it).first;
        row[m_smileysColumns.m_path] = (*it).second;
    }
    m_smileysList = new Gtk::TreeView();
    m_smileysList->set_model(m_smileysModel);
    Gtk::TreeView::Column *sColumn = Gtk::manage(new Gtk::TreeView::Column());
    sColumn->pack_start(m_smileysColumns.m_icon, false);
    sColumn->pack_start(m_smileysColumns.m_smiley);
    m_smileysList->append_column(*sColumn);
    m_smileysList->set_headers_visible(false);
    m_smileysList->get_selection()->set_mode(Gtk::SELECTION_SINGLE);
    m_smileysList->get_selection()->select(m_smileysModel->children().begin());
    m_smileysList->signal_row_activated().connect(sigc::mem_fun(this, &ConfigDialog::onSmileysDoubleClick));
    swin->add(*m_smileysList);
    sbox->pack_start(*swin, Gtk::PACK_EXPAND_WIDGET);

    Gtk::VBox *bBox = Gtk::manage(new Gtk::VBox());
    m_addSmiley = new Gtk::Button(_("Add"));
    m_addSmiley->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onAddSmiley));
    bBox->pack_start(*m_addSmiley, Gtk::PACK_SHRINK, 3);
    m_modifySmiley = new Gtk::Button(_("Modify"));
    m_modifySmiley->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onModifySmiley));
    bBox->pack_start(*m_modifySmiley, Gtk::PACK_SHRINK, 3);
    m_deleteSmiley = new Gtk::Button(_("Delete"));
    m_deleteSmiley->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onDeleteSmiley));
    bBox->pack_start(*m_deleteSmiley, Gtk::PACK_SHRINK, 3);
    m_importSmiley = new Gtk::Button(_("Import"));
    m_importSmiley->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onImportSmiley));
    bBox->pack_start(*m_importSmiley, Gtk::PACK_SHRINK, 3);
    m_exportSmiley = new Gtk::Button(_("Export"));
    m_exportSmiley->signal_clicked().connect(sigc::mem_fun(this, &ConfigDialog::onExportSmiley));
    bBox->pack_start(*m_exportSmiley, Gtk::PACK_SHRINK, 3);
    sbox->pack_start(*bBox, Gtk::PACK_SHRINK, 3);

    tab->pack_start(*sbox, Gtk::PACK_EXPAND_WIDGET);

    onSmileys();

    show_all();
    m_tabbook.append_page(*tab, _("Smileys"));
}

void ConfigDialog::onSmileys()
{
    if(m_useSmileys->get_active())
    {
        m_addSmiley->set_sensitive(true);
        m_importSmiley->set_sensitive(true);
        if(m_smileysModel->children().size())
        {
            m_modifySmiley->set_sensitive(true);
            m_deleteSmiley->set_sensitive(true);
            m_exportSmiley->set_sensitive(true);
        }
        else
        {
            m_modifySmiley->set_sensitive(false);
            m_deleteSmiley->set_sensitive(false);
            m_exportSmiley->set_sensitive(false);
        }
    }
    else
    {
        m_addSmiley->set_sensitive(false);
        m_modifySmiley->set_sensitive(false);
        m_deleteSmiley->set_sensitive(false);
        m_importSmiley->set_sensitive(false);
        m_exportSmiley->set_sensitive(false);
    }
}

void ConfigDialog::onAddSmiley()
{
    SmileyDialog dialog(_("Add smiley"), "", "", *this);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK && !dialog.getSmiley().empty()
       && !dialog.getPath().empty() && !hasSmiley(dialog.getSmiley()))
    {
        Gtk::TreeModel::Row row = *(m_smileysModel->append());
        row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
        row[m_smileysColumns.m_smiley] = dialog.getSmiley();
        row[m_smileysColumns.m_path] = dialog.getPath();
        onSmileys();
    }
}

bool ConfigDialog::hasSmiley(const Glib::ustring &smiley)
{
    Gtk::TreeModel::Children children = m_smileysModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row[m_smileysColumns.m_smiley] == smiley)
            return true;
    }
    return false;
}

void ConfigDialog::onModifySmiley()
{
    Gtk::TreeModel::iterator it = m_smileysList->get_selection()->get_selected();
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        SmileyDialog dialog(_("Modify smiley"), row[m_smileysColumns.m_smiley], row[m_smileysColumns.m_path], *this);
        int result = dialog.run();
        if(result == Gtk::RESPONSE_OK && !dialog.getSmiley().empty()
           && !dialog.getPath().empty())
        {
            if(dialog.getSmiley() == row[m_smileysColumns.m_smiley])
            {
                row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                row[m_smileysColumns.m_path] = dialog.getPath();
            }
            else
            {
                if(hasSmiley(dialog.getSmiley()))
                {
                    m_smileysModel->erase(it);
                    Gtk::TreeModel::Children children = m_smileysModel->children();
                    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
                    {
                        row = *iter;
                        if(row[m_smileysColumns.m_smiley] == dialog.getSmiley())
                        {
                            row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                            row[m_smileysColumns.m_smiley] = dialog.getSmiley();
                            row[m_smileysColumns.m_path] = dialog.getPath();
                        }
                    }
                }
                else
                {
                    row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                    row[m_smileysColumns.m_smiley] = dialog.getSmiley();
                    row[m_smileysColumns.m_path] = dialog.getPath();
                }
            }
        }
        onSmileys();
    }
}

void ConfigDialog::onDeleteSmiley()
{
    Gtk::TreeModel::iterator it = m_smileysList->get_selection()->get_selected();
    if(it)
    {
        m_smileysModel->erase(it);
        onSmileys();
    }
    m_smileysList->get_selection()->select(m_smileysModel->children().begin());
}

void ConfigDialog::onImportSmiley()
{
    if(m_showImportwarning && m_smileysModel->children().size())
    {
        Gtk::MessageDialog msg(*this, _("Warning"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK_CANCEL);
        msg.set_secondary_text(_("File import overwrites current settings"));
        int result = msg.run();
        if(result == Gtk::RESPONSE_CANCEL)
        {
            m_showImportwarning = false;
            return;
        }
    }
    Gtk::FileChooserDialog dialog(_("Select file"), Gtk::FILE_CHOOSER_ACTION_OPEN);
    dialog.set_transient_for(*this);
    if(m_showImportwarning) dialog.set_filename(DXIM_DATADIR G_DIR_SEPARATOR_S "icons" G_DIR_SEPARATOR_S "smileys" G_DIR_SEPARATOR_S "dxim.scfg");
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
#if GTKMM_MAJOR_VERSION == 2
    Gtk::FileFilter filter;
 	filter.set_name(_("Smiley files"));
 	filter.add_pattern("*.scfg");
#else
    Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
    filter->set_name(_("Smiley files"));
    filter->add_pattern("*.scfg");
#endif
    dialog.add_filter(filter);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        Glib::ustring filename = dialog.get_filename();
        Glib::ustring dir = Glib::path_get_dirname(filename);
        std::ifstream file(Glib::locale_from_utf8(filename).c_str());
        if(file.is_open())
        {
            Gtk::TreeModel::Children children = m_smileysModel->children();
            while(children.size())
            {
                m_smileysModel->erase(children.begin());
            }
            std::string buf;
            while(getline(file, buf))
            {
                if(buf[0] != '#')
                {
                    Glib::ustring smiley, path;
                    size_t pos = buf.find_first_of('=');
                    if(pos != std::string::npos)
                    {
                        path = buf.substr(0, pos);
                        path = dxutils::absolutePath(dir, path);
                        smiley = buf.substr(pos+1);
                        if(!path.empty() && !smiley.empty() && Glib::file_test(path, Glib::FILE_TEST_EXISTS)
                            && !Glib::file_test(path, Glib::FILE_TEST_IS_DIR))
                        {
                            Gtk::TreeModel::Row row = *(m_smileysModel->append());
                            row[m_smileysColumns.m_icon] = dxutils::createIcon(path);
                            row[m_smileysColumns.m_smiley] = smiley;
                            row[m_smileysColumns.m_path] = path;
                        }
                    }
                }
            }
            file.close();
        }
    }
    m_showImportwarning = false;
    onSmileys();
    m_smileysList->get_selection()->select(m_smileysModel->children().begin());
}

void ConfigDialog::onExportSmiley()
{
    Gtk::FileChooserDialog dialog(_("Save smiley settings as"), Gtk::FILE_CHOOSER_ACTION_SAVE);
    dialog.set_transient_for(*this);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
#if GTKMM_MAJOR_VERSION == 2
    Gtk::FileFilter filter;
 	filter.set_name(_("Smiley files"));
 	filter.add_pattern("*.scfg");
#else
    Glib::RefPtr<Gtk::FileFilter> filter = Gtk::FileFilter::create();
    filter->set_name(_("Smiley files"));
    filter->add_pattern("*.scfg");
#endif
    dialog.add_filter(filter);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        Glib::ustring filename = dialog.get_filename();
        if(dxutils::fileExtension(filename)!="scfg")
        {
            filename += ".scfg";
        }
        std::ofstream file(filename.c_str(), std::ios::out|std::ios::app);
        if(file.is_open())
        {
            file << HEADER;
            Gtk::TreeModel::Children children = m_smileysModel->children();
            for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
            {
                Gtk::TreeModel::Row row = *iter;
                file << dxutils::relativePath(Glib::path_get_dirname(filename), row[m_smileysColumns.m_path])
                    << "=" << row[m_smileysColumns.m_smiley] << std::endl;
            }
        }
    }
}

void ConfigDialog::onSmileysDoubleClick(const Gtk::TreeModel::Path &path, Gtk::TreeViewColumn */*column*/)
{
    Gtk::TreeModel::iterator it = m_smileysModel->get_iter(path);
    if(it)
    {
        Gtk::TreeModel::Row row = *it;
        SmileyDialog dialog(_("Modify smiley"), row[m_smileysColumns.m_smiley], row[m_smileysColumns.m_path], *this);
        int result = dialog.run();
        if(result == Gtk::RESPONSE_OK && !dialog.getSmiley().empty()
           && !dialog.getPath().empty())
        {
            if(dialog.getSmiley() == row[m_smileysColumns.m_smiley])
            {
                row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                row[m_smileysColumns.m_path] = dialog.getPath();
            }
            else
            {
                if(hasSmiley(dialog.getSmiley()))
                {
                    m_smileysModel->erase(it);
                    Gtk::TreeModel::Children children = m_smileysModel->children();
                    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
                    {
                        row = *iter;
                        if(row[m_smileysColumns.m_smiley] == dialog.getSmiley())
                        {
                            row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                            row[m_smileysColumns.m_smiley] = dialog.getSmiley();
                            row[m_smileysColumns.m_path] = dialog.getPath();
                        }
                    }
                }
                else
                {
                    row[m_smileysColumns.m_icon] = dxutils::createIcon(dialog.getPath());
                    row[m_smileysColumns.m_smiley] = dialog.getSmiley();
                    row[m_smileysColumns.m_path] = dialog.getPath();
                }
            }
        }
        onSmileys();
    }
}

void ConfigDialog::onRaiseWindow()
{
    if(m_raiseWindow->get_active())
    {
        m_notifyMessageTray->set_sensitive(false);
        m_notifyMessageTray->set_active(false);
    }
    else
    {
        m_notifyMessageTray->set_sensitive(true);
    }
}

