/*
 *      fieldsrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "fieldsrequestdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

static bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

FieldsRequestDialog::FieldsRequestDialog(Gtk::Window& parent, PURPLE::FieldsRequest* request)
: Gtk::Dialog(request->getTitle().empty()?_("New request"):request->getTitle(), parent), m_request(request)
{
    add_button(m_request->getCancelText(), Gtk::RESPONSE_CANCEL);
    add_button(m_request->getOkText(), Gtk::RESPONSE_OK);
    get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getPrimary(), Gtk::ALIGN_START)));
    if(!m_request->getSecondary().empty()) get_vbox()->pack_start(*Gtk::manage(new Gtk::Label(m_request->getSecondary(), Gtk::ALIGN_START)));
    createFields();
    show_all();
}

FieldsRequestDialog::~FieldsRequestDialog()
{
}

void FieldsRequestDialog::on_response(int response)
{
    if(response == Gtk::RESPONSE_OK)
    {
        fillFields();
        m_request->acceptRequest();
    }
    else
        m_request->dennyRequest();
}

void FieldsRequestDialog::createFields()
{
    Gtk::Table *tab = Gtk::manage(new Gtk::Table());
    std::vector<PURPLE::FieldsRequest::Field> fields = m_request->getFields();
    std::vector<PURPLE::FieldsRequest::Field>::iterator it = fields.begin();
    Glib::ustring group = "";
    for(int i=0; it!=fields.end(); it++, i++)
    {
        PURPLE::FieldsRequest::Field field = *it;
        if(group != field.getGroup())
        {
            group = field.getGroup();
            if(!group.empty())
            {
                Gtk::Label *label = Gtk::manage(new Gtk::Label("", Gtk::ALIGN_START));
                label->set_markup(Glib::ustring::compose("<b>%1</b>", group));
                tab->attach(*label, 0, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                i++;
            }
        }
        tab->attach(*Gtk::manage(new Gtk::Label(field.getLabel(), Gtk::ALIGN_START)), 0, 1, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            if(field.getMultiline())
            {
                Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
                swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
                swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
                Gtk::TextView *text = Gtk::manage(new Gtk::TextView());
                text->set_editable(field.getEditable());
                Glib::RefPtr<Gtk::TextBuffer> buffer = text->get_buffer();
                buffer->insert(buffer->end(), field.getStringValue());
                swin->add(*text);
                tab->attach(*swin, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 3, 3);
                m_fields.push_back(text);
            }
            else
            {
                Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
                edit->set_editable(field.getEditable());
                if(field.getMasked())
                {
                    edit->set_visibility(false);
                    edit->set_invisible_char('*');
                }
                edit->set_text(field.getStringValue());
                tab->attach(*edit, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_fields.push_back(edit);
            }
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
            edit->set_text(field.getStringValue());
            tab->attach(*edit, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
            m_fields.push_back(edit);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            Gtk::CheckButton *check = Gtk::manage(new Gtk::CheckButton());
            check->set_active(field.getBoolValue());
            tab->attach(*check, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
            m_fields.push_back(check);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            struct Columns : public Gtk::TreeModel::ColumnRecord
            {
                Gtk::TreeModelColumn<Glib::ustring> m_label;
                Gtk::TreeModelColumn<int> m_value;
                Columns() { add(m_label); add(m_value); }
            };
            const Columns columns;
            Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
            Gtk::ComboBox *combo = Gtk::manage(new Gtk::ComboBox());
            combo->set_model(model);
            combo->pack_start(columns.m_label);
            std::vector<std::string> s = field.getLabels();
            std::vector<std::string>::iterator it = s.begin();
            for(int j=0; it!=s.end(); it++, j++)
            {
                Gtk::TreeModel::Row row = *(model->append());
                row[columns.m_label] = *it;
                row[columns.m_value] = j;
            }
            combo->set_active(0);
            tab->attach(*combo, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
            m_fields.push_back(combo);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
            swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
            swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
            struct Columns : public Gtk::TreeModel::ColumnRecord
            {
                Gtk::TreeModelColumn<Glib::ustring> m_label;
                Columns() { add(m_label); }
            };
            const Columns columns;
            Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
            Gtk::TreeView *list = Gtk::manage(new Gtk::TreeView());
            list->set_model(model);
            list->append_column("Label", columns.m_label);
            list->set_headers_visible(false);
            Glib::RefPtr<Gtk::TreeSelection> selectionModel = list->get_selection();
            selectionModel->set_mode(field.getMultiple()?Gtk::SELECTION_MULTIPLE:Gtk::SELECTION_SINGLE);
            std::vector<std::string> s = field.getLabels();
            std::vector<std::string> sel = field.getSelected();
            FOREACH(std::vector<std::string>, s, sit)
            {
                Gtk::TreeModel::Row row = *(model->append());
                row[columns.m_label] = *sit;
                if(vectorContains(sel, *sit)) list->get_selection()->select(row);
            }
            swin->add(*list);
            tab->attach(*swin, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 3, 3);
            m_fields.push_back(list);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FIMAGE)
        {
            Gtk::Image *image = Gtk::manage(new Gtk::Image(field.getStringValue()));
            tab->attach(*image, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        }
    }
    get_vbox()->pack_start(*tab, Gtk::PACK_SHRINK);
}

void FieldsRequestDialog::fillFields()
{
    PURPLE::FieldsRequest::Fields fields = m_request->getFields();
    PURPLE::FieldsRequest::Fields::iterator it;
    guint i = 0;
    for(it=fields.begin(); it!=fields.end(); it++)
    {
        PURPLE::FieldsRequest::Field& field = *it;
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            if(field.getMultiline())
            {
                Gtk::TextView *text = static_cast<Gtk::TextView*>(m_fields[i]);
                Glib::RefPtr<Gtk::TextBuffer> buffer = text->get_buffer();
                field.setValue(buffer->get_text());
            }
            else
            {
                field.setValue(static_cast<Gtk::Entry*>(m_fields[i])->get_text());
            }
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            field.setValue(static_cast<Gtk::Entry*>(m_fields[i])->get_text());
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            field.setValue(static_cast<Gtk::CheckButton*>(m_fields[i])->get_active());
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            Gtk::ComboBox *combo = static_cast<Gtk::ComboBox*>(m_fields[i]);
            Gtk::TreeModel::iterator iter = combo->get_active();
            if(iter)
            {
                Gtk::TreeModel::Row row = *iter;
                if(row)
                {
                    int value;
                    row.get_value(1, value);
                    field.setValue(value);
                }
                else
                    field.setValue(0);
            }
            else
            {
                field.setValue(0);
            }
            i++;
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            std::vector<std::string> v;
            Gtk::TreeView *list = static_cast<Gtk::TreeView*>(m_fields[i]);
            Glib::RefPtr<Gtk::TreeSelection> selectionModel = list->get_selection();
            Gtk::TreeModel::Children children = list->get_model()->children();
            for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
            {
                if(selectionModel->is_selected(iter))
                {
                    Glib::ustring value;
                    iter->get_value(0, value);
                    v.push_back(value);
                }
            }
            field.setSelected(v);
            i++;
        }
        else
            continue;
    }
    m_request->setFields(fields);
}

