/*
 *      plugineditor.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "plugineditor.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

/* Constructor - Modify existing plugin */
PluginEditor::PluginEditor(Gtk::Window& parent, PURPLE::Plugin plugin)
    : Gtk::Dialog(_("Modify plugin"), parent)
{
    this->m_plugin = plugin;
    this->m_options = plugin.getOptions();
    this->initializeEditor();
}

/* Destructor */
PluginEditor::~PluginEditor()
{
}

void PluginEditor::initializeEditor()
{
    Gtk::Label *label = Gtk::manage(new Gtk::Label());
    label->set_markup("<b>"+m_plugin.getName()+"</b>");
    get_vbox()->pack_start(*label);
    Gtk::Table *tab = Gtk::manage(new Gtk::Table());
    for(size_t i=0; i<m_options.size(); i++)
    {
        PURPLE::Plugin::Option option = m_options[i];
        std::vector<std::string> svalues = option.getStringList();
        std::map<std::string, int> ivalues = option.getIntList();
        tab->attach(*Gtk::manage(new Gtk::Label(dxutils::normalizeLabelString(option.getText(), '_'), Gtk::ALIGN_START)), 0, 1, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                Gtk::Entry *edit = Gtk::manage(new Gtk::Entry());
                edit->set_text(option.getStringValue());
                tab->attach(*edit, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(edit);
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                struct Columns : public Gtk::TreeModel::ColumnRecord
                {
                    Gtk::TreeModelColumn<Glib::ustring> m_value;

                    Columns() { add(m_value); }
                };
                const Columns columns;
                Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
                Gtk::ComboBox *combo = Gtk::manage(new Gtk::ComboBox());
                combo->set_model(model);
                combo->pack_start(columns.m_value);
                std::vector<std::string>::iterator value = svalues.begin();
                int j=0;
                for(; value!=svalues.end(); ++value, j++)
                {
                    Gtk::TreeModel::Row row = *(model->append());
                    row[columns.m_value] = *value;
                    if(!option.getStringValue().compare(*value))
                        combo->set_active(j);
                }
                tab->attach(*combo, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(combo);
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                Gtk::SpinButton *spin = Gtk::manage(new Gtk::SpinButton());
                spin->set_range(option.getIntMin(), option.getIntMax());
                spin->set_increments(1, 5);
                spin->set_value(option.getIntValue());
                tab->attach(*spin, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(spin);
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                struct Columns : public Gtk::TreeModel::ColumnRecord
                {
                    Gtk::TreeModelColumn<Glib::ustring> m_value;

                    Columns() { add(m_value); }
                };
                const Columns columns;
                Glib::RefPtr<Gtk::ListStore> model = Gtk::ListStore::create(columns);
                Gtk::ComboBox *combo = Gtk::manage(new Gtk::ComboBox());
                combo->set_model(model);
                combo->pack_start(columns.m_value);
                std::map<std::string, int>::iterator value = ivalues.begin();
                int j=0;
                for(; value!=ivalues.end(); ++value, j++)
                {
                    Gtk::TreeModel::Row row = *(model->append());
                    row[columns.m_value] = (*value).first;
                    if(option.getIntValue() == (*value).second)
                        combo->set_active(j);
                }
                tab->attach(*combo, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(combo);
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                Gtk::CheckButton *check = Gtk::manage(new Gtk::CheckButton());
                check->set_active(option.getBoolValue());
                tab->attach(*check, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                m_optionsFields.push_back(check);
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                m_optionsFields.push_back(NULL);
                break;
        }
    }
    get_vbox()->pack_start(*tab);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    Gtk::Button *okbutton = Gtk::manage(new Gtk::Button(Gtk::Stock::OK));
    okbutton->signal_clicked().connect(sigc::mem_fun(this, &PluginEditor::onOK));
    Gtk::Button *closebutton = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebutton->signal_clicked().connect(sigc::mem_fun(this, &Gtk::Dialog::hide));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*okbutton, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebutton, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    show_all();
}

int PluginEditor::getIntlistValue(std::map<std::string, int> map, std::string key)
{
    std::map<std::string, int>::iterator it = map.find(key);
    if(it!=map.end())
        return (*it).second;
    return -1;
}

void PluginEditor::onOK()
{
    PURPLE::Plugin::Options::iterator it;
    guint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Plugin::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                option.setValue(static_cast<Gtk::Entry*>(m_optionsFields[i])->get_text());
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                Gtk::ComboBox *combo = static_cast<Gtk::ComboBox*>(m_optionsFields[i]);
                Gtk::TreeModel::iterator iter = combo->get_active();
                if(iter)
                {
                    Gtk::TreeModel::Row row = *iter;
                    if(row)
                    {
                        Glib::ustring value;
                        row.get_value(0, value);
                        option.setValue(value);
                    }
                    else
                        option.setValue("");
                }
                else
                    option.setValue("");
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                option.setValue(Glib::ustring::compose("%1", static_cast<Gtk::SpinButton*>(m_optionsFields[i])->get_value_as_int()));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                Gtk::ComboBox *combo = static_cast<Gtk::ComboBox*>(m_optionsFields[i]);
                Gtk::TreeModel::iterator iter = combo->get_active();
                if(iter)
                {
                    Gtk::TreeModel::Row row = *iter;
                    if(row)
                    {
                        Glib::ustring value;
                        row.get_value(0, value);
                        option.setValue(Glib::ustring::compose("%1", getIntlistValue(option.getIntList(), value)));
                    }
                    else
                        option.setValue(Glib::ustring::compose("%1", getIntlistValue(option.getIntList(), "")));
                }
                else
                    option.setValue(Glib::ustring::compose("%1", getIntlistValue(option.getIntList(), "")));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                option.setValue(static_cast<Gtk::CheckButton*>(m_optionsFields[i])->get_active() ? "true" : "false");
                option.save();
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                break;
        }
    }
}

