/*
 *      roomlistdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "roomlistdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "widgets/imagelabel.h"

RoomListDialog::RoomListDialog(Gtk::Window& parent, PURPLE::RoomList *rlist, dxgCallbacks *cb)
: Gtk::Dialog(_("Chats list"), parent), m_cb(cb), m_rlist(rlist), m_finished(false)
{
    get_vbox()->pack_start(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(m_rlist->getAccount().getProtocol().getPurpleID()), m_rlist->getAccount().getUsername())));
    m_search = Gtk::manage(new Gtk::Entry);
    m_search->signal_changed().connect(sigc::mem_fun(this, &RoomListDialog::onSearch));
    Gtk::HBox *search = Gtk::manage(new Gtk::HBox);
    search->pack_start(*Gtk::manage(new Gtk::Label(_("Search"), Gtk::ALIGN_START)));
    search->pack_start(*m_search);
    get_vbox()->pack_start(*search);
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_rooms.set_size_request(-1, 155);
    swin->add(m_rooms);
    get_vbox()->pack_start(*swin, Gtk::PACK_EXPAND_WIDGET);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    m_select = Gtk::manage(new Gtk::Button(_("_Select"), true));
    m_select->set_sensitive(false);
    m_select->signal_clicked().connect(sigc::mem_fun(this, &RoomListDialog::onSelect));
    Gtk::Button *closebutton = Gtk::manage(new Gtk::Button(Gtk::Stock::CLOSE));
    closebutton->signal_clicked().connect(sigc::mem_fun(this, &RoomListDialog::onClose));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    btnbox->pack_end(*m_select, Gtk::PACK_SHRINK);
    btnbox->pack_end(*closebutton, Gtk::PACK_SHRINK);
    get_vbox()->pack_start(*btnbox, false, false, 3);
    show_all();

    m_cb->signalRoomlistColumns.connect(sigc::mem_fun(this, &RoomListDialog::onColumns));
    m_cb->signalRoomlistAddRoom.connect(sigc::mem_fun(this, &RoomListDialog::onAddRoom));
    m_cb->signalRoomlistFinished.connect(sigc::mem_fun(this, &RoomListDialog::onFinished));
}

RoomListDialog::~RoomListDialog()
{
}

void RoomListDialog::onColumns(PURPLE::RoomList *list, std::vector<std::string> columns)
{
    m_rooms.append_column(_("Chat"), m_roomColumns.m_chat);
    m_rooms.get_column(0)->set_resizable(true);
    if(m_rlist == list)
    {
        for(size_t i=0; i<columns.size(); i++)
        {
            Gtk::TreeModelColumn<Glib::ustring> a;
            m_roomColumns.add(a);
            m_rooms.append_column(columns[i], a);
            m_rooms.get_column(i+1)->set_resizable(true);
        }
    }
    m_roomModel = Gtk::ListStore::create(m_roomColumns);
    m_roomModelFilter = Gtk::TreeModelFilter::create(m_roomModel);
    m_roomModelFilter->set_visible_func(sigc::mem_fun(this, &RoomListDialog::filterRooms));
    m_rooms.set_model(m_roomModelFilter);
    m_roomSelectionModel = m_rooms.get_selection();
    m_roomSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
}

void RoomListDialog::onAddRoom(PURPLE::RoomList *list, std::vector<std::string> fields)
{
    if(m_rlist == list)
    {
        Gtk::TreeModel::Row row = *(m_roomModel->append());
        row[m_roomColumns.m_chat] = fields[0];
        for(size_t i=1; i<fields.size(); i++)
        {
            row.set_value(i, fields[i]);
        }
        if(m_roomModel->children().size()==1) m_roomSelectionModel->select(row);
    }
}

void RoomListDialog::onFinished(PURPLE::RoomList *list)
{
    if(m_rlist == list)
    {
        m_roomModel->set_sort_column(0, Gtk::SORT_ASCENDING);
        m_select->set_sensitive(true);
        m_finished = true;
    }
}

void RoomListDialog::onSearch()
{
    m_roomModelFilter->refilter();
    if(m_rooms.get_model()->children().size()) m_select->set_sensitive();
    else m_select->set_sensitive(false);
}

bool RoomListDialog::filterRooms(const Gtk::TreeModel::const_iterator &iter)
{
    if(iter)
    {
        if(m_search->get_text().empty()) return true;
        else
        {
            Glib::ustring name;
            iter->get_value(0, name);
            if(name.find(m_search->get_text()) != Glib::ustring::npos) return true;
            else return false;
        }
    }
    return true;
}

void RoomListDialog::onSelect()
{
    Gtk::TreeModel::iterator it = m_roomSelectionModel->get_selected();
    if(it)
        signalClosed(true, (*it)[m_roomColumns.m_chat]);
    else
        signalClosed(false, "");
    hide();
}

void RoomListDialog::onClose()
{
    if(!m_finished) m_rlist->cancelListing();
    signalClosed(false, "");
    hide();
}

