/*
 *      dxim.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "config.h"
#ifdef _WIN32
  #define _WIN32_LEAN_AND_MEAN
  #include <windows.h>
#else
  #include <sys/file.h>
  #include <unistd.h>
#endif //_WIN32
#include <purplemm.h>
#include <utilsmm.h>
#include <iostream>
#include "i18n.h"
#include "dialogs/accountdialog.h"
#include "dialogs/actionrequestdialog.h"
#include "dialogs/inputrequestdialog.h"
#include "dialogs/imdialog.h"
#include "dialogs/configdialog.h"
#include "dialogs/chatdialog.h"
#include "dialogs/chatjoindialog.h"
#include "dialogs/choicerequestdialog.h"
#include "dialogs/fieldsrequestdialog.h"
#include "dialogs/plugindialog.h"
#include "dialogs/selectaccountdialog.h"
#include "tabitems/tabbook.h"
#include "icons.h"
#include "dxutils.h"
#include "roster/roster.h"
#include "dxim.h"
#include "tabitems/tabitem.h"
#include "tabitems/chattabitem.h"
#include "tabitems/imtabitem.h"
#include "preferences.h"
#ifdef USE_SCREENSAVER
  #include <X11/Xutil.h>
  #include <X11/extensions/scrnsaver.h>
  #include <gdk/gdkx.h>
#endif //USE_SCREENSAVER

#ifndef _WIN32
void dxunlink(void)
{
    unlink(Glib::ustring(Glib::get_tmp_dir()+G_DIR_SEPARATOR_S+"dxim-gtkmm-"+Glib::get_user_name()).c_str());
}
#endif

// For handling Single Instance of Application
bool isInstanceRunningAlready(int argc, char *argv[])
{
#ifdef _WIN32
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex(NULL, FALSE,TEXT("dxim-gtkmm.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxim already running"), TEXT("dxim"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        return TRUE;
    }
#else
    Glib::ustring path = Glib::get_tmp_dir()+G_DIR_SEPARATOR_S+"dxim-gtkmm-"+Glib::get_user_name();
    struct flock fl;
    int fd;
    fd = open(path.c_str(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.c_str());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        Gtk::Main kit(argc, argv);
        Gtk::Window window;
        window.set_title("dxim");
        Gtk::VBox box;
        window.add(box);
        Gtk::Label label("dxim already running");
        box.pack_start(label);
        Gtk::Button button(Gtk::Stock::CLOSE);
        box.pack_start(button);
        button.signal_clicked().connect(&Gtk::Main::quit);
        window.show_all_children();
        Gtk::Main::run(window);
        _exit(1);
    }
    atexit(dxunlink);
#endif
    return false;
}

static gboolean on_cmd_next_unread(GtkAccelGroup* group, GObject* acceleratable, guint keyval, GdkModifierType modifier, gpointer user_data)
{
    return static_cast<dxim*>(user_data)->onCmdNextUnread();
}

static gboolean on_cmd_select_tab(GtkAccelGroup* group, GObject* acceleratable, guint keyval, GdkModifierType modifier, gpointer user_data)
{
    return static_cast<dxim*>(user_data)->onCmdSelectTab(keyval-GDK_KEY_0-1);
}

dxim::dxim(bool offline)
{
    set_title("dxim");
    if(!DXPREFERENCES.m_w) DXPREFERENCES.m_w = 800;
    if(!DXPREFERENCES.m_h) DXPREFERENCES.m_h = 600;
    set_default_size(DXPREFERENCES.m_w,DXPREFERENCES.m_h);
    if(DXPREFERENCES.m_x>=0 && DXPREFERENCES.m_y>=0)
        move(DXPREFERENCES.m_x, DXPREFERENCES.m_y);
    if(DXPREFERENCES.m_maximized)
        maximize();
    set_default_icon(ICO_SMALLICON->get_pixbuf());

    m_cb = new dxgCallbacks();
    m_cb->signalError.connect(sigc::mem_fun(this, &dxim::onPurpleError));
    m_cb->signalAccountConnected.connect(sigc::mem_fun(this, &dxim::onAccountConnected));
    m_cb->signalAccountDisconnected.connect(sigc::mem_fun(this, &dxim::onAccountDisconnected));
    m_cb->signalNewIMMessage.connect(sigc::mem_fun(this, &dxim::onNewIMMsg));
    m_cb->signalNewChatMessage.connect(sigc::mem_fun(this, &dxim::onNewChatMsg));
    m_cb->signalNewIM.connect(sigc::mem_fun(this, &dxim::onNewIM));
    m_cb->signalNewChat.connect(sigc::mem_fun(this, &dxim::onNewChat));
    m_cb->signalActionRequest.connect(sigc::mem_fun(this, &dxim::onNewActionRequest));
    m_cb->signalFieldsRequest.connect(sigc::mem_fun(this, &dxim::onNewFieldsRequest));
    m_cb->signalInputRequest.connect(sigc::mem_fun(this, &dxim::onNewInputRequest));
    m_cb->signalChoiceRequest.connect(sigc::mem_fun(this, &dxim::onNewChoiceRequest));
    m_cb->signalFileRequest.connect(sigc::mem_fun(this, &dxim::onNewFileRequest));
    m_cb->signalNotifyMessage.connect(sigc::mem_fun(this, &dxim::onNotifyMessage));
    m_cb->signalNotifyEmail.connect(sigc::mem_fun(this, &dxim::onNotifyEmail));
    m_cb->signalNotifyUserinfo.connect(sigc::mem_fun(this, &dxim::onNotifyUserinfo));
    m_cb->signalNotifyUri.connect(sigc::mem_fun(this, &dxim::onNotifyUri));
    m_cb->signalNewFileTransfer.connect(sigc::mem_fun(this, &dxim::onNewFileTransfer));
    m_cb->signalCloseConvTab.connect(sigc::mem_fun(this, &dxim::onCloseConvTab));

    createMenus();
    createAccel();
    createTrayIcon();

    add(m_vbox);
    m_vbox.pack_start(*m_uiManager->get_widget("/MenuBar"), Gtk::PACK_SHRINK);
    m_tabbook.signal_switch_page().connect(sigc::mem_fun(this, &dxim::onSwitchTab));
    m_splitter.pack1(m_tabbook, true, true);
    m_roster = new Roster(m_cb);
    m_roster->signalAccountDisconnected.connect(sigc::mem_fun(this, &dxim::closeAccount));
    m_splitter.pack2(*m_roster, false, true);
    m_vbox.pack_start(m_splitter);
    show_all_children();
    if(DXPREFERENCES.m_rosterWidth > 1) m_splitter.set_position(DXPREFERENCES.m_w-DXPREFERENCES.m_rosterWidth);
    if(!DXPREFERENCES.m_rosterShown) m_splitter.get_child2()->hide();

    m_cb->getPurplemm()->initPurple(offline);
    m_cb->getPurplemm()->loadSavedPlugins("/dxim/plugins/loaded");

    m_log = NULL;
    m_fttab = NULL;
    m_presence = PURPLE_STATUS_OFFLINE;
    m_newMsg = false;

    Glib::signal_timeout().connect(sigc::mem_fun(this, &dxim::onAwayTimeout), 1000);
}

dxim::~dxim()
{}

void dxim::onCmdQuit()
{
    int width, height;
    get_size(width, height);
    if(width && height)
    {
        DXPREFERENCES.m_w = width;
        DXPREFERENCES.m_h = height;
    }
    int x, y;
    get_window()->get_root_origin(x, y);
    if (x >= 0 && y >= 0)
    {
        DXPREFERENCES.m_x = x;
        DXPREFERENCES.m_y = y;
    }
    bool maximized = false;
    if(Gdk::WINDOW_STATE_MAXIMIZED & get_state())
        maximized = true;
    DXPREFERENCES.m_maximized = maximized;
    DXPREFERENCES.m_rosterWidth = m_splitter.get_child2()->get_width();
    DXPREFERENCES.save();
    Gtk::Main::quit();
}

void dxim::createMenus()
{
    m_uiManager = Gtk::UIManager::create();
    Glib::RefPtr<Gtk::ActionGroup> group = Gtk::ActionGroup::create();
    group->add(Gtk::Action::create("dximMenu", _("_dxim")));
    group->add(Gtk::Action::create("ManageAccounts", _("Manage accounts")), Gtk::AccelKey("<control>a"), sigc::mem_fun(this, &dxim::onCmdAccounts));
    group->add(Gtk::Action::create("AddBuddy", _("Add buddy")), sigc::mem_fun(this, &dxim::onCmdAddBuddy));
    group->add(Gtk::Action::create("AddChat", _("Add chat")), sigc::mem_fun(this, &dxim::onCmdAddChat));
    group->add(Gtk::Action::create("AddGroup", _("Add group")), sigc::mem_fun(this, &dxim::onCmdAddGroup));
    group->add(Gtk::Action::create("NewIM", _("New IM")), Gtk::AccelKey("<control>m"), sigc::mem_fun(this, &dxim::onCmdNewIM));
    group->add(Gtk::Action::create("JoinChat", _("Join chat")), Gtk::AccelKey("<control>t"), sigc::mem_fun(this, &dxim::onCmdJoinChat));
    group->add(Gtk::Action::create("FileTransfers", Gtk::StockID("dxim-filetransfer")), Gtk::AccelKey("<control>f"), sigc::mem_fun(this, &dxim::onCmdFileTransfers));
    m_logAction = Gtk::Action::create("LogViewer", Gtk::StockID("dxim-logs"));
    if(!DXPREFERENCES.m_logging) m_logAction->set_sensitive(false);
    group->add(m_logAction, Gtk::AccelKey("<control>g"), sigc::mem_fun(this, &dxim::onCmdLogViewer));
    group->add(Gtk::Action::create("Quit", Gtk::Stock::QUIT), sigc::mem_fun(this, &dxim::onCmdQuit));
    group->add(Gtk::Action::create("StatusMenu", _("_Status")));
    group->add(Gtk::Action::create("Available", Gtk::StockID("dxim-online")), sigc::mem_fun(this, &dxim::onCmdAvailable));
    group->add(Gtk::Action::create("Away", Gtk::StockID("dxim-away")), sigc::mem_fun(this, &dxim::onCmdAway));
    group->add(Gtk::Action::create("Busy", Gtk::StockID("dxim-dnd")), sigc::mem_fun(this, &dxim::onCmdDnd));
    group->add(Gtk::Action::create("Invisible", Gtk::StockID("dxim-invisible")), sigc::mem_fun(this, &dxim::onCmdInvisible));
    group->add(Gtk::Action::create("StatusMsg", _("Set status message")), sigc::mem_fun(this, &dxim::onCmdStatusMsg));
    group->add(Gtk::Action::create("Offline", Gtk::StockID("dxim-offline")), sigc::mem_fun(this, &dxim::onCmdOffline));
    group->add(Gtk::Action::create("Reconnect", _("Reconnect")), sigc::mem_fun(this, &dxim::onCmdReconnect));
    group->add(Gtk::Action::create("EditMenu", _("_Edit")));
    m_closetabAction = Gtk::Action::create("CloseTab", Gtk::Stock::CLOSE, _("Close current tab"));
    m_closetabAction->set_sensitive(false);
    group->add(m_closetabAction, Gtk::AccelKey("<control>w"), sigc::mem_fun(this, &dxim::onCmdCloseTab));
    m_clearAction = Gtk::Action::create("Clear", Gtk::Stock::CLEAR, _("Clear window"));
    m_clearAction->set_sensitive(false);
    group->add(m_clearAction, Gtk::AccelKey("<control>l"), sigc::mem_fun(this, &dxim::onCmdClear));
    m_clearallAction = Gtk::Action::create("ClearAll", Gtk::Stock::CLEAR, _("Clear all windows"));
    m_clearallAction->set_sensitive(false);
    group->add(m_clearallAction, Gtk::AccelKey("<control><shift>l"), sigc::mem_fun(this, &dxim::onCmdClearAll));
    Glib::RefPtr<Gtk::ToggleAction> friendsList = Gtk::ToggleAction::create("FriendsList", _("Friends list"));
    friendsList->set_active(DXPREFERENCES.m_rosterShown);
    group->add(friendsList, Gtk::AccelKey("<control>u"), sigc::mem_fun(this, &dxim::onCmdFriendsList));
    Glib::RefPtr<Gtk::ToggleAction> friendsListChat = Gtk::ToggleAction::create("FriendsListChat", _("Friends list in chats"));
    friendsListChat->set_active(DXPREFERENCES.m_chatRosterShown);
    group->add(friendsListChat, sigc::mem_fun(this, &dxim::onCmdFriendsListChat));
    Glib::RefPtr<Gtk::ToggleAction> showAvatars = Gtk::ToggleAction::create("ShowAvatars", _("Show avatars"));
    showAvatars->set_active(DXPREFERENCES.m_showAvatars);
    group->add(showAvatars, sigc::mem_fun(this, &dxim::onCmdShowAvatars));
    Glib::RefPtr<Gtk::ToggleAction> hideOffline = Gtk::ToggleAction::create("HideOffline", _("Hide offline friends"));
    hideOffline->set_active(DXPREFERENCES.m_hideOffline);
    group->add(hideOffline, sigc::mem_fun(this, &dxim::onCmdHideOffline));
    Glib::RefPtr<Gtk::ToggleAction> showStatusmsg = Gtk::ToggleAction::create("ShowStatusmsg", _("Show friends status message"));
    showStatusmsg->set_active(DXPREFERENCES.m_showStatusmsg);
    group->add(showStatusmsg, sigc::mem_fun(this, &dxim::onCmdShowStatusmsg));
    group->add(Gtk::Action::create("Plugins", _("Plugins")), sigc::mem_fun(this, &dxim::onCmdPlugins));
    group->add(Gtk::Action::create("Preferences", Gtk::Stock::PREFERENCES, _("_Preferences")), sigc::mem_fun(this, &dxim::onCmdPreferences));
    group->add(Gtk::Action::create("HelpMenu", _("_Help")));
    group->add(Gtk::Action::create("About", Gtk::Stock::ABOUT), sigc::mem_fun(this, &dxim::onCmdAbout));
    m_uiManager->insert_action_group(group);
    add_accel_group(m_uiManager->get_accel_group());
    try
    {
        Glib::ustring ui_info =
                "<ui>"
                "  <menubar name='MenuBar'>"
                "    <menu action='dximMenu'>"
                "      <menuitem action='ManageAccounts'/>"
                "      <separator/>"
                "      <menuitem action='AddBuddy'/>"
                "      <menuitem action='AddChat'/>"
                "      <menuitem action='AddGroup'/>"
                "      <separator/>"
                "      <menuitem action='NewIM'/>"
                "      <menuitem action='JoinChat'/>"
                "      <separator/>"
                "      <menuitem action='FileTransfers'/>"
                "      <separator/>"
                "      <menuitem action='LogViewer'/>"
                "      <separator/>"
                "      <menuitem action='Quit'/>"
                "    </menu>"
                "    <menu action='EditMenu'>"
                "      <menuitem action='CloseTab'/>"
                "      <separator/>"
                "      <menuitem action='Clear'/>"
                "      <menuitem action='ClearAll'/>"
                "      <separator/>"
                "      <menuitem action='FriendsList'/>"
                "      <menuitem action='FriendsListChat'/>"
                "      <menuitem action='ShowAvatars'/>"
                "      <menuitem action='HideOffline'/>"
                "      <menuitem action='ShowStatusmsg' />"
                "      <separator/>"
                "      <menuitem action='Plugins'/>"
                "      <menuitem action='Preferences'/>"
                "    </menu>"
                "    <menu action='StatusMenu'>"
                "      <menuitem action='Available'/>"
                "      <menuitem action='Away'/>"
                "      <menuitem action='Busy'/>"
                "      <menuitem action='Invisible'/>"
                "      <separator/>"
                "      <menuitem action='StatusMsg'/>"
                "      <separator/>"
                "      <menuitem action='Offline'/>"
                "      <separator/>"
                "      <menuitem action='Reconnect'/>"
                "    </menu>"
                "    <menu action='HelpMenu'>"
                "      <menuitem action='About'/>"
                "    </menu>"
                "  </menubar>"
                "  <popup name='Popup'>"
                "    <menu action='StatusMenu'>"
                "      <menuitem action='Available'/>"
                "      <menuitem action='Away'/>"
                "      <menuitem action='Busy'/>"
                "      <menuitem action='Invisible'/>"
                "      <separator/>"
                "      <menuitem action='StatusMsg'/>"
                "    </menu>"
                "    <separator/>"
                "    <menuitem action='NewIM'/>"
                "    <menuitem action='JoinChat'/>"
                "    <separator/>"
                "    <menuitem action='About' />"
                "    <separator/>"
                "    <menuitem action='Quit' />"
                "  </popup>"
                "</ui>";
        m_uiManager->add_ui_from_string(ui_info);
    }
    catch(const Glib::Error& ex)
    {
        std::cerr << "Building menus failed: " << ex.what() << std::endl;
    }
}

void dxim::createAccel()
{
    Glib::RefPtr<Gtk::AccelGroup> group = get_accel_group();
    GClosure* closure = g_cclosure_new(G_CALLBACK(on_cmd_next_unread), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_n, GDK_CONTROL_MASK, static_cast<GtkAccelFlags>(0), closure);
    GClosure* closure1 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_1, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure1);
    GClosure* closure2 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_2, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure2);
    GClosure* closure3 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_3, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure3);
    GClosure* closure4 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_4, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure4);
    GClosure* closure5 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_5, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure5);
    GClosure* closure6 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_6, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure6);
    GClosure* closure7 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_7, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure7);
    GClosure* closure8 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_8, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure8);
    GClosure* closure9 = g_cclosure_new(G_CALLBACK(on_cmd_select_tab), this, NULL);
    gtk_accel_group_connect(group->gobj(), GDK_KEY_9, GDK_MOD1_MASK, static_cast<GtkAccelFlags>(0), closure9);
    add_accel_group(group);
}

void dxim::createTrayIcon()
{
    m_statusIcon = Gtk::StatusIcon::create(Gtk::StockID("dxim-offline"));
    m_statusIcon->set_tooltip_text("dxim");
    m_statusIcon->signal_activate().connect(sigc::mem_fun(this, &dxim::onTrayActivate));
    m_statusIcon->signal_popup_menu().connect(sigc::mem_fun(this, &dxim::onTrayPopup));
}

void dxim::onCmdAccounts()
{
    AccountDialog dialog(*this, m_cb->getPurplemm());
    dialog.run();
    m_roster->updateAccountsTooltip();
}

void dxim::onCmdAddBuddy()
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }

    PURPLE::Account account;
    SelectAccountDialog adialog(*this, m_cb->getPurplemm());
    int result = adialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        account = adialog.getAccount();
        adialog.hide();
    }
    else
    {
        return;
    }

    Gtk::Dialog dialog(_("Add friend"));
    Gtk::Table table;
    table.attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table.attach(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(account.getProtocol().getPurpleID()), account.getUsername())), 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table.attach(*Gtk::manage(new Gtk::Label(_("ID:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    Gtk::Entry id;
    table.attach(id, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table.attach(*Gtk::manage(new Gtk::Label(_("Nick:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    Gtk::Entry nick;
    table.attach(nick, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table.attach(*Gtk::manage(new Gtk::Label(_("Group:"), Gtk::ALIGN_START)), 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    Gtk::ComboBoxText groups(true);
    std::vector<std::string> v = account.getAllGroups();
    bool friends = false;
    FOREACH(std::vector<std::string>, v, it)
    {
        if((*it) == _("Friends"))
            friends = true;
        groups.append(*it);
    }
    if(!friends) groups.insert(0, _("Friends"));
    groups.set_active(0);
    table.attach(groups, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    dialog.get_vbox()->pack_start(table);
    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.show_all();
    result = dialog.run();
    if(result == Gtk::RESPONSE_OK && !id.get_text().empty())
    {
        Glib::ustring group = groups.get_entry_text();
        if(group.empty()) group = _("Friends");
        account.addBuddy(id.get_text(),
                 group,
                 _("I want to add you in buddies"),
                 nick.get_text());
    }
}

void dxim::onCmdAddChat()
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }

    PURPLE::Account account;
    SelectAccountDialog adialog(*this, m_cb->getPurplemm());
    int result = adialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        account = adialog.getAccount();
        adialog.hide();
    }
    else
    {
        return;
    }

    ChatDialog dialog(account);
    dialog.run();
}

void dxim::onCmdAddGroup()
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }

    PURPLE::Account account;
    SelectAccountDialog adialog(*this, m_cb->getPurplemm());
    int result = adialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        account = adialog.getAccount();
        adialog.hide();
    }
    else
    {
        return;
    }

    Gtk::Dialog dialog(_("Add group"));
    Gtk::HBox groupBox;
    Gtk::Label groupLabel(_("New group:"));
    Gtk::Entry group;
    groupBox.pack_start(groupLabel, Gtk::PACK_SHRINK);
    groupBox.pack_start(group, Gtk::PACK_EXPAND_WIDGET);
    dialog.get_vbox()->pack_start(groupBox);
    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.show_all();
    result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        account.addGroup(group.get_text());
    }
}

void dxim::onCmdAbout()
{
    Glib::ustring comments = Glib::ustring::compose("Gtkmm %1.%2.%3\nPurplemm %4\nLibpurple %5",
            GTKMM_MAJOR_VERSION, GTKMM_MINOR_VERSION, GTKMM_MICRO_VERSION,
            m_cb->getPurplemm()->getPurplemmVersion(), m_cb->getPurplemm()->getLibpurpleVersion());
    Gtk::AboutDialog about;
    about.set_program_name(Glib::ustring::compose("dxim %1\n(Gtkmm toolkit)", VERSION));
    about.set_logo(ICO_ABOUTICON->get_pixbuf());
    about.set_website("https://dxim.dxsolutions.org");
    about.set_copyright(_("Copyright (C) 2012~\nDavid Vachulka (arch_dvx@users.sourceforge.net)\nand others"));
    about.set_comments(comments);
    std::vector<Glib::ustring> authors;
    authors.push_back("David Vachulka <arch_dvx@users.sourceforge.net>");
    about.set_authors(authors);
    about.set_license(_(
            "This program is free software; you can redistribute "
            "it and/or modify it under the terms of the GNU "
            "General Public License as published by the Free "
            "Software Foundation; either version 3 of the "
            "License, or (at your option) any later version.\n\n"

            "This program is distributed in the hope that it "
            "will be useful,  but WITHOUT ANY WARRANTY; without "
            "even the implied warranty of MERCHANTABILITY or "
            "FITNESS FOR A PARTICULAR PURPOSE. See the GNU "
            "General Public License for more details.\n\n"

            "You should have received a copy of the GNU General "
            "Public License along with this program; if not, "
            "write to the Free Software Foundation, Inc., 51 "
            "Franklin Street, Fifth Floor, Boston, MA  "
                        "02110-1301, USA"));
    about.set_wrap_license(true);
    about.run();
}

void dxim::onCmdNewIM()
{
    if(!this->get_visible()) this->show();
    updateTrayIconPresence();
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }
    IMDialog dialog(*this, m_cb->getPurplemm());
    dialog.run();
}

void dxim::onCmdJoinChat()
{
    if(!this->get_visible()) this->show();
    updateTrayIconPresence();
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }
    ChatJoinDialog *dialog = new ChatJoinDialog(*this, m_cb->getPurplemm(), m_cb);
    dialog->show();
    dialog->present();
}

void dxim::onTrayActivate()
{
    if(!this->get_visible()) this->show();
    else this->on_hide();
    updateTrayIconPresence();
}

void dxim::onTrayPopup(guint button, guint32 activate_time)
{
    Gtk::Menu* menu = static_cast<Gtk::Menu*>(m_uiManager->get_widget("/Popup"));
    if(menu)
        menu->popup(button, activate_time);
}

void dxim::onCmdReconnect()
{
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        if(accounts[i].isConnecting())
            continue;
        if(accounts[i].isConnected())
        {
            accounts[i].disconnect();
        }
        accounts[i].connect();
    }
}

void dxim::onCmdAvailable()
{
    updatePresence(PURPLE_STATUS_AVAILABLE);
}

void dxim::onCmdAway()
{
    updatePresence(PURPLE_STATUS_AWAY);
}

void dxim::onCmdDnd()
{
    updatePresence(PURPLE_STATUS_UNAVAILABLE);
}

void dxim::onCmdInvisible()
{
    updatePresence(PURPLE_STATUS_INVISIBLE);
}

void dxim::onCmdOffline()
{
    updatePresence(PURPLE_STATUS_OFFLINE);
}

void dxim::onCmdStatusMsg()
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_WARNING, "", _("Warning"), _("Any account isn't connected.\nConnect someone and try again."), _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
        updateMenu();
        return;
    }
    Gtk::Dialog dialog(_("Change status message"));
    Gtk::HBox msgBox;
    Gtk::Label msgLabel(_("Status message:"));
    Gtk::Entry msg;
    msgBox.pack_start(msgLabel, Gtk::PACK_SHRINK);
    msgBox.pack_start(msg, Gtk::PACK_EXPAND_WIDGET);
    dialog.get_vbox()->pack_start(msgBox);
    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.show_all();
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
        for(size_t i=0; i<accounts.size(); i++)
            accounts[i].setStatusMessage(msg.get_text());
    }
}

void dxim::onCmdCloseTab()
{
    if(m_tabbook.get_n_pages())
    {
        TabItem *current = static_cast<TabItem*>(m_tabbook.get_nth_page(m_tabbook.get_current_page()));
        TabType type = current->getType();
        if(type == TABFILETRANSFER)
        {
            m_fttab->hide();
            updateMenu();
            return;
        }
        if(type == TABIM || type == TABCHAT)
        {
            static_cast<ConvTabItem*>(current)->getConversation()->leave();
        }
        m_tabbook.remove_page(*current);
        updateMenu();
    }
}

void dxim::onCmdClear()
{
    if(m_tabbook.get_n_pages())
    {
        int current = m_tabbook.get_current_page();
        static_cast<TabItem*>(m_tabbook.get_nth_page(current))->clearText();
    }
}

void dxim::onCmdClearAll()
{
    if(m_tabbook.get_n_pages())
    {
        TabItem *item = NULL;
        for(int i=0; i<m_tabbook.get_n_pages(); i++)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            item->clearText();
            item->removeHighligth();
        }
        updateTrayIconPresence();
    }
}

void dxim::onCmdFileTransfers()
{
    if(!m_fttab)
    {
        ImageLabel *label = Gtk::manage(new ImageLabel(Gtk::StockID("dxim-filetransfer"), _("File transfers")));
        m_fttab = new FTTabItem(m_cb, &m_tabbook, label);
        m_tabbook.append_page(*m_fttab, *label);
        m_fttab->signalFTTabItemClosed.connect(sigc::mem_fun(this, &dxim::updateMenu));
    }
    m_fttab->show();
    m_tabbook.set_current_page(m_tabbook.page_num(*m_fttab));
    updateMenu();
}

void dxim::onCmdLogViewer()
{
    if(!m_log)
        m_log = new LogDialog(*this);
    m_log->show_all();
}

void dxim::onCmdFriendsList()
{
    DXPREFERENCES.m_rosterShown = !DXPREFERENCES.m_rosterShown;
    if(DXPREFERENCES.m_rosterShown) m_splitter.get_child2()->show();
    else m_splitter.get_child2()->hide();
}

void dxim::onCmdFriendsListChat()
{
    DXPREFERENCES.m_chatRosterShown = !DXPREFERENCES.m_chatRosterShown;
    if(m_tabbook.get_n_pages())
    {
        TabItem *item = NULL;
        for(int i=0; i<m_tabbook.get_n_pages(); i++)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            item->showHideUsers();
        }
    }
}

void dxim::onCmdShowAvatars()
{
    DXPREFERENCES.m_showAvatars = !DXPREFERENCES.m_showAvatars;
    m_roster->updateShowAvatars();
}

void dxim::onCmdHideOffline()
{
    DXPREFERENCES.m_hideOffline = !DXPREFERENCES.m_hideOffline;
    m_roster->filterChange();
}

void dxim::onCmdShowStatusmsg()
{
    DXPREFERENCES.m_showStatusmsg = !DXPREFERENCES.m_showStatusmsg;
    m_roster->updateShowStatusmsg();
}

void dxim::onCmdPlugins()
{
    PluginDialog dialog(*this, m_cb->getPurplemm());
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        m_cb->getPurplemm()->saveLoadedPlugins("/dxim/plugins/loaded");
    }
}

void dxim::onCmdPreferences()
{
    Glib::ustring oldStatus = DXPREFERENCES.m_themeStatusPath;
    Glib::ustring oldRole = DXPREFERENCES.m_themeRolePath;
    bool oldShowRosterTree = DXPREFERENCES.m_showRosterTree;
    ConfigDialog dialog(*this);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        if(oldStatus!=DXPREFERENCES.m_themeStatusPath)
            createStatusIcons();
        if(oldRole!=DXPREFERENCES.m_themeRolePath)
            createRoleIcons();
        if(oldShowRosterTree!=DXPREFERENCES.m_showRosterTree)
            m_roster->showTreeChange();
        m_tabbook.set_tab_pos(dxutils::getTabPosition());
        updateMenu();
        if(m_tabbook.get_n_pages())
        {
            for(int i=0; i<m_tabbook.get_n_pages(); i++)
            {
                static_cast<TabItem*>(m_tabbook.get_nth_page(i))->updatePreferences();
            }
        }
        m_roster->updateTheme();
        m_cb->getPurplemm()->setIdle(DXPREFERENCES.m_useAway, DXPREFERENCES.m_awayTimeout);
        if(m_log)
        {
            m_log->setLogPath(DXPREFERENCES.m_logPath);
            m_log->updatePreferences();
        }
    }
}

void dxim::updatePresence(PurpleStatusPrimitive presence)
{
    m_presence = presence;
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getEnabledAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        accounts[i].setStatus(m_presence);
    }
    if(this->get_visible()) updateTrayIconPresence();
}

//update trayicon for presence
void dxim::updateTrayIconPresence(PurpleStatusPrimitive presence)
{
    if(presence == PURPLE_STATUS_UNSET) presence = m_presence;
    switch(presence){
        case PURPLE_STATUS_AVAILABLE: m_statusIcon->set(Gtk::StockID("dxim-online")); break;
        case PURPLE_STATUS_INVISIBLE: m_statusIcon->set(Gtk::StockID("dxim-invisible")); break;
        case PURPLE_STATUS_AWAY: m_statusIcon->set(Gtk::StockID("dxim-away")); break;
        case PURPLE_STATUS_UNAVAILABLE: m_statusIcon->set(Gtk::StockID("dxim-dnd")); break;
        case PURPLE_STATUS_OFFLINE: m_statusIcon->set(Gtk::StockID("dxim-offline")); break;
        case PURPLE_STATUS_EXTENDED_AWAY: m_statusIcon->set(Gtk::StockID("dxim-idle")); break;
        //some presence mising
        default: m_statusIcon->set(Gtk::StockID("dxim-online")); break;
    }
    m_newMsg = false;
}

void dxim::updateTrayIconNewmsg()
{
    m_statusIcon->set(Gtk::StockID("dxim-newm"));
    m_newMsg = true;
}

void dxim::closeAccount(PURPLE::Account /*account*/)
{
    if(!m_cb->getPurplemm()->isSomeAccountConnected())
    {
        m_presence = PURPLE_STATUS_OFFLINE;
        if(!m_newMsg) updateTrayIconPresence();
    }
}

void dxim::onAccountConnected(PURPLE::Account account)
{
    if(m_presence == PURPLE_STATUS_OFFLINE)
    {
        m_presence = PURPLE_STATUS_AVAILABLE;
        updateTrayIconPresence();
    }
    //hack for disable idle after connected
    account.setStatus(m_presence);
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountConnect) dxutils::instance().showNotify(Glib::ustring::compose(_("Connected %1-%2"), account.getProtocol().getName(), account.getUsername()));
    if(m_tabbook.get_n_pages())
    {
        TabItem *item = NULL;
        for(int i=m_tabbook.get_n_pages()-1; i>-1; i--)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            if(item->getType() == TABIM || item->getType() == TABCHAT)
            {
                ConvTabItem *convItem = static_cast<ConvTabItem*>(item);
                if(convItem->getConversation()->getAccount() == account)
                {
                    convItem->appendStyledText(Glib::ustring::compose(_("%1 [%2]: Connected"), account.getUsername(), account.getProtocol().getName()), "user", "", false);
                }
            }
        }
    }
}

void dxim::onAccountDisconnected(PURPLE::Account account, Glib::ustring reason)
{
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountDisconnect)
    {
        if(reason.empty()) dxutils::instance().showNotify(Glib::ustring::compose(_("Disconnected %1-%2"), account.getProtocol().getName(), account.getUsername()));
        else dxutils::instance().showNotify(Glib::ustring::compose(_("Disconnected %1-%2\n%3"), account.getProtocol().getName(), account.getUsername(), reason));
    }
    closeAccount(account);
    if(m_tabbook.get_n_pages())
    {
        TabItem *item = NULL;
        for(int i=m_tabbook.get_n_pages()-1; i>-1; i--)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            if(item->getType() == TABIM || item->getType() == TABCHAT)
            {
                ConvTabItem *convItem = static_cast<ConvTabItem*>(item);
                if(convItem->getConversation()->getAccount() == account)
                {
                    convItem->appendStyledText(Glib::ustring::compose(_("%1 [%2]: Disconnected"), account.getUsername(), account.getProtocol().getName()), "user", "", false);
                }
            }
        }
    }
}

void dxim::onNewIMMsg(PURPLE::Conversation *, Glib::ustring, Glib::ustring, Glib::ustring, bool, bool)
{
    if(!this->get_visible() && DXPREFERENCES.m_notifyMessageTray)
    {
        updateTrayIconNewmsg();
    }
}

void dxim::onNewChatMsg(PURPLE::Conversation *, Glib::ustring, Glib::ustring, Glib::ustring, bool delayed, bool, bool)
{
    if(!delayed && !this->get_visible() && DXPREFERENCES.m_notifyMessageTray)
    {
        updateTrayIconNewmsg();
    }
}

void dxim::onNewIM(PURPLE::Conversation *conv)
{
    ImTabItem *tab = m_tabbook.createImTab(conv, m_cb);
    tab->signalHighlight.connect(sigc::mem_fun(this, &dxim::onTabHighlight));
    tab->signalShowLog.connect(sigc::mem_fun(this, &dxim::onShowLog));
    updateMenu();
}

void dxim::onNewChat(PURPLE::Conversation *conv)
{
    ChatTabItem *tab = m_tabbook.createChatTab(conv, m_cb);
    if(!this->get_visible() && DXPREFERENCES.m_raiseWindow) this->show();
    tab->signalHighlight.connect(sigc::mem_fun(this, &dxim::onTabHighlight));
    tab->signalShowLog.connect(sigc::mem_fun(this, &dxim::onShowLog));
    updateMenu();
}

void dxim::onNewActionRequest(PURPLE::ActionRequest *request)
{
    ActionRequestDialog *dialog = new ActionRequestDialog(*this, request);
    dialog->show();
    dialog->present();
}

void dxim::onNewChoiceRequest(PURPLE::ChoiceRequest *request)
{
    ChoiceRequestDialog dialog(*this, request);
    dialog.run();
}

void dxim::onNewFieldsRequest(PURPLE::FieldsRequest *request)
{
    FieldsRequestDialog dialog(*this, request);
    dialog.run();
}

void dxim::onNewFileRequest(PURPLE::FileRequest *request)
{
    Gtk::FileChooserDialog dialog(request->getTitle(), request->getSaveDialog()?Gtk::FILE_CHOOSER_ACTION_SAVE:Gtk::FILE_CHOOSER_ACTION_OPEN);
    dialog.set_transient_for(*this);
    dialog.set_current_folder(Glib::get_home_dir());
    dialog.set_current_name(dxutils::fileName(request->getFilename()));
    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    dialog.add_button(request->getSaveDialog()?Gtk::Stock::SAVE:Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
    int result = dialog.run();
    if(result == Gtk::RESPONSE_OK)
    {
        request->setFilename(dialog.get_filename());
    }
    else
    {
        request->dennyRequest();
    }
}

void dxim::onNewInputRequest(PURPLE::InputRequest *request)
{
    InputRequestDialog dialog(*this, request);
    dialog.run();
}

void dxim::onPurpleError(Glib::ustring error)
{
    MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_ERROR, "", _("Error"), error, _("_Ok"), "");
    tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    updateMenu();
}

void dxim::onNotifyMessage(PurpleNotifyMsgType type, Glib::ustring title, Glib::ustring text)
{
    MsgTabItem *tab = m_tabbook.createMsgTab(ICO_INFO, type, "", title, text, _("_Ok"), "");
    tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    updateMenu();
}

void dxim::onNotifyEmail(Glib::ustring text, Glib::ustring url)
{
    MsgTabItem *tab = m_tabbook.createMsgTab(ICO_EMAIL, PURPLE_NOTIFY_MSG_INFO, "", _("New email"), text, _("_Show in browser"), url);
    tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    updateMenu();
}

void dxim::onNotifyUserinfo(PURPLE::Account account, Glib::ustring buddy, Glib::ustring text)
{
    Glib::ustring id = Glib::ustring::compose("%1-%2-%3", account.getUsername(), account.getProtocol().getName(), buddy);
    MsgTabItem *tab = NULL;
    if(m_tabbook.get_n_pages())
    {
        for(int i=0; i<m_tabbook.get_n_pages(); i++)
        {
            if(static_cast<TabItem*>(m_tabbook.get_nth_page(i))->getType() == TABMSG
               && static_cast<MsgTabItem*>(m_tabbook.get_nth_page(i))->getID() == id)
            {
                tab = static_cast<MsgTabItem*>(m_tabbook.get_nth_page(i));
                break;
            }
        }
    }
    if(!tab)
    {
        tab = m_tabbook.createMsgTab(ICO_INFO, PURPLE_NOTIFY_MSG_INFO, id, buddy, text, _("_Ok"), "");
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    }
    else
    {
        tab->clearMsgTab();
        tab->appendText(text);
    }
    updateMenu();
}

void dxim::onNotifyUri(Glib::ustring uri)
{
    MsgTabItem *tab = m_tabbook.createMsgTab(ICO_EMAIL, PURPLE_NOTIFY_MSG_INFO, "", _("Link"), uri, _("_Show in browser"), uri);
    tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    updateMenu();
}

void dxim::onNewFileTransfer(PURPLE::FileTransfer *ft)
{
    onCmdFileTransfers();
    m_fttab->newFileTransfer(ft);
}

void dxim::onCloseConvTab(PURPLE::Conversation *conv)
{
    if(m_tabbook.get_n_pages())
    {
        for(int i=0; i<m_tabbook.get_n_pages(); i++)
        {
            TabType type = static_cast<TabItem*>(m_tabbook.get_nth_page(i))->getType();
            if(type == TABIM || type == TABCHAT)
            {
                ConvTabItem *item = static_cast<ConvTabItem*>(m_tabbook.get_nth_page(i));
                if(*item->getConversation() == *conv)
                {
                    m_tabbook.remove_page(i);
                    break;
                }
            }
        }
        updateMenu();
        updateTrayIconPresence();
    }
}

#if GTKMM_MAJOR_VERSION == 2
    void dxim::onSwitchTab(GtkNotebookPage */*p*/, unsigned int n)
    {
        static_cast<TabItem*>(m_tabbook.get_nth_page(n))->removeHighligth();
        static_cast<TabItem*>(m_tabbook.get_nth_page(n))->focusCommandline();
        updateTrayIconPresence(m_presence);
    }
#else
    void dxim::onSwitchTab(Widget */*p*/, unsigned int n)
    {
        static_cast<TabItem*>(m_tabbook.get_nth_page(n))->removeHighligth();
        static_cast<TabItem*>(m_tabbook.get_nth_page(n))->focusCommandline();
        updateTrayIconPresence();
    }
#endif

void dxim::onTabHighlight()
{
    if(!this->get_visible())
    {
        if(DXPREFERENCES.m_raiseWindow) this->show();
        else updateTrayIconNewmsg();
    }
}

void dxim::onShowLog(PURPLE::Account account, Glib::ustring conversation, Glib::ustring path)
{
    Glib::ustring id = Glib::ustring::compose("Log-%1-%2-%3", account.getUsername(), account.getProtocol().getName(), conversation);
    MsgTabItem *tab = NULL;
    if(m_tabbook.get_n_pages())
    {
        for(int i=0; i<m_tabbook.get_n_pages(); i++)
        {
            if(static_cast<TabItem*>(m_tabbook.get_nth_page(i))->getType() == TABMSG
               && static_cast<MsgTabItem*>(m_tabbook.get_nth_page(i))->getID() == id)
            {
                tab = static_cast<MsgTabItem*>(m_tabbook.get_nth_page(i));
                break;
            }
        }
    }
    if(!tab)
    {
        tab = m_tabbook.createMsgTab(Gtk::StockID("dxim-logs"), id, conversation, _("_Ok"));
        tab->signalMsgTabItemClosed.connect(sigc::mem_fun(this, &dxim::onCloseMsgTab));
    }
    m_tabbook.set_current_page(m_tabbook.page_num(*tab));
    tab->loadLog(path);
    updateMenu();
}

void dxim::onCloseMsgTab(int tab)
{
    m_tabbook.remove_page(tab);
    updateMenu();
}

void dxim::updateMenu()
{
    if(DXPREFERENCES.m_logging) m_logAction->set_sensitive(true);
    else m_logAction->set_sensitive(false);
    bool enable = false;
    for(int i=0; i<m_tabbook.get_n_pages(); i++)
    {
        if(static_cast<TabItem*>(m_tabbook.get_nth_page(i))->get_visible())
        {
            enable = true;
            break;
        }
    }
    m_closetabAction->set_sensitive(enable);
    m_clearAction->set_sensitive(enable);
    m_clearallAction->set_sensitive(enable);
}

bool dxim::onAwayTimeout()
{
    if(!DXPREFERENCES.m_useAway)
    {
        m_cb->getPurplemm()->idleTouch();
        return true;
    }
    time_t idleTime = 0;
    bool away=false;
#ifdef _WIN32
    LASTINPUTINFO lif;
    lif.cbSize = sizeof(LASTINPUTINFO);
    DWORD tickCount;
    GetLastInputInfo(&lif);
    tickCount = GetTickCount();
    idleTime = (tickCount-lif.dwTime)/1000;
#else
 #ifdef USE_SCREENSAVER
    /* Query xscreensaver */
    static XScreenSaverInfo *mit_info = NULL;
    static int has_extension = -1;
    int event_base, error_base;
    if(has_extension == -1)
        has_extension = XScreenSaverQueryExtension(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), &event_base, &error_base);
    if(has_extension)
    {
        if (mit_info == NULL)
            mit_info = XScreenSaverAllocInfo();
        XScreenSaverQueryInfo(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), GDK_ROOT_WINDOW(), mit_info);
        idleTime = (mit_info->idle)/1000;
    }
    else
        idleTime = 0;
 #endif
#endif
    if(idleTime>DXPREFERENCES.m_awayTimeout*60)
    {
        away = true;
    }
    if(!away && !idleTime) m_cb->getPurplemm()->idleTouch();
    return true;
}

bool dxim::onCmdNextUnread()
{
    if(m_tabbook.get_n_pages())
    {
        TabItem *item = NULL;
        for(int i=m_tabbook.get_current_page(); i<m_tabbook.get_n_pages(); i++)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            if(item->isHighlighted())
            {
                m_tabbook.set_current_page(i);
                return true;
            }
        }
        for(int i=m_tabbook.get_current_page(); i>-1; i--)
        {
            item = static_cast<TabItem*>(m_tabbook.get_nth_page(i));
            if(item->isHighlighted())
            {
                m_tabbook.set_current_page(i);
                return true;
            }
        }
    }
    return true;
}

bool dxim::onCmdSelectTab(guint index)
{
    m_tabbook.set_current_page(index);
    return true;
}

#define USAGE_MSG _("\
\nUsage: dxim [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [PATH]          Use PATH for configuration.\n\
        -o, --offline      Start with all accounts set offline.\n\
\n")

int compare(const char* s1, const char* s2)
{
    int c1,c2;
    do
    {
        c1=*s1++;
        c2=*s2++;
    }
    while(c1 && (c1==c2));
    return c1-c2;
}

int main(int argc, char *argv[])
{
    bool offline = false;
    for(int i=0; i<argc; ++i)
    {
        if(compare(argv[i],"-v")==0 || compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
            exit(0);
        }
        if(compare(argv[i],"-h")==0 || compare(argv[i],"--help")==0)
        {
            fprintf(stdout, USAGE_MSG);
            exit(0);
        }
        if(compare(argv[i],"-l")==0)
        {
            DXPREFERENCES.setIniPath(argv[i+1]);
        }
        if(compare(argv[i],"-o")==0 || compare(argv[i],"--offline")==0)
        {
            offline = true;
        }
    }
#ifdef ENABLE_NLS
    bindtextdomain("dxim", LOCALEDIR);
    bind_textdomain_codeset("dxim", "UTF-8");
    bind_textdomain_codeset("pidgin", "UTF-8");
    textdomain("dxim");
#endif //ENABLE_NLS
    DXPREFERENCES.load();
    if(DXPREFERENCES.m_oneInstance && isInstanceRunningAlready(argc, argv))
        return 0;
    Gtk::Main kit(argc, argv);
    makeIcons();
    dxim window(offline);
    window.show();
    if(DXPREFERENCES.m_minToTray)
        window.hide();
    kit.run();
    return 0;
}

