/*
 *      roster.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <purplemm.h>
#include "roster.h"
#include "dialogs/chatdialog.h"
#include "dialogs/imdialog.h"
#include "dialogs/chatjoindialog.h"
#include "dialogs/accounteditor.h"
#include "dxutils.h"
#include "config.h"
#include "widgets/imagelabel.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"
#include "dialogs/logdialog.h"

#define MAX_AVATAR_SIZE 32

dxColor makeOfflineColor(dxColor color)
{
    dxColor offline = color;
    gushort r = offline.get_red();
    gushort g = offline.get_green();
    gushort b = offline.get_blue();
    if(r==g && r==b && !r)
    {
        offline = dxColor("#7F7F7F");
    }
    else
    {
        r=(50*r)/100;
        g=(50*g)/100;
        b=(50*b)/100;
        offline.set_red(r);
        offline.set_green(g);
        offline.set_blue(b);
    }
    return offline;
}

Glib::ustring makeMarkup(bool online, Glib::ustring name, Glib::ustring statusMsg)
{
    if(online)
    {
        if(statusMsg.empty())
        {
            return Glib::ustring::compose("<span foreground=\"%1\">%2</span>",
                                            DXPREFERENCES.m_textColor.to_string(),
                                            name);
        }
        else
        {
            return Glib::ustring::compose("<span foreground=\"%1\">%2</span>\n<span foreground=\"%3\">%4</span>",
                                            DXPREFERENCES.m_textColor.to_string(),
                                            name,
                                            makeOfflineColor(DXPREFERENCES.m_textColor).to_string(),
                                            statusMsg);
        }
    }
    else
    {
        if(statusMsg.empty())
        {
            return Glib::ustring::compose("<span foreground=\"%1\">%2</span>",
                                            makeOfflineColor(DXPREFERENCES.m_textColor).to_string(),
                                            name);
        }
        else
        {
            return Glib::ustring::compose("<span foreground=\"%1\">%2\n%3</span>",
                                            makeOfflineColor(DXPREFERENCES.m_textColor).to_string(),
                                            name,
                                            statusMsg);
        }
    }
}

Roster::Roster(dxgCallbacks *cb) : m_cb(cb)
{
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_rosterModel = Gtk::TreeStore::create(m_rosterColumns);
    m_rosterModel->set_sort_column(2, Gtk::SORT_ASCENDING);
    m_rosterModelFilter = Gtk::TreeModelFilter::create(m_rosterModel);
    m_rosterModelFilter->set_visible_func(sigc::mem_fun(this, &Roster::filterRoster));
    m_roster.set_model(m_rosterModelFilter);
    m_roster.append_column(" ", m_rosterColumns.m_icon);
    m_usernameRenderer.property_ellipsize() = Pango::ELLIPSIZE_END;
    Gtk::TreeView::Column *uColumn = Gtk::manage(new Gtk::TreeView::Column("username", m_usernameRenderer));
    uColumn->add_attribute(m_usernameRenderer.property_markup(), m_rosterColumns.m_markup);
    uColumn->set_expand(true);
    m_roster.append_column(*uColumn);
    m_protocolColumn = Gtk::manage(new Gtk::TreeView::Column("protocol"));
    m_protocolColumn->pack_start(m_rosterColumns.m_protocol, false);
    m_roster.append_column(*m_protocolColumn);
    if(!DXPREFERENCES.m_showRosterTree) m_protocolColumn->set_visible(false);
    m_avatarColumn = Gtk::manage(new Gtk::TreeView::Column("avatar"));
    m_avatarColumn->pack_start(m_rosterColumns.m_avatar, false);
    m_roster.append_column(*m_avatarColumn);
    if(!DXPREFERENCES.m_showAvatars) m_avatarColumn->set_visible(false);
    m_roster.set_headers_visible(false);
    m_roster.set_tooltip_column(9);
#if GTKMM_MAJOR_VERSION == 2
    if(DXPREFERENCES.m_sameList) m_roster.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_roster.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
#else
    if(DXPREFERENCES.m_sameList) m_roster.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    m_roster.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
#endif
    swin->add(m_roster);
    add(*swin);

    m_cb->signalBuddylistLoaded.connect(sigc::mem_fun(this, &Roster::onBuddylistLoaded));
    m_cb->signalAccountAdded.connect(sigc::mem_fun(this, &Roster::onAccountAdded));
    m_cb->signalAccountRemoved.connect(sigc::mem_fun(this, &Roster::onAccountRemoved));
    m_cb->signalAccountConnecting.connect(sigc::mem_fun(this, &Roster::onAccountConnecting));
    m_cb->signalAccountConnected.connect(sigc::mem_fun(this, &Roster::onAccountConnected));
    m_cb->signalAccountDisconnected.connect(sigc::mem_fun(this, &Roster::onAccountDisconnected));
    m_cb->signalAccountStatusChanged.connect(sigc::mem_fun(this, &Roster::onAccountStatusChanged));
    m_cb->signalAccountSignedOn.connect(sigc::mem_fun(this, &Roster::onAccountSignedOn));
    m_cb->signalBuddySignedOn.connect(sigc::mem_fun(this, &Roster::onBuddySignedOn));
    m_cb->signalBuddySignedOff.connect(sigc::mem_fun(this, &Roster::onBuddySignedOff));
    m_cb->signalBuddyStatusChanged.connect(sigc::mem_fun(this, &Roster::onBuddyStatusChanged));
    m_cb->signalBuddyIconChanged.connect(sigc::mem_fun(this, &Roster::onBuddyIconChanged));
    m_cb->signalBuddyAdded.connect(sigc::mem_fun(this, &Roster::onBuddyAdded));
    m_cb->signalBuddyRemoved.connect(sigc::mem_fun(this, &Roster::onBuddyRemoved));
    m_cb->signalChatAdded.connect(sigc::mem_fun(this, &Roster::onChatAdded));
    m_cb->signalChatRemoved.connect(sigc::mem_fun(this, &Roster::onChatRemoved));

    m_roster.signal_button_press_event().connect(sigc::mem_fun(this, &Roster::onButtonPressed), false);
    m_roster.signal_row_activated().connect(sigc::mem_fun(this, &Roster::onDoubleClick));
}

void Roster::updateTheme()
{
#if GTKMM_MAJOR_VERSION == 2
    if(DXPREFERENCES.m_sameList) m_roster.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_roster.modify_font(get_pango_context()->get_font_description());
    m_roster.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
#else
    if(DXPREFERENCES.m_sameList) m_roster.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_roster.override_font(get_pango_context()->get_font_description());
    m_roster.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
#endif
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Children accounts = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator iter = accounts.begin(); iter != accounts.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            PURPLE::Account account = row[m_rosterColumns.m_account];
            row[m_rosterColumns.m_markup] = makeMarkup(account.isConnected(), account.getUsername(), "");
            row[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(account.getStatus());
            Gtk::TreeModel::Children groups = row->children();
            for(Gtk::TreeModel::Children::iterator groupIt = groups.begin(); groupIt != groups.end(); ++groupIt)
            {
                (*groupIt)[m_rosterColumns.m_markup] = makeMarkup(true, (*groupIt)[m_rosterColumns.m_ID], "");
                Gtk::TreeModel::Children children = (*groupIt)->children();
                for(Gtk::TreeModel::Children::iterator childIt = children.begin(); childIt != children.end(); ++childIt)
                {
                    if((*childIt)[m_rosterColumns.m_type] == Roster::CHAT)
                    {
                        PURPLE::Chat chat = (*childIt)[m_rosterColumns.m_chat];
                        (*childIt)[m_rosterColumns.m_markup] = makeMarkup(true, chat.getAlias(), "");
                    }
                    else
                    {
                        PURPLE::Buddy buddy = (*childIt)[m_rosterColumns.m_buddy];
                        (*childIt)[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
                        (*childIt)[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(buddy.getStatusPrimitive());
                    }
                }
            }
        }
    }
    else
    {
        Gtk::TreeModel::Children children = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator childIt = children.begin(); childIt != children.end(); ++childIt)
        {
            if((*childIt)[m_rosterColumns.m_type] == Roster::CHAT)
            {
                PURPLE::Chat chat = (*childIt)[m_rosterColumns.m_chat];
                (*childIt)[m_rosterColumns.m_markup] = makeMarkup(true, chat.getAlias(), "");
            }
            else
            {
                PURPLE::Buddy buddy = (*childIt)[m_rosterColumns.m_buddy];
                (*childIt)[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
                (*childIt)[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(buddy.getStatusPrimitive());
            }
        }
    }
}

bool Roster::onButtonPressed(GdkEventButton* event)
{
    bool value = false;
    if(event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
        Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
        if(refSelection)
        {
            Gtk::TreeModel::Path path;
            m_roster.get_path_at_pos(event->x, event->y, path);
            refSelection->select(path);
            Gtk::TreeModel::iterator iter = refSelection->get_selected();
            if(iter)
            {
                itemType type = (*iter)[m_rosterColumns.m_type];
                if(type == Roster::CHAT)
                {
                    Gtk::Menu *popup = new Gtk::Menu();
                    Gtk::MenuItem* item = new Gtk::MenuItem(_("Join"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupJoin));
                    popup->append(*item);
                    if(DXPREFERENCES.m_logging)
                    {
                        item = new Gtk::MenuItem(_("Show logs"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupChatLogs));
                        popup->append(*item);
                    }
                    popup->append(*(new Gtk::SeparatorMenuItem()));
                    item = new Gtk::MenuItem(_("Modify"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupModifyChat));
                    popup->append(*item);
                    item = new Gtk::MenuItem(_("Remove"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupRemoveChat));
                    popup->append(*item);
                    popup->accelerate(*this);
                    popup->show_all();
                    popup->popup(event->button, event->time);
                    value = true;
                }
                else if(type == Roster::ACCOUNT)
                {
                    PURPLE::Account acc = (*iter)[m_rosterColumns.m_account];
                    Gtk::Menu *popup = new Gtk::Menu();
                    if(acc.isEnabled())
                    {
                        Gtk::MenuItem *item = new Gtk::MenuItem(_("Status"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAccountStatus));
                        popup->append(*item);
                        popup->append(*(new Gtk::SeparatorMenuItem()));
                        item = new Gtk::MenuItem(_("Reconnect"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupReconnectAccount));
                        popup->append(*item);
                        popup->append(*(new Gtk::SeparatorMenuItem()));
                        item = new Gtk::MenuItem(_("Disable"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupEnableAccount));
                        popup->append(*item);
                    }
                    else
                    {
                        Gtk::MenuItem *item = new Gtk::MenuItem(_("Enable"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupEnableAccount));
                        popup->append(*item);
                    }
                    popup->append(*(new Gtk::SeparatorMenuItem()));
                    Gtk::MenuItem *item = new Gtk::MenuItem(_("Account settings"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupConfigAccount));
                    popup->append(*item);
                    item = new Gtk::MenuItem(_("Alias"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAliasAccount));
                    popup->append(*item);
                    if(DXPREFERENCES.m_logging)
                    {
                        item = new Gtk::MenuItem(_("Show logs"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAccountLogs));
                        popup->append(*item);
                    }
                    if(acc.isConnected())
                    {
                        popup->append(*(new Gtk::SeparatorMenuItem()));
                        Gtk::MenuItem *item = new Gtk::MenuItem(_("New IM"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAccountIm));
                        popup->append(*item);
                        if(acc.supportsChats())
                        {
                            item = new Gtk::MenuItem(_("Join chat"));
                            item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAccountChat));
                            popup->append(*item);
                        }
                        popup->append(*(new Gtk::SeparatorMenuItem()));
                        item = new Gtk::MenuItem(_("Add buddy"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAddBuddy));
                        popup->append(*item);
                        if(acc.supportsChats())
                        {
                            item = new Gtk::MenuItem(_("Add chat"));
                            item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAddChat));
                            popup->append(*item);
                        }
                        item = new Gtk::MenuItem(_("Add group"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupAddGroup));
                        popup->append(*item);
                        std::vector<std::string> v = acc.getCommandsList();
                        if(v.size())
                            popup->append(*(new Gtk::SeparatorMenuItem()));
                        FOREACH(std::vector<std::string>, v, it)
                        {
                            if((*it).compare("----"))
                            {
                            item = new Gtk::MenuItem((*it));
                            item->signal_activate().connect(sigc::bind<Glib::ustring>(sigc::mem_fun(this, &Roster::onAccountCb), (*it)));
                            popup->append(*item);
                            }
                            else
                            {
                                popup->append(*(new Gtk::SeparatorMenuItem()));
                            }
                        }
                    }
                    popup->accelerate(*this);
                    popup->show_all();
                    popup->popup(event->button, event->time);
                    value = true;
                }
                else if(type == Roster::BUDDY)
                {
                    Gtk::Menu *popup = new Gtk::Menu();
                    PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                    Gtk::MenuItem* item = NULL;
                    bool online = buddy.isOnline();
                    if(online)
                    {
                        item = new Gtk::MenuItem(_("IM"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupIm));
                        popup->append(*item);
                    }
                    item = new Gtk::MenuItem(_("Info"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupInfo));
                    popup->append(*item);
                    if(online && buddy.getAccount().supportsSendFile())
                    {
                        item = new Gtk::MenuItem(_("Send file"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupSendFile));
                        popup->append(*item);
                    }
                    if(DXPREFERENCES.m_logging)
                    {
                        item = new Gtk::MenuItem(_("Show logs"));
                        item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupBuddyLogs));
                        popup->append(*item);
                    }
                    popup->append(*(new Gtk::SeparatorMenuItem()));
                    item = new Gtk::MenuItem(_("Rename"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupRenameBuddy));
                    popup->append(*item);
                    item = new Gtk::MenuItem(_("Move"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupMoveBuddy));
                    popup->append(*item);
                    item = new Gtk::MenuItem(_("Remove"));
                    item->signal_activate().connect(sigc::mem_fun(this, &Roster::onPopupRemoveBuddy));
                    popup->append(*item);
                    std::vector<PURPLE::Buddy::BuddyMenuItem> v = buddy.getMenuItems();
                    if(v.size()) popup->append(*(new Gtk::SeparatorMenuItem()));
                    FOREACH(std::vector<PURPLE::Buddy::BuddyMenuItem>, v, it)
                    {
                        Gtk::MenuItem* item = new Gtk::MenuItem((*it).getLabel(), true);
                        item->signal_activate().connect(sigc::bind<PURPLE::Buddy::BuddyMenuItem>(sigc::mem_fun(this, &Roster::onBuddyCb), (*it)));
                        popup->append(*item);
                    }
                    popup->accelerate(*this);
                    popup->show_all();
                    popup->popup(event->button, event->time);
                    value = true;
                }
            }
        }
    }
    return value;
}

void Roster::onDoubleClick(const Gtk::TreeModel::Path &/*path*/, Gtk::TreeViewColumn */*column*/)
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            itemType type = (*iter)[m_rosterColumns.m_type];
            if(type == Roster::CHAT)
            {
                PURPLE::Chat chat = (*iter)[m_rosterColumns.m_chat];
                chat.join();
            }
            else if(type == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                m_cb->getPurplemm()->createConversation(buddy.getAccount(), buddy);
            }
        }
    }
}

void Roster::onAccountCb(Glib::ustring cmd)
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account acc = (*iter)[m_rosterColumns.m_account];
                acc.callCommand(cmd);
            }
        }
    }
}

void Roster::onPopupAccountIm()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                IMDialog dialog((*iter)[m_rosterColumns.m_account], m_cb->getPurplemm());
                dialog.run();
            }
        }
    }
}

void Roster::onPopupAccountChat()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                ChatJoinDialog *dialog = new ChatJoinDialog ((*iter)[m_rosterColumns.m_account], m_cb);
                dialog->show();
                dialog->present();
            }
        }
    }
}

void Roster::onPopupAccountStatus()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                Gtk::Dialog dialog(_("Set status"));
                Gtk::Table table;
                table.attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(account.getProtocol().getPurpleID()), account.getUsername())), 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Status:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                struct StatusModelColumns : public Gtk::TreeModel::ColumnRecord
                {
                    Gtk::TreeModelColumn<Glib::ustring> m_name;
                    Gtk::TreeModelColumn<Glib::ustring> m_id;

                    StatusModelColumns() { add(m_name); add(m_id); }
                };
                const StatusModelColumns statusColumns;
                Glib::RefPtr<Gtk::ListStore> statusModel = Gtk::ListStore::create(statusColumns);
                Gtk::ComboBox statuses;
                statuses.set_model(statusModel);
                statuses.pack_start(statusColumns.m_name);
                std::vector<PURPLE::Account::AccountStatusItem> v = account.getStatusMenuItems();
                FOREACH(std::vector<PURPLE::Account::AccountStatusItem>, v, it)
                {
                    Gtk::TreeModel::Row row = *(statusModel->append());
                    row[statusColumns.m_name] = (*it).getName();
                    row[statusColumns.m_id] = (*it).getID();
                }
                statuses.set_active(0);
                table.attach(statuses, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Status message:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::Entry msg;
                msg.set_text(account.getStatusMessage());
                table.attach(msg, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                dialog.get_vbox()->pack_start(table);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    Glib::ustring id = "";
                    Gtk::TreeModel::iterator iter = statuses.get_active();
                    if(iter)
                    {
                        Gtk::TreeModel::Row row = *iter;
                        if(row)
                        {
                            id = row[statusColumns.m_id];
                        }
                    }
                    if(!id.empty())
                    {
                        account.setStatus(id);
                        if(account.isConnected()) account.setStatusMessage(msg.get_text());
                    }
                }
            }
        }
    }
}

void Roster::onBuddyCb(PURPLE::Buddy::BuddyMenuItem item)
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                item.runCallback();
            }
        }
    }
}

void Roster::onPopupJoin()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::CHAT)
            {
                PURPLE::Chat chat = (*iter)[m_rosterColumns.m_chat];
                chat.join();
            }
        }
    }
}

void Roster::onPopupIm()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                m_cb->getPurplemm()->createConversation(buddy.getAccount(), buddy);
            }
        }
    }
}

void Roster::onPopupInfo()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                buddy.retrieveInfo();
            }
        }
    }
}

void Roster::onPopupSendFile()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                buddy.sendFile();
            }
        }
    }
}

void Roster::onPopupAddBuddy()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                Gtk::Dialog dialog(_("Add friend"));
                Gtk::Table table;
                table.attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(account.getProtocol().getPurpleID()), account.getUsername())), 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("ID:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::Entry id;
                table.attach(id, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Nick:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::Entry nick;
                table.attach(nick, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Group:"), Gtk::ALIGN_START)), 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::ComboBoxText groups(true);
                std::vector<std::string> v = account.getAllGroups();
                bool friends = false;
                FOREACH(std::vector<std::string>, v, it)
                {
                    if((*it) == _("Friends"))
                        friends = true;
                    groups.append(*it);
                }
                if(!friends) groups.insert(0, _("Friends"));
                groups.set_active(0);
                table.attach(groups, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                dialog.get_vbox()->pack_start(table);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK && !id.get_text().empty())
                {
                    Glib::ustring group = groups.get_entry_text();
                    if(group.empty()) group = _("Friends");
                    account.addBuddy(id.get_text(),
                             group,
                             _("I want to add you in buddies"),
                             nick.get_text());
                }
            }
        }
    }
}

void Roster::onPopupAddChat()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                ChatDialog dialog(account);
                dialog.run();
            }
        }
    }
}

void Roster::onPopupAddGroup()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                Gtk::Dialog dialog(_("Add group"));
                Gtk::HBox groupBox;
                Gtk::Label groupLabel(_("New group:"));
                Gtk::Entry group;
                groupBox.pack_start(groupLabel, Gtk::PACK_SHRINK);
                groupBox.pack_start(group, Gtk::PACK_EXPAND_WIDGET);
                dialog.get_vbox()->pack_start(groupBox);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    account.addGroup(group.get_text());
                }
            }
        }
    }
}

void Roster::onPopupModifyChat()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::CHAT)
            {
                PURPLE::Chat chat = (*iter)[m_rosterColumns.m_chat];
                Glib::ustring alias = chat.getAlias();
                ChatDialog dialog(chat);
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    onChatRemoved(chat.getAccount(), alias);
                    addChat(chat);
                }
            }
        }
    }
}

void Roster::onPopupRemoveChat()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::CHAT)
            {
                PURPLE::Chat chat = (*iter)[m_rosterColumns.m_chat];
                Gtk::MessageDialog dialog(Glib::ustring::compose(_("Do you want really remove %1?"), chat.getAlias()), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
                int result = dialog.run();
                if(result == Gtk::RESPONSE_YES)
                {
                    chat.getAccount().removeChat(chat);
                }
            }
        }
    }
}

void Roster::onPopupRenameBuddy()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = m_rosterModelFilter->convert_iter_to_child_iter(refSelection->get_selected());
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                Gtk::Dialog dialog(_("Rename friend"));
                Gtk::Table table;
                table.attach(*Gtk::manage(new Gtk::Label(_("ID:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(buddy.getAccount().getProtocol().getPurpleID()), buddy.getUsername())), 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Alias:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::Entry alias;
                table.attach(alias, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                alias.set_text(buddy.getAlias());
                Gtk::CheckButton serveralias(_("Set on server too"));
                table.attach(serveralias, 0, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                if(buddy.getAccount().hasServerAliases())
                    serveralias.set_active(true);
                else
                    serveralias.set_active(false);
                dialog.get_vbox()->pack_start(table);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                if(!buddy.getAccount().hasServerAliases())
                    serveralias.hide();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK && !alias.get_text().empty()
                    && buddy.getAlias().compare(alias.get_text()))
                {
                    buddy.setAlias(alias.get_text(), serveralias.get_active());
                    (*iter)[m_rosterColumns.m_name] = alias.get_text();
                    (*iter)[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), alias.get_text(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
                    updateTooltip(*iter);
                }
            }
        }
    }
}

void Roster::onPopupMoveBuddy()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                Gtk::Dialog dialog(_("Move friend"));
                Gtk::Table table;
                table.attach(*Gtk::manage(new Gtk::Label(_("ID:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new ImageLabel(dxutils::getProtocolIcon(buddy.getAccount().getProtocol().getPurpleID()), buddy.getUsername())), 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                table.attach(*Gtk::manage(new Gtk::Label(_("Group:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                Gtk::ComboBoxText groups(true);
                std::vector<std::string> v = buddy.getAccount().getAllGroups();
                bool friends = false;
                FOREACH(std::vector<std::string>, v, it)
                {
                    if((*it) != buddy.getGroupName())
                        groups.append(*it);
                    if((*it) == _("Friends"))
                        friends = true;
                }
                if(buddy.getGroupName() != _("Friends") && !friends) groups.insert(0, _("Friends"));
                groups.set_active(0);
                table.attach(groups, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
                dialog.get_vbox()->pack_start(table);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    std::string oldgroup = buddy.getGroupName();
                    std::string newgroup = groups.get_entry_text();
                    if(newgroup.empty()) newgroup = _("Friends");
                    if(oldgroup != newgroup) buddy.getAccount().moveBuddy(buddy, newgroup);
                }
            }
        }
    }
}

void Roster::onPopupRemoveBuddy()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                Gtk::MessageDialog dialog(Glib::ustring::compose(_("Do you want really remove %1?"), buddy.getUsername()), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
                int result = dialog.run();
                if(result == Gtk::RESPONSE_YES)
                {
                    buddy.getAccount().removeBuddy(buddy);
                }
            }
        }
    }
}

void Roster::onPopupReconnectAccount()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                if(account.isConnecting())
                    return;
                if(account.isConnected())
                    account.disconnect();
                account.connect();
            }
        }
    }
}

void Roster::onPopupEnableAccount()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                if(account.isEnabled())
                    account.disconnect();
                else
                    account.connect();
            }
        }
    }
}

void Roster::onPopupAliasAccount()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                Gtk::Dialog dialog(_("Set alias"));
                Gtk::HBox aliasBox;
                Gtk::Label aliasLabel(_("Alias"));
                Gtk::Entry alias;
                alias.set_text(account.getAlias());
                aliasBox.pack_start(aliasLabel, Gtk::PACK_SHRINK);
                aliasBox.pack_start(alias, Gtk::PACK_EXPAND_WIDGET);
                dialog.get_vbox()->pack_start(aliasBox);
                dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
                dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
                dialog.show_all();
                int result = dialog.run();
                if(result == Gtk::RESPONSE_OK)
                {
                    account.setAlias(alias.get_text());
                }
            }
        }
    }
}

void Roster::onPopupConfigAccount()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                AccountEditor editor(account);
                if(editor.run() == Gtk::RESPONSE_OK)
                {
                    NewAccount *nac = new NewAccount;
                    editor.fillAccount(nac);
                    account.setOptions(nac->options);
                    account.setPassword(nac->password, nac->rememberPassword);
                    account.setCheckMail(nac->checkEmail);
                    account.setAlias(nac->alias);
                    nac->iconPath.empty()?account.removeBuddyIcon():account.setBuddyIcon(nac->iconPath);
                }
            }
        }
    }
}

void Roster::onPopupChatLogs()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::CHAT)
            {
                PURPLE::Chat chat = (*iter)[m_rosterColumns.m_chat];
                Glib::ustring username = chat.getAccount().getUsername();
                username = username.substr(0, username.find_last_of('/'));
                if(dxutils::hasLog(username, chat.getChatName(), false))
                {
                    LogDialog dialog(chat);
                    dialog.run();
                }
                else
                {
                    Gtk::MessageDialog(Glib::ustring::compose(_("Any logs for chat %1"), chat.getChatName())).run();
                }
            }
        }
    }
}

void Roster::onPopupBuddyLogs()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                Glib::ustring username = buddy.getAccount().getUsername();
                username = username.substr(0, username.find_last_of('/'));
                if(dxutils::hasLog(username, buddy.getUsername(), false))
                {
                    LogDialog dialog(buddy);
                    dialog.run();
                }
                else
                {
                    Gtk::MessageDialog(Glib::ustring::compose(_("Any logs for buddy %1"), buddy.getUsername())).run();
                }
            }
        }
    }
}

void Roster::onPopupAccountLogs()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_roster.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::ACCOUNT)
            {
                PURPLE::Account account = (*iter)[m_rosterColumns.m_account];
                Glib::ustring username = account.getUsername();
                username = username.substr(0, username.find_last_of('/'));
                if(dxutils::hasLog(username, "", true))
                {
                    LogDialog dialog(account);
                    dialog.run();
                }
                else
                {
                    Gtk::MessageDialog(Glib::ustring::compose(_("Any logs for account %1"), username)).run();
                }
            }
        }
    }
}

void Roster::onBuddylistLoaded()
{
    fillRoster();
}

void Roster::onAccountAdded(PURPLE::Account account)
{
    addAccount(account);
}

void Roster::onAccountRemoved(Glib::ustring account, Glib::ustring protocolID)
{
    Gtk::TreeModel::Row item = findAccountItem(account, protocolID);
    if(bool(item))
    {
        m_rosterModel->erase(item);
    }
}

void Roster::onAccountConnecting(PURPLE::Account account, Glib::ustring /*msg*/)
{
    Gtk::TreeModel::Row acc = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
    if(bool(acc))
    {
        acc[m_rosterColumns.m_icon] = ICO_CONNECTING->get_pixbuf();
    }
}

void Roster::onAccountConnected(PURPLE::Account account)
{
    Gtk::TreeModel::Row acc = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
    if(bool(acc))
    {
        acc[m_rosterColumns.m_markup] = makeMarkup(true, account.getUsername(), account.getStatusMessage());
        updateTooltip(acc);
        acc[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(account.getStatus());
    }
}

void Roster::onAccountDisconnected(PURPLE::Account account, Glib::ustring /*reason*/)
{
    Gtk::TreeModel::Row parent = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
    if(bool(parent))
    {
        parent[m_rosterColumns.m_markup] = makeMarkup(false, account.getUsername(), "");
        Gtk::TreeModel::Children groups = parent->children();
        Gtk::TreeModel::Children::iterator giter = groups.begin();
        while(giter != groups.end())
        {
            Gtk::TreeModel::Children gchildren = (*giter)->children();
            Gtk::TreeModel::Children::iterator iter = gchildren.begin();
            while(iter != gchildren.end())
            {
                iter = m_rosterModel->erase(iter);
            }
            giter = m_rosterModel->erase(giter);
        }
        updateTooltip(parent);
        parent[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE);
    }
}

void Roster::onAccountStatusChanged(PURPLE::Account account)
{
    Gtk::TreeModel::Row acc = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
    if(bool(acc))
    {
        acc[m_rosterColumns.m_markup] = makeMarkup(true, account.getUsername(), account.getStatusMessage());
        updateTooltip(acc);
        acc[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(account.getStatus());
    }
}

void Roster::onAccountSignedOn(PURPLE::Account account)
{
    std::vector<PURPLE::Buddy> b = account.getBuddies();
    for(guint i=0; i<b.size(); i++)
        addBuddy(b[i]);
    std::vector<PURPLE::Chat> ch = account.getChats();
    for(guint i=0; i<ch.size(); i++)
        addChat(ch[i]);
    Gtk::TreeModel::Row acc = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
    if(bool(acc))
    {
        m_roster.expand_row(m_rosterModel->get_path(*acc), true);
    }
}

void Roster::onBuddySignedOn(PURPLE::Buddy buddy)
{
    Gtk::TreeModel::Row item = findBuddyItem(buddy.getAccount(), buddy.getUsername());
    if(bool(item))
    {
        if(dxutils::now()-item[m_rosterColumns.m_lastChange]>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyConnect) dxutils::instance().showNotify(Glib::ustring::compose(_("%1\n(%2)\nis online"), buddy.getAlias(), buddy.getUsername()));
            if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundConnect) dxutils::playFile(DXPREFERENCES.m_pathConnect);
        }
        item[m_rosterColumns.m_name] = buddy.getAlias();
        item[m_rosterColumns.m_markup] = makeMarkup(true, buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
        item[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(buddy.getStatusPrimitive());
        item[m_rosterColumns.m_hidden] = false;
        item[m_rosterColumns.m_lastChange] = dxutils::now();
        Gtk::TreeModel::Row group = *item->parent();
        group[m_rosterColumns.m_hidden] = !buddy.getAccount().isGroupOnline(buddy.getGroupName());
        updateTooltip(item);
        Gtk::TreeModel::Row acc = findAccountItem(buddy.getAccount().getUsername(), buddy.getAccount().getProtocol().getPurpleID());
        if(bool(acc))
        {
            m_roster.expand_row(m_rosterModel->get_path(*acc), true);
        }
    }
}

void Roster::onBuddySignedOff(PURPLE::Buddy buddy)
{
    Gtk::TreeModel::Row item = findBuddyItem(buddy.getAccount(), buddy.getUsername());
    if(bool(item))
    {
        if(dxutils::now()-item[m_rosterColumns.m_lastChange]>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyDisconnect) dxutils::instance().showNotify(Glib::ustring::compose(_("%1\n(%2)\nis offline"), buddy.getAlias(), buddy.getUsername()));
            if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundDisconnect) dxutils::playFile(DXPREFERENCES.m_pathDisconnect);
        }
        item[m_rosterColumns.m_markup] = makeMarkup(false, buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
        item[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(PURPLE_STATUS_OFFLINE);
        item[m_rosterColumns.m_hidden] = true;
        item[m_rosterColumns.m_lastChange] = dxutils::now();
        Gtk::TreeModel::Row group = *item->parent();
        group[m_rosterColumns.m_hidden] = !buddy.getAccount().isGroupOnline(buddy.getGroupName());
        updateTooltip(item);
    }
}

void Roster::onBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive status)
{
    Gtk::TreeModel::Row item = findBuddyItem(buddy.getAccount(), buddy.getUsername());
    if(bool(item))
    {
        item[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
        item[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(status);
        item[m_rosterColumns.m_lastChange] = dxutils::now();
        updateTooltip(item);
    }
}

void Roster::onBuddyIconChanged(PURPLE::Buddy buddy)
{
    Gtk::TreeModel::Row item = findBuddyItem(buddy.getAccount(), buddy.getUsername());
    if(bool(item))
    {
        item[m_rosterColumns.m_avatar] = dxutils::createIcon(buddy.getIconPath(), MAX_AVATAR_SIZE);
    }
}

void Roster::onBuddyAdded(PURPLE::Buddy buddy)
{
    addBuddy(buddy);
}

void Roster::onBuddyRemoved(PURPLE::Account account, Glib::ustring buddy)
{
    Gtk::TreeModel::Row item = findBuddyItem(account, buddy);
    if(bool(item))
    {
        Gtk::TreeModel::Row group = *item->parent();
        m_rosterModel->erase(item);
        if(!group->children().size())
            m_rosterModel->erase(group);
    }
}

void Roster::onChatAdded(PURPLE::Chat chat)
{
    addChat(chat);
}

void Roster::onChatRemoved(PURPLE::Account account, Glib::ustring chat)
{
    Gtk::TreeModel::Row item = findChatItem(account, chat);
    if(bool(item))
    {
        Gtk::TreeModel::Row group = *item->parent();
        m_rosterModel->erase(item);
        if(!group->children().size())
            m_rosterModel->erase(group);
    }
}

Gtk::TreeModel::Row Roster::addAccount(PURPLE::Account account)
{
    Gtk::TreeModel::Row row = *(m_rosterModel->append());
    row[m_rosterColumns.m_ID] = account.getUsername();
    row[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(account.getStatus());
    row[m_rosterColumns.m_name] = account.getUsername();
    row[m_rosterColumns.m_markup] = makeMarkup(false, account.getUsername(), "");
    row[m_rosterColumns.m_type] = Roster::ACCOUNT;
    row[m_rosterColumns.m_account] = account;
    row[m_rosterColumns.m_hidden] = false;
    row[m_rosterColumns.m_protocol] = dxutils::getProtocolIcon(account.getProtocol().getPurpleID());
    updateTooltip(row);
    return row;
}

Gtk::TreeModel::Row Roster::addGroup(PURPLE::Account account, Glib::ustring name)
{
    Gtk::TreeModel::Row acc = findAccountItem(account.getUsername(),
            account.getProtocol().getPurpleID());
    if(bool(acc))
    {
        Gtk::TreeModel::Row group = findGroupItem(acc, name);
        if(!bool(group))
        {
            group = *(m_rosterModel->append(acc.children()));
            group[m_rosterColumns.m_ID] = name;
            group[m_rosterColumns.m_name] = name;
            group[m_rosterColumns.m_markup] = makeMarkup(true, name, "");
            group[m_rosterColumns.m_type] = Roster::GROUP;
            group[m_rosterColumns.m_account] = account;
        }
        group[m_rosterColumns.m_hidden] = true;
        return group;
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::addBuddy(PURPLE::Buddy buddy)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Row account = findAccountItem(buddy.getAccount().getUsername(),
                buddy.getAccount().getProtocol().getPurpleID());
        if(bool(account))
        {
            PURPLE::Account acc = account[m_rosterColumns.m_account];
            Gtk::TreeModel::Row group = addGroup(acc, buddy.getGroupName());
            if(bool(group))
            {
                Gtk::TreeModel::Row newbuddy = findBuddyItem(buddy.getAccount(), buddy.getUsername());
                if(!bool(newbuddy))
                {
                    newbuddy = *(m_rosterModel->append(group.children()));
                    newbuddy[m_rosterColumns.m_ID] = buddy.getUsername();
                    newbuddy[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(buddy.getStatusPrimitive());
                    newbuddy[m_rosterColumns.m_name] = buddy.getAlias();
                    newbuddy[m_rosterColumns.m_markup] = makeMarkup(false, buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
                    newbuddy[m_rosterColumns.m_type] = Roster::BUDDY;
                    newbuddy[m_rosterColumns.m_buddy] = buddy;
                    newbuddy[m_rosterColumns.m_account] = acc;
                    newbuddy[m_rosterColumns.m_hidden] = true;
                    newbuddy[m_rosterColumns.m_avatar] = dxutils::createIcon(buddy.getIconPath(), MAX_AVATAR_SIZE);
                    updateTooltip(newbuddy);
                }
                newbuddy[m_rosterColumns.m_lastChange] = dxutils::now();
                group[m_rosterColumns.m_hidden] = !buddy.getAccount().isGroupOnline(buddy.getGroupName());
                return newbuddy;
            }
        }
    }
    else
    {
        Gtk::TreeModel::Row newbuddy = findBuddyItem(buddy.getAccount(), buddy.getUsername());
        if(!bool(newbuddy))
        {
            newbuddy = *(m_rosterModel->append());
            newbuddy[m_rosterColumns.m_ID] = buddy.getUsername();
            newbuddy[m_rosterColumns.m_icon] = dxutils::getPresenceIcon(buddy.getStatusPrimitive());
            newbuddy[m_rosterColumns.m_name] = buddy.getAlias();
            newbuddy[m_rosterColumns.m_markup] = makeMarkup(false, buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
            newbuddy[m_rosterColumns.m_type] = Roster::BUDDY;
            newbuddy[m_rosterColumns.m_buddy] = buddy;
            newbuddy[m_rosterColumns.m_account] = buddy.getAccount();
            newbuddy[m_rosterColumns.m_hidden] = true;
            newbuddy[m_rosterColumns.m_avatar] = dxutils::createIcon(buddy.getIconPath(), MAX_AVATAR_SIZE);
            updateTooltip(newbuddy);
        }
        newbuddy[m_rosterColumns.m_lastChange] = dxutils::now();
        return newbuddy;
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::addChat(PURPLE::Chat chat)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Row account = findAccountItem(chat.getAccount().getUsername(),
                chat.getAccount().getProtocol().getPurpleID());
        if(bool(account))
        {
            PURPLE::Account acc = account[m_rosterColumns.m_account];
            Gtk::TreeModel::Row group = addGroup(acc, chat.getGroupName());
            if(bool(group))
            {
                Gtk::TreeModel::Row newchat = findChatItem(chat.getAccount(), chat.getAlias());
                if(!bool(newchat))
                {
                    newchat = *(m_rosterModel->append(group.children()));
                    newchat[m_rosterColumns.m_ID] = chat.getAlias();
                    newchat[m_rosterColumns.m_icon] = ICO_CHAT->get_pixbuf();
                    newchat[m_rosterColumns.m_name] = chat.getAlias();
                    newchat[m_rosterColumns.m_markup] = makeMarkup(true, chat.getAlias(), "");
                    newchat[m_rosterColumns.m_type] = Roster::CHAT;
                    newchat[m_rosterColumns.m_chat] = chat;
                    newchat[m_rosterColumns.m_account] = acc;
                    newchat[m_rosterColumns.m_hidden] = false;
                    updateTooltip(newchat);
                }
                group[m_rosterColumns.m_hidden] = !chat.getAccount().isGroupOnline(chat.getGroupName());
                return newchat;
            }
        }
    }
    else
    {
        Gtk::TreeModel::Row newchat = findChatItem(chat.getAccount(), chat.getAlias());
        if(!bool(newchat))
        {
            newchat = *(m_rosterModel->append());
            newchat[m_rosterColumns.m_ID] = chat.getAlias();
            newchat[m_rosterColumns.m_icon] = ICO_CHAT->get_pixbuf();
            newchat[m_rosterColumns.m_name] = chat.getAlias();
            newchat[m_rosterColumns.m_markup] = makeMarkup(true, chat.getAlias(), "");
            newchat[m_rosterColumns.m_type] = Roster::CHAT;
            newchat[m_rosterColumns.m_chat] = chat;
            newchat[m_rosterColumns.m_account] = chat.getAccount();
            newchat[m_rosterColumns.m_hidden] = false;
            updateTooltip(newchat);
        }
        return newchat;
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::findAccountItem(Glib::ustring ID, Glib::ustring protocol)
{
    Gtk::TreeModel::Children children = m_rosterModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        Gtk::TreeModel::Row row = *iter;
        PURPLE::Account account = row[m_rosterColumns.m_account];
        if(row[m_rosterColumns.m_type] == Roster::ACCOUNT && row[m_rosterColumns.m_ID] == ID
                && account.getProtocol().getPurpleID() == protocol)
        {
            return row;
        }
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::findGroupItem(Gtk::TreeModel::Row parent, Glib::ustring text)
{
    Gtk::TreeModel::Children children = parent->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        Gtk::TreeModel::Row row = *iter;
        if(row[m_rosterColumns.m_ID] == text
                && row[m_rosterColumns.m_type] == Roster::GROUP)
        {
            return row;
        }
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::findBuddyItem(PURPLE::Account account, Glib::ustring buddy)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Row parent = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
        if(bool(parent))
        {
            Gtk::TreeModel::Children children = parent->children();
            for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
            {
                Gtk::TreeModel::Row group = *iter;
                Gtk::TreeModel::Children buddies = group->children();
                for(Gtk::TreeModel::Children::iterator biter = buddies.begin(); biter != buddies.end(); ++biter)
                {
                    Gtk::TreeModel::Row row = *biter;
                    if(row[m_rosterColumns.m_type] == Roster::BUDDY)
                    {
                        if(buddy.compare(row[m_rosterColumns.m_ID])==0)
                            return row;
                    }
                }
            }
        }
    }
    else
    {
        Gtk::TreeModel::Children children = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            PURPLE::Account baccount = row[m_rosterColumns.m_account];
            if(row[m_rosterColumns.m_type] == Roster::BUDDY && baccount == account
                    && buddy.compare(row[m_rosterColumns.m_ID])==0)
            {
                return row;
            }
        }
    }
    return Gtk::TreeModel::Row();
}

Gtk::TreeModel::Row Roster::findChatItem(PURPLE::Account account, Glib::ustring chat)
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Row parent = findAccountItem(account.getUsername(), account.getProtocol().getPurpleID());
        if(bool(parent))
        {
            Gtk::TreeModel::Children children = parent->children();
            for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
            {
                Gtk::TreeModel::Row group = *iter;
                Gtk::TreeModel::Children buddies = group->children();
                for(Gtk::TreeModel::Children::iterator giter = buddies.begin(); giter != buddies.end(); ++giter)
                {
                    Gtk::TreeModel::Row row = *giter;
                    if(row[m_rosterColumns.m_type] == Roster::CHAT)
                    {
                        if(chat.compare(row[m_rosterColumns.m_ID])==0)
                            return row;
                    }
                }
            }
        }
    }
    else
    {
        Gtk::TreeModel::Children children = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            PURPLE::Account baccount = row[m_rosterColumns.m_account];
            if(row[m_rosterColumns.m_type] == Roster::CHAT && baccount == account
                    && chat.compare(row[m_rosterColumns.m_ID])==0)
            {
                return row;
            }
        }
    }
    return Gtk::TreeModel::Row();
}

void Roster::updateShowAvatars()
{
    if(DXPREFERENCES.m_showAvatars) m_avatarColumn->set_visible(true);
    else m_avatarColumn->set_visible(false);
}

void Roster::filterChange()
{
    m_rosterModelFilter->refilter();
}

void Roster::showTreeChange()
{
    m_rosterModel->clear();
    fillRoster();
    if(DXPREFERENCES.m_showRosterTree) m_protocolColumn->set_visible(true);
    else m_protocolColumn->set_visible(false);
}

void Roster::fillRoster()
{
    std::vector<PURPLE::Account> accounts = m_cb->getPurplemm()->getAccountsList();
    for(guint i=0; i<accounts.size(); i++)
    {
        if(DXPREFERENCES.m_showRosterTree)
        {
            Gtk::TreeModel::Row acc = addAccount(accounts[i]);
            std::vector<std::string> g = accounts[i].getGroups();
            std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
            std::vector<PURPLE::Chat> ch = accounts[i].getChats();
            for(guint j=0; j<g.size(); j++)
            {
                Gtk::TreeModel::Row group = addGroup(accounts[i], g[j]);
                for(guint k=0; k<b.size(); k++)
                {
                    if(g[j] == b[k].getGroupName())
                    {
                        addBuddy(b[k]);
                    }
                }
                for(guint m=0; m<ch.size(); m++)
                {
                    if(g[j] == ch[m].getGroupName())
                    {
                        addChat(ch[m]);
                    }
                }
                group[m_rosterColumns.m_hidden] = !accounts[i].isGroupOnline(g[j]);
            }
            m_roster.expand_row(m_rosterModel->get_path(*acc), true);
        }
        else
        {
            std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
            for(guint j=0; j<b.size(); j++)
            {
                addBuddy(b[j]);
            }
            std::vector<PURPLE::Chat> ch = accounts[i].getChats();
            for(guint k=0; k<ch.size(); k++)
            {
                addChat(ch[k]);
            }
        }
    }
}

void Roster::updateShowStatusmsg()
{
    if(DXPREFERENCES.m_showRosterTree)
    {
        Gtk::TreeModel::Children accounts = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator iter = accounts.begin(); iter != accounts.end(); ++iter)
        {
            Gtk::TreeModel::Row row = *iter;
            Gtk::TreeModel::Children groups = row->children();
            for(Gtk::TreeModel::Children::iterator groupIt = groups.begin(); groupIt != groups.end(); ++groupIt)
            {
                Gtk::TreeModel::Children children = (*groupIt)->children();
                for(Gtk::TreeModel::Children::iterator childIt = children.begin(); childIt != children.end(); ++childIt)
                {
                    if((*childIt)[m_rosterColumns.m_type] == Roster::BUDDY)
                    {
                        PURPLE::Buddy buddy = (*childIt)[m_rosterColumns.m_buddy];
                        (*childIt)[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
                    }
                }
            }
        }
    }
    else
    {
        Gtk::TreeModel::Children children = m_rosterModel->children();
        for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
        {
            if((*iter)[m_rosterColumns.m_type] == Roster::BUDDY)
            {
                PURPLE::Buddy buddy = (*iter)[m_rosterColumns.m_buddy];
                (*iter)[m_rosterColumns.m_markup] = makeMarkup(buddy.isOnline(), buddy.getAlias(), DXPREFERENCES.m_showStatusmsg ? buddy.getStatusMsg() : "");
            }
        }
    }
}

bool Roster::filterRoster(const Gtk::TreeModel::const_iterator &iter)
{
    if(iter)
    {
        Glib::ustring name;
        iter->get_value(2, name);
        if(name.empty())
        {
            return true;
        }
        Gtk::TreeModel::Row row = *iter;
        itemType type = row[m_rosterColumns.m_type];
        if(type != Roster::ACCOUNT)
        {
            if(type == Roster::CHAT)
            {
                return true;
            }
            PURPLE::Account acc = row[m_rosterColumns.m_account];
            bool online = acc.isValid() && acc.isConnected();
            if(!online)
            {
                return false;
            }
            else
            {
                if(row[m_rosterColumns.m_hidden] && DXPREFERENCES.m_hideOffline)
                {
                    return false;
                }
            }
        }
    }
    return true;
}

void Roster::updateTooltip(Gtk::TreeModel::Row row)
{
    itemType type = row[m_rosterColumns.m_type];
    Glib::ustring tip = "";
    if(type == Roster::ACCOUNT)
    {
        PURPLE::Account acc = row[m_rosterColumns.m_account];
        tip = acc.getUsername()+"\n";
        if(!acc.getAlias().empty()) tip.append(Glib::ustring::compose(_("Alias: %1\n"), acc.getAlias()));
        tip.append(Glib::ustring::compose(_("Protocol: %1\n"), acc.getProtocol().getName()));
        tip.append(Glib::ustring::compose(_("Status: %1"), acc.getStatusName()));
        if(acc.isConnected() && !acc.getStatusMessage().empty())
        {
            tip.append("\n");
            tip.append(Glib::ustring::compose(_("Message: %1"), acc.getStatusMessage()));
        }
    }
    else if(type == Roster::CHAT)
    {
        PURPLE::Chat chat = row[m_rosterColumns.m_chat];
        tip = Glib::ustring::compose("%1\n", chat.getAlias());
        std::vector<PURPLE::Account::ChatEntry> fields = chat.getChatParameters();
        FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
        {
            PURPLE::Account::ChatEntry entry = *it;
            if(entry.getSecret())
            {
                if(entry.getValue().empty()) tip.append(Glib::ustring::compose("%1\n", dxutils::normalizeLabelString(entry.getLabel(), '_')));
                else tip.append(Glib::ustring::compose("%1 *****\n", dxutils::normalizeLabelString(entry.getLabel(), '_')));
            }
            else
                tip.append(Glib::ustring::compose("%1 %2\n", dxutils::normalizeLabelString(entry.getLabel(), '_'), entry.getValue()));
        }
        tip.append(Glib::ustring::compose(_("Autojoin: %1"), chat.getAutojoin() ? _("Yes") : _("No")));
    }
    else if(type == Roster::BUDDY)
    {
        PURPLE::Buddy buddy = row[m_rosterColumns.m_buddy];
        tip = Glib::ustring::compose("%1\n", buddy.getServerAlias());
        tip.append(Glib::ustring::compose(_("ID: %1\n"), buddy.getUsername()));
        tip.append(Glib::ustring::compose(_("Status: %1"), buddy.getStatusName()));
        if(!buddy.getStatusMsg().empty())
        {
            tip.append("\n");
            tip.append(Glib::ustring::compose(_("Message: %1"), buddy.getStatusMsg()));
        }
    }
    row[m_rosterColumns.m_tooltip] = tip;
}

void Roster::updateAccountsTooltip()
{
    Gtk::TreeModel::Children accounts = m_rosterModel->children();
    for(Gtk::TreeModel::Children::iterator accountIt = accounts.begin(); accountIt != accounts.end(); ++accountIt)
    {
        updateTooltip(*accountIt);
    }
}

