/*
 *      convtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <algorithm>
#include "convtabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "utilsmm.h"
#include "preferences.h"

static bool vectorContains(std::vector<Glib::ustring> v, const Glib::ustring &value)
{
    FOREACH(std::vector<Glib::ustring>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

ConvTabItem::ConvTabItem(PURPLE::Conversation *conv, Gtk::Notebook *parent, ImageLabel *label, TabType type) :
TabItem(parent, label, type), m_conv(conv), m_it(m_history.begin()), m_pipe(NULL)
{
    createTextTags();
#if GTKMM_MAJOR_VERSION == 2
    m_commandline.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_commandline.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    if(DXPREFERENCES.m_sameCmd) m_commandline.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_styleProvider = Gtk::CssProvider::create();
    m_commandline.get_style_context()->add_provider(m_styleProvider, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    m_styleProvider->load_from_data(Glib::ustring::compose("GtkEntry { color: %1; background: %2; }", DXPREFERENCES.m_textColor.get_color().to_string(), DXPREFERENCES.m_backroundColor.get_color().to_string()));
    if(DXPREFERENCES.m_sameCmd) m_commandline.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    m_commandline.signal_activate().connect(sigc::mem_fun(this, &ConvTabItem::onCommandline));
    m_commandline.signal_insert_text().connect(sigc::mem_fun(this, &ConvTabItem::onInsertText));
    m_commandline.signal_key_press_event().connect(sigc::mem_fun(this, &ConvTabItem::onKeyPress));
    m_commands.push_back("help");
    m_commands.push_back("say");
    m_commands.push_back("log");
#ifndef _WIN32
    m_commands.push_back("exec");
#endif // _WIN32
    if(m_type == TABCHAT) m_commands.push_back("ignored");
    std::vector<std::string> cmds = m_conv->getCommandsList();
    FOREACH(std::vector<std::string>, cmds, it)
        m_commands.push_back(*it);
    if(!vectorContains(m_commands, "me"))
        m_commands.push_back("me");
    std::sort(m_commands.begin(), m_commands.end());

    m_lastKeyPress = 0;
    m_imState = ACTIVE;
}

void ConvTabItem::createTextTags()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    Glib::RefPtr<Gtk::TextTag> tag;
    tag = buffer->create_tag("nick0");
    tag->property_foreground().set_value("#C4A000");
    tag = buffer->create_tag("nick1");
    tag->property_foreground().set_value("#CE5C00");
    tag = buffer->create_tag("nick2");
    tag->property_foreground().set_value("#8F5902");
    tag = buffer->create_tag("nick3");
    tag->property_foreground().set_value("#4E9A06");
    tag = buffer->create_tag("nick4");
    tag->property_foreground().set_value("#204A87");
    tag = buffer->create_tag("nick5");
    tag->property_foreground().set_value("#75507B");
    tag = buffer->create_tag("nick6");
    tag->property_foreground().set_value("#A40000");
    tag = buffer->create_tag("nick7");
    tag->property_foreground().set_value("#555753");
    tag = buffer->create_tag("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = buffer->create_tag("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = buffer->create_tag("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = buffer->create_tag("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = buffer->create_tag("logs");
    tag->property_foreground().set_value(DXPREFERENCES.m_logColor.to_string());
    tag = buffer->create_tag("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
}

void ConvTabItem::updatePreferences()
{
#if GTKMM_MAJOR_VERSION == 2
    m_text.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_text.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_text.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    Glib::RefPtr<Gtk::TextTag> tag = m_text.get_buffer()->get_tag_table()->lookup("mymsg");
    tag->property_foreground().set_value(DXPREFERENCES.m_mymsgColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("error");
    tag->property_foreground().set_value(DXPREFERENCES.m_errorColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("highlight");
    tag->property_foreground().set_value(DXPREFERENCES.m_hilightColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("user");
    tag->property_foreground().set_value(DXPREFERENCES.m_userColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("logs");
    tag->property_foreground().set_value(DXPREFERENCES.m_logColor.to_string());
    tag = m_text.get_buffer()->get_tag_table()->lookup("action");
    tag->property_foreground().set_value(DXPREFERENCES.m_actionColor.to_string());
#if GTKMM_MAJOR_VERSION == 2
    m_commandline.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_commandline.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    if(DXPREFERENCES.m_sameCmd) m_commandline.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_commandline.modify_font(get_pango_context()->get_font_description());
#else
    m_styleProvider->load_from_data(Glib::ustring::compose("GtkEntry { color: %1; background: %2; }", DXPREFERENCES.m_textColor.get_color().to_string(), DXPREFERENCES.m_backroundColor.get_color().to_string()));
    if(DXPREFERENCES.m_sameCmd) m_commandline.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
    else m_commandline.override_font(get_pango_context()->get_font_description());
#endif
    m_logger.stopLogging();
}

void ConvTabItem::focusCommandline()
{
    makeLastRowVisible(true);
    m_commandline.grab_focus();
}

void ConvTabItem::onCommandline()
{
    if(!m_conv->getAccount().isConnected())
        return;
    if(m_commandline.get_text().length() == 0)
        return;
    Glib::ustring commandtext = m_commandline.get_text();
    if(commandtext[0] == '/')
    {
        Glib::ustring::size_type pos = commandtext.find_first_of(' ');
        Glib::ustring command = commandtext.substr(1, pos-1);
        Glib::ustring arg = "";
        if(pos != Glib::ustring::npos) arg = commandtext.substr(pos+1);
        if(command.lowercase().compare("help")==0)
        {
            if(arg.empty())
            {
                Glib::ustring commands = "";
                for(size_t i=0; i<m_commands.size(); i++)
                {
                    if(i+1!=m_commands.size())
                        commands.append(m_commands[i]+", ");
                    else
                        commands.append(m_commands[i]+".");
                }
                appendStyledText(Glib::ustring::compose(_("Available commands: %1"), commands), "mymsg", "", false);
                m_commandline.set_text("");
            }
            else if(arg.lowercase().compare("help")==0 || arg.lowercase().compare("me")==0 || arg.lowercase().compare("say")==0
                    || arg.lowercase().compare("ignored")==0 || arg.lowercase().compare("exec")==0 || arg.lowercase().compare("log")==0)
            {
                showInternalHelp(arg);
                m_commandline.set_text("");
            }
            else
            {
                Glib::ustring response = m_conv->getCommandHelp(arg);
                if(response.empty())
                    appendStyledText(Glib::ustring::compose(_("Command '%1' doesn't exist or hasn't help"), arg), "error", "", false);
                else
                {
                    appendStyledText(Glib::ustring::compose(_("Help for: %1"), response), "mymsg", "", false);
                }
            }
        }
        else if(command.lowercase().compare("exec")==0)
        {
            if(arg.empty()) showInternalHelp("exec");
            else
            {
                if(!m_pipe)
                {
                    m_pipe = new dxPipe();
                    m_pipe->signalText.connect(sigc::mem_fun(this, &ConvTabItem::onPipe));
                }
                m_pipeStrings.clear();
                pos = arg.find_first_of(' ');
                Glib::ustring opt = "";
                if(pos != Glib::ustring::npos) opt = arg.substr(0, pos);
                if(opt.lowercase().compare("-o")==0)
                {
                    m_sendPipe = true;
                    appendStyledText(Glib::ustring::compose(_("Command %1 started"), arg.substr(pos+1)), "mymsg", "", false);
                    m_pipe->execCmd(arg.substr(pos+1));
                }
                else if(opt.lowercase().compare("-c")==0 || arg.lowercase().compare("-c")==0)
                {
                    m_sendPipe = false;
                    appendStyledText(Glib::ustring::compose(_("Command %1 stopped"), m_pipe->getCommand()), "mymsg", "", false);
                    m_pipeStrings.clear();
                    m_pipe->stopCmd();
                }
                else
                {
                    m_sendPipe = false;
                    appendStyledText(Glib::ustring::compose(_("Command %1 started"), arg), "mymsg", "", false);
                    m_pipe->execCmd(arg);
                }
            }
        }
        else if(command.lowercase().compare("me")==0)
        {
            if(!arg.empty()) m_conv->sendActionMessage(escapeText(arg));
        }
        else if(command.lowercase().compare("say")==0)
        {
            if(!arg.empty()) m_conv->sendMessage(escapeText(arg));
        }
        else if(command.lowercase().compare("ignored")==0)
        {
            Glib::ustring ignored = "";
            std::vector<std::string> v = m_conv->getIgnored();
            for(size_t i=0; i<v.size(); i++)
            {
                if(i+1!=v.size())
                    ignored.append(v[i]+", ");
                else
                    ignored.append(v[i]+".");
            }
            if(v.size())
                appendStyledText(Glib::ustring::compose(_("Ignored buddies: %1"), ignored), "mymsg", "", false);
            else
                appendStyledText(_("Nobody ignored"), "mymsg", "", false);
        }
        else if(command.lowercase().compare("log")==0)
        {
            Glib::ustring logcommand = dxutils::stringSection(arg, ' ', 0);
            Glib::ustring logfile = dxutils::stringSection(arg, ' ', 1);
            if(logcommand.lowercase().compare("list")==0)
            {
                listLogFiles();
            }
            else if(logcommand.lowercase().compare("show")==0)
            {
                if(logfile.empty())
                    logfile = purple_utf8_strftime("%Y-%m-%d", NULL);
                Glib::ustring folder = m_conv->getName();
                folder = folder.substr(0, folder.find_last_of('/'));
                Glib::ustring username = m_conv->getAccount().getUsername();
                username = username.substr(0, username.find_last_of('/'));
                Glib::ustring logpath = DXPREFERENCES.m_logPath+G_DIR_SEPARATOR_S+username+G_DIR_SEPARATOR_S+folder+G_DIR_SEPARATOR_S+logfile;
                if(!Glib::file_test(logpath, Glib::FILE_TEST_EXISTS)
                            || Glib::file_test(logpath, Glib::FILE_TEST_IS_DIR))
                {
                    appendStyledText(Glib::ustring::compose(_("Log file %1 doesn't exist"), logfile), "error", "", false);
                    m_commandline.set_text("");
                    m_lastKeyPress = 0;
                    m_imState = ACTIVE;
                    return;
                }
                if(dxutils::isRightLogFile(logpath, logfile)) signalShowLog(m_conv->getAccount(), m_conv->getName(), logpath);
            }
            else
            {
                appendStyledText(Glib::ustring::compose(dxutils::replaceLessGreater(_("'%1' isn't <list|show>")), logcommand), "error", "", false);
            }
        }
        else
        {
            int ret = m_conv->sendCommand(commandtext.substr(1));
            processCommandReturn(ret, command);
        }
    }
    else
    {
        m_conv->sendMessage(escapeText(commandtext));
    }
    if(m_history.size()>24)
        m_history.erase(m_history.begin());
    m_history.push_back(commandtext);
    m_it = m_history.begin();
    m_commandline.set_text("");
    m_lastKeyPress = 0;
    m_imState = ACTIVE;
}

void ConvTabItem::onInsertText(Glib::ustring const &/*text*/, int */*position*/)
{
    m_lastKeyPress = dxutils::now();
}

bool ConvTabItem::onKeyPress(GdkEventKey* event)
{
    //next element from history
    if(event->keyval == GDK_KEY_uparrow || event->keyval == GDK_KEY_Up)
    {
        if(!m_history.empty())
        {
            if(m_it == m_history.begin())
                m_it = m_history.end()-1;
            else
                --m_it;
            m_commandline.set_text(*m_it);
            m_commandline.set_position(-1);
        }
    }
    //previous element from history
    else if(event->keyval == GDK_KEY_downarrow || event->keyval == GDK_KEY_Down)
    {
        if(!m_history.empty())
        {
            m_it++;
            if(m_it == m_history.end())
                m_it = m_history.begin();
            m_commandline.set_text(*m_it);
            m_commandline.set_position(-1);
        }
    }
    //tab completion
    else if(event->keyval == GDK_KEY_Tab)
    {
        Glib::ustring commandtext = m_commandline.get_text();
        if(!commandtext.substr(0,6).lowercase().compare("/help ")
           && commandtext.length()>6)
        {
            completeHelpCommand();
        }
        else if(commandtext[0] == '/')
        {
            Glib::ustring::size_type pos = commandtext.find_first_of(' ');
            Glib::ustring command = commandtext.substr(1, pos-1);
            if(pos == Glib::ustring::npos
                || !commandtext.substr(pos+1).length())
            {
                for(size_t i=0; i<m_commands.size(); i++)
                {
                    if(command.lowercase().compare(m_commands[i]) == 0)
                    {
                        if((i+1) < m_commands.size()) m_commandline.set_text("/"+m_commands[++i]+" ");
                        else m_commandline.set_text("/"+m_commands[0]+" ");
                        break;
                    }
                    else if(command.lowercase().compare(m_commands[i].substr(0, command.lowercase().length())) == 0)
                    {
                        m_commandline.set_text("/"+m_commands[i]+" ");
                        break;
                    }
                }
                m_commandline.set_position(-1);
            }
            else
            {
                if(getType() == TABCHAT)
                {
                    int curpos = m_commandline.get_position();
                    m_commandline.set_text(getCompletion(commandtext, curpos));
                    m_commandline.set_position(curpos);
                }
            }
        }
        else if(commandtext[0] != '/'
            && (commandtext.find(' ')==Glib::ustring::npos || commandtext.substr(commandtext.find(' ')+1).empty())
            && getType() == TABCHAT)
        {
            m_commandline.set_text(getCompletion(commandtext));
            m_commandline.set_position(-1);
        }
        else if(commandtext.find(' ') != Glib::ustring::npos && getType() == TABCHAT)
        {
            int curpos = m_commandline.get_position();
            m_commandline.set_text(getCompletion(commandtext, curpos));
            m_commandline.set_position(curpos);
        }
    }
    return true;
}

//append text with bold or colored nick
void ConvTabItem::appendNickText(const Glib::ustring &time, const Glib::ustring &nick, const Glib::ustring &msg, bool log)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->insert(buffer->end(), time);
    if(DXPREFERENCES.m_coloredNick) buffer->insert_with_tag(buffer->end(), nick+DXPREFERENCES.m_nickCompletionChar+" ", dxutils::getNickColor(nick).c_str());
    else buffer->insert_with_tag(buffer->end(), nick+DXPREFERENCES.m_nickCompletionChar+" ", "bold");
    parseHtml(msg, "");
    makeLastRowVisible();
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logNickLine(time, nick, msg);
    }
}

//append my text
void ConvTabItem::appendMyMsg(const Glib::ustring &time, const Glib::ustring &msg)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->insert(buffer->end(), time);
    if(DXPREFERENCES.m_coloredNick) buffer->insert_with_tag(buffer->end(), m_conv->getMyNick()+DXPREFERENCES.m_nickCompletionChar+" ",
                                                            dxutils::getNickColor(m_conv->getMyNick()).c_str());
    else buffer->insert_with_tag(buffer->end(), m_conv->getMyNick()+DXPREFERENCES.m_nickCompletionChar+" ", "bold");
    parseHtml(msg, "mymsg");
    makeLastRowVisible();
    if(DXPREFERENCES.m_logging)
    {
        startLogging();
        m_logger.logMyMsgLine(time, m_conv->getMyNick(), msg);
    }
}

//append styled text
void ConvTabItem::appendStyledText(const Glib::ustring& msg, const Glib::ustring& style, Glib::ustring time, bool log)
{
    if(time.empty()) time = purple_utf8_strftime("[%H:%M:%S] ", NULL);
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->insert(buffer->end(), time);
    parseHtml(msg, style.c_str());
    makeLastRowVisible();
    if(DXPREFERENCES.m_logging && log)
    {
        startLogging();
        m_logger.logStyledLine(time, style, msg);
    }
}

//append history msg
void ConvTabItem::appendHistoryMsg(const Glib::ustring& nick, const Glib::ustring& msg, const Glib::ustring& time)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    buffer->insert(buffer->end(), time);
    if(DXPREFERENCES.m_coloredNick) buffer->insert_with_tag(buffer->end(), nick+": ", dxutils::getNickColor(nick).c_str());
    else buffer->insert_with_tag(buffer->end(), nick+": ", "bold");
    parseHtml(msg, "logs");
    makeLastRowVisible();
}

void ConvTabItem::makeLastRowVisible(bool force)
{
    bool scroll = false;
    if(m_text.get_vadjustment()->get_value() >= (m_text.get_vadjustment()->get_upper() - m_text.get_vadjustment()->get_page_size() - 1e-12))
        scroll = true;
    if(scroll || force)
    {
        Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
        m_text.scroll_to(buffer->create_mark("end", buffer->end(), false), 0);
    }
}

//process return m_conv->sendCommand
void ConvTabItem::processCommandReturn(int ret, const Glib::ustring &command)
{
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            break;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            appendStyledText(Glib::ustring::compose(_("Unknown command '%1'."), command), "error");
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            appendStyledText(Glib::ustring::compose(_("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%1'."), command), "error");
            break;
        case PURPLE_CMD_STATUS_FAILED:
            appendStyledText(Glib::ustring::compose(_("Your command '%1' failed for an unknown reason."), command), "error");
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conv->getType() == PURPLE_CONV_TYPE_IM)
                appendStyledText(Glib::ustring::compose(_("That command '%1' only works in chats, not IMs."), command), "error");
            else
                appendStyledText(Glib::ustring::compose(_("That command '%1' only works in IMs, not chats."), command), "error");
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            appendStyledText(Glib::ustring::compose(_("That command '%1' doesn't work on this protocol."), command), "error");
            break;
    }
}

void ConvTabItem::startLogging()
{
    if(m_logger.isReady()) return;
    Glib::ustring folder = m_conv->getName();
    folder = folder.substr(0, folder.find_last_of('/'));
    Glib::ustring username = m_conv->getAccount().getUsername();
    username = username.substr(0, username.find_last_of('/'));
    m_logger.setPath(DXPREFERENCES.m_logPath+G_DIR_SEPARATOR_S+username+G_DIR_SEPARATOR_S+folder);
}

void ConvTabItem::showInternalHelp(const Glib::ustring &command)
{
    if(command.lowercase().compare("help")==0)
    {
        appendStyledText(_("Help for: help [command]: lists available commands or show help for command"), "mymsg", "", false);
    }
    else if(command.lowercase().compare("me")==0)
    {
        appendStyledText(dxutils::replaceLessGreater(_("Help for: me <text>: sends text in 3rd person")), "mymsg", "", false);
    }
    else if(command.lowercase().compare("say")==0)
    {
        appendStyledText(dxutils::replaceLessGreater(_("Help for: say <text>: sends text")), "mymsg", "", false);
    }
    else if(command.lowercase().compare("ignored")==0)
    {
        appendStyledText(_("Help for: ignored: show list ignored buddies"), "mymsg", "", false);
    }
    else if(command.lowercase().compare("exec")==0)
    {
        appendStyledText(dxutils::replaceLessGreater(_("Help for: exec [-o|-c] <command>, executes command, -o sends output to im, -c closes running command")), "mymsg", "", false);
    }
    else if(command.lowercase().compare("log")==0)
    {
        appendStyledText(_("Help for: log"), "mymsg", "", false);
        appendStyledText(_("LOG list, shows list of log files for current im/chat"), "mymsg", "", false);
        appendStyledText(_("LOG show [date], shows current log file for current im/chat, if date is empty, else file for given date"), "mymsg", "", false);
        appendStyledText(_("date example: 2011-01-31"), "mymsg", "", false);
    }
}

void ConvTabItem::removeLastLine()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    Gtk::TextIter from, to;
    to = buffer->end();
    from = to;
    from.backward_line();
    buffer->erase(from, to);
}

Glib::ustring ConvTabItem::getLastLine()
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    Gtk::TextIter from, to;
    to = buffer->end();
    from = to;
    from.backward_line();
    return buffer->get_text(from, to);
}

void ConvTabItem::completeHelpCommand()
{
    Glib::ustring commandtext = m_commandline.get_text();
    Glib::ustring::size_type pos = commandtext.find_first_of(' ');
    Glib::ustring arg = commandtext.substr(pos+1);
    for(size_t i=0; i<m_commands.size(); i++)
    {
        if(arg.lowercase().compare(m_commands[i]) == 0)
        {
            if((i+1) < m_commands.size()) m_commandline.set_text("/help "+m_commands[++i]);
            else m_commandline.set_text("/help "+m_commands[0]);
            break;
        }
        else if(arg.lowercase().compare(m_commands[i].substr(0, arg.lowercase().length())) == 0)
        {
            m_commandline.set_text("/help "+m_commands[i]);
            break;
        }
    }
    m_commandline.set_position(-1);
}

Glib::ustring ConvTabItem::escapeText(Glib::ustring text)
{
    size_t i = 0;
    size_t length = text.length();
    while(i<length)
    {
        if(text[i]=='<')
        {
            text.replace(i, 1, "&lt;");
            length += 3;
        }
        else if(text[i]=='>')
        {
            text.replace(i, 1, "&gt;");
            length += 3;
        }
        i++;
    }
    return text;
}

void ConvTabItem::listLogFiles()
{
    Glib::ustring folder = m_conv->getName();
    folder = folder.substr(0, folder.find_last_of('/'));
    Glib::ustring username = m_conv->getAccount().getUsername();
    username = username.substr(0, username.find_last_of('/'));
    Glib::ustring path = DXPREFERENCES.m_logPath+G_DIR_SEPARATOR_S+username+G_DIR_SEPARATOR_S+folder;
    std::vector<Glib::ustring> files;
    Glib::Dir dir(path);
    std::string name = dir.read_name();
    while(!name.empty())
    {
        if(name[0] != '.')
        {
            Glib::ustring filePath = Glib::build_filename(path, name);
            if(Glib::file_test(filePath, Glib::FILE_TEST_EXISTS)
                && !Glib::file_test(filePath, Glib::FILE_TEST_IS_DIR)
                && dxutils::isRightLogFile(filePath, name))
            {
                files.push_back(name);
            }
        }
        name = dir.read_name();
    }
    std::sort(files.begin(), files.end());
    if(files.size())
    {
        appendStyledText(Glib::ustring::compose("<b><u>%1</u></b>", _("Log files:")), "", "", false);
        Glib::ustring lastYear = dxutils::stringSection(files[0], '-', 0);
        int lastMonth = PURPLE::s2t<gint>(dxutils::stringSection(files[0], '-', 1));
        appendStyledText(Glib::ustring::compose("<b>%1:</b>", lastYear), "", "", false);
        appendStyledText(Glib::ustring::compose("<b><i>%1</i>:</b>", dxutils::getMonth(lastMonth)), "", "", false);
        Glib::ustring list = "";
        for(size_t i=0; i<files.size(); i++)
        {
            if(lastYear!=dxutils::stringSection(files[i], '-', 0))
            {
                lastYear = dxutils::stringSection(files[i], '-', 0);
                lastMonth = PURPLE::s2t<gint>(dxutils::stringSection(files[i], '-', 1));
                appendStyledText(list.substr(0, list.size()-2), "", "", false);
                appendStyledText(Glib::ustring::compose("<b>%1:</b>", lastYear), "", "", false);
                appendStyledText(Glib::ustring::compose("<b><i>%1</i>:</b>", dxutils::getMonth(lastMonth)), "", "", false);
                list.clear();
                list.append(dxutils::stringSection(files[i], '-', 2)+", ");
            }
            else if(lastMonth!=PURPLE::s2t<gint>(dxutils::stringSection(files[i], '-', 1)))
            {
                lastMonth = PURPLE::s2t<gint>(dxutils::stringSection(files[i], '-', 1));
                appendStyledText(list.substr(0, list.size()-2), "", "", false);
                appendStyledText(Glib::ustring::compose("<b><i>%1</i>:</b>", dxutils::getMonth(lastMonth)), "", "", false);
                list.clear();
                list.append(dxutils::stringSection(files[i], '-', 2)+", ");
            }
            else list.append(dxutils::stringSection(files[i], '-', 2)+", ");
        }
        if(!list.empty()) appendStyledText(list.substr(0, list.size()-2), "", "", false);
    }
    else
    {
        appendStyledText(Glib::ustring::compose(_("No log files for %1-%2."), username, folder), "error", "", false);
    }
}

void ConvTabItem::onPipe(Glib::ustring text)
{
    if(m_sendPipe)
    {
        m_pipeStrings.push_back(text);
        if(!m_conn.connected()) m_conn = Glib::signal_timeout().connect(sigc::mem_fun(this, &ConvTabItem::onPipeTimeout), 3000);
    }
    else
    {
        appendStyledText(text, "", "", false);
    }
}

bool ConvTabItem::onPipeTimeout()
{
    if(m_pipeStrings.size() > 3)
    {
        if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
        if(m_conv->isValid())
        {
            m_conv->sendMessage(escapeText(m_pipeStrings[0]));
        }
        m_pipeStrings.erase(m_pipeStrings.begin());
    }
    else
    {
        while(m_pipeStrings.size())
        {
            if(m_pipeStrings[0].empty()) m_pipeStrings[0] = " ";
            if(m_conv->isValid())
            {
                m_conv->sendMessage(m_pipeStrings[0]);
            }
            m_pipeStrings.erase(m_pipeStrings.begin());
        }
        return false;
    }
    return true;
}

