/*
 *      fttabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "fttabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "icons.h"
#include "preferences.h"

static Glib::ustring statusForFT(PURPLE::FileTransfer *ft)
{
    if(ft->isCanceled()) return _("Canceled");
    if(!ft->isStarted()) return _("Waiting");
    if(ft->isFinished()) return _("Finished");
    if(ft->isReceiving()) return Glib::ustring::compose(_("Downloading %1 of %2"), dxutils::getFileSize(ft->getBytesSent()), dxutils::getFileSize(ft->getFileSize()));
    else return Glib::ustring::compose(_("Uploading %1 of %2"), dxutils::getFileSize(ft->getBytesSent()), dxutils::getFileSize(ft->getFileSize()));
}

FTTabItem::FTTabItem(dxgCallbacks *cb, Gtk::Notebook *parent, ImageLabel *label) :
TabItem(parent, label, TABFILETRANSFER), m_cb(cb)
{
    Gtk::ScrolledWindow *swin = Gtk::manage(new Gtk::ScrolledWindow());
    swin->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
    swin->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_ftModel = Gtk::ListStore::create(m_ftColumns);
    m_fts.set_model(m_ftModel);
    Gtk::TreeView::Column *fColumn = Gtk::manage(new Gtk::TreeView::Column(_("File")));
    fColumn->pack_start(m_ftColumns.m_icon, false);
    fColumn->pack_start(m_ftColumns.m_fileName);
    fColumn->set_resizable(true);
    m_fts.append_column(*fColumn);
    m_fts.append_column(_("Status"), m_ftProgress);
    Gtk::TreeViewColumn *sColumn = m_fts.get_column(1);
    if(sColumn)
    {
        sColumn->add_attribute(m_ftProgress.property_value(), m_ftColumns.m_progress);
        sColumn->add_attribute(m_ftProgress.property_text(), m_ftColumns.m_progressText);
        sColumn->set_resizable(true);
    }
    m_fts.append_column(_("Remaining time"), m_ftColumns.m_remainingTime);
    m_fts.get_column(2)->set_resizable(true);
    m_ftSelectionModel = m_fts.get_selection();
    m_ftSelectionModel->set_mode(Gtk::SELECTION_SINGLE);
    m_ftSelectionModel->signal_changed().connect(sigc::mem_fun(this, &FTTabItem::onFTChanged));
    swin->add(m_fts);
    pack_start(*swin, Gtk::PACK_EXPAND_WIDGET);
    m_details = new Gtk::Expander(_("Details"));
    Gtk::Table *table = Gtk::manage(new Gtk::Table());
    table->attach(*Gtk::manage(new Gtk::Label(_("Path:"), Gtk::ALIGN_START)), 0, 1, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_path = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_path, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Account:"), Gtk::ALIGN_START)), 0, 1, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_account = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_account, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    table->attach(*Gtk::manage(new Gtk::Label(_("Protocol:"), Gtk::ALIGN_START)), 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_protocol = new ImageLabel("");
    table->attach(*m_protocol, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_direction = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_direction, 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_buddy = new Gtk::Label("", Gtk::ALIGN_START);
    table->attach(*m_buddy, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 3, 3);
    m_details->add(*table);
    m_details->set_sensitive(false);
    pack_start(*m_details, Gtk::PACK_SHRINK);
    Gtk::HButtonBox *btnbox = Gtk::manage(new Gtk::HButtonBox(Gtk::BUTTONBOX_END, 6));
    btnbox->pack_end(*Gtk::manage(new Gtk::VBox()), Gtk::PACK_EXPAND_WIDGET);
    m_buttonClearCanceled = new Gtk::Button(_("Clear _canceled"), true);
    m_buttonClearCanceled->signal_clicked().connect(sigc::mem_fun(this, &FTTabItem::onClearCanceled));
    m_buttonClearCanceled->set_sensitive(false);
    btnbox->pack_end(*m_buttonClearCanceled, Gtk::PACK_SHRINK);
    m_buttonClearFinished = new Gtk::Button(_("Clear _finished"), true);
    m_buttonClearFinished->signal_clicked().connect(sigc::mem_fun(this, &FTTabItem::onClearFinished));
    m_buttonClearFinished->set_sensitive(false);
    btnbox->pack_end(*m_buttonClearFinished, Gtk::PACK_SHRINK);
    m_buttonClose = new Gtk::Button(Gtk::Stock::CLOSE);
    m_buttonClose->signal_clicked().connect(sigc::mem_fun(this, &FTTabItem::onClose));
    btnbox->pack_end(*m_buttonClose, Gtk::PACK_SHRINK);
    pack_start(*btnbox, false, false, 3);
    show_all();

    m_cb->signalFileTransferStarted.connect(sigc::mem_fun(this, &FTTabItem::onFileTransferStarted));
    m_cb->signalFileTransferFinished.connect(sigc::mem_fun(this, &FTTabItem::onFileTransferFinished));
    m_cb->signalFileTransferCanceled.connect(sigc::mem_fun(this, &FTTabItem::onFileTransferCanceled));
    m_cb->signalFileTransferProgress.connect(sigc::mem_fun(this, &FTTabItem::onFileTransferProgress));

    m_fts.signal_button_press_event().connect(sigc::mem_fun(this, &FTTabItem::onButtonPressed), false);
}

void FTTabItem::onClose()
{
    hide();
    signalFTTabItemClosed();
}

void FTTabItem::onClearFinished()
{
    PURPLE::FileTransfer *transfer = NULL;
    Gtk::TreeModel::Children children = m_ftModel->children();
    Gtk::TreeModel::Children::iterator iter = children.begin();
    while(iter != children.end())
    {
        transfer = (*iter)[m_ftColumns.m_ft];
        if(transfer->isFinished()) iter = m_ftModel->erase(iter);
        else ++iter;
    }
    m_buttonClearFinished->set_sensitive(false);
    if(!m_ftModel->children().size())
    {
        m_details->set_expanded(false);
        m_details->set_sensitive(false);
    }
}

void FTTabItem::onClearCanceled()
{
    PURPLE::FileTransfer *transfer = NULL;
    Gtk::TreeModel::Children children = m_ftModel->children();
    Gtk::TreeModel::Children::iterator iter = children.begin();
    while(iter != children.end())
    {
        transfer = (*iter)[m_ftColumns.m_ft];
        if(transfer->isCanceled()) iter = m_ftModel->erase(iter);
        else ++iter;
    }
    m_buttonClearCanceled->set_sensitive(false);
    if(!m_ftModel->children().size())
    {
        m_details->set_expanded(false);
        m_details->set_sensitive(false);
    }
}

void FTTabItem::onFTChanged()
{
    Gtk::TreeModel::iterator it = m_fts.get_selection()->get_selected();
    if(it)
    {
        PURPLE::FileTransfer *transfer = (*it)[m_ftColumns.m_ft];
        m_path->set_text(transfer->getLocalFileName());
        m_account->set_text(transfer->getAccount().getUsername());
        m_protocol->setIcon(dxutils::getProtocolIcon(transfer->getAccount().getProtocol().getPurpleID()));
        m_protocol->setText(transfer->getAccount().getProtocol().getName());
        m_direction->set_text(transfer->isReceiving()?_("From:"):_("To:"));
        m_buddy->set_text(transfer->getRemoteUser());
    }
}

void FTTabItem::newFileTransfer(PURPLE::FileTransfer* ft)
{
    if(ft->isSending())
    {
        Gtk::TreeModel::Row row = *(m_ftModel->append());
        row[m_ftColumns.m_icon] = ICO_UP->get_pixbuf();
        row[m_ftColumns.m_fileName] = dxutils::fileName(ft->getLocalFileName());
        row[m_ftColumns.m_progress] = 0;
        row[m_ftColumns.m_progressText] = statusForFT(ft);
        row[m_ftColumns.m_remainingTime] = "?";
        row[m_ftColumns.m_ft] = ft;
        m_details->set_sensitive(true);
        m_ftSelectionModel->select(row);
    }
}

void FTTabItem::onFileTransferStarted(PURPLE::FileTransfer* ft)
{
    if(ft->isReceiving())
    {
        Gtk::TreeModel::Row row = *(m_ftModel->append());
        row[m_ftColumns.m_icon] = ICO_DOWN->get_pixbuf();
        row[m_ftColumns.m_fileName] = dxutils::fileName(ft->getLocalFileName());
        row[m_ftColumns.m_progress] = 0;
        row[m_ftColumns.m_progressText] = statusForFT(ft);
        row[m_ftColumns.m_remainingTime] = "?";
        row[m_ftColumns.m_ft] = ft;
        m_details->set_sensitive(true);
        m_ftSelectionModel->select(row);
    }
    else
        onFileTransferProgress(ft);
}

void FTTabItem::onFileTransferFinished(PURPLE::FileTransfer* ft)
{
    PURPLE::FileTransfer *transfer = NULL;
    Gtk::TreeModel::Children children = m_ftModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        transfer = (*iter)[m_ftColumns.m_ft];
        if(transfer == ft)
        {
            Gtk::TreeModel::Row row = *iter;
            row[m_ftColumns.m_icon] = ICO_FINISH->get_pixbuf();
            row[m_ftColumns.m_progress] = 100;
            row[m_ftColumns.m_progressText] = statusForFT(ft);
            row[m_ftColumns.m_remainingTime] = "";
            m_buttonClearFinished->set_sensitive(true);
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(Glib::ustring::compose(_("Transfer of %1 finished"), dxutils::fileName(ft->getLocalFileName())));
        }
    }
}

void FTTabItem::onFileTransferCanceled(PURPLE::FileTransfer* ft)
{
    PURPLE::FileTransfer *transfer = NULL;
    Gtk::TreeModel::Children children = m_ftModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        transfer = (*iter)[m_ftColumns.m_ft];
        if(transfer == ft)
        {
            Gtk::TreeModel::Row row = *iter;
            row[m_ftColumns.m_icon] = ICO_CANCEL->get_pixbuf();
            row[m_ftColumns.m_progressText] = statusForFT(ft);
            row[m_ftColumns.m_remainingTime] = "";
            m_buttonClearCanceled->set_sensitive(true);
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(Glib::ustring::compose(_("Transfer of %1 canceled"), dxutils::fileName(ft->getLocalFileName())));
        }
    }
}

void FTTabItem::onFileTransferProgress(PURPLE::FileTransfer* ft)
{
    PURPLE::FileTransfer *transfer = NULL;
    Gtk::TreeModel::Children children = m_ftModel->children();
    for(Gtk::TreeModel::Children::iterator iter = children.begin(); iter != children.end(); ++iter)
    {
        transfer = (*iter)[m_ftColumns.m_ft];
        if(transfer == ft)
        {
            Gtk::TreeModel::Row row = *iter;
            row[m_ftColumns.m_progress] = (guint)(100*ft->getBytesSent())/ft->getFileSize();
            if(ft->isFinished())
            {
                row[m_ftColumns.m_icon] = ICO_FINISH->get_pixbuf();
                row[m_ftColumns.m_remainingTime] = "";
            }
            else if(ft->isCanceled())
            {
                row[m_ftColumns.m_icon] = ICO_CANCEL->get_pixbuf();
                row[m_ftColumns.m_remainingTime] = "";
            }
            else
            {
                row[m_ftColumns.m_remainingTime] = dxutils::getRemaining(ft->getFileSize()-ft->getBytesSent(), ft->getSpeed());
            }
            row[m_ftColumns.m_progressText] = statusForFT(ft);
        }
    }
}

bool FTTabItem::onButtonPressed(GdkEventButton* event)
{
    bool value = false;
    if(event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
        Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_fts.get_selection();
        if(refSelection)
        {
            Gtk::TreeModel::Path path;
            m_fts.get_path_at_pos(event->x, event->y, path);
            refSelection->select(path);
            Gtk::TreeModel::iterator iter = refSelection->get_selected();
            if(iter)
            {
                PURPLE::FileTransfer *ft = (*iter)[m_ftColumns.m_ft];
                if(!ft->isCanceled() && !ft->isFinished())
                {
                    Gtk::Menu *popup = new Gtk::Menu();
                    Gtk::MenuItem* item = new Gtk::MenuItem(_("Cancel"));
                    item->signal_activate().connect(sigc::mem_fun(this, &FTTabItem::onPopupCancel));
                    popup->append(*item);
                    popup->accelerate(*this);
                    popup->show_all();
                    popup->popup(event->button, event->time);
                    value = true;
                }
            }
        }
    }
    return value;
}

void FTTabItem::onPopupCancel()
{
    Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_fts.get_selection();
    if(refSelection)
    {
        Gtk::TreeModel::iterator iter = refSelection->get_selected();
        if(iter)
        {
            PURPLE::FileTransfer *ft = (*iter)[m_ftColumns.m_ft];
            ft->cancelFT();
        }
    }
}



