/*
 *      tabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "tabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "icons.h"
#include "utilsmm.h"

TabItem::TabItem(Gtk::Notebook *parent, ImageLabel *label, TabType type) :
m_parent(parent), m_label(label), m_type(type), m_hovering(false)
#if GTKMM_MAJOR_VERSION ==2
, m_default(Gdk::XTERM), m_hand(Gdk::HAND2)
{
#else
{
    m_default = Gdk::Cursor::create(Gdk::XTERM);
    m_hand = Gdk::Cursor::create(Gdk::HAND2);
#endif
    m_text.set_editable(false);
    m_text.set_can_focus(false);
    m_text.set_wrap_mode(Gtk::WRAP_WORD);
#if GTKMM_MAJOR_VERSION == 2
    m_text.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_text.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_text.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    m_text.signal_motion_notify_event().connect(sigc::mem_fun(this, &TabItem::onMotionEvent));
    m_text.signal_event_after().connect(sigc::mem_fun(this, &TabItem::onEventAfter));
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    m_link = buffer->create_tag("link");
    m_link->property_foreground().set_value(DXPREFERENCES.m_linkColor.to_string());
    m_link->property_underline().set_value(Pango::UNDERLINE_SINGLE);
    Glib::RefPtr<Gtk::TextTag> tag = buffer->create_tag("bold");
    tag->property_weight().set_value(Pango::WEIGHT_BOLD);
    m_status = STANDARD;
}

bool TabItem::isCurrent()
{
    return this == m_parent->get_nth_page(m_parent->get_current_page());
}

void TabItem::clearText()
{
    if(m_type == TABIM || m_type == TABCHAT)
    {
        Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
        buffer->erase(buffer->begin(), buffer->end());
    }
}

void TabItem::updatePreferences()
{
#if GTKMM_MAJOR_VERSION == 2
    m_text.modify_base(Gtk::STATE_NORMAL, DXPREFERENCES.m_backroundColor.get_color());
    m_text.modify_text(Gtk::STATE_NORMAL, DXPREFERENCES.m_textColor.get_color());
    m_text.modify_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#else
    m_text.override_background_color(DXPREFERENCES.m_backroundColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_color(DXPREFERENCES.m_textColor.get_color(), Gtk::STATE_FLAG_NORMAL);
    m_text.override_font(Pango::FontDescription(DXPREFERENCES.m_imFont));
#endif
    m_link->property_foreground().set_value(DXPREFERENCES.m_linkColor.to_string());
    if(m_status == HILIGHT) m_label->setColor(DXPREFERENCES.m_hltabColor);
    if(m_status == UNREAD) m_label->setColor(DXPREFERENCES.m_unreadtabColor);
}

void TabItem::parseHtml(Glib::ustring text, const Glib::ustring &style)
{
    text = createSmileys(text);
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    size_t i = 0;
    size_t tagLength = 0;
    bool bold = false;
    bool under = false;
    bool italic = false;
    Glib::RefPtr<Gtk::TextTag> tagStyle = buffer->get_tag_table()->lookup(style);
    dxColor foreColor, backColor;
    if(!style.empty() && tagStyle)
    {
#if GTKMM_MAJOR_VERSION == 2
        foreColor = dxColor(tagStyle->property_foreground_gdk());
#else
        foreColor = dxColor(tagStyle->property_foreground_rgba());
#endif
    }
    else
    {
        foreColor = DXPREFERENCES.m_textColor;
    }
    backColor = DXPREFERENCES.m_backroundColor;
    Glib::ustring normalText = "";
    Glib::ustring tag = "";
    std::list<guchar> span; //0 nothing, 1 bold, 2 under, 3 fore, 4 back
    size_t length = text.length();
    while(i<length)
    {
        if(text[i]=='<')
        {
            tagLength = 1;
            for(size_t j=i+1; j<length; j++)
            {
                if(text[j]=='>')
                {
                    tagLength++;
                    break;
                }
                else if(text[j]=='<')
                {
                    break;
                }
                tagLength++;
            }
            tag = text.substr(i, tagLength);
            dxutils::debugLine(text);
            dxutils::debugLine(tag);
            if(dxutils::stringContains(tag, "<br")) //tag which will replaced by PURPLE::stripHtml
            {
                normalText.append(tag);
                i += tagLength;
            }
            else
            {
                if(dxutils::stringContains(tag, "<b>") || dxutils::stringContains(tag, "</b>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    bold = !bold;
                }
                else if(dxutils::stringContains(tag, "<u>") || dxutils::stringContains(tag, "</u>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    under = !under;
                }
                else if(dxutils::stringContains(tag, "<i>") || dxutils::stringContains(tag, "</i>")
                    || dxutils::stringContains(tag, "<em>") || dxutils::stringContains(tag, "</em>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    italic = !italic;
                }
                else if(dxutils::stringContains(tag, "<img"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
                    buffer->insert_pixbuf(buffer->end(), createPixbuf(dxutils::stringSection(tag,'\'',1,false)));
                }
                else if(dxutils::stringContains(tag, "<a"))
                {
                    for(size_t j=i+tagLength; j<length; j++)
                    {
                        if(text[j]=='<' && text[j+1]=='/' && (text[j+2]=='a'||text[j+2]=='A'))
                        {
                            tagLength += 4;
                            break;
                        }
                        tagLength++;
                    }
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    Glib::ustring link;
                    if(dxutils::stringContains(tag, "=\"", false))
                        link = dxutils::stringSection(tag,'\"',1,false);
                    else
                        link = dxutils::stringSection(tag,'\'',1,false);
                    if(dxutils::stringContains(link, "http://") || dxutils::stringContains(link, "https://")
                       || dxutils::stringContains(link, "ftp://") || dxutils::stringContains(link, "www."))
                    {
                        Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
                        buffer->insert_with_tag(buffer->end(), link, m_link);
                    }
                    else
                    {
                        normalText.append(dxutils::stringSection(link, ':', 1, true));
                    }
                }
                else if(dxutils::stringContains(tag, "<span"))
                {
                    std::vector<Glib::ustring> items = dxutils::stringSplit(tag, '=');
                    FOREACH(std::vector<Glib::ustring>, items, it)
                    {
                        if(dxutils::stringContains((*it), "'font-weight: bold"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            bold = !bold;
                            span.push_back(1);
                        }
                        else if(dxutils::stringContains((*it), "'text-decoration: underline"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            under = !under;
                            span.push_back(2);
                        }
                        else if(dxutils::stringContains((*it), "'color"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection(dxutils::stringSection((*it), ' ', 1, false), ';', 0, false);
                            foreColor = dxColor(color);
                            span.push_back(3);
                        }
                        else if(dxutils::stringContains((*it), "'background"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection(dxutils::stringSection((*it), ' ', 1, false), ';', 0, false);
                            backColor = dxColor(color);
                            span.push_back(4);
                        }
                    }
                }
                else if(dxutils::stringContains(tag, "</span>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    switch(span.back()) {
                        case 1: bold = !bold; break;
                        case 2: under = !under; break;
                        case 3: foreColor = DXPREFERENCES.m_textColor; break;
                        case 4: backColor = DXPREFERENCES.m_backroundColor; break;
                        default: break;
                    }
                    if(span.size()) span.pop_back();
                }
                else if(dxutils::stringContains(tag, "<font"))
                {
                    std::vector<Glib::ustring> items = dxutils::stringSplit(tag, '=');
                    FOREACH(std::vector<Glib::ustring>, items, it)
                    {
                        dxutils::debugLine(*it);
                        if(dxutils::stringContains((*it), "color"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection((*++it), '\"', 0, false);
                            foreColor = dxColor(color);
                        }
                        if(dxutils::stringContains((*it), "back"))
                        {
                            if(!normalText.empty())
                            {
                                appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                                normalText.clear();
                            }
                            Glib::ustring color = dxutils::stringSection((*++it), '\"', 0, false);
                            backColor = dxColor(color);
                        }
                    }
                }
                else if(dxutils::stringContains(tag, "</font>"))
                {
                    if(!normalText.empty())
                    {
                        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
                        normalText.clear();
                    }
                    foreColor = DXPREFERENCES.m_textColor;
                    backColor = DXPREFERENCES.m_backroundColor;
                }
                text.erase(i, tagLength);
                length = length-tagLength;
            }
        }
        else
        {
            normalText.append(1, text[i]);
            i++;
        }
    }
    if(!normalText.empty())
    {
        appendText(PURPLE::stripHtml(normalText), bold, under, italic, foreColor, backColor);
    }
    buffer->insert(buffer->end(), "\n");
}

void TabItem::appendText(const Glib::ustring &text, bool bold, bool underline, bool italic, dxColor fore, dxColor back)
{
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    std::vector< Glib::RefPtr<Gtk::TextTag> > tags;
    Glib::RefPtr<Gtk::TextTag> tag;
    if(bold)
    {
        tag = buffer->get_tag_table()->lookup("bold");
        if(!tag)
        {
            tag = buffer->create_tag("bold");
            tag->property_weight().set_value(Pango::WEIGHT_BOLD);
        }
        tags.push_back(tag);
    }
    if(underline)
    {
        tag = buffer->get_tag_table()->lookup("underline");
        if(!tag)
        {
            tag = buffer->create_tag("underline");
            tag->property_underline().set_value(Pango::UNDERLINE_SINGLE);
        }
        tags.push_back(tag);
    }
    if(italic)
    {
        tag = buffer->get_tag_table()->lookup("italic");
        if(!tag)
        {
            tag = buffer->create_tag("italic");
            tag->property_style().set_value(Pango::STYLE_ITALIC);
        }
        tags.push_back(tag);
    }
    if(fore != DXPREFERENCES.m_textColor)
    {
        tag = buffer->get_tag_table()->lookup(Glib::ustring::compose("fore%1", fore.to_string()));
        if(!tag)
        {
            tag = buffer->create_tag(Glib::ustring::compose("fore%1", fore.to_string()));
            tag->property_foreground().set_value(fore.to_string());
        }
        tags.push_back(tag);
    }
    if(back != DXPREFERENCES.m_backroundColor)
    {
        tag = buffer->get_tag_table()->lookup(Glib::ustring::compose("back%1", back.to_string()));
        if(!tag)
        {
            tag = buffer->create_tag(Glib::ustring::compose("back%1", back.to_string()));
            tag->property_background().set_value(back.to_string());
        }
        tags.push_back(tag);
    }
    buffer->insert_with_tags(buffer->end(), text, tags);
}

Glib::ustring TabItem::createSmileys(Glib::ustring text)
{
    dxStringIt it = DXPREFERENCES.m_smileysMap.begin();
    for(; it!=DXPREFERENCES.m_smileysMap.end(); ++it)
    {
        size_t pos = text.find((*it).first);
        if(pos!=Glib::ustring::npos)
        {
            while(pos!=Glib::ustring::npos) {
                text.replace(pos, (*it).first.length(), Glib::ustring::compose("<img src='%1'>", (*it).second));
                pos = text.find((*it).first, pos);
            }
        }
    }
    return text;
}

bool TabItem::onMotionEvent(GdkEventMotion* event)
{
    int x, y;
    m_text.window_to_buffer_coords(Gtk::TEXT_WINDOW_WIDGET, static_cast<int>(event->x), static_cast<int>(event->y), x, y);
    Gtk::TextIter iter;
    m_text.get_iter_at_location(iter, x, y);
    if(iter.has_tag(m_link) && !m_hovering)
    {
        m_text.get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(m_hand);
        m_hovering = true;
    }
    else if(!iter.has_tag(m_link) && m_hovering)
    {
        m_text.get_window(Gtk::TEXT_WINDOW_TEXT)->set_cursor(m_default);
        m_hovering = false;
    }
//    gdk_window_get_pointer(Gtk::Widget::get_window()->gobj(), NULL, NULL, NULL);
    return false;
}

void TabItem::onEventAfter(GdkEvent* event)
{
    if(event->type != GDK_BUTTON_RELEASE) return;
    GdkEventButton* button_event = &event->button;
    if(button_event->button != 1) return;
    Glib::RefPtr<Gtk::TextBuffer> buffer = m_text.get_buffer();
    Gtk::TextIter begin, end;
    buffer->get_selection_bounds(begin, end);
    if(begin != end) return;
    int x, y;
    m_text.window_to_buffer_coords(Gtk::TEXT_WINDOW_WIDGET, static_cast<int>(button_event->x), static_cast<int>(button_event->y), x, y);
    Gtk::TextIter iter;
    m_text.get_iter_at_location(iter, x, y);
    if(!iter.has_tag(m_link)) return;
    begin = end = iter;
    begin.backward_to_tag_toggle(m_link);
    end.forward_to_tag_toggle(m_link);
    dxutils::launchLink(begin.get_slice(end));
}

