/*
 * accountmm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "accountmm.h"
#include "buddymm.h"
#include "chatmm.h"
#include "purplemm.h"
#include "utilsmm.h"
#include "pi18n.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()
#define CNAME Purple::instance()->getName().c_str()

Account::ChatEntry::ChatEntry(std::string label, std::string identifier, std::string value, bool required, bool isInt, int min, int max, bool secret)
: m_label(label), m_identifier(identifier), m_value(value), m_required(required), m_isInt(isInt),
        m_min(min), m_max(max), m_secret(secret)
{}

Account::AccountStatusItem::AccountStatusItem(PurpleStatusType* status)
: m_id(p2s(purple_status_type_get_id(status))), m_name(p2s(purple_status_type_get_name(status)))
{}

Account::Account()
    : m_account(NULL)
{}

Account::Account(PurpleAccount* _account, Protocol _proto)
    : m_account(_account),
      m_proto(_proto)
{
    if(m_account && !m_proto.isValid())
    {
        if(m_account && m_account->gc)
            m_proto = Protocol(m_account->gc->prpl);
        else
            m_proto = Purple::instance()->getProtocolById(m_account->protocol_id);
    }
}

bool Account::operator==(const Account& account) const
{
    return this->isValid() && account.isValid() && account.m_account == this->m_account;
}

bool Account::operator!=(const Account& account) const
{
    return !isValid() || !account.isValid() || account.m_account != this->m_account;
}

bool Account::isEnabled() const
{
    return purple_account_get_enabled(m_account, CNAME);
}

std::string Account::getUsername() const
{
    assert(isValid());
    std::string result(m_account->username);
    return result;
}

std::string Account::getPassword() const
{
    assert(isValid());
    return p2s(purple_account_get_password(m_account));
}

std::string Account::getDisplayName() const
{
    assert(isValid());
    std::string ret = p2s(purple_account_get_name_for_display(m_account));
    if(ret.empty()) return getUsername();
    else return ret;
}

std::string Account::getAlias() const
{
    assert(isValid());
    return p2s(purple_account_get_alias(m_account));
}

void Account::setAlias(std::string alias)
{
    assert(isValid());
    purple_account_set_alias(m_account, alias.c_str());
}

void Account::setPassword(std::string password, bool remember)
{
    assert(isValid());
    purple_account_set_password(m_account, password.c_str());
    purple_account_set_remember_password(m_account, remember);
}

bool Account::getRememberPassword() const
{
    assert(isValid());
    return purple_account_get_remember_password(m_account);
}

void Account::registerAccount() const
{
    assert(isValid());
    purple_account_register(m_account);
}

std::vector<std::string> Account::getCommandsList() const
{
    assert(isValid());
    GList *actions, *l;
    std::vector<std::string> cmds;
    PurpleConnection *gc = getPurpleConnection();
    PurplePlugin *plugin = (gc && PURPLE_CONNECTION_IS_CONNECTED(gc)) ? gc->prpl : NULL;
    if(!plugin || !PURPLE_PLUGIN_HAS_ACTIONS(plugin))
        return cmds;
    actions = PURPLE_PLUGIN_ACTIONS(plugin, gc);
    for(l = actions; l != NULL; l = l->next)
    {
        if(l->data)
            cmds.push_back(((PurplePluginAction *) l->data)->label);
        else
            cmds.push_back("----");
    }
    g_list_free(actions);
    return cmds;
}

bool Account::callCommand(const std::string& cmd) const
{
    assert(isValid());
    GList *actions, *l;
    PurpleConnection *gc = getPurpleConnection();
    PurplePlugin *plugin = (gc && PURPLE_CONNECTION_IS_CONNECTED(gc)) ? gc->prpl : NULL;
    if(!plugin || !PURPLE_PLUGIN_HAS_ACTIONS(plugin))
        return false;
    actions = PURPLE_PLUGIN_ACTIONS(plugin, gc);
    for(l = actions; l != NULL; l = l->next)
    {
        if(l->data)
        {
            PurplePluginAction* action = (PurplePluginAction*)l->data;
            if(action->label == cmd)
            {
                action->plugin = plugin;
                action->context = gc;
                action->callback(action);
                return true;
            }
        }
    }
    return false;
}

Protocol::Options Account::getOptions() const
{
    assert(isValid());
    Protocol::Options options = getProtocol().getOptions();
    FOREACH(Protocol::Options, options, it)
    {
        Protocol::Option& option = *it;
        switch(option.getType())
        {
            case Protocol::Option::STR:
            case Protocol::Option::STRLIST:
                option.setValue(purple_account_get_string(m_account, option.getName().c_str(), option.getStringValue().c_str()));
                break;
            case Protocol::Option::INT:
                option.setValue(t2s(purple_account_get_int(m_account, option.getName().c_str(), option.getIntValue())));
                break;
            case Protocol::Option::BOOL:
                option.setValue(purple_account_get_bool(m_account, option.getName().c_str(), option.getBoolValue()) ? "true" : "false");
                break;
            case Protocol::Option::NONE:
                /* not supported. */
                break;
        }
    }
    return options;
}

void Account::setOptions(const Protocol::Options& options)
{
    assert(isValid());
    for(Protocol::Options::const_iterator it = options.begin(); it != options.end(); ++it)
    {
        const Protocol::Option& option = *it;
        switch(option.getType())
        {
            case Protocol::Option::STR:
            case Protocol::Option::STRLIST:
                purple_account_set_string(m_account,
                              option.getName().c_str(),
                              option.getStringValue().c_str());
                break;
            case Protocol::Option::INT:
                purple_account_set_int(m_account,
                               option.getName().c_str(),
                               option.getIntValue());
                break;
            case Protocol::Option::BOOL:
                purple_account_set_bool(m_account,
                                option.getName().c_str(),
                                option.getBoolValue());
                break;
            case Protocol::Option::NONE:
                /* Not happy. */
                break;
        }
    }
}

void Account::setBuddyIcon(std::string filename)
{
    assert(isValid());
    if(getProtocol().hasBuddyIcon() && !filename.empty())
    {
        PurplePluginProtocolInfo *prplinfo = getProtocol().getPurpleProtocol();
        GError* temp_error = NULL;
        gchar *contents;
        gsize len;
        if(g_file_get_contents(filename.c_str(), &contents, &len, &temp_error))
        {
            if((prplinfo->icon_spec.max_filesize != 0) &&
                (len > prplinfo->icon_spec.max_filesize))
            {
                std::ostringstream oss;
                oss << "Unable to get image content on " << m_proto.getName().c_str() << "-" << getUsername().c_str() << ": image too large";
                if(CALLBACKS) CALLBACKS->showError(oss.str());
            }
            else
            {
                purple_buddy_icons_set_account_icon(m_account, (guchar *)contents, len);
            }
        }
        else if(temp_error)
        {
            std::ostringstream oss;
            oss << "Unable to get image content on " << m_proto.getName().c_str() << "-" << getUsername().c_str() << ": " << temp_error->message;
            if(CALLBACKS) CALLBACKS->showError(oss.str());
            g_error_free(temp_error);
        }
    }
}

std::string Account::getBuddyIcon() const
{
    assert(isValid());
    std::string path = "";
    if(getProtocol().hasBuddyIcon())
    {
        PurpleStoredImage *img = purple_buddy_icons_find_account_icon(m_account);
        if(img)
        {
            path = Purple::instance()->getConfigPath();
            path += G_DIR_SEPARATOR_S;
            path += "icons";
            path += G_DIR_SEPARATOR_S;
            path += purple_imgstore_get_filename(img);
        }
    }
    return path;
}

void Account::removeBuddyIcon()
{
    assert(isValid());
    purple_buddy_icons_set_account_icon(m_account, NULL, 0);
}

void Account::setCheckMail(bool check)
{
    assert(isValid());
    if(getProtocol().hasCheckMail()) purple_account_set_check_mail(m_account, check);
}

bool Account::getCheckMail()
{
    assert(isValid());
    if(getProtocol().hasCheckMail())
        return purple_account_get_check_mail(m_account);
    else
        return false;
}

void Account::setStatus(PurpleStatusPrimitive prim, std::string message)
{
    assert(isValid());
    PurpleSavedStatus *saved_status = purple_savedstatus_find_transient_by_type_and_message(prim, message.empty()?NULL:message.c_str());
    if(saved_status == NULL)
    {
        saved_status = purple_savedstatus_new(NULL, prim);
        if(!message.empty())
            purple_savedstatus_set_message(saved_status, message.c_str());
    }
    purple_savedstatus_activate_for_account(saved_status, m_account);
    if(prim != PURPLE_STATUS_OFFLINE && !isConnected())
        purple_account_connect(m_account);
    if(prim == PURPLE_STATUS_OFFLINE && CALLBACKS)
        CALLBACKS->accountDisconnected(*this, "");
}

void Account::setStatus(std::string id)
{
    assert(isValid());
    purple_account_set_status(m_account, id.c_str(), true, NULL);
    PurpleStatusPrimitive prim = purple_status_type_get_primitive(purple_status_get_type(purple_account_get_status(m_account, id.c_str())));
    if(prim != PURPLE_STATUS_OFFLINE && !isConnected())
        purple_account_connect(m_account);
    if(prim == PURPLE_STATUS_OFFLINE && CALLBACKS)
        CALLBACKS->accountDisconnected(*this, "");
}

void Account::setStatusMessage(const std::string& message)
{
    assert(isValid());
    if(message.empty())
        setStatus(getStatus());
    PurpleStatusPrimitive prim = getStatus();
    PurpleSavedStatus *saved_status = purple_savedstatus_find_transient_by_type_and_message(prim, message.empty()?NULL:message.c_str());
    if(saved_status == NULL)
    {
        saved_status = purple_savedstatus_new(NULL, prim);
        if(!message.empty())
            purple_savedstatus_set_message(saved_status, message.c_str());
    }
    purple_savedstatus_activate_for_account(saved_status, m_account);
    if(prim != PURPLE_STATUS_OFFLINE && !isConnected())
        purple_account_connect(m_account);
}

PurpleStatusPrimitive Account::getStatus() const
{
    assert(isValid());
    return isConnected()?purple_status_type_get_primitive(purple_status_get_type(purple_presence_get_active_status(m_account->presence)))
                        :PURPLE_STATUS_OFFLINE;
}

std::string Account::getStatusID() const
{
    assert(isValid());
    return p2s(purple_status_type_get_id(purple_status_get_type(purple_presence_get_active_status(m_account->presence))));
}

std::string Account::getStatusName() const
{
    assert(isValid());
    return isConnected()?p2s(purple_status_type_get_name(purple_status_get_type(purple_presence_get_active_status(m_account->presence))))
                        :p2s(purple_primitive_get_name_from_type(PURPLE_STATUS_OFFLINE));
}

std::string Account::getStatusMessage() const
{
    assert(isValid());
    std::string msg;
    PurpleStatus *s = purple_account_get_status(m_account, getStatusID().c_str());
    if(!s)
        return msg;
    return isConnected()?p2s(purple_status_get_attr_string(s, "message")):msg;
}

bool Account::hasServerAliases() const
{
    assert(isValid());
    return purple_account_get_ui_bool(m_account, CNAME, "server_aliases", true);
}

void Account::setServerAliases(bool b)
{
    assert(isValid());
    purple_account_set_ui_bool(m_account, CNAME, "server_aliases", b);
}

PurpleConnection* Account::getPurpleConnection() const
{
    assert(isValid());
    return purple_account_get_connection(m_account);
}

bool Account::isConnected() const
{
    assert(isValid());
    return purple_account_is_connected(m_account);
}

bool Account::isConnecting() const
{
    assert(isValid());
    return purple_account_is_connecting(m_account);
}

std::vector<Buddy> Account::getBuddies() const
{
    assert(isValid());
    std::vector<Buddy> buddies;
    if(!purple_get_blist())
        return buddies;
    GSList* bl = purple_find_buddies(m_account, NULL);
    GSList* cur;
    for(cur = bl; cur != NULL; cur = cur->next)
    {
        if(!PURPLE_BLIST_NODE_IS_BUDDY((PurpleBlistNode*)cur->data))
            continue;
        PurpleBuddy* b = (PurpleBuddy*)cur->data;
        buddies.push_back(Buddy(b));
    }
    g_slist_free(bl);
    return buddies;
}

std::vector<Chat> Account::getChats() const
{
    assert(isValid());
    std::vector<Chat> chats;
    if(!purple_get_blist())
        return chats;
    PurpleBuddyList *list = purple_get_blist();
    PurpleBlistNode *node;
    node = list->root;
    while(node)
    {
        if(PURPLE_BLIST_NODE_IS_CHAT(node))
        {
            PurpleChat* ch = (PurpleChat*)node;
            if(ch->account == m_account) chats.push_back(Chat(ch));
        }
        node = purple_blist_node_next(node, FALSE);
    }
    return chats;
}

std::vector<std::string> Account::getGroups() const
{
    assert(isValid());
    std::vector<std::string> groups;
    if(!purple_get_blist())
        return groups;
    PurpleBuddyList *list = purple_get_blist();
    PurpleBlistNode *node;
    node = list->root;
    while(node)
    {
        if(PURPLE_BLIST_NODE_IS_GROUP(node))
        {
            PurpleGroup* g = (PurpleGroup*)node;
            if(purple_group_on_account(g, m_account)) groups.push_back(p2s(g->name));
        }
        node = purple_blist_node_next(node, FALSE);
    }
    return groups;
}

std::vector<std::string> Account::getAllGroups() const
{
    assert(isValid());
    std::vector<std::string> groups;
    if(!purple_get_blist())
        return groups;
    PurpleBuddyList *list = purple_get_blist();
    PurpleBlistNode *node;
    node = list->root;
    while(node)
    {
        if(PURPLE_BLIST_NODE_IS_GROUP(node))
        {
            groups.push_back(p2s(((PurpleGroup*)node)->name));
        }
        node = purple_blist_node_next(node, FALSE);
    }
    return groups;
}

bool Account::isGroupOnline(std::string group) const
{
    assert(isValid());
    if(!purple_get_blist())
        return false;
    if(!isConnected())
        return false;
    PurpleBuddyList *list = purple_get_blist();
    PurpleBlistNode *node;
    node = list->root;
    while(node)
    {
        if(PURPLE_BLIST_NODE_IS_CHAT(node))
        {
            PurpleChat* ch = (PurpleChat*)node;
            PurpleGroup* g = purple_chat_get_group(ch);
            std::string gname = g && g->name ? g->name : "";
            if(ch->account == m_account && gname == group) return true;
        }
        if(PURPLE_BLIST_NODE_IS_BUDDY(node))
        {
            PurpleBuddy* b = (PurpleBuddy*)node;
            PurpleGroup* g = purple_buddy_get_group(b);
            std::string gname = g && g->name ? g->name : "";
            if(b->account == m_account && PURPLE_BUDDY_IS_ONLINE(b)
                && gname == group) return true;
        }
        node = purple_blist_node_next(node, FALSE);
    }
    return false;
}

std::vector<Account::AccountStatusItem> Account::getStatusMenuItems() const
{
    assert(isValid());
    std::vector<Account::AccountStatusItem> statuses;
    for(GList *iter = purple_account_get_status_types(m_account); iter; iter = iter->next)
    {
        PurpleStatusType *status_type = reinterpret_cast<PurpleStatusType*>(iter->data);
        if(purple_status_type_is_independent(status_type))
            continue;
        statuses.push_back(AccountStatusItem(status_type));
    }
    return statuses;
}

void Account::displayRoomList() const
{
    assert(isValid());
    purple_roomlist_get_list(m_account->gc);
}

void Account::connect()
{
    assert(isValid());
    purple_account_set_enabled(m_account, CNAME, true);
    purple_account_connect(m_account);
}

void Account::disconnect()
{
    assert(isValid());
    removeReconnection();
    purple_account_set_enabled(m_account, CNAME, false);
    if(CALLBACKS) CALLBACKS->accountDisconnected(*this, "");
}

int Account::delayReconnect() const
{
    assert(isValid());
    int delay = purple_account_get_ui_int(m_account, CNAME, "delay-reconnect", 15);
    delay *= 2;
    int id = purple_account_get_ui_int(m_account, CNAME, "id-reconnect", -1);
    if(id >= 0)
        g_source_remove(id);
    id = g_timeout_add(delay*1000, Account::reconnect, m_account);
    purple_account_set_ui_int(m_account, CNAME, "delay-reconnect", delay);
    purple_account_set_ui_int(m_account, CNAME, "id-reconnect", id);
    return delay;
}

void Account::removeReconnection() const
{
    assert(isValid());
    int id = purple_account_get_ui_int(m_account, CNAME, "id-reconnect", -1);
    if(id >= 0)
    {
        g_source_remove(id);
    }
    purple_account_set_ui_int(m_account, CNAME, "delay-reconnect", 15);
    purple_account_set_ui_int(m_account, CNAME, "id-reconnect", -1);
}

std::vector<std::string> Account::getDenyList() const
{
    assert(isValid());
    std::vector<std::string> list;
    GSList *l;
    for(l = m_account->deny; l != NULL; l = l->next)
        list.push_back((const char*)l->data);
    return list;
}

bool Account::deny(const std::string& who) const
{
    assert(isValid());
    if(m_account->perm_deny != PURPLE_PRIVACY_DENY_USERS)
    {
        m_account->perm_deny = PURPLE_PRIVACY_DENY_USERS;
        if(purple_account_is_connected(m_account))
            serv_set_permit_deny(purple_account_get_connection(m_account));
    }
    return purple_privacy_deny_add(m_account, who.c_str(), FALSE);
}

bool Account::allow(const std::string& who) const
{
    assert(isValid());
    return purple_privacy_deny_remove(m_account, who.c_str(), FALSE);
}

void Account::addBuddy(const std::string& username, const std::string& group, const std::string& invite, const std::string& alias) const
{
    assert(isValid());
    assert(username.empty() == false);
    assert(group.empty() == false);
    PurpleGroup* grp = purple_find_group(group.c_str());
    if(!grp)
    {
        grp = purple_group_new(group.c_str());
        purple_blist_add_group(grp, NULL);
    }
    PurpleBuddy* buddy = purple_buddy_new(m_account, username.c_str(), alias.empty()?username.c_str():alias.c_str());
    purple_blist_add_buddy(buddy, NULL, grp, NULL);
    purple_account_add_buddy_with_invite(m_account, buddy, invite.c_str());
    if(CALLBACKS) CALLBACKS->buddyAdded(Buddy(buddy));
}

void Account::removeBuddy(Buddy buddy)
{
    assert(isValid());
    purple_account_remove_buddy(m_account, buddy.getPurpleBuddy(), buddy.getPurpleGroup());
    purple_blist_remove_buddy(buddy.getPurpleBuddy());
    if(CALLBACKS) CALLBACKS->buddyRemoved(*this, buddy.getUsername());
}

Buddy Account::findBuddy(const std::string& username) const
{
    assert(isValid());
    PurpleBuddy *b = NULL;
    GSList* bl = purple_find_buddies(m_account, username.c_str());
    GSList* cur;
    for(cur = bl; cur != NULL; cur = cur->next)
    {
        if(!PURPLE_BLIST_NODE_IS_BUDDY((PurpleBlistNode*)cur->data))
            continue;
        b = (PurpleBuddy*)cur->data;
        break;
    }
    g_slist_free(bl);
    if(b) return Buddy(b);
    else return Buddy();
}

void Account::moveBuddy(Buddy buddy, const std::string& group)
{
    assert(isValid());
    if(buddy.isValid())
    {
        PurpleGroup* grp = purple_find_group(group.c_str());
        if(!grp)
        {
            grp = purple_group_new(group.c_str());
            purple_blist_add_group(grp, NULL);
        }
        purple_blist_add_buddy(buddy.getPurpleBuddy(), NULL, grp, NULL);
        if(CALLBACKS)
        {
            CALLBACKS->buddyRemoved(*this, buddy.getUsername());
            CALLBACKS->buddyAdded(buddy);
        }
    }
}

void Account::addGroup(const std::string& group) const
{
    assert(isValid());
    PurpleGroup* grp = purple_find_group(group.c_str());
    if(!grp)
    {
        grp = purple_group_new(group.c_str());
        purple_blist_add_group(grp, NULL);
    }
}

void Account::addChat(std::vector<ChatEntry> parameters, const std::string& name, const std::string& group, bool autojoin) const
{
    assert(isValid());
    PurpleGroup* grp = purple_find_group(group.c_str());
    if(!grp)
    {
        grp = purple_group_new(group.c_str());
        purple_blist_add_group(grp, NULL);
    }
    PurpleChat *chat;
    GHashTable *hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
    /* Parse parameters */
    FOREACH(std::vector<ChatEntry>, parameters, it)
    {
        g_hash_table_replace(hash, g_strdup((*it).getIdentifier().c_str()),
                g_strdup((*it).getValue().c_str()));
    }
    chat = purple_chat_new(m_account, name.c_str(), hash);
    if(chat != NULL)
    {
        purple_blist_node_set_bool((PurpleBlistNode*)chat, "purplemm-autojoin", autojoin);
        purple_blist_add_chat(chat, grp, NULL);
        if(CALLBACKS) CALLBACKS->chatAdded(Chat(chat));
        if(autojoin) serv_join_chat(purple_account_get_connection(m_account), chat->components);
    }
}

void Account::moveChat(Chat chat, const std::string& group)
{
    assert(isValid());
    if(chat.isValid())
    {
        PurpleGroup* grp = purple_find_group(group.c_str());
        if(!grp)
        {
            grp = purple_group_new(group.c_str());
            purple_blist_add_group(grp, NULL);
        }
        purple_blist_add_chat(chat.getPurpleChat(), grp, NULL);
        if(CALLBACKS)
        {
            CALLBACKS->chatRemoved(*this, chat.getAlias());
            CALLBACKS->chatAdded(chat);
        }
    }
}

void Account::removeChat(Chat chat) const
{
    assert(isValid());
    purple_blist_remove_chat(chat.getPurpleChat());
    if(CALLBACKS) CALLBACKS->chatRemoved(*this, chat.getAlias());
}

Chat Account::findChat(const std::string& name) const
{
    assert(isValid());
    if(!purple_get_blist())
        return Chat();
    PurpleBuddyList *list = purple_get_blist();
    PurpleBlistNode *node;
    PurpleChat *chat = NULL;
    node = list->root;
    while(node)
    {
        if(PURPLE_BLIST_NODE_IS_CHAT(node))
        {
            PurpleChat* ch = (PurpleChat*)node;
            if(ch->account == m_account && !name.compare(ch->alias))
            {
                chat = ch;
                break;
            }
        }
        node = purple_blist_node_next(node, FALSE);
    }
    if(chat) return Chat(chat);
    else return Chat();
}

PurpleStatusPrimitive Account::getBuddyStatus(const std::string& username) const
{
    assert(isValid());
    PurpleBuddy *buddy = purple_find_buddy(m_account, username.c_str());
    if(buddy)
    {
        return purple_status_type_get_primitive(purple_status_get_type(purple_presence_get_active_status(buddy->presence)));
    }
    return PURPLE_STATUS_OFFLINE;
}

bool Account::supportsChats() const
{
    assert(isValid());
    PurpleConnection *gc = getPurpleConnection();
    if(!gc)
        return false;
    PurplePluginProtocolInfo *prpl_info = NULL;
    prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
    return (prpl_info->chat_info != NULL);
}

bool Account::supportsSendFile() const
{
    assert(isValid());
    PurpleConnection *gc = getPurpleConnection();
    if(!gc)
        return false;
    PurplePluginProtocolInfo *prpl_info = NULL;
    prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
    return (prpl_info->send_file != NULL);
}

bool Account::supportRoomlist() const
{
    assert(isValid());
    PurpleConnection *gc = getPurpleConnection();
    if(!gc)
        return false;
    PurplePluginProtocolInfo *prpl_info = NULL;
    prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
    return (prpl_info->roomlist_get_list != NULL);
}

std::vector<Account::ChatEntry> Account::getChatParameters() const
{
    assert(isValid());
    assert(isConnected());
    PurpleConnection* gc = getPurpleConnection();
    std::vector<Account::ChatEntry> v;
    GList *list = NULL, *tmp;
    GHashTable *defaults = NULL;
    if(PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info != NULL)
        list = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info(gc);
    if(PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info_defaults != NULL)
        defaults = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info_defaults(gc, NULL);
    for(tmp = list; tmp; tmp = tmp->next)
    {
        struct proto_chat_entry *pce = static_cast<struct proto_chat_entry*>(tmp->data);
        char* value = (char*)g_hash_table_lookup(defaults, pce->identifier);
        ChatEntry entry(p2s(pce->label), p2s(pce->identifier), value?value:"", pce->required,
                pce->is_int, pce->min, pce->max, pce->secret);
        v.push_back(entry);
        g_free(pce);
    }
    g_list_free(list);
    g_hash_table_destroy(defaults);
    return v;
}

bool Account::joinChat(std::vector<Account::ChatEntry> parameters) const
{
    assert(isValid());
    if(!isConnected())
    {
        return false;
    }
    PurpleChat *chat;
    GHashTable *hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
    /* Parse parameters */
    FOREACH(std::vector<ChatEntry>, parameters, it)
    {
        g_hash_table_replace(hash, g_strdup((*it).getIdentifier().c_str()),
                g_strdup((*it).getValue().c_str()));
    }
    chat = purple_chat_new(m_account, NULL, hash);
    if(chat != NULL)
    {
        PurpleConversation *conv;
        PurplePluginProtocolInfo *prpl_info;
        const char *name;
        char *chat_name;
        prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(purple_find_prpl(purple_account_get_protocol_id(m_account)));
        if(prpl_info && prpl_info->get_chat_name)
            chat_name = prpl_info->get_chat_name(hash);
        else
            chat_name = NULL;
        if(chat_name)
            name = chat_name;
        else
            name = purple_chat_get_name(chat);
        conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_CHAT, name, m_account);
        if(conv != NULL)
            purple_conversation_present(conv);
        serv_join_chat(purple_account_get_connection(m_account), hash);
        g_free(chat_name);
    }
    return chat != NULL;
}

/* STATIC */

PurpleConnectionUiOps Account::connOps =
{
    Account::connecting,
    Account::connected,
    NULL,/* disconnected */
    NULL, /* notice */
    NULL,
    NULL, /* network_connected */
    NULL, /* network_disconnected */
    Account::disconnectReason,
    NULL,
    NULL,
    NULL
};

PurpleAccountUiOps Account::accOps =
{
    accountAdded,
    accountStatusChanged,
    requestAdd,
    requestAuthorize,
    requestClose,
    NULL,
    NULL,
    NULL,
    NULL
};

void* Account::getHandler()
{
    static int handler;
    return &handler;
}

void Account::init()
{
    purple_connections_set_ui_ops(&connOps);
    purple_accounts_set_ui_ops(&accOps);
    purple_signal_connect(purple_accounts_get_handle(), "account-added",
                getHandler(), PURPLE_CALLBACK(accountCreated),
                NULL);
    purple_signal_connect(purple_accounts_get_handle(), "account-removed",
                getHandler(), PURPLE_CALLBACK(accountRemoved),
                NULL);
    purple_signal_connect(purple_connections_get_handle(), "signed-on", getHandler(),
                G_CALLBACK(accountSignedOn),
                GINT_TO_POINTER(PURPLE_CONV_ACCOUNT_ONLINE));
    purple_signal_connect(purple_connections_get_handle(), "signing-off", getHandler(),
                G_CALLBACK(accountSigningOff), NULL);
}

void Account::uninit()
{
    purple_accounts_set_ui_ops(NULL);
    purple_connections_set_ui_ops(NULL);
    purple_signals_disconnect_by_handle(getHandler());
}

gboolean Account::reconnect(void* data)
{
    Account acc((PurpleAccount*)data);
    purple_account_set_ui_int(acc.m_account, CNAME, "id-reconnect", -1);
    acc.connect();
    return FALSE;
}

void Account::accountCreated(PurpleAccount* a)
{
    Account account(a);
    if(account.isValid() && CALLBACKS) CALLBACKS->accountAdded(account);
}

void Account::accountRemoved(PurpleAccount* a)
{
    Account account(a);
    account.removeReconnection();
    if(account.isValid() && CALLBACKS) CALLBACKS->accountRemoved(account.getUsername(), account.getProtocol().getPurpleID());
}

static char *make_info(PurpleAccount *account, PurpleConnection *gc, const char *remote_user,
          const char *id, const char *alias, const char *msg)
{
    if(msg != NULL && *msg == '\0')
        msg = NULL;
    return g_strdup_printf("%s%s%s%s has made %s his or her buddy%s%s",
            remote_user,
            (alias != NULL ? " ("  : ""),
            (alias != NULL ? alias : ""),
            (alias != NULL ? ")"   : ""),
            (id != NULL
                ? id
                : (purple_connection_get_display_name(gc) != NULL
                   ? purple_connection_get_display_name(gc)
                   : purple_account_get_username(account))),
            (msg != NULL ? ": " : "."),
            (msg != NULL ? msg  : ""));
}

typedef struct
{
    PurpleAccount *account;
    char *username;
    char *alias;
} AddUserData;

static void
free_add_user_data(AddUserData *data)
{
    g_free(data->username);
    if (data->alias != NULL)
        g_free(data->alias);
    g_free(data);
}

static void
add_user_cb(AddUserData *data)
{
    PurpleConnection *gc = purple_account_get_connection(data->account);
    if(g_list_find(purple_connections_get_all(), gc))
    {
        purple_blist_request_add_buddy(data->account, data->username,
                                     NULL, data->alias);
    }
    free_add_user_data(data);
}

void Account::accountAdded(PurpleAccount *account, const char *remote_user,
                const char *id, const char *alias,
                const char *msg)
{
    char *buffer;
    PurpleConnection *gc;
    gc = purple_account_get_connection(account);
    buffer = make_info(account, gc, remote_user, id, alias, msg);
    purple_notify_info(NULL, NULL, buffer, NULL);
    g_free(buffer);
}

void Account::accountStatusChanged(PurpleAccount *account, PurpleStatus*)
{
    Account a(account);
    if(a.isValid() && CALLBACKS) CALLBACKS->accountStatusChanged(a);
}

void Account::requestAdd(PurpleAccount *account, const char *remote_user,
              const char *id, const char *alias,
              const char *msg)
{
    char *buffer;
    PurpleConnection *gc;
    AddUserData *data;
    gc = purple_account_get_connection(account);
    data = g_new0(AddUserData, 1);
    data->account  = account;
    data->username = g_strdup(remote_user);
    data->alias    = (alias != NULL ? g_strdup(alias) : NULL);
    buffer = make_info(account, gc, remote_user, id, alias, msg);
    purple_request_action(NULL, NULL, "Add buddy to your list?",
                        buffer, PURPLE_DEFAULT_ACTION_NONE,
                        account, remote_user, NULL,
                        data, 2,
                        "Add",    G_CALLBACK(add_user_cb),
                        "Cancel", G_CALLBACK(free_add_user_data));
    g_free(buffer);
}

typedef struct {
    PurpleAccountRequestAuthorizationCb auth_cb;
    PurpleAccountRequestAuthorizationCb deny_cb;
    void *data;
    char *username;
    char *alias;
    PurpleAccount *account;
} auth_and_add;

static void
free_auth_and_add(auth_and_add *aa)
{
    g_free(aa->username);
    g_free(aa->alias);
    g_free(aa);
}

static void
authorize_and_add_cb(auth_and_add *aa)
{
    aa->auth_cb(aa->data);
    purple_blist_request_add_buddy(aa->account, aa->username,
                               NULL, aa->alias);
    free_auth_and_add(aa);
}

static void
deny_no_add_cb(auth_and_add *aa)
{
    aa->deny_cb(aa->data);
    free_auth_and_add(aa);
}

void *Account::requestAuthorize(PurpleAccount *account,
                const char *remote_user,
                const char *id,
                const char *alias,
                const char *message,
                gboolean on_list,
                PurpleAccountRequestAuthorizationCb auth_cb,
                PurpleAccountRequestAuthorizationCb deny_cb,
                void *user_data)
{
    char *buffer;
    PurpleConnection *gc;
    void *uihandle;
    gc = purple_account_get_connection(account);
    if(message != NULL && *message == '\0')
        message = NULL;
    buffer = g_strdup_printf("%s%s%s%s wants to add %s to his or her buddy list%s%s",
                remote_user,
                        (alias != NULL ? " ("  : ""),
                        (alias != NULL ? alias : ""),
                        (alias != NULL ? ")"   : ""),
                        (id != NULL
                        ? id
                        : (purple_connection_get_display_name(gc) != NULL
                        ? purple_connection_get_display_name(gc)
                        : purple_account_get_username(account))),
                        (message != NULL ? ": " : "."),
                        (message != NULL ? message  : ""));
    if(!on_list)
    {
        auth_and_add *aa = g_new(auth_and_add, 1);
        aa->auth_cb = auth_cb;
        aa->deny_cb = deny_cb;
        aa->data = user_data;
        aa->username = g_strdup(remote_user);
        aa->alias = g_strdup(alias);
        aa->account = account;
        uihandle = purple_request_action(NULL, "Authorize buddy?", buffer, NULL,
            PURPLE_DEFAULT_ACTION_NONE,
            account, remote_user, NULL,
            aa, 2,
            "Authorize", authorize_and_add_cb,
            "Deny", deny_no_add_cb);
    }
    else
    {
        uihandle = purple_request_action(NULL, "Authorize buddy?", buffer, NULL,
            PURPLE_DEFAULT_ACTION_NONE,
            account, remote_user, NULL,
            user_data, 2,
            "Authorize", auth_cb,
            "Deny", deny_cb);
    }
    g_free(buffer);
    return uihandle;
}

void Account::requestClose(void *uihandle)
{
    purple_request_close(PURPLE_REQUEST_ACTION, uihandle);
}

void Account::connecting(PurpleConnection *gc,
        const char *text,
        size_t step,
        size_t step_count)
{
    Account account(gc->account);
    std::string msg;
    if(!step)
    {
        std::ostringstream oss;
        oss << "Connection to " << account.getUsername() << " in progress...";
        msg = oss.str();
    }
    else
    {
        std::ostringstream oss;
        oss << "(" << step << "/" << step_count-1 << "): " << text;
        msg = oss.str();
    }
    if(account.isValid() && CALLBACKS) CALLBACKS->accountConnecting(account, msg);
}

void Account::connected(PurpleConnection* gc)
{
    Account account(gc->account);
    account.removeReconnection();
    if(account.isValid() && CALLBACKS)
    {
        CALLBACKS->accountConnected(account);
        std::vector<Chat> chats = account.getChats();
        FOREACH(std::vector<Chat>, chats, it)
        {
            if((*it).getAutojoin())
                (*it).join();
        }
    }
}

void Account::accountSigningOff(PurpleConnection *gc)
{
    GList *list = purple_get_chats();
    PurpleAccount *account = purple_connection_get_account(gc);
    /* We are about to sign off. See which chats we are currently in, and mark them for rejoin on reconnect. */
    while(list)
    {
        PurpleConversation *conv = (PurpleConversation*)list->data;
        if(!purple_conv_chat_has_left(PURPLE_CONV_CHAT(conv)) && purple_conversation_get_account(conv) == account)
        {
            purple_conversation_set_data(conv, "want-to-rejoin", GINT_TO_POINTER(TRUE));
            purple_conversation_write(conv, NULL, _("The account has disconnected and you are no longer in this chat. You will automatically rejoin the chat when the account reconnects."),
                                        PURPLE_MESSAGE_SYSTEM, time(NULL));
        }
        list = list->next;
    }
}

void Account::accountSignedOn(PurpleConnection *gc, gpointer /*event*/)
{
    Account account(gc->account);
    if(account.isValid() && CALLBACKS) CALLBACKS->accountSignedOn(account);
    GList *iter;
    for(iter = purple_get_conversations(); iter; iter = iter->next)
    {
        PurpleConversation *conv = (PurpleConversation*)iter->data;
        if(PURPLE_CONNECTION_IS_CONNECTED(gc) && conv->type == PURPLE_CONV_TYPE_CHAT &&
                conv->account == gc->account && purple_conversation_get_data(conv, "want-to-rejoin"))
        {
            GHashTable *comps = NULL;
            PurpleChat *chat = purple_blist_find_chat(conv->account, conv->name);
            if(chat == NULL)
            {
                if(PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info_defaults != NULL)
                    comps = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)->chat_info_defaults(gc, conv->name);
            }
            else
            {
                comps = chat->components;
            }
            serv_join_chat(gc, comps);
            if(chat == NULL && comps != NULL)
                g_hash_table_destroy(comps);
        }
    }
}

void Account::disconnectReason(PurpleConnection *gc,
                PurpleConnectionError reason,
                const char *text)
{
    Account acc(gc->account);
    if(acc.isValid() && CALLBACKS)
    {
        CALLBACKS->accountDisconnected(acc, p2s(text));
    }
    switch(reason)
    {
    case PURPLE_CONNECTION_ERROR_NETWORK_ERROR:
    case PURPLE_CONNECTION_ERROR_ENCRYPTION_ERROR:
    case PURPLE_CONNECTION_ERROR_OTHER_ERROR:
        acc.delayReconnect();
        break;
    default:
        break; /* Do not auto reconnect. */
    }
}

} /* namespace PURPLE */
