/*
 * accountmm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef ACCOUNTMM_H
#define ACCOUNTMM_H

#include <purple.h>
#include <string>
#include <vector>
#include "protocolmm.h"
#include "exportmm.h"

namespace PURPLE
{
class Buddy;
class Chat;

/** This class represents an account. */
class PURPLEMM_EXPORT Account
{
    PurpleAccount* m_account;
    Protocol m_proto;

    static PurpleConnectionUiOps connOps;
    static PurpleAccountUiOps accOps;
    static void* getHandler();
    static void accountSignedOn(PurpleConnection *gc, gpointer event);
    static void accountSigningOff(PurpleConnection *gc);
    static void accountRemoved(PurpleAccount*);
    static void accountCreated(PurpleAccount*);
    static void accountAdded(PurpleAccount *account, const char *remote_user,
                    const char *id, const char *alias,
                    const char *msg);
    static void accountStatusChanged(PurpleAccount *account, PurpleStatus *status);
    static void requestAdd(PurpleAccount *account, const char *remote_user,
                  const char *id, const char *alias,
                  const char *msg);
    static void *requestAuthorize(PurpleAccount *account,
                const char *remote_user,
                const char *id,
                const char *alias,
                const char *message,
                gboolean on_list,
                PurpleAccountRequestAuthorizationCb auth_cb,
                PurpleAccountRequestAuthorizationCb deny_cb,
                void *user_data);
    static void requestClose(void *uihandle);
    static void connecting(PurpleConnection *gc,
                           const char *text,
                           size_t step,
                           size_t step_count);
    static void connected(PurpleConnection* gc);
    static void disconnectReason(PurpleConnection *gc,
                                  PurpleConnectionError reason,
                                  const char *text);
    static gboolean reconnect(void*);
public:
    /** This class represents an chat fields. */
    class PURPLEMM_EXPORT ChatEntry {
    private:
        std::string m_label;        // User-friendly name of the entry
        std::string m_identifier;   // Used by the PRPL to identify the option
        std::string m_value;        // Entry value
        bool m_required;            // True if is field required
        bool m_isInt;               // True if the entry expects an integer
        int m_min;                  // Minimum value in case of integer
        int m_max;                  // Maximum value in case of integer
        bool m_secret;              // True if the entry is secret (password)
    public:
        /** Create chat entry instance */
        ChatEntry(std::string label, std::string identifier, std::string value, bool required, bool isInt, int min, int max, bool secret);

        /** Get user-friendly name of the entry */
        std::string getLabel() const { return m_label; }
        /** Get identify the option */
        std::string getIdentifier() const { return m_identifier; }
        /** Get entry value */
        std::string getValue() const { return m_value; }
        /** Set entry value */
        void setValue(std::string value) { m_value = value; }
        /** Get it's required */
        bool getRequired() const { return m_required; }
        /** Get entry expects an integer */
        bool getIsInt() const { return m_isInt; }
        /** Get minimum value in case of integer */
        int getMin() const { return m_min; }
        /** Get maximum value in case of integer */
        int getMax() const { return m_max; }
        /** Get entry is secret (password) */
        bool getSecret() const { return m_secret; }
    };

    /** This class represents a account status item */
    class PURPLEMM_EXPORT AccountStatusItem {
    private:
        std::string m_id, m_name;
    public:
        /** Create account status item instance*/
        AccountStatusItem(PurpleStatusType *status);
        /** Get item id */
        std::string getID() const { return m_id; }
        /** Get item name */
        std::string getName() const { return m_name; }
    };

    /** Initialization of libpurple accounts' stuffs. */
    static void init();
    /** Uninitialization of libpurple accounts' stuffs. */
    static void uninit();

    /** Empty constructor */
    Account();
    /** Create an account instance.
     *
     * @param account  the libpurple's account object.
     * @param proto  optional argument to provide the protocol object.
     */
    Account(PurpleAccount* account, Protocol proto = Protocol());

    /** Comparaison between two accounts */
    bool operator==(const Account&) const;
    /** Comparaison between two accounts */
    bool operator!=(const Account&) const;
    /** Get true if Account is valid */
    bool isValid() const { return m_account != NULL; }
    /** Get true if Account is enabled */
    bool isEnabled() const;
    /** Get PurpleAccount */
    PurpleAccount* getPurpleAccount() const { return m_account; }
    /** Get PurpleConnection */
    PurpleConnection* getPurpleConnection() const;
    /** Get account username */
    std::string getUsername() const;
    /** Get my account display name */
    std::string getDisplayName() const;
    /** Get account password */
    std::string getPassword() const;
    /** Set new account password
     *
     * @param password  new account password
     * @param remember  remember password
     */
    void setPassword(std::string password, bool remember=true);
    /** Get account should save its password */
    bool getRememberPassword() const;
    /** Get account alias */
    std::string getAlias() const;
    /** Set account alias */
    void setAlias(std::string alias);
    /** Enable/Disable store of aliases server-side. */
    void setServerAliases(bool b);
    /** Get account has server aliases */
    bool hasServerAliases() const;
    /** Register account on server. */
    void registerAccount() const;
    /** Get account protocol options */
    Protocol::Options getOptions() const;
    /** Set account protocol options */
    void setOptions(const Protocol::Options& options);
    /** Set whether or not this account should check for mail */
    void setCheckMail(bool check);
    /** Get whether or not this account should check for mail */
    bool getCheckMail();
    /** Set path to the buddy icon to use for this account. */
    void setBuddyIcon(std::string filename);
    /** Get path to the buddy icon used for this account. */
    std::string getBuddyIcon() const;
    /** Remove buddy icon */
    void removeBuddyIcon();
    /** Set status */
    void setStatus(PurpleStatusPrimitive pri, std::string message = "");
    /** Set status */
    void setStatus(std::string id);
    /** Set status message*/
    void setStatusMessage(const std::string& message);
    /** Get status */
    PurpleStatusPrimitive getStatus() const;
    /** Get status id */
    std::string getStatusID() const;
    /** Get status name */
    std::string getStatusName() const;
    /** Get status message */
    std::string getStatusMessage() const;
    /** Get room list from this account.
     * The roomlist callbacks will be called.
     */
    void displayRoomList() const;
    /** Get account protocol */
    Protocol getProtocol() const { return m_proto; }
    /** Is account connected? */
    bool isConnected() const;
    /** Is account connecting? */
    bool isConnecting() const;
    /** Auto reconnect to this account with a delay. */
    int delayReconnect() const;
    /** Abort the auto-reconnection. */
    void removeReconnection() const;
    /** Get list of available account commands, separators is included as "----". */
    std::vector<std::string> getCommandsList() const;
    /** Call a command. */
    bool callCommand(const std::string& command) const;
    /** Get account buddies */
    std::vector<Buddy> getBuddies() const;
    /** Get account chats */
    std::vector<Chat> getChats() const;
    /** Get account groups */
    std::vector<std::string> getGroups() const;
    /** Get groups */
    std::vector<std::string> getAllGroups() const;
    /** True if group has chat or buddy online
     *
     * @param group name for check
     */
    bool isGroupOnline(std::string group) const;
    /** Get account status items*/
    std::vector<AccountStatusItem> getStatusMenuItems() const;
    /** Connect account */
    void connect();
    /** Disconnect account */
    void disconnect();
    /** Get list of denied people */
    std::vector<std::string> getDenyList() const;
    /** Deny a user. */
    bool deny(const std::string& who) const;
    /** Allow a user. */
    bool allow(const std::string& who) const;
    /** Add a buddy on this account
     *
     * @param username  user name
     * @param alias     user alias
     * @param group     group name
     * @param invite    invite message
     */
    void addBuddy(const std::string& username, const std::string& group, const std::string& invite, const std::string& alias="") const;
    /** Remove a buddy from this account
     *
     * @param buddy  Buddy's instance
     */
    void removeBuddy(Buddy buddy);
    /** Find Buddy
     *
     * @param username  Buddy username
     */
    Buddy findBuddy(const std::string& username) const;
    /** Move buddy to another group
     *
     * @param buddy     Buddy's instance
     * @param group     Group name
     */
    void moveBuddy(Buddy buddy, const std::string& group);
    /** Add group on this account
     *
     * @param group  Group name
     */
    void addGroup(const std::string& group) const;
    /** Add chat on this account
     *
     * @param parameters    Chat entries
     * @param name          Chat name
     * @param group         Group name
     * @param autojoin      Automatically join on account connected
     */
    void addChat(std::vector<ChatEntry> parameters, const std::string& name, const std::string& group, bool autojoin=false) const;
    /** Move a chat to another group
     *
     * @param chat  Chat's instance
     * @param group Group name
     */
    void moveChat(Chat chat, const std::string& group);
    /** Remove a chat from this account
     *
     * @param chat  Chat's instance
     */
    void removeChat(Chat chat) const;
    /** Find Chat
     *
     * @param name  Chat name
     */
    Chat findChat(const std::string& name) const;
    /** Get buddy status */
    PurpleStatusPrimitive getBuddyStatus(const std::string& username) const;
    /** Does this account support chats? */
    bool supportsChats() const;
    /** Does this account support sending file? */
    bool supportsSendFile() const;
    /** Does this account support get roomlist */
    bool supportRoomlist() const;
    /** Join a chat */
    bool joinChat(std::vector<ChatEntry> parameters) const;
    /** Get parameters list */
    std::vector<ChatEntry> getChatParameters() const;
};

} /* namespace PURPLE */

#endif /* ACCOUNTMM_H */
