/*
 * buddymm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "purplemm.h"
#include "buddymm.h"
#include "accountmm.h"
#include "utilsmm.h"
#include "pi18n.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()

static void show_retrieveing_info(PurpleConnection *conn, const char *name)
{
    PurpleNotifyUserInfo *info = purple_notify_user_info_new();
    purple_notify_user_info_add_pair(info, _("Information"), _("Retrieving..."));
    purple_notify_userinfo(conn, name, info, NULL, NULL);
    purple_notify_user_info_destroy(info);
}

Buddy::BuddyMenuItem::BuddyMenuItem(PurpleMenuAction *action, void *node)
: m_label(p2s(action->label)), m_cb(action->callback), m_userData(action->data), m_node(node)
{}

void Buddy::BuddyMenuItem::runCallback()
{
    typedef void (*_callback)(gpointer, gpointer);
    _callback cb = (_callback) m_cb;
    cb(m_node, m_userData);
}

Buddy::Buddy()
    : m_buddy(NULL)
{}

Buddy::Buddy(PurpleBuddy* buddy)
    : m_buddy(buddy)
{}

bool Buddy::operator==(const Buddy& buddy) const
{
    return this->isValid() && buddy.isValid() && buddy.m_buddy == this->m_buddy;
}

bool Buddy::operator!=(const Buddy& buddy) const
{
    return !isValid() || !buddy.isValid() || buddy.m_buddy != this->m_buddy;
}

std::string Buddy::getUsername() const
{
    assert(isValid());
    std::string name;
    if(m_buddy->name && *m_buddy->name)
        name = m_buddy->name;
    return name;
}

std::string Buddy::getAlias() const
{
    assert(isValid());
    std::string alias;
    if(m_buddy->alias && *m_buddy->alias)
        alias = m_buddy->alias;
    else
        alias = getServerAlias();
    return alias;
}

std::string Buddy::getServerAlias() const
{
    assert(isValid());
    std::string alias;
    if(m_buddy->server_alias && *m_buddy->server_alias)
        alias = m_buddy->server_alias;
    else
        alias = getUsername();
    return alias;
}

void Buddy::setAlias(std::string alias, bool serverSide) const
{
    assert(isValid());
    purple_blist_alias_buddy(m_buddy, alias.c_str());
    if(serverSide && getAccount().hasServerAliases())
        serv_alias_buddy(m_buddy);
}

std::string Buddy::getGroupName() const
{
    assert(isValid());
    std::string name;
    PurpleGroup* group = purple_buddy_get_group(m_buddy);
    if(group && group->name && *group->name) name = group->name;
    return name;
}

void Buddy::retrieveInfo() const
{
    assert(isValid());
    assert(m_buddy->account != NULL);
    show_retrieveing_info(purple_account_get_connection(m_buddy->account), purple_buddy_get_name(m_buddy));
    serv_get_info(purple_account_get_connection(m_buddy->account), purple_buddy_get_name(m_buddy));
}

void Buddy::sendFile()
{
    assert(isValid());
    serv_send_file(purple_account_get_connection(purple_buddy_get_account(m_buddy)),
               purple_buddy_get_name(m_buddy),
               NULL);
}

std::string Buddy::getRealName() const
{
    assert(isValid());
    const char* rn = purple_buddy_get_server_alias(m_buddy);
    if(rn && *rn)
        return p2s(rn);
    else
    {
        const char* servernick = purple_blist_node_get_string((PurpleBlistNode*)m_buddy, "servernick");
        if(servernick && *servernick)
            return p2s(servernick);
        else
            return getUsername();
    }
}

bool Buddy::isOnline() const
{
    assert(isValid());
    return PURPLE_BUDDY_IS_ONLINE(m_buddy);
}

bool Buddy::isAvailable() const
{
    assert(isValid());
    return purple_presence_is_available(purple_buddy_get_presence(m_buddy));
}

PurpleStatusPrimitive Buddy::getStatusPrimitive() const
{
    assert(isValid());
    return purple_status_type_get_primitive(purple_status_get_type(purple_presence_get_active_status(m_buddy->presence)));
}

std::string Buddy::getStatusName() const
{
    assert(isValid());
    return p2s(purple_status_type_get_name(purple_status_get_type(purple_presence_get_active_status(m_buddy->presence))));
}

std::string Buddy::getStatusMsg() const
{
    assert(isValid());
    std::string s;
    PurpleStatus* status = purple_presence_get_active_status(m_buddy->presence);
    if(!status)
        return s;
    PurplePlugin* prpl = purple_find_prpl(purple_account_get_protocol_id(m_buddy->account));
    PurplePluginProtocolInfo* prpl_info = NULL;
    if(prpl)
        prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(prpl);
    if(prpl_info && prpl_info->status_text && m_buddy->account->gc)
    {
        char* tmp = prpl_info->status_text(m_buddy);
        const char* end;
        if(tmp)
        {
            if(!g_utf8_validate(tmp, -1, &end))
            {
                char* utf8 = g_strndup(tmp, g_utf8_pointer_to_offset(tmp, end));
                g_free(tmp);
                tmp = utf8;
            }
            char* tmp2 = purple_markup_strip_html(tmp);
            s = tmp2;
            g_free(tmp2);
            g_free(tmp);
        }
    }
    return s;
}

time_t Buddy::getIdleTime() const
{
    assert(isValid());
    PurplePresence *presence = purple_buddy_get_presence(m_buddy);
    return purple_presence_get_idle_time(presence);
}

Account Buddy::getAccount() const
{
    assert(isValid());
    return Account(m_buddy->account);
}

std::vector<Buddy::BuddyMenuItem> Buddy::getMenuItems() const
{
    assert(isValid());
    std::vector<BuddyMenuItem> v;
    PurpleConnection *gc = getAccount().getPurpleConnection();
    if(!gc)
        return v;
    PurpleBlistNode *node = PURPLE_BLIST_NODE(m_buddy);
    //Get protocol menu items
    GList *l, *ll;
    PurplePluginProtocolInfo *prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
    if(prpl_info && prpl_info->blist_node_menu)
    {
        for(l = ll = prpl_info->blist_node_menu(node); l; l = l->next)
        {
            PurpleMenuAction *act = (PurpleMenuAction *) l->data;
            v.push_back(BuddyMenuItem(act, node));
        }
        g_list_free(ll);
    }
    else
        return v;
    //Get extended menu items
    for(l = ll = purple_blist_node_get_extended_menu(node); l; l = l->next)
    {
        PurpleMenuAction *act = (PurpleMenuAction *) l->data;
        v.push_back(BuddyMenuItem(act, node));
    }
    g_list_free(ll);
    return v;
}

std::string Buddy::getIconPath() const
{
    assert(isValid());
    std::string path;
    PurpleBuddyIcon* bicon = purple_buddy_icons_find(m_buddy->account, m_buddy->name);
    if(bicon)
        path = p2s(purple_buddy_icon_get_full_path(bicon));
    return path;
}

PurpleGroup* Buddy::getPurpleGroup() const
{
    assert(isValid());
    return purple_buddy_get_group(m_buddy);
}

/* STATIC */

PurpleBlistUiOps Buddy::blistOps =
{
    NULL,//new_list,
    NULL,//new_node,
    NULL,//blist_show,
    NULL,//node_update,
    Buddy::removedNode,//node_remove,
    NULL,
    NULL,
    NULL,//finch_request_add_buddy,
    NULL,//finch_request_add_chat,
    NULL,//finch_request_add_group,
    NULL,
    NULL,
    NULL,
    NULL
};

void Buddy::init()
{
    purple_blist_set_ui_ops(&blistOps);
    purple_signal_connect(purple_blist_get_handle(), "buddy-signed-on", getHandler(), PURPLE_CALLBACK(buddySignonOn), NULL);
    purple_signal_connect(purple_blist_get_handle(), "buddy-signed-off", getHandler(), PURPLE_CALLBACK(buddySignonOff), NULL);
    purple_signal_connect(purple_blist_get_handle(), "buddy-status-changed", getHandler(), PURPLE_CALLBACK(buddyStatusChanged), NULL);
    purple_signal_connect(purple_blist_get_handle(), "buddy-icon-changed", getHandler(), PURPLE_CALLBACK(buddyIconChanged), NULL);
}

void Buddy::uninit()
{
    purple_blist_set_ui_ops(NULL);
    purple_signals_disconnect_by_handle(getHandler());
}

void* Buddy::getHandler()
{
    static int handler;

    return &handler;
}

void Buddy::buddySignonOn(PurpleBuddy* buddy)
{
    Buddy b(buddy);
    if(b.isValid() && CALLBACKS) CALLBACKS->buddySignedOn(b);
}

void Buddy::buddySignonOff(PurpleBuddy* buddy)
{
    Buddy b(buddy);
    if(b.isValid() && CALLBACKS) CALLBACKS->buddySignedOff(b);
}

void Buddy::buddyStatusChanged(PurpleBuddy *buddy, PurpleStatus *old_status, PurpleStatus *status)
{
    Buddy b(buddy);
    if(b.isValid() && CALLBACKS) CALLBACKS->buddyStatusChanged(b, purple_status_type_get_primitive(purple_status_get_type(old_status)), purple_status_type_get_primitive(purple_status_get_type(status)));
}

void Buddy::buddyIconChanged(PurpleBuddy *buddy)
{
    Buddy b(buddy);
    if(b.isValid() && CALLBACKS) CALLBACKS->buddyIconChanged(b);
}

void Buddy::removedNode(PurpleBuddyList */*list*/, PurpleBlistNode *node)
{
    if(PURPLE_BLIST_NODE_IS_BUDDY(node))
    {
        Account a(((PurpleBuddy*)node)->account);
        if(a.isValid() && CALLBACKS) CALLBACKS->buddyRemoved(a, ((PurpleBuddy*)node)->name);
    }
    else if(PURPLE_BLIST_NODE_IS_CHAT(node))
    {
        Account a(((PurpleChat*)node)->account);
        if(a.isValid() && CALLBACKS) CALLBACKS->chatRemoved(a, ((PurpleChat*)node)->alias);
    }
}

} /* namespace PURPLE */
