/*
 *      callbacksmm.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CALLBACKSMM_H
#define CALLBACKSMM_H

#include "exportmm.h"
#include "accountmm.h"
#include "buddymm.h"
#include "chatmm.h"
#include "conversationmm.h"
#include "filetransfermm.h"
#include "roomlistmm.h"
#include "requestmm.h"
#include "notifymm.h"

namespace PURPLE
{

/** The application should implement these callback functions to
 * be able to receive notifications from the library.
 */
class PURPLEMM_EXPORT Callbacks
{
public:
    /** Allow your application to be notified about errors on library layer */
    virtual void showError(std::string msg) = 0;
    /** Allow your application to be notified about libpurple debug messages */
    virtual void showDebug(std::string debug) = 0;
    /** Buddy list loaded */
    virtual void buddylistLoaded() = 0;
    /** New account added */
    virtual void accountAdded(Account account) = 0;
    /** Account removed */
    virtual void accountRemoved(std::string account, std::string protocolID) = 0;
    /** Account connecting */
    virtual void accountConnecting(Account account, std::string msg) = 0;
    /** Acount connected*/
    virtual void accountConnected(Account account) = 0;
    /** Account disconnected*/
    virtual void accountDisconnected(Account account, std::string reason) = 0;
    /** Account status changed*/
    virtual void accountStatusChanged(Account account) = 0;
    /** Account signed on, so show buddy list */
    virtual void accountSignedOn(Account account) = 0;
    /** Buddy signed on */
    virtual void buddySignedOn(Buddy buddy) = 0;
    /** Buddy signed off */
    virtual void buddySignedOff(Buddy buddy) = 0;
    /** Buddy status changed */
    virtual void buddyStatusChanged(Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus) = 0;
    /** Buddy icon changed */
    virtual void buddyIconChanged(Buddy buddy) = 0;
    /** Buddy added */
    virtual void buddyAdded(Buddy buddy) = 0;
    /** Buddy removed (fired also in account disconnected) */
    virtual void buddyRemoved(Account account, std::string buddy) = 0;
    /** Chat added */
    virtual void chatAdded(Chat chat) = 0;
    /** Chat removed (fired also in account disconnected) */
    virtual void chatRemoved(Account account, std::string chat) = 0;
    /** New IM created */
    virtual void newIM(Conversation *conv) = 0;
    /** New Chat created */
    virtual void newChat(Conversation *conv) = 0;
    /** IM closed */
    virtual void closedIM(Conversation *conv) = 0;
    /** Chat closed */
    virtual void closedChat(Conversation *conv) = 0;
    /** New my message
     * @param conv          The conversation.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param action        Is message in 3rd person (/me)?
     * @param messageID     message ID, usefull for XEP-0184
     */
    virtual void newMyMessage(Conversation *conv, std::string msg, time_t time, bool action, std::string messageID) = 0;
    /** Message recived, usefull for XEP-0184
     * @param conv          The conversation.
     * @param messageID     message ID, usefull for XEP-0184
     */
    virtual void messageRecived(Conversation *conv, std::string messageID) = 0;
    /** New IM message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    virtual void newIMMessage(Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action) = 0;
    /** New Chat message
     * @param conv          The conversation.
     * @param sender        The who sent the message.
     * @param msg           The message.
     * @param time          The time the message was sent.
     * @param delayed       Is message delayed?
     * @param highlight     Is your nick on msg?
     * @param action        Is message in 3rd person (/me)?
     */
    virtual void newChatMessage(Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action) = 0;
    /** New system mesage for conversation
     *
     * @param conv      conversation
     * @param msg       message text
     * @param time      time the message was sent
     */
    virtual void newConvSystemMessage(Conversation *conv, std::string msg, time_t time) = 0;
    /** New error mesage for conversation
     *
     * @param conv      conversation
     * @param msg       error text
     * @param time      time the error was sent
     */
    virtual void newConvErrorMessage(Conversation *conv, std::string msg, time_t time) = 0;
    /** ChatBuddy joined chat */
    virtual void chatBuddyJoined(Conversation *conv, ChatBuddy buddy) = 0;
    /** ChatBuddy updated, e.g PurpleConvChatBuddyFlags changed */
    virtual void chatBuddyUpdated(Conversation *conv, std::string buddy) = 0;
    /** ChatBuddy left chat */
    virtual void chatBuddyLeft(Conversation *conv, std::string buddy) = 0;
    /** ChatBuddy renamed
     * @param conv      conversation
     * @param who       buddy which was renamed
     * @param nbuddy    new buddy established by rename
     */
    virtual void chatBuddyRenamed(Conversation *conv, std::string who, ChatBuddy nbuddy) = 0;
    /** Chat topic changed
     * @param conv      conversation
     * @param who       topic changer
     * @param topic     new topic
     */
    virtual void chatTopicChanged(Conversation *conv, std::string who, std::string topic) = 0;
    /** Buddy typing on IM
     * @param conv      conversation
     * @param typing    =true typing, =false stoped typing
     */
    virtual void buddyTyping(Conversation *conv, bool typing) = 0;
    /** New FileTransfer */
    virtual void newFileTransfer(FileTransfer *ft) = 0;
    /** FileTransfer finished */
    virtual void fileTransferFinished(FileTransfer *ft) = 0;
    /** FileTransfer started */
    virtual void fileTransferStarted(FileTransfer *ft) = 0;
    /** FileTransfer progress */
    virtual void fileTransferProgress(FileTransfer *ft, double percent) = 0;
    /** FileTransfer canceled */
    virtual void fileTransferCanceled(FileTransfer *ft) = 0;
    /** A new list was created */
    virtual void roomlistCreated(RoomList *rlist) = 0;
    /** Sets the columns of RoomList */
    virtual void roomlistColumns(RoomList *rlist, std::vector<std::string> columns) = 0;
    /** Add a room to the list
     * @param rlist     RoomList parent
     * @param fields    first item is name, other columns fields
    */
    virtual void roomlistAddRoom(RoomList *rlist, std::vector<std::string> fields) = 0;
    /** Listing RoomList finished */
    virtual void roomlistFinished(RoomList *rlist) = 0;
    /** Notification message to the user
     * @param type          notification type
     * @param title         title of the message
     * @param primary       main point of the message
     * @param secondary     secondary information
    */
    virtual void notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary) = 0;
    /** Email notification to the user
     * @param subject   subject of the email
     * @param from      from address
     * @param to        destination address
     * @param url       URL where the message can be read
    */
    virtual void notifyEmail(std::string subject, std::string from, std::string to, std::string url) = 0;
    /** Notification with formatted text
     * @param title         title of the message
     * @param primary       main point of the message
     * @param secondary     secondary information
     * @param formated      formatted text
    */
    virtual void notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated) = 0;
    /** Notification information about buddy
     * @param account   requested account
     * @param buddy     buddy which about
     * @param info      vector with UserInfo values
    */
    virtual void notifyUserinfo(Account account, std::string buddy, std::vector<Notify::UserInfo> info) = 0;
    /** Opens a URI or somehow presents it to the user
     * @param uri    The URI to display or go to.
    */
    virtual void notifyURI(std::string uri) = 0;
    /** Play sound for given event */
    virtual void playSound(PurpleSoundEventID event) = 0;
    /** New InputRequest */
    virtual void newInputRequest(InputRequest *request) = 0;
    /** New ChoiceRequest */
    virtual void newChoiceRequest(ChoiceRequest *request) = 0;
    /** New ActionRequest */
    virtual void newActionRequest(ActionRequest *request) = 0;
    /** New FileRequest */
    virtual void newFileRequest(FileRequest *request) = 0;
    /** New FieldsRequest */
    virtual void newFieldsRequest(FieldsRequest *request) = 0;
    /** Close given request (ex. dialog) */
    virtual void closeRequest(BasicRequest *request) = 0;
};

} /* namespace PURPLE */

#endif // CALLBACKSMM_H
