/*
 * chatmm.cpp
 *
 * Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "chatmm.h"
#include "accountmm.h"
#include "utilsmm.h"

namespace PURPLE
{
Chat::Chat()
    : m_chat(NULL), m_autojoin(false)
{}

Chat::Chat(PurpleChat* chat)
    : m_chat(chat)
{
    m_autojoin = purple_blist_node_get_bool((PurpleBlistNode*)m_chat, "purplemm-autojoin");
}

bool Chat::operator==(const Chat& chat) const
{
    return this->isValid() && chat.isValid() && chat.m_chat == this->m_chat;
}

bool Chat::operator!=(const Chat& chat) const
{
    return !isValid() || !chat.isValid() || chat.m_chat != this->m_chat;
}

std::string Chat::getAlias() const
{
    assert(isValid());
    return std::string(m_chat->alias);
}

void Chat::setAlias(std::string alias)
{
    assert(isValid());
    purple_blist_alias_chat(m_chat, alias.c_str());
}

void Chat::setAutojoin(bool autojoin)
{
    assert(isValid());
    m_autojoin = autojoin;
    purple_blist_node_set_bool((PurpleBlistNode*)m_chat, "purplemm-autojoin", m_autojoin);
}

std::string Chat::getGroupName() const
{
    assert(isValid());
    std::string name;
    PurpleGroup* group = purple_chat_get_group(m_chat);
    if(group && group->name && *group->name) name = group->name;
    return name;
}

std::vector<Account::ChatEntry> Chat::getChatParameters() const
{
    assert(isValid());
    std::vector<Account::ChatEntry> v;
    if(getAccount().isConnected())
    {
        PurplePluginProtocolInfo *prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(purple_find_prpl(getAccount().getProtocol().getPurpleID().c_str()));
        PurpleConnection *gc = getAccount().getPurpleConnection();
        GList *list = NULL, *tmp;
        if(prpl_info->chat_info != NULL)
            list = prpl_info->chat_info(gc);
        for(tmp = list; tmp; tmp = tmp->next)
        {
            struct proto_chat_entry *pce = static_cast<struct proto_chat_entry*>(tmp->data);
            char* value = (char*)g_hash_table_lookup(m_chat->components, pce->identifier);
            Account::ChatEntry entry(p2s(pce->label), p2s(pce->identifier), value?value:"", pce->required,
                    pce->is_int, pce->min, pce->max, pce->secret);
            v.push_back(entry);
            g_free(pce);
        }
        g_list_free(list);
    }
    return v;
}

void Chat::setChatParameters(std::vector<Account::ChatEntry> parameters)
{
    assert(isValid());
    GHashTable *hash = purple_chat_get_components(m_chat);
    /* Parse parameters */
    FOREACH(std::vector<Account::ChatEntry>, parameters, it)
    {
        g_hash_table_replace(hash, g_strdup((*it).getIdentifier().c_str()),
                g_strdup((*it).getValue().c_str()));
    }
}

Account Chat::getAccount() const
{
    assert(isValid());
    return Account(m_chat->account);
}

std::string Chat::getChatName() const
{
    std::string chat = "";
    PurpleAccount *account = purple_chat_get_account(m_chat);
    PurplePluginProtocolInfo *prpl_info = NULL;
    prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(purple_find_prpl(purple_account_get_protocol_id(account)));
    if(prpl_info && prpl_info->get_chat_name)
    {
        char *name = prpl_info->get_chat_name(purple_chat_get_components(m_chat));
        chat = p2s(name);
        g_free(name);
    }
    return chat;
}

PurpleGroup* Chat::getPurpleGroup() const
{
    assert(isValid());
    return purple_chat_get_group(m_chat);
}

void Chat::join() const
{
    assert(isValid());
    serv_join_chat(purple_account_get_connection(getAccount().getPurpleAccount()), m_chat->components);
}

} /* namespace PURPLE */
