/*
 * conversationmm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include <sstream>
#include <algorithm>
#include "conversationmm.h"
#include "purplemm.h"
#include "buddymm.h"
#include "utilsmm.h"
#include "pi18n.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()
#define CONVERSATION(conv) ((Conversation*)(conv)->ui_data)

static void show_retrieveing_info(PurpleConnection *conn, const char *name)
{
    PurpleNotifyUserInfo *info = purple_notify_user_info_new();
    purple_notify_user_info_add_pair(info, _("Information"), _("Retrieving..."));
    purple_notify_userinfo(conn, name, info, NULL, NULL);
    purple_notify_user_info_destroy(info);
}

static ChatBuddy createChatBuddy(PURPLE::Conversation *conv, PurpleConvChatBuddy *buddy)
{
    std::string name = "";
    std::string alias = "";
    std::string realName = "";
    bool isBuddy = false;
    bool isMe = false;
    PurpleConvChatBuddyFlags role = PURPLE_CBFLAGS_NONE;
    PurpleStatusPrimitive status = PURPLE_STATUS_AVAILABLE;
    if(conv->isValid() && buddy)
    {
        if(buddy->name && *buddy->name)
            name = buddy->name;
        alias = buddy->alias && *buddy->alias ? buddy->alias : name;
        PurplePluginProtocolInfo* prpl = conv->getAccount().getProtocol().getPurpleProtocol();
        if(prpl->get_cb_real_name)
        {
            gchar* tmp = prpl->get_cb_real_name(conv->getAccount().getPurpleConnection(), conv->getPurpleChat()->id, buddy->name);
            realName = tmp;
            g_free(tmp);
        }
        else
        {
            realName = name;
        }
        isBuddy = buddy->buddy;
        isMe = name == purple_conv_chat_get_nick(conv->getPurpleChat());
        role = buddy->flags;
#if PURPLE_VERSION_CHECK(2,8,0)
        if(role & PURPLE_CBFLAGS_AWAY)
            role = (PurpleConvChatBuddyFlags)(role & ~ PURPLE_CBFLAGS_AWAY);
#endif
        if(role & PURPLE_CBFLAGS_TYPING)
            role = (PurpleConvChatBuddyFlags)(role & ~ PURPLE_CBFLAGS_TYPING);
#if PURPLE_VERSION_CHECK(2,8,0)
        PurpleConvChatBuddyFlags flags = buddy->flags;
        if(flags & PURPLE_CBFLAGS_AWAY)
            status = PURPLE_STATUS_AWAY;
#endif
    }
    return ChatBuddy(name, alias, realName, isBuddy, isMe, role, status);
}

ChatBuddy::ChatBuddy(std::string name, std::string alias, std::string realName, bool buddy, bool me,
                     PurpleConvChatBuddyFlags chatRole, PurpleStatusPrimitive chatStatus)
    : m_name(name), m_alias(alias), m_realName(realName), m_buddy(buddy), m_me(me), m_chatRole(chatRole),
      m_chatStatus(chatStatus)
{}

ChatBuddy::ChatBuddy()
    : m_name(""), m_alias(""), m_realName(""), m_buddy(false), m_me(false), m_chatRole(PURPLE_CBFLAGS_NONE),
      m_chatStatus(PURPLE_STATUS_AVAILABLE)
{}

std::string ChatBuddy::getName() const
{
    return m_name;
}

std::string ChatBuddy::getAlias() const
{
    return m_alias;
}

std::string ChatBuddy::getRealName() const
{
    return m_realName;
}

bool ChatBuddy::isBuddy() const
{
    return m_buddy;
}

PurpleConvChatBuddyFlags ChatBuddy::getChatRole() const
{
    return m_chatRole;
}

PurpleStatusPrimitive ChatBuddy::getChatStatus() const
{
    return m_chatStatus;
}

bool ChatBuddy::isMe() const
{
    return m_me;
}

bool ChatBuddy::operator==(const ChatBuddy& cbuddy) const
{
    return this->m_name == cbuddy.m_name;
}

bool ChatBuddy::operator!=(const ChatBuddy& cbuddy) const
{
    return this->m_name != cbuddy.m_name;
}

bool ChatBuddy::operator<(const ChatBuddy& cbuddy) const
{
    return this->m_name < cbuddy.m_name;
}

Conversation::Conversation()
    : m_conv(NULL)
{}

Conversation::Conversation(PurpleConversation* conv)
    : m_conv(conv)
{}

bool Conversation::operator==(const Conversation& conv) const
{
    return conv.m_conv == this->m_conv;
}

bool Conversation::operator!=(const Conversation& conv) const
{
    return conv.m_conv != this->m_conv;
}

Account Conversation::getAccount() const
{
    assert(isValid());
    if(m_conv->account) return Purple::instance()->findAccount(m_conv->account->username, m_conv->account->protocol_id);
    return Account();
}

std::string Conversation::getMyNick() const
{
    assert(isValid());
    if(getType()==PURPLE_CONV_TYPE_CHAT) return p2s(purple_conv_chat_get_nick(getPurpleChat()));
    else
    {
        std::string ret = getAccount().getAlias();
        if(ret.empty()) return getAccount().getUsername();
        else return ret;
    }
}

std::string Conversation::getName() const
{
    assert(isValid());
    std::string name;
    if(m_conv->name && *m_conv->name)
        name = m_conv->name;
    return name;
}

std::string Conversation::getTitle() const
{
    assert(isValid());
    std::string title;
    if(m_conv->title && *m_conv->title)
        title = m_conv->title;
    return title;
}

std::string Conversation::getChanTopic() const
{
    assert(isValid());
    return p2s(purple_conv_chat_get_topic(getPurpleChat()));
}

PurpleConversationType Conversation::getType() const
{
    assert(isValid());
    return purple_conversation_get_type(m_conv);
}

void Conversation::leave()
{
    assert(isValid());
    PurpleConversation* c = m_conv;
    m_conv = NULL;
    c->ui_data = NULL;
    purple_conversation_destroy(c);
}

void Conversation::invite(const std::string& buddy, const std::string& message)
{
    assert(isValid());
    serv_chat_invite(purple_conversation_get_gc(m_conv),
                    purple_conv_chat_get_id(PURPLE_CONV_CHAT(m_conv)),
                    message.c_str(), buddy.c_str());
}

bool Conversation::setTopic(const std::string& topic)
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    PurplePluginProtocolInfo *prpl_info = NULL;
    PurpleConnection *gc = purple_conversation_get_gc(m_conv);
    if(!gc || !(prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)))
        return false;
    if(prpl_info->set_chat_topic == NULL)
        return false;
    char *utf8 = purple_utf8_try_convert(topic.c_str());
    prpl_info->set_chat_topic(gc, purple_conv_chat_get_id(getPurpleChat()), utf8);
    g_free(utf8);
    return true;
}

bool Conversation::hasTopic() const
{
    assert(isValid());
    if(getType() != PURPLE_CONV_TYPE_CHAT)
        return false;
    PurplePluginProtocolInfo *prpl_info = NULL;
    PurpleConnection *gc = purple_conversation_get_gc(m_conv);
    if(!gc || !(prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl)))
        return false;
    if(prpl_info->set_chat_topic)
        return true;
    return false;
}

PurpleConvChat* Conversation::getPurpleChat() const
{
    assert(isValid());
    return PURPLE_CONV_CHAT(m_conv);
}

PurpleConvIm* Conversation::getPurpleIm() const
{
    assert(isValid());
    return PURPLE_CONV_IM(m_conv);
}

int Conversation::sendCommand(const std::string& cmd) const
{
    assert(isValid());
    char* error = NULL;
    PurpleCmdStatus ret = purple_cmd_do_command(m_conv, cmd.c_str(), cmd.c_str(), &error);
    return ret;
}

void Conversation::sendMessage(std::string text)
{
    assert(isValid());
    char *utf8 = purple_utf8_try_convert(text.c_str());
    switch(getType())
    {
        case PURPLE_CONV_TYPE_IM:
            purple_conv_im_send_with_flags(getPurpleIm(), utf8, PURPLE_MESSAGE_SEND);
            break;
        case PURPLE_CONV_TYPE_CHAT:
            purple_conv_chat_send(getPurpleChat(), utf8);
            break;
        default:
            break;
    }
    g_free(utf8);
    purple_idle_touch();
}

void Conversation::sendActionMessage(std::string text)
{
    assert(isValid());
    std::string command;
    if(hasMe()) command = "me "+text;
    else command = "/me "+text;
    char* error = NULL;
    PurpleCmdStatus ret = purple_cmd_do_command(m_conv, command.c_str(), command.c_str(), &error);
    if(ret == PURPLE_CMD_STATUS_OK)
        return;
    else
    {
        //terrible but needed
        if(!getAccount().getProtocol().getPurpleID().compare("prpl-jabber")) sendMessage(command);
        else sendMessage(text);
    }
}

void Conversation::sendTyping(bool typing)
{
    assert(isValid());
    int timeout;
    if(typing)
    {
        timeout = serv_send_typing(purple_conversation_get_gc(m_conv),
                               purple_conversation_get_name(m_conv),
                               PURPLE_TYPING);
        purple_conv_im_set_type_again(getPurpleIm(), timeout);
    }
    else
    {
        purple_conv_im_stop_send_typed_timeout(getPurpleIm());
        timeout = serv_send_typing(purple_conversation_get_gc(m_conv),
                               purple_conversation_get_name(m_conv),
                               PURPLE_NOT_TYPING);
    }
}

std::vector<std::string> Conversation::getCommandsList() const
{
    assert(isValid());
    std::vector<std::string> cmds;
    GList *list = purple_cmd_list(m_conv);
    GList *l;
    for(l = list; l != NULL; l = l->next)
    {
        cmds.push_back(p2s((const char *)l->data));
    }
    g_list_free(list);
    return cmds;
}

bool Conversation::hasMe() const
{
    assert(isValid());
    bool ret = false;
    GList *list = purple_cmd_list(m_conv);
    GList *l;
    for(l = list; l != NULL; l = l->next)
    {
        if(!g_ascii_strcasecmp((const char *)l->data, "me"))
        {
            ret = true;
            break;
        }
    }
    g_list_free(list);
    return ret;
}

std::string Conversation::getCommandHelp(std::string command) const
{
    assert(isValid());
    std::ostringstream help;
    GList *list, *l;
    if(!command.empty())
    {
        list = purple_cmd_help(m_conv, command.c_str());
        for(l = list; l; l = l->next)
        {
            if(l->next)
                help << p2s((const char*)l->data) << "\n";
            else
                help << p2s((const char*)l->data);
        }
    }
    else
    {
        list = purple_cmd_list(m_conv);
        for(l = list; l; l = l->next)
        {
            if(l->next)
                help << p2s((const char*)l->data) << ", ";
            else
                help << p2s((const char*)l->data) << ".";
        }
        g_list_free(list);
    }
    return help.str();
}

void Conversation::ignore(std::string name)
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    purple_conv_chat_ignore(getPurpleChat(), name.c_str());
}

void Conversation::unignore(std::string name)
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    purple_conv_chat_unignore(getPurpleChat(), name.c_str());
}

std::vector<std::string> Conversation::getIgnored() const
{
    assert(isValid());
    std::vector<std::string> v;
    GList *ignored;
    for(ignored = purple_conv_chat_get_ignored(getPurpleChat()); ignored != NULL; ignored = ignored->next)
    {
        v.push_back(p2s((const char *)ignored->data));
    }
    return v;
}

bool sortChatBuddies(ChatBuddy i, ChatBuddy j)
{
    return g_ascii_strcasecmp(i.getName().c_str(), j.getName().c_str())<0;
}

std::vector<ChatBuddy> Conversation::getChatBuddies()
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    std::vector<ChatBuddy> v;
    GList *l = purple_conv_chat_get_users(getPurpleChat());
    for(; l != NULL; l = l->next)
    {
        ChatBuddy cb = createChatBuddy(this, (PurpleConvChatBuddy *)l->data);
        if(cb.isValid()) v.push_back(cb);
    }
    std::sort(v.begin(), v.end(), sortChatBuddies);
    return v;
}

ChatBuddy Conversation::findChatBuddy(std::string buddy)
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    if(buddy.empty()) return ChatBuddy();
    PurpleConvChatBuddy *cbuddy = purple_conv_chat_cb_find(getPurpleChat(), buddy.c_str());
    if(!cbuddy) return ChatBuddy();
    return createChatBuddy(this, cbuddy);
}

bool Conversation::hasChatBuddy(std::string buddy)
{
    assert(isValid());
    assert(getType() == PURPLE_CONV_TYPE_CHAT);
    if(buddy.empty()) return false;
    return purple_conv_chat_find_user(getPurpleChat(), buddy.c_str());
}

PurpleStatusPrimitive Conversation::getIMBuddyStatus() const
{
    assert(isValid());
    PurpleBuddy *buddy = purple_find_buddy(getAccount().getPurpleAccount(), getName().c_str());
    if(buddy)
    {
        return purple_status_type_get_primitive(purple_status_get_type(purple_presence_get_active_status(buddy->presence)));
    }
    return PURPLE_STATUS_OFFLINE;
}

void Conversation::retrieveChatBuddyInfo(std::string buddy)
{
    if(isValid())
    {
        PurpleConnection *conn = purple_account_get_connection(getAccount().getPurpleAccount());
        int id = purple_conv_chat_get_id(getPurpleChat());
        if(id < 0)
        {
            show_retrieveing_info(conn, buddy.c_str());
            serv_get_info(conn, buddy.c_str());
        }
        char *who = NULL;
        PurplePluginProtocolInfo *prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(conn->prpl);
        if(prpl_info != NULL && prpl_info->get_cb_real_name)
            who = prpl_info->get_cb_real_name(conn, id, buddy.c_str());
        if(prpl_info == NULL || prpl_info->get_cb_info == NULL)
        {
            show_retrieveing_info(conn, who ? who : buddy.c_str());
            serv_get_info(conn, who ? who : buddy.c_str());
            g_free(who);
            return;
        }
        show_retrieveing_info(conn, who ? who : buddy.c_str());
        prpl_info->get_cb_info(conn, id, buddy.c_str());
        g_free(who);
    }
}

bool Conversation::isChatBuddyIgnored(std::string buddy) const
{
    if(isValid())
        return purple_conv_chat_is_user_ignored(getPurpleChat(), buddy.c_str());
    return false;
}

void Conversation::createIM(const ChatBuddy &cbuddy)
{
    if(cbuddy.isValid())
    {
        purple_conversation_new(PURPLE_CONV_TYPE_IM, getAccount().getPurpleAccount(), cbuddy.getRealName().c_str());
    }
}

void Conversation::addMessageID(std::string messageID)
{
    m_messageIDs.push_back(messageID);
}

std::string Conversation::getLastMessageID() const
{
    if(m_messageIDs.size()) return m_messageIDs.back();
    else return "";
}

/* STATIC */

PurpleConversationUiOps Conversation::convOps =
{
    NULL,
    NULL,
    NULL,
    NULL,
    Conversation::writeConv,
    NULL,
    Conversation::renameBuddy,
    NULL,
    Conversation::updateBuddy,
    Conversation::present,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

void Conversation::init()
{
    purple_conversations_set_ui_ops(&convOps);
    purple_signal_connect(purple_conversations_get_handle(), "conversation-created",
                          getHandler(), PURPLE_CALLBACK(create), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "deleting-conversation",
                          getHandler(), PURPLE_CALLBACK(destroy), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "chat-buddy-joined",
                          getHandler(), PURPLE_CALLBACK(addBuddy), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "chat-buddy-leaving",
                          getHandler(), PURPLE_CALLBACK(removeBuddy), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "chat-topic-changed",
                          getHandler(), PURPLE_CALLBACK(topicChanged), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "buddy-typing",
                          getHandler(), PURPLE_CALLBACK(buddyTyping), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "buddy-typing-stopped",
                          getHandler(), PURPLE_CALLBACK(buddyTyping), NULL);
    purple_signal_connect(purple_conversations_get_handle(), "chat-left",
                          getHandler(), PURPLE_CALLBACK(chatLeft), NULL);
    //support fox xmpp XEP-0184
    PurplePlugin *jabber = purple_find_prpl("prpl-jabber");
    if(jabber)
    {
        gboolean ok;
        purple_plugin_ipc_call(jabber, "add_feature", &ok, "urn:xmpp:receipts");
        if(CALLBACKS)
        {
            if(ok) CALLBACKS->showDebug("urn:xmpp:receipts feature added");
            else CALLBACKS->showDebug("urn:xmpp:receipts feature not added (will work anyway with most clients)");
        }
        purple_signal_connect(jabber, "jabber-receiving-xmlnode", getHandler(),
                              PURPLE_CALLBACK(xmlnodeReceivedCb), NULL);
        purple_signal_connect_priority(jabber, "jabber-sending-xmlnode", getHandler(),
                                       PURPLE_CALLBACK(xmlnodeSendingCb), NULL, -101);
    }
}

void Conversation::uninit()
{
    purple_conversations_set_ui_ops(NULL);
    purple_signals_disconnect_by_handle(getHandler());
}

void* Conversation::getHandler()
{
    static int handler;
    return &handler;
}

void Conversation::create(PurpleConversation *c)
{
    Conversation *conv = CONVERSATION(c);
    if(conv == NULL)
    {
        conv = new Conversation(c);
        c->ui_data = conv;
    }
    if(conv->isValid())
    {
        switch(conv->getType())
        {
            case PURPLE_CONV_TYPE_IM:
                if(CALLBACKS) CALLBACKS->newIM(conv);
                break;
            case PURPLE_CONV_TYPE_CHAT:
                if(CALLBACKS) CALLBACKS->newChat(conv);
                break;
            default:
                break;
        }
    }
}

void Conversation::destroy(PurpleConversation* c)
{
    Conversation *conv = new Conversation(c);
    if(conv && conv->isValid())
    {
        switch(conv->getType())
        {
            case PURPLE_CONV_TYPE_IM:
                if(CALLBACKS) CALLBACKS->closedIM(conv);
                break;
            case PURPLE_CONV_TYPE_CHAT:
                if(CALLBACKS) CALLBACKS->closedChat(conv);
                break;
            default:
                break;
        }
    }
}

void Conversation::present(PurpleConversation */*c*/)
{
}

void Conversation::writeConv(PurpleConversation *c, const char *who, const char* alias,
        const char *message, PurpleMessageFlags flags, time_t mtime)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid())
    {
        if(flags & PURPLE_MESSAGE_INVISIBLE)
        {
            return;
        }
        if(flags & PURPLE_MESSAGE_ERROR)
        {
            if(CALLBACKS) CALLBACKS->newConvErrorMessage(conv, p2s(message), mtime);
            return;
        }
        if(flags & PURPLE_MESSAGE_SYSTEM)
        {
            if(CALLBACKS) CALLBACKS->newConvSystemMessage(conv, p2s(message), mtime);
            return;
        }
        time_t tnow = now();
        bool delayed = localtime(&tnow)->tm_yday != localtime(&mtime)->tm_yday;/*flags & PURPLE_MESSAGE_DELAYED;*/
        bool highlight = flags & PURPLE_MESSAGE_NICK;
        bool action = purple_message_meify((char*)message, -1);
        if(flags & PURPLE_MESSAGE_SEND)
        {
            if(CALLBACKS && !delayed)
            {
                CALLBACKS->newMyMessage(conv, p2s(message), mtime, action, conv->getLastMessageID());
                return;
            }
        }
        if(c->type==PURPLE_CONV_TYPE_IM)
        {
            if(CALLBACKS) CALLBACKS->newIMMessage(conv, p2s(alias), p2s(message), mtime, delayed, highlight, action);
        }
        else if(c->type==PURPLE_CONV_TYPE_CHAT)
        {
            if(CALLBACKS) CALLBACKS->newChatMessage(conv, p2s(alias), p2s(message), mtime, delayed, highlight, action);
        }
        else
        {
            std::ostringstream oss;
            oss << "DEBUG [conv] (" << conv->getName() << ") " << purple_utf8_strftime("(%H:%M:%S)", localtime(&mtime)) << " " << who << " " << message;
            if(CALLBACKS) CALLBACKS->showDebug(oss.str());
        }
    }
}

void Conversation::addBuddy(PurpleConversation *c, const char *name, PurpleConvChatBuddyFlags /*flags*/, gboolean /*new_arrival*/)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid())
    {
        ChatBuddy cb = createChatBuddy(conv, purple_conv_chat_cb_find(conv->getPurpleChat(), name));
        if(cb.isValid() && CALLBACKS) CALLBACKS->chatBuddyJoined(conv, cb);
    }
}

void Conversation::updateBuddy(PurpleConversation* c, const char* user)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid())
    {
        PurpleConvChatBuddy *cb = purple_conv_chat_cb_find(conv->getPurpleChat(), user);
        if(!cb)
        {
            std::ostringstream oss;
            oss << "ERROR [conv] Update " << user << " which is an unknown chat buddy";
            if(CALLBACKS) CALLBACKS->showDebug(oss.str());
            return;
        }
        if(CALLBACKS) CALLBACKS->chatBuddyUpdated(conv, p2s(user));
    }
}

void Conversation::removeBuddy(PurpleConversation* c, const char *name, const char */*reason*/)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid())
    {
        if(CALLBACKS) CALLBACKS->chatBuddyLeft(conv, p2s(name));
    }
}

void Conversation::renameBuddy(PurpleConversation *c, const char *old,
                    const char *new_n, const char *new_a)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid())
    {
        if(!conv->hasChatBuddy(p2s(old)))
        {
            std::ostringstream oss;
            oss << "ERROR [conv] Rename from " << old << " to " << new_n << " (" << new_a << ") which is an unknown chat buddy";
            if(CALLBACKS) CALLBACKS->showDebug(oss.str());
            return;
        }
        ChatBuddy nbuddy = createChatBuddy(conv, purple_conv_chat_cb_find(conv->getPurpleChat(), new_n));
        if(!nbuddy.isValid())
        {
            std::ostringstream oss;
            oss << "New chat buddy " << new_n << " (" << new_a << ") unknown";
            if(CALLBACKS) CALLBACKS->showDebug(oss.str());
            return;
        }
        if(CALLBACKS) CALLBACKS->chatBuddyRenamed(conv, p2s(old), nbuddy);
    }
}

void Conversation::chatLeft(PurpleConversation *conv)
{
    if(conv->ui_data) //we send /part
    {
        //leave chat therefore call closedChat
        conv->ui_data = NULL;
        purple_conversation_destroy(conv);
    }
}

void Conversation::xmlnodeReceivedCb(PurpleConnection *gc, xmlnode **packet, gpointer /*null*/)
{
    if(*packet != NULL)
    {
        if(strcmp((*packet)->name, "message") == 0)
        {
            xmlnode* nodeRequest = xmlnode_get_child (*packet, "request");
            const char* strFrom	= xmlnode_get_attrib(*packet , "from");
            //Answer to an request and verify namespace
            if(nodeRequest)
            {
                const char* strId = xmlnode_get_attrib(*packet , "id");
                const char* strNS = xmlnode_get_namespace(nodeRequest);
                if(strcmp(strNS, "urn:xmpp:receipts") == 0)
                {
                    xmlnode *message = xmlnode_new("message");
                    xmlnode_set_attrib(message, "to", strFrom);
                    xmlnode *received = xmlnode_new_child(message, "received");
                    xmlnode_set_namespace(received, "urn:xmpp:receipts");
                    xmlnode_set_attrib(received, "id", strId);
                    purple_signal_emit(purple_connection_get_prpl(gc), "jabber-sending-xmlnode", gc, &message);
                    if(message != NULL)
                        xmlnode_free(message);
                }
            }
            //Find incoming receipt and call the display-function
            xmlnode* nodeReceived = xmlnode_get_child (*packet, "received");
            if(nodeReceived)
            {
                const char* strNS = xmlnode_get_namespace(nodeReceived);
                if(strcmp(strNS, "urn:xmpp:receipts") == 0)
                {
                    PurpleAccount *acct = purple_connection_get_account (gc);
                    if(!acct) return;
                    const char* strTo = xmlnode_get_attrib(*packet , "from");
                    PurpleConversation *c = purple_find_conversation_with_account(PURPLE_CONV_TYPE_ANY, strTo, acct);
                    if(!c) return;
                    Conversation *conv = CONVERSATION(c);
                    if(conv && conv->isValid())
                    {
                        if(CALLBACKS) CALLBACKS->messageRecived(conv, p2s(xmlnode_get_attrib(nodeReceived , "id")));
                    }
                }
            }
        }
    }
}

void Conversation::xmlnodeSendingCb(PurpleConnection *gc, xmlnode **packet, gpointer /*null*/)
{
    if(*packet != NULL)
    {
        if((*packet)->name)
        {
            if(strcmp((*packet)->name, "message") == 0)
            {
                xmlnode *nodeBody = xmlnode_get_child (*packet, "body");
                //Only for messages containing text (no status messages etc)
                if(nodeBody)
                {
                    xmlnode *child = xmlnode_new_child (*packet, "request");
                    xmlnode_set_attrib (child, "xmlns", "urn:xmpp:receipts");
                    PurpleAccount *acct = purple_connection_get_account (gc);
                    if(!acct) return;
                    const char* strTo = xmlnode_get_attrib(*packet , "to");
                    PurpleConversation *c = purple_find_conversation_with_account(PURPLE_CONV_TYPE_ANY, strTo, acct);
                    if(!c) return;
                    Conversation *conv = CONVERSATION(c);
                    if(conv && conv->isValid())
                    {
                        conv->addMessageID(p2s(xmlnode_get_attrib(*packet , "id")));
                    }
                }
            }
        }
    }
}

void Conversation::topicChanged(PurpleConversation* c, const char* who, const char* topic)
{
    Conversation *conv = CONVERSATION(c);
    if(conv && conv->isValid() && CALLBACKS) CALLBACKS->chatTopicChanged(conv, p2s(who), p2s(topic));
}

void Conversation::buddyTyping(PurpleAccount* account, const char* who, gpointer /*null*/)
{
    PurpleConversation *c = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM, who, account);
    if(!c)
        return;
    Conversation *conv = CONVERSATION(c);
    if(!conv || !conv->isValid() || who == NULL)
        return;
    PurpleConvIm *im = conv->getPurpleIm();
    switch(purple_conv_im_get_typing_state(im))
    {
        case PURPLE_TYPING:
            if(CALLBACKS) CALLBACKS->buddyTyping(conv, true);
            break;
        default:
            if(CALLBACKS) CALLBACKS->buddyTyping(conv, false);
    }
}

} /* namespace PURPLE */
