/*
 * conversationmm.h
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CONVERSATIONMM_H
#define CONVERSATIONMM_H

#include <purple.h>
#include <string>
#include <vector>
#include <map>
#include "exportmm.h"

namespace PURPLE
{

class Account;
class Buddy;
class ChatBuddy;

/** This class represents an conversation - IM or chat. */
class PURPLEMM_EXPORT Conversation
{
    PurpleConversation* m_conv;

    static void* getHandler();
    static PurpleConversationUiOps convOps;
    static void create(PurpleConversation*);
    static void destroy(PurpleConversation*);
    static void present(PurpleConversation*);
    static void writeConv(PurpleConversation *conv, const char *who, const char* alias,
            const char *message, PurpleMessageFlags flags,
            time_t mtime);
    static void addBuddy(PurpleConversation *conv, const char *name,
                         PurpleConvChatBuddyFlags flags,
                         gboolean new_arrival);
    static void updateBuddy(PurpleConversation* c, const char* user);
    static void removeBuddy(PurpleConversation* conv, const char *name, const char *reason);
    static void topicChanged(PurpleConversation* conv, const char* who, const char* topic);
    static void buddyTyping(PurpleAccount* account, const char* who, gpointer null);
    static void renameBuddy(PurpleConversation *conv, const char *old,
                         const char *new_n, const char *new_a);
    static void chatLeft(PurpleConversation* conv);
    static void xmlnodeReceivedCb(PurpleConnection *gc, xmlnode **packet, gpointer null);
    static void xmlnodeSendingCb(PurpleConnection *gc, xmlnode **packet, gpointer null);
public:
    /** Initialization of libpurple conversation' stuffs. */
    static void init();
    /** Uninitialization of libpurple conversation' stuffs. */
    static void uninit();

    /** Empty constructor */
    Conversation();
    /** Create a Conversation instance
     *
     * @param conv  libpurple conversation instance
     */
    Conversation(PurpleConversation* conv);

    /** Comparaisons */
    bool operator==(const Conversation& conv) const;
    /** Comparaisons */
    bool operator!=(const Conversation& conv) const;
    /** Is Conversation valid? */
    bool isValid() const { return m_conv != NULL; }
    /** Get PurpleConversation */
    PurpleConversation* getPurpleConversation() const { return m_conv; }
    /** Get PurpleConvChat */
    PurpleConvChat* getPurpleChat() const;
    /** Get PurpleConvIm */
    PurpleConvIm* getPurpleIm() const;
    /** Get my nick in Conversation */
    std::string getMyNick() const;
    /** Get Conversation name*/
    std::string getName() const;
    /** Get Conversation window title */
    std::string getTitle() const;
    /** Get Conversation topic */
    std::string getChanTopic() const;
    /** Get Conversation Account */
    Account getAccount() const;
    /** Get PurpleConversationType */
    PurpleConversationType getType() const;
    /** Leave the conversation. */
    void leave();
    /** Send a command to this conversation.
     *
     * @param cmd  cmd to execute (without client cmd prefix)
     *             e.g client cmd = /me test, cmd = me test
     */
    int sendCommand(const std::string& cmd) const;
    /** Send a message in this conversation.
     *
     * @param text  text to send.
     */
    void sendMessage(std::string text);
    /** Send a action (in 3rd person) message in this conversation.
     *
     * @param text  text to send.
     */
    void sendActionMessage(std::string text);
    /** Send typing notification */
    void sendTyping(bool typing);
    /** Invite a buddy into this conversation.
     *
     * This is available only for the CHAT conversations.
     *
     * @param buddy  buddy's name
     * @param message  message to send with invitation.
     */
    void invite(const std::string& buddy, const std::string& message);
    /** Set topic of conversation. */
    bool setTopic(const std::string& topic);
    /** Has topic of conversation, e.g you can view or change topic */
    bool hasTopic() const;
    /** Get list of available conversation commands. */
    std::vector<std::string> getCommandsList() const;
    /**
     *
     * @param command   command which need help; can be empty
     * @return          help for command, if command empty list of all available commands
     */
    std::string getCommandHelp(std::string command) const;
    /** Ignores a user in a chat room.
      *
      * @param name   name of the user.
     */
    void ignore(std::string name);
    /** Unignores a user in a chat room.
      *
      * @param name   name of the user.
     */
    void unignore(std::string name);
    /** Get list of ignored users. */
    std::vector<std::string> getIgnored() const;
    /** Get users in a chat room */
    std::vector<ChatBuddy> getChatBuddies();
    /** Find chatbuddy */
    ChatBuddy findChatBuddy(std::string buddy);
    /** Has chatbuddy */
    bool hasChatBuddy(std::string buddy);
    /** Get IM buddy status */
    PurpleStatusPrimitive getIMBuddyStatus() const;
    /** Ask server to send server info.
     *
     * It'll be showed with notifyUserinfo callback.
     */
    void retrieveChatBuddyInfo(std::string buddy);
    /** Is chatbuddy ignored */
    bool isChatBuddyIgnored(std::string buddy) const;
    /** Create a IM Conversation
     *
     * @param cbuddy  ChatBuddy instance.
     */
    void createIM(const ChatBuddy& cbuddy);
    /** Add messageID, usefull for XEP-0184
     *
     * @param messageID
     */
    void addMessageID(std::string messageID);
    /** Get last messageID
     */
    std::string getLastMessageID() const;
private:
    bool hasMe() const;

    std::vector<std::string> m_messageIDs;
};

/** This class represents an chat buddy. */
class PURPLEMM_EXPORT ChatBuddy
{
public:
    /** Empty constructor */
    ChatBuddy();
    /** Create a ChatBuddy instance
     *
     * @param name
     * @param alias
     * @param realName  Chatbuddy real name
     * @param buddy  chat participant is on the buddy list
     * @param me  Is ChatBuddy I?
     * @param chatRole
     * @param chatStatus
     */
    ChatBuddy(std::string name, std::string alias, std::string realName, bool buddy, bool me,
              PurpleConvChatBuddyFlags chatRole, PurpleStatusPrimitive chatStatus);

    /** Comparaisons */
    bool operator==(const ChatBuddy& cbuddy) const;
    /** Comparaisons */
    bool operator!=(const ChatBuddy& cbuddy) const;
    /** Comparaisons */
    bool operator<(const ChatBuddy& cbuddy) const;

    /** Is ChatBuddy valid? */
    bool isValid() const { return !m_name.empty(); }
    /** Get ChatBuddy name */
    std::string getName() const;
    /** Get ChatBuddy alias */
    std::string getAlias() const;
    /** Get Chatbuddy real name */
    std::string getRealName() const;
    /** chat participant is on the buddy list */
    bool isBuddy() const;
    /** Is ChatBuddy I? */
    bool isMe() const;
    /** Get PurpleConvChatBuddyFlags */
    PurpleConvChatBuddyFlags getChatRole() const;
    /** Get buddy status */
    PurpleStatusPrimitive getChatStatus() const;
private:
    std::string m_name;
    std::string m_alias;
    std::string m_realName;
    bool m_buddy;
    bool m_me;
    PurpleConvChatBuddyFlags m_chatRole;
    PurpleStatusPrimitive m_chatStatus;
};

} /* namespace PURPLE */

#endif /* CONVERSATIONMM_H */
