/*
 * filetransfermm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "purplemm.h"
#include "filetransfermm.h"
#include "utilsmm.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()

FileTransfer::FileTransfer()
: m_xfer(NULL)
{}

FileTransfer::FileTransfer(PurpleXfer* xfer)
: m_xfer(xfer)
{
    if (purple_xfer_get_type(xfer) == PURPLE_XFER_RECEIVE)
    {
        m_name = p2s(purple_xfer_get_filename(xfer));
        m_path = p2s(purple_xfer_get_local_filename(xfer));
    }
    else
    {
        char *tmp, *utf8;
        tmp = g_path_get_basename(purple_xfer_get_local_filename(xfer));
        utf8 = g_filename_to_utf8(tmp, -1, NULL, NULL, NULL);
        g_free(tmp);
        m_name = p2s(utf8);
        g_free(utf8);
        m_path = p2s(purple_xfer_get_local_filename(xfer));
    }
    m_account = Account(xfer->account);
    m_buddy = p2s(purple_xfer_get_remote_user(xfer));
    m_started = false;
    m_canceled = false;
    m_finished = false;
    m_download = purple_xfer_get_type(xfer) == PURPLE_XFER_RECEIVE;
    m_lastChange = time(NULL);
    m_fileSize = purple_xfer_get_size(xfer);
    m_bytesSent = 0;
    m_speed = 0;
    m_xfer->ui_data = this;
}

void FileTransfer::init()
{
    purple_xfers_set_ui_ops(&ftOps);
}

void FileTransfer::uninit()
{
    purple_xfers_set_ui_ops(NULL);
}

bool FileTransfer::operator ==(const FileTransfer& ft) const
{
    return isValid() && ft.isValid() && this->m_xfer == ft.m_xfer;
}

void FileTransfer::cancelFT()
{
    assert(isValid());
    if(m_download) purple_xfer_cancel_remote(m_xfer);
    else purple_xfer_cancel_local(m_xfer);
}

std::string FileTransfer::getRemoteUser() const
{
    assert(isValid());
    return m_buddy;
}

Account FileTransfer::getAccount() const
{
    assert(isValid());
    return m_account;
}

std::string FileTransfer::getFileName() const
{
    return m_name;
}

std::string FileTransfer::getLocalFileName() const
{
    assert(isValid());
    return m_path;
}

size_t FileTransfer::getFileSize() const
{
    assert(isValid());
    return m_fileSize;
}

size_t FileTransfer::getBytesSent() const
{
    assert(isValid());
    return m_bytesSent;
}

size_t FileTransfer::getSpeed() const
{
    assert(isValid());
    return m_speed;
}

bool FileTransfer::isSending() const
{
    assert(isValid());
    return !m_download;
}

bool FileTransfer::isReceiving() const
{
    assert(isValid());
    return m_download;
}

bool FileTransfer::isStarted() const
{
    assert(isValid());
    return m_started;
}

bool FileTransfer::isFinished() const
{
    assert(isValid());
    return m_finished;
}

bool FileTransfer::isCanceled() const
{
    assert(isValid());
    return m_canceled;
}

void FileTransfer::updateProgress(size_t bytesSent, double percent)
{
    assert(isValid());
    time_t currentTime = time(NULL);
    bool sendProgress = ((currentTime - m_lastChange) != 0) && !m_finished;
    m_speed += bytesSent - m_bytesSent;
    m_bytesSent = bytesSent;
    m_lastChange = currentTime;
    if(sendProgress && CALLBACKS)
    {
        CALLBACKS->fileTransferProgress(this, percent);
        m_speed = 0;
    }
}

/* STATIC */

PurpleXferUiOps FileTransfer::ftOps =
{
    NULL,
    FileTransfer::destroy,
    FileTransfer::addXfer,
    FileTransfer::updateProgress,
    FileTransfer::cancelLocal,
    FileTransfer::cancelRemote,
    NULL,
    NULL,
    NULL,
    NULL
};

void FileTransfer::destroy(PurpleXfer* xfer)
{
    if(xfer)
    {
        FileTransfer *ft = (FileTransfer*)xfer->ui_data;
        if(ft && ft->isValid())
        {
            if(purple_xfer_is_completed(xfer) && CALLBACKS)
            {
                ft->updateFinished(true);
                CALLBACKS->fileTransferFinished(ft);
            }
        }
    }
}

void FileTransfer::addXfer(PurpleXfer* xfer)
{
    if(xfer)
    {
        FileTransfer *ft = new FileTransfer(xfer);
        if(ft->isValid() && CALLBACKS)
        {
            CALLBACKS->newFileTransfer(ft);
        }
    }
}

void FileTransfer::updateProgress(PurpleXfer* xfer, double percent)
{
    if(xfer)
    {
        FileTransfer *ft = (FileTransfer*)xfer->ui_data;
        if(ft)
        {
            if(!ft->getBytesSent() && CALLBACKS)
            {
                ft->updateStarted(true);
                CALLBACKS->fileTransferStarted(ft);
            }
            ft->updateProgress(purple_xfer_get_bytes_sent(xfer), percent);
        }
    }
}

void FileTransfer::cancelLocal(PurpleXfer* xfer)
{
    if(xfer)
    {
        FileTransfer *ft = (FileTransfer*)xfer->ui_data;
        if(ft && ft->isValid() && CALLBACKS)
        {
            ft->updateCanceled(true);
            CALLBACKS->fileTransferCanceled(ft);
        }
    }
}

void FileTransfer::cancelRemote(PurpleXfer* xfer)
{
    if(xfer)
    {
        FileTransfer *ft = (FileTransfer*)xfer->ui_data;
        if(ft && ft->isValid() && CALLBACKS)
        {
            ft->updateCanceled(true);
            CALLBACKS->fileTransferCanceled(ft);
        }
    }
}

} /* namespace PURPLE */

