/*
 * pluginmm.cpp
 *
 * Copyright(C) 2009-2010 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "pluginmm.h"
#include "utilsmm.h"

namespace PURPLE
{

Plugin::Option::Option()
    : m_type(Plugin::Option::NONE)
{}

Plugin::Option::Option(Plugin::Option::type_t _type, std::string _name, std::string _text, std::string _value)
    : m_type(_type),
      m_name(_name),
      m_value(_value),
      m_text(_text)
{}

bool Plugin::Option::operator==(std::string s) const
{
    return s == m_name;
}

int Plugin::Option::getIntValue() const
{
    return s2t<int>(m_value);
}

bool Plugin::Option::getBoolValue() const
{
    return (m_value == "true" || m_value == "1");
}

void Plugin::Option::save()
{
    switch(m_type) {
    case INT:
    case INTLIST:
        purple_prefs_set_int(m_name.c_str(), getIntValue());
        break;
    case STR:
    case STRLIST:
        purple_prefs_set_string(m_name.c_str(), m_value.c_str());
        break;
    case BOOL:
        purple_prefs_set_bool(m_name.c_str(), getBoolValue());
        break;
    default:
        break;
    }
}

Plugin::Plugin()
    : m_plug(0)
{}

Plugin::Plugin(PurplePlugin* _plug)
    : m_plug(_plug)
{}

std::string Plugin::getID() const
{
    assert(isValid());
    return m_plug->info->id;
}

std::string Plugin::getName() const
{
    assert(isValid());
    return m_plug->info->name;
}

std::string Plugin::getVersion() const
{
    assert(isValid());
    return m_plug->info->version;
}

std::string Plugin::getSummary() const
{
    assert(isValid());
    return m_plug->info->summary;
}

std::string Plugin::getDescription() const
{
    assert(isValid());
    return m_plug->info->description;
}

std::string Plugin::getAuthor() const
{
    assert(isValid());
    return m_plug->info->author;
}

std::string Plugin::getHomepage() const
{
    assert(isValid());
    return m_plug->info->homepage;
}

std::string Plugin::getPath() const
{
    assert(isValid());
    return m_plug->path;
}

Plugin::Option makeBoolOption(const char *name, const char *text)
{
    return Plugin::Option(Plugin::Option::BOOL, name, text, purple_prefs_get_bool(name)?"true":"false");
}

Plugin::Option makeIntListOption(const char *name, const char *text, PurplePluginPref *pref)
{
    Plugin::Option o = Plugin::Option(Plugin::Option::INTLIST, name, text);
    o.setValue(t2s(purple_prefs_get_int(name)));
    GList *l = purple_plugin_pref_get_choices(pref);
    std::map<std::string, int> v;
    char *txt;
    while(l != NULL && (txt = (char *)l->data) != NULL)
    {
        l = g_list_next(l);
        v.insert(std::pair<std::string, int>(txt, GPOINTER_TO_INT(l->data)));
        l = g_list_next(l);
    }
    o.setIntList(v);
    return o;
}

Plugin::Option makeIntOption(const char *name, const char *text, PurplePluginPref *pref)
{
    switch(purple_plugin_pref_get_type(pref)) {
        case PURPLE_PLUGIN_PREF_CHOICE:
            return makeIntListOption(name, text, pref);
        case PURPLE_PLUGIN_PREF_NONE:
        default:
            gint max, min;
            purple_plugin_pref_get_bounds(pref, &min, &max);
            Plugin::Option o = Plugin::Option(Plugin::Option::INT, name, text, t2s(purple_prefs_get_int(name)));
            o.setIntMin(min);
            o.setIntMax(max);
            return o;
    }
}

Plugin::Option makeStringListOption(const char *name, const char *text, PurplePluginPref *pref)
{
    Plugin::Option o = Plugin::Option(Plugin::Option::STRLIST, name, text);
    o.setValue(purple_prefs_get_string(name));
    GList *l = purple_plugin_pref_get_choices(pref);
    std::vector<std::string> v;
    char *txt;
    while(l != NULL && (txt = (char *)l->data) != NULL)
    {
        l = g_list_next(l);
        v.push_back(txt);
        l = g_list_next(l);
    }
    o.setStringList(v);
    return o;
}

Plugin::Option makeStringOption(const char *name, const char *text, PurplePluginPref *pref)
{
    switch(purple_plugin_pref_get_type(pref)) {
        case PURPLE_PLUGIN_PREF_CHOICE:
            return makeStringListOption(name, text, pref);
        case PURPLE_PLUGIN_PREF_NONE:
        default:
            return Plugin::Option(Plugin::Option::STR, name, text, purple_prefs_get_string(name));
    }
}

Plugin::Options Plugin::getOptions() const
{
    assert(isValid());
    Options o;
    if(hasOptions())
    {
        PurplePluginPrefFrame *frame = m_plug->info->prefs_info->get_plugin_pref_frame(m_plug);
        GList *pp;
        for(pp = purple_plugin_pref_frame_get_prefs(frame); pp != NULL; pp = pp->next)
        {
            PurplePluginPref *pref = (PurplePluginPref *)pp->data;
            const char *name = purple_plugin_pref_get_name(pref);
            const char *text = purple_plugin_pref_get_label(pref);
            if(name == NULL)
            {
                if(text == NULL)
                    continue;
                Option opt = Option(Option::NONE, "", text);
                o.push_back(opt);
                continue;
            }
            switch(purple_prefs_get_type(name)) {
                case PURPLE_PREF_BOOLEAN:
                    o.push_back(makeBoolOption(name, text));
                    break;
                case PURPLE_PREF_INT:
                    o.push_back(makeIntOption(name, text, pref));
                    break;
                case PURPLE_PREF_STRING:
                    o.push_back(makeStringOption(name, text, pref));
                    break;
                default:
                    break;
            }
        }
    }
    return o;
}

bool Plugin::hasOptions() const
{
    assert(isValid());
    return m_plug->info->prefs_info && m_plug->info->prefs_info->get_plugin_pref_frame;
}

bool Plugin::isLoaded() const
{
    assert(isValid());
    return purple_plugin_is_loaded(m_plug);
}

bool Plugin::isLoadable() const
{
    assert(isValid());
    return !purple_plugin_is_unloadable(m_plug);
}

void Plugin::load()
{
    assert(isValid());
    if(purple_plugin_is_unloadable(m_plug))
        throw LoadError("Plugin is unloadable");
    purple_plugin_load(m_plug);
}

void Plugin::unload()
{
    assert(isValid());
    if(purple_plugin_is_unloadable(m_plug))
        throw LoadError("Plugin is unloadable");
    if(!purple_plugin_unload(m_plug))
        throw LoadError(std::string("Unable to unload the plugin: ") + m_plug->error);
}

} /* namespace PURPLE */
