/*
 * pluginmm.h
 *
 * Copyright(C) 2009-2010 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PLUGINMM_H
#define PLUGINMM_H

#include <purple.h>
#include <string>
#include <vector>
#include <map>
#include "exceptionmm.h"
#include "exportmm.h"

namespace PURPLE
{

/** This class represents a plugin. */
class PURPLEMM_EXPORT Plugin
{
    PurplePlugin* m_plug;
public:
    /** This class represents a plugin option. */
    class PURPLEMM_EXPORT Option
    {
    public:
        enum type_t {
            NONE, //only show label
            BOOL,
            STR,
            STRLIST,
            INT,
            INTLIST
        };
    private:
        enum type_t m_type;
        std::string m_name;
        std::string m_value;
        std::string m_text;
        std::vector<std::string> m_slist;
        std::map<std::string, int> m_ilist;
        //for Option::INT
        int m_min, m_max;
    public:
        Option(enum type_t type, std::string name, std::string text, std::string value = "");
        Option();
        
        bool operator==(std::string s) const;

        enum type_t getType() const { return m_type; }
        std::string getName() const { return m_name; }
        std::string getText() const { return m_text; }
        std::string getStringValue() const { return m_value; }
        int getIntValue() const;
        bool getBoolValue() const;
        std::vector<std::string> getStringList() const { return m_slist; }
        std::map<std::string, int> getIntList() const { return m_ilist; }
        int getIntMin() const { return m_min; }
        int getIntMax() const { return m_max; }
        void setValue(std::string v) { m_value = v; }
        void setStringList(std::vector<std::string> l) { m_slist = l; }
        void setIntList(std::map<std::string, int> l) { m_ilist = l; }
        void setIntMin(int m) { m_min = m; }
        void setIntMax(int m) { m_max = m; }
        void save();
    };
    
    typedef std::vector<Option> Options;
    
    /** Empty constructor */
    Plugin();
    /** Create a Plugin instance
     *
     * @param plug  the libpurple's plugin instance.
     */
    Plugin(PurplePlugin* plug);

    /** Get true if plugin is valid */
    bool isValid() const { return m_plug != NULL; }
    /** Get plugin ID */
    std::string getID() const;
    /** Get plugin name */
    std::string getName() const;
    /** Get plugin version */
    std::string getVersion() const;
    /** Get plugin summary */
    std::string getSummary() const;
    /** Get plugin description */
    std::string getDescription() const;
    /** Get plugin author */
    std::string getAuthor() const;
    /** Get plugin homepage */
    std::string getHomepage() const;
    /** Get plugin path */
    std::string getPath() const;
    /** Get plugin options */
    Options getOptions() const;
    /** Get true if Plugin is loaded */
    bool isLoaded() const;
    /** Get true if Plugin is loadable */
    bool isLoadable() const;
    /** Get true if Plugin has options*/
    bool hasOptions() const;
    /** Load plugin */
    void load();
    /** Unload plugin */
    void unload();
private:
    STREXCEPTION(LoadError);
};

} /* namespace PURPLE */

#endif /* PLUGINMM_H */
