/*
 * protocolmm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "protocolmm.h"
#include "utilsmm.h"

namespace PURPLE
{

Protocol::Option::Option()
    : m_type(Protocol::Option::NONE)
{}

Protocol::Option::Option(Protocol::Option::type_t _type, std::string _name, std::string _text, std::string _value)
    : m_type(_type),
      m_name(_name),
      m_value(_value),
      m_text(_text)
{}

Protocol::Option::Option(PurplePrefType _type, std::string _name, std::string _text, std::string _value)
    : m_name(_name),
      m_value(_value),
      m_text(_text)
{
    switch(_type)
    {
        case PURPLE_PREF_BOOLEAN:
            m_type = Protocol::Option::BOOL;
            break;
        case PURPLE_PREF_STRING:
            m_type = Protocol::Option::STR;
            break;
        case PURPLE_PREF_STRING_LIST:
            m_type = Protocol::Option::STRLIST;
            break;
        case PURPLE_PREF_INT:
            m_type = Protocol::Option::INT;
            break;
        default:
            m_type = Protocol::Option::NONE;
            break;
    }
}

int Protocol::Option::getIntValue() const
{
    return s2t<int>(m_value);
}

bool Protocol::Option::getBoolValue() const
{
    return (m_value == "true" || m_value == "1");
}

std::map<std::string, std::string> Protocol::Option::getStringList() const
{
    if(m_type == Protocol::Option::STRLIST)
        return m_list;
    else
    {
        std::map<std::string, std::string> m;
        return m;
    }
}

bool Protocol::Option::operator==(std::string s) const
{
    return s == m_name;
}

Protocol::UsernameSplit::UsernameSplit()
    : m_text(""), m_defaultValue(""), m_separator('/'), m_reverse(false)
{}

Protocol::UsernameSplit::UsernameSplit(std::string text, std::string defaultValue, char separator, bool reverse)
    : m_text(text), m_defaultValue(defaultValue), m_separator(separator), m_reverse(reverse)
{}

std::string Protocol::UsernameSplit::getText() const
{
    return m_text;
}

std::string Protocol::UsernameSplit::getDefaultValue() const
{
    return m_defaultValue;
}

char Protocol::UsernameSplit::getSeparator() const
{
    return m_separator;
}

bool Protocol::UsernameSplit::getReverse() const
{
    return m_reverse;
}

Protocol::Protocol()
    : m_plugin(NULL)
{}

Protocol::Protocol(PurplePlugin* _plugin)
    : m_plugin(_plugin)
{}

bool Protocol::operator==(const Protocol& proto) const
{
    return proto.getPurpleID() == getPurpleID();
}

bool Protocol::operator!=(const Protocol& proto) const
{
    return proto.getPurpleID() != getPurpleID();
}

std::string Protocol::getName() const
{
    assert(isValid());
    return m_plugin->info->name;
}

std::string Protocol::getPurpleID() const
{
    assert(isValid());
    return m_plugin->info->id;
}

Protocol::Options Protocol::getOptions() const
{
    assert(isValid());
    Options options;
    PurplePluginProtocolInfo* prplinfo = PURPLE_PLUGIN_PROTOCOL_INFO(m_plugin);
    GList *iter, *list, *node;
    PurpleKeyValuePair *kvp;
    std::map<std::string, std::string> map;
    for (iter = prplinfo->protocol_options; iter; iter = iter->next)
    {
        PurpleAccountOption *option = static_cast<PurpleAccountOption*>(iter->data);
        PurplePrefType type = purple_account_option_get_type(option);

        Option opt = Option(type,
                    purple_account_option_get_setting(option),
                    purple_account_option_get_text(option));
        switch(type)
        {
            case PURPLE_PREF_BOOLEAN:
                opt.setValue(purple_account_option_get_default_bool(option) ? "true" : "false");
                break;
            case PURPLE_PREF_STRING:
            {
                const char* s = purple_account_option_get_default_string(option);
                if(s && *s)
                    opt.setValue(s);
                break;
            }
            case PURPLE_PREF_STRING_LIST:
            {
                const char* s = purple_account_option_get_default_list_value(option);
                if(s && *s)
                    opt.setValue(s);
                list = purple_account_option_get_list(option);
                for(node = list; node != NULL; node = node->next)
                {
                    kvp = (PurpleKeyValuePair *)node->data;
                    map.insert(std::pair<std::string, std::string>(std::string(kvp->key), std::string((char*)kvp->value)));
                }
                opt.setStringList(map);
                break;
            }
            case PURPLE_PREF_INT:
                opt.setValue(t2s(purple_account_option_get_default_int(option)));
                break;
            default:
                /* This option is not supported */
                continue;
        }
        options.push_back(opt);
    }
    return options;
}

std::vector<Protocol::UsernameSplit> Protocol::getUsernameSplit() const
{
    assert(isValid());
    std::vector<UsernameSplit> splits;
    PurplePluginProtocolInfo *info = PURPLE_PLUGIN_PROTOCOL_INFO(m_plugin);
    if(info)
    {
        GList *it = info->user_splits;
        for(; it; it = it->next)
        {
            PurpleAccountUserSplit *option = reinterpret_cast<PurpleAccountUserSplit*>(it->data);
            UsernameSplit split(p2s(purple_account_user_split_get_text(option)),
                            p2s(purple_account_user_split_get_default_value(option)),
                            purple_account_user_split_get_separator(option),
                            purple_account_user_split_get_reverse(option));
            splits.push_back(split);
        }
    }
    return splits;
}


PurplePluginProtocolInfo* Protocol::getPurpleProtocol() const
{
    assert(isValid());
    return PURPLE_PLUGIN_PROTOCOL_INFO(m_plugin);
}

bool Protocol::hasCheckMail() const
{
    assert(isValid());
    PurplePluginProtocolInfo *info = PURPLE_PLUGIN_PROTOCOL_INFO(m_plugin);
    if(info && info->options & OPT_PROTO_MAIL_CHECK)
    {
        return true;
    }
    return false;
}

bool Protocol::hasBuddyIcon() const
{
    assert(isValid());
    PurplePluginProtocolInfo *info = PURPLE_PLUGIN_PROTOCOL_INFO(m_plugin);
    if(info && info->icon_spec.format != NULL)
    {
        return true;
    }
    return false;
}

} /* namespace PURPLE */
