/*
 *      purplemm.h
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef PURPLEMM_H
#define PURPLEMM_H

#include <string>
#include <purple.h>
#include "exportmm.h"
#include "callbacksmm.h"
#include "protocolmm.h"
#include "pluginmm.h"

/** \mainpage Documentation
 *
 * <code>Libpurplemm</code> is C++ bindings for <a href='http://developer.pidgin.im/wiki/WhatIsLibpurple'><code>libpurple</code></a>.
 */

namespace PURPLE
{

/** Class to interface with libpurple. */
class PURPLEMM_EXPORT Purple {

public:
    /** Create Purple instance
     * @param callbacks  purplemm callbacks for notifications from library
     */
    Purple(Callbacks *callbacks);
    /** Create Purple instance
     * @param callbacks  purplemm callbacks for notifications from library
     * @param events  client implementation of PurpleEventLoopUiOps
     */
    Purple(Callbacks *callbacks, PurpleEventLoopUiOps *events);
    /** Create Purple instance
     * @param callbacks  purplemm callbacks for notifications from library
     * @param name  client name
     * @param version  client version
     * @param website  client website
     * @param configPath  path for saving configuration
     */
    Purple(Callbacks *callbacks, std::string name, std::string version, std::string website, std::string configPath);
    /** Create Purple instance
     * @param callbacks  purplemm callbacks for notifications from library
     * @param events  client implementation of PurpleEventLoopUiOps
     * @param name  client name
     * @param version  client version
     * @param website  client website
     * @param configPath  path for saving configuration
     */
    Purple(Callbacks *callbacks, PurpleEventLoopUiOps *events, std::string name, std::string version, std::string website, std::string configPath);
    virtual ~Purple();

    static GHashTable *uiInfo;
    static PurpleCoreUiOps coreOps;
    static PurpleDebugUiOps debugOps;
    static PurpleEventLoopUiOps *eventOps;

    static Purple* instance();
    /** init libpurple
     * @param offline   TRUE if start with accounts offline
     */
    bool initPurple(bool offline=false);
    /** Get TRUE if purple inited offline */
    bool isOffline() const;
    /** Idle setting
     * @param idle      TRUE for use away idle
     * @param idleTime  time for become idle
     */
    void setIdle(bool idle, int idleTime);
    /** Get TRUE if is idle set */
    bool isIdle() const;
    /** Get idle time */
    int getIdleTime() const;
    /** Touch idle tracker */
    void idleTouch();
    /** Set time at which accounts purportedly became idle */
    void idleSet(time_t time);
    /** Get Purplemm version */
    std::string getPurplemmVersion();
    /** Get libpurple version */
    std::string getLibpurpleVersion();
    /** Get Callbacks*/
    Callbacks *getCallbacks();
    /** Get client name */
    std::string getName() { return m_name; }
    /** Get client version */
    std::string getVersion() { return m_version; }
    /** Get client website */
    std::string getWebsite() { return m_website; }
    /** Get path for saving configuration */
    std::string getConfigPath() { return m_configPath; }
    /** Get plugin list */
    std::vector<Plugin> getPluginsList();
    /** Get protocol list */
    std::vector<Protocol> getProtocolsList();
    /** Get Protocol
     *
     * @param id    protocol id, ex. prpl-jabber
     * @return Protocol object
     */
    Protocol getProtocolById(std::string id);
    /** Has protocol
     *
     * @param id    protocol id, ex. prpl-jabber
     */
    bool hasProtocol(std::string id);
    /** Get accounts list */
    std::vector<Account> getAccountsList();
    /** Get enabled account list */
    std::vector<Account> getEnabledAccountsList();
    /** Get connected account list */
    std::vector<Account> getConnectedAccountsList();
    /** Some account connected? */
    bool isSomeAccountConnected();
    /** Add new account
     *
     * @param proto             Protocol for Account
     * @param username          Account username
     * @param password          Account password
     * @param rememberPassword  remember (save) password?
     * @param options           Account options
     * @param registerAccount   register Account on server?
     * @return added Account
     */
    Account addAccount(const Protocol& proto, const std::string& username, const std::string& password, bool rememberPassword, const Protocol::Options& options, bool registerAccount);
    /** Delete account */
    void deleteAccount(Account account);
    /** Find account
     *
     * @param username  Account username
     * @param protocol  Protocol id
     */
    Account findAccount(const std::string& username, const std::string& protocol) const;
    /**
     * Get boolean pref value
     *
     * @param name      Name of the pref
     * @return          Value of the pref; value if name doesn't exists
     */
    bool getBoolPref(const std::string& name, bool value) const;
    /**
     * Get integer pref value
     *
     * @param name      Name of the pref
     * @return          Value of the pref; value if name doesn't exists
     */
    int getIntPref(const std::string& name, int value) const;
    /**
     * Get string pref value
     *
     * @param name      Name of the pref
     * @return          Value of the pref; value if name doesn't exists
     */
    std::string getStringPref(const std::string& name, const std::string& value) const;
    /**
     * Add new pref
     * for pref /xx/aa/bb
     * you have to call fox example: addNonePref("/xx"); addNonePref("/xx/aa"); addBoolPref("/xx/aa/bb", true);
     *
     * @param name      Name of pref
     */
    void addNonePref(const std::string& name);
    /**
     * Add new bool pref
     *
     * @param name      Name of the pref
     * @param value     Initial value to set
     */
    void addBoolPref(const std::string& name, bool value);
    /**
     * Add new integer pref
     *
     * @param name      Name of the pref
     * @param value     Initial value to set
     */
    void addIntPref(const std::string& name, int value);
    /**
     * Add new string pref
     *
     * @param name      Name of the pref
     * @param value     Initial value to set
     */
    void addStringPref(const std::string& name, const std::string& value);
    /**
     * Set boolean pref value
     *
     * @param name      Name of the pref
     * @param value     Value to set
     */
    void setBoolPref(const std::string& name, bool value);
    /**
     * Set integer pref value
     *
     * @param name      Name of the pref
     * @param value     Value to set
     */
    void setIntPref(const std::string& name, int value);
    /**
     * Set string pref value
     *
     * @param name      Name of the pref
     * @param value     Value to set
     */
    void setStringPref(const std::string& name, const std::string& value);
    /**
     * Saves the list of loaded plugins to the specified preference key
     *
     * @param key   preference key, ex. /dxim/plugins
     */
    void saveLoadedPlugins(const std::string& key);
    /**
     * Attempts to load all the plugins in the specified preference key that were loaded when purple last quit.
     *
     * @param key   preference key, ex. /dxim/plugins
     */
    void loadSavedPlugins(const std::string& key);
    /** Create a Conversation
     *
     * @param account  Account instance
     * @param buddy  Buddy instance.
     */
    void createConversation(const Account& account, const Buddy& buddy);
    /** Create a Conversation
     *
     * @param account  Account instance
     * @param name  Chat name or buddy name
     * @param chat  True if need new PurpleConvChat
     */
    void createConversation(const Account& account, const std::string& name, bool chat=true);

    static void printDebug(PurpleDebugLevel level, const char *category, const char *arg_s);
    static void init();
    static void uninit();
    static GHashTable *getUiInfo();
private:
    static Purple* m_pInstance;
    Callbacks *m_callbacks;
    std::string m_name;
    std::string m_version;
    std::string m_website;
    std::string m_configPath;
    bool m_idle, m_offline;
    int m_idleTime;

    static void connectAccounts();
};

} /* namespace PURPLE */

#endif // PURPLEMM_H
