/*
 * requestmm.cpp
 *
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include <fstream>
#include "purplemm.h"
#include "accountmm.h"
#include "requestmm.h"
#include "utilsmm.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()

std::string getUniqueFilename()
{
    std::string file = g_get_tmp_dir();
    file += G_DIR_SEPARATOR_S;
    file += "purplemm_image_";
    file += time2s("%Y%m%d%H%M%S", now());
    file += t2s(rand()%10);
    return file;
}

void Request::init()
{
    purple_request_set_ui_ops(&requestOps);
}

void Request::uninit()
{
    purple_request_set_ui_ops(NULL);
}

PurpleRequestUiOps Request::requestOps =
{
    Request::requestInput, // request_input
    Request::requestChoice, // request_choice
    Request::requestAction, // request_action
    Request::requestFields, // request_fields
    Request::requestFile, // request_file
    Request::closeRequest, // close_request
    NULL, // request_folder
    NULL, // request_action_with_icon
    NULL, // purple_reserved
    NULL, // purple_reserved
    NULL // purple_reserved

};

void* Request::requestInput(const char *title, const char *primary,
                            const char *secondary, const char *default_value,
                            gboolean /*multiline*/, gboolean masked, gchar */*hint*/,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char */*who*/,
                            PurpleConversation */*conv*/, void *user_data)
{
    InputRequest *request = new InputRequest(PURPLE_REQUEST_INPUT, Account(account), p2s(title), p2s(primary), p2s(secondary),
                                            p2s(default_value), masked, p2s(ok_text), (PurpleRequestInputCb)ok_cb,
                                            p2s(cancel_text), (PurpleRequestInputCb)cancel_cb, user_data);
    if(CALLBACKS) CALLBACKS->newInputRequest(request);
    return request;
}

void* Request::requestChoice(const char *title, const char *primary,
                            const char *secondary, int default_value,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char *who,
                            PurpleConversation */*conv*/, void *user_data,
                            va_list choices)
{
    ChoiceRequest *request = new ChoiceRequest(PURPLE_REQUEST_CHOICE, Account(account), p2s(title), p2s(primary), p2s(secondary),
                                            p2s(who), default_value, choices, p2s(ok_text), (PurpleRequestChoiceCb)ok_cb, p2s(cancel_text),
                                            (PurpleRequestChoiceCb)cancel_cb, user_data);
    if(CALLBACKS) CALLBACKS->newChoiceRequest(request);
    return request;
}

void* Request::requestAction(const char *title, const char *primary,
                            const char *secondary, int default_action,
                            PurpleAccount *account, const char *who,
                            PurpleConversation */*conv*/, void *user_data,
                            size_t action_count, va_list actions)
{
    ActionRequest *request = new ActionRequest(PURPLE_REQUEST_ACTION, Account(account), p2s(title), p2s(primary), p2s(secondary),
                                                p2s(who), default_action, action_count, actions, user_data);
    if(CALLBACKS) CALLBACKS->newActionRequest(request);
    return request;
}

void* Request::requestFields(const char *title, const char *primary,
                            const char *secondary, PurpleRequestFields *fields,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char *who, PurpleConversation */*conv*/,
                            void *user_data)
{
    FieldsRequest *request = new FieldsRequest(PURPLE_REQUEST_FIELDS, Account(account), p2s(title), p2s(primary), p2s(secondary),
                                                p2s(who), fields, p2s(ok_text), (PurpleRequestFieldsCb)ok_cb, p2s(cancel_text),
                                                (PurpleRequestFieldsCb)cancel_cb, user_data);
    if(CALLBACKS) CALLBACKS->newFieldsRequest(request);
    return request;
}

void* Request::requestFile(const char *title, const char *filename,
                            gboolean savedialog, GCallback ok_cb, GCallback cancel_cb,
                            PurpleAccount *account, const char */*who*/, PurpleConversation */*conv*/,
                            void *user_data)
{
    FileRequest *request = new FileRequest(PURPLE_REQUEST_FILE, Account(account), p2s(title), p2s(filename),
                                            savedialog, (PurpleRequestFileCb)ok_cb, (PurpleRequestFileCb)cancel_cb,
                                            user_data);
    if(CALLBACKS) CALLBACKS->newFileRequest(request);
    return request;
}

void Request::closeRequest(PurpleRequestType /*type*/, void* ui_handle)
{
    assert(ui_handle);
    BasicRequest *request = reinterpret_cast<BasicRequest*>(ui_handle);
    if(request && CALLBACKS)
        CALLBACKS->closeRequest(request);
}

BasicRequest::BasicRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary)
: m_account(account), m_type(type), m_title(title), m_primary(primary), m_secondary(secondary)
{}

Account BasicRequest::getAccount() const
{
    return m_account;
}

PurpleRequestType BasicRequest::getType() const
{
    return m_type;
}

std::string BasicRequest::getTitle() const
{
    return m_title;
}

std::string BasicRequest::getPrimary() const
{
    return m_primary;
}

std::string BasicRequest::getSecondary() const
{
    return m_secondary;
}

InputRequest::InputRequest()
: BasicRequest(PURPLE_REQUEST_INPUT, Account(), "", "", ""), m_defaultValue(""), m_masked(false), m_okText(""), m_okCb(NULL),
  m_cancelText(""), m_cancelCb(NULL), m_userData(NULL)
{}

InputRequest::InputRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary, const std::string& defaultValue,
    bool masked, const std::string& okText, PurpleRequestInputCb okCb, const std::string& cancelText, PurpleRequestInputCb cancelCb, void* userData)
: BasicRequest(type, account, title, primary, secondary), m_defaultValue(defaultValue), m_masked(masked), m_okText(okText), m_okCb(okCb),
  m_cancelText(cancelText), m_cancelCb(cancelCb), m_userData(userData)
{}

void InputRequest::sendInput(std::string input)
{
    m_okCb(m_userData, input.c_str());
    purple_request_close(PURPLE_REQUEST_INPUT, this);
}

void InputRequest::dennyRequest()
{
    if(m_cancelCb) m_cancelCb(m_userData, "");
    purple_request_close(PURPLE_REQUEST_INPUT, this);
}

std::string InputRequest::getDefaultValue() const
{
    return m_defaultValue;
}

bool InputRequest::getMasked() const
{
    return m_masked;
}

std::string InputRequest::getOkText() const
{
    return m_okText;
}

std::string InputRequest::getCancelText() const
{
    return m_cancelText;
}

FileRequest::FileRequest()
: BasicRequest(PURPLE_REQUEST_FILE, Account(), "", "", ""), m_filename(""), m_saveDialog(true), m_okCb(NULL),
  m_cancelCb(NULL), m_userData(NULL)
{}

FileRequest::FileRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& filename,
        bool saveDialog, PurpleRequestFileCb okCb, PurpleRequestFileCb cancelCb, void* userData)
: BasicRequest(type, account, title, "", ""), m_filename(filename), m_saveDialog(saveDialog), m_okCb(okCb),
  m_cancelCb(cancelCb), m_userData(userData)
{}

std::string FileRequest::getFilename() const
{
    return m_filename;
}

bool FileRequest::getSaveDialog() const
{
    return m_saveDialog;
}

void FileRequest::setFilename(std::string filename)
{
    m_okCb(m_userData, filename.c_str());
    purple_request_close(PURPLE_REQUEST_FILE, this);
}

void FileRequest::dennyRequest()
{
    if(m_cancelCb) m_cancelCb(m_userData, NULL);
    purple_request_close(PURPLE_REQUEST_FILE, this);
}

ActionRequest::ActionRequest()
: BasicRequest(PURPLE_REQUEST_ACTION, Account(), "", "", ""), m_who(""), m_defaultAction(-1),
  m_userData(NULL)
{}

ActionRequest::ActionRequest(PurpleRequestType type, const Account& account,  const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, int defaultAction, size_t actioncount, va_list actions, void* userData)
: BasicRequest(type, account, title, primary, secondary), m_who(who), m_defaultAction(defaultAction), m_userData(userData)
{
    for(size_t i = 0; i < actioncount; i++)
    {
        std::string txt = p2s(va_arg(actions, const char *));
        PurpleRequestActionCb cb = va_arg(actions, PurpleRequestActionCb);
        m_callbacks.insert(std::pair<std::string,PurpleRequestActionCb>(txt, cb));
    }
}

std::string ActionRequest::getWho() const
{
    return m_who;
}

int ActionRequest::getDefaultAction() const
{
    return m_defaultAction;
}

std::map<std::string, PurpleRequestActionCb> ActionRequest::getCallbacks() const
{
    return m_callbacks;
}

void ActionRequest::runCallback(PurpleRequestActionCb cb)
{
    if(cb)
    {
        std::map<std::string,PurpleRequestActionCb>::iterator it;
        int i;
        for(it=m_callbacks.begin(),i=0; it != m_callbacks.end(); it++,i++)
        {
            if(cb == (*it).second)
            {
                cb(m_userData, i);
                purple_request_close(PURPLE_REQUEST_ACTION, this);
            }
        }
    }
}

ChoiceRequest::ChoiceRequest()
: BasicRequest(PURPLE_REQUEST_CHOICE, Account(), "", "", ""), m_who(""), m_defaultValue(-1), m_okText(""), m_okCb(NULL),
  m_cancelText(""), m_cancelCb(NULL), m_userData(NULL)
{}

ChoiceRequest::ChoiceRequest(PurpleRequestType type, const Account& account,  const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, int defaultValue, va_list choices, const std::string& okText, PurpleRequestChoiceCb okCb, const std::string& cancelText,
        PurpleRequestChoiceCb cancelCb, void* userData)
: BasicRequest(type, account, title, primary, secondary), m_who(who), m_defaultValue(defaultValue), m_okText(okText), m_okCb(okCb),
  m_cancelText(cancelText), m_cancelCb(cancelCb), m_userData(userData)
{
    const char *text;
    while((text = va_arg(choices, char*)))
    {
        m_choices.insert(std::pair<std::string,int>(p2s(text), va_arg(choices, int)));
    }
}

std::string ChoiceRequest::getWho() const
{
    return m_who;
}

int ChoiceRequest::getDefaultValue() const
{
    return m_defaultValue;
}

std::map<std::string, int> ChoiceRequest::getChoices() const
{
    return m_choices;
}

std::string ChoiceRequest::getOkText() const
{
    return m_okText;
}

std::string ChoiceRequest::getCancelText() const
{
    return m_cancelText;
}

void ChoiceRequest::sendChoice(int choice)
{
    if(m_okCb)
    {
        m_okCb(m_userData, choice);
        purple_request_close(PURPLE_REQUEST_CHOICE, this);
    }
}

void ChoiceRequest::dennyRequest()
{
    if(m_cancelCb)
    {
        m_cancelCb(m_userData, 0);
        purple_request_close(PURPLE_REQUEST_CHOICE, this);
    }
}

FieldsRequest::FieldsRequest()
: BasicRequest(PURPLE_REQUEST_FIELDS, Account(), "", "", ""), m_okText(""), m_okCb(NULL),
  m_cancelText(""), m_cancelCb(NULL), m_userData(NULL)
{}

FieldsRequest::FieldsRequest(PurpleRequestType type, const Account& account,  const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, PurpleRequestFields *fields, const std::string& okText, PurpleRequestFieldsCb okCb, const std::string& cancelText,
        PurpleRequestFieldsCb cancelCb, void* userData)
: BasicRequest(type, account, title, primary, secondary), m_who(who), m_okText(okText), m_okCb(okCb),
  m_cancelText(cancelText), m_cancelCb(cancelCb), m_fields(fields), m_userData(userData)
{
    GList *gl, *fl;
    std::string groupLabel = "";
    for(gl = purple_request_fields_get_groups(fields);
         gl != NULL;
         gl = gl->next)
    {
        GList *field_list;
        PurpleRequestFieldGroup *group = (PurpleRequestFieldGroup *) gl->data;
        field_list = purple_request_field_group_get_fields(group);
        if(purple_request_field_group_get_title(group) != NULL)
        {
            groupLabel = p2s(purple_request_field_group_get_title(group));
        }
        else
        {
            groupLabel = "";
        }
        for(fl = field_list; fl != NULL; fl = fl->next)
        {
            PurpleRequestFieldType type;
            std::string label = "";
            std::string key = "";
            PurpleRequestField *fld = (PurpleRequestField *) fl->data;
            if (!purple_request_field_is_visible(fld)) {
                continue;
            }
            type = purple_request_field_get_type(fld);
            key = p2s(purple_request_field_get_id(fld));
            label = p2s(purple_request_field_get_label(fld));
            if(label.empty()) label = key;
            if(type == PURPLE_REQUEST_FIELD_STRING)
            {
                m_mmfields.push_back(Field(Field::FSTR, key, label, groupLabel, p2s(purple_request_field_string_get_default_value(fld)),
                                    purple_request_field_string_is_multiline(fld), purple_request_field_string_is_masked(fld),
                                    purple_request_field_string_is_editable(fld)));
            }
            else if(type == PURPLE_REQUEST_FIELD_INTEGER)
            {
                m_mmfields.push_back(Field(Field::FINT, key, label, groupLabel, purple_request_field_int_get_default_value(fld)));
            }
            else if(type == PURPLE_REQUEST_FIELD_BOOLEAN)
            {
                m_mmfields.push_back(Field(Field::FBOOL, key, label, groupLabel, purple_request_field_bool_get_default_value(fld)));
            }
            else if(type == PURPLE_REQUEST_FIELD_CHOICE)
            {
                int i = 0;
                GList *labels = purple_request_field_choice_get_labels(fld);
                GList *l;
                std::vector<std::string> v;
                for(l = labels; l != NULL; l = l->next, i++)
                {
                    v.push_back(std::string((char *) l->data));
                }
                m_mmfields.push_back(Field(Field::FCHOICE, key, label, groupLabel, purple_request_field_choice_get_default_value(fld), v));
            }
            else if(type == PURPLE_REQUEST_FIELD_LIST)
            {
                GList *l;
                std::vector<std::string> v, s;
                for(l = purple_request_field_list_get_items(fld); l != NULL; l = l->next)
                {
                    v.push_back(std::string((char *) l->data));
                    if(purple_request_field_list_is_selected(fld, (char *) l->data))
                        s.push_back(std::string((char *) l->data));
                }
                m_mmfields.push_back(Field(Field::FLIST, key, label, groupLabel, purple_request_field_list_get_multi_select(fld), v, s));
            }
            else if(type == PURPLE_REQUEST_FIELD_IMAGE)
            {
                std::string file = getUniqueFilename();
                std::ofstream stream;
                stream.open(file.c_str(), std::ios_base::binary);
                if(stream.good())
                    stream.write(purple_request_field_image_get_buffer(fld), purple_request_field_image_get_size(fld));
                m_mmfields.push_back(Field(Field::FIMAGE, key, label, groupLabel, file));
            }
//            else if(type == PURPLE_REQUEST_FIELD_ACCOUNT)
//            {
//            }
            else
            {
                std::ostringstream oss;
                oss << "DEBUG [fieldsrequest] Skipped a field " << label << " with type " << type;
                if(CALLBACKS) CALLBACKS->showDebug(oss.str());
                delete fld;
                continue;
            }
        }
    }
}

std::string FieldsRequest::getWho() const
{
    return m_who;
}

std::string FieldsRequest::getOkText() const
{
    return m_okText;
}

std::string FieldsRequest::getCancelText() const
{
    return m_cancelText;
}

void FieldsRequest::setFields(Fields fields)
{
    m_mmfields = fields;
}

void FieldsRequest::acceptRequest()
{
    FOREACH(Fields, m_mmfields, it)
    {
        PURPLE::FieldsRequest::Field field = *it;
        PurpleRequestField *fld = purple_request_fields_get_field(m_fields, field.getKey().c_str());
        if (!fld) continue;
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            purple_request_field_string_set_value(fld, field.getStringValue().c_str());
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            purple_request_field_int_set_value(fld, field.getIntValue());
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            purple_request_field_bool_set_value(fld, field.getBoolValue());
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            purple_request_field_choice_set_value(fld, field.getIntValue());
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            std::vector<std::string> v = field.getSelected();
            FOREACH(std::vector<std::string>, v, it)
            {
                purple_request_field_list_add_selected(fld, (*it).c_str());
            }
        }
        else
            continue;
    }
    m_okCb(m_userData, m_fields);
    purple_request_close(PURPLE_REQUEST_FIELDS, this);
}

void FieldsRequest::dennyRequest()
{
    if(m_cancelCb) m_cancelCb(m_userData, m_fields);
    purple_request_close(PURPLE_REQUEST_FIELDS, this);
}

FieldsRequest::Field::Field()
: m_type(FNONE), m_key(""), m_label(""), m_group("")
{}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group,
    std::string defaultValue, bool multiline, bool masked, bool editable)
: m_type(type), m_key(key), m_label(label), m_group(group),
m_multiline(multiline), m_masked(masked), m_editable(editable), m_value(defaultValue)
{}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group, int defaultValue)
: m_type(type), m_key(key), m_label(label), m_group(group)
{
    setValue(defaultValue);
}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group, bool defaultValue)
: m_type(type), m_key(key), m_label(label), m_group(group)
{
    setValue(defaultValue);
}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group, int defaultValue, std::vector<std::string> labels)
: m_type(type), m_key(key), m_label(label), m_group(group), m_labels(labels)
{
    setValue(defaultValue);
}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group, bool multiple, std::vector<std::string> items, std::vector<std::string> selected)
: m_type(type), m_key(key), m_label(label), m_group(group), m_multiple(multiple), m_labels(items), m_selected(selected)
{}

FieldsRequest::Field::Field(enum field_t type, std::string key, std::string label, std::string group,
                    std::string value)
: m_type(type), m_key(key), m_label(label), m_group(group), m_value(value)
{}

void FieldsRequest::Field::setValue(int value)
{
    m_value = t2s(value);
}

void FieldsRequest::Field::setValue(bool value)
{
    m_value = value ? "true" : "false";
}

int FieldsRequest::Field::getIntValue() const
{
    return s2t<int>(m_value);
}

bool FieldsRequest::Field::getBoolValue() const
{
    return (m_value == "true" || m_value == "1");
}

} /* namespace PURPLE */

