/*
 * requestmm.h
 *
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef REQUESTMM_H
#define REQUESTMM_H

#include <purple.h>
#include <string>
#include <vector>
#include <map>
#include "exportmm.h"

namespace PURPLE
{
class Account;

/** This class represents a request */
class PURPLEMM_EXPORT Request
{
    static PurpleRequestUiOps requestOps;
    static void* requestInput(const char *title, const char *primary,
                            const char *secondary, const char *default_value,
                            gboolean multiline, gboolean masked, gchar *hint,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char *who,
                            PurpleConversation *conv, void *user_data);
    static void* requestChoice(const char *title, const char *primary,
                            const char *secondary, int default_value,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char *who,
                            PurpleConversation *conv, void *user_data,
                            va_list choices);
    static void* requestAction(const char *title, const char *primary,
                            const char *secondary, int default_action,
                            PurpleAccount *account, const char *who,
                            PurpleConversation *conv, void *user_data,
                            size_t action_count, va_list actions);
    static void* requestFields(const char *title, const char *primary,
                            const char *secondary, PurpleRequestFields *fields,
                            const char *ok_text, GCallback ok_cb,
                            const char *cancel_text, GCallback cancel_cb,
                            PurpleAccount *account, const char *who, PurpleConversation *conv,
                            void *user_data);
    static void* requestFile(const char *title, const char *filename,
                            gboolean savedialog, GCallback ok_cb, GCallback cancel_cb,
                            PurpleAccount *account, const char *who, PurpleConversation *conv,
                            void *user_data);
    static void closeRequest(PurpleRequestType type, void *ui_handle);
public:
    /** Initialization of libpurple request' stuffs. */
    static void init();
    /** Uninitialization of libpurple request' stuffs. */
    static void uninit();
};

/** This class represents parent of requests class */
class PURPLEMM_EXPORT BasicRequest
{
public:
    BasicRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary);

    /** Get account this request is for */
    Account getAccount() const;
    /** Get request type */
    PurpleRequestType getType() const;
    /** Get title of the message */
    std::string getTitle() const;
    /** Get main point of the message */
    std::string getPrimary() const;
    /** Get secondary information */
    std::string getSecondary() const;
private:
    Account m_account;
    PurpleRequestType m_type;
    std::string m_title;
    std::string m_primary;
    std::string m_secondary;
};

/** This class represents text input request */
class PURPLEMM_EXPORT InputRequest : public BasicRequest
{
public:
    /** Empty constructor */
    InputRequest();
    /** Create a InputRequest instance */
    InputRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary, const std::string& defaultValue, bool masked,
        const std::string& okText, PurpleRequestInputCb okCb, const std::string& cancelText,
        PurpleRequestInputCb cancelCb, void* userData);

    /** Send result of input request */
    void sendInput(std::string input);
    /** Denny request */
    void dennyRequest();
    /** Get default value */
    std::string getDefaultValue() const;
    /** Get TRUE if the inputted text should be masked in some way (such as by displaying characters as stars) */
    bool getMasked() const;
    /** Get text for the OK button */
    std::string getOkText() const;
    /** Get text for the Cancel button */
    std::string getCancelText() const;
private:
    std::string m_defaultValue;
    bool m_masked;
    std::string m_okText;
    PurpleRequestInputCb m_okCb;
    std::string m_cancelText;
    PurpleRequestInputCb m_cancelCb;
    void* m_userData;
};

/** This class represents filename request */
class PURPLEMM_EXPORT FileRequest : public BasicRequest
{
public:
    /** Empty constructor */
    FileRequest();
    /** Create a FileRequest instance */
    FileRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& filename,
        bool saveDialog, PurpleRequestFileCb okCb, PurpleRequestFileCb cancelCb, void* userData);

    /** Get default filename */
    std::string getFilename() const;
    /** True if this dialog is being used to save a file */
    bool getSaveDialog() const;
    /** Set filename */
    void setFilename(std::string filename);
    /** Denny request */
    void dennyRequest();
private:
    std::string m_filename;
    bool m_saveDialog;
    PurpleRequestFileCb m_okCb;
    PurpleRequestFileCb m_cancelCb;
    void* m_userData;
};

/** This class represents action request */
class PURPLEMM_EXPORT ActionRequest : public BasicRequest
{
public:
    /** Empty constructor */
    ActionRequest();
    /** Create a ActionRequest instance */
    ActionRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, int defaultAction, size_t actioncount, va_list actions, void* userData);

    /** Get username of the buddy associated with this request, maybe empty */
    std::string getWho() const;
    /** Get default action, zero-indexed */
    int getDefaultAction() const;
    /** Get callbacks map
     * first is label that appears on the button
     * second is callback
    */
    std::map<std::string, PurpleRequestActionCb> getCallbacks() const;
    /** Run callback */
    void runCallback(PurpleRequestActionCb cb);
private:
    std::string m_who;
    int m_defaultAction;
    std::map<std::string, PurpleRequestActionCb> m_callbacks;
    void* m_userData;
};

/** This class represents choice request */
class PURPLEMM_EXPORT ChoiceRequest : public BasicRequest
{
public:
    /** Empty constructor */
    ChoiceRequest();
    /** Create a ChoiceRequest instance */
    ChoiceRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, int defaultValue, va_list choices, const std::string& okText, PurpleRequestChoiceCb okCb, const std::string& cancelText,
        PurpleRequestChoiceCb cancelCb, void* userData);

    /** Get username of the buddy associated with this request, maybe empty */
    std::string getWho() const;
    /** Get default default choice */
    int getDefaultValue() const;
    /** Get choices map
     * first is label
     * second is int value
    */
    std::map<std::string, int> getChoices() const;
    /** Get text for the OK button */
    std::string getOkText() const;
    /** Get text for the Cancel button */
    std::string getCancelText() const;
    /** Send Choice */
    void sendChoice(int choice);
    /** Denny request */
    void dennyRequest();
private:
    std::string m_who;
    int m_defaultValue;
    std::map<std::string, int> m_choices;
    std::string m_okText;
    PurpleRequestChoiceCb m_okCb;
    std::string m_cancelText;
    PurpleRequestChoiceCb m_cancelCb;
    void* m_userData;
};

/** This class represents fields request */
class PURPLEMM_EXPORT FieldsRequest : public BasicRequest
{
public:
    /** This class represents field. */
    class PURPLEMM_EXPORT Field
    {
        public:
            enum field_t {
                FNONE,
                FSTR, //string
                FINT, //integer
                FBOOL, //boolean
                FCHOICE,
                FLIST,
                FIMAGE
            };
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    std::string value, bool multiline, bool masked, bool editable);
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    int value);
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    bool value);
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    int value, std::vector<std::string> labels);
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    bool multiple, std::vector<std::string> items, std::vector<std::string> selected);
            /** Create a Field instance */
            Field(enum field_t type, std::string key, std::string label, std::string group,
                    std::string value);
            /** Empty constructor */
            Field();

            /** Get Field type */
            enum field_t getType() const { return m_type; }
            /** Get Field key */
            std::string getKey() const { return m_key; }
            /** Get Field label (for UI) */
            std::string getLabel() const { return m_label; }
            /** Get Field group */
            std::string getGroup() const { return m_group; }
            /** Get FSTR,FIMAGE Field value */
            std::string getStringValue() const { return m_value; }
            /** Set Field value */
            void setValue(std::string value) { m_value = value; }
            /** Set Field value */
            void setValue(int value);
            /** Set Field value */
            void setValue(bool value);
            /** Get FSTR Field whether or not this should be a multiline string */
            bool getMultiline() const { return m_multiline; }
            /** Get FSTR Field whether or not a string field is masked (commonly used for password fields) */
            bool getMasked() const { return m_masked; }
            /** Get FSTR Field whether or not a string field is editable */
            bool getEditable() const { return m_editable; }
            /** Get FINT,FCHOICE Field value */
            int getIntValue() const;
            /** Get FBOOL Field value */
            bool getBoolValue() const;
            /** Get FCHOICE,FLIST Field labels */
            std::vector<std::string> getLabels() const { return m_labels; }
            /** Get FLIST TRUE if multiple selection is enabled or FALSE otherwise */
            bool getMultiple() const { return m_multiple; }
            /** Get FLIST Field selected items */
            std::vector<std::string> getSelected() const { return m_selected; }
            /** Set FLIST Field selected items */
            void setSelected(std::vector<std::string> selected) { m_selected = selected; }
        private:
            enum field_t m_type;
            std::string m_key;
            std::string m_label;
            std::string m_group;
            bool m_multiline;
            bool m_masked;
            bool m_editable;
            bool m_multiple;
            std::string m_value;
            std::vector<std::string> m_labels;
            std::vector<std::string> m_selected;
    };

    typedef std::vector<Field> Fields;

    /** Empty constructor */
    FieldsRequest();
    /** Create a FieldsRequest instance */
    FieldsRequest(PurpleRequestType type, const Account& account, const std::string& title, const std::string& primary, const std::string& secondary,
        const std::string& who, PurpleRequestFields *fields, const std::string& okText, PurpleRequestFieldsCb okCb, const std::string& cancelText,
        PurpleRequestFieldsCb cancelCb, void* userData);

    /** Get username of the buddy associated with this request, maybe empty */
    std::string getWho() const;
    /** Get text for the OK button */
    std::string getOkText() const;
    /** Get text for the Cancel button */
    std::string getCancelText() const;
    /** Get Field items */
    Fields getFields() const { return m_mmfields; }
    /** Set Field items */
    void setFields(Fields fields);
    /** Accept request */
    void acceptRequest();
    /** Denny request */
    void dennyRequest();
private:
    std::string m_who;
    std::string m_okText;
    PurpleRequestFieldsCb m_okCb;
    std::string m_cancelText;
    PurpleRequestFieldsCb m_cancelCb;
    PurpleRequestFields *m_fields;
    Fields m_mmfields;
    void* m_userData;
};

} /* namespace PURPLE */

#endif // REQUESTMM_H
