/*
 * filetransfermm.cpp
 *
 * Copyright(C) 2009 Romain Bignon
 * Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <cassert>
#include "roomlistmm.h"
#include "utilsmm.h"
#include "purplemm.h"

namespace PURPLE
{
#define CALLBACKS Purple::instance()->getCallbacks()
#define ROOMLIST(rlist) ((RoomList*)(rlist)->ui_data)

RoomList::RoomList()
    : m_rlist(NULL)
{}

RoomList::RoomList(PurpleRoomlist* rlist)
    : m_rlist(rlist)
{
    m_rlist->ui_data = this;
}

void RoomList::init()
{
    purple_roomlist_set_ui_ops(&rlistOps);
}

void RoomList::uninit()
{
    purple_roomlist_set_ui_ops(NULL);
}

bool RoomList::operator==(const RoomList& list) const
{
    return this->isValid() && list.isValid() && list.m_rlist == this->m_rlist;
}

Account RoomList::getAccount() const
{
    assert(isValid());
    return Account(m_rlist->account);
}

void RoomList::cancelListing()
{
    assert(isValid());
    if(isInProgress()) purple_roomlist_cancel_get_list(m_rlist);
}

bool RoomList::isInProgress() const
{
    assert(isValid());
    return m_rlist->in_progress;
}

/* STATIC */

PurpleRoomlistUiOps RoomList::rlistOps =
{
    NULL,
    &RoomList::create,
    &RoomList::setFields,
    &RoomList::addRoom,
    &RoomList::inProgress,
    &RoomList::destroy,
    NULL,
    NULL,
    NULL,
    NULL
};

void RoomList::create(PurpleRoomlist* list)
{
    RoomList *rlist = ROOMLIST(list);
    if(rlist == NULL) rlist = new RoomList(list);
    if(rlist->isValid() && CALLBACKS) CALLBACKS->roomlistCreated(rlist);
}

void RoomList::setFields(PurpleRoomlist* list, GList* fields)
{
    RoomList *rlist = ROOMLIST(list);
    if(rlist && rlist->isValid())
    {
        std::vector<std::string> columns;
        GList* l;
        for(l = fields; l; l = l->next)
        {
            PurpleRoomlistField *f = (PurpleRoomlistField*)l->data;
            if(!f->hidden)
                columns.push_back(std::string(f->label));
        }
        if(CALLBACKS) CALLBACKS->roomlistColumns(rlist, columns);
    }
}

void RoomList::addRoom(PurpleRoomlist* list, PurpleRoomlistRoom* room)
{
    PurpleConnection *gc = purple_account_get_connection(list->account);
    if(!gc)
        return;
    RoomList *rlist = ROOMLIST(list);
    if(rlist && rlist->isValid() && room->type == PURPLE_ROOMLIST_ROOMTYPE_ROOM)
    {
        std::vector<std::string> fields;
        fields.push_back(p2s(room->name));
        GList *iter, *field;
        for(iter = purple_roomlist_room_get_fields(room),
             field = purple_roomlist_get_fields(list);
             iter && field;
             iter = iter->next, field = field->next)
        {
            PurpleRoomlistField *f = (PurpleRoomlistField*)field->data;
            if(purple_roomlist_field_get_hidden(f))
                continue;
            switch(purple_roomlist_field_get_type(f)) {
                case PURPLE_ROOMLIST_FIELD_BOOL:
                    fields.push_back(iter->data ? "True" : "False");
                    break;
                case PURPLE_ROOMLIST_FIELD_INT:
                    fields.push_back(t2s((size_t)iter->data));
                    break;
                case PURPLE_ROOMLIST_FIELD_STRING:
                    fields.push_back(std::string((const char*)iter->data));
                    break;
            }
        }
        if(CALLBACKS) CALLBACKS->roomlistAddRoom(rlist, fields);
    }
}

void RoomList::inProgress(PurpleRoomlist* list, gboolean flag)
{
    RoomList *rlist = ROOMLIST(list);
    if(rlist && rlist->isValid() && !flag && CALLBACKS)
        CALLBACKS->roomlistFinished(rlist);
}

void RoomList::destroy(PurpleRoomlist* /*list*/)
{
}

} /* namespace PURPLE */

