/*
 *      callback.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QDateTime>
#include "callback.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

static GMainLoop *gMainLoop = NULL;

gpointer purpleMainEventLoop(gpointer data)
{
    if(gMainLoop)
    {
        dxutils::debugLine("[CALLBACK] gMainLoop already created");
    }
    gMainLoop = g_main_loop_new(NULL, FALSE);
    dxutils::debugLine("[CALLBACK] Starting gMainLoop");
    g_main_loop_run(gMainLoop);
    g_main_loop_unref(gMainLoop);
    dxutils::debugLine("[CALLBACK] gMainLoop stopped");
    gMainLoop = NULL;
    g_thread_exit(NULL);
    return NULL;
}

dxqCallbacks::dxqCallbacks(QObject *parent) :
    QObject(parent)
{
#if !defined(QT_WITH_GLIB)
#if GLIB_VERSION_MIN_REQUIRED > GLIB_VERSION_2_30
    g_thread_new("dxim-qt", purpleMainEventLoop, NULL);
#else
    if(!g_thread_supported())
    {
        g_thread_init(NULL);
    }
    g_thread_create(purpleMainEventLoop, NULL, FALSE, NULL);
#endif
#endif
}

dxqCallbacks::~dxqCallbacks()
{
#if !defined(QT_WITH_GLIB)
    if(gMainLoop)
    {
        dxutils::debugLine("[CALLBACK] Stopping gMainLoop");
        g_main_loop_quit(gMainLoop);
    }
    else
    {
        dxutils::debugLine("[CALLBACK] No gMainLoop created");
    }
#endif
}

/** Allow your application to be notified about errors on library layer */
void dxqCallbacks::showError(std::string msg)
{
    emit signalPurpleError(dxutils::fromStdString(msg));
}

/** Allow your application to be notified about libpurple debug messages */
void dxqCallbacks::showDebug(std::string debug)
{
    dxutils::debugLine(dxutils::fromStdString(debug));
}

/** Libpurple inited */
void dxqCallbacks::buddylistLoaded()
{
    emit signalBuddylistLoaded();
}

/** New account added */
void dxqCallbacks::accountAdded(PURPLE::Account account)
{
    emit signalAccountAdded(account);
}

/** Account removed */
void dxqCallbacks::accountRemoved(std::string account, std::string protocolID)
{
    emit signalAccountRemoved(dxutils::fromStdString(account), dxutils::fromStdString(protocolID));
}

/** Account connecting */
void dxqCallbacks::accountConnecting(PURPLE::Account account, std::string msg)
{
    emit signalAccountConnecting(account, dxutils::fromStdString(msg));
}

/** Acount connected*/
void dxqCallbacks::accountConnected(PURPLE::Account account)
{
    emit signalAccountConnected(account);
}

/** Acount disconnected*/
void dxqCallbacks::accountDisconnected(PURPLE::Account account, std::string reason)
{
    emit signalAccountDisconnected(account, dxutils::fromStdString(reason));
}

/** Account status changed*/
void dxqCallbacks::accountStatusChanged(PURPLE::Account account)
{
    emit signalAccountStatusChanged(account);
}

/** Account signed on, so show buddy list */
void dxqCallbacks::accountSignedOn(PURPLE::Account account)
{
    emit signalAccountSignedOn(account);
}

/** Buddy signed on */
void dxqCallbacks::buddySignedOn(PURPLE::Buddy buddy)
{
    emit signalBuddySignedOn(buddy);
}

/** Buddy signed off */
void dxqCallbacks::buddySignedOff(PURPLE::Buddy buddy)
{
    emit signalBuddySignedOff(buddy);
}

/** Buddy status changed */
void dxqCallbacks::buddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus)
{
    emit signalBuddyStatusChanged(buddy, oldStatus, newStatus);
}

/** Buddy icon changed */
void dxqCallbacks::buddyIconChanged(PURPLE::Buddy buddy)
{
    emit signalBuddyIconChanged(buddy);
}
/** Buddy added */
void dxqCallbacks::buddyAdded(PURPLE::Buddy buddy)
{
    emit signalBuddyAdded(buddy);
}

/** Buddy removed */
void dxqCallbacks::buddyRemoved(PURPLE::Account account, std::string buddy)
{
    emit signalBuddyRemoved(account, dxutils::fromStdString(buddy));
}

/** Chat added */
void dxqCallbacks::chatAdded(PURPLE::Chat chat)
{
    emit signalChatAdded(chat);
}

/** Chat removed */
void dxqCallbacks::chatRemoved(PURPLE::Account account, std::string chat)
{
    emit signalChatRemoved(account, dxutils::fromStdString(chat));
}

/** New IM created */
void dxqCallbacks::newIM(PURPLE::Conversation *conv)
{
    emit signalNewIM(conv);
}

/** New Chat created */
void dxqCallbacks::newChat(PURPLE::Conversation *conv)
{
    emit signalNewChat(conv);
}

/** IM closed */
void dxqCallbacks::closedIM(PURPLE::Conversation *conv)
{
    emit signalCloseConvTab(conv);
}

/** Chat closed */
void dxqCallbacks::closedChat(PURPLE::Conversation *conv)
{
    emit signalCloseConvTab(conv);
}

/** New my message
* @param conv          The conversation.
* @param msg           The message.
* @param time          The time the message was sent.
* @param action        Is message in 3rd person (/me)?
*/
void dxqCallbacks::newMyMessage(PURPLE::Conversation *conv, std::string msg, time_t time, bool action, std::string messageID)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    QDateTime qtime = QDateTime::fromSecsSinceEpoch(time);
#else
    QDateTime qtime;
    qtime.setTime_t(time);
#endif
    emit signalNewMyMessage(conv, dxutils::fromStdString(PURPLE::markupHtml(msg)), qtime.toString("hh:mm:ss"), action, dxutils::fromStdString(messageID));
}

void dxqCallbacks::messageRecived(PURPLE::Conversation *conv, std::string messageID)
{
    if(DXPREFERENCES.m_receiptsMark && !messageID.empty()) emit signalMessageRecived(conv, dxutils::fromStdString(messageID));
}

/** New IM message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxqCallbacks::newIMMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    QDateTime qtime = QDateTime::fromSecsSinceEpoch(time);
    QString delay = QLocale::system().dateTimeFormat(QLocale::ShortFormat);
#else
    QDateTime qtime;
    qtime.setTime_t(time);
    Qt::DateFormat delay = Qt::SystemLocaleShortDate;
#endif
    emit signalNewMessage(conv, dxutils::fromStdString(sender), dxutils::fromStdString(PURPLE::markupHtml(msg)), delayed?qtime.toString(delay):qtime.toString("hh:mm:ss"), highlight, delayed, action);
}

/** New Chat message
 * @param conv          The conversation.
 * @param sender        The who sent the message.
 * @param msg           The message.
 * @param time          The time the message was sent.
 * @param delayed       Is message delayed?
 * @param highlight     Is your nick on msg?
 * @param action        Is message in 3rd person (/me)?
 */
void dxqCallbacks::newChatMessage(PURPLE::Conversation *conv, std::string sender, std::string msg, time_t time, bool delayed, bool highlight, bool action)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    QDateTime qtime = QDateTime::fromSecsSinceEpoch(time);
    QString delay = QLocale::system().dateTimeFormat(QLocale::ShortFormat);
#else
    QDateTime qtime;
    qtime.setTime_t(time);
    Qt::DateFormat delay = Qt::SystemLocaleShortDate;
#endif
    emit signalNewMessage(conv, dxutils::fromStdString(sender), dxutils::fromStdString(PURPLE::markupHtml(msg)), delayed?qtime.toString(delay):qtime.toString("hh:mm:ss"), highlight, delayed, action);
}

/** New system mesage for conversation
 *
 * @param conv  The conversation
 * @param msg  The message text
 * @param time  The time the message was sent
 */
void dxqCallbacks::newConvSystemMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    QDateTime qtime = QDateTime::fromSecsSinceEpoch(time);
#else
    QDateTime qtime;
    qtime.setTime_t(time);
#endif
    emit signalNewConvSystemMessage(conv, dxutils::fromStdString(msg), qtime.toString("hh:mm:ss"));
}

/** New error mesage for conversation
*
* @param conv      conversation
* @param msg       error text
* @param time      time the error was sent
*/
void dxqCallbacks::newConvErrorMessage(PURPLE::Conversation *conv, std::string msg, time_t time)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    QDateTime qtime = QDateTime::fromSecsSinceEpoch(time);
#else
    QDateTime qtime;
    qtime.setTime_t(time);
#endif
    emit signalNewConvErrorMessage(conv, dxutils::fromStdString(msg), qtime.toString("hh:mm:ss"));
}

/** ChatBuddy joined chat */
void dxqCallbacks::chatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    emit signalChatBuddyJoined(conv, buddy);
}

/** ChatBuddy updated */
void dxqCallbacks::chatBuddyUpdated(PURPLE::Conversation *conv, std::string buddy)
{
    emit signalChatBuddyUpdated(conv, dxutils::fromStdString(buddy));
}

/** ChatBuddy left chat */
void dxqCallbacks::chatBuddyLeft(PURPLE::Conversation *conv, std::string buddy)
{
    emit signalChatBuddyLeft(conv, dxutils::fromStdString(buddy));
}

/** ChatBuddy renamed
 * @param obuddy  old buddy which was renamed
 * @param nbuddy  new buddy established by rename
 */
void dxqCallbacks::chatBuddyRenamed(PURPLE::Conversation *conv, std::string buddy, PURPLE::ChatBuddy nbuddy)
{
    emit signalChatBuddyRenamed(conv, dxutils::fromStdString(buddy), nbuddy);
}

/** Chat topic changed
 * @param who  topic changer
 * @param topic  new topic
 */
void dxqCallbacks::chatTopicChanged(PURPLE::Conversation *conv, std::string who, std::string topic)
{
    emit signalChatTopicChanged(conv, dxutils::fromStdString(who), dxutils::fromStdString(topic));
}

/** Buddy typing on IM
 * @param typing =true typing, =false stoped typing
 */
void dxqCallbacks::buddyTyping(PURPLE::Conversation *conv, bool typing)
{
    if(typing)
        emit signalBuddyTyping(conv, STARTED);
    else
        emit signalBuddyTyping(conv, STOPED);
}

/** New FileTransfer */
void dxqCallbacks::newFileTransfer(PURPLE::FileTransfer *ft)
{
    emit signalNewFT(ft);
}

/** FileTransfer finished */
void dxqCallbacks::fileTransferFinished(PURPLE::FileTransfer *ft)
{
    emit signalFinishedFT(ft);
}

/** FileTransfer started */
void dxqCallbacks::fileTransferStarted(PURPLE::FileTransfer *ft)
{
    emit signalStartedFT(ft);
}

/** FileTransfer progress */
void dxqCallbacks::fileTransferProgress(PURPLE::FileTransfer *ft, double percent)
{
    Q_UNUSED(percent)
    emit signalProgressFT(ft);
}

/** FileTransfer canceled */
void dxqCallbacks::fileTransferCanceled(PURPLE::FileTransfer *ft)
{
    emit signalCanceledFT(ft);
}

/** A new list was created */
void dxqCallbacks::roomlistCreated(PURPLE::RoomList *rlist)
{
    emit signalRoomlistCreated(rlist);
}

/** Sets the columns of RoomList */
void dxqCallbacks::roomlistColumns(PURPLE::RoomList *rlist, std::vector<std::string> columns)
{
    QVector<QString> ncolumns;
    std::vector<std::string>::iterator it;
    for(it=columns.begin(); it!=columns.end(); it++)
        ncolumns.append(dxutils::fromStdString((*it)));
    emit signalRoomlistColumns(rlist, ncolumns);
}

/** Add a room to the list */
void dxqCallbacks::roomlistAddRoom(PURPLE::RoomList *rlist, std::vector<std::string> fields)
{
    if(fields.size())
    {
        QVector<QString> nfields;
        std::vector<std::string>::iterator it;
        for(it=fields.begin(); it!=fields.end(); it++)
            nfields.append(dxutils::fromStdString((*it)));
        emit signalRoomlistAddRoom(rlist, nfields);
    }
}

/** Listing RoomList finished */
void dxqCallbacks::roomlistFinished(PURPLE::RoomList *rlist)
{
    emit signalRoomlistFinished(rlist);
}

/** Notification message to the user
 * @param type  notification type
 * @param title  title of the message
 * @param primary  main point of the message
 * @param secondary  secondary information
*/
void dxqCallbacks::notifyMessage(PurpleNotifyMsgType type, std::string title, std::string primary, std::string secondary)
{
    QPixmap icon;
    switch(type){
        case PURPLE_NOTIFY_MSG_ERROR:
            icon = QPixmap(DXIM_DATADIR "/icons/errorbig.png");
            break;
        case PURPLE_NOTIFY_MSG_INFO:
            icon = QPixmap(DXIM_DATADIR "/icons/infobig.png");
            break;
        case PURPLE_NOTIFY_MSG_WARNING:
            icon = QPixmap(DXIM_DATADIR "/icons/warningbig.png");
            break;
    };
    emit signalNotifyMessage(icon, dxutils::fromStdString(title), QString("%1\n%2").arg(dxutils::fromStdString(primary)).arg(dxutils::fromStdString(secondary)));
}

/** Email notification to the user
 * @param subject  subject of the email
 * @param from  from address
 * @param to  estination address
 * @param url  URL where the message can be read
*/
void dxqCallbacks::notifyEmail(std::string subject, std::string from, std::string to, std::string url)
{
    emit signalNotifyEmail(_("From: %1\nTo: %2\nSubject: %3").arg(dxutils::fromStdString(from)).arg(dxutils::fromStdString(to)).arg(dxutils::fromStdString(subject)), dxutils::fromStdString(url));
}

/** Notification with formatted text
 * @param title  title of the message
 * @param primary  main point of the message
 * @param secondary  secondary information
 * @param text  formatted text
*/
void dxqCallbacks::notifyFormatted(std::string title, std::string primary, std::string secondary, std::string formated)
{
    emit signalNotifyMessage(QPixmap(DXIM_DATADIR "/icons/infobig.png"), dxutils::fromStdString(title), QString("%1\n%2\n%3").arg(dxutils::fromStdString(primary)).arg(dxutils::fromStdString(secondary)).arg(dxutils::fromStdString(formated).replace("<br>", "\n")));
}

/** Notification information about buddy
 * @param account   requested account
 * @param buddy     Buddy which about
 * @param info      map<label,value>
*/
void dxqCallbacks::notifyUserinfo(PURPLE::Account account, std::string buddy, std::vector<PURPLE::Notify::UserInfo> info)
{
    QString text, value;
    std::vector<PURPLE::Notify::UserInfo>::iterator it;
    for(it=info.begin(); it!=info.end(); it++)
    {
        value = dxutils::fromStdString(PURPLE::markupHtml((*it).getValue()));
        if(value.startsWith("<img id='"))
        {
            int id = value.section("'",1,1).toInt();
            value = QString("<img src='%1'>").arg(dxutils::fromStdString(PURPLE::imagePathByID(id)));
        }
        if(value.isEmpty()) text.append(QString("<b>%1</b>:<br />").arg(dxutils::fromStdString((*it).getLabel())));
        else text.append(QString("<b>%1</b>: %2<br />").arg(dxutils::fromStdString((*it).getLabel())).arg(value));
    }
    emit signalNotifyUserinfo(account, dxutils::fromStdString(buddy), text);
}

/** Opens a URI or somehow presents it to the user
 * @param uri    The URI to display or go to.
*/
void dxqCallbacks::notifyURI(std::string uri)
{
    emit signalNotifyURI(dxutils::fromStdString(uri));
}

/** Play sound for given event */
void dxqCallbacks::playSound(PurpleSoundEventID event)
{
    emit signalPlaySound(event);
}

/** New InputRequest */
void dxqCallbacks::newInputRequest(PURPLE::InputRequest *request)
{
    emit signalNewInputRequest(request);
}

/** New ChoiceRequest */
void dxqCallbacks::newChoiceRequest(PURPLE::ChoiceRequest *request)
{
    emit signalNewChoiceRequest(request);
}

/** New ActionRequest */
void dxqCallbacks::newActionRequest(PURPLE::ActionRequest *request)
{
    emit signalNewActionRequest(request);
}

/** New FileRequest */
void dxqCallbacks::newFileRequest(PURPLE::FileRequest *request)
{
    emit signalNewFileRequest(request);
}

/** New FieldsRequest */
void dxqCallbacks::newFieldsRequest(PURPLE::FieldsRequest *request)
{
    emit signalNewFieldsRequest(request);
}

/** Close given request (ex. dialog) */
void dxqCallbacks::closeRequest(PURPLE::BasicRequest *request)
{
    Q_UNUSED(request)
}

