/*
 *      accounteditordialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QDialogButtonBox>
#include <QComboBox>
#include <QFormLayout>
#include <QFileDialog>
#include "accounteditordialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

/* Constructor - Modify existing account */
AccountEditorDialog::AccountEditorDialog(PURPLE::Account account, QWidget *parent) :
    QDialog(parent)
{
    this->m_editing = true;
    this->m_protocol = account.getProtocol();
    this->m_account = account;
    this->m_options = account.getOptions();
    this->m_gtalk = false;
    this->m_iconPath = dxutils::fromStdString(account.getBuddyIcon());
    initializeEditor();
    setWindowTitle(_("Modify account"));
}

/* Constructor - Create new account */
AccountEditorDialog::AccountEditorDialog(PURPLE::Protocol protocol, bool gtalk, QWidget *parent) :
    QDialog(parent)
{
    this->m_editing = false;
    this->m_protocol = protocol;
    this->m_account = PURPLE::Account();
    this->m_options = protocol.getOptions();
    this->m_gtalk = gtalk;
    this->m_iconPath = "";
    initializeEditor();
    setWindowTitle(_("Create new account"));
}

void AccountEditorDialog::initializeEditor()
{
    QTabWidget *tabbook = new QTabWidget(this);
    createAccountTab(tabbook);
    createProtocolTab(tabbook);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addWidget(tabbook);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
}

void AccountEditorDialog::createAccountTab(QTabWidget *tabs)
{
    QWidget *tab = new QWidget;
    QFormLayout *matrix = new QFormLayout;
    if(m_gtalk)
    {
        matrix->addRow(new QLabel(_("Protocol")), new QLabel(QString("<img src=\"%1\"> Google Talk").arg(dxutils::protocolToIcon("gtalk"))));
    }
    else
    {
        matrix->addRow(new QLabel(_("Protocol")), new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(m_protocol.getPurpleID()))).arg(dxutils::fromStdString(m_protocol.getName()))));
    }
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    QLineEdit *edit = NULL;
    QString username = "";
    if(m_editing) username = dxutils::fromStdString(m_account.getUsername());
    if(splits.size())
    {
        edit = new QLineEdit;
        matrix->addRow(new QLabel(_("User name")), edit);
        if(m_editing)
        {
            edit->setText(username.section(splits[0].getSeparator(), 0, 0));
            username = username.section(splits[0].getSeparator(), 1);
            if(m_account.isEnabled()) edit->setEnabled(false);
        }
        m_usernameFields.append(edit);
        for(size_t i=0; i<splits.size(); i++)
        {
            edit = new QLineEdit;
            matrix->addRow(new QLabel(dxutils::fromStdString(splits[i].getText())), edit);
            if(m_editing)
            {
                if(i+1 == splits.size())
                {
                    edit->setText(username);
                }
                else
                {
                    edit->setText(username.section(splits[i+1].getSeparator(), 0, 0));
                    username = username.section(splits[i+1].getSeparator(), 1);
                }
                if(m_account.isEnabled()) edit->setEnabled(false);
            }
            m_usernameFields.append(edit);
        }
        if(!m_editing && m_gtalk)
        {
            m_usernameFields[1]->setText("gmail.com");
        }
    }
    else
    {
        edit = new QLineEdit;
        matrix->addRow(new QLabel(_("User name")), edit);
        if(m_editing)
        {
            edit->setText(dxutils::fromStdString(m_account.getUsername()));
            if(m_account.isEnabled()) edit->setEnabled(false);
        }
        m_usernameFields.append(edit);
    }
    m_password = new QLineEdit;
    m_password->setEchoMode(QLineEdit::PasswordEchoOnEdit);
    matrix->addRow(new QLabel(_("Password")), m_password);
    if(m_editing) m_password->setText(dxutils::fromStdString(m_account.getPassword()));
    m_rememberPassword = new QCheckBox(_("Remember password"));
    if(!m_editing || m_account.getRememberPassword()) m_rememberPassword->setChecked(true);
    matrix->addWidget(m_rememberPassword);
    m_alias = new QLineEdit;
    matrix->addRow(new QLabel(_("Alias")), m_alias);
    if(m_editing) m_alias->setText(dxutils::fromStdString(m_account.getAlias()));
    m_checkEmail = new QCheckBox(_("Check new mails"));
    if(m_editing) m_checkEmail->setChecked(m_account.getCheckMail());
    if(!m_protocol.hasCheckMail()) m_checkEmail->hide();
    matrix->addWidget(m_checkEmail);
    m_iconLabel = new QLabel(_("Icon"));
    m_iconBox = new QHBoxLayout;
    m_iconBtn = new QPushButton;
    m_iconBtn->setIconSize(QSize(32,32));
    connect(m_iconBtn, SIGNAL(clicked()), this, SLOT(slotIconPath()));
    m_removeIcon = new QPushButton(_("Remove"));
    connect(m_removeIcon, SIGNAL(clicked()), this, SLOT(slotRemoveIcon()));
    m_iconBox->addWidget(m_iconBtn);
    m_iconBox->addWidget(m_removeIcon);
    if(m_protocol.hasBuddyIcon())
    {
        if(!m_editing || m_account.getBuddyIcon().empty())
        {
            m_iconBtn->setText(_("Icon isn't set"));
            m_removeIcon->hide();
        }
        else
        {
            m_iconPath = dxutils::fromStdString(m_account.getBuddyIcon());
            m_iconBtn->setIcon(QPixmap(dxutils::fromStdString(m_account.getBuddyIcon())));
        }
    }
    else
    {
        m_iconLabel->hide();
        m_iconBtn->hide();
        m_removeIcon->hide();
    }
    m_iconBox->addStretch(1);
    matrix->addRow(m_iconLabel, m_iconBox);
    m_registerAccount = new QCheckBox(_("Register account"));
    m_registerAccount->setChecked(false);
    if(m_editing) m_registerAccount->hide();
    matrix->addWidget(m_registerAccount);
    QVBoxLayout *main = new QVBoxLayout;
    main->addLayout(matrix);
    main->addStretch(1);
    tab->setLayout(main);
    tabs->addTab(tab, _("&Account settings"));
}

void AccountEditorDialog::createProtocolTab(QTabWidget* tabs)
{
    QWidget *tab = new QWidget;
    QFormLayout *matrix = new QFormLayout;
    QLineEdit *text;
    QComboBox *combo;
    QCheckBox *check;
    FOREACH(PURPLE::Protocol::Options, m_options, it)
    {
        PURPLE::Protocol::Option& option = *it;
        std::map<std::string, std::string> values = option.getStringList();
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                text = new QLineEdit;
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                matrix->addRow(new QLabel(dxutils::fromStdString(option.getText())), text);
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                combo = new QComboBox;
                std::map<std::string, std::string>::iterator value = values.begin();
                int j=0;
                for(; value!=values.end(); ++value, j++)
                {
                    combo->addItem(dxutils::fromStdString((*value).first));
                    if(!option.getStringValue().compare((*value).second))
                        combo->setCurrentIndex(j);
                }
                m_optionsFields.append(combo);
                matrix->addRow(new QLabel(dxutils::fromStdString(option.getText())), combo);
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                text = new QLineEdit;
                text->setValidator(new QIntValidator(text));
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                matrix->addRow(new QLabel(dxutils::fromStdString(option.getText())), text);
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                check = new QCheckBox;
                check->setChecked(option.getBoolValue());
                m_optionsFields.append(check);
                matrix->addRow(new QLabel(dxutils::fromStdString(option.getText())), check);
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                m_optionsFields.append(NULL);
                break;
        }
    }
    QVBoxLayout *main = new QVBoxLayout;
    main->addLayout(matrix);
    main->addStretch(1);
    tab->setLayout(main);
    tabs->addTab(tab, _("&Protocol options"));
}

std::string AccountEditorDialog::getStrlistValue(std::map<std::string, std::string> map, std::string key)
{
    std::string result;
    std::map<std::string, std::string>::iterator it = map.find(key);
    if(it!=map.end())
        result = (*it).second;
    return result;
}

void AccountEditorDialog::fillProtocolOptions()
{
    PURPLE::Protocol::Options::iterator it;
    quint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Protocol::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Protocol::Option::STR:
            {
                option.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_optionsFields[i])->text()));
                break;
            }
            case PURPLE::Protocol::Option::STRLIST:
            {
                QComboBox *combo = static_cast<QComboBox*>(m_optionsFields[i]);
                option.setValue(getStrlistValue(option.getStringList(), dxutils::toStdString(combo->currentText())));
                break;
            }
            case PURPLE::Protocol::Option::INT:
            {
                option.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_optionsFields[i])->text()));
                break;
            }
            case PURPLE::Protocol::Option::BOOL:
            {
                option.setValue(static_cast<QCheckBox*>(m_optionsFields[i])->isChecked() ? "true" : "false");
                break;
            }
            case PURPLE::Protocol::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

std::string AccountEditorDialog::joinUsername()
{
    std::vector<PURPLE::Protocol::UsernameSplit> splits = m_protocol.getUsernameSplit();
    QString username = m_usernameFields[0]->text();
    if(splits.size())
    {
        for(size_t i=0; i<splits.size(); i++)
        {
            username.append(splits[i].getSeparator());
            username.append(m_usernameFields[i+1]->text());
        }
    }
    return dxutils::toStdString(username);
}

/* Fills new Account */
void AccountEditorDialog::fillAccount(NewAccount *account)
{
    account->protocol = m_protocol;
    account->username = joinUsername();
    account->password = dxutils::toStdString(m_password->text());
    account->registerAccount = m_registerAccount->isChecked();
    account->checkEmail = m_checkEmail->isChecked();
    account->iconPath = dxutils::toStdString(m_iconPath);
    account->rememberPassword = m_rememberPassword->isChecked();
    account->alias = dxutils::toStdString(m_alias->text());
    fillProtocolOptions();
    account->options = m_options;
}

/* Handles click on m_iconBtn */
void AccountEditorDialog::slotIconPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString file = QFileDialog::getOpenFileName(this,
                                _("Select buddy icon"),
                                "",
                                "*.png *.jpg",
                                NULL,
                                options);
    if(!file.isEmpty())
    {
        m_iconPath = file;
        m_iconBtn->setText("");
        m_iconBtn->setIcon(QPixmap(file));
        m_removeIcon->show();
    }
}

/* Handles click on m_removeIcon */
void AccountEditorDialog::slotRemoveIcon()
{
    m_iconPath = "";
    m_iconBtn->setText(_("Icon isn't set"));
    m_iconBtn->setIcon(QIcon());
    m_removeIcon->hide();
}
