/*
 *      chatdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QDialogButtonBox>
#include <QLabel>
#include "defs.h"
#include "dxutils.h"
#include "chatdialog.h"
#include "config.h"
#include "i18n.h"

ChatDialog::ChatDialog(PURPLE::Account account, QWidget *parent) :
    QDialog(parent)
{
    m_account = account;
    m_chat = PURPLE::Chat();
    m_modifing = false;
    createWidgets();
    setWindowTitle(_("Add chat"));
}

ChatDialog::ChatDialog(PURPLE::Chat chat, QWidget *parent) :
    QDialog(parent)
{
    m_account = chat.getAccount();
    m_chat = chat;
    m_modifing = true;
    createWidgets();
    setWindowTitle(_("Modify chat"));
}

void ChatDialog::createWidgets()
{
    m_grid = new QGridLayout;
    m_grid->addWidget(new QLabel(_("Account:")), 0, 0);
    m_grid->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(m_account.getProtocol().getPurpleID()))).arg(dxutils::fromStdString(m_account.getUsername()))), 0, 1);
    m_grid->addWidget(new QLabel(_("Group:")), 1, 0);
    m_groups = new QComboBox;
    std::vector<std::string> v = m_account.getAllGroups();
    bool chats = false;
    FOREACH(std::vector<std::string>, v, it)
    {
        if(dxutils::fromStdString(*it) == _("Chats"))
            chats = true;
        m_groups->addItem(dxutils::fromStdString(*it));
    }
    if(!chats) m_groups->insertItem(0, _("Chats"));
    m_groups->setEditable(true);
    m_groups->setCurrentIndex(0);
    m_grid->addWidget(m_groups, 1, 1);
    m_grid->addWidget(new QLabel(_("Alias:")), 2, 0);
    m_alias = new QLineEdit;
    if(m_modifing) m_alias->setText(dxutils::fromStdString(m_chat.getAlias()));
    m_grid->addWidget(m_alias, 2, 1);
    std::vector<PURPLE::Account::ChatEntry> fields;
    if(m_modifing) fields = m_chat.getChatParameters();
    else fields = m_account.getChatParameters();
    QLabel *label;
    QLineEdit *text;
    QSpinBox *spinner;
    int i = 3;
    FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
    {
        PURPLE::Account::ChatEntry entry = *it;
        label = new QLabel(dxutils::normalizeLabelString(entry.getLabel(), '_'));
        m_grid->addWidget(label, i, 0);
        if(entry.getIsInt())
        {
            spinner = new QSpinBox();
            spinner->setMinimum(entry.getMin());
            spinner->setMaximum(entry.getMax());
            m_grid->addWidget(spinner, i, 1);
            m_fields.append(spinner);
        }
        else
        {
            text = new QLineEdit(dxutils::fromStdString(entry.getValue()));
            if(entry.getSecret()) text->setEchoMode(QLineEdit::PasswordEchoOnEdit);
            m_grid->addWidget(text, i, 1);
            m_fields.append(text);
        }
        i++;
    }
    m_autoJoin = new QCheckBox(_("Autojoin chat"));
    if(m_modifing) m_autoJoin->setChecked(m_chat.getAutojoin());
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addLayout(m_grid);
    mainFrame->addWidget(m_autoJoin);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
}

void ChatDialog::accept()
{
    std::string alias;
    if(m_alias->text().isEmpty()) alias = dxutils::toStdString(static_cast<QLineEdit*>(m_fields[0])->text());
    else alias = dxutils::toStdString(m_alias->text());
    //chat name empty => return
    if(alias.empty()) return;
    std::string group = dxutils::toStdString(m_groups->currentText());
    if(group.empty()) group = dxutils::toStdString(_("Chats"));
    if(m_modifing)
    {
        m_chat.setChatParameters(fillChatEntryFields());
        m_chat.setAlias(alias);
        m_chat.setAutojoin(m_autoJoin->isChecked());
        if(m_chat.getGroupName().compare(group))
        {
            m_account.moveChat(m_chat, group);
        }
    }
    else
    {
        m_account.addChat(fillChatEntryFields(), alias, group, m_autoJoin->isChecked());
    }
    QDialog::accept();
}

std::vector<PURPLE::Account::ChatEntry> ChatDialog::fillChatEntryFields()
{
    std::vector<PURPLE::Account::ChatEntry> fields = m_account.getChatParameters();
    std::vector<PURPLE::Account::ChatEntry>::iterator it;
    uint i;
    for(it=fields.begin(), i=0; it!=fields.end(); it++,i++)
    {
        PURPLE::Account::ChatEntry& entry = *it;
        if(entry.getIsInt())
        {
            QString str;
            str.setNum(static_cast<QSpinBox*>(m_fields[i])->value());
            entry.setValue(dxutils::toStdString(str));
        }
        else
        {
            entry.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_fields[i])->text()));
        }
    }
    return fields;
}

