/*
 *      configdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QDialogButtonBox>
#include <QCheckBox>
#include <QDir>
#include <QDesktopServices>
#include <QFileDialog>
#include <QGroupBox>
#include <QFormLayout>
#include <QMessageBox>
#include <QFontDialog>
#include <QStandardPaths>
#include "configdialog.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"

#define HEADER "# dxim smiley settings\n# path is relative to file\n# Example: /home/xxx/smile.png=:)\n"

SmileyDialog::SmileyDialog(const QString &title, const QString &smiley, const QString &path, QWidget *parent)
    : QDialog(parent)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QFormLayout *form = new QFormLayout;
    m_smiley = new QLineEdit;
    m_smiley->setText(smiley);
    form->addRow(_("Smiley:"), m_smiley);
    QHBoxLayout *pathl = new QHBoxLayout;
    m_path = new QLabel(path);
    m_path->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    m_selectPath = new QPushButton("...");
    connect(m_selectPath, SIGNAL(clicked()), this, SLOT(slotPath()));
    pathl->addWidget(m_path);
    pathl->addWidget(m_selectPath);
    form->addRow(_("Path:"), pathl);
    m_preview = new QLabel;
    m_preview->setPixmap(QPixmap(path));
    form->addRow(_("Preview:"), m_preview);
    mainLayout->addLayout(form);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(title);
}

void SmileyDialog::slotPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString file = QFileDialog::getOpenFileName(this,
                                _("Select file"),
                                m_path->text(),
                                "",
                                NULL,
                                options);
    if(!file.isEmpty())
    {
        m_path->setText(file);
        m_preview->setPixmap(QPixmap(file));
    }
}

ConfigDialog::ConfigDialog(QWidget *parent) :
    QDialog(parent)
{
    m_showImportwarning = true;
    QTabWidget *tabbook = new QTabWidget(this);
    createGeneralTab(tabbook);
    createIconsTab(tabbook);
    createLookTab(tabbook);
    createImtextTab(tabbook);
    createNotificationTab(tabbook);
    createSmileysTab(tabbook);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabbook);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(_("Preferences"));
}

void ConfigDialog::accept()
{
    //GeneralTab
    DXPREFERENCES.m_oneInstance = m_oneInstance->isChecked();
    DXPREFERENCES.m_useAway = m_useAway->isChecked();
    DXPREFERENCES.m_awayTimeout = m_awayTimeout->value();
    DXPREFERENCES.m_minToTray = m_minToTray->isChecked();
    DXPREFERENCES.m_logging = m_logging->isChecked();
    DXPREFERENCES.m_logPath = m_logPath->text();
    DXPREFERENCES.m_logLoadCurrent = m_logLoadCurrent->isChecked();
    DXPREFERENCES.m_showRosterTree = m_showRosterTree->isChecked();
    DXPREFERENCES.m_receiptsMark = m_receiptsMark->isChecked();
    //IconsTab
    DXPREFERENCES.m_themeStatusPath = m_themesStatusList->currentItem()->text();
    DXPREFERENCES.m_themesStatusList.clear();
    for(int i=0; i<m_themesStatusList->count(); i++)
        DXPREFERENCES.m_themesStatusList.append(m_themesStatusList->item(i)->text()+";");
    DXPREFERENCES.m_themeRolePath = m_themesRoleList->currentItem()->text();
    DXPREFERENCES.m_themesRoleList.clear();
    for(int i=0; i<m_themesRoleList->count(); i++)
        DXPREFERENCES.m_themesRoleList.append(m_themesRoleList->item(i)->text()+";");
    //LookTab
    DXPREFERENCES.m_unreadtabColor = m_unreadtabColor->getColor();
    DXPREFERENCES.m_hltabColor = m_hltabColor->getColor();
    DXPREFERENCES.m_tabPosition = m_tabPosition->currentIndex();
    //ImTextTab
    DXPREFERENCES.m_textColor = m_textColor->getColor();
    DXPREFERENCES.m_backroundColor = m_backroundColor->getColor();
    DXPREFERENCES.m_userColor = m_userColor->getColor();
    DXPREFERENCES.m_actionColor = m_actionColor->getColor();
    DXPREFERENCES.m_errorColor = m_errorColor->getColor();
    DXPREFERENCES.m_hilightColor = m_hilightColor->getColor();
    DXPREFERENCES.m_linkColor = m_linkColor->getColor();
    DXPREFERENCES.m_mymsgColor = m_mymsgColor->getColor();
    DXPREFERENCES.m_logColor = m_logColor->getColor();
    QFont font;
    font.fromString(m_imFont->text());
    DXPREFERENCES.m_imFont = font;
    DXPREFERENCES.m_sameCmd = m_sameCmd->isChecked();
    DXPREFERENCES.m_sameList = m_sameList->isChecked();
    DXPREFERENCES.m_nickCompletionChar = m_nickCompletionChar->text().left(1);
    DXPREFERENCES.m_coloredNick = m_coloredNick->isChecked();
    //NotificationTab
    DXPREFERENCES.m_sounds = m_sounds->isChecked();
    DXPREFERENCES.m_soundConnect = m_soundConnect->isChecked();
    DXPREFERENCES.m_pathConnect = m_pathConnect->text();
    DXPREFERENCES.m_soundDisconnect = m_soundDisconnect->isChecked();
    DXPREFERENCES.m_pathDisconnect = m_pathDisconnect->text();
    DXPREFERENCES.m_soundMessage = m_soundMessage->isChecked();
    DXPREFERENCES.m_pathMessage = m_pathMessage->text();
    DXPREFERENCES.m_notify = m_notify->isChecked();
    if(!dxutils::instance().haveNotify())
        DXPREFERENCES.m_notifyPosition = m_notifyPosition->currentIndex();
    DXPREFERENCES.m_notifyAccountConnect = m_notifyAccountConnect->isChecked();
    DXPREFERENCES.m_notifyAccountDisconnect = m_notifyAccountDisconnect->isChecked();
    DXPREFERENCES.m_notifyBuddyConnect = m_notifyBuddyConnect->isChecked();
    DXPREFERENCES.m_notifyBuddyDisconnect = m_notifyBuddyDisconnect->isChecked();
    DXPREFERENCES.m_notifyMessage = m_notifyMessage->isChecked();
    DXPREFERENCES.m_notifyFiletransfer = m_notifyFiletransfer->isChecked();
    DXPREFERENCES.m_raiseWindow = m_raiseWindow->isChecked();
    DXPREFERENCES.m_notifyMessageTray = m_notifyMessageTray->isChecked();
    //SmileysTab
    DXPREFERENCES.m_useSmileys = m_useSmileys->isChecked();
    DXPREFERENCES.m_smileysMap = m_smileysMap;
    DXPREFERENCES.save();
    QDialog::accept();
}

void ConfigDialog::createGeneralTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    m_oneInstance = new QCheckBox(_("Run one instance"));
    m_oneInstance->setChecked(DXPREFERENCES.m_oneInstance);
    main->addWidget(m_oneInstance);
    QHBoxLayout *alayout = new QHBoxLayout;
    m_useAway = new QCheckBox(_("Time for status \"Away\" in minutes"));
    m_useAway->setChecked(DXPREFERENCES.m_useAway);
    connect(m_useAway, SIGNAL(stateChanged(int)), this, SLOT(slotUseAway()));
    alayout->addWidget(m_useAway);
    m_awayTimeout = new QSpinBox;
    m_awayTimeout->setRange(1, 500);
    m_awayTimeout->setValue(DXPREFERENCES.m_awayTimeout);
    alayout->addWidget(m_awayTimeout);
    main->addLayout(alayout);
    m_minToTray = new QCheckBox(_("Start application minimized to tray"));
    m_minToTray->setChecked(DXPREFERENCES.m_minToTray);
    main->addWidget(m_minToTray);
    m_logging = new QCheckBox(_("Logging chats"));
    m_logging->setChecked(DXPREFERENCES.m_logging);
    connect(m_logging, SIGNAL(stateChanged(int)), this, SLOT(slotLogging()));
    main->addWidget(m_logging);
    QHBoxLayout *loglayout = new QHBoxLayout;
    loglayout->addWidget(new QLabel(_("Log path")));
    m_logPath = new QLabel;
    if(!QDir(DXPREFERENCES.m_logPath).exists())
    {
        m_logPath->setText(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
        m_logPath->setToolTip(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0));
    }
    else
    {
        m_logPath->setText(DXPREFERENCES.m_logPath);
        m_logPath->setToolTip(DXPREFERENCES.m_logPath);
    }
    m_logPath->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    loglayout->addWidget(m_logPath);
    m_setLogPath = new QPushButton("...");
    connect(m_setLogPath, SIGNAL(clicked()), this, SLOT(slotLogPath()));
    loglayout->addWidget(m_setLogPath);
    loglayout->addStretch(1);
    main->addLayout(loglayout);
    m_logLoadCurrent = new QCheckBox(_("Load current log for opened IM"));
    if(!DXPREFERENCES.m_logging) m_logLoadCurrent->setChecked(false);
    else m_logLoadCurrent->setChecked(DXPREFERENCES.m_logLoadCurrent);
    main->addWidget(m_logLoadCurrent);
    m_showRosterTree = new QCheckBox(_("Show accounts and groups in friends list"));
    m_showRosterTree->setChecked(DXPREFERENCES.m_showRosterTree);
    main->addWidget(m_showRosterTree);
    m_receiptsMark = new QCheckBox(_("Use delivery receipts mark on XMPP protocol"));
    m_receiptsMark->setChecked(DXPREFERENCES.m_receiptsMark);
    main->addWidget(m_receiptsMark);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&General"));
}

void ConfigDialog::slotUseAway()
{
    if(m_useAway->isChecked())
    {
        m_awayTimeout->setEnabled(true);
    }
    else
    {
        m_awayTimeout->setEnabled(false);
    }
}

void ConfigDialog::slotLogging()
{
    if(m_logging->isChecked())
    {
        m_setLogPath->setEnabled(true);
        m_logLoadCurrent->setEnabled(true);
    }
    else
    {
        m_setLogPath->setEnabled(false);
        m_logLoadCurrent->setChecked(false);
        m_logLoadCurrent->setEnabled(false);
    }
}

void ConfigDialog::slotLogPath()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select log directory"),
                                m_logPath->text(),
                                options);
    if(!directory.isEmpty())
    {
        m_logPath->setText(directory);
        m_logPath->setToolTip(directory);
    }
}

void ConfigDialog::createIconsTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QGroupBox *statusGroup = new QGroupBox(_("Status icons themes"));
    QHBoxLayout *statusLayout = new QHBoxLayout;
    QVBoxLayout *statusListLayout = new QVBoxLayout;
    m_themesStatusList = new QListWidget;
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    m_themesStatusList->addItems(DXPREFERENCES.m_themesStatusList.split(';', QString::SkipEmptyParts));
#else
    m_themesStatusList->addItems(DXPREFERENCES.m_themesStatusList.split(';', Qt::SkipEmptyParts));
#endif
    for(int i=0; i<m_themesStatusList->count(); i++)
    {
        if(DXPREFERENCES.m_themeStatusPath.compare(m_themesStatusList->item(i)->text())==0)
            m_themesStatusList->setCurrentItem(m_themesStatusList->item(i));
    }
    connect(m_themesStatusList, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), this, SLOT(slotThemeStatus(QListWidgetItem*)));
    statusListLayout->addWidget(m_themesStatusList);
    QHBoxLayout *statusIconsLayout = new QHBoxLayout;
    m_iconAway = new QLabel();
    m_iconAway->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/away.png"));
    m_iconAway->setToolTip(_("Away"));
    statusIconsLayout->addWidget(m_iconAway);
    m_iconIdle = new QLabel();
    m_iconIdle->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/idle.png"));
    m_iconIdle->setToolTip(_("Idle"));
    statusIconsLayout->addWidget(m_iconIdle);
    m_iconDnd = new QLabel();
    m_iconDnd->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/dnd.png"));
    m_iconDnd->setToolTip(_("Busy"));
    statusIconsLayout->addWidget(m_iconDnd);
    m_iconInvisible = new QLabel();
    m_iconInvisible->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/invisible.png"));
    m_iconInvisible->setToolTip(_("Invisible"));
    statusIconsLayout->addWidget(m_iconInvisible);
    m_iconOffline = new QLabel();
    m_iconOffline->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/offline.png"));
    m_iconOffline->setToolTip(_("Offline"));
    statusIconsLayout->addWidget(m_iconOffline);
    m_iconOnline = new QLabel();
    m_iconOnline->setPixmap(QPixmap(DXPREFERENCES.m_themeStatusPath+"/online.png"));
    m_iconOnline->setToolTip(_("Online"));
    statusIconsLayout->addWidget(m_iconOnline);
    statusIconsLayout->addStretch(1);
    statusListLayout->addLayout(statusIconsLayout);
    statusLayout->addLayout(statusListLayout);
    QVBoxLayout *statusButtonsLayout = new QVBoxLayout;
    m_themeStatusAdd = new QPushButton(_("Add"));
    connect(m_themeStatusAdd, SIGNAL(clicked()), this, SLOT(slotAddStatusTheme()));
    statusButtonsLayout->addWidget(m_themeStatusAdd);
    m_themeStatusDelete = new QPushButton(_("Delete"));
    connect(m_themeStatusDelete, SIGNAL(clicked()), this, SLOT(slotDeleteStatusTheme()));
    statusButtonsLayout->addWidget(m_themeStatusDelete);
    if(m_themesStatusList->count()>1) m_themeStatusDelete->setEnabled(true);
    else m_themeStatusDelete->setEnabled(false);
    statusButtonsLayout->addStretch(1);
    statusLayout->addLayout(statusButtonsLayout);
    statusGroup->setLayout(statusLayout);
    QGroupBox *rolesGroup = new QGroupBox(_("Role icons themes"));
    QHBoxLayout *rolesLayout = new QHBoxLayout;
    QVBoxLayout *rolesListLayout = new QVBoxLayout;
    m_themesRoleList = new QListWidget;
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    m_themesRoleList->addItems(DXPREFERENCES.m_themesRoleList.split(';', QString::SkipEmptyParts));
#else
    m_themesRoleList->addItems(DXPREFERENCES.m_themesRoleList.split(';', Qt::SkipEmptyParts));
#endif
    for(int i=0; i<m_themesRoleList->count(); i++)
    {
        if(DXPREFERENCES.m_themeRolePath.compare(m_themesRoleList->item(i)->text())==0)
            m_themesRoleList->setCurrentItem(m_themesRoleList->item(i));
    }
    connect(m_themesRoleList, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), this, SLOT(slotThemeRoles(QListWidgetItem*)));
    rolesListLayout->addWidget(m_themesRoleList);
    QHBoxLayout *rolesIconsLayout = new QHBoxLayout;
    m_iconFounder = new QLabel();
    m_iconFounder->setPixmap(QPixmap(DXPREFERENCES.m_themeRolePath+"/founder.png"));
    m_iconFounder->setToolTip(_("Founder"));
    rolesIconsLayout->addWidget(m_iconFounder);
    m_iconHalfop = new QLabel();
    m_iconHalfop->setPixmap(QPixmap(DXPREFERENCES.m_themeRolePath+"/halfop.png"));
    m_iconHalfop->setToolTip(_("Halfop"));
    rolesIconsLayout->addWidget(m_iconHalfop);
    m_iconNormal = new QLabel();
    m_iconNormal->setPixmap(QPixmap(DXPREFERENCES.m_themeRolePath+"/normal.png"));
    m_iconNormal->setToolTip(_("Normal"));
    rolesIconsLayout->addWidget(m_iconNormal);
    m_iconOp = new QLabel();
    m_iconOp->setPixmap(QPixmap(DXPREFERENCES.m_themeRolePath+"/op.png"));
    m_iconOp->setToolTip(_("Op"));
    rolesIconsLayout->addWidget(m_iconOp);
    m_iconVoice = new QLabel();
    m_iconVoice->setPixmap(QPixmap(DXPREFERENCES.m_themeRolePath+"/voice.png"));
    m_iconVoice->setToolTip(_("Voice"));
    rolesIconsLayout->addWidget(m_iconVoice);
    rolesIconsLayout->addStretch(1);
    rolesListLayout->addLayout(rolesIconsLayout);
    QVBoxLayout *rolesButtonsLayout = new QVBoxLayout;
    m_themeRoleAdd = new QPushButton(_("Add"));
    connect(m_themeRoleAdd, SIGNAL(clicked()), this, SLOT(slotAddRoleTheme()));
    rolesButtonsLayout->addWidget(m_themeRoleAdd);
    m_themeRoleDelete = new QPushButton(_("Delete"));
    connect(m_themeRoleDelete, SIGNAL(clicked()), this, SLOT(slotDeleteRoleTheme()));
    rolesButtonsLayout->addWidget(m_themeRoleDelete);
    if(m_themesRoleList->count()>1) m_themeRoleDelete->setEnabled(true);
    else m_themeRoleDelete->setEnabled(false);
    rolesButtonsLayout->addStretch(1);
    rolesLayout->addLayout(rolesListLayout);
    rolesLayout->addLayout(rolesButtonsLayout);
    rolesGroup->setLayout(rolesLayout);
    main->addWidget(statusGroup);
    main->addWidget(rolesGroup);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("I&cons"));
}

void ConfigDialog::slotThemeStatus(QListWidgetItem *current)
{
    m_iconAway->setPixmap(QPixmap(current->text()+"/away.png"));
    m_iconIdle->setPixmap(QPixmap(current->text()+"/idle.png"));
    m_iconDnd->setPixmap(QPixmap(current->text()+"/dnd.png"));
    m_iconInvisible->setPixmap(QPixmap(current->text()+"/invisible.png"));
    m_iconOffline->setPixmap(QPixmap(current->text()+"/offline.png"));
    m_iconOnline->setPixmap(QPixmap(current->text()+"/online.png"));
}

void ConfigDialog::slotAddStatusTheme()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select theme directory"),
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
                                options);
    if(!directory.isEmpty() && QFileInfo(directory+"/online.png").exists())
    {
        m_themesStatusList->addItem(directory);
    }
    if(m_themesStatusList->count()>1) m_themeStatusDelete->setEnabled(true);
}

void ConfigDialog::slotDeleteStatusTheme()
{
    delete m_themesStatusList->currentItem();
    if(m_themesStatusList->count()==1) m_themeStatusDelete->setEnabled(false);
}

void ConfigDialog::slotThemeRoles(QListWidgetItem *current)
{
    m_iconFounder->setPixmap(QPixmap(current->text()+"/founder.png"));
    m_iconHalfop->setPixmap(QPixmap(current->text()+"/halfop.png"));
    m_iconNormal->setPixmap(QPixmap(current->text()+"/normal.png"));
    m_iconOp->setPixmap(QPixmap(current->text()+"/op.png"));
    m_iconVoice->setPixmap(QPixmap(current->text()+"/voice.png"));
}

void ConfigDialog::slotAddRoleTheme()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
    QString directory = QFileDialog::getExistingDirectory(this,
                                _("Select theme directory"),
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
                                options);
    if(!directory.isEmpty() && QFileInfo(directory+"/normal.png").exists())
    {
        m_themesRoleList->addItem(directory);
    }
    if(m_themesRoleList->count()>1) m_themeRoleDelete->setEnabled(true);
}

void ConfigDialog::slotDeleteRoleTheme()
{
    delete m_themesRoleList->currentItem();
    if(m_themesRoleList->count()==1) m_themeRoleDelete->setEnabled(false);
}

void ConfigDialog::createLookTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QHBoxLayout *layout = new QHBoxLayout;
    QVBoxLayout *colors = new QVBoxLayout;
    m_unreadtabColor = new ColorButton(DXPREFERENCES.m_unreadtabColor, _("Unreaded Tab Color"));
    connect(m_unreadtabColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotTabColor()));
    m_hltabColor = new ColorButton(DXPREFERENCES.m_hltabColor, _("Highlighted Tab Color"));
    connect(m_hltabColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotTabColor()));
    colors->addWidget(m_unreadtabColor);
    colors->addWidget(m_hltabColor);
    QHBoxLayout *position = new QHBoxLayout;
    m_tabPosition = new QComboBox;
    m_tabPosition->addItem(_("Bottom"));
    m_tabPosition->addItem(_("Left"));
    m_tabPosition->addItem(_("Top"));
    m_tabPosition->addItem(_("Right"));
    m_tabPosition->setCurrentIndex(DXPREFERENCES.m_tabPosition);
    connect(m_tabPosition, SIGNAL(currentIndexChanged(int)), this, SLOT(slotTabPosition()));
    position->addWidget(new QLabel(_("Tab position")));
    position->addWidget(m_tabPosition);
    colors->addLayout(position);
    colors->addStretch(1);
    layout->addLayout(colors);
    m_lookTabs = new TabWidget;
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Standard tab")), "linux");
    m_lookTabs->setTabIcon(0, QIcon(DXIM_DATADIR "/icons/chat.png"));
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Unreaded tab")), "bsd");
    m_lookTabs->setTabIcon(1, QIcon(DXIM_DATADIR "/icons/chat.png"));
    m_lookTabs->setTabTextColor(1, DXPREFERENCES.m_unreadtabColor);
    m_lookTabs->addTab(new QLabel(QString("\t")+_("Highlighted tab")), "hurd");
    m_lookTabs->setTabIcon(2, QIcon(DXIM_DATADIR "/icons/chatnewm.png"));
    m_lookTabs->setTabTextColor(2, DXPREFERENCES.m_hltabColor);
    layout->addWidget(m_lookTabs);
    main->addLayout(layout);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&Look"));
    updateTabPosition();
}

void ConfigDialog::slotTabColor()
{
    m_lookTabs->setTabTextColor(1, m_unreadtabColor->getColor());
    m_lookTabs->setTabTextColor(2, m_hltabColor->getColor());
}

void ConfigDialog::slotTabPosition()
{
    updateTabPosition();
}

void ConfigDialog::updateTabPosition()
{
    switch(m_tabPosition->currentIndex()) {
    case 0: //bottom
        m_lookTabs->setTabPosition(QTabWidget::South);
        break;
    case 1: //left
        m_lookTabs->setTabPosition(QTabWidget::West);
        break;
    case 2: //top
        m_lookTabs->setTabPosition(QTabWidget::North);
        break;
    case 3: //right
        m_lookTabs->setTabPosition(QTabWidget::East);
        break;
    default:
        m_lookTabs->setTabPosition(QTabWidget::South);
    }
}

void ConfigDialog::createImtextTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    QHBoxLayout *layout = new QHBoxLayout;
    QVBoxLayout *colors = new QVBoxLayout;
    m_textColor = new ColorButton(DXPREFERENCES.m_textColor, _("Text color"));
    connect(m_textColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_textColor);
    m_backroundColor = new ColorButton(DXPREFERENCES.m_backroundColor, _("Text backround color"));
    connect(m_backroundColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_backroundColor);
    m_userColor = new ColorButton(DXPREFERENCES.m_userColor, _("User events text color"));
    connect(m_userColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_userColor);
    m_actionColor = new ColorButton(DXPREFERENCES.m_actionColor, _("Message in 3rd person text color"));
    connect(m_actionColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_actionColor);
    m_errorColor = new ColorButton(DXPREFERENCES.m_errorColor, _("Error text color"));
    connect(m_errorColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_errorColor);
    m_hilightColor = new ColorButton(DXPREFERENCES.m_hilightColor, _("Highlight message text color"));
    connect(m_hilightColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_hilightColor);
    m_linkColor = new ColorButton(DXPREFERENCES.m_linkColor, _("Link color"));
    connect(m_linkColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_linkColor);
    m_mymsgColor = new ColorButton(DXPREFERENCES.m_mymsgColor, _("My message color"));
    connect(m_mymsgColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_mymsgColor);
    m_logColor = new ColorButton(DXPREFERENCES.m_logColor, _("History line color"));
    connect(m_logColor, SIGNAL(colorChanged(QColor)), this, SLOT(slotImTextColor()));
    colors->addWidget(m_logColor);
    QHBoxLayout *font = new QHBoxLayout;
    m_imFont = new QLabel(DXPREFERENCES.m_imFont.toString());
    m_imFont->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    m_imFont->setFont(DXPREFERENCES.m_imFont);
    font->addWidget(m_imFont);
    m_imFontModify = new QPushButton("...");
    connect(m_imFontModify, SIGNAL(clicked()), this, SLOT(slotModifyIMFont()));
    font->addWidget(m_imFontModify);
    colors->addLayout(font);
    m_sameCmd = new QCheckBox(_("Use same font for commandline"));
    m_sameCmd->setChecked(DXPREFERENCES.m_sameCmd);
    colors->addWidget(m_sameCmd);
    m_sameList = new QCheckBox(_("Use same font for user list"));
    m_sameList->setChecked(DXPREFERENCES.m_sameList);
    colors->addWidget(m_sameList);
    QHBoxLayout *nick = new QHBoxLayout;
    nick->addWidget(new QLabel(_("Nick completion char")));
    m_nickCompletionChar = new QLineEdit;
    m_nickCompletionChar->setText(DXPREFERENCES.m_nickCompletionChar);
    nick->addWidget(m_nickCompletionChar);
    colors->addLayout(nick);
    m_coloredNick = new QCheckBox(_("Use colored nick"));
    m_coloredNick->setChecked(DXPREFERENCES.m_coloredNick);
    colors->addWidget(m_coloredNick);
    colors->addStretch(1);
    layout->addLayout(colors);
    m_imText = new QTextEdit;
    m_imText->setReadOnly(true);
    m_imText->setFont(DXPREFERENCES.m_imFont);
    updateImText();
    layout->addWidget(m_imText);
    main->addLayout(layout);
    tab->setLayout(main);
    tabbook->addTab(tab, _("&IM Text"));
}

void ConfigDialog::slotImTextColor()
{
    updateImText();
}

void ConfigDialog::updateImText()
{
    m_imText->clear();
    QString sheet = QString::fromLatin1("a { text-decoration: underline; color: %1 }"
        ).arg(m_linkColor->getColor().name());
    m_imText->document()->setDefaultStyleSheet(sheet);
    m_imText->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }"
                                "a { color:%3; }")
                                .arg(m_textColor->getColor().name())
                                .arg(m_backroundColor->getColor().name())
                                .arg(m_linkColor->getColor().name()));
    m_imText->append(QString("[1.1.2000 00:00:00] <font color=\"%1\">dvx: msg from history</font>").arg(m_logColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">dvx has joined to dxim</font>").arg(m_userColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">dvx: Welcome in dxim</font>").arg(m_mymsgColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">bm: dvx, dxim is nice</font>").arg(m_hilightColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">dvx is online</font>").arg(m_actionColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">No error</font>").arg(m_errorColor->getColor().name()));
    m_imText->append(QString("[00:00:00] <font color=\"%1\">dvx:</font> <a href=\"http://dxsolutions.org\">http://dxsolutions.org</a>").arg(m_mymsgColor->getColor().name()));
}

void ConfigDialog::createNotificationTab(QTabWidget *tabbook)
{
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    m_sounds = new QCheckBox(_("Use sounds"));
    m_sounds->setChecked(DXPREFERENCES.m_sounds);
    connect(m_sounds, SIGNAL(stateChanged(int)), this, SLOT(slotSound()));
    main->addWidget(m_sounds);
    QGroupBox *soundGroup = new QGroupBox(_("Events"));
    QVBoxLayout *soundLayout = new QVBoxLayout;
    m_soundConnect = new QCheckBox(_("Friend connected"));
    m_soundConnect->setChecked(DXPREFERENCES.m_soundConnect);
    connect(m_soundConnect, SIGNAL(stateChanged(int)), this, SLOT(slotSoundConnect()));
    soundLayout->addWidget(m_soundConnect);
    QHBoxLayout *connectLayout = new QHBoxLayout;
    m_pathConnect = new QLabel(DXPREFERENCES.m_pathConnect);
    m_pathConnect->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    connectLayout->addWidget(m_pathConnect, 1);
    m_setpathConnect = new QPushButton("...");
    connect(m_setpathConnect, SIGNAL(clicked()), this, SLOT(slotPath()));
    connectLayout->addWidget(m_setpathConnect);
    m_playConnect = new QPushButton(QIcon(DXIM_DATADIR "/icons/play.png"), "");
    connect(m_playConnect, SIGNAL(clicked()), this, SLOT(slotPlay()));
    connectLayout->addWidget(m_playConnect);
    soundLayout->addLayout(connectLayout);
    m_soundDisconnect = new QCheckBox(_("Friend disconnected"));
    m_soundDisconnect->setChecked(DXPREFERENCES.m_soundDisconnect);
    connect(m_soundDisconnect, SIGNAL(stateChanged(int)), this, SLOT(slotSoundDisconnect()));
    soundLayout->addWidget(m_soundDisconnect);
    QHBoxLayout *disconnectLayout = new QHBoxLayout;
    m_pathDisconnect = new QLabel(DXPREFERENCES.m_pathDisconnect);
    m_pathDisconnect->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    disconnectLayout->addWidget(m_pathDisconnect, 1);
    m_setpathDisconnect = new QPushButton("...");
    connect(m_setpathDisconnect, SIGNAL(clicked()), this, SLOT(slotPath()));
    disconnectLayout->addWidget(m_setpathDisconnect);
    m_playDisconnect = new QPushButton(QIcon(DXIM_DATADIR "/icons/play.png"), "");
    connect(m_playDisconnect, SIGNAL(clicked()), this, SLOT(slotPlay()));
    disconnectLayout->addWidget(m_playDisconnect);
    soundLayout->addLayout(disconnectLayout);
    m_soundMessage = new QCheckBox(_("Highlighted or im message"));
    m_soundMessage->setChecked(DXPREFERENCES.m_soundMessage);
    connect(m_soundMessage, SIGNAL(stateChanged(int)), this, SLOT(slotSoundMessage()));
    soundLayout->addWidget(m_soundMessage);
    QHBoxLayout *messageLayout = new QHBoxLayout;
    m_pathMessage = new QLabel(DXPREFERENCES.m_pathMessage);
    m_pathMessage->setFrameStyle(QFrame::Sunken|QFrame::Panel);
    messageLayout->addWidget(m_pathMessage, 1);
    m_setpathMessage = new QPushButton("...");
    connect(m_setpathMessage, SIGNAL(clicked()), this, SLOT(slotPath()));
    messageLayout->addWidget(m_setpathMessage);
    m_playMessage = new QPushButton(QIcon(DXIM_DATADIR "/icons/play.png"), "");
    connect(m_playMessage, SIGNAL(clicked()), this, SLOT(slotPlay()));
    messageLayout->addWidget(m_playMessage);
    soundLayout->addLayout(messageLayout);
    soundGroup->setLayout(soundLayout);
    main->addWidget(soundGroup);
    QHBoxLayout *notifyuseLayout = new QHBoxLayout;
    m_notify = new QCheckBox(_("Use notify popup"));
    m_notify->setChecked(DXPREFERENCES.m_notify);
    connect(m_notify, SIGNAL(stateChanged(int)), this, SLOT(slotNotify()));
    notifyuseLayout->addWidget(m_notify);
    m_showNotify = new QPushButton(_("Show"));
    connect(m_showNotify, SIGNAL(clicked()), this, SLOT(slotShowNotify()));
    notifyuseLayout->addWidget(m_showNotify);
    notifyuseLayout->addStretch(1);
    main->addLayout(notifyuseLayout);
    if(!dxutils::instance().haveNotify())
    {
        QHBoxLayout *notifyposLayout = new QHBoxLayout;
        notifyposLayout->addWidget(new QLabel(_("Notify position")));
        m_notifyPosition = new QComboBox;
        m_notifyPosition->addItem(_("Left Top"));
        m_notifyPosition->addItem(_("Right Top"));
        m_notifyPosition->addItem(_("Left Bottom"));
        m_notifyPosition->addItem(_("Right Bottom"));
        if(DXPREFERENCES.m_notifyPosition>=0 && DXPREFERENCES.m_notifyPosition<4) m_notifyPosition->setCurrentIndex(DXPREFERENCES.m_notifyPosition);
        notifyposLayout->addWidget(m_notifyPosition);
        notifyposLayout->addStretch(1);
        main->addLayout(notifyposLayout);
    }
    QGroupBox *notifyGroup = new QGroupBox(_("Events"));
    QVBoxLayout *notifyLayout = new QVBoxLayout;
    m_notifyAccountConnect = new QCheckBox(_("Account connected"));
    m_notifyAccountConnect->setChecked(DXPREFERENCES.m_notifyAccountConnect);
    notifyLayout->addWidget(m_notifyAccountConnect);
    m_notifyAccountDisconnect = new QCheckBox(_("Account disconnected"));
    m_notifyAccountDisconnect->setChecked(DXPREFERENCES.m_notifyAccountDisconnect);
    notifyLayout->addWidget(m_notifyAccountDisconnect);
    m_notifyBuddyConnect = new QCheckBox(_("Friend connected"));
    m_notifyBuddyConnect->setChecked(DXPREFERENCES.m_notifyBuddyConnect);
    notifyLayout->addWidget(m_notifyBuddyConnect);
    m_notifyBuddyDisconnect = new QCheckBox(_("Friend disconnected"));
    m_notifyBuddyDisconnect->setChecked(DXPREFERENCES.m_notifyBuddyDisconnect);
    notifyLayout->addWidget(m_notifyBuddyDisconnect);
    m_notifyMessage = new QCheckBox(_("Highlighted or im message"));
    m_notifyMessage->setChecked(DXPREFERENCES.m_notifyMessage);
    notifyLayout->addWidget(m_notifyMessage);
    m_notifyFiletransfer = new QCheckBox(_("File transfer finished or canceled"));
    m_notifyFiletransfer->setChecked(DXPREFERENCES.m_notifyFiletransfer);
    notifyLayout->addWidget(m_notifyFiletransfer);
    notifyGroup->setLayout(notifyLayout);
    main->addWidget(notifyGroup);
    m_raiseWindow = new QCheckBox(_("Show application window on highlighted or im message"));
    m_raiseWindow->setChecked(DXPREFERENCES.m_raiseWindow);
    connect(m_raiseWindow, SIGNAL(stateChanged(int)), this, SLOT(slotRaiseWindow()));
    main->addWidget(m_raiseWindow);
    m_notifyMessageTray = new QCheckBox(_("Change tray icon for all messages if application in tray"));
    if(m_raiseWindow->isChecked())
    {
        m_notifyMessageTray->setEnabled(false);
        m_notifyMessageTray->setChecked(false);
    }
    else
    {
        m_notifyMessageTray->setChecked(DXPREFERENCES.m_notifyMessageTray);
    }
    main->addWidget(m_notifyMessageTray);
    main->addStretch(1);
    tab->setLayout(main);
    tabbook->addTab(tab, _("N&otification"));
    slotSound();
    slotNotify();
}

void ConfigDialog::slotSound()
{
    if(m_sounds->isChecked())
    {
        m_soundConnect->setEnabled(true);
        m_soundDisconnect->setEnabled(true);
        m_soundMessage->setEnabled(true);
    }
    else
    {
        m_soundConnect->setEnabled(false);
        m_soundDisconnect->setEnabled(false);
        m_soundMessage->setEnabled(false);
    }
    slotSoundConnect();
    slotSoundDisconnect();
    slotSoundMessage();
}

void ConfigDialog::slotSoundConnect()
{
    if(m_sounds->isChecked() && m_soundConnect->isChecked())
    {
        m_setpathConnect->setEnabled(true);
        m_playConnect->setEnabled(true);
    }
    else
    {
        m_setpathConnect->setEnabled(false);
        m_playConnect->setEnabled(false);
    }
}

void ConfigDialog::slotSoundDisconnect()
{
    if(m_sounds->isChecked() && m_soundDisconnect->isChecked())
    {
        m_setpathDisconnect->setEnabled(true);
        m_playDisconnect->setEnabled(true);
    }
    else
    {
        m_setpathDisconnect->setEnabled(false);
        m_playDisconnect->setEnabled(false);
    }
}

void ConfigDialog::slotSoundMessage()
{
    if(m_sounds->isChecked() && m_soundMessage->isChecked())
    {
        m_setpathMessage->setEnabled(true);
        m_playMessage->setEnabled(true);
    }
    else
    {
        m_setpathMessage->setEnabled(false);
        m_playMessage->setEnabled(false);
    }
}

void ConfigDialog::slotPath()
{
    QPushButton *btn = qobject_cast<QPushButton*>(sender());
    if(btn == m_setpathConnect)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathConnect->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathConnect->setText(file);
        }
    }
    else if(btn == m_setpathDisconnect)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathDisconnect->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathDisconnect->setText(file);
        }
    }
    else if(btn == m_setpathMessage)
    {
        QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
        QString file = QFileDialog::getOpenFileName(this,
                                    _("Select file"),
                                    m_pathMessage->text(),
                                    _("Sound file (*.wav)"),
                                    NULL,
                                    options);
        if(!file.isEmpty())
        {
            m_pathMessage->setText(file);
        }
    }
}

void ConfigDialog::slotPlay()
{
    QPushButton *btn = qobject_cast<QPushButton*>(sender());
    if(btn == m_playConnect)
        dxutils::playFile(m_pathConnect->text());
    else if(btn == m_playDisconnect)
        dxutils::playFile(m_pathDisconnect->text());
    else if(btn == m_playMessage)
        dxutils::playFile(m_pathMessage->text());
}

void ConfigDialog::slotNotify()
{
    if(m_notify->isChecked())
    {
        if(!dxutils::instance().haveNotify())
            m_notifyPosition->setEnabled(true);
        m_notifyAccountConnect->setEnabled(true);
        m_notifyAccountDisconnect->setEnabled(true);
        m_notifyBuddyConnect->setEnabled(true);
        m_notifyBuddyDisconnect->setEnabled(true);
        m_notifyMessage->setEnabled(true);
        m_notifyFiletransfer->setEnabled(true);
    }
    else
    {
        if(!dxutils::instance().haveNotify())
            m_notifyPosition->setEnabled(false);
        m_notifyAccountConnect->setEnabled(false);
        m_notifyAccountDisconnect->setEnabled(false);
        m_notifyBuddyConnect->setEnabled(false);
        m_notifyBuddyDisconnect->setEnabled(false);
        m_notifyMessage->setEnabled(false);
        m_notifyFiletransfer->setEnabled(false);
    }
}

void ConfigDialog::slotShowNotify()
{
    if(dxutils::instance().haveNotify())
        dxutils::instance().showNotify(_("%1 is available").arg("dvx"));
    else
        dxutils::instance().showNotify(_("%1 is available").arg("dvx"), m_notifyPosition->currentIndex());
}

void ConfigDialog::slotRaiseWindow()
{
    if(m_raiseWindow->isChecked())
    {
        m_notifyMessageTray->setEnabled(false);
        m_notifyMessageTray->setChecked(false);
    }
    else
    {
        m_notifyMessageTray->setEnabled(true);
    }
}

void ConfigDialog::createSmileysTab(QTabWidget* tabbook)
{
    m_smileysMap = DXPREFERENCES.m_smileysMap;
    QWidget *tab = new QWidget;
    QVBoxLayout *main = new QVBoxLayout;
    m_useSmileys = new QCheckBox(_("Use smileys"));
    m_useSmileys->setChecked(DXPREFERENCES.m_useSmileys);
    connect(m_useSmileys, SIGNAL(clicked()), this, SLOT(slotUseSmileys()));
    main->addWidget(m_useSmileys);
    QHBoxLayout *smileys = new QHBoxLayout;
    m_smileysList = new QListWidget;
    fillSmileys();
    smileys->addWidget(m_smileysList);
    QVBoxLayout *buttons = new QVBoxLayout;
    m_addSmiley = new QPushButton(_("Add"));
    connect(m_addSmiley, SIGNAL(clicked()), this, SLOT(slotAddSmiley()));
    buttons->addWidget(m_addSmiley);
    m_modifySmiley = new QPushButton(_("Modify"));
    connect(m_modifySmiley, SIGNAL(clicked()), this, SLOT(slotModifySmiley()));
    buttons->addWidget(m_modifySmiley);
    m_deleteSmiley = new QPushButton(_("Delete"));
    connect(m_deleteSmiley, SIGNAL(clicked()), this, SLOT(slotDeleteSmiley()));
    buttons->addWidget(m_deleteSmiley);
    m_importSmiley = new QPushButton(_("Import"));
    connect(m_importSmiley, SIGNAL(clicked()), this, SLOT(slotImportSmileys()));
    buttons->addWidget(m_importSmiley);
    m_exportSmiley = new QPushButton(_("Export"));
    connect(m_exportSmiley, SIGNAL(clicked()), this, SLOT(slotExportSmileys()));
    buttons->addWidget(m_exportSmiley);
    buttons->addStretch(1);
    smileys->addLayout(buttons);
    main->addLayout(smileys);
    tab->setLayout(main);
    tabbook->addTab(tab, _("S&mileys"));
    slotUseSmileys();
}

void ConfigDialog::slotUseSmileys()
{
    if(m_useSmileys->isChecked())
    {
        m_addSmiley->setEnabled(true);
        m_importSmiley->setEnabled(true);
        if(m_smileysList->count())
        {
            m_modifySmiley->setEnabled(true);
            m_deleteSmiley->setEnabled(true);
            m_exportSmiley->setEnabled(true);
        }
        else
        {
            m_modifySmiley->setEnabled(false);
            m_deleteSmiley->setEnabled(false);
            m_exportSmiley->setEnabled(false);
        }
    }
    else
    {
        m_addSmiley->setEnabled(false);
        m_modifySmiley->setEnabled(false);
        m_deleteSmiley->setEnabled(false);
        m_importSmiley->setEnabled(false);
        m_exportSmiley->setEnabled(false);
    }
}

void ConfigDialog::slotAddSmiley()
{
    SmileyDialog dialog(_("Add smiley"), "", "");
    if(dialog.exec() == QDialog::Accepted)
    {
        m_smileysMap.insert(dialog.getSmiley(), dialog.getPath());
        fillSmileys();
        m_smileysList->setCurrentRow(m_smileysList->count()-1);
        slotUseSmileys();
    }
}

void ConfigDialog::slotModifySmiley()
{
    QMap<QString,QString>::iterator it = m_smileysMap.find(m_smileysList->currentItem()?m_smileysList->currentItem()->text():m_smileysList->item(0)->text());
    if(it!=m_smileysMap.end())
    {
        SmileyDialog dialog(_("Modify smiley"), it.key(), it.value());
        if(dialog.exec() == QDialog::Accepted)
        {
            m_smileysMap.remove(it.key());
            m_smileysMap.insert(dialog.getSmiley(), dialog.getPath());
            fillSmileys();
            slotUseSmileys();
        }
    }
}

void ConfigDialog::slotDeleteSmiley()
{
    QMap<QString,QString>::iterator it = m_smileysMap.find(m_smileysList->currentItem()?m_smileysList->currentItem()->text():m_smileysList->item(0)->text());
    if(it!=m_smileysMap.end())
    {
        if(QMessageBox::question(this, _("Question"), _("Delete smiley '%1'?").arg(it.key()), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Ok)
        {
            m_smileysMap.remove(it.key());
            fillSmileys();
            slotUseSmileys();
        }
    }
}

void ConfigDialog::slotImportSmileys()
{
    if(m_showImportwarning && m_smileysMap.size())
    {
        if(QMessageBox::question(this, _("Warning"), _("File import overwrites current settings"), QMessageBox::Ok|QMessageBox::Cancel) == QMessageBox::Cancel)
        {
            m_showImportwarning = false;
            return;
        }
    }
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString path = QFileDialog::getOpenFileName(this,
                                _("Select file"),
                                m_showImportwarning?DXIM_DATADIR "/icons/smileys/dxim.scfg":"",
                                _("Smileys config files *.scfg"),
                                NULL,
                                options);
    if(!path.isEmpty())
    {
        m_smileysList->clear();
        m_smileysMap.clear();
        QFile file(path);
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            QMessageBox::warning(this, _("Error Loading File"), _("Unable to open file: %1").arg(path));
            return;
        }
        QDir dir(QFileInfo(path).path());
        while(!file.atEnd())
        {
            QByteArray line = file.readLine();
            QString sline = QString(line);
            if(sline[0]!='#') //# start comment line
            {
                QString smiley = sline.section('=',1).trimmed();
                QString spath = dir.cleanPath(dir.absoluteFilePath(sline.section('=',0,0)));
                if(!smiley.isEmpty() && !spath.isEmpty()) m_smileysMap.insert(smiley, spath);
            }
        }
        fillSmileys();
        slotUseSmileys();
    }
    m_showImportwarning = false;
}

void ConfigDialog::slotExportSmileys()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString path = QFileDialog::getSaveFileName(this,
                                _("Save smiley settings as"),
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
                                "*.scfg",
                                NULL,
                                options);
    if(!path.isEmpty())
    {
        QFile file(path);
        if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
        {
            QMessageBox::warning(this, _("Error Writing File"), _("Unable to open file: %1").arg(path));
            return;
        }
        QDir dir(QFileInfo(path).path());
        file.write(HEADER);
        QMap<QString,QString>::iterator it;
        for(it=m_smileysMap.begin(); it!=m_smileysMap.end(); ++it)
        {
            file.write(dir.relativeFilePath(it.value()).toLatin1());
            file.write("=");
            file.write(it.key().toLatin1());
            file.write("\n");
        }
        file.close();
    }
}

void ConfigDialog::fillSmileys()
{
    m_smileysList->clear();
    QMap<QString,QString>::iterator it;
    for(it=m_smileysMap.begin(); it!=m_smileysMap.end(); ++it)
    {
        m_smileysList->addItem(new QListWidgetItem(QIcon(it.value()), it.key()));
    }
}

void ConfigDialog::slotModifyIMFont()
{
    bool ok;
    QFont font;
    font.fromString(m_imFont->text());
    font = QFontDialog::getFont(&ok, font, this);
    if(ok)
    {
        m_imFont->setText(font.toString());
        m_imFont->setFont(font);
        m_imText->setFont(font);
    }
}
