/*
 *      fieldsrequestdialog.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QLabel>
#include <QPushButton>
#include <QDialogButtonBox>
#include <QCheckBox>
#include <QLineEdit>
#include <QTextEdit>
#include <QListWidget>
#include <QComboBox>
#include <QGridLayout>
#include "dxutils.h"
#include "fieldsrequestdialog.h"
#include "config.h"
#include "i18n.h"

static bool vectorContains(std::vector<std::string> v, const std::string &value)
{
    FOREACH(std::vector<std::string>, v, it)
    {
        if(value == *it)
            return true;
    }
    return false;
}

FieldsRequestDialog::FieldsRequestDialog(PURPLE::FieldsRequest *request, QWidget *parent) :
    QDialog(parent), m_request(request)
{
    QDialogButtonBox *buttons = new QDialogButtonBox();
    QPushButton *buttonOk = new QPushButton(dxutils::fromStdString(request->getOkText()));
    buttons->addButton(buttonOk, QDialogButtonBox::AcceptRole);
    QPushButton *buttonCancel = new QPushButton(dxutils::fromStdString(request->getCancelText()));
    buttons->addButton(buttonCancel, QDialogButtonBox::RejectRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    if(!m_request->getPrimary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getPrimary())));
    if(!m_request->getSecondary().empty()) mainFrame->addWidget(new QLabel(dxutils::fromStdString(request->getSecondary())));
    createFields(mainFrame);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(m_request->getTitle().empty()? _("Request") : dxutils::fromStdString(m_request->getTitle()));
}

void FieldsRequestDialog::accept()
{
    fillFields();
    m_request->acceptRequest();
    QDialog::accept();
}

void FieldsRequestDialog::reject()
{
    m_request->dennyRequest();
    QDialog::reject();
}

void FieldsRequestDialog::createFields(QVBoxLayout *frame)
{
    std::vector<PURPLE::FieldsRequest::Field> v = m_request->getFields();
    QString group = "";
    QGridLayout *matrix = new QGridLayout;
    frame->addLayout(matrix);
    QLineEdit *tfield;
    QTextEdit *text;
    QComboBox *combo;
    QListWidget *list;
    QCheckBox *check;
    int i = 0;
    FOREACH(std::vector<PURPLE::FieldsRequest::Field>, v, it)
    {
        PURPLE::FieldsRequest::Field field = *it;
        if(group != dxutils::fromStdString(field.getGroup()))
        {
            group = dxutils::fromStdString(field.getGroup());
            if(!group.isEmpty()) frame->addWidget(new QLabel(QString("<b>%1</b>").arg(group)));
            matrix = new QGridLayout;
            frame->addLayout(matrix);
            i = 0;
        }
        if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            if(field.getMultiline())
            {
                text = new QTextEdit;
                text->setText(dxutils::fromStdString(field.getStringValue()));
                text->setReadOnly(!field.getEditable());
                matrix->addWidget(text, i, 1);
                m_fields.append(text);
            }
            else
            {
                tfield = new QLineEdit;
                if(field.getMasked()) tfield->setEchoMode(QLineEdit::PasswordEchoOnEdit);
                tfield->setReadOnly(!field.getEditable());
                tfield->setText(dxutils::fromStdString(field.getStringValue()));
                matrix->addWidget(tfield, i, 1);
                m_fields.append(tfield);
            }
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            tfield = new QLineEdit;
            tfield->setValidator(new QIntValidator(tfield));
            tfield->setText(dxutils::fromStdString(field.getStringValue()));
            matrix->addWidget(tfield, i, 1);
            m_fields.append(tfield);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            check = new QCheckBox;
            check->setChecked(field.getBoolValue());
            matrix->addWidget(check, i, 1);
            m_fields.append(check);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            combo = new QComboBox;
            std::vector<std::string> s = field.getLabels();
            FOREACH(std::vector<std::string>, s, sit)
            {
                combo->addItem(dxutils::fromStdString(*sit));
            }
            matrix->addWidget(combo, i, 1);
            m_fields.append(combo);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            list = new QListWidget;
            if(field.getMultiple()) list->setSelectionMode(QAbstractItemView::ExtendedSelection);
            else list->setSelectionMode(QAbstractItemView::SingleSelection);
            std::vector<std::string> s = field.getLabels();
            std::vector<std::string> sel = field.getSelected();
            FOREACH(std::vector<std::string>, s, sit)
            {
                QListWidgetItem *item = new QListWidgetItem(dxutils::fromStdString(*sit));
                list->addItem(item);
                if(vectorContains(sel, *sit))
                    item->setSelected(true);
            }
            matrix->addWidget(list, i, 1);
            m_fields.append(list);
        }
        else if(field.getType() == PURPLE::FieldsRequest::Field::FIMAGE)
        {
            matrix->addWidget(new QLabel(dxutils::fromStdString(field.getLabel())), i, 0);
            QLabel *image = new QLabel;
            image->setPixmap(QPixmap(dxutils::fromStdString(field.getStringValue())));
            matrix->addWidget(image, i, 1);
        }
        i++;
    }
}

void FieldsRequestDialog::fillFields()
{
    PURPLE::FieldsRequest::Fields fields = m_request->getFields();
        PURPLE::FieldsRequest::Fields::iterator it;
        uint i = 0;
        for(it=fields.begin(); it!=fields.end(); it++)
        {
            PURPLE::FieldsRequest::Field& field = *it;
            if(field.getType() == PURPLE::FieldsRequest::Field::FSTR)
            {
                if(field.getMultiline())
                {
                    field.setValue(dxutils::toStdString(static_cast<QTextEdit*>(m_fields[i])->toPlainText()));
                }
                else
                {
                    field.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_fields[i])->text()));
                }
                i++;
            }
            else if(field.getType() == PURPLE::FieldsRequest::Field::FINT)
            {
                field.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_fields[i])->text()));
                i++;
            }
            else if(field.getType() == PURPLE::FieldsRequest::Field::FBOOL)
            {
                field.setValue(static_cast<QCheckBox*>(m_fields[i])->isChecked());
                i++;
            }
            else if(field.getType() == PURPLE::FieldsRequest::Field::FCHOICE)
            {
                field.setValue(static_cast<QComboBox*>(m_fields[i])->currentIndex());
                i++;
            }
            else if(field.getType() == PURPLE::FieldsRequest::Field::FLIST)
            {
                QList<QListWidgetItem*> list = static_cast<QListWidget*>(m_fields[i])->selectedItems();
                std::vector<std::string> v;
                foreach(QListWidgetItem *item, list)
                {
                    v.push_back(dxutils::toStdString(item->text()));
                }
                field.setSelected(v);
                i++;
            }
            else
                continue;
        }
        m_request->setFields(fields);
}

