/*
 *      plugindialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QStringList>
#include <QLabel>
#include <QHeaderView>
#include <QDialogButtonBox>
#include <QBoxLayout>
#include <QComboBox>
#include <QMessageBox>
#include "plugindialog.h"
#include "plugineditordialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

PluginDialog::PluginDialog(PURPLE::Purple *purple, QWidget *parent) :
    QDialog(parent), m_purple(purple)
{
    m_plugins = purple->getPluginsList();
    QStringList headers;
    headers << _("Plugin") << _("Enabled");
    m_pluginView = new QTreeWidget(this);
    m_pluginView->setHeaderLabels(headers);
    m_pluginView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_pluginView->setAlternatingRowColors(true);
    m_pluginView->setRootIsDecorated(false);
    m_pluginView->setSelectionMode(QAbstractItemView::SingleSelection);

    m_details = new QGroupBox(_("Details"));
    QGridLayout *groupLayout = new QGridLayout;
    groupLayout->addWidget(new QLabel(_("Summary:")), 0, 0);
    m_summary = new QLabel;
    groupLayout->addWidget(m_summary, 0, 1);
    groupLayout->addWidget(new QLabel(_("Version:")), 1, 0);
    m_version = new QLabel;
    groupLayout->addWidget(m_version, 1, 1);
    groupLayout->addWidget(new QLabel(_("Description:")), 2, 0);
    m_description = new QLabel;
    groupLayout->addWidget(m_description, 2, 1);
    groupLayout->addWidget(new QLabel(_("Author:")), 3, 0);
    m_author = new QLabel;
    groupLayout->addWidget(m_author, 3, 1);
    groupLayout->addWidget(new QLabel(_("Path:")), 4, 0);
    m_path = new QLabel;
    groupLayout->addWidget(m_path, 4, 1);
    groupLayout->addWidget(new QLabel(_("Homepage:")), 5, 0);
    m_homepage = new QLabel;
    m_homepage->setOpenExternalLinks(true);
    groupLayout->addWidget(m_homepage, 5, 1);
    m_details->setLayout(groupLayout);

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Save|QDialogButtonBox::Close);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    m_buttonEnable = new QPushButton(_("&Enable"));
    m_buttonEnable->setEnabled(false);
    buttons->addButton(m_buttonEnable, QDialogButtonBox::ActionRole);
    connect(m_buttonEnable, SIGNAL(clicked()), this, SLOT(slotEnablePlugin()));
    m_buttonModify = new QPushButton(_("&Modify"));
    m_buttonModify->setEnabled(false);
    buttons->addButton(m_buttonModify, QDialogButtonBox::ActionRole);
    connect(m_buttonModify, SIGNAL(clicked()), this, SLOT(slotModifyPlugin()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addWidget(m_pluginView);
    mainFrame->addWidget(m_details);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
    setWindowTitle(_("Plugins list"));
    updateTable();
    updateDetails();

    connect(m_pluginView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(slotCurrentChanged(QTreeWidgetItem*)));
}

void PluginDialog::accept()
{
    m_purple->saveLoadedPlugins("/dxim/plugins/loaded");
    QDialog::accept();
}

void PluginDialog::slotEnablePlugin()
{
    for(size_t i=0; i<m_plugins.size(); i++)
    {
        if(i == (size_t)m_pluginView->currentIndex().row())
        {
            if(!m_plugins[i].isLoadable())
                return;
            if(m_pluginView->currentItem()->text(1) == _("Yes"))
            {
                m_plugins[i].unload();
                m_buttonEnable->setText(_("&Enable"));
            }
            else
            {
                m_plugins[i].load();
                m_buttonEnable->setText(_("Di&sable"));
            }
        }
    }
    updateTable();
}

void PluginDialog::slotModifyPlugin()
{
    for(size_t i=0; i<m_plugins.size(); i++)
    {
        if(i == (size_t)m_pluginView->currentIndex().row())
        {
            PluginEditorDialog editor(m_plugins[i], this);
            editor.setModal(true);
            editor.exec();
        }
    }
}

void PluginDialog::slotCurrentChanged(QTreeWidgetItem *current)
{
    if(current)
    {
        if(current->text(1) == _("Yes"))
            m_buttonEnable->setText(_("Di&sable"));
        else
            m_buttonEnable->setText(_("&Enable"));
        size_t row = (size_t)m_pluginView->currentIndex().row();
        if(m_plugins[row].isLoaded())
        {
            m_buttonEnable->setEnabled(true);
            if(m_plugins[row].hasOptions()) m_buttonModify->setEnabled(true);
            else m_buttonModify->setEnabled(false);
        }
        else
        {
            if(m_plugins[row].isLoadable())
            {
                m_buttonEnable->setEnabled(true);
            }
            else
            {
                m_buttonEnable->setEnabled(false);
            }
            m_buttonModify->setEnabled(false);
        }
        updateDetails();
    }
}

void PluginDialog::updateTable()
{
    if(m_plugins.size())
    {
        QTreeWidgetItem *current = NULL;
        m_pluginView->clear();
        for(size_t i=0; i<m_plugins.size(); i++)
        {
            QTreeWidgetItem *item = new QTreeWidgetItem(m_pluginView);
            item->setText(0, dxutils::fromStdString(m_plugins[i].getName()));
            item->setText(1, m_plugins[i].isLoaded()? _("Yes") : _("No"));
            item->setFlags(item->flags() & ~Qt::ItemIsEditable);
            if(i==0) current = item;
        }
        m_buttonEnable->setEnabled(true);
        if(m_plugins[0].isLoaded())
        {
            m_buttonEnable->setText(_("Di&sable"));
            if(m_plugins[0].hasOptions()) m_buttonModify->setEnabled(true);
            else m_buttonModify->setEnabled(false);
        }
        else
        {
            m_buttonEnable->setText(_("&Enable"));
            m_buttonModify->setEnabled(false);
        }
        if(current) m_pluginView->setCurrentItem(current);
    }
    else
    {
        m_buttonModify->setEnabled(false);
        m_buttonEnable->setEnabled(false);
    }
}

void PluginDialog::updateDetails()
{
    size_t row = (size_t)m_pluginView->currentIndex().row();
    if(row < m_plugins.size())
    {
        m_description->setText(dxutils::fromStdString(m_plugins[row].getDescription()));
        m_version->setText(dxutils::fromStdString(m_plugins[row].getVersion()));
        m_summary->setText(dxutils::fromStdString(m_plugins[row].getSummary()));
        m_author->setText(dxutils::fromStdString(m_plugins[row].getAuthor()));
        m_homepage->setText(QString("<a href=\"%1\">%1</a>").arg(dxutils::fromStdString(m_plugins[row].getHomepage())));
        m_path->setText(dxutils::fromStdString(m_plugins[row].getPath()));
    }
    else
    {
        m_description->setText("");
        m_version->setText("");
        m_summary->setText("");
        m_author->setText("");
        m_homepage->setText("");
        m_path->setText("");
    }
    resize(0, 0);
}

