/*
 *      plugineditordialog.cpp
 *
 *      Copyright 2013 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QDialogButtonBox>
#include <QComboBox>
#include <QFormLayout>
#include "plugineditordialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

/* Constructor - Modify existing plugin */
PluginEditorDialog::PluginEditorDialog(PURPLE::Plugin plugin, QWidget *parent) :
    QDialog(parent)
{
    this->m_plugin = plugin;
    this->m_options = plugin.getOptions();
    initializeEditor();
    setWindowTitle(_("Modify plugin"));
}

void PluginEditorDialog::initializeEditor()
{
    QLabel *label = new QLabel(QString("<b>%1</b>").arg(dxutils::fromStdString(m_plugin.getName())));
    QFormLayout *matrix = new QFormLayout;
    QLineEdit *text;
    QComboBox *combo;
    QCheckBox *check;
    QSpinBox *spin;
    FOREACH(PURPLE::Plugin::Options, m_options, it)
    {
        PURPLE::Plugin::Option& option = *it;
        std::vector<std::string> svalues = option.getStringList();
        std::map<std::string, int> ivalues = option.getIntList();
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                text = new QLineEdit;
                text->setText(dxutils::fromStdString(option.getStringValue()));
                m_optionsFields.append(text);
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), text);
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                combo = new QComboBox;
                std::vector<std::string>::iterator value = svalues.begin();
                for(; value!=svalues.end(); ++value)
                {
                    combo->addItem(dxutils::fromStdString(*value));
                }
                m_optionsFields.append(combo);
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), combo);
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                spin = new QSpinBox;
                spin->setRange(option.getIntMin(), option.getIntMax());
                spin->setValue(option.getIntValue());
                m_optionsFields.append(spin);
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), spin);
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                combo = new QComboBox;
                std::map<std::string, int>::iterator value = ivalues.begin();
                int j=0;
                for(; value!=ivalues.end(); ++value, j++)
                {
                    combo->addItem(dxutils::fromStdString((*value).first));
                    if(option.getIntValue() == (*value).second)
                        combo->setCurrentIndex(j);
                }
                m_optionsFields.append(combo);
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), combo);
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                check = new QCheckBox;
                check->setChecked(option.getBoolValue());
                m_optionsFields.append(check);
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), check);
                break;
            }
            case PURPLE::Plugin::Option::NONE:
                matrix->addRow(new QLabel(dxutils::normalizeLabelString(option.getText(), '_')), new QLabel(""));
                m_optionsFields.append(NULL);
                break;
        }
    }
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(this);
    mainFrame->addWidget(label);
    mainFrame->addLayout(matrix);
    mainFrame->addWidget(buttons);
    setLayout(mainFrame);
}

int PluginEditorDialog::getIntlistValue(std::map<std::string, int> map, std::string key)
{
    std::map<std::string, int>::iterator it = map.find(key);
    if(it!=map.end())
        return (*it).second;
    return -1;
}

void PluginEditorDialog::accept()
{
    saveProtocolOptions();
    QDialog::accept();
}

void PluginEditorDialog::saveProtocolOptions()
{
    PURPLE::Plugin::Options::iterator it;
    quint32 i;
    for(it=m_options.begin(), i=0; it!=m_options.end(); it++,i++)
    {
        PURPLE::Plugin::Option& option = *it;
        switch(option.getType())
        {
            case PURPLE::Plugin::Option::STR:
            {
                option.setValue(dxutils::toStdString(static_cast<QLineEdit*>(m_optionsFields[i])->text()));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::STRLIST:
            {
                QComboBox *combo = static_cast<QComboBox*>(m_optionsFields[i]);
                option.setValue(dxutils::toStdString(combo->currentText()));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INT:
            {
                option.setValue(PURPLE::t2s<int>(static_cast<QSpinBox*>(m_optionsFields[i])->value()));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::INTLIST:
            {
                QComboBox *combo = static_cast<QComboBox*>(m_optionsFields[i]);
                option.setValue(PURPLE::t2s<int>(getIntlistValue(option.getIntList(), dxutils::toStdString(combo->currentText()))));
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::BOOL:
            {
                option.setValue(static_cast<QCheckBox*>(m_optionsFields[i])->isChecked() ? "true" : "false");
                option.save();
                break;
            }
            case PURPLE::Plugin::Option::NONE:
                /* not supported. */
                break;
        }
    }
}

