/*
 *      dxim.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifdef _WIN32
  #define _WIN32_LEAN_AND_MEAN
  #include <windows.h>
#else
  #include <sys/file.h>
  #include <unistd.h>
  #include <stdlib.h>
#endif //_WIN32
#include <QApplication>
#include <QSettings>
#include <QDebug>
#include <QMenuBar>
#include <QCloseEvent>
#include <QInputDialog>
#include <QMessageBox>
#include <QVBoxLayout>
#include <QFileDialog>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QTranslator>
#include <QLibraryInfo>
#include <QStandardPaths>
#include "dxim.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "dxutils.h"
#include "tabitems/imtabitem.h"
#include "tabitems/chattabitem.h"
#include "tabitems/msgtabitem.h"
#include "dialogs/aboutdialog.h"
#include "dialogs/imdialog.h"
#include "dialogs/chatdialog.h"
#include "dialogs/chatjoindialog.h"
#include "dialogs/choicerequestdialog.h"
#include "dialogs/roomlistdialog.h"
#include "dialogs/inputrequestdialog.h"
#include "dialogs/actionrequestdialog.h"
#include "dialogs/fieldsrequestdialog.h"
#include "dialogs/accountdialog.h"
#include "dialogs/configdialog.h"
#include "dialogs/plugindialog.h"

#ifndef _WIN32
void dxunlink(void)
{
    unlink(QString(QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxim-qt-"+getenv("USER")).toLatin1().data());
}
#endif

// For handling Single Instance of Application
bool isInstanceRunningAlready(int argc, char *argv[])
{
#ifdef _WIN32
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex(NULL, FALSE,TEXT("dxim-qt.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxim already running"), TEXT("dxim"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        return TRUE;
    }
#else
    QString path = QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxim-qt-"+getenv("USER");
    struct flock fl;
    int fd;
    fd = open(path.toLatin1().data(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.toLatin1().data());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        QApplication a(argc, argv);
        QVBoxLayout *layout = new QVBoxLayout;
        layout->addWidget(new QLabel("dxim already running"));
        QPushButton *button = new QPushButton("&Close");
        QObject::connect(button, SIGNAL(clicked()), qApp, SLOT(quit()));
        layout->addWidget(button);
        QWidget window;
        window.setLayout(layout);
        window.show();
        a.exec();
        _exit(1);
    }
    atexit(dxunlink);
#endif
    return false;
}

//Constructor
dxim::dxim(bool offline, QWidget *parent):
    QMainWindow(parent), m_rosterModel(this), m_rosterSortModel(this)
{
    m_newMsg = false;
    m_presence = PURPLE_STATUS_OFFLINE;
    m_logDialog = NULL;

    m_cb = new dxqCallbacks();
#if !defined(QT_WITH_GLIB)
    qRegisterMetaType<QVector<QString> >("QVector<QString>");
    qRegisterMetaType<PURPLE::Account>("PURPLE::Account");
    qRegisterMetaType<PURPLE::Conversation>("PURPLE::Conversation");
    qRegisterMetaType<PURPLE::ChatBuddy>("PURPLE::ChatBuddy");
    qRegisterMetaType<QVector<PURPLE::ChatBuddy> >("QVector<PURPLE::ChatBuddy>");
    qRegisterMetaType<IMStateType>("IMStateType");
    qRegisterMetaType<PURPLE::Buddy>("PURPLE::Buddy");
    qRegisterMetaType<PurpleStatusPrimitive>("PurpleStatusPrimitive");
    qRegisterMetaType<PURPLE::Chat>("PURPLE::Chat");
    qRegisterMetaType<PURPLE::RoomList>("PURPLE::RoomList");
    qRegisterMetaType<PURPLE::InputRequest>("PURPLE::InputRequest");
    qRegisterMetaType<PURPLE::ActionRequest>("PURPLE::ActionRequest");
    qRegisterMetaType<PURPLE::FileRequest>("PURPLE::FileRequest");
    qRegisterMetaType<PURPLE::FieldsRequest>("PURPLE::FieldsRequest");
    qRegisterMetaType<PURPLE::FileTransfer>("PURPLE::FileTransfer");
#endif
    connect(m_cb, SIGNAL(signalPurpleError(QString)), this, SLOT(slotPurpleError(QString)));
    connect(m_cb, SIGNAL(signalBuddylistLoaded()), this, SLOT(slotBuddylistLoaded()));
    connect(m_cb, SIGNAL(signalAccountAdded(PURPLE::Account)), this, SLOT(slotAccountAdded(PURPLE::Account)));
    connect(m_cb, SIGNAL(signalAccountRemoved(QString,QString)), this, SLOT(slotAccountRemoved(QString,QString)));
    connect(m_cb, SIGNAL(signalAccountConnecting(PURPLE::Account,QString)), this, SLOT(slotConnecting(PURPLE::Account,QString)));
    connect(m_cb, SIGNAL(signalAccountConnected(PURPLE::Account)), this, SLOT(slotConnected(PURPLE::Account)));
    connect(m_cb, SIGNAL(signalAccountDisconnected(PURPLE::Account,QString)), this, SLOT(slotDisconnected(PURPLE::Account,QString)));
    connect(m_cb, SIGNAL(signalAccountStatusChanged(PURPLE::Account)), this, SLOT(slotAccountStatusChanged(PURPLE::Account)));
    connect(m_cb, SIGNAL(signalAccountSignedOn(PURPLE::Account)), this, SLOT(slotAccountSignedOn(PURPLE::Account)));
    connect(m_cb, SIGNAL(signalBuddySignedOn(PURPLE::Buddy)), this, SLOT(slotBuddySignedOn(PURPLE::Buddy)));
    connect(m_cb, SIGNAL(signalBuddySignedOff(PURPLE::Buddy)), this, SLOT(slotBuddySignedOff(PURPLE::Buddy)));
    connect(m_cb, SIGNAL(signalBuddyStatusChanged(PURPLE::Buddy,PurpleStatusPrimitive,PurpleStatusPrimitive)), this, SLOT(slotBuddyStatusChanged(PURPLE::Buddy,PurpleStatusPrimitive,PurpleStatusPrimitive)));
    connect(m_cb, SIGNAL(signalBuddyIconChanged(PURPLE::Buddy)), this, SLOT(slotBuddyIconChanged(PURPLE::Buddy)));
    connect(m_cb, SIGNAL(signalBuddyAdded(PURPLE::Buddy)), this, SLOT(slotBuddyAdded(PURPLE::Buddy)));
    connect(m_cb, SIGNAL(signalBuddyRemoved(PURPLE::Account,QString)), this, SLOT(slotBuddyRemoved(PURPLE::Account,QString)));
    connect(m_cb, SIGNAL(signalChatAdded(PURPLE::Chat)), this, SLOT(slotChatAdded(PURPLE::Chat)));
    connect(m_cb, SIGNAL(signalChatRemoved(PURPLE::Account,QString)), this, SLOT(slotChatRemoved(PURPLE::Account,QString)));
    connect(m_cb, SIGNAL(signalNewIM(PURPLE::Conversation*)), this, SLOT(slotNewIM(PURPLE::Conversation*)));
    connect(m_cb, SIGNAL(signalNewChat(PURPLE::Conversation*)), this, SLOT(slotNewChat(PURPLE::Conversation*)));
    connect(m_cb, SIGNAL(signalCloseConvTab(PURPLE::Conversation*)), this, SLOT(slotCloseConvTab(PURPLE::Conversation*)));
    connect(m_cb, SIGNAL(signalNewMessage(PURPLE::Conversation*,QString,QString,QString,bool,bool,bool)), this, SLOT(slotNewMsg(PURPLE::Conversation*,QString,QString,QString,bool,bool,bool)));
    connect(m_cb, SIGNAL(signalNewMyMessage(PURPLE::Conversation*,QString,QString,bool,QString)), this, SLOT(slotNewMyMsg(PURPLE::Conversation*,QString,QString,bool,QString)));
    connect(m_cb, SIGNAL(signalMessageRecived(PURPLE::Conversation*,QString)), this, SLOT(slotMessageRecived(PURPLE::Conversation*,QString)));
    connect(m_cb, SIGNAL(signalNewConvSystemMessage(PURPLE::Conversation*,QString,QString)), this, SLOT(slotNewConvSystemMessage(PURPLE::Conversation*,QString,QString)));
    connect(m_cb, SIGNAL(signalNewConvErrorMessage(PURPLE::Conversation*,QString,QString)), this, SLOT(slotNewConvErrorMessage(PURPLE::Conversation*,QString,QString)));
    connect(m_cb, SIGNAL(signalBuddyTyping(PURPLE::Conversation*,IMStateType)), this, SLOT(slotBuddyTyping(PURPLE::Conversation*,IMStateType)));
    connect(m_cb, SIGNAL(signalChatBuddyJoined(PURPLE::Conversation*,PURPLE::ChatBuddy)), this, SLOT(slotChatBuddyJoined(PURPLE::Conversation*,PURPLE::ChatBuddy)));
    connect(m_cb, SIGNAL(signalChatBuddyLeft(PURPLE::Conversation*,QString)), this, SLOT(slotChatBuddyLeft(PURPLE::Conversation*,QString)));
    connect(m_cb, SIGNAL(signalChatBuddyRenamed(PURPLE::Conversation*,QString,PURPLE::ChatBuddy)), this, SLOT(slotChatBuddyRenamed(PURPLE::Conversation*,QString,PURPLE::ChatBuddy)));
    connect(m_cb, SIGNAL(signalChatBuddyUpdated(PURPLE::Conversation*,QString)), this, SLOT(slotChatBuddyUpdated(PURPLE::Conversation*,QString)));
    connect(m_cb, SIGNAL(signalChatTopicChanged(PURPLE::Conversation*,QString,QString)), this, SLOT(slotChatTopicChanged(PURPLE::Conversation*,QString,QString)));
    connect(m_cb, SIGNAL(signalNewInputRequest(PURPLE::InputRequest*)), this, SLOT(slotNewInputRequest(PURPLE::InputRequest*)));
    connect(m_cb, SIGNAL(signalNewActionRequest(PURPLE::ActionRequest*)), this, SLOT(slotNewActionRequest(PURPLE::ActionRequest*)));
    connect(m_cb, SIGNAL(signalNewChoiceRequest(PURPLE::ChoiceRequest*)), this, SLOT(slotNewChoiceRequest(PURPLE::ChoiceRequest*)));
    connect(m_cb, SIGNAL(signalNewFieldsRequest(PURPLE::FieldsRequest*)), this, SLOT(slotNewFieldsRequest(PURPLE::FieldsRequest*)));
    connect(m_cb, SIGNAL(signalNewFileRequest(PURPLE::FileRequest*)), this, SLOT(slotNewFileRequest(PURPLE::FileRequest*)));
    connect(m_cb, SIGNAL(signalNewFT(PURPLE::FileTransfer*)), this, SLOT(slotCmdFileTransfer()));
    connect(m_cb, SIGNAL(signalNotifyMessage(QPixmap,QString,QString)), this, SLOT(slotNotifyMessage(QPixmap,QString,QString)));
    connect(m_cb, SIGNAL(signalNotifyEmail(QString,QString)), this, SLOT(slotNotifyEmail(QString,QString)));
    connect(m_cb, SIGNAL(signalNotifyURI(QString)), this, SLOT(slotNotifyURI(QString)));
    connect(m_cb, SIGNAL(signalNotifyUserinfo(PURPLE::Account,QString,QString)), this, SLOT(slotNotifyUserinfo(PURPLE::Account,QString,QString)));
    m_purple = new PURPLE::Purple(m_cb, "dxim", VERSION, "http://dxim.dxsolutions.org", dxutils::toStdString(DXPREFERENCES.getIniPath()));

    resize(DXPREFERENCES.m_w, DXPREFERENCES.m_h);
    move(DXPREFERENCES.m_x, DXPREFERENCES.m_y);
    if(DXPREFERENCES.m_maximized) showMaximized();
    setWindowIcon(QIcon(DXIM_DATADIR "/icons/dximsmall.png"));
    setWindowTitle("dxim");
    QWidget *widget = new QWidget;
    setCentralWidget(widget);
    QVBoxLayout *mainFrame = new QVBoxLayout;
    m_splitter = new QSplitter;
    m_tabbook = new TabWidget(this);
    m_tabbook->setTabPosition(QTabWidget::South);
    m_splitter->addWidget(m_tabbook);
    m_rosterSortModel.setSourceModel(&m_rosterModel);
    m_roster = new RosterView(m_purple, this);
    m_roster->setModel(&m_rosterSortModel);
    m_roster->setHeaderHidden(true);
    m_roster->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(m_roster, SIGNAL(signalCloseAccount(PURPLE::Account)), this, SLOT(slotCloseAccount(PURPLE::Account)));
    RosterItemDelegate *delegate = new RosterItemDelegate();
    m_roster->setItemDelegate(delegate);
    m_rosterSortModel.sort(0);
    m_splitter->addWidget(m_roster);
    QList<int> sizes;
    sizes << DXPREFERENCES.m_w-DXPREFERENCES.m_rosterWidth << DXPREFERENCES.m_rosterWidth;
    m_splitter->setSizes(sizes);
    mainFrame->addWidget(m_splitter);
    widget->setLayout(mainFrame);

    createActions();
    createMenus();
    createTrayIcon();

    m_trayIcon->show();

    connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(slotIconActivated(QSystemTrayIcon::ActivationReason)));
    connect(m_tabbook, SIGNAL(currentChanged(int)), this, SLOT(slotTabChanged(int)));
    connect(&m_idle, SIGNAL(secondsIdle(int)), this, SLOT(slotSecondsIdle(int)));

    installEventFilter(this);

    m_idle.start();

    m_ftTab = new FtTabItem(m_cb);
    connect(m_ftTab, SIGNAL(closeFtTabItem()), this, SLOT(slotCloseFtTabItem()));

    updateTabPosition();

    if(DXPREFERENCES.m_minToTray)
        hide();

    m_purple->initPurple(offline);
    m_purple->loadSavedPlugins("/dxim/plugins/loaded");
}

//Destructor
dxim::~dxim()
{
    DXPREFERENCES.m_x = x();
    DXPREFERENCES.m_y = y();
    DXPREFERENCES.m_w = width();
    DXPREFERENCES.m_h = height();
    DXPREFERENCES.m_maximized = isMaximized();
    DXPREFERENCES.m_rosterWidth = m_splitter->sizes()[1];
    DXPREFERENCES.save();
    m_idle.stop();
}

//handle click on close button
void dxim::closeEvent(QCloseEvent *ev)
{
    hide();
    ev->ignore();
}

void dxim::createActions()
{
    m_quitAction = new QAction(QIcon(":icons/quit"), _("&Quit"), this);
    connect(m_quitAction, SIGNAL(triggered()), this, SLOT(slotCmdQuit()));
    m_logsAction = new QAction(QIcon(DXIM_DATADIR "/icons/logs.png"), _("Log viewer"), this);
    if(!DXPREFERENCES.m_logging) m_logsAction->setEnabled(false);
    connect(m_logsAction, SIGNAL(triggered()), this, SLOT(slotCmdLogs()));
    m_closetabAction = new QAction(QIcon(DXIM_DATADIR "/icons/close.png"), _("Close tab"), this);
    m_closetabAction->setEnabled(false);
    connect(m_closetabAction, SIGNAL(triggered()), this, SLOT(slotCmdCloseTab()));
    m_clearAction = new QAction(QIcon(DXIM_DATADIR "/icons/clear.png"), _("Clear window"), this);
    m_clearAction->setEnabled(false);
    connect(m_clearAction, SIGNAL(triggered()), this, SLOT(slotCmdClear()));
    m_clearAllAction = new QAction(_("Clear all windows"), this);
    m_clearAllAction->setEnabled(false);
    connect(m_clearAllAction, SIGNAL(triggered()), this, SLOT(slotCmdClearAll()));
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    m_logsAction->setShortcut(Qt::CTRL|Qt::Key_G);
    m_closetabAction->setShortcut(Qt::CTRL|Qt::Key_W);
    m_clearAction->setShortcut(Qt::CTRL|Qt::Key_L);
    m_clearAllAction->setShortcut(Qt::CTRL|Qt::SHIFT|Qt::Key_L);
#else
    m_logsAction->setShortcut(Qt::CTRL+Qt::Key_G);
    m_closetabAction->setShortcut(Qt::CTRL+Qt::Key_W);
    m_clearAction->setShortcut(Qt::CTRL+Qt::Key_L);
    m_clearAllAction->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_L);
#endif
}

void dxim::createMenus()
{
    m_fileMenu = menuBar()->addMenu(_("&dxim"));
    QAction *manageaccountsAction = new QAction(_("Manage accounts"), this);
    manageaccountsAction->setShortcut(Qt::CTRL+Qt::Key_A);
    connect(manageaccountsAction, SIGNAL(triggered()), this, SLOT(slotCmdManageAccounts()));
    m_fileMenu->addAction(manageaccountsAction);
    m_fileMenu->addSeparator();
    connect(m_fileMenu->addAction( _("Add buddy")), SIGNAL(triggered()), this, SLOT(slotCmdAddBuddy()));
    connect(m_fileMenu->addAction( _("Add chat")), SIGNAL(triggered()), this, SLOT(slotCmdAddChat()));
    connect(m_fileMenu->addAction( _("Add group")), SIGNAL(triggered()), this, SLOT(slotCmdAddGroup()));
    m_fileMenu->addSeparator();
    QAction *newimAction = new QAction(_("New IM"), this);
    newimAction->setShortcut(Qt::CTRL+Qt::Key_M);
    connect(newimAction, SIGNAL(triggered()), this, SLOT(slotCmdNewIM()));
    m_fileMenu->addAction(newimAction);
    QAction *joinchatAction = new QAction(_("Join chat"), this);
    joinchatAction->setShortcut(Qt::CTRL+Qt::Key_T);
    connect(joinchatAction, SIGNAL(triggered()), this, SLOT(slotCmdJoinChat()));
    m_fileMenu->addAction(joinchatAction);
    m_fileMenu->addSeparator();
    QAction *filetransferAction = new QAction(QIcon(DXIM_DATADIR "/icons/ft.png"), _("File transfers"), this);
    filetransferAction->setShortcut(Qt::CTRL+Qt::Key_F);
    connect(filetransferAction, SIGNAL(triggered()), this, SLOT(slotCmdFileTransfer()));
    m_fileMenu->addAction(filetransferAction);
    m_fileMenu->addSeparator();
    m_fileMenu->addAction(m_logsAction);
    m_fileMenu->addSeparator();
    m_fileMenu->addAction(m_quitAction);

    m_editMenu = menuBar()->addMenu(_("&Edit"));
    m_editMenu->addAction(m_closetabAction);
    m_editMenu->addSeparator();
    m_editMenu->addAction(m_clearAction);
    m_editMenu->addAction(m_clearAllAction);
    m_editMenu->addSeparator();
    QAction *rosterAction = new QAction(_("Friends list"), this);
    rosterAction->setShortcut(Qt::CTRL+Qt::Key_U);
    rosterAction->setCheckable(true);
    rosterAction->setChecked(DXPREFERENCES.m_rosterShown);
    connect(rosterAction, SIGNAL(triggered()), this, SLOT(slotCmdRoster()));
    m_editMenu->addAction(rosterAction);
    QAction *chatAction = new QAction(_("Friends list in chats"), this);
    chatAction->setCheckable(true);
    chatAction->setChecked(DXPREFERENCES.m_chatRosterShown);
    connect(chatAction, SIGNAL(triggered()), this, SLOT(slotCmdChatRoster()));
    m_editMenu->addAction(chatAction);
    QAction *avatarsAction = new QAction(_("Show avatars"), this);
    avatarsAction->setCheckable(true);
    avatarsAction->setChecked(DXPREFERENCES.m_showAvatars);
    connect(avatarsAction, SIGNAL(triggered()), this, SLOT(slotCmdAvatars()));
    m_editMenu->addAction(avatarsAction);
    QAction *hideOfflineAction = new QAction(_("Hide offline friends"), this);
    hideOfflineAction->setCheckable(true);
    hideOfflineAction->setChecked(DXPREFERENCES.m_hideOffline);
    connect(hideOfflineAction, SIGNAL(triggered()), this, SLOT(slotCmdHideOffline()));
    connect(hideOfflineAction, SIGNAL(triggered(bool)), &m_rosterSortModel, SLOT(slotShowOfflineBuddy(bool)));
    m_editMenu->addAction(hideOfflineAction);
    QAction *statusAction = new QAction(_("Show friends status message"), this);
    statusAction->setCheckable(true);
    statusAction->setChecked(DXPREFERENCES.m_showStatusmsg);
    connect(statusAction, SIGNAL(triggered()), this, SLOT(slotCmdShowStatusmsg()));
    m_editMenu->addAction(statusAction);
    m_editMenu->addSeparator();
    QAction *pluginsAction = new QAction(_("Plugins"), this);
    connect(pluginsAction, SIGNAL(triggered()), this, SLOT(slotCmdPlugins()));
    m_editMenu->addAction(pluginsAction);
    QAction *preferencesAction = new QAction(_("&Preferences"), this);
    connect(preferencesAction, SIGNAL(triggered()), this, SLOT(slotCmdPreferences()));
    m_editMenu->addAction(preferencesAction);

    m_statusMenu = menuBar()->addMenu(_("&Status"));
    connect(m_statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/online.png"), _("Available")), SIGNAL(triggered()), this, SLOT(slotCmdOnline()));
    connect(m_statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/away.png"), _("Away")), SIGNAL(triggered()), this, SLOT(slotCmdAway()));
    connect(m_statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/dnd.png"), _("Busy")), SIGNAL(triggered()), this, SLOT(slotCmdDnd()));
    connect(m_statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/invisible.png"), _("Invisible")), SIGNAL(triggered()), this, SLOT(slotCmdInvisible()));
    m_statusMenu->addSeparator();
    connect(m_statusMenu->addAction(_("Set status message")), SIGNAL(triggered()), this, SLOT(slotCmdStatusMsg()));
    m_statusMenu->addSeparator();
    connect(m_statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/offline.png"), _("Offline")), SIGNAL(triggered()), this, SLOT(slotCmdOffline()));
    m_statusMenu->addSeparator();
    connect(m_statusMenu->addAction(_("Reconnect")), SIGNAL(triggered()), this, SLOT(slotCmdReconnect()));

    m_helpMenu = menuBar()->addMenu(_("&Help"));
    connect(m_helpMenu->addAction(QIcon(DXIM_DATADIR "/icons/dxim.png"), _("&About...")), SIGNAL(triggered()), this, SLOT(slotCmdAbout()));
    connect(m_helpMenu->addAction(QPixmap(":/trolltech/qmessagebox/images/qtlogo-64.png"), _("About &Qt")), SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void dxim::createTrayIcon()
{
    QMenu *statusMenu = new QMenu(_("&Status"), this);
    connect(statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/online.png"), _("Available")), SIGNAL(triggered()), this, SLOT(slotCmdOnline()));
    connect(statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/away.png"), _("Away")), SIGNAL(triggered()), this, SLOT(slotCmdAway()));
    connect(statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/dnd.png"), _("Busy")), SIGNAL(triggered()), this, SLOT(slotCmdDnd()));
    connect(statusMenu->addAction(QIcon(DXIM_DATADIR "/icons/statuses/invisible.png"), _("Invisible")), SIGNAL(triggered()), this, SLOT(slotCmdInvisible()));
    statusMenu->addSeparator();
    connect(statusMenu->addAction(_("Set status message")), SIGNAL(triggered()), this, SLOT(slotCmdStatusMsg()));
    m_trayMenu = new QMenu(this);
    m_trayMenu->addMenu(statusMenu);
    m_trayMenu->addSeparator();
    connect(m_trayMenu->addAction(_("New IM")), SIGNAL(triggered()), this, SLOT(slotCmdNewIM()));
    connect(m_trayMenu->addAction(_("Join chat")), SIGNAL(triggered()), this, SLOT(slotCmdJoinChat()));
    m_trayMenu->addSeparator();
    m_trayMenu->addAction(m_quitAction);
    m_trayIcon = new QSystemTrayIcon(this);
    m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/offline.png"));
    m_trayIcon->setContextMenu(m_trayMenu);
    m_trayIcon->setToolTip("dxim");
}

bool dxim::eventFilter(QObject *obj, QEvent *ev)
{
    if(ev->type() == QEvent::KeyPress)
    {
        QKeyEvent *e = (QKeyEvent *)ev;
        //next unread tab
        if(e->key() == Qt::Key_N && e->modifiers() == Qt::ControlModifier)
        {
            int index = m_tabbook->currentIndex();
            for(int i=index; i<m_tabbook->count(); i++)
            {
                if(m_tabbook->tabTextColor(i) != palette().windowText().color())
                {
                    m_tabbook->setCurrentIndex(i);
                    return true;
                }
            }
            for(int i=index; i>-1; i--)
            {
                if(m_tabbook->tabTextColor(i) != palette().windowText().color())
                {
                    m_tabbook->setCurrentIndex(i);
                    return true;
                }
            }
            return true;
        }
    }
    return QWidget::eventFilter(obj, ev);
}

dxqCallbacks *dxim::getCallbacks()
{
    return m_cb;
}

void dxim::slotCmdManageAccounts()
{
    AccountDialog *dialog = new AccountDialog(m_purple, this);
    dialog->setModal(true);
    dialog->exec();
    m_rosterModel.updateAccountsTooltip();
}

void dxim::slotCmdAddBuddy()
{
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }

    std::vector<PURPLE::Account> accounts = m_purple->getConnectedAccountsList();
    PURPLE::Account account;
    QDialog adialog(this);
    QGridLayout *amatrix = new QGridLayout;
    QComboBox *ids = new QComboBox;
    ids->setInsertPolicy(QComboBox::NoInsert);
    for(uint i=0; i<accounts.size(); i++)
    {
        ids->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID()))), dxutils::fromStdString(accounts[i].getUsername()));
    }
    amatrix->addWidget(new QLabel(_("Account:")), 0, 0);
    amatrix->addWidget(ids, 0, 1);
    QDialogButtonBox *abuttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(abuttons, SIGNAL(accepted()), &adialog, SLOT(accept()));
    connect(abuttons, SIGNAL(rejected()), &adialog, SLOT(reject()));
    QVBoxLayout *frame = new QVBoxLayout(&adialog);
    frame->addLayout(amatrix);
    frame->addWidget(abuttons);
    adialog.setLayout(frame);
    adialog.setWindowTitle(_("Select account"));
    if(adialog.exec()==QDialog::Accepted)
    {
        account = accounts[ids->currentIndex()];
    }
    else
    {
        return;
    }

    QDialog dialog(this);
    QGridLayout *matrix = new QGridLayout;
    matrix->addWidget(new QLabel(_("Account:")), 0, 0);
    matrix->addWidget(new QLabel(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(account.getProtocol().getPurpleID()))).arg(dxutils::fromStdString(account.getUsername()))), 0, 1);
    matrix->addWidget(new QLabel(_("ID:")), 1, 0);
    QLineEdit *id = new QLineEdit;
    matrix->addWidget(id, 1, 1);
    matrix->addWidget(new QLabel(_("Nick:")), 2, 0);
    QLineEdit *nick = new QLineEdit;
    matrix->addWidget(nick, 2, 1);
    matrix->addWidget(new QLabel(_("Group:")), 3, 0);
    QComboBox *groups = new QComboBox;
    std::vector<std::string> v = account.getAllGroups();
    bool friends = false;
    FOREACH(std::vector<std::string>, v, it)
    {
        if(dxutils::fromStdString(*it) == _("Friends"))
            friends = true;
        groups->addItem(dxutils::fromStdString(*it));
    }
    if(!friends) groups->insertItem(0, _("Friends"));
    groups->setEditable(true);
    groups->setCurrentIndex(0);
    matrix->addWidget(groups, 3, 1);
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
    QVBoxLayout *mainFrame = new QVBoxLayout(&dialog);
    mainFrame->addLayout(matrix);
    mainFrame->addWidget(buttons);
    dialog.setLayout(mainFrame);
    dialog.setWindowTitle(_("Add friend"));
    if(dialog.exec()==QDialog::Accepted
        && !id->text().isEmpty())
    {
        std::string group = dxutils::toStdString(groups->currentText());
        if(group.empty()) group = dxutils::toStdString(_("Friends"));
        account.addBuddy(dxutils::toStdString(id->text()),
                         group,
                         dxutils::toStdString(_("I want to add you in buddies")),
                         dxutils::toStdString(nick->text()));
    }
}

void dxim::slotCmdAddChat()
{
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }

    std::vector<PURPLE::Account> accounts = m_purple->getConnectedAccountsList();
    PURPLE::Account account;
    QDialog adialog(this);
    QGridLayout *amatrix = new QGridLayout;
    QComboBox *ids = new QComboBox;
    ids->setInsertPolicy(QComboBox::NoInsert);
    for(uint i=0; i<accounts.size(); i++)
    {
        ids->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID()))), dxutils::fromStdString(accounts[i].getUsername()));
    }
    amatrix->addWidget(new QLabel(_("Account:")), 0, 0);
    amatrix->addWidget(ids, 0, 1);
    QDialogButtonBox *abuttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(abuttons, SIGNAL(accepted()), &adialog, SLOT(accept()));
    connect(abuttons, SIGNAL(rejected()), &adialog, SLOT(reject()));
    QVBoxLayout *frame = new QVBoxLayout(&adialog);
    frame->addLayout(amatrix);
    frame->addWidget(abuttons);
    adialog.setLayout(frame);
    adialog.setWindowTitle(_("Select account"));
    if(adialog.exec()==QDialog::Accepted)
    {
        account = accounts[ids->currentIndex()];
    }
    else
    {
        return;
    }

    ChatDialog *dialog = new ChatDialog(account, this);
    dialog->setModal(true);
    dialog->exec();
}

void dxim::slotCmdAddGroup()
{
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }

    std::vector<PURPLE::Account> accounts = m_purple->getConnectedAccountsList();
    PURPLE::Account account;
    QDialog adialog(this);
    QGridLayout *amatrix = new QGridLayout;
    QComboBox *ids = new QComboBox;
    ids->setInsertPolicy(QComboBox::NoInsert);
    for(uint i=0; i<accounts.size(); i++)
    {
        ids->addItem(QIcon(dxutils::protocolToIcon(dxutils::fromStdString(accounts[i].getProtocol().getPurpleID()))), dxutils::fromStdString(accounts[i].getUsername()));
    }
    amatrix->addWidget(new QLabel(_("Account:")), 0, 0);
    amatrix->addWidget(ids, 0, 1);
    QDialogButtonBox *abuttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(abuttons, SIGNAL(accepted()), &adialog, SLOT(accept()));
    connect(abuttons, SIGNAL(rejected()), &adialog, SLOT(reject()));
    QVBoxLayout *frame = new QVBoxLayout(&adialog);
    frame->addLayout(amatrix);
    frame->addWidget(abuttons);
    adialog.setLayout(frame);
    adialog.setWindowTitle(_("Select account"));
    if(adialog.exec()==QDialog::Accepted)
    {
        account = accounts[ids->currentIndex()];
    }
    else
    {
        return;
    }

    bool ok;
    QString group = QInputDialog::getText(this, _("Add group"), _("New group:"), QLineEdit::Normal, "", &ok);
    if(!ok)
        return;
    account.addGroup(dxutils::toStdString(group));
}

//handle m_quitAction
void dxim::slotCmdQuit()
{
    qApp->quit();
}

//handle m_trayIcon click
void dxim::slotIconActivated(QSystemTrayIcon::ActivationReason reason)
{
    if(reason == QSystemTrayIcon::Trigger)
    {
        if(this->isVisible())
        {
            DXPREFERENCES.m_maximized = this->isMaximized();
            this->hide();
        }
        else
        {
            if(DXPREFERENCES.m_maximized) this->showMaximized();
            else this->showNormal();
            this->raise();
            this->activateWindow();
        }
        updateTrayIconPresence();
    }
}

//update presence by menu items
void dxim::updatePresence(PurpleStatusPrimitive presence)
{
    m_presence = presence;
    updateTrayIconPresence();
    std::vector<PURPLE::Account> accounts = m_purple->getEnabledAccountsList();
    for(uint i=0; i<accounts.size(); i++)
        accounts[i].setStatus(m_presence);
}

void dxim::slotCmdReconnect()
{
    std::vector<PURPLE::Account> accounts = m_purple->getEnabledAccountsList();
    for(uint i=0; i<accounts.size(); i++)
    {
        if(accounts[i].isConnecting())
            continue;
        if(accounts[i].isConnected())
        {
            accounts[i].disconnect();
        }
        accounts[i].connect();
    }
}

void dxim::slotCmdOnline()
{
    updatePresence(PURPLE_STATUS_AVAILABLE);
}

void dxim::slotCmdAway()
{
    updatePresence(PURPLE_STATUS_AWAY);
}

void dxim::slotCmdDnd()
{
    updatePresence(PURPLE_STATUS_UNAVAILABLE);
}

void dxim::slotCmdInvisible()
{
    updatePresence(PURPLE_STATUS_INVISIBLE);
}

void dxim::slotCmdOffline()
{
    updatePresence(PURPLE_STATUS_OFFLINE);
}

void dxim::slotCmdStatusMsg()
{
    if(!this->isVisible()) this->hide();
    {
        if(this->isMaximized()) this->showMaximized();
        else this->showNormal();
        this->raise();
        this->activateWindow();
    }
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }
    bool ok;
    QString msg = QInputDialog::getText(this, _("Change status message"), _("Status message:"), QLineEdit::Normal, "", &ok);
    if(!ok)
        return;
    std::vector<PURPLE::Account> accounts = m_purple->getEnabledAccountsList();
    for(uint i=0; i<accounts.size(); i++)
        if(accounts[i].isConnected())
            accounts[i].setStatusMessage(dxutils::toStdString(msg));
}

void dxim::slotCmdAbout()
{
    AboutDialog *about = new AboutDialog(dxutils::fromStdString(m_purple->getPurplemmVersion()), dxutils::fromStdString(m_purple->getLibpurpleVersion()),this);
    about->setModal(true);
    about->show();
}

void dxim::slotCmdCloseTab()
{
    int index = m_tabbook->currentIndex();
    ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(index));
    if(im)
    {
        m_tabbook->removeTab(index);
        im->closeConversation();
        return;
    }
    ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(index));
    if(chat)
    {

        m_tabbook->removeTab(index);
        chat->getConversation()->leave();
        return;
    }
    FtTabItem *ft = qobject_cast<FtTabItem*>(m_tabbook->widget(index));
    if(ft)
    {
        m_tabbook->removeTab(index);
        slotUpdateMenu();
        return;
    }
    MsgTabItem *msg = qobject_cast<MsgTabItem*>(m_tabbook->widget(index));
    if(msg)
    {
        m_tabbook->removeTab(index);
        msg->deleteLater();
        slotUpdateMenu();
        return;
    }
}

void dxim::slotCloseFtTabItem()
{
    m_tabbook->removeTab(m_tabbook->currentIndex());
    slotUpdateMenu();
}

void dxim::slotCmdClear()
{
    ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->currentWidget());
    if(im)
    {
        im->clearText();
        return;
    }
    ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->currentWidget());
    if(chat)
        chat->clearText();
}

void dxim::slotCmdClearAll()
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            im->clearText();
            m_tabbook->setTabIcon(i, QIcon(im->getConversation()->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?DXIM_DATADIR "/icons/imoff.png":DXIM_DATADIR "/icons/im.png"));
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat)
        {
            chat->clearText();
            m_tabbook->setTabIcon(i, QIcon(DXIM_DATADIR "/icons/chat.png"));
        }
        m_tabbook->setTabTextColor(i, palette().windowText().color());
    }
}

void dxim::slotCmdRoster()
{
    DXPREFERENCES.m_rosterShown = !DXPREFERENCES.m_rosterShown;
    if(DXPREFERENCES.m_rosterShown)
        m_roster->show();
    else
        m_roster->hide();
}

void dxim::slotCmdChatRoster()
{
    DXPREFERENCES.m_chatRosterShown = !DXPREFERENCES.m_chatRosterShown;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat)
        {
            chat->showHideRoster();
        }
    }
}

void dxim::slotCmdAvatars()
{
    DXPREFERENCES.m_showAvatars = !DXPREFERENCES.m_showAvatars;
    m_rosterModel.buddySizeHint();
    m_roster->update();
}

void dxim::slotCmdHideOffline()
{
    DXPREFERENCES.m_hideOffline = !DXPREFERENCES.m_hideOffline;
}

void dxim::slotCmdShowStatusmsg()
{
    DXPREFERENCES.m_showStatusmsg = !DXPREFERENCES.m_showStatusmsg;
    m_rosterModel.buddySizeHint();
    m_roster->update();
}


void dxim::slotCmdPlugins()
{
    PluginDialog *dialog = new PluginDialog(m_purple, this);
    dialog->setModal(true);
    dialog->exec();
}

void dxim::slotCmdPreferences()
{
    QColor unread = DXPREFERENCES.m_unreadtabColor;
    QColor high = DXPREFERENCES.m_hltabColor;
    bool showRosterTree = DXPREFERENCES.m_showRosterTree;
    ConfigDialog *dialog = new ConfigDialog(this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        if(showRosterTree != DXPREFERENCES.m_showRosterTree)
        {
            m_rosterModel.clear();
            fillRoster();
            m_roster->expandAll();
        }
        updateTabPosition();
        slotUpdateMenu();
        for(int i=0; i<m_tabbook->count(); i++)
        {
            ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
            if(im)
            {
                im->updatePreferences();
            }
            ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
            if(chat)
            {
                chat->updatePreferences();
            }
            MsgTabItem *msg = qobject_cast<MsgTabItem*>(m_tabbook->widget(i));
            if(msg)
            {
                msg->updateTheme();
            }
            if(m_tabbook->tabTextColor(i)==unread) m_tabbook->setTabTextColor(i, DXPREFERENCES.m_unreadtabColor);
            if(m_tabbook->tabTextColor(i)==high) m_tabbook->setTabTextColor(i, DXPREFERENCES.m_hltabColor);
        }
        m_roster->updateTheme();
        m_rosterModel.updateStatusIconTheme();
        m_purple->setIdle(DXPREFERENCES.m_useAway, DXPREFERENCES.m_awayTimeout);
        if(m_logDialog) m_logDialog->setLogPath(DXPREFERENCES.m_logPath);
    }
}

void dxim::slotSecondsIdle(int sec)
{
    bool away = false;
    if(sec>DXPREFERENCES.m_awayTimeout*60 && DXPREFERENCES.m_useAway)
    {
        away = true;
    }
    if(!away && !sec) m_purple->idleTouch();
}

void dxim::slotPurpleError(QString error)
{
    MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/errorbig.png"), "", error, _("&Ok"));
    m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Error"));
    connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
    updateTabPosition();
    slotUpdateMenu();
}

void dxim::slotBuddylistLoaded()
{
    fillRoster();
    m_roster->expandAll();
}

void dxim::fillRoster()
{
    std::vector<PURPLE::Account> accounts = m_purple->getAccountsList();
    for(uint i=0; i<accounts.size(); i++)
    {
        if(DXPREFERENCES.m_showRosterTree)
        {
            RosterItem *accountItem = addAccount(accounts[i]);
            std::vector<std::string> g = accounts[i].getGroups();
            std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
            std::vector<PURPLE::Chat> ch = accounts[i].getChats();
            for(guint j=0; j<g.size(); j++)
            {
                RosterItem *groupItem = 0;
                if(!m_rosterModel.hasRosterItem(RosterItem::Group, dxutils::fromStdString(g[j]), accounts[i]))
                {
                    groupItem = new RosterItem(dxutils::fromStdString(g[j]),
                                           dxutils::fromStdString(g[j]),
                                           RosterItem::Group, accounts[i]);
                    groupItem->setFlags(groupItem->flags() & ~Qt::ItemIsEditable);
                    m_rosterModel.addRosterItem(accountItem, groupItem);
                }
                else
                    groupItem = m_rosterModel.getRosterItem(RosterItem::Group, dxutils::fromStdString(g[j]), accounts[i]);
                for(guint k=0; k<b.size(); k++)
                {
                    if(g[j] == b[k].getGroupName())
                    {
                        RosterItem *buddyItem = new RosterItem(dxutils::fromStdString(b[k].getAlias()),
                                                   dxutils::fromStdString(b[k].getUsername()),
                                                   RosterItem::Buddy, accounts[i]);
                        buddyItem->setBuddy(b[k]);
                        buddyItem->setFlags(buddyItem->flags() & ~Qt::ItemIsEditable);
                        m_rosterModel.addRosterItem(groupItem, buddyItem);
                    }
                }
                for(guint m=0; m<ch.size(); m++)
                {
                    if(g[j] == ch[m].getGroupName())
                    {
                        RosterItem *chatItem = new RosterItem(dxutils::fromStdString(ch[m].getAlias()),
                                                  dxutils::fromStdString(ch[m].getAlias()),
                                                  RosterItem::Chat, accounts[i]);
                        chatItem->setChat(ch[m]);
                        chatItem->setFlags(chatItem->flags() & ~Qt::ItemIsEditable);
                        m_rosterModel.addRosterItem(groupItem, chatItem);
                    }
                }
            }
        }
        else
        {
            std::vector<PURPLE::Buddy> b = accounts[i].getBuddies();
            std::vector<PURPLE::Chat> ch = accounts[i].getChats();
            for(guint j=0; j<b.size(); j++)
            {
                RosterItem *buddyItem = new RosterItem(dxutils::fromStdString(b[j].getAlias()),
                                           dxutils::fromStdString(b[j].getUsername()),
                                           RosterItem::Buddy, accounts[i]);
                buddyItem->setBuddy(b[j]);
                buddyItem->setFlags(buddyItem->flags() & ~Qt::ItemIsEditable);
                m_rosterModel.addRosterItem(0, buddyItem);
            }
            for(guint k=0; k<ch.size(); k++)
            {
                RosterItem *chatItem = new RosterItem(dxutils::fromStdString(ch[k].getAlias()),
                                          dxutils::fromStdString(ch[k].getAlias()),
                                          RosterItem::Chat, accounts[i]);
                chatItem->setChat(ch[k]);
                chatItem->setFlags(chatItem->flags() & ~Qt::ItemIsEditable);
                m_rosterModel.addRosterItem(0, chatItem);
            }
        }
    }
    m_rosterSortModel.invalidate();
}

void dxim::slotAccountAdded(PURPLE::Account account)
{
    addAccount(account);
}

void dxim::slotAccountRemoved(QString account, QString protocolID)
{
    m_rosterModel.removeAccountItem(account, protocolID);
}

void dxim::slotConnecting(PURPLE::Account account, QString msg)
{
    dxutils::debugLine(_("Connecting %1-%2: %3").arg(dxutils::fromStdString(account.getProtocol().getName())).arg(dxutils::fromStdString(account.getUsername())).arg(msg));
    m_rosterModel.accountConnecting(account);
}

void dxim::slotConnected(PURPLE::Account account)
{
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountConnect) dxutils::instance().showNotify(_("Connected %1-%2").arg(dxutils::fromStdString(account.getProtocol().getName())).arg(dxutils::fromStdString(account.getUsername())));
    if(m_presence == PURPLE_STATUS_OFFLINE)
    {
        m_presence = PURPLE_STATUS_AVAILABLE;
        updateTrayIconPresence();
    }
    //hack for disable idle after connected
    account.setStatus(m_presence);
    m_rosterModel.accountPresence(account);
    for(int i=m_tabbook->count()-1; i>-1; i--)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            if(im->getConversation()->getAccount() == account)
            {
                im->appendStyledText("", _("%1 [%2]: Connected").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())), DXPREFERENCES.m_userColor, false);
            }
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat)
        {
            if(chat->getConversation()->getAccount() == account)
            {
                chat->appendStyledText("", _("%1 [%2]: Connected").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())), DXPREFERENCES.m_userColor, false);
            }
        }
    }
}

void dxim::slotDisconnected(PURPLE::Account account, QString reason)
{
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyAccountDisconnect)
    {
        if(reason.isEmpty()) dxutils::instance().showNotify(_("Disconnected %1-%2").arg(dxutils::fromStdString(account.getProtocol().getName())).arg(dxutils::fromStdString(account.getUsername())));
        else dxutils::instance().showNotify(_("Disconnected %1-%2\n%3").arg(dxutils::fromStdString(account.getProtocol().getName())).arg(dxutils::fromStdString(account.getUsername())).arg(reason));
    }
    m_rosterModel.accountPresence(account);
    slotCloseAccount(account);
    for(int i=m_tabbook->count()-1; i>-1; i--)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            if(im->getConversation()->getAccount() == account)
            {
                im->appendStyledText("", _("%1 [%2]: Disconnected").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())), DXPREFERENCES.m_userColor, false);
            }
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat)
        {
            if(chat->getConversation()->getAccount() == account)
            {
                chat->appendStyledText("", _("%1 [%2]: Disconnected").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())), DXPREFERENCES.m_userColor, false);
            }
        }
    }
}

void dxim::slotAccountStatusChanged(PURPLE::Account account)
{
    m_rosterModel.accountPresence(account);
}

void dxim::slotAccountSignedOn(PURPLE::Account account)
{
    RosterItem *accountItem = m_rosterModel.getRosterItem(RosterItem::Account, dxutils::fromStdString(account.getUsername()), account);
    std::vector<std::string> g = account.getGroups();
    std::vector<PURPLE::Buddy> b = account.getBuddies();
    std::vector<PURPLE::Chat> ch = account.getChats();
    for(guint j=0; j<g.size(); j++)
    {
        RosterItem *groupItem = 0;
        if(DXPREFERENCES.m_showRosterTree)
        {
            if(!m_rosterModel.hasRosterItem(RosterItem::Group, dxutils::fromStdString(g[j]), account))
            {
                groupItem = new RosterItem(dxutils::fromStdString(g[j]),
                                       dxutils::fromStdString(g[j]),
                                       RosterItem::Group, account);
                groupItem->setFlags(groupItem->flags() & ~Qt::ItemIsEditable);
                m_rosterModel.addRosterItem(accountItem, groupItem);
            }
            else
                groupItem = m_rosterModel.getRosterItem(RosterItem::Group, dxutils::fromStdString(g[j]), account);
        }
        for(guint k=0; k<b.size(); k++)
        {
            if(g[j] == b[k].getGroupName())
            {
                RosterItem *buddyItem = 0;
                if(!m_rosterModel.hasRosterItem(RosterItem::Buddy, dxutils::fromStdString(b[k].getUsername()), account))
                {
                    buddyItem = new RosterItem(dxutils::fromStdString(b[k].getAlias()),
                                               dxutils::fromStdString(b[k].getUsername()),
                                               RosterItem::Buddy, account);
                    buddyItem->setBuddy(b[k]);
                    buddyItem->setFlags(buddyItem->flags() & ~Qt::ItemIsEditable);
                    m_rosterModel.addRosterItem(groupItem, buddyItem);
                }
                else
                {
                    buddyItem = m_rosterModel.getRosterItem(RosterItem::Buddy, dxutils::fromStdString(b[k].getUsername()), account);
                    buddyItem->buddyStatusChanged();
                }
            }
        }
        for(guint m=0; m<ch.size(); m++)
        {
            if(g[j] == ch[m].getGroupName())
            {
                if(!m_rosterModel.hasRosterItem(RosterItem::Chat, dxutils::fromStdString(ch[m].getAlias()), account))
                {
                    RosterItem *chatItem = new RosterItem(dxutils::fromStdString(ch[m].getAlias()),
                                              dxutils::fromStdString(ch[m].getAlias()),
                                              RosterItem::Chat, account);
                    chatItem->setChat(ch[m]);
                    chatItem->setFlags(chatItem->flags() & ~Qt::ItemIsEditable);
                    m_rosterModel.addRosterItem(groupItem, chatItem);
                }
            }
        }
        m_rosterSortModel.invalidate();
        if(groupItem) m_roster->expand(m_rosterSortModel.mapFromSource(groupItem->index()));
    }
}

void dxim::slotBuddySignedOn(PURPLE::Buddy buddy)
{
    m_rosterModel.buddySignedOn(buddy);
    m_rosterSortModel.invalidate();
    if(DXPREFERENCES.m_showRosterTree) m_roster->expand(m_rosterSortModel.mapFromSource(m_rosterModel.getRosterItem(RosterItem::Group, dxutils::fromStdString(buddy.getGroupName()), buddy.getAccount())->index()));
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            im->onBuddySignedOn(buddy);
        }
    }
}

void dxim::slotBuddySignedOff(PURPLE::Buddy buddy)
{
    m_rosterModel.buddySignedOff(buddy);
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            im->onBuddySignedOff(buddy);
        }
    }
}

void dxim::slotBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive oldStatus, PurpleStatusPrimitive newStatus)
{
    Q_UNUSED(oldStatus)
    m_rosterModel.buddyStatusChanged(buddy);
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im)
        {
            im->onBuddyStatusChanged(buddy, newStatus);
        }
    }
}

void dxim::slotBuddyIconChanged(PURPLE::Buddy buddy)
{
    m_rosterModel.buddyIconChanged(buddy);
}

void dxim::slotBuddyAdded(PURPLE::Buddy buddy)
{
    RosterItem *accountItem = m_rosterModel.getRosterItem(RosterItem::Account, dxutils::fromStdString(buddy.getAccount().getUsername()), buddy.getAccount());
    RosterItem *buddyItem = new RosterItem(dxutils::fromStdString(buddy.getAlias()),
                                           dxutils::fromStdString(buddy.getUsername()),
                                           RosterItem::Buddy, buddy.getAccount());
    buddyItem->setBuddy(buddy);
    buddyItem->setFlags(buddyItem->flags() & ~Qt::ItemIsEditable);
    RosterItem *groupItem = 0;
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(!m_rosterModel.hasRosterItem(RosterItem::Group, dxutils::fromStdString(buddy.getGroupName()), buddy.getAccount()))
        {
            groupItem = new RosterItem(dxutils::fromStdString(buddy.getGroupName()),
                                       dxutils::fromStdString(buddy.getGroupName()),
                                       RosterItem::Group, buddy.getAccount());
            groupItem->setFlags(groupItem->flags() & ~Qt::ItemIsEditable);
            m_rosterModel.addRosterItem(accountItem, groupItem);
        }
        else
            groupItem = m_rosterModel.getRosterItem(RosterItem::Group, dxutils::fromStdString(buddy.getGroupName()), buddy.getAccount());
    }
    m_rosterModel.addRosterItem(groupItem, buddyItem);
    m_rosterSortModel.invalidate();
    if(groupItem) m_roster->expand(m_rosterSortModel.mapFromSource(groupItem->index()));
}

void dxim::slotBuddyRemoved(PURPLE::Account account, QString buddy)
{
    m_rosterModel.removeRosterItem(RosterItem::Buddy, buddy, account);
}

void dxim::slotChatAdded(PURPLE::Chat chat)
{
    PURPLE::Account account = chat.getAccount();
    RosterItem *accountItem = m_rosterModel.getRosterItem(RosterItem::Account, dxutils::fromStdString(account.getUsername()), account);
    RosterItem *chatItem = new RosterItem(dxutils::fromStdString(chat.getAlias()),
                                          dxutils::fromStdString(chat.getAlias()),
                                          RosterItem::Chat, account);
    chatItem->setChat(chat);
    chatItem->setFlags(chatItem->flags() & ~Qt::ItemIsEditable);
    RosterItem *groupItem = 0;
    if(DXPREFERENCES.m_showRosterTree)
    {
        if(!m_rosterModel.hasRosterItem(RosterItem::Group, dxutils::fromStdString(chat.getGroupName()), account))
        {
            groupItem = new RosterItem(dxutils::fromStdString(chat.getGroupName()),
                                       dxutils::fromStdString(chat.getGroupName()),
                                       RosterItem::Group, account);
            groupItem->setFlags(groupItem->flags() & ~Qt::ItemIsEditable);
            m_rosterModel.addRosterItem(accountItem, groupItem);
        }
        else
            groupItem = m_rosterModel.getRosterItem(RosterItem::Group, dxutils::fromStdString(chat.getGroupName()), account);
    }
    m_rosterModel.addRosterItem(groupItem, chatItem);
    m_rosterSortModel.invalidate();
    if(groupItem) m_roster->expand(m_rosterSortModel.mapFromSource(groupItem->index()));
}

void dxim::slotChatRemoved(PURPLE::Account account, QString chat)
{
    m_rosterModel.removeRosterItem(RosterItem::Chat, chat, account);
}

void dxim::slotNewIM(PURPLE::Conversation *conv)
{
    createIMTab(conv);
    slotUpdateMenu();
}

void dxim::slotNewChat(PURPLE::Conversation *conv)
{
    createChatTab(conv);
    slotUpdateMenu();
}

void dxim::slotCloseConvTab(PURPLE::Conversation *conv)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && *im->getConversation()==*conv)
        {
            m_tabbook->removeTab(i);
            im->deleteLater();
            break;
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && *chat->getConversation()==*conv)
        {
            m_tabbook->removeTab(i);
            chat->deleteLater();
            break;
        }
    }
    slotUpdateMenu();
}

void dxim::slotNewMsg(PURPLE::Conversation *conv, QString sender, QString msg, QString time, bool highlight, bool history, bool action)
{
    if(!history && !isVisible() && DXPREFERENCES.m_notifyMessageTray)
    {
        updateTrayIconNewmsg();
    }
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onNewMsg(sender, msg, time, highlight, history, action);
            return;
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onNewMsg(sender, msg, time, highlight, history, action);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        createIMTab(conv)->onNewMsg(sender, msg, time, highlight, history, action);
        return;
    }
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onNewMsg(sender, msg, time, highlight, history, action);
    }
}

void dxim::slotNewMyMsg(PURPLE::Conversation *conv, QString msg, QString time, bool action, QString messageID)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onNewMyMsg(msg, time, action, messageID);
            return;
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onNewMyMsg(msg, time, action);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        createIMTab(conv)->onNewMyMsg(msg, time, action, messageID);
        return;
    }
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onNewMyMsg(msg, time, action);
    }
}

void dxim::slotMessageRecived(PURPLE::Conversation *conv, QString messageID)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onMessageRecived(messageID);
            return;
        }
    }
}

void dxim::slotNewConvSystemMessage(PURPLE::Conversation *conv, QString msg, QString time)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onNewConvSystemMessage(msg, time);
            return;
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onNewConvSystemMessage(msg, time);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        createIMTab(conv)->onNewConvSystemMessage(msg, time);
        return;
    }
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onNewConvSystemMessage(msg, time);
    }
}

void dxim::slotNewConvErrorMessage(PURPLE::Conversation *conv, QString msg, QString time)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onNewConvErrorMessage(msg, time);
            return;
        }
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onNewConvErrorMessage(msg, time);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        createIMTab(conv)->onNewConvErrorMessage(msg, time);
        return;
    }
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onNewConvErrorMessage(msg, time);
    }
}

void dxim::slotBuddyTyping(PURPLE::Conversation *conv, IMStateType typing)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(i));
        if(im && im->getConversation()==conv)
        {
            im->onBuddyTyping(typing);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_IM)
    {
        createIMTab(conv)->onBuddyTyping(typing);
    }
}

void dxim::slotChatBuddyJoined(PURPLE::Conversation *conv, PURPLE::ChatBuddy buddy)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onChatBuddyJoined(buddy);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onChatBuddyJoined(buddy);
    }
}

void dxim::slotChatBuddyUpdated(PURPLE::Conversation *conv, QString buddy)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            if(!conv->hasChatBuddy(dxutils::toStdString(buddy))) return;
            PURPLE::ChatBuddy cb = conv->findChatBuddy(dxutils::toStdString(buddy));
            if(cb.isValid()) chat->onChatBuddyUpdated(cb);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        if(!conv->hasChatBuddy(dxutils::toStdString(buddy))) return;
        ChatTabItem *chat = createChatTab(conv);
        PURPLE::ChatBuddy cb = conv->findChatBuddy(dxutils::toStdString(buddy));
        if(cb.isValid()) chat->onChatBuddyUpdated(cb);
    }
}

void dxim::slotChatBuddyLeft(PURPLE::Conversation *conv, QString buddy)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onChatBuddyLeft(buddy);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onChatBuddyLeft(buddy);
    }
}

void dxim::slotChatBuddyRenamed(PURPLE::Conversation *conv, QString who, PURPLE::ChatBuddy nbuddy)
{
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onChatBuddyRenamed(who, nbuddy);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onChatBuddyRenamed(who, nbuddy);
    }
}

void dxim::slotChatTopicChanged(PURPLE::Conversation *conv, QString who, QString topic)
{
    Q_UNUSED(who)
    for(int i=0; i<m_tabbook->count(); i++)
    {
        ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(i));
        if(chat && chat->getConversation()==conv)
        {
            chat->onChatTopicChanged(topic);
            return;
        }
    }
    //we haven't tab
    if(conv->getType()==PURPLE_CONV_TYPE_CHAT)
    {
        createChatTab(conv)->onChatTopicChanged(topic);
    }
}

void dxim::slotNewInputRequest(PURPLE::InputRequest *request)
{
    InputRequestDialog *dialog = new InputRequestDialog(request, this);
    dialog->show();
}

void dxim::slotNewActionRequest(PURPLE::ActionRequest *request)
{
    ActionRequestDialog *dialog = new ActionRequestDialog(request, this);
    dialog->show();
}

void dxim::slotNewChoiceRequest(PURPLE::ChoiceRequest *request)
{
    ChoiceRequestDialog *dialog = new ChoiceRequestDialog(request, this);
    dialog->show();
}

void dxim::slotNewFieldsRequest(PURPLE::FieldsRequest *request)
{
    FieldsRequestDialog *dialog = new FieldsRequestDialog(request, this);
    dialog->show();
}

void dxim::slotNewFileRequest(PURPLE::FileRequest *request)
{
    if(request->getSaveDialog())
    {
        QString fileName = QFileDialog::getSaveFileName(this, dxutils::fromStdString(request->getTitle()), dxutils::fromStdString(request->getFilename()));
        if(!fileName.isEmpty()) request->setFilename(dxutils::toStdString(fileName));
        else request->dennyRequest();
    }
    else
    {
        QString fileName = QFileDialog::getOpenFileName(this, dxutils::fromStdString(request->getTitle()), dxutils::fromStdString(request->getFilename()));
        if(!fileName.isEmpty()) request->setFilename(dxutils::toStdString(fileName));
        else request->dennyRequest();
    }
}

void dxim::slotNotifyMessage(QPixmap icon, QString title, QString text)
{
    MsgTabItem *tab = new MsgTabItem(icon, "", text, _("&Ok"));
    m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), title);
    connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
    updateTabPosition();
    slotUpdateMenu();
}

void dxim::slotNotifyEmail(QString text, QString url)
{
    MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/infobig.png"), "", text, _("&Show in browser"), url);
    m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/email.png"), _("New email"));
    connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
    updateTabPosition();
    slotUpdateMenu();
}

void dxim::slotNotifyURI(QString uri)
{
    MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/infobig.png"), "", uri, _("&Show in browser"), uri);
    m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Link"));
    connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
    updateTabPosition();
    slotUpdateMenu();
}

void dxim::slotNotifyUserinfo(PURPLE::Account account, QString buddy, QString info)
{
    QString id = QString("%1-%2-%3").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())).arg(buddy);
    MsgTabItem *tab = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        MsgTabItem *msg = qobject_cast<MsgTabItem*>(m_tabbook->widget(i));
        if(msg)
        {
            if(id == msg->getID())
            {
                tab = msg;
                break;
            }
        }
    }
    if(tab)
    {
        tab->insertText(info);
    }
    else
    {
        tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/infobig.png"), id, info, _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), buddy);
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
    }
    slotUpdateMenu();
}

RosterItem* dxim::addAccount(PURPLE::Account account)
{
    RosterItem *item = new RosterItem(dxutils::fromStdString(account.getUsername()),
                                      dxutils::fromStdString(account.getUsername()),
                                      RosterItem::Account, account);
    item->setFlags(item->flags() & ~Qt::ItemIsEditable);
    m_rosterModel.addRosterItem(0, item);
    return item;
}

void dxim::slotCloseAccount(PURPLE::Account /*account*/)
{
    if(!m_purple->isSomeAccountConnected())
    {
        m_presence = PURPLE_STATUS_OFFLINE;
        if(!m_newMsg) updateTrayIconPresence();
    }
}

void dxim::updateTabPosition()
{
    switch(DXPREFERENCES.m_tabPosition) {
    case 0: //bottom
        m_tabbook->setTabPosition(QTabWidget::South);
        break;
    case 1: //left
        m_tabbook->setTabPosition(QTabWidget::West);
        break;
    case 2: //top
        m_tabbook->setTabPosition(QTabWidget::North);
        break;
    case 3: //right
        m_tabbook->setTabPosition(QTabWidget::East);
        break;
    default:
        m_tabbook->setTabPosition(QTabWidget::South);
    }
}

void dxim::slotUpdateMenu()
{
    if(DXPREFERENCES.m_logging) m_logsAction->setEnabled(true);
    else m_logsAction->setEnabled(false);
    if(m_tabbook->count())
    {
        m_closetabAction->setEnabled(true);
        m_clearAction->setEnabled(true);
        m_clearAllAction->setEnabled(true);
    }
    else
    {
        m_closetabAction->setEnabled(false);
        m_clearAction->setEnabled(false);
        m_clearAllAction->setEnabled(false);
    }
}

//update trayicon for presence
void dxim::updateTrayIconPresence(PurpleStatusPrimitive presence)
{
    if(presence == PURPLE_STATUS_UNSET) presence = m_presence;
    switch(presence){
        case PURPLE_STATUS_AVAILABLE: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/online.png")); break;
        case PURPLE_STATUS_INVISIBLE: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/invisible.png")); break;
        case PURPLE_STATUS_AWAY: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/away.png")); break;
        case PURPLE_STATUS_UNAVAILABLE: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/dnd.png")); break;
        case PURPLE_STATUS_OFFLINE: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/offline.png")); break;
        case PURPLE_STATUS_EXTENDED_AWAY: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/idle.png")); break;
        //some presence mising
        default: m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/statuses/online.png")); break;
    }
    m_newMsg = false;
}

void dxim::updateTrayIconNewmsg()
{
    m_trayIcon->setIcon(QIcon(DXIM_DATADIR "/icons/newm.png"));
    m_newMsg = true;
}

//create ImTabItem
ImTabItem* dxim::createIMTab(PURPLE::Conversation *conv)
{
    ImTabItem *tab = new ImTabItem(conv);
    connect(tab, SIGNAL(highlight()), this, SLOT(slotImTabHighlight()));
    connect(tab, SIGNAL(composing(bool)), this, SLOT(slotImTabComposing(bool)));
    connect(tab, SIGNAL(offline(bool)), this, SLOT(slotImTabOffline(bool)));
    connect(tab, SIGNAL(newmsg()), this, SLOT(slotImTabNewmsg()));
    connect(tab, SIGNAL(newIMName(QString)), this, SLOT(slotImTabNewName(QString)));
    connect(tab, SIGNAL(showLog(PURPLE::Account,QString,QString)),this, SLOT(slotShowLog(PURPLE::Account,QString,QString)));
    m_tabbook->addTab(tab, QIcon(conv->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?DXIM_DATADIR "/icons/imoff.png":DXIM_DATADIR "/icons/im.png"), dxutils::fromStdString(conv->getTitle()));
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
//    m_tabbook->setCurrentWidget(tab);
    updateTabPosition();
    return tab;
}

//create ChatTabItem
ChatTabItem* dxim::createChatTab(PURPLE::Conversation *conv)
{
    ChatTabItem *tab = new ChatTabItem(conv);
    connect(tab, SIGNAL(highlight()), this, SLOT(slotChatTabHighlight()));
    connect(tab, SIGNAL(newmsg()), this, SLOT(slotChatTabNewmsg()));
    connect(tab, SIGNAL(showLog(PURPLE::Account,QString,QString)),this, SLOT(slotShowLog(PURPLE::Account,QString,QString)));
    m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/chat.png"), dxutils::fromStdString(conv->getTitle()));
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    updateTabPosition();
    return tab;
}

void dxim::slotCmdNewIM()
{
    if(!this->isVisible()) this->hide();
    {
        if(this->isMaximized()) this->showMaximized();
        else this->showNormal();
        this->raise();
        this->activateWindow();
    }
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }
    std::vector<PURPLE::Account> accounts = m_purple->getConnectedAccountsList();
    ImDialog *dialog = new ImDialog(accounts, this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        QString ID, protocol, IMID, IMnick;
        dialog->fillIMInfo(ID, protocol, IMID, IMnick);
        if(IMID.isEmpty()) return;
        for(uint i=0; i<accounts.size(); i++)
        {
            if(dxutils::fromStdString(accounts[i].getUsername())==ID && dxutils::fromStdString(accounts[i].getProtocol().getPurpleID())==protocol)
            {
                std::vector<PURPLE::Buddy> v = accounts[i].getBuddies();
                PURPLE::Buddy buddy;
                for(uint j=0; j<v.size(); j++)
                {
                    if(dxutils::fromStdString(v[j].getUsername())==IMID)
                    {
                        buddy = v[j];
                        break;
                        if(IMnick.isEmpty()) IMnick = IMID;
                        m_purple->createConversation(accounts[i], buddy);
                        return;
                    }
                }
                m_purple->createConversation(accounts[i], dxutils::toStdString(IMID), false);
            }
        }
    }
}

void dxim::slotCmdJoinChat()
{
    if(!this->isVisible()) this->hide();
    {
        if(this->isMaximized()) this->showMaximized();
        else this->showNormal();
        this->raise();
        this->activateWindow();
    }
    if(!m_purple->isSomeAccountConnected())
    {
        MsgTabItem *tab = new MsgTabItem(QPixmap(DXIM_DATADIR "/icons/warningbig.png"), "", _("Any account isn't connected.\nConnect someone and try again."), _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/info.png"), _("Warning"));
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
        slotUpdateMenu();
        return;
    }
    ChatJoinDialog *dialog = new ChatJoinDialog(m_purple->getConnectedAccountsList(), m_cb, this);
//    dialog->setModal(true);
//    dialog->exec();
    dialog->show();
}

void dxim::slotCmdFileTransfer()
{
    m_tabbook->addTab(m_ftTab, QIcon(DXIM_DATADIR "/icons/ft.png"), _("File transfers"));
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    m_tabbook->setCurrentWidget(m_ftTab);
    updateTabPosition();
    slotUpdateMenu();
}

void dxim::slotCmdLogs()
{
    if(!m_logDialog)
    {
        m_logDialog = new LogDialog(this);
    }
    m_logDialog->show();
    m_logDialog->raise();
    m_logDialog->activateWindow();
}

void dxim::slotChatTabHighlight()
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if(!isVisible())
    {
        if(DXPREFERENCES.m_raiseWindow)
        {
            if(DXPREFERENCES.m_maximized) showMaximized();
            else showNormal();
            raise();
            activateWindow();
            updateTrayIconPresence();
            QApplication::alert(this);
        }
        else
        {
            updateTrayIconNewmsg();
        }
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New highlighted message on %1").arg(m_tabbook->tabText(index)));
        if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    }
    if(index==m_tabbook->currentIndex()) return;
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New highlighted message on %1").arg(m_tabbook->tabText(index)));
    if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    m_tabbook->setTabIcon(index, QIcon(DXIM_DATADIR "/icons/chatnewm.png"));
    m_tabbook->setTabTextColor(index, DXPREFERENCES.m_hltabColor);
}

void dxim::slotChatTabNewmsg()
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if(index==m_tabbook->currentIndex()) return;
    m_tabbook->setTabIcon(index, QIcon(DXIM_DATADIR "/icons/chatnewm.png"));
    if(m_tabbook->tabTextColor(index)!=DXPREFERENCES.m_hltabColor) m_tabbook->setTabTextColor(index, DXPREFERENCES.m_unreadtabColor);
}

void dxim::slotImTabHighlight()
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if(!isVisible())
    {
        if(DXPREFERENCES.m_raiseWindow)
        {
            if(DXPREFERENCES.m_maximized) showMaximized();
            else showNormal();
            raise();
            activateWindow();
            updateTrayIconPresence();
            QApplication::alert(this);
        }
        else
        {
            updateTrayIconNewmsg();
        }
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New message on IM with %1").arg(m_tabbook->tabText(index)));
        if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    }
    if(index==m_tabbook->currentIndex()) return;
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New message on IM with %1").arg(m_tabbook->tabText(index)));
    if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    m_tabbook->setTabIcon(index, QIcon(DXIM_DATADIR "/icons/imnewm.png"));
    m_tabbook->setTabTextColor(index, DXPREFERENCES.m_hltabColor);
}

void dxim::slotImTabNewmsg()
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if(!isVisible())
    {
        if(DXPREFERENCES.m_raiseWindow)
        {
            if(DXPREFERENCES.m_maximized) showMaximized();
            else showNormal();
            raise();
            activateWindow();
            updateTrayIconPresence();
            QApplication::alert(this);
        }
        else
        {
            updateTrayIconNewmsg();
        }
        if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New message on IM with %1").arg(m_tabbook->tabText(index)));
        if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    }
    if(index==m_tabbook->currentIndex()) return;
    if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyMessage) dxutils::instance().showNotify(_("New message on IM with %1").arg(m_tabbook->tabText(index)));
    if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundMessage) dxutils::playFile(DXPREFERENCES.m_pathMessage);
    m_tabbook->setTabIcon(index, QIcon(DXIM_DATADIR "/icons/imnewm.png"));
    if(m_tabbook->tabTextColor(index)!=DXPREFERENCES.m_hltabColor) m_tabbook->setTabTextColor(index, DXPREFERENCES.m_unreadtabColor);
}

void dxim::slotImTabComposing(bool compose)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    ImTabItem *im = qobject_cast<ImTabItem*>(sender());
    m_tabbook->setTabIcon(index, compose?QIcon(DXIM_DATADIR "/icons/imcompose.png"):QIcon(im->getConversation()->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?DXIM_DATADIR "/icons/imoff.png":DXIM_DATADIR "/icons/im.png"));
}

void dxim::slotImTabOffline(bool offline)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    m_tabbook->setTabIcon(index, offline?QIcon(DXIM_DATADIR "/icons/imoff.png"):QIcon(DXIM_DATADIR "/icons/im.png"));
}

void dxim::slotImTabNewName(const QString &name)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    m_tabbook->setTabText(index, name);
}

void dxim::slotTabChanged(int index)
{
    ImTabItem *im = qobject_cast<ImTabItem*>(m_tabbook->widget(index));
    if(im)
    {
        m_tabbook->setTabIcon(index, QIcon(im->getConversation()->getIMBuddyStatus()==PURPLE_STATUS_OFFLINE?DXIM_DATADIR "/icons/imoff.png":DXIM_DATADIR "/icons/im.png"));
        im->focusCommandline();
    }
    ChatTabItem *chat = qobject_cast<ChatTabItem*>(m_tabbook->widget(index));
    if(chat)
    {
        m_tabbook->setTabIcon(index, QIcon(DXIM_DATADIR "/icons/chat.png"));
        chat->focusCommandline();
    }
    m_tabbook->setTabTextColor(index, palette().windowText().color());
}

void dxim::slotCloseMsgTabItem()
{
    QWidget *tab = m_tabbook->currentWidget();
    m_tabbook->removeTab(m_tabbook->currentIndex());
    tab->deleteLater();
    slotUpdateMenu();
}

void dxim::slotShowLog(PURPLE::Account account, const QString &name, const QString &path)
{
    QString id = QString("Log-%1-%2-%3").arg(dxutils::fromStdString(account.getUsername())).arg(dxutils::fromStdString(account.getProtocol().getName())).arg(name);
    MsgTabItem *tab = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        MsgTabItem *msg = qobject_cast<MsgTabItem*>(m_tabbook->widget(i));
        if(msg)
        {
            if(id == msg->getID())
            {
                tab = msg;
                break;
            }
        }
    }
    if(!tab)
    {
        tab = new MsgTabItem(id, _("&Ok"));
        m_tabbook->addTab(tab, QIcon(DXIM_DATADIR "/icons/logs.png"), name);
        connect(tab, SIGNAL(signalMsgTabClosed()), this, SLOT(slotCloseMsgTabItem()));
        updateTabPosition();
    }
    tab->showLog(path);
    m_tabbook->setCurrentWidget(tab);
    slotUpdateMenu();
}

//load Qt translation
void loadTranslation()
{
    QString langPath = "";
    QStringList paths;
    QString appdir = QCoreApplication::applicationDirPath();
    paths.append(appdir);
    paths.append(appdir + "/../share/" + QCoreApplication::applicationName().toLower());
    paths.append(appdir + "/../Resources");
    foreach(const QString& path, paths)
    {
        if(QFile::exists(path + "/translations/"))
        {
            langPath = path + "/translations/";
            break;
        }
    }
    QString lang = QLocale::system().name();
    static QTranslator qt_translator;
    if(!langPath.isEmpty())
    {
        if(!qt_translator.load("qt_"+lang, langPath))
        {
            qt_translator.load("qt_"+lang.left(2), langPath);
        }
    }
    else
    {
        if(qt_translator.load("qt_"+lang, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
        {
            qt_translator.load("qt_"+lang.left(2), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
        }
    }
    QCoreApplication::installTranslator(&qt_translator);
}

const QString USAGE_MSG = _("\
\nUsage: dxim [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [PATH]          Use PATH for configuration.\n\
        -o, --offline      Start with all accounts set offline.\n\
\n");

int main(int argc, char *argv[])
{
    bool offline = false;
    for(int i=0; i<argc; ++i)
    {
        if(QString::compare(argv[i],"-v")==0 || QString::compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
            exit(0);
        }
        if(QString::compare(argv[i],"-h")==0 || QString::compare(argv[i],"--help")==0)
        {
            fprintf(stdout, "%s\n", USAGE_MSG.toLatin1().constData());
            exit(0);
        }
        if(QString::compare(argv[i],"-l")==0)
        {
            DXPREFERENCES.setIniPath(argv[i+1]);
        }
        if(QString::compare(argv[i],"-o")==0 || QString::compare(argv[i],"--offline")==0)
        {
            offline = true;
        }
    }
#ifdef ENABLE_NLS
    bindtextdomain("dxim", LOCALEDIR);
    bind_textdomain_codeset("dxim", "UTF-8");
    bind_textdomain_codeset("pidgin", "UTF-8");
    textdomain("dxim");
#endif //ENABLE_NLS
    DXPREFERENCES.load();
    if(DXPREFERENCES.m_oneInstance && isInstanceRunningAlready(argc, argv))
        return 0;
    QApplication a(argc, argv);
    QApplication::setOrganizationName("dxSolutions");
    QApplication::setApplicationName(PACKAGE);
    QApplication::setApplicationVersion(VERSION);
    loadTranslation(); //load qt library translation
    dxim w(offline);
    if(DXPREFERENCES.m_minToTray) w.hide();
    else w.show();
    return a.exec();
}
