/*
 *      dxutils.cpp
 *
 *      Copyright 2011 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QColor>
#include <QBuffer>
#include <QFile>
#include <QDebug>
#include <QTime>
#include <QDir>
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
#include <QRegExp>
#include <QSound>
#else
#include <QRegularExpression>
#endif
#include <string>
#include "preferences.h"
#include "dxutils.h"
#include "notify/notifywindow.h"
#include "config.h"
#include "i18n.h"

const QString tags[] = { "<img","<b>","</b>","<u>","</u>","<i>","</i>","<em>","</em>","<a","</a>","<span","</span>",
                        "<br>", "<strong>", "</strong>" };

const int numTags = (sizeof(tags)/sizeof(tags[0]));

const QString removeTags[] = { "<html", "</html>", "<body", "</body>", "<p>", "</p>" };

const int numRemoveTags = (sizeof(removeTags)/sizeof(removeTags[0]));

dxutils::dxutils()
{
    m_notifiesHeight = 0;
    m_notifyPosition = -1;
    m_firstNotify = false;
    m_haveNotify = false;
#ifndef _WIN32
    QDir dir;
    QStringList paths = QString(getenv("PATH")).split(':');
    foreach(QString path, paths)
    {
        if(dir.exists(path+"/notify-send"))
        {
            m_haveNotify = true;
            break;
        }
    }
#endif // _WIN32
}

dxutils &dxutils::instance()
{
    static dxutils idxutils;
    return idxutils;
}

// checks is char nick/word delimiter
bool dxutils::isDelimiter(char c)
{
    switch(c) {
        case ' ':
        case '.':
        case ',':
        case '/':
        case '\\':
        case '`':
        case '\'':
        case '!':
        case '(':
        case ')':
        case '{':
        case '}':
        case '|':
        case '[':
        case ']':
        case '\"':
        case ':':
        case ';':
        case '<':
        case '>':
        case '?':
            return true;
        default:
            return false;
    }
}

//Return file size in human readable form
QString dxutils::getFileSize(quint64 size)
{
    double fsize = 0.0;
    if(size > 1000000000)
    {
        fsize = size/1073741824.0;
        return QString("%1 %2").arg(fsize, 0, 'f', 2).arg(_("GB"));
    }
    if(size > 100000)
    {
        fsize = size/1048576.0;
        return QString("%1 %2").arg(fsize, 0, 'f', 2).arg(_("MB"));
    }
    if(size > 1000)
    {
        fsize = size/1024.0;
        return QString("%1 %2").arg(fsize, 0, 'f', 2).arg(_("KB"));
    }
    return QString("%1 %2").arg(size).arg(_("bytes"));
}

//std::string to QString
QString dxutils::fromStdString(const std::string &str)
{
    return QString::fromUtf8(str.c_str());
}

//QString to std::string
std::string dxutils::toStdString(const QString &str)
{
    return str.toUtf8().constData();
}

//presence to QIcon
QString dxutils::presenceToIcon(PurpleStatusPrimitive presence)
{
    switch(presence) {
    case PURPLE_STATUS_UNSET:
    case PURPLE_STATUS_OFFLINE:
    {
        return DXPREFERENCES.m_themeStatusPath+"/offline.png";
    }
    case PURPLE_STATUS_UNAVAILABLE:
    {
        return DXPREFERENCES.m_themeStatusPath+"/dnd.png";
    }
    case PURPLE_STATUS_INVISIBLE:
    {
        return DXPREFERENCES.m_themeStatusPath+"/invisible.png";
    }
    case PURPLE_STATUS_AWAY:
    {
        return DXPREFERENCES.m_themeStatusPath+"/away.png";
    }
    case PURPLE_STATUS_EXTENDED_AWAY:
    {
        return DXPREFERENCES.m_themeStatusPath+"/idle.png";
    }
    case PURPLE_STATUS_AVAILABLE:
    case PURPLE_STATUS_MOBILE:
    case PURPLE_STATUS_TUNE:
    default:
    {
        return DXPREFERENCES.m_themeStatusPath+"/online.png";
    }
    }
    return DXIM_DATADIR "/icons/empty.png";
}

QString dxutils::protocolToIcon(const QString &protocol)
{
    if(protocol.contains("jabber")) return DXIM_DATADIR "/icons/protocols/xmpp.png";
    else if(protocol.contains("msn")) return DXIM_DATADIR "/icons/protocols/wlm.png";
    else if(protocol.contains("irc")) return DXIM_DATADIR "/icons/protocols/irc.png";
    else if(protocol.contains("icq") || protocol.contains("icyque")) return DXIM_DATADIR "/icons/protocols/icq.png";
    else if(protocol.contains("aim")) return DXIM_DATADIR "/icons/protocols/aim.png";
    else if(protocol.contains("bonjour")) return DXIM_DATADIR "/icons/protocols/bonjour.png";
    else if(protocol.contains("gg")) return DXIM_DATADIR "/icons/protocols/gadu-gadu.png";
    else if(protocol.contains("novell")) return DXIM_DATADIR "/icons/protocols/novell.png";
    else if(protocol.contains("myspace")) return DXIM_DATADIR "/icons/protocols/myspace.png";
    else if(protocol.contains("silc")) return DXIM_DATADIR "/icons/protocols/silc.png";
    else if(protocol.contains("meanwhile")) return DXIM_DATADIR "/icons/protocols/meanwhile.png";
    else if(protocol.contains("simple")) return DXIM_DATADIR "/icons/protocols/simple.png";
    else if(protocol.contains("yahoo")) return DXIM_DATADIR "/icons/protocols/yahoo.png";
    else if(protocol.contains("skype")) return DXIM_DATADIR "/icons/protocols/skype.png";
    else if(protocol.contains("mxit")) return DXIM_DATADIR "/icons/protocols/mxit.png";
    else if(protocol.contains("zephyr")) return DXIM_DATADIR "/icons/protocols/zephyr.png";
    else if(protocol.contains("telegram")) return DXIM_DATADIR "/icons/protocols/telegram.png";
    else if(protocol.contains("slack")) return DXIM_DATADIR "/icons/protocols/slack.png";
    else return DXIM_DATADIR "/icons/empty.png";
}

QString dxutils::getNickColor(const QString &nick)
{
    int len = nick.length();
    quint32 h = 0;
    for(int i=0; i<len; i++)
        h = ((h << 5) + h) ^ nick[i].toLatin1();
    switch(h%8)
    {
        case 0: return QColor(196,160,0).name();
        case 1: return QColor(206,92,0).name();
        case 2: return QColor(143,89,2).name();
        case 3: return QColor(78,154,6).name();
        case 4: return QColor(32,74,135).name();
        case 5: return QColor(117,80,123).name();
        case 6: return QColor(164,0,0).name();
        default: return QColor(85,87,83).name();
    }
}

//imstate to human readable string
QString dxutils::imStateToString(const QString &nick, IMStateType type)
{
    switch(type){
        case STARTED:
            return _("%1 is composing.").arg(nick);
        case PAUSED:
            return _("%1 paused compose.").arg(nick);
        case STOPED:
            return _("%1 stoped compose.").arg(nick);
        case LEFTIM:
            return _("%1 left conversation.").arg(nick);
        default:
            return "";
    }
}

/*Return download/send speed in human readable form
 * speed = difference between two position during 1 second
 */
QString dxutils::getSpeed(quint64 speed)
{
    double fspeed = 0.0;
    if(speed > 100000)
    {
        fspeed = speed/(1048576.0);
        return _("%1 MB/s").arg(fspeed, 0, 'f', 2);
    }
    fspeed = speed/(1024.0);
    return _("%1 KB/s").arg(fspeed, 0, 'f', 2);
}

//Return remaining time of download/send
QString dxutils::getRemaining(quint64 size, quint64 speed)
{
    if(!speed) return "?";
    quint64 remainingTime = size/speed;
    if(remainingTime <= 0 || remainingTime > 54000) return "?";
    else
    {
        quint64 secs = remainingTime;
        quint64 hours = remainingTime/3600;
        secs -= hours*3600;
        quint64 mins = secs/60;
        secs -= mins*60;
        return QString("%1:%2:%3").arg(hours).arg(mins, 2, 10, QLatin1Char('0')).arg(secs, 2, 10, QLatin1Char('0'));
    }
}

QImage dxutils::getImageFromPath(const QString &path, int maxSize)
{
    QFile f(path);
    QByteArray imageData;
    if(f.exists() && f.size() > 0 && f.open(QIODevice::ReadOnly))
    {
        imageData = f.readAll();
        f.close();
    }
    if(!imageData.isEmpty())
    {
        QImage img;
        img.loadFromData(imageData);
        if(maxSize>0 && (img.width()>maxSize || img.height()>maxSize))
        {
            img = img.scaled(maxSize, maxSize, Qt::KeepAspectRatio);
        }
        return img;
    }
    else
    {
        return QImage();
    }
}

void dxutils::showNotify(QString text, int position)
{
    if(m_haveNotify)
    {
        QString exec = QString("notify-send -i %1 dxim '%2'").arg(DXIM_DATADIR "/icons/dxim.png").arg(text);
        system(exec.toUtf8().constData());
    }
    else
    {
        text.replace("\n", "<br>");
        if(!m_firstNotify || position!=m_notifyPosition)
        {
            m_firstNotify = true;
            m_notifiesHeight = 0;
            m_notifyPosition = position;
        }
        NotifyWindow *notify = new NotifyWindow(text, m_notifiesHeight, 5);
        notify->showNotify(position);
        m_notifiesHeight += notify->height()+4;
    }
}

void dxutils::removeNotify()
{
    if(m_firstNotify)
        m_firstNotify = !m_firstNotify;
}

//ChatRole to string
QString dxutils::getChatRoleString(PurpleConvChatBuddyFlags role)
{
    if(role & PURPLE_CBFLAGS_FOUNDER)
        return _("Founder");
    else if(role & PURPLE_CBFLAGS_OP)
        return _("Moderator");
    else if(role & PURPLE_CBFLAGS_HALFOP)
        return _("Half-op");
    else if(role & PURPLE_CBFLAGS_VOICE)
        return _("Voice");
    else
        return _("Any role");
}

//ChatRole to icon
QString dxutils::getChatRoleIcon(PurpleConvChatBuddyFlags role)
{
    if(role & PURPLE_CBFLAGS_FOUNDER)
        return DXPREFERENCES.m_themeRolePath+"/founder.png";
    else if(role & PURPLE_CBFLAGS_OP)
        return DXPREFERENCES.m_themeRolePath+"/op.png";
    else if(role & PURPLE_CBFLAGS_HALFOP)
        return DXPREFERENCES.m_themeRolePath+"/halfop.png";
    else if(role & PURPLE_CBFLAGS_VOICE)
        return DXPREFERENCES.m_themeRolePath+"/voice.png";
    else
        return DXPREFERENCES.m_themeRolePath+"/normal.png";
}

void dxutils::debugLine(const QString &line)
{
#ifdef DEBUG
    qDebug() << QTime::currentTime().toString("[hh:mm:ss] ") << line;
#endif
}

std::string dxutils::styleToString(QColor color)
{
    if(color == DXPREFERENCES.m_actionColor)
        return "action";
    if(color == DXPREFERENCES.m_errorColor)
        return "error";
    if(color == DXPREFERENCES.m_userColor)
        return "user";
    if(color == DXPREFERENCES.m_hilightColor)
        return "highlight";
    return "";
}

//Get offline color
QColor dxutils::makeOfflineColor(QColor clr)
{
    int r,g,b;
    r=clr.red();
    g=clr.green();
    b=clr.blue();
    if(r==g && r==b && !r)
    {
        r=255;
        g=255;
        b=255;
    }
    r=(50*r)/100;
    g=(50*g)/100;
    b=(50*b)/100;
    return QColor(r,g,b);
}

QString dxutils::normalizeLabelString(std::string label, char normalize)
{
    QString text = fromStdString(label);
    text.remove(normalize);
    return text;
}

//is file with name on path right dxim log file?
bool dxutils::isRightLogFile(const QString& path, const QString& name)
{
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
    if(QRegExp("^\\d\\d\\d\\d-\\d\\d-\\d\\d+$").exactMatch(name))
    {
        QFile file(path);
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return false;
        char buf[22];
        qint64 lineLength = file.readLine(buf, sizeof(buf));
        if(lineLength != -1)
        {
            if(QRegExp("^\\[(.*)(\\d\\d:\\d\\d:\\d\\d\\])").indexIn(QString(buf)) != -1)
                return true;
            else
                return false;
        }
        else
            return false;
    }
    return false;
#else
    if(QRegularExpression("^\\d\\d\\d\\d-\\d\\d-\\d\\d+$").match(name).hasMatch())
    {
        QFile file(path);
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return false;
        char buf[22];
        qint64 lineLength = file.readLine(buf, sizeof(buf));
        if(lineLength != -1)
        {
            if(QRegularExpression("^\\[(.*)(\\d\\d:\\d\\d:\\d\\d\\])").match(QString(buf)).hasPartialMatch())
                return true;
            else
                return false;
        }
        else
            return false;
    }
    return false;
#endif
}

//get month string
QString dxutils::getMonth(int month)
{
    switch(month) {
        case 1: return _("January");
        case 2: return _("February");
        case 3: return _("March");
        case 4: return _("April");
        case 5: return _("May");
        case 6: return _("June");
        case 7: return _("July");
        case 8: return _("August");
        case 9: return _("September");
        case 10: return _("October");
        case 11: return _("November");
        case 12: return _("December");
        default: return "Rok ma jen 12 mesicu:)";
    }
}

//get month string-number for name
QString dxutils::getMonthNumber(const QString& month)
{
    if(month==_("January")) return "01";
    else if(month==_("February")) return "02";
    else if(month==_("March")) return "03";
    else if(month==_("April")) return "04";
    else if(month==_("May")) return "05";
    else if(month==_("June")) return "06";
    else if(month==_("July")) return "07";
    else if(month==_("August")) return "08";
    else if(month==_("September")) return "09";
    else if(month==_("October")) return "10";
    else if(month==_("November")) return "11";
    else if(month==_("December")) return "12";
    return "00";
}

void dxutils::playFile(const QString &file)
{
    QDir dir;
    if(dir.exists(file))
    {
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
#ifdef _WIN32
        PlaySoundA((LPCSTR)file.toUtf8().constData(), NULL, SND_FILENAME | SND_ASYNC);
#else
        static const char * players[]={"aplay","play","esdplay","artsplay","ossplay",NULL};
        QString exec;
        QStringList paths = QString(getenv("PATH")).split(':');
        foreach(QString path, paths)
        {
            for(int i=0; players[i]!=NULL; i++)
            {
                if(dir.exists(path+"/"+players[i]))
                {
                    exec = path+"/"+players[i];
                    break;
                }
            }
        }
        if(exec.isEmpty()) return;
        if(exec.contains("aplay")) exec += " -q";
        exec += " "+enquote(file)+" &";
        system(exec.toUtf8().constData());
#endif
#else
        QSound::play(file);
#endif
    }
}

QString dxutils::createSmileys(QString text)
{
    for(QMap<QString,QString>::const_iterator it=DXPREFERENCES.m_smileysMap.constBegin(); it!=DXPREFERENCES.m_smileysMap.constEnd(); it++)
    {
        if(text.contains(it.key()))
        {
            text.replace(it.key(), "<img src='"+it.value()+"'>");
        }
    }
    return text;
}

bool isRightTag(const QString &tag)
{
    for(int i=0; i<numTags; i++)
    {
        if(tag.startsWith(tags[i]))
            return true;
    }
    return false;
}

bool isRemoveTag(const QString &tag)
{
    for(int i=0; i<numRemoveTags; i++)
    {
        if(tag.startsWith(removeTags[i]))
            return true;
    }
    return false;
}

QString dxutils::parseHtml(QString text)
{
    int i=0;
    int tagLength=0;
    int length=text.length();
    QString tag="";
    while(i<length)
    {
        if(text[i]=='<')
        {
            tagLength = 1;
            for(int j=i+1; j<length; j++)
            {
                if(text[j]=='>')
                {
                    tagLength++;
                    break;
                }
                tagLength++;
            }
            tag = text.mid(i,tagLength).toLower();
            if(isRightTag(tag))
            {
                i += tagLength;
            }
            else if(isRemoveTag(tag))
            {
                text.remove(i,tagLength);
                length = length-tagLength;
                i++;
            }
            else
            {
                text.replace(i, 1, "&lt;");
                i += tagLength+3;
            }
        }
        else
        {
            i++;
        }
    }
    return text;
}

//check path for status icons
QString dxutils::checkThemeStatusPath(const QString &path)
{
    QString themeDefaultPath = DXIM_DATADIR "/icons/statuses";
    if(QFileInfo(path+"/online.png").exists()) return path;
    return themeDefaultPath;
}

//check path list for status icons
QString dxutils::checkThemesStatusList(const QString &list)
{
    QString themeDefaultPath = DXIM_DATADIR "/icons/statuses;";
    QString themes;
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    QStringList pathes = list.split(';', QString::SkipEmptyParts);
#else
    QStringList pathes = list.split(';', Qt::SkipEmptyParts);
#endif
    for(int i=0; i<pathes.size(); ++i)
    {
        if(QFileInfo(pathes[i]+"/online.png").exists()) themes.append(pathes[i]+";");
    }
    if(!themes.isEmpty()) return themes;
    return themeDefaultPath;
}

//check path for role icons
QString dxutils::checkThemeRolePath(const QString &path)
{
    QString themeDefaultPath = DXIM_DATADIR "/icons/roles";
    if(QFileInfo(path+"/normal.png").exists()) return path;
    return themeDefaultPath;
}

//check path list for role icons
QString dxutils::checkThemesRoleList(const QString &list)
{
    QString themeDefaultPath = DXIM_DATADIR "/icons/roles;";
    QString themes;
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    QStringList pathes = list.split(';', QString::SkipEmptyParts);
#else
    QStringList pathes = list.split(';', Qt::SkipEmptyParts);
#endif
    for(int i=0; i<pathes.size(); ++i)
    {
        if(QFileInfo(pathes[i]+"/normal.png").exists()) themes.append(pathes[i]+";");
    }
    if(!themes.isEmpty()) return themes;
    return themeDefaultPath;
}

bool dxutils::haveNotify()
{
    return m_haveNotify;
}

bool dxutils::removeDir(const QString &dirName)
{
    bool result = true;
    QDir dir(dirName);
    if(dir.exists(dirName))
    {
        Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst))
        {
            if(info.isDir())
            {
                result = removeDir(info.absoluteFilePath());
            }
            else
            {
                result = QFile::remove(info.absoluteFilePath());
            }
            if(!result)
            {
                return result;
            }
        }
        result = dir.rmdir(dirName);
    }
    return result;
}

QString dxutils::wrapText(QString text, int width)
{
    QFontMetrics fnt(DXPREFERENCES.m_imFont);
    int tw = 0;
    int cw;
    int s = -1;
    for(int i=0; i<text.length(); i++)
    {
#if QT_VERSION < QT_VERSION_CHECK(5,11,0)
        cw = fnt.width(text[i]);
#else
        cw = fnt.horizontalAdvance(text[i]);
#endif
        if(text[i]==' ') s = i;
        if(tw+cw>width)
        {
            if(s!=-1)
            {
                text.replace(s,1,'\n');
                i=s+1;
                s=-1;
            }
            tw=0;
        }
        tw+=cw;
    }
    return text;
}

bool dxutils::hasLog(const QString &accountName, const QString &conversationName, bool account)
{
    QString path = account?DXPREFERENCES.m_logPath+"/"+accountName:DXPREFERENCES.m_logPath+"/"+accountName+"/"+conversationName;
    if(account)
    {
        QDir dir(path);
        dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
        dir.setSorting(QDir::Name);
        QFileInfoList list = dir.entryInfoList();
        for(int i=0; i<list.size(); ++i)
        {
            QFileInfo dirInfo = list.at(i);
            QDir adir(dirInfo.absoluteFilePath());
            adir.setFilter(QDir::Files | QDir::NoDotAndDotDot);
            adir.setSorting(QDir::Name);
            QFileInfoList alist = adir.entryInfoList();
            for(int j=0; j<alist.size(); ++j)
            {
                QFileInfo fileInfo = alist.at(j);
                if(fileInfo.isFile() && isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
                {
                    return true;
                }
            }
        }
    }
    else
    {
        QDir dir(path);
        dir.setFilter(QDir::Files | QDir::NoDotAndDotDot);
        dir.setSorting(QDir::Name);
        QFileInfoList list = dir.entryInfoList();
        for(int i=0; i<list.size(); ++i)
        {
            QFileInfo fileInfo = list.at(i);
            if(isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
            {
                return true;
            }
        }
    }
    return false;
}

QString dxutils::enquote(const QString &string)
{
    QString s = string;
    return QString("\"%1\"").arg(s.replace(QLatin1Char('\\'),QLatin1String("\\\\"))).replace(QLatin1Char('"'),QLatin1String("\\\""));
}

