/*
 *      preferences.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "preferences.h"
#include <QSettings>
#include <QApplication>
#include <QDesktopServices>
#include <QProcessEnvironment>
#include <QDir>
#include <QStandardPaths>
#include "dxutils.h"

Preferences::Preferences()
{
    //General settings tab in Configdialog
    m_oneInstance = false;
    m_useAway = true;
    m_awayTimeout = 5;
    m_minToTray = false;
    m_logging = true;
    m_showRosterTree = true;
    m_receiptsMark = true;
#ifdef Q_OS_WIN
    m_logPath = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxim/logs";
#else
    m_logPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.dxim/logs";
#endif
    m_logLoadCurrent = false;
    //Icons settings tab in Configdialog
    m_themeStatusPath = DXIM_DATADIR "/icons/statuses";
    m_themesStatusList = m_themeStatusPath+";";
    m_themeRolePath = DXIM_DATADIR "/icons/roles";
    m_themesRoleList = m_themeRolePath+";";
    //Look tab in ConfigDialog
    m_unreadtabColor = QColor("blue");
    m_hltabColor = QColor("red");
    m_tabPosition = 0;
    //Im Text settings tab in Configdialog
    m_textColor = QColor("black");
    m_backroundColor = QColor("white");
    m_userColor = QColor(43,57,35);
    m_actionColor = QColor(255,165,0);
    m_errorColor = QColor(255,0,0);
    m_hilightColor = QColor(11,154,11);
    m_linkColor = QColor(142,196,210);
    m_mymsgColor = QColor(130,150,253);
    m_logColor = QColor(70,49,49);
    m_sameCmd = false;
    m_sameList = false;
    m_nickCompletionChar = ":";
    m_coloredNick = true;
    //Notification settings tab in Configdialog
    m_sounds = false;
    m_soundConnect = false;
    m_pathConnect = DXIM_DATADIR "/sounds/connected.wav";
    m_soundDisconnect = false;
    m_pathDisconnect = DXIM_DATADIR "/sounds/disconnected.wav";
    m_soundMessage = false;
    m_pathMessage = DXIM_DATADIR "/sounds/message.wav";
    m_notify = false;
    m_notifyPosition = 3;
    m_notifyAccountConnect = false;
    m_notifyAccountDisconnect = false;
    m_notifyBuddyConnect = false;
    m_notifyBuddyDisconnect = false;
    m_notifyMessage = false;
    m_notifyFiletransfer = false;
    m_raiseWindow = false;
    m_notifyMessageTray = false;
    //Smileys settings tab in ConfigDialog
    m_useSmileys = false;
    //Other settings value
    m_hideOffline = true;
    m_rosterShown = true;
    m_chatRosterShown = true;
    m_showAvatars = true;
    m_showStatusmsg = true;
    m_x = 50;
    m_y = 50;
    m_w = 888;
    m_h = 666;
    m_maximized = false;
    m_rosterWidth = 220;

    setIniPath("");
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::setIniPath(const QString &path)
{
    if(path.isEmpty())
    {
        m_iniPath = g_build_filename(g_get_user_config_dir(), "dxim", NULL);
        if(!(g_file_test(m_iniPath.toUtf8().constData(), G_FILE_TEST_IS_DIR)))
        {
            g_mkdir_with_parents(m_iniPath.toUtf8().constData(), 0755);
        }
        return;
    }
    m_iniPath = path;
    if(!g_file_test(m_iniPath.toUtf8().constData(), G_FILE_TEST_IS_DIR))
    {
        g_mkdir_with_parents(m_iniPath.toUtf8().constData(), 0755);
    }
}

QString Preferences::getIniPath()
{
    return m_iniPath;
}

void Preferences::load()
{
    QSettings set(m_iniPath+"/dxim-qt", QSettings::IniFormat);
    //General settings tab in Configdialog
    set.beginGroup("BASIC");
    m_oneInstance = set.value("oneInstance", m_oneInstance).toBool();
    m_useAway = set.value("useAway", m_useAway).toBool();
    m_awayTimeout = set.value("awayTimeout", m_awayTimeout).toInt();
    m_minToTray = set.value("minToTray", m_minToTray).toBool();
    m_logging = set.value("logging", m_logging).toBool();
    m_logPath = set.value("logPath", m_logPath).toString();
    m_logLoadCurrent = set.value("logLoadCurrent", m_logLoadCurrent).toBool();
    if(!m_logging) m_logLoadCurrent = false;
    m_showRosterTree = set.value("showRosterTree", m_showRosterTree).toBool();
    m_receiptsMark = set.value("receiptsMark", m_receiptsMark).toBool();
    set.endGroup();
    //Icons settings tab in Configdialog
    set.beginGroup("THEMES");
    m_themeStatusPath = dxutils::checkThemeStatusPath(set.value("themeStatusPath", m_themeStatusPath).toString());
    m_themesStatusList = dxutils::checkThemesStatusList(set.value("themesStatusList", m_themesStatusList).toString());
    if(!m_themesStatusList.contains(m_themeStatusPath)) m_themesStatusList.append(m_themeStatusPath+";");
    m_themeRolePath = dxutils::checkThemeRolePath(set.value("themesRolePath", m_themeRolePath).toString());
    m_themesRoleList = dxutils::checkThemesRoleList(set.value("themesRoleList", m_themesRoleList).toString());
    if(!m_themesRoleList.contains(m_themeRolePath)) m_themesRoleList.append(m_themeRolePath+";");
    set.endGroup();
    //Look tab in ConfigDialog
    set.beginGroup("LOOK");
    m_unreadtabColor = set.value("unreadtabColor", m_unreadtabColor).value<QColor>();
    m_hltabColor = set.value("hltabColor", m_hltabColor).value<QColor>();
    m_tabPosition = set.value("tabPosition", m_tabPosition).toInt();
    set.endGroup();
    //Im Text settings tab in Configdialog
    set.beginGroup("IMTEXT");
    m_textColor = set.value("textColor", m_textColor).value<QColor>();
    m_backroundColor = set.value("backroundColor", m_backroundColor).value<QColor>();
    m_userColor = set.value("userColor", m_userColor).value<QColor>();
    m_actionColor = set.value("actionColor", m_actionColor).value<QColor>();
    m_errorColor = set.value("errorColor", m_errorColor).value<QColor>();
    m_hilightColor = set.value("hilightColor", m_hilightColor).value<QColor>();
    m_linkColor = set.value("linkColor", m_linkColor).value<QColor>();
    m_mymsgColor = set.value("mymsgColor", m_mymsgColor).value<QColor>();
    m_logColor = set.value("logColor", m_logColor).value<QColor>();
    m_imFont.fromString(set.value("imFont", "Sans Serif,9,-1,5,50,0,0,0,0,0").toString());
    m_sameCmd = set.value("sameCmd", m_sameCmd).toBool();
    m_sameList = set.value("sameList", m_sameList).toBool();
    m_nickCompletionChar = set.value("nickCompletionChar", m_nickCompletionChar).toString().left(1);
    m_coloredNick = set.value("coloredNick", m_coloredNick).toBool();
    set.endGroup();
    //Notification settings tab in Configdialog
    set.beginGroup("NOTIFICATION");
    m_sounds = set.value("sounds", m_sounds).toBool();
    m_soundConnect = set.value("soundConnect", m_soundConnect).toBool();
    m_pathConnect = set.value("pathConnect", m_pathConnect).toString();
    m_soundDisconnect = set.value("soundDisconnect", m_soundDisconnect).toBool();
    m_pathDisconnect = set.value("pathDisconnect", m_pathDisconnect).toString();
    m_soundMessage = set.value("soundMessage", m_soundMessage).toBool();
    m_pathMessage = set.value("pathMessage", m_pathMessage).toString();
    m_notify = set.value("notify", m_notify).toBool();
    m_notifyPosition = set.value("notifyPosition", m_notifyPosition).toInt();
    m_notifyAccountConnect = set.value("notifyAccountConnect", m_notifyAccountConnect).toBool();
    m_notifyAccountDisconnect = set.value("notifyAccountDisconnect", m_notifyAccountDisconnect).toBool();
    m_notifyBuddyConnect = set.value("notifyBuddyConnect", m_notifyBuddyConnect).toBool();
    m_notifyBuddyDisconnect = set.value("notifyBuddyDisconnect", m_notifyBuddyDisconnect).toBool();
    m_notifyMessage = set.value("notifyMessage", m_notifyMessage).toBool();
    m_notifyFiletransfer = set.value("notifyFiletransfer", m_notifyFiletransfer).toBool();
    m_raiseWindow = set.value("raiseWindow", m_raiseWindow).toBool();
    m_notifyMessageTray = set.value("notifyMessageTray", m_notifyMessageTray).toBool();
    if(m_raiseWindow) m_notifyMessageTray = false;
    set.endGroup();
    //Smileys settings tab in Configdialog
    set.beginGroup("SMILEYS");
    m_useSmileys = set.value("useSmileys", m_useSmileys).toBool();
    int smileysNum = set.value("number", 0).toInt();
    m_smileysMap.clear();
    if(smileysNum)
    {
        for(int i=0; i<smileysNum; i++)
        {
            QString key, value;
            key = set.value(QString("smiley%1").arg(i), "").toString();
            value = set.value(QString("path%1").arg(i), "").toString();
            if(!key.isEmpty())
                m_smileysMap.insert(key, value);
        }
    }
    set.endGroup();
    //Other settings value
    set.beginGroup("OTHER");
    m_hideOffline = set.value("hideOffline", m_hideOffline).toBool();
    m_rosterShown = set.value("rosterShown", m_rosterShown).toBool();
    m_chatRosterShown = set.value("chatRosterShown", m_chatRosterShown).toBool();
    m_showAvatars = set.value("showAvatars", m_showAvatars).toBool();
    m_showStatusmsg = set.value("showStatusmsg", m_showStatusmsg).toBool();
    m_x = set.value("x", m_x).toInt();
    m_y = set.value("y", m_y).toInt();
    m_w = set.value("w", m_w).toInt();
    m_h = set.value("h", m_h).toInt();
    m_maximized = set.value("maximized", m_maximized).toBool();
    m_rosterWidth = set.value("rosterWidth", m_rosterWidth).toInt();
    set.endGroup();
}

void Preferences::save()
{
    QSettings set(m_iniPath+"/dxim-qt", QSettings::IniFormat);
    foreach(QString key, set.allKeys())
        set.remove(key);
    //General settings tab in Configdialog
    set.beginGroup("BASIC");
    set.setValue("oneInstance", m_oneInstance);
    set.setValue("useAway", m_useAway);
    set.setValue("awayTimeout", m_awayTimeout);
    set.setValue("minToTray", m_minToTray);
    set.setValue("logging", m_logging);
    set.setValue("logPath", m_logPath);
    set.setValue("logLoadCurrent", m_logLoadCurrent);
    set.setValue("showRosterTree", m_showRosterTree);
    set.setValue("receiptsMark", m_receiptsMark);
    set.endGroup();
    //Icons settings tab in Configdialog
    set.beginGroup("THEMES");
    set.setValue("themeStatusPath", m_themeStatusPath);
    set.setValue("themesStatusList", m_themesStatusList);
    set.setValue("themesRolePath", m_themeRolePath);
    set.setValue("themesRoleList", m_themesRoleList);
    set.endGroup();
    //Look tab in ConfigDialog
    set.beginGroup("LOOK");
    set.setValue("unreadtabColor", m_unreadtabColor);
    set.setValue("hltabColor", m_hltabColor);
    set.setValue("tabPosition", m_tabPosition);
    set.endGroup();
    //Im Text settings tab in Configdialog
    set.beginGroup("IMTEXT");
    set.setValue("textColor", m_textColor);
    set.setValue("backroundColor", m_backroundColor);
    set.setValue("userColor", m_userColor);
    set.setValue("actionColor", m_actionColor);
    set.setValue("errorColor", m_errorColor);
    set.setValue("hilightColor", m_hilightColor);
    set.setValue("linkColor", m_linkColor);
    set.setValue("mymsgColor", m_mymsgColor);
    set.setValue("logColor", m_logColor);
    set.setValue("imFont", m_imFont.toString());
    set.setValue("sameCmd", m_sameCmd);
    set.setValue("sameList", m_sameList);
    set.setValue("nickCompletionChar", m_nickCompletionChar);
    set.setValue("coloredNick", m_coloredNick);
    set.endGroup();
    //Notification settings tab in Configdialog
    set.beginGroup("NOTIFICATION");
    set.setValue("sounds", m_sounds);
    set.setValue("soundConnect", m_soundConnect);
    set.setValue("pathConnect", m_pathConnect);
    set.setValue("soundDisconnect", m_soundDisconnect);
    set.setValue("pathDisconnect", m_pathDisconnect);
    set.setValue("soundMessage", m_soundMessage);
    set.setValue("pathMessage", m_pathMessage);
    set.setValue("notify", m_notify);
    set.setValue("notifyPosition", m_notifyPosition);
    set.setValue("notifyAccountConnect", m_notifyAccountConnect);
    set.setValue("notifyAccountDisconnect", m_notifyAccountDisconnect);
    set.setValue("notifyBuddyConnect", m_notifyBuddyConnect);
    set.setValue("notifyBuddyDisconnect", m_notifyBuddyDisconnect);
    set.setValue("notifyMessage", m_notifyMessage);
    set.setValue("notifyFiletransfer", m_notifyFiletransfer);
    set.setValue("raiseWindow", m_raiseWindow);
    set.setValue("notifyMessageTray", m_notifyMessageTray);
    set.endGroup();
    //Smileys settings tab in Configdialog
    set.beginGroup("SMILEYS");
    set.setValue("useSmileys", m_useSmileys);
    set.setValue("number", m_smileysMap.size());
    if(m_smileysMap.size())
    {
        QMap<QString,QString>::iterator it;
        int i;
        for(i=0, it=m_smileysMap.begin(); it!=m_smileysMap.end(); it++,i++)
        {
            set.setValue(QString("smiley%1").arg(i), it.key());
            set.setValue(QString("path%1").arg(i), it.value());
        }
    }
    set.endGroup();
    //Other settings value
    set.beginGroup("OTHER");
    set.setValue("hideOffline", m_hideOffline);
    set.setValue("rosterShown", m_rosterShown);
    set.setValue("chatRosterShown", m_chatRosterShown);
    set.setValue("showAvatars", m_showAvatars);
    set.setValue("showStatusmsg", m_showStatusmsg);
    set.setValue("x", m_x);
    set.setValue("y", m_y);
    set.setValue("w", m_w);
    set.setValue("h", m_h);
    set.setValue("maximized", m_maximized);
    set.setValue("rosterWidth", m_rosterWidth);
    set.endGroup();
    set.sync();
}
