/*
 *      rosteritem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "rosteritem.h"
#include <QImage>
#include <QFile>
#include "dxutils.h"
#include "preferences.h"
#include "config.h"
#include "i18n.h"

#define MAX_STATUS_SIZE     16
#define MAX_AVATAR_SIZE     32
#define SPACING             4

RosterItem::RosterItem(const QString &name, const QString &ID, RosterItemType type, PURPLE::Account account)
{
    setText(name);
    setID(ID);
    setAccount(account);
    setRosterItemType(type);
    m_lastPresenceChange = QTime::currentTime();
}

void RosterItem::setName(const QString &name)
{
    setText(name);
    updateTooltip();
}

QString RosterItem::getName()
{
    return text();
}

void RosterItem::setID(const QString& ID)
{
    setData(ID, RosterItem::RID);
}

QString RosterItem::getID()
{
    return data(RosterItem::RID).toString();
}

void RosterItem::setRosterItemType(RosterItem::RosterItemType type)
{
    setData(static_cast<int>(type), RosterItem::RType);
    if(type==Chat)
        setIcon(QIcon(DXIM_DATADIR "/icons/chat.png"));
}

RosterItem::RosterItemType RosterItem::getRosterItemType()
{
    return static_cast<RosterItem::RosterItemType>(data(RosterItem::RType).toInt());
}

void RosterItem::setAccount(PURPLE::Account account)
{
    QVariant v;
    v.setValue(account);
    setData(v, RosterItem::RAccount);
    if(getRosterItemType()==Account)
    {
        updateTooltip();
    }
}

PURPLE::Account RosterItem::getAccount()
{
    return data(RosterItem::RAccount).value<PURPLE::Account>();
}

void RosterItem::setBuddy(PURPLE::Buddy buddy)
{
    QVariant v;
    v.setValue(buddy);
    setData(v, RosterItem::RBuddy);
    setIcon(QIcon(dxutils::presenceToIcon(getBuddy().getStatusPrimitive())));
    updateTooltip();
}

PURPLE::Buddy RosterItem::getBuddy()
{
    return data(RosterItem::RBuddy).value<PURPLE::Buddy>();
}

void RosterItem::setChat(PURPLE::Chat chat)
{
    QVariant v;
    v.setValue(chat);
    setData(v, RosterItem::RChat);
    updateTooltip();
}

PURPLE::Chat RosterItem::getChat()
{
    return data(RosterItem::RChat).value<PURPLE::Chat>();
}

void RosterItem::accountConnecting()
{
    if(getRosterItemType()==Account)
    {
        setIcon(QIcon(DXIM_DATADIR "/icons/connecting.png"));
    }
}

void RosterItem::buddySignedOn()
{
    if(getRosterItemType()==Buddy)
    {
        setName(dxutils::fromStdString(getBuddy().getAlias()));
        QTime now = QTime::currentTime();
        if(m_lastPresenceChange.secsTo(now)>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyConnect) dxutils::instance().showNotify(_("%1\n(%2)\nis online").arg(dxutils::fromStdString(getBuddy().getAlias())).arg(dxutils::fromStdString(getBuddy().getUsername())));
            if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundConnect) dxutils::playFile(DXPREFERENCES.m_pathConnect);
        }
        setIcon(QIcon(dxutils::presenceToIcon(getBuddy().getStatusPrimitive())));
        m_lastPresenceChange = now;
        updateTooltip();
    }
}

void RosterItem::buddySignedOff()
{
    if(getRosterItemType()==Buddy)
    {
        QTime now = QTime::currentTime();
        if(m_lastPresenceChange.secsTo(now)>2)
        {
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyBuddyDisconnect) dxutils::instance().showNotify(_("%1\n(%2)\nis offline").arg(dxutils::fromStdString(getBuddy().getAlias())).arg(dxutils::fromStdString(getBuddy().getUsername())));
            if(DXPREFERENCES.m_sounds && DXPREFERENCES.m_soundDisconnect) dxutils::playFile(DXPREFERENCES.m_pathDisconnect);
        }
        setIcon(QIcon(dxutils::presenceToIcon(PURPLE_STATUS_OFFLINE)));
        m_lastPresenceChange = now;
        updateTooltip();
    }
}

void RosterItem::buddyStatusChanged()
{
    if(getRosterItemType()==Buddy)
    {
        setIcon(QIcon(dxutils::presenceToIcon(getBuddy().getStatusPrimitive())));
        m_lastPresenceChange = QTime::currentTime();
        updateTooltip();
    }
}

void RosterItem::updateStatusIconTheme()
{
    if(getRosterItemType()==Buddy)
    {
        setIcon(QIcon(dxutils::presenceToIcon(getBuddy().getStatusPrimitive())));
    }
    else if(getRosterItemType()==Account)
    {
        setIcon(QIcon(dxutils::presenceToIcon(getAccount().getStatus())));
    }
}

void RosterItem::updateTooltip()
{
    if(getRosterItemType()==Chat)
    {
        PURPLE::Chat chat = getChat();
        if(chat.isValid())
        {
            QString tip = QString("%1\n").arg(dxutils::fromStdString(chat.getAlias()));
            std::vector<PURPLE::Account::ChatEntry> fields = chat.getChatParameters();
            FOREACH(std::vector<PURPLE::Account::ChatEntry>, fields, it)
            {
                PURPLE::Account::ChatEntry entry = *it;
                if(entry.getSecret())
                {
                    if(entry.getValue().empty()) tip.append(QString("%1\n").arg(dxutils::normalizeLabelString(entry.getLabel(), '_')));
                    else tip.append(QString("%1 *****\n").arg(dxutils::normalizeLabelString(entry.getLabel(), '_')));
                }
                else
                    tip.append(QString("%1 %2\n").arg(dxutils::normalizeLabelString(entry.getLabel(), '_')).arg(dxutils::fromStdString(entry.getValue())));
            }
            tip.append(_("Autojoin: %1").arg(chat.getAutojoin() ? _("Yes") : _("No")));
            setToolTip(tip);
        }
    }
    else if(getRosterItemType()==Buddy)
    {
        PURPLE::Buddy buddy = getBuddy();
        if(buddy.isValid())
        {
            QString tip = QString("%1\n").arg(dxutils::fromStdString(buddy.getServerAlias()));
            tip.append(_("ID: %1\n").arg(dxutils::fromStdString(buddy.getUsername())));
            tip.append(_("Status: %1").arg(dxutils::fromStdString(buddy.getStatusName())));
            if(!buddy.getStatusMsg().empty())
            {
                tip.append('\n');
                tip.append(_("Message: %1").arg(dxutils::fromStdString(buddy.getStatusMsg())));
            }
            setToolTip(tip);
        }
    }
    else if(getRosterItemType()==Account)
    {
        PURPLE::Account account = getAccount();
        if(account.isValid())
        {
            setIcon(QIcon(dxutils::presenceToIcon(account.getStatus())));
            QString tip = QString("%1\n").arg(dxutils::fromStdString(account.getUsername()));
            if(!account.getAlias().empty()) tip.append(_("Alias: %1\n").arg(dxutils::fromStdString(account.getAlias())));
            tip.append(_("Protocol: %1\n").arg(dxutils::fromStdString(account.getProtocol().getName())));
            tip.append(_("Status: %1").arg(dxutils::fromStdString(account.getStatusName())));
            if(account.isConnected() && !account.getStatusMessage().empty())
            {
                tip.append('\n');
                tip.append(_("Message: %1").arg(dxutils::fromStdString(account.getStatusMessage())));
            }
            setToolTip(tip);
        }
    }
}

RosterItemDelegate::RosterItemDelegate()
{
}

QSize RosterItemDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex & index) const
{
    QVariant value = index.data(Qt::SizeHintRole);
    if(value.isValid())
        return value.toSize();
    int height = option.fontMetrics.height()+4;
    if(index.data(RosterItem::RType).toInt()==RosterItem::Buddy)
    {
        PURPLE::Buddy buddy = index.data(RosterItem::RBuddy).value<PURPLE::Buddy>();
        if(DXPREFERENCES.m_showStatusmsg && !buddy.getStatusMsg().empty()) height = 2*option.fontMetrics.height()+6;
        if(DXPREFERENCES.m_showAvatars && QFile::exists(dxutils::fromStdString(buddy.getIconPath())))
        {
            QImage avatar = dxutils::getImageFromPath(dxutils::fromStdString(buddy.getIconPath()), MAX_AVATAR_SIZE);
            height = qMax(height, qMin(avatar.height(), MAX_AVATAR_SIZE)+2);
        }
    }
    else if(index.data(RosterItem::RType).toInt()==RosterItem::Account)
    {
        PURPLE::Account account = index.data(RosterItem::RAccount).value<PURPLE::Account>();
        if(!account.getStatusMessage().empty() && account.isConnected()) height = 2*option.fontMetrics.height()+6;
    }
    return QSize(80, height);
}

void RosterItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    painter->save();
    RosterItem::RosterItemType type = static_cast<RosterItem::RosterItemType>(index.data(RosterItem::RType).toInt());
    painter->setRenderHint(QPainter::TextAntialiasing);
    QVariant value = index.data(Qt::DecorationRole);
    QColor selectedBg(60, 140, 222);
    QColor selectedText(Qt::white);
    QColor nameTextColor(DXPREFERENCES.m_textColor);
    QColor statusTextColor = dxutils::makeOfflineColor(nameTextColor);
    QFont font;
    if(DXPREFERENCES.m_sameList) font = DXPREFERENCES.m_imFont;
    QFontMetrics fontMetrics(font);
    switch(type){
    case RosterItem::Group: {
        QPen penDivision;
        if(option.state & QStyle::State_Selected)
        {
            painter->fillRect(option.rect, selectedBg);
            penDivision.setColor(selectedBg);
            nameTextColor = selectedText;
        }
        QRect rect = option.rect;
        rect.setLeft(rect.x());
        rect.moveTop(rect.y() + (option.rect.height() - fontMetrics.height())/2);
        painter->setFont(font);
        painter->setPen(nameTextColor);
        painter->drawText(rect, index.data(Qt::DisplayRole).toString());
        break;
    }
    case RosterItem::Account: {
        PURPLE::Account account = index.data(RosterItem::RAccount).value<PURPLE::Account>();
        QPixmap pixmap;
        if(value.type() == QVariant::Icon)
        {
            QIcon icon = qvariant_cast<QIcon>(value);
            pixmap = icon.pixmap(QSize(MAX_STATUS_SIZE, MAX_STATUS_SIZE), QIcon::Normal, QIcon::On);
        }
        QPen penDivision;
        if (option.state & QStyle::State_Selected)
        {
            painter->fillRect(option.rect, selectedBg);
            penDivision.setColor(selectedBg);
            nameTextColor = selectedText;
        }
        QRect rect = option.rect;
        rect.setWidth(MAX_STATUS_SIZE);
        rect.setHeight(MAX_STATUS_SIZE);
        rect.moveTop(rect.y() + (option.rect.height() - MAX_STATUS_SIZE)/2);
        rect.moveLeft(rect.left() + SPACING);
        painter->drawPixmap(rect, pixmap);
        QString statusText = dxutils::fromStdString(account.getStatusMessage());
        if(!account.isConnected()) statusText = "";
        rect = option.rect;
        rect.setLeft(rect.x() + MAX_STATUS_SIZE + 2*SPACING);
        if(statusText.isEmpty()) rect.moveTop(rect.y() + (option.rect.height() - fontMetrics.height())/2);
        else rect.moveTop(rect.y() + option.rect.height()/2 - fontMetrics.height());
        painter->setFont(font);
        if(account.isConnected())
            painter->setPen(nameTextColor);
        else
            painter->setPen(statusTextColor);
        painter->drawText(rect, index.data(Qt::DisplayRole).toString());
        painter->setPen(statusTextColor);
        rect.setTop(rect.y() + rect.height()/2);
        rect.moveTop(rect.y() - 2);
        statusText = fontMetrics.elidedText(statusText, Qt::ElideRight, rect.width() - SPACING);
        painter->drawText(rect, statusText);
        QImage image = dxutils::getImageFromPath(dxutils::protocolToIcon(dxutils::fromStdString(account.getProtocol().getPurpleID())), MAX_STATUS_SIZE);
        rect = option.rect;
        int w = image.width();
        int h = image.height();
        rect.setWidth(w);
        rect.setHeight(h);
        rect.moveTop(rect.y() + (option.rect.height() - h)/2);
        rect.moveLeft(option.rect.x() + option.rect.width() - w - 2);
        painter->drawImage(rect, image);
        break;
    }
    case RosterItem::Chat: {
        QPixmap pixmap;
        if(value.type() == QVariant::Icon)
        {
            QIcon icon = qvariant_cast<QIcon>(value);
            pixmap = icon.pixmap(QSize(MAX_STATUS_SIZE, MAX_STATUS_SIZE), QIcon::Normal, QIcon::On);
        }
        QPen penDivision;
        if (option.state & QStyle::State_Selected)
        {
            painter->fillRect(option.rect, selectedBg);
            penDivision.setColor(selectedBg);
            nameTextColor = selectedText;
        }
        QRect rect = option.rect;
        rect.setWidth(MAX_STATUS_SIZE);
        rect.setHeight(MAX_STATUS_SIZE);
        rect.moveTop(rect.y() + (option.rect.height() - MAX_STATUS_SIZE)/2);
        rect.moveLeft(rect.left() + SPACING);
        painter->drawPixmap(rect, pixmap);
        rect = option.rect;
        rect.setLeft(rect.x() + MAX_STATUS_SIZE + 2*SPACING);
        rect.moveTop(rect.y() + (option.rect.height() - fontMetrics.height())/2);
        painter->setFont(font);
        painter->setPen(nameTextColor);
        painter->drawText(rect, index.data(Qt::DisplayRole).toString());
        break;
    }
    case RosterItem::Buddy: {
        PURPLE::Buddy buddy = index.data(RosterItem::RBuddy).value<PURPLE::Buddy>();
        bool drawAvatar = DXPREFERENCES.m_showAvatars && QFile::exists(dxutils::fromStdString(buddy.getIconPath()));
        QPixmap pixmap;
        if(value.type() == QVariant::Icon)
        {
            QIcon icon = qvariant_cast<QIcon>(value);
            pixmap = icon.pixmap(QSize(MAX_STATUS_SIZE, MAX_STATUS_SIZE), QIcon::Normal, QIcon::On);
        }
        QPen penDivision;
        if (option.state & QStyle::State_Selected)
        {
            painter->fillRect(option.rect, selectedBg);
            penDivision.setColor(selectedBg);
            nameTextColor = selectedText;
        }
        if(!buddy.isOnline())
        {
            nameTextColor = statusTextColor;
        }
        QRect rect = option.rect;
        rect.setWidth(MAX_STATUS_SIZE);
        rect.setHeight(MAX_STATUS_SIZE);
        rect.moveTop(rect.y() + (option.rect.height() - MAX_STATUS_SIZE)/2);
        rect.moveLeft(rect.left() + SPACING);
        painter->drawPixmap(rect, pixmap);
        QString statusText = DXPREFERENCES.m_showStatusmsg ? dxutils::fromStdString(buddy.getStatusMsg()) : "";
        rect = option.rect;
        rect.setLeft(rect.x() + MAX_STATUS_SIZE + 2*SPACING);
        if(statusText.isEmpty()) rect.moveTop(rect.y() + (option.rect.height() - fontMetrics.height())/2);
        else rect.moveTop(rect.y() + option.rect.height()/2 - fontMetrics.height());
        painter->setFont(font);
        painter->setPen(nameTextColor);
        painter->drawText(rect, index.data(Qt::DisplayRole).toString());
        painter->setPen(statusTextColor);
        rect.setTop(rect.y() + rect.height()/2);
        rect.moveTop(rect.y() - 2);
        rect.setWidth(drawAvatar ? rect.width() - SPACING - MAX_AVATAR_SIZE : rect.width() - SPACING);
        if(!statusText.isEmpty())
        {
            statusText = fontMetrics.elidedText(statusText, Qt::ElideRight, rect.width());
            painter->drawText(rect, statusText);
        }
        if(drawAvatar)
        {
            QImage image = dxutils::getImageFromPath(dxutils::fromStdString(buddy.getIconPath()), MAX_AVATAR_SIZE);
            rect = option.rect;
            int w = image.width();
            int h = image.height();
            rect.setWidth(w);
            rect.setHeight(h);
            rect.moveTop(rect.y() + (option.rect.height() - h)/2);
            rect.moveLeft(option.rect.x() + option.rect.width() - w - 2);
            painter->drawImage(rect, image);
        }
        break;
    }
    default:
        break;
    }
    painter->restore();
}
