/*
 *      rosteritemmodel.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "rosteritemmodel.h"
#include "dxutils.h"

RosterItemModel::RosterItemModel(QObject* parent) :
    QStandardItemModel(parent)
{
}

RosterItem* RosterItemModel::getRosterItem(RosterItem::RosterItemType type, const QString &ID, PURPLE::Account account)
{
    return getRosterItem(index(0, 0, QModelIndex()), type, ID, account);
}

RosterItem* RosterItemModel::getRosterItem(const QModelIndex &start, RosterItem::RosterItemType type, const QString &ID, PURPLE::Account account)
{
    QModelIndex p = parent(start);
    int from = start.row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, start.column(), p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType()==type
                && item->getID()==ID
                && item->getAccount().getUsername()==account.getUsername()
                && item->getAccount().getProtocol().getPurpleID()==account.getProtocol().getPurpleID())
        {
            return item;
        }
        if(hasChildren(idx))
        {
            item = getRosterItem(index(0, idx.column(), idx), type, ID, account);
            if(item) return item;
        }
    }
    return 0;
}

bool RosterItemModel::hasRosterItem(RosterItem::RosterItemType type, const QString &ID, PURPLE::Account account)
{
    return hasRosterItem(index(0, 0, QModelIndex()), type, ID, account);
}

bool RosterItemModel::hasRosterItem(const QModelIndex &start, RosterItem::RosterItemType type, const QString &ID, PURPLE::Account account)
{
    QModelIndex p = parent(start);
    int from = start.row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, start.column(), p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType()==type
                && item->getID()==ID
                && item->getAccount().getUsername()==account.getUsername()
                && item->getAccount().getProtocol().getPurpleID()==account.getProtocol().getPurpleID())
        {
            return true;
        }
        if(hasChildren(idx))
        {
            bool result = hasRosterItem(index(0, idx.column(), idx), type, ID, account);
            if(result) return true;
        }
    }
    return false;
}

void RosterItemModel::addRosterItem(RosterItem *parent, RosterItem *item)
{
    if(parent)
        parent->appendRow(item);
    else
        invisibleRootItem()->appendRow(item);
}

void RosterItemModel::removeRosterItem(RosterItem::RosterItemType type, const QString &ID, PURPLE::Account account)
{
    RosterItem *item = getRosterItem(type, ID, account);
    if(item)
    {
        RosterItem *parent = static_cast<RosterItem*>(item->parent());
        removeRow(item->row(), indexFromItem(parent));
        if(parent && parent->getRosterItemType()==RosterItem::Group && !parent->hasChildren())
            removeRow(parent->row(), indexFromItem(parent->parent()));
    }
}

void RosterItemModel::removeAccountItem(const QString& account, const QString& protocol)
{
    QModelIndex p = parent(index(0, 0, QModelIndex()));
    int from = index(0, 0, QModelIndex()).row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, 0, p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType()==RosterItem::Account
            && dxutils::fromStdString(item->getAccount().getUsername())==account
            && dxutils::fromStdString(item->getAccount().getProtocol().getPurpleID())==protocol)
        {
            RosterItem *parent = static_cast<RosterItem*>(item->parent());
            removeRow(item->row(), indexFromItem(parent));
        }
    }
}

void RosterItemModel::clear()
{
    QStandardItemModel::clear();
}

void RosterItemModel::accountConnecting(PURPLE::Account account)
{
    RosterItem *item = getRosterItem(RosterItem::Account, dxutils::fromStdString(account.getUsername()), account);
    if(item)
        item->accountConnecting();
}

void RosterItemModel::accountPresence(PURPLE::Account account)
{
    RosterItem *item = getRosterItem(RosterItem::Account, dxutils::fromStdString(account.getUsername()), account);
    if(item)
        item->updateTooltip();
}

void RosterItemModel::buddySignedOn(PURPLE::Buddy buddy)
{
    RosterItem *item = getRosterItem(RosterItem::Buddy, dxutils::fromStdString(buddy.getUsername()), buddy.getAccount());
    if(item)
        item->buddySignedOn();
}

void RosterItemModel::buddySignedOff(PURPLE::Buddy buddy)
{
    RosterItem *item = getRosterItem(RosterItem::Buddy, dxutils::fromStdString(buddy.getUsername()), buddy.getAccount());
    if(item)
        item->buddySignedOff();
}

void RosterItemModel::buddyStatusChanged(PURPLE::Buddy buddy)
{
    RosterItem *item = getRosterItem(RosterItem::Buddy, dxutils::fromStdString(buddy.getUsername()), buddy.getAccount());
    if(item)
        item->buddyStatusChanged();
}

void RosterItemModel::buddyIconChanged(PURPLE::Buddy buddy)
{
    RosterItem *item = getRosterItem(RosterItem::Buddy, dxutils::fromStdString(buddy.getUsername()), buddy.getAccount());
    if(item)
        emit itemChanged(item);
}

void RosterItemModel::updateStatusIconTheme()
{
    QModelIndex p = parent(index(0, 0, QModelIndex()));
    int from = index(0, 0, QModelIndex()).row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, 0, p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType() == RosterItem::Account)
        {
            item->updateStatusIconTheme();
            for(int j=0; j<item->rowCount(); j++)
            {
                QStandardItem *group = item->child(j);
                for(int k=0; k<group->rowCount(); k++)
                {
                    RosterItem *child = static_cast<RosterItem*>(group->child(k));
                    if(child->getRosterItemType() == RosterItem::Buddy)
                    {
                        child->updateStatusIconTheme();
                    }
                }
            }            
        }
    }
}

void RosterItemModel::updateAccountsTooltip()
{
    QModelIndex p = parent(index(0, 0, QModelIndex()));
    int from = index(0, 0, QModelIndex()).row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, 0, p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType()==RosterItem::Account)
        {
            item->updateTooltip();
        }
    }
}

//called by 'show avatars' or 'show friends status message' change
void RosterItemModel::buddySizeHint()
{
    QModelIndex p = parent(index(0, 0, QModelIndex()));
    int from = index(0, 0, QModelIndex()).row();
    int to = rowCount(p);
    for(int r = from; r < to; ++r)
    {
        QModelIndex idx = index(r, 0, p);
        if(!idx.isValid())
            continue;
        RosterItem *item = static_cast<RosterItem*>(itemFromIndex(idx));
        if(item->getRosterItemType() == RosterItem::Account
                && item->getAccount().isConnected())
        {
            for(int j=0; j<item->rowCount(); j++)
            {
                QStandardItem *group = item->child(j);
                for(int k=0; k<group->rowCount(); k++)
                {
                    RosterItem *child = static_cast<RosterItem*>(group->child(k));
                    if(child->getRosterItemType() == RosterItem::Buddy)
                    {
                        this->dataChanged(child->index(), child->index());
                    }
                }
            }            
        }
    }
}

