/*
 *      fttabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QApplication>
#include <QVBoxLayout>
#include <QItemDelegate>
#include <QPainter>
#include <QProgressBar>
#include <QFileInfo>
#include <QHeaderView>
#include <QDialogButtonBox>
#include <QGroupBox>
#include <QGridLayout>
#include <QMenu>
#include <QAction>
#include "fttabitem.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"

static int percent(quint64 position, quint64 size)
{
    if(size) return (int)100*position/size;
    else return 0;
}

static QString statusForFT(PURPLE::FileTransfer *ft)
{
    if(ft->isCanceled()) return _("Canceled");
    if(!ft->isStarted()) return _("Waiting");
    if(ft->isFinished()) return _("Finished");
    if(ft->isReceiving()) return _("Downloading %1 of %2").arg(dxutils::getFileSize(ft->getBytesSent())).arg(dxutils::getFileSize(ft->getFileSize()));
    else return _("Uploading %1 of %2").arg(dxutils::getFileSize(ft->getBytesSent())).arg(dxutils::getFileSize(ft->getFileSize()));
}

class FTViewDelegate : public QItemDelegate
{
public:
    inline FTViewDelegate(FtTabItem *item) : QItemDelegate(item) {}
    inline void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
    {
        Q_UNUSED(painter)
        Q_UNUSED(option)
        Q_UNUSED(index)
        if(index.column()!=1)
        {
            QItemDelegate::paint(painter, option, index);
            return;
        }
        QStyleOptionProgressBar progressBarOption;
        progressBarOption.state = QStyle::State_Enabled;
        progressBarOption.direction = QApplication::layoutDirection();
        progressBarOption.rect = option.rect;
        progressBarOption.fontMetrics = QApplication::fontMetrics();
        progressBarOption.minimum = 0;
        progressBarOption.maximum = 100;
        progressBarOption.textAlignment = Qt::AlignCenter;
        progressBarOption.textVisible = true;
        quint64 position = qobject_cast<FtTabItem*>(parent())->ftForRow(index.row())->getBytesSent();
        quint64 size = qobject_cast<FtTabItem*>(parent())->ftForRow(index.row())->getFileSize();
        progressBarOption.progress = percent(position, size);
        progressBarOption.text = statusForFT(qobject_cast<FtTabItem*>(parent())->ftForRow(index.row()));
        QApplication::style()->drawControl(QStyle::CE_ProgressBar, &progressBarOption, painter);
    }
};

FtTabItem::FtTabItem(dxqCallbacks *cb, QWidget *parent) :
    QWidget(parent)
{
    QStringList headers;
    headers << _("File") << _("Status") << _("Remaining time");
    m_fileView = new QTreeWidget(this);
    m_fileView->setItemDelegate(new FTViewDelegate(this));
    m_fileView->setHeaderLabels(headers);
    m_fileView->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_fileView->setAlternatingRowColors(true);
    m_fileView->setRootIsDecorated(false);
    m_fileView->setContextMenuPolicy(Qt::CustomContextMenu);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_fileView);
    setLayout(mainLayout);
    QFontMetrics fm = fontMetrics();
    QHeaderView *header = m_fileView->header();
#if QT_VERSION < QT_VERSION_CHECK(5,11,0)
    header->resizeSection(0, fm.width("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(1, fm.width("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(2, fm.width(headers.at(2) + "  "));
#else
    header->resizeSection(0, fm.horizontalAdvance("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(1, fm.horizontalAdvance("  Downloading 1024 MB from 1024 MB for 666KB/s .."));
    header->resizeSection(2, fm.horizontalAdvance(headers.at(2) + "  "));
#endif

    m_details = new QGroupBox(_("Details"));
    QGridLayout *groupLayout = new QGridLayout;
    groupLayout->addWidget(new QLabel(_("Path:")), 0, 0);
    m_path = new QLabel;
    groupLayout->addWidget(m_path, 0, 1);
    groupLayout->addWidget(new QLabel(_("Account:")), 1, 0);
    m_account = new QLabel;
    groupLayout->addWidget(m_account, 1, 1);
    groupLayout->addWidget(new QLabel(_("Protocol:")), 2, 0);
    m_protocol = new QLabel;
    groupLayout->addWidget(m_protocol, 2, 1);
    m_direction = new QLabel;
    groupLayout->addWidget(m_direction, 3, 0);
    m_buddy = new QLabel;
    groupLayout->addWidget(m_buddy, 3, 1);
    m_details->setLayout(groupLayout);
    mainLayout->addWidget(m_details);
    m_details->hide();

    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Close);
    connect(buttons, SIGNAL(rejected()), this, SLOT(slotOk()));
    m_canceled = new QPushButton(_("Clear &canceled"));
    m_canceled->setEnabled(false);
    buttons->addButton(m_canceled, QDialogButtonBox::ActionRole);
    connect(m_canceled, SIGNAL(clicked()), this, SLOT(slotClearCanceled()));
    m_finished = new QPushButton(_("Clear &finished"));
    m_finished->setEnabled(false);
    buttons->addButton(m_finished, QDialogButtonBox::ActionRole);
    connect(m_finished, SIGNAL(clicked()), this, SLOT(slotClearFinished()));
    mainLayout->addWidget(buttons);

    connect(cb, SIGNAL(signalNewFT(PURPLE::FileTransfer*)), this, SLOT(slotNewFT(PURPLE::FileTransfer*)));
    connect(cb, SIGNAL(signalStartedFT(PURPLE::FileTransfer*)), this, SLOT(slotStartedFT(PURPLE::FileTransfer*)));
    connect(cb, SIGNAL(signalCanceledFT(PURPLE::FileTransfer*)), this, SLOT(slotCanceledFT(PURPLE::FileTransfer*)));
    connect(cb, SIGNAL(signalFinishedFT(PURPLE::FileTransfer*)), this, SLOT(slotFinishedFT(PURPLE::FileTransfer*)));
    connect(cb, SIGNAL(signalProgressFT(PURPLE::FileTransfer*)), this, SLOT(slotProgressFT(PURPLE::FileTransfer*)));
    connect(m_fileView, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(slotItemChanged(QTreeWidgetItem*)));
    connect(m_fileView, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(slotItemPopup(const QPoint &)));
}

PURPLE::FileTransfer *FtTabItem::ftForRow(int row)
{
    return m_fileTransfers.at(row);
}

int FtTabItem::rowForFT(PURPLE::FileTransfer *ft) const
{
    int row = 0;
    foreach(PURPLE::FileTransfer* item, m_fileTransfers)
    {
        if(item==ft) return row;
        ++row;
    }
    return -1;
}

void FtTabItem::slotNewFT(PURPLE::FileTransfer *ft)
{
    if(ft->isSending())
    {
        m_fileTransfers.append(ft);
        // Create and add a row view for this filetransfer.
        QTreeWidgetItem *item = new QTreeWidgetItem(m_fileView);
        item->setText(0, QFileInfo(dxutils::fromStdString(ft->getLocalFileName())).fileName());
        item->setText(1, _("Waiting"));
        item->setText(2, dxutils::getRemaining(0,0));
        item->setFlags(item->flags() & ~Qt::ItemIsEditable);
        item->setTextAlignment(1, Qt::AlignHCenter);
        m_fileView->setCurrentItem(item);
        m_details->show();
    }
}

void FtTabItem::slotStartedFT(PURPLE::FileTransfer *ft)
{
    if(ft->isReceiving())
    {
        m_fileTransfers.append(ft);
        // Create and add a row view for this filetransfer.
        QTreeWidgetItem *item = new QTreeWidgetItem(m_fileView);
        item->setText(0, QFileInfo(dxutils::fromStdString(ft->getLocalFileName())).fileName());
        item->setText(1, statusForFT(ft));
        item->setText(2, dxutils::getRemaining(0,0));
        item->setFlags(item->flags() & ~Qt::ItemIsEditable);
        item->setTextAlignment(1, Qt::AlignHCenter);
        m_fileView->setCurrentItem(item);
        m_details->show();
    }
    else
        slotProgressFT(ft);
}

void FtTabItem::slotCanceledFT(PURPLE::FileTransfer *ft)
{
    int row = rowForFT(ft);
    if(row>-1&&row<m_fileTransfers.size())
    {
        QTreeWidgetItem *item = m_fileView->topLevelItem(row);
        if(item)
        {
            item->setText(1, statusForFT(ft));
            item->setText(2, "");
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(_("Transfer of %1 canceled").arg(item->text(0)));
            m_canceled->setEnabled(true);
        }
    }
}

void FtTabItem::slotFinishedFT(PURPLE::FileTransfer *ft)
{
    int row = rowForFT(ft);
    if(row>-1&&row<m_fileTransfers.size())
    {
        QTreeWidgetItem *item = m_fileView->topLevelItem(row);
        if(item)
        {
            item->setText(1, statusForFT(ft));
            item->setText(2, "");
            if(DXPREFERENCES.m_notify && DXPREFERENCES.m_notifyFiletransfer) dxutils::instance().showNotify(_("Transfer of %1 finished").arg(item->text(0)));
            m_finished->setEnabled(true);
        }
    }
}

void FtTabItem::slotProgressFT(PURPLE::FileTransfer *ft)
{
    int row = rowForFT(ft);
    if(row>-1&&row<m_fileTransfers.size())
    {
        QTreeWidgetItem *item = m_fileView->topLevelItem(row);
        if(item)
        {
            item->setText(1, statusForFT(ft));
            if(ft->isCanceled() || ft->isFinished())
            {
                item->setText(2, "--:--");
            }
            else
            {
                item->setText(2, dxutils::getRemaining(ft->getFileSize()-ft->getBytesSent(), ft->getSpeed()));
            }
        }
    }
}

void FtTabItem::slotOk()
{
    emit closeFtTabItem();
}

void FtTabItem::slotClearCanceled()
{
    for(int i=m_fileTransfers.size()-1; i>-1; i--)
    {
        if(m_fileTransfers.at(i)->isCanceled())
        {
            m_fileTransfers.removeAt(i);
            m_fileView->takeTopLevelItem(i);
        }
    }
    m_canceled->setEnabled(false);
    if(!m_fileTransfers.size())
        m_details->hide();
}

void FtTabItem::slotClearFinished()
{
    for(int i=m_fileTransfers.size()-1; i>-1; i--)
    {
        if(m_fileTransfers.at(i)->isFinished())
        {
            m_fileTransfers.removeAt(i);
            m_fileView->takeTopLevelItem(i);
        }
    }
    m_finished->setEnabled(false);
    if(!m_fileTransfers.size())
        m_details->hide();
}

void FtTabItem::slotItemChanged(QTreeWidgetItem* current)
{
    int row = m_fileView->indexOfTopLevelItem(current);
    if(row>=0 && row<m_fileTransfers.size())
    {
        PURPLE::FileTransfer *transfer = ftForRow(row);
        m_path->setText(dxutils::fromStdString(transfer->getLocalFileName()));
        m_account->setText(dxutils::fromStdString(transfer->getAccount().getUsername()));
        m_protocol->setText(QString("<img src=\"%1\"> %2").arg(dxutils::protocolToIcon(dxutils::fromStdString(transfer->getAccount().getProtocol().getPurpleID()))).arg(dxutils::fromStdString(transfer->getAccount().getProtocol().getName())));
        m_direction->setText(transfer->isReceiving()?_("From:"):_("To:"));
        m_buddy->setText(dxutils::fromStdString(transfer->getRemoteUser()));
    }
}

void FtTabItem::slotItemPopup(const QPoint &pos)
{
    QTreeWidgetItem *item = 0;
    item = m_fileView->itemAt(pos);
    if(item)
    {
        int row = m_fileView->indexOfTopLevelItem(item);
        PURPLE::FileTransfer *transfer = ftForRow(row);
        if(!transfer->isCanceled() && !transfer->isFinished())
        {
            QMenu popup;
            QAction *action = new QAction(_("Cancel"), m_fileView);
            connect(action, SIGNAL(triggered()), this, SLOT(slotItemCancel()));
            popup.addAction(action);
            popup.exec(m_fileView->viewport()->mapToGlobal(pos));
        }
    }
}

void FtTabItem::slotItemCancel()
{
    ftForRow(m_fileView->indexOfTopLevelItem(m_fileView->currentItem()))->cancelFT();
}
