/*
 *      imtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTimer>
#include <QDir>
#include <QTextBlock>
#include "imtabitem.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

#define HISTORY_MAX 25

ImTabItem::ImTabItem(PURPLE::Conversation *conv, QWidget *parent) :
    QWidget(parent), m_name(dxutils::fromStdString(conv->getTitle())),
    m_ID(dxutils::fromStdString(conv->getName())), m_conversation(conv),
    m_currentPosition(0), m_process(NULL), m_sendPipe(false)
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    m_lastKeyPress = QDateTime::fromSecsSinceEpoch(0);
#else
    m_lastKeyPress.setTime_t(0);
#endif
    m_imState = ACTIVE;
    m_composeTimer = new QTimer(this);
    m_pipeTimer = new QTimer(this);

    m_info = new QLabel;
    m_info->setText(dxutils::fromStdString(m_conversation->getName())+" ("+dxutils::fromStdString(m_conversation->getAccount().getUsername())+")");
    m_presence = new QLabel;
    m_presence->setPixmap(QPixmap(dxutils::presenceToIcon(m_conversation->getIMBuddyStatus())));
    m_protocol = new QLabel;
    m_protocol->setPixmap(QPixmap(dxutils::protocolToIcon(dxutils::fromStdString(m_conversation->getAccount().getProtocol().getPurpleID()))));
    QHBoxLayout *idLayout = new QHBoxLayout;
    idLayout->addWidget(m_presence);
    idLayout->addWidget(m_info);
    idLayout->addSpacerItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    idLayout->addWidget(m_protocol);
    m_text = new QTextBrowser;
    m_text->setOpenExternalLinks(true);
    m_text->setFont(DXPREFERENCES.m_imFont);
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_commandline = new QLineEdit;
    m_commandline->installEventFilter(this);
    m_commandline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    if(DXPREFERENCES.m_sameCmd) m_commandline->setFont(DXPREFERENCES.m_imFont);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(idLayout);
    mainLayout->addWidget(m_text);
    mainLayout->addWidget(m_commandline);
    setLayout(mainLayout);

    connect(m_commandline, SIGNAL(returnPressed()), this, SLOT(slotProcessCommandline()));
    connect(m_composeTimer, SIGNAL(timeout()), this, SLOT(slotComposeTimeout()));
    connect(m_pipeTimer, SIGNAL(timeout()), this, SLOT(slotPipeTimeout()));

    m_composeTimer->start(2000);

    if(DXPREFERENCES.m_logLoadCurrent) loadCurrentLog();
}

ImTabItem::~ImTabItem()
{
    m_composeTimer->stop();
}

void ImTabItem::updatePreferences()
{
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_commandline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_text->setFont(DXPREFERENCES.m_imFont);
    if(DXPREFERENCES.m_sameCmd) m_commandline->setFont(DXPREFERENCES.m_imFont);
    else m_commandline->setFont(QFont());
    m_logger.stopLogging();
}

void ImTabItem::focusCommandline()
{
    m_commandline->setFocus();
}

bool ImTabItem::eventFilter(QObject *obj, QEvent *ev)
{
    if(obj == m_commandline && ev->type() == QEvent::KeyPress)
    {
        m_lastKeyPress = QDateTime::currentDateTime();
        QKeyEvent *e = (QKeyEvent *)ev;
        if(e->key() == Qt::Key_Tab)
        {
            processCompletion();
            return true;
        }
        else if(e->key() == Qt::Key_Up)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_history.size())
            {
                if(!line.isEmpty() && line!=m_history[m_currentPosition])
                {
                    m_history.append(line);
                    if(m_history.size() > HISTORY_MAX)
                        m_history.removeFirst();
                    m_currentPosition = m_history.size()-1;
                }
                if(m_currentPosition > 0 && !line.isEmpty())
                    --m_currentPosition;
                m_commandline->setText(m_history[m_currentPosition]);
            }
        }
        else if(e->key() == Qt::Key_Down)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_history.size())
            {
                if(!line.isEmpty() && line!=m_history[m_currentPosition])
                {
                    m_history.append(line);
                    if(m_history.size() > HISTORY_MAX)
                        m_history.removeFirst();
                    m_currentPosition = m_history.size()-1;
                }
                if(m_currentPosition < m_history.size()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_history[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
        }
    }
    return QWidget::eventFilter(obj, ev);
}

void ImTabItem::processCompletion()
{
    QString line = m_commandline->text();
    if(m_commands.size()==0)
    {
        m_commands << "exec" << "help" << "say" << "log";
        std::vector<std::string> cmds = m_conversation->getCommandsList();
        FOREACH(std::vector<std::string>, cmds, it)
            m_commands.append(dxutils::fromStdString((*it)));
        if(!m_commands.contains("me"))
            m_commands.append("me");
        m_commands.sort();
    }
    if(line.section(' ',0,0).compare("/help", Qt::CaseInsensitive) == 0 && !line.section(' ',1).isEmpty())
    {
        completeHelpCommand();
        return;
    }
    if(line[0] == '/' && line.section(' ', 1).isEmpty())
    {
        line = line.section(' ',0,0).section('/',1);
        for(int i = 0; i < m_commands.size(); i++)
        {
            if(line.compare(m_commands[i], Qt::CaseInsensitive) == 0)
            {
                if((i+1) < m_commands.size()) m_commandline->setText("/"+m_commands[++i]+" ");
                else m_commandline->setText("/"+m_commands[0]+" ");
                break;
            }
            else if(line.compare(m_commands[i].left(line.length()), Qt::CaseInsensitive) == 0)
            {
                m_commandline->setText("/"+m_commands[i]+" ");
                break;
            }
        }
    }
}

void ImTabItem::onNewMsg(QString sender, QString msg, QString time, bool mhighlight, bool history, bool action)
{
    m_messageIDs.append("");
    if(sender!=m_name)
    {
        m_name = sender;
        emit newIMName(m_name);
    }
    emit composing(false);
    if(history)
        appendStyledText(time, sender+": "+msg, DXPREFERENCES.m_logColor);
    else
    {
        if(mhighlight)
        {
            appendStyledText(time, sender+(action?" ":": ")+msg, DXPREFERENCES.m_hilightColor);
            emit highlight();
        }
        else
        {
            if(action) appendStyledText(time, sender+" "+msg, DXPREFERENCES.m_actionColor);
            else appendNickText(time, sender, msg);
            emit newmsg();
        }
    }
}

void ImTabItem::onNewMyMsg(QString msg, QString time, bool action, QString messageID)
{
    m_messageIDs.append(messageID);
    if(action) appendStyledText(time, dxutils::fromStdString(m_conversation->getMyNick())+" "+msg, DXPREFERENCES.m_actionColor);
    else appendMyMsgText(time, msg);
}

void ImTabItem::onMessageRecived(QString messageID)
{
    QTextBlock block = m_text->document()->findBlockByNumber(m_messageIDs.indexOf(messageID));
    QTextCursor cursor(block);
    cursor.movePosition(QTextCursor::EndOfBlock);
    cursor.insertText(" \u2713");
}

void ImTabItem::onNewConvSystemMessage(QString msg, QString time)
{
    m_messageIDs.append("");
    appendStyledText(time, msg, DXPREFERENCES.m_userColor);
}

void ImTabItem::onNewConvErrorMessage(QString msg, QString time)
{
    m_messageIDs.append("");
    appendStyledText(time, msg, DXPREFERENCES.m_errorColor);
}

void ImTabItem::onBuddyTyping(IMStateType typing)
{
    if(typing==STARTED) emit composing(true);
    else emit composing(false);
}

void ImTabItem::onBuddySignedOn(PURPLE::Buddy buddy)
{
    if(m_conversation->isValid() && m_conversation->getName() == buddy.getUsername())
    {
        if(m_conversation->getAccount().isValid() && m_conversation->getAccount().findBuddy(m_conversation->getName()) == buddy)
        {
            m_presence->setPixmap(QPixmap(dxutils::presenceToIcon(buddy.getStatusPrimitive())));
            emit offline(false);
        }
    }
}

void ImTabItem::onBuddySignedOff(PURPLE::Buddy buddy)
{
    if(m_conversation->isValid() && m_conversation->getName() == buddy.getUsername())
    {
        if(m_conversation->getAccount().isValid() && m_conversation->getAccount().findBuddy(m_conversation->getName()) == buddy)
        {
            m_presence->setPixmap(QPixmap(dxutils::presenceToIcon(PURPLE_STATUS_OFFLINE)));
            emit offline(true);
        }
    }
}

void ImTabItem::onBuddyStatusChanged(PURPLE::Buddy buddy, PurpleStatusPrimitive newStatus)
{
    if(m_conversation->isValid() && m_conversation->getName() == buddy.getUsername())
    {
        if(m_conversation->getAccount().isValid() && m_conversation->getAccount().findBuddy(m_conversation->getName()) == buddy)
            m_presence->setPixmap(QPixmap(dxutils::presenceToIcon(newStatus)));
    }
}

void ImTabItem::slotProcessCommandline()
{
    if(!m_conversation->getAccount().isConnected())
        return;
    QString commandtext = m_commandline->text();
    if(commandtext.isEmpty())
        return;
    m_history.append(commandtext);
    if (m_history.size() > HISTORY_MAX)
        m_history.removeFirst();
    m_currentPosition = m_history.size()-1;
    m_commandline->clear();
    if(commandtext[0]=='/')
    {
        if(!commandtext.section(' ', 0, 0).compare("/exec", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1);
            if(command.isEmpty()) showInternalHelp("exec");
            else
            {
                if(!m_process)
                {
                    m_process = new QProcess;
                    connect(m_process, SIGNAL(readyReadStandardOutput()), this, SLOT(slotOnProcess()));
                }
                if(!command.section(' ', 0, 0).compare("-o", Qt::CaseInsensitive))
                {
                    m_sendPipe = true;
                    appendStyledText("", _("Command %1 started").arg(command.section(' ',1)), DXPREFERENCES.m_hilightColor, false);
                    m_process->setProgram(command.section(' ',1,1));
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
                    m_process->setArguments(command.section(' ',2).split(' ', QString::SkipEmptyParts));
#else
                    m_process->setArguments(command.section(' ',2).split(' ', Qt::SkipEmptyParts));
#endif
                    m_process->start();
                }
                else if(!command.section(' ', 0, 0).compare("-c", Qt::CaseInsensitive))
                {
                    m_sendPipe = false;
                    appendStyledText("", _("Command %1 stopped").arg(m_process->program()), DXPREFERENCES.m_hilightColor, false);
                    m_pipeStrings.clear();
                    m_process->close();
                }
                else
                {
                    m_sendPipe = false;
                    appendStyledText("", _("Command %1 started").arg(command), DXPREFERENCES.m_hilightColor, false);
                    m_process->setProgram(command.section(' ',0,0));
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
                    m_process->setArguments(command.section(' ',1).split(' ', QString::SkipEmptyParts));
#else
                    m_process->setArguments(command.section(' ',1).split(' ', Qt::SkipEmptyParts));
#endif
                    m_process->start();
                }
            }
        }
        else if(!commandtext.section(' ', 0, 0).compare("/help", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1);
            if(command.isEmpty())
            {
                QString commands = "";
                for(int i = 0; i < m_commands.size(); i++)
                {
                    if(i+1 == m_commands.size()) commands.append(m_commands[i] + ".");
                    else commands.append(m_commands[i] + ", ");
                }
                appendStyledText("", _("Available commands: %1").arg(commands), DXPREFERENCES.m_hilightColor, false);
            }
            else if(command.compare("exec", Qt::CaseInsensitive)==0 || command.compare("help", Qt::CaseInsensitive)==0 || command.compare("me", Qt::CaseInsensitive)==0
                    || command.compare("say", Qt::CaseInsensitive)==0 || command.compare("log", Qt::CaseInsensitive)==0)
            {
                showInternalHelp(command);
            }
            else
            {
                QString response = dxutils::fromStdString(m_conversation->getCommandHelp(dxutils::toStdString(command)));
                if(response.isEmpty())
                    appendStyledText("", _("Command '%1' doesn't exist or hasn't help").arg(command), DXPREFERENCES.m_errorColor, false);
                else
                    appendStyledText("", _("Help for: %1").arg(response), DXPREFERENCES.m_hilightColor, false);
            }
        }
        else if(!commandtext.section(' ', 0, 0).compare("/me", Qt::CaseInsensitive))
        {
            if(!commandtext.section(' ', 1).isEmpty()) m_conversation->sendActionMessage(dxutils::toStdString(commandtext.section(' ', 1)));
        }
        else if(!commandtext.section(' ', 0, 0).compare("/say", Qt::CaseInsensitive))
        {
            if(!commandtext.section(' ', 1).isEmpty()) m_conversation->sendMessage(dxutils::toStdString(commandtext.section(' ', 1)));
        }
        else if(!commandtext.section(' ', 0, 0).compare("/log", Qt::CaseInsensitive))
        {
            QString command = commandtext.section(' ', 1, 1);
            QString logfile = commandtext.section(' ', 2, 2);
            if(!command.compare("list", Qt::CaseInsensitive))
            {
                listLogFiles();
            }
            else if(!command.compare("show", Qt::CaseInsensitive))
            {
                if(logfile.isEmpty()) logfile = QDateTime::currentDateTime().toString("yyyy-MM-dd");
                QString path = DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0);
                if(!QFile().exists(path+"/"+logfile))
                {
                    appendStyledText("", _("Log file %1 doesn't exist").arg(logfile), DXPREFERENCES.m_errorColor, false);
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
                    m_lastKeyPress = QDateTime::fromSecsSinceEpoch(0);
#else
                    m_lastKeyPress.setTime_t(0);
#endif
                    m_imState = ACTIVE;
                    return;
                }
                if(dxutils::isRightLogFile(path+"/"+logfile, logfile))
                {
                    emit showLog(m_conversation->getAccount(), dxutils::fromStdString(m_conversation->getName()), path+"/"+logfile);
                }
            }
            else
            {
                appendStyledText("", _("'%1' isn't <list|show>").arg(command), DXPREFERENCES.m_errorColor, false);
            }
        }
        else
        {
            int ret = m_conversation->sendCommand(dxutils::toStdString(commandtext.section('/', 1)));
            processCommandReturn(ret, commandtext.section('/', 1).section(' ', 0, 0));
        }
    }
    else
        m_conversation->sendMessage(dxutils::toStdString(commandtext));
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    m_lastKeyPress = QDateTime::fromSecsSinceEpoch(0);
#else
    m_lastKeyPress.setTime_t(0);
#endif
    m_imState = ACTIVE;
}

//process return m_conversation->sendCommand
void ImTabItem::processCommandReturn(int ret, const QString &command)
{
    switch(ret) {
        case PURPLE_CMD_STATUS_OK:
            break;
        case PURPLE_CMD_STATUS_NOT_FOUND:
            appendStyledText("", _("Unknown command '%1'.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_ARGS:
            appendStyledText("", _("Syntax Error:  You typed the wrong number of arguments "
                                "to that command '%1'.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_FAILED:
            appendStyledText("", _("Your command '%1' failed for an unknown reason.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_TYPE:
            if(m_conversation->getType() == PURPLE_CONV_TYPE_IM)
                appendStyledText("", _("That command '%1' only works in chats, not IMs.").arg(command), DXPREFERENCES.m_errorColor, false);
            else
                appendStyledText("", _("That command '%1' only works in IMs, not chats.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
        case PURPLE_CMD_STATUS_WRONG_PRPL:
            appendStyledText("", _("That command '%1' doesn't work on this protocol.").arg(command), DXPREFERENCES.m_errorColor, false);
            break;
    }
}

void ImTabItem::showInternalHelp(const QString &command)
{
    if(command.compare("exec", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: exec [-o|-c] <command>, executes command, -o sends output to im, -c closes running command"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("help", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: help [command]: lists available commands or show help for command"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("me", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: me <text>: sends text in 3rd person"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("say", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: say <text>: sends text"), DXPREFERENCES.m_hilightColor, false);
    }
    else if(command.compare("log", Qt::CaseInsensitive)==0)
    {
        appendStyledText("", _("Help for: log"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("LOG list, shows list of log files for current im/chat"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("LOG show [date], shows current log file for current im/chat, if date is empty, else file for given date"), DXPREFERENCES.m_hilightColor, false);
        appendStyledText("", _("date example: 2011-01-31"), DXPREFERENCES.m_hilightColor, false);
    }
}

void ImTabItem::appendStyledText(QString time, QString msg, QColor clr, bool log)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    m_text->append(time+QString("<font color=\"%1\">").arg(clr.name())+dxutils::parseHtml(msg)+"</font>");
    if(log && startLogging())
    {
        m_logger.logStyledLine(dxutils::toStdString(time), dxutils::styleToString(clr), dxutils::toStdString(msg));
    }
}

void ImTabItem::appendNickText(QString time, QString nick, QString msg, bool log)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    if(DXPREFERENCES.m_coloredNick) m_text->append(time+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+" </font>"+dxutils::parseHtml(msg));
    else m_text->append(time+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+" </b>"+dxutils::parseHtml(msg));
    if(log && startLogging())
    {
        m_logger.logNickLine(dxutils::toStdString(time), dxutils::toStdString(nick), dxutils::toStdString(msg));
    }
}

void ImTabItem::appendMyMsgText(QString time, QString msg)
{
    if(time.isEmpty()) time = QTime::currentTime().toString("hh:mm:ss");
    time = "["+time+"] ";
    msg = dxutils::createSmileys(msg);
    if(DXPREFERENCES.m_coloredNick) m_text->append(time+QString("<font color=\"%1\">").arg(dxutils::getNickColor(dxutils::fromStdString(m_conversation->getMyNick())))+dxutils::fromStdString(m_conversation->getMyNick())+DXPREFERENCES.m_nickCompletionChar+" </font>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_mymsgColor.name())+dxutils::parseHtml(msg)+"</font>");
    else m_text->append(time+"<b>"+dxutils::fromStdString(m_conversation->getMyNick())+DXPREFERENCES.m_nickCompletionChar+" </b>"+QString("<font color=\"%1\">").arg(DXPREFERENCES.m_mymsgColor.name())+dxutils::parseHtml(msg)+"</font>");
    if(startLogging()) m_logger.logMyMsgLine(dxutils::toStdString(time), m_conversation->getMyNick(), dxutils::toStdString(msg));
}

void ImTabItem::clearText()
{
    m_messageIDs.clear();
    m_text->clear();
}

void ImTabItem::closeConversation()
{
    m_composeTimer->stop();
    m_conversation->leave();
}

bool ImTabItem::startLogging()
{
    if(!DXPREFERENCES.m_logging) return false;
    if(m_logger.isReady()) return true;
    m_logger.setPath(dxutils::toStdString(DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0)));
    return true;
}

void ImTabItem::completeHelpCommand()
{
    QString line = m_commandline->text();
    for(int i = 0; i < m_commands.size(); i++)
    {
        if(line.section(' ',1).compare(m_commands[i], Qt::CaseInsensitive) == 0)
        {
            if((i+1) < m_commands.size()) m_commandline->setText("/help "+m_commands[++i]);
            else m_commandline->setText("/help "+m_commands[0]);
            break;
        }
        else if(line.section(' ',1).compare(m_commands[i].left(line.section(' ',1).length()), Qt::CaseInsensitive) == 0)
        {
            m_commandline->setText("/help "+m_commands[i]);
            break;
        }
    }
}

void ImTabItem::slotComposeTimeout()
{
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
    if(!m_lastKeyPress.toSecsSinceEpoch()) return;
#else
    if(!m_lastKeyPress.toTime_t()) return;
#endif
    QDateTime now = QDateTime::currentDateTime();
    if(m_lastKeyPress.secsTo(now)>5)
    {
        if(m_imState==STARTED)
        {
            m_imState = PAUSED;
            m_conversation->sendTyping(false);
        }
    }
    else
    {
        if(m_imState==ACTIVE || m_imState==PAUSED)
        {
            m_imState = STARTED;
            m_conversation->sendTyping(true);
        }
    }
}

void ImTabItem::slotOnProcess()
{
    QByteArray byteArray = m_process->readAllStandardOutput();
#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
    QStringList lines = QString(byteArray).split('\n', QString::SkipEmptyParts);
#else
    QStringList lines = QString(byteArray).split('\n', Qt::SkipEmptyParts);
#endif
    foreach(QString line, lines)
    {
        if(m_sendPipe)
        {
            if(!m_pipeTimer->isActive()) m_pipeTimer->start(3000);
            m_pipeStrings.append(line);
        }
        else appendStyledText("", line, DXPREFERENCES.m_mymsgColor, false);
    }
}

void ImTabItem::slotPipeTimeout()
{
    if(m_pipeStrings.size() > 3)
    {
        if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
        if(m_conversation->isValid())
        {
            m_conversation->sendMessage(dxutils::toStdString(m_pipeStrings[0]));
        }
        m_pipeStrings.pop_front();
    }
    else
    {
        while(m_pipeStrings.size())
        {
            if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
            if(m_conversation->isValid())
            {
                m_conversation->sendMessage(dxutils::toStdString(m_pipeStrings[0]));
            }
            m_pipeStrings.pop_front();
        }
    }
}

void ImTabItem::loadCurrentLog()
{
    clearText();
    QString path = DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0);
    QString name = QDateTime::currentDateTime().toString("yyyy-MM-dd");
    QFile file(path+"/"+name);
    if(!dxutils::isRightLogFile(path+"/"+name, name))
    {
        return;
    }
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        return;
    }
    while(!file.atEnd())
    {
        QByteArray line = file.readLine();
        m_messageIDs.append("");
        appendLogLine(QString::fromUtf8(line));
    }
}

QColor ImTabItem::stringToColor(QString style)
{
    if(style.compare("action", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_actionColor;
    if(style.compare("error", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_errorColor;
    if(style.compare("user", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_userColor;
    if(style.compare("mymsg", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_mymsgColor;
    if(style.compare("highlight", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_hilightColor;
    return DXPREFERENCES.m_textColor;
}

void ImTabItem::appendLogLine(const QString &line)
{
    int pos = line.indexOf(']')+2;
    if(pos<2) pos = 0;
    if(line.length()>pos+1 && line[pos]=='\014') //log style mark
    {
        QColor style = stringToColor(line.section('\014',1,1));
        if(line.section('\014',2)[0]=='<') //has nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_text->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+"</font>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
            else
                m_text->append(line.section('\014',0,0)+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+"</b>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
        }
        else m_text->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(line.section('\014',2))+"</font>");
    }
    else
    {
        if(line.length()>pos+1 && line[pos]=='<') //line with nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_text->append(line.section('<',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+DXPREFERENCES.m_nickCompletionChar+"</font>"+dxutils::parseHtml(msg));
            else
                m_text->append(line.section('<',0,0)+"<b>"+nick+DXPREFERENCES.m_nickCompletionChar+"</b>"+dxutils::parseHtml(msg));
        }
        else m_text->append(dxutils::parseHtml(line));
    }
}

void ImTabItem::listLogFiles()
{
    QDir dir(DXPREFERENCES.m_logPath+"/"+dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)+"/"+m_ID.section('/',0,0));
    dir.setFilter(QDir::Files | QDir::Readable);
    dir.setSorting(QDir::Name);
    QStringList files;
    QFileInfoList list = dir.entryInfoList();
    for(int i = 0; i < list.size(); ++i)
    {
        QFileInfo fileInfo = list.at(i);
        if(dxutils::isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
        {
            files << fileInfo.fileName();
        }
    }
    if(files.size())
    {
        appendStyledText("", QString("<b><u>%1</u></b>").arg(_("Log files:")), DXPREFERENCES.m_textColor, false);
        files.sort();
        QString lastYear = files[0].section('-',0,0);
        int lastMonth = files[0].section('-',1,1).toInt();
        appendStyledText("", QString("<b>%1:</b>").arg(lastYear), DXPREFERENCES.m_textColor, false);
        appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
        QString list = "";
        for(int i=0; i<files.size(); i++)
        {
            if(lastYear!=files[i].section('-',0,0))
            {
                lastYear = files[i].section('-',0,0);
                lastMonth = files[i].section('-',1,1).toInt();
                appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b>%1:</b>").arg(lastYear), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
                list.clear();
                list.append(files[i].section('-',2,2)+", ");
            }
            else if(lastMonth!=files[i].section('-',1,1).toInt())
            {
                lastMonth = files[i].section('-',1,1).toInt();
                appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
                appendStyledText("", QString("<b><i>%1</i>:</b>").arg(dxutils::getMonth(lastMonth)), DXPREFERENCES.m_textColor, false);
                list.clear();
                list.append(files[i].section('-',2,2)+", ");
            }
            else
            {
                list.append(files[i].section('-',2,2)+", ");
            }
        }
        if(!list.isEmpty()) appendStyledText("", list.left(list.length()-2), DXPREFERENCES.m_textColor, false);
    }
    else
    {
        appendStyledText("", _("No log files for %1-%2.").arg(dxutils::fromStdString(m_conversation->getAccount().getUsername()).section('/',0,0)).arg(m_ID.section('/',0,0)), DXPREFERENCES.m_errorColor, false);
    }
}

