/*
 *      msgtabitem.cpp
 *
 *      Copyright 2012 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QVBoxLayout>
#include <QTime>
#include <QDesktopServices>
#include <QUrl>
#include <QLabel>
#include <QPushButton>
#include <QDialogButtonBox>
#include <QFileInfo>
#include "msgtabitem.h"
#include "preferences.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

MsgTabItem::MsgTabItem(QPixmap icon, QString id, QString text, QString okText, QString url, QWidget *parent) :
    QWidget(parent), m_url(url), m_id(id)
{
    QLabel *info = new QLabel;
    info->setPixmap(icon);
    m_edit = new QTextBrowser;
    m_edit->setOpenExternalLinks(true);
    m_edit->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_edit->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    m_edit->append("<b>"+QTime::currentTime().toString("hh:mm:ss")+"</b>");
    m_edit->append(text);
    QDialogButtonBox *buttons = new QDialogButtonBox();
    QPushButton *ok = new QPushButton(okText);
    buttons->addButton(ok, QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(slotOK()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(info);
    mainLayout->addWidget(m_edit);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
}

MsgTabItem::MsgTabItem(QString id, QString okText, QString url, QWidget *parent) :
    QWidget(parent), m_url(url), m_id(id)
{
    m_edit = new QTextBrowser;
    m_edit->setOpenExternalLinks(true);
    m_edit->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_edit->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
    QDialogButtonBox *buttons = new QDialogButtonBox();
    QPushButton *ok = new QPushButton(okText);
    buttons->addButton(ok, QDialogButtonBox::AcceptRole);
    connect(buttons, SIGNAL(accepted()), this, SLOT(slotOK()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(m_edit);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
}

MsgTabItem::~MsgTabItem()
{}

void MsgTabItem::insertText(const QString &text)
{
    m_edit->clear();
    m_edit->append("<b>"+QTime::currentTime().toString("hh:mm:ss")+"</b>");
    m_edit->append(text);
}

void MsgTabItem::showLog(const QString &path)
{
    m_edit->clear();
    QFileInfo info(path);
    m_edit->append("<b><u>"+_("Log from %1:").arg(info.fileName())+"</u></b>");
    if(!dxutils::isRightLogFile(path, info.fileName()))
    {
        return;
    }
    QFile file(path);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        return;
    }
    while(!file.atEnd())
    {
        QByteArray line = file.readLine();
        appendLogLine(QString::fromUtf8(line));
    }
}

void MsgTabItem::updateTheme()
{
    m_edit->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(DXPREFERENCES.m_linkColor.name()));
    m_edit->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(DXPREFERENCES.m_textColor.name()).arg(DXPREFERENCES.m_backroundColor.name()));
}

void MsgTabItem::slotOK()
{
    if(!m_url.isEmpty()) QDesktopServices::openUrl(QUrl::fromUserInput(m_url));
    emit signalMsgTabClosed();
}

QColor MsgTabItem::stringToColor(QString style)
{
    if(style.compare("action", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_actionColor;
    if(style.compare("error", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_errorColor;
    if(style.compare("user", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_userColor;
    if(style.compare("mymsg", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_mymsgColor;
    if(style.compare("highlight", Qt::CaseInsensitive)==0)
        return DXPREFERENCES.m_hilightColor;
    return DXPREFERENCES.m_textColor;
}

void MsgTabItem::appendLogLine(const QString &line)
{
    int pos = line.indexOf(']')+2;
    if(pos<2) pos = 0;
    if(line.length()>pos+1 && line[pos]=='\014') //log style mark
    {
        QColor style = stringToColor(line.section('\014',1,1));
        if(line.section('\014',2)[0]=='<') //has nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_edit->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+":"+"</font>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
            else
                m_edit->append(line.section('\014',0,0)+"<b>"+nick+":</b>"
                           +QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(msg)+"</font>");
        }
        else m_edit->append(line.section('\014',0,0)+QString("<font color=\"%1\">").arg(style.name())+dxutils::parseHtml(line.section('\014',2))+"</font>");
    }
    else
    {
        if(line.length()>pos+1 && line[pos]=='<') //line with nick
        {
            QString nick = line.section('<',1,1).section('>',0,0);
            QString msg = line.section('>',1);
            if(DXPREFERENCES.m_coloredNick)
                m_edit->append(line.section('<',0,0)+QString("<font color=\"%1\">").arg(dxutils::getNickColor(nick))+nick+":</font>"+dxutils::parseHtml(msg));
            else
                m_edit->append(line.section('<',0,0)+"<b>"+nick+":</b>"+dxutils::parseHtml(msg));
        }
        else m_edit->append(dxutils::parseHtml(line));
    }
}

